/***************************/
/*	CGump Class for easy   */
/*  gump handling		   */
/***************************/

#include "uox3.h"
#include "gump.h"
#include "packets.h"

CGump::CGump( bool myNoMove, bool myNoClose )
{
	NoMove = myNoMove;
	NoClose = myNoClose;

	Type = 21; // Default Type
	Serial = 0;	// Do nothing on close
	PageCount = 0;
}

CGump::~CGump()
{
}

// Procedure for "raw" adding new tags/texts
void CGump::Add( std::string Tag, std::string Text )
{
	if( Tag.size() > 0 )
		TagList.push_back( Tag );

	if( Text.size() > 0 )
		TextList.push_back( Text );
}

void CGump::Send( cSocket *target )
{
	if( target == NULL )
		return;

	CPSendGumpMenu toSend;
	toSend.GumpID( Type );
	toSend.UserID( Serial );

	size_t line = 0;
	for( line = 0; line < TagList.size(); ++line )
		toSend.AddCommand( TagList[line].c_str() );

	for( line = 0; line < TextList.size(); ++line )
		toSend.AddText( TextList[line].c_str() );

	toSend.Finalize();
	target->Send( &toSend );
}

void CGump::SetNoMove( bool myNoMove )
{
	NoMove = myNoMove;
}

void CGump::SetNoClose( bool myNoClose )
{
	NoClose = myNoClose;
}

void CGump::SetType( UI32 newType )
{
	Type = newType;
}

void CGump::SetSerial( UI32 newSerial )
{
	Serial = newSerial;
}

// Add a background picture
void CGump::AddBackground( UI16 x, UI16 y, UI16 GumpID, UI16 width, UI16 height )
{
	char temp[64];
	sprintf( temp, "resizepic %u %u %u %u %u", x, y, GumpID, width, height );
	TagList.push_back( temp );
}

// Add a gump
void CGump::AddGump( UI16 x, UI16 y, UI16 GumpID )
{
	char temp[64];
	sprintf( temp, "gumppic %u %u %u", x, y, GumpID );
	TagList.push_back( temp );
}

// Add a Button
void CGump::AddButton( UI16 x, UI16 y, UI16 ImageUp, UI16 ImageDown, UI16 Behaviour, UI16 Page, UI32 UniqueID  )
{
	char temp[128];
	sprintf( temp, "button %u %u %u %u %u %u %u", x, y, ImageUp, ImageDown, Behaviour, Page, UniqueID );
	TagList.push_back( temp );
}

// Add an "auto-Text"
void CGump::AddText( UI16 x, UI16 y, UI16 hue, std::string Text )
{
	char temp[128];
	size_t TextID = TextList.size();
	sprintf( temp, "text %u %u %u %u", x, y, hue, static_cast< UI32 >(TextID) );

	TextList.push_back( Text );
	TagList.push_back( temp );
}

// Start a new page and return the new page no.
UI32 CGump::StartPage( void )
{
	char temp[32];
	sprintf( temp, "page %u", PageCount );
	TagList.push_back( temp );

	++PageCount;
	return PageCount - 1;
}

// Callback for gumps.cpp
void MultiGumpCallback( cSocket *mySocket, SERIAL GumpSerial, UI32 Button )
{
	if( GumpSerial == 0 ) // Do nothing on close gump
		return;


	switch( GumpSerial )
	{
		// Custom Gump Callbacks
		case 1:
			{

			}


	};
}


