#include "uox3.h"
#include "weight.h"
#include "fileio.h"
#include "packets.h"
#include "classes.h"
#include "regions.h"

#include "speech.h"
#include "cRaces.h"

// New class written based upon old UOX char_st.  Number of old members removed
// and a number of members types modified as well

// Version History
// 1.0		Abaddon		2nd April, 2000
//			Initial implementation
//			Bools reduced down to a single short with get/set mechanisms for setting/clearings bits
//			No documentation currently done, all functions obvious so far
//			Note:  individual byte setting within longs/shorts need speeding up
//			consider memcpy into correct word, note that this will be endian specific!
// 1.1		Abaddon		23rd July, 2000
//			Addition of SaveItem routine for text mode only
// 1.2		Abaddon		25 July, 2000
//			All Accessor funcs plus a few others are now flagged as const functions, meaning that
//			those functions GUARENTEE never to alter data, at compile time
//			Thoughts about cBaseObject and prelim plans made
// 1.3		Abaddon		28 July, 2000
//			cBaseObject initial implementation plus rework of CItem to deal with only what it needs to
//			Proper constructor written
//			Plans for CItem derived classes thought upon

cBaseObject *		DEFITEM_CONTOBJ			= NULL;
const bool			DEFITEM_REMOVE			= false;
const UI08			DEFITEM_GLOWEFFECT		= 0;
const SERIAL		DEFITEM_GLOW			= INVALIDSERIAL;
const COLOUR		DEFITEM_GLOWCOLOUR		= INVALIDCOLOUR;
const SI08			DEFITEM_MADEWITH		= 0;
const SI32			DEFITEM_RANDVALUE		= 0;
const SI16			DEFITEM_GOOD			= -1;
const SI08			DEFITEM_RANK			= 0;
const ARMORCLASS	DEFITEM_ARMORCLASS		= 0;
const UI16			DEFITEM_RESTOCK			= 0;
const SI32			DEFITEM_SELLVALUE		= 0;
const SI32			DEFITEM_BUYVALUE		= 0;
const UI08			DEFITEM_PRIV			= 0;
const SI08			DEFITEM_MOVEABLE		= 0;
const TIMERVAL		DEFITEM_GATETIME		= 0;
const TIMERVAL		DEFITEM_DECAYTIME		= 0;
const TIMERVAL		DEFITEM_MURDERTIME		= 0;
const UI08			DEFITEM_SPEED			= 0;
const SI16			DEFITEM_MAXHP			= 0;
const UI08			DEFITEM_DOORDIR			= 0;
const UI16			DEFITEM_AMOUNT			= 1;
const UI32			DEFITEM_MOREX			= 0;
const UI32			DEFITEM_MOREY			= 0;
const UI32			DEFITEM_ENHANCED		= 0;
const UI08			DEFITEM_WEATHERBOOLS	= 0;
const UI32			DEFITEM_MOREZ			= 0;
const UI32			DEFITEM_MORE			= 0;
const UI32			DEFITEM_MOREB			= 0;
const UI08			DEFITEM_BOOLS			= 0;
const UI08			DEFITEM_LAYER			= 0;
const UI08			DEFITEM_TYPE			= 0;
const UI08			DEFITEM_TYPE2			= 0;
const SI08			DEFITEM_OFFSPELL		= 0;
const UI16			DEFITEM_ENTRYMADEFROM	= 0;
const SERIAL		DEFITEM_CREATOR			= INVALIDSERIAL;

CItem::CItem( ITEM nItem, bool zeroSer ) : 
contObj( DEFITEM_CONTOBJ ), remove( DEFITEM_REMOVE ), glow_effect( DEFITEM_GLOWEFFECT ), glow( DEFITEM_GLOW ), glowColour( DEFITEM_GLOWCOLOUR ), 
madewith( DEFITEM_MADEWITH ), rndvaluerate( DEFITEM_RANDVALUE ), good( DEFITEM_GOOD ), rank( DEFITEM_RANK ), armorClass( DEFITEM_ARMORCLASS ), 
restock( DEFITEM_RESTOCK ), sellValue( DEFITEM_SELLVALUE ), buyValue( DEFITEM_BUYVALUE ), priv( DEFITEM_PRIV ), movable( DEFITEM_MOVEABLE ), 
gatetime( DEFITEM_GATETIME ), decaytime( DEFITEM_DECAYTIME ), murdertime( DEFITEM_MURDERTIME ), spd( DEFITEM_SPEED ), maxhp( DEFITEM_MAXHP ), 
doordir( DEFITEM_DOORDIR ), amount( DEFITEM_AMOUNT ), morex( DEFITEM_MOREX ), morey( DEFITEM_MOREY ), enhanced( DEFITEM_ENHANCED ), 
weatherBools( DEFITEM_WEATHERBOOLS ), morez( DEFITEM_MOREZ ), more( DEFITEM_MORE ), moreb( DEFITEM_MOREB ), bools( DEFITEM_BOOLS ), 
layer( DEFITEM_LAYER ), type( DEFITEM_TYPE ), type2( DEFITEM_TYPE2 ), offspell( DEFITEM_OFFSPELL ), entryMadeFrom( DEFITEM_ENTRYMADEFROM ), 
creator( DEFITEM_CREATOR )
{
	if( zeroSer )
	{
		SetSerial( cwmWorldState->GetItemCount2(), nItem );
		cwmWorldState->IncItemCount2();
	}
	else
		SetSerial( INVALIDSERIAL, nItem );

	objType = OT_ITEM;
	Contains.resize( 0 );
	strcpy( name2, "#" );
	name = "#";
	desc[0] = 0;
	race = 65535;
}

CItem::~CItem()
{
	// Destructor to clean things up when deleted


	if( !isFree() )	// We're not the default item in the handler
	{
		for( CItem *i = FirstItem(); !FinishedItems(); i = NextItem() )
		{
			if( i != NULL )
				i->SetCont( NULL );
		}
		RemoveSelfFromCont();
		RemoveSelfFromOwner();
//		MapRegion->RemoveItem( this );
	}
}

//o--------------------------------------------------------------------------
//|	Function		-	UI32 GetCont()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the container's serial
//o--------------------------------------------------------------------------
cBaseObject * CItem::GetCont( void ) const
{
	return contObj;
}

SERIAL CItem::GetContSerial( void ) const
{
	if( contObj != NULL )
		return contObj->GetSerial();
	return INVALIDSERIAL;
}

//o--------------------------------------------------------------------------
//|	Function		-	UI32 GetMore()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the more value of the object
//o--------------------------------------------------------------------------
UI32 CItem::GetMore( void ) const
{
	return more;
}

//o--------------------------------------------------------------------------
//|	Function		-	UI32 GetMoreB()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the moreb value of the object
//o--------------------------------------------------------------------------
UI32 CItem::GetMoreB( void ) const
{
	return moreb;
}

//o--------------------------------------------------------------------------
//|	Function		-	UI08 GetCont( UI08 part )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns one of the words of the container's serial
//|						Valid values for part are 1->4.  If outside that, behaves
//|						as if it were 1
//o--------------------------------------------------------------------------
UI08 CItem::GetCont( UI08 part ) const
{
	SERIAL contserial = INVALIDSERIAL;
	if( contObj != NULL )
		contserial = contObj->GetSerial();

	switch( part )
	{
		default:
		case 1:		return (UI08)(contserial>>24);
		case 2:		return (UI08)(contserial>>16);
		case 3:		return (UI08)(contserial>>8);
		case 4:		return (UI08)(contserial%256);
	}
	return (UI08)(contserial>>24);
}

//o--------------------------------------------------------------------------
//|	Function		-	UI08 GetMore( UI08 part )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns one of the words of the more value
//|						Valid values for part are 1->4.  If outside that, behaves
//|						as if it were 1
//o--------------------------------------------------------------------------
UI08 CItem::GetMore( UI08 part ) const
{
	switch( part )
	{
		default:
		case 1:		return (UI08)(more>>24);
		case 2:		return (UI08)(more>>16);
		case 3:		return (UI08)(more>>8);
		case 4:		return (UI08)(more%256);
	}
	return (UI08)(more>>24);
}

//o--------------------------------------------------------------------------
//|	Function		-	UI08 GetMoreB( UI08 part )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns one of the words of the item's moreb value
//|						Valid values for part are 1->4.  If outside that, behaves
//|						as if it were 1
//o--------------------------------------------------------------------------
UI08 CItem::GetMoreB( UI08 part ) const
{
	switch( part )
	{
		default:
		case 1:		return (UI08)(moreb>>24);
		case 2:		return (UI08)(moreb>>16);
		case 3:		return (UI08)(moreb>>8);
		case 4:		return (UI08)(moreb%256);
	}
	return (UI08)(moreb>>24);
}

//o--------------------------------------------------------------------------
//|	Function		-	SetSerial( UI32 newValue )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the item's serial
//o--------------------------------------------------------------------------
void CItem::SetSerial( SERIAL newValue, ITEM c )
{
	nitemsp.Remove( newValue );
	cBaseObject::SetSerial( newValue );
	if( c != INVALIDSERIAL && newValue != INVALIDSERIAL )
		nitemsp.AddSerial( newValue, c );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool CItem::SetCont( cBaseObject *newCont )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Set's the item's container value to newValue
//|						Takes it out of the old container, and puts in the new
//|						Copes with being on paperdolls, ground and in containers
//|						Also copes with removing and adding to a map region
//|						Returns false if item needs deleting, true if fine
//o--------------------------------------------------------------------------
bool CItem::SetContSerial( SERIAL newSerial )
{
	if( newSerial != INVALIDSERIAL )
	{
		if( newSerial >= BASEITEMSERIAL )
			return SetCont( calcItemObjFromSer( newSerial ) );
		else
			return SetCont( calcCharObjFromSer( newSerial ) );
	}
	return SetCont( NULL );
}

bool CItem::SetCont( cBaseObject *newCont )
{
	Dirty();
	RemoveSelfFromCont();
	contObj = newCont;
	if( newCont != NULL )
	{
		if( newCont->GetObjType() == OT_CHAR )
		{
			CChar *charWearing = (CChar *)newCont;
			if( charWearing != NULL )
			{
				if( !charWearing->WearItem( this ) )
				{
					if( layer >= MAXLAYERS )
					{
						contObj = NULL;
						return false;
					}
#ifdef _DEBUG
					Console.Print( "Setting container failure in item %i with layer %i\n", serial, layer );
#endif
					contObj = NULL;
					return false;	// disable for now
				}
				Weight->addItemWeight( charWearing, this );
				return true;
			}
			else
			{
				contObj = NULL;
				return false;
			}
		}
		else
		{
			CItem *itemHolder = (CItem *)newCont;
			if( itemHolder != NULL )
			{
				// ok heres what hair/beards should be handled like (sereg)
				if( ( GetID() >= 0x203B ) && ( GetID() <= 0x204D ) )
				{
					CChar *itemPackOwner = getPackOwner( itemHolder );
					if( itemPackOwner != NULL )
					{
						CItem *oldItem = itemPackOwner->GetItemAtLayer( GetLayer() );
						if( oldItem != NULL )
							Items->DeleItem( oldItem );

						SetCont( itemPackOwner );
/*
						RemoveFromSight();
						Refresh();
*/
					}
				}
				else
					itemHolder->HoldItem( this );
				Weight->addItemWeight( itemHolder, this );
				return true;
			}
			else
			{
				contObj = NULL;
				return false;
			}
		}
	}
	else	// this is in case we're initing the default item
		MapRegion->AddItem( this );
	return true;
}

//o--------------------------------------------------------------------------
//|	Function		-	void SetMore( UI32 newValue )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the item's more value to newValue
//o--------------------------------------------------------------------------
void CItem::SetMore( UI32 newValue )
{
	more = newValue;
}

//o--------------------------------------------------------------------------
//|	Function		-	void SetMoreB( UI32 newValue )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the MoreB to newValue
//o--------------------------------------------------------------------------
void CItem::SetMoreB( UI32 newValue )
{
	moreb = newValue;
}

//o--------------------------------------------------------------------------
//|	Function		-	void SetOwner( cBaseObject *newValue )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the owner to newValue, ensuring it's removed from
//|						the old owner
//o--------------------------------------------------------------------------
void CItem::SetOwner( cBaseObject *newValue )
{
	RemoveSelfFromOwner();
	cBaseObject::SetOwner( newValue );
	AddSelfToOwner();
}

//o--------------------------------------------------------------------------
//|	Function		-	SetSpawn( SERIAL newValue, ITEM index )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the item's spawner to newValue, ensuring it's
//|						removed from the old hash entry (if any)
//|						Also ensures it is added to the hash table afterwards
//o--------------------------------------------------------------------------
void CItem::SetSpawn( SERIAL newValue, ITEM index )
{
	if( GetSpawn() != INVALIDSERIAL )
		nspawnsp.Remove( GetSpawn(), index );
	cBaseObject::SetSpawn( newValue );
	if( GetSpawn() != INVALIDSERIAL )
		nspawnsp.AddSerial( GetSpawn(), index );
}

//o--------------------------------------------------------------------------
//|	Function		-	void SetMore( UI08 newValue, UI08 part )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the more value's part byte to newValue 
//o--------------------------------------------------------------------------
void CItem::SetMore( UI08 newValue, UI08 part )
{
	UI08 part1 = (UI08)(more>>24);
	UI08 part2 = (UI08)(more>>16);
	UI08 part3 = (UI08)(more>>8);
	UI08 part4 = (UI08)(more%256);
	switch( part )
	{
		case 1:		part1 = newValue;		break;
		case 2:		part2 = newValue;		break;
		case 3:		part3 = newValue;		break;
		case 4:		part4 = newValue;		break;
	}
	more = (part1<<24) + (part2<<16) + (part3<<8) + part4;
}

//o--------------------------------------------------------------------------
//|	Function		-	void SetMoreB( UI08 newValue, UI08 part )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the moreb value's part byte to newValue
//o--------------------------------------------------------------------------
void CItem::SetMoreB( UI08 newValue, UI08 part )
{
	UI08 part1 = (UI08)(moreb>>24);
	UI08 part2 = (UI08)(moreb>>16);
	UI08 part3 = (UI08)(moreb>>8);
	UI08 part4 = (UI08)(moreb%256);
	switch( part )
	{
		case 1:		part1 = newValue;		break;
		case 2:		part2 = newValue;		break;
		case 3:		part3 = newValue;		break;
		case 4:		part4 = newValue;		break;
	}
	moreb = (part1<<24) + (part2<<16) + (part3<<8) + part4;
}

//o--------------------------------------------------------------------------
//|	Function		-	void SetMore( UI08 part1, UI08 part2, UI08 part3, UI08 part4 )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the more value.
//o--------------------------------------------------------------------------
void CItem::SetMore( UI08 part1, UI08 part2, UI08 part3, UI08 part4 )
{
	more = (part1<<24) + (part2<<16) + (part3<<8) + part4;
}

//o--------------------------------------------------------------------------
//|	Function		-	void SetMoreB( UI08 part1, UI08 part2, UI08 part3, UI08 part4 )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the moreb value.
//o--------------------------------------------------------------------------
void CItem::SetMoreB( UI08 part1, UI08 part2, UI08 part3, UI08 part4 )
{
	moreb = (part1<<24) + (part2<<16) + (part3<<8) + part4;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool isFree()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if the item is free
//o--------------------------------------------------------------------------
bool CItem::isFree( void ) const
{
	return ( (bools&0x01) == 0x01 );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool isDoorOpen()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if it's a door and is open
//o--------------------------------------------------------------------------
bool CItem::isDoorOpen( void ) const
{
	return ( (bools&0x02) == 0x02 );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool isPileable()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if the item is pileable
//o--------------------------------------------------------------------------
bool CItem::isPileable( void ) const
{
	return ( (bools&0x04) == 0x04 );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool isDyeable()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if the item is dyeable
//o--------------------------------------------------------------------------
bool CItem::isDyeable( void ) const
{
	return ( (bools&0x08) == 0x08 );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool isCorpse()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if the item is a corpse
//o--------------------------------------------------------------------------
bool CItem::isCorpse( void ) const
{
	return ( (bools&0x10) == 0x10 );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool isWipeable()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if is wipeable
//o--------------------------------------------------------------------------
bool CItem::isWipeable( void ) const
{
	return ( (bools&0x20) == 0x20 );
}

//o---------------------------------------------------------------------------o
//|   Function    -  bool isGuarded( void ) const
//|   Date        -  Unknown
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns true if the ITEM is guarded
//o---------------------------------------------------------------------------o
bool CItem::isGuarded( void ) const
{
	return ( (bools&0x40) == 0x40 );
}

//o---------------------------------------------------------------------------o
//|   Function    -  bool isSpawnerList( void ) const
//|   Date        -  Unknown
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns true if the item's spawn points to an NPC list
//|					 instead of an NPC
//o---------------------------------------------------------------------------o
bool CItem::isSpawnerList( void ) const
{
	return ( (bools&0x80) == 0x80 );
}

void CItem::SetFree( bool newValue )
{
	if( newValue )
		bools |= 0x01;
	else
		bools &= ~0x01;
}
void CItem::SetDoorOpen( bool newValue )
{
	if( newValue )
		bools |= 0x02;
	else
		bools &= ~0x02;
}
void CItem::SetPileable( bool newValue )
{
	if( newValue )
		bools |= 0x04;
	else
		bools &= ~0x04;
}
void CItem::SetDye( bool newValue )
{
	if( newValue )
		bools |= 0x08;
	else
		bools &= ~0x08;
}
void CItem::SetCorpse( bool newValue )
{
	if( newValue )
		bools |= 0x10;
	else
		bools &= ~0x10;
}
void CItem::SetWipeable( bool newValue )
{
	if( newValue )
		bools |= 0x20;
	else
		bools &= ~0x20;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void SetGuarded( bool newVal ) 
//|   Date        -  Unknown
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the ITEMs's guarded status
//o---------------------------------------------------------------------------o
void CItem::SetGuarded( bool newValue )
{
	if( newValue )
		bools |= 0x40;
	else
		bools &= ~0x40;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void SetSpawnerList( bool newVal ) 
//|   Date        -  24th January, 2002
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the item's spawnerlist property
//o---------------------------------------------------------------------------o
void CItem::SetSpawnerList( bool newValue )
{
	if( newValue )
		bools |= 0x80;
	else
		bools &= ~0x80;
}

const char *CItem::GetName2( void ) const
{
	return name2;
}
SERIAL CItem::GetCreator( void ) const
{
	return creator;
}
const char *CItem::GetDesc( void ) const
{
	return desc;
}

void CItem::SetName2( const char *newValue )
{
	strncpy( name2, newValue, MAX_NAME );
}
void CItem::SetCreator( SERIAL newValue )
{
	creator = newValue;
}
void CItem::SetDesc( const char *newValue )
{
	strncpy( desc, newValue, 100 );
}

void CItem::IncX( SI16 newValue )
{
	x += newValue;
	Dirty();
}
void CItem::IncY( SI16 newValue )
{
	y += newValue;
	Dirty();
}
void CItem::IncZ( SI16 newValue )
{
	z += newValue;
	Dirty();
}
void CItem::SetLocation( const cBaseObject *toSet )
{
	if( GetCont() == NULL )
		MapRegion->RemoveItem( this );
	if( toSet==NULL )
		return;
	x = toSet->GetX();
	y = toSet->GetY();
	z = toSet->GetZ();
	worldNumber = toSet->WorldNumber();
	if( GetCont() == NULL )
	{
		MapRegion->AddItem( this );
		CMultiObj *mMulti = findMulti( this );
		SetMulti( mMulti );
	}
// Thyme - Necessary for Region Spawning with Items to work properly
	if( GetCont() != NULL )
		ShouldSave( true );
	Dirty();
}
void CItem::SetLocation( SI16 newX, SI16 newY, SI08 newZ )
{
	if( GetCont() == NULL )
		MapRegion->RemoveItem( this );
	x = newX;
	y = newY;
	z = newZ;
	if( GetCont() == NULL )
	{
		MapRegion->AddItem( this );
		CMultiObj *mMulti = findMulti( this );
		SetMulti( mMulti );
	}
// Thyme - Necessary for Region Spawning with Items to work properly
	if( GetCont() != NULL )
		ShouldSave( true );
	Dirty();
}

void CItem::SetLocation( SI16 newX, SI16 newY, SI08 newZ, UI08 world )
{
	if( GetCont() == NULL )
		MapRegion->RemoveItem( this );
	x = newX;
	y = newY;
	z = newZ;
	worldNumber = world;
	if( GetCont() == NULL )
	{
		MapRegion->AddItem( this );
		CMultiObj *mMulti = findMulti( this );
		SetMulti( mMulti );
	}
// Thyme - Necessary for Region Spawning with Items to work properly
	if( GetCont() != NULL )
		ShouldSave( true );
	Dirty();
}

UI08 CItem::GetLayer( void ) const
{
	return layer;
}
void CItem::SetLayer( UI08 newValue ) 
{
	if( contObj != NULL && contObj->GetObjType() == OT_CHAR )	// if we're on a char
	{
		CChar *charAffected = (CChar *)contObj;
		if( charAffected != NULL )
		{
//			if( !charAffected->TakeOffItem( GetLayer() ) )
//			{
//				Console.Error( "Char %s(%X) was never wearing item on layer %u\n", charAffected->GetName(), contserial, GetLayer() );
//			}
			if( layer != 0 )
				charAffected->TakeOffItem( layer );
			layer = newValue;
			charAffected->WearItem( this );
			return;
		}
	}
	layer = newValue;
}

UI08 CItem::GetType( void ) const
{
	return type;
}
UI08 CItem::GetType2( void ) const
{
	return type2;
}

void CItem::SetType( UI08 newValue )
{
	type = newValue;
}
void CItem::SetType2( UI08 newValue )
{
	type2 = newValue;
}

SI08 CItem::GetOffSpell( void ) const
{
	return offspell;
}
void CItem::SetOffSpell( SI08 newValue )
{
	offspell = newValue;
}

UI32 CItem::GetMoreX( void ) const
{
	return morex;
}
UI32 CItem::GetMoreY( void ) const
{
	return morey;
}
UI32 CItem::GetMoreZ( void ) const
{
	return morez;
}

void CItem::SetMoreX( UI32 newValue )
{
	morex = newValue;
}
void CItem::SetMoreY( UI32 newValue )
{
	morey = newValue;
}
void CItem::SetMoreZ( UI32 newValue )
{
	morez = newValue;
}

UI16 CItem::GetAmount( void ) const
{
	return amount;
}
void CItem::SetAmount( UI32 newValue )
{
	cBaseObject *getCont = GetCont();
	if( getCont != NULL )
		Weight->subtractItemWeight( getCont, this );

	if( newValue > MAX_STACK )
		amount = MAX_STACK;
	else
		amount = static_cast<UI16>(newValue);

	if( getCont != NULL )
		Weight->addItemWeight( getCont, this );
	Dirty();
}
UI08 CItem::GetDoorDir( void ) const
{
	return doordir;
}
void CItem::SetDoorDir( UI08 newValue )
{
	doordir = newValue;
}

SI16 CItem::GetMaxHP( void ) const
{
	return maxhp;
}

void CItem::SetMaxHP( SI16 newValue )
{
	maxhp = newValue;
}

UI08 CItem::GetSpeed( void ) const
{
	return spd;
}
void CItem::SetSpeed( UI08 newValue )
{
	spd = newValue;
}

SI08 CItem::GetMovable( void ) const
{
	return movable;
}
void CItem::SetMovable( SI08 newValue )
{
	movable = newValue;
}

TIMERVAL CItem::GetGateTime( void ) const
{
	return gatetime;
}
TIMERVAL CItem::GetDecayTime( void ) const
{
	return decaytime;
}
TIMERVAL CItem::GetMurderTime( void ) const
{
	return murdertime;
}

void CItem::SetGateTime( TIMERVAL newValue )
{
	gatetime = newValue;
}
void CItem::SetDecayTime( TIMERVAL newValue )
{
	decaytime = newValue;
}
void CItem::SetMurderTime( TIMERVAL newValue )
{
	murdertime = newValue;
}

UI08 CItem::GetPriv( void ) const
{
	return priv;
}
void CItem::SetPriv( UI08 newValue )
{
	priv = newValue;
}

SI32 CItem::GetSellValue( void ) const
{
	return sellValue;
}
void CItem::SetSellValue( SI32 newValue )
{
	sellValue = newValue;
}

SI32 CItem::GetBuyValue( void ) const
{
	return buyValue;
}
void CItem::SetBuyValue( SI32 newValue )
{
	buyValue = newValue;
}

UI16 CItem::GetRestock( void ) const
{
	return restock;
}
void CItem::SetRestock( UI16 newValue )
{
	restock = newValue;
}

ARMORCLASS CItem::GetArmourClass( void ) const
{
	return armorClass;
}
void CItem::SetArmourClass( ARMORCLASS newValue )
{
	armorClass = newValue;
}

SI08 CItem::GetRank( void ) const
{
	return rank;
}
void CItem::SetRank( SI08 newValue )
{
	rank = newValue;
}

SI16 CItem::GetGood( void ) const
{
	return good;
}
void CItem::SetGood( SI16 newValue )
{
	good = newValue;
}

SI32 CItem::GetRndValueRate( void ) const
{
	return rndvaluerate;
}
void CItem::SetRndValueRate( SI32 newValue )
{
	rndvaluerate = newValue;
}

SI08 CItem::GetMadeWith( void ) const
{
	return madewith;
}
void CItem::SetMadeWith( SI08 newValue )
{
	madewith = newValue;
}

SERIAL CItem::GetGlow( void ) const
{
	return glow;
}
void CItem::SetGlow( SERIAL newValue )
{
	glow = newValue;
}

COLOUR CItem::GetGlowColour( void ) const
{
	return glowColour;
}

void CItem::SetGlowColour( COLOUR newValue )
{
	glowColour = newValue;
}

UI08 CItem::GetGlowEffect( void ) const
{
	return glow_effect;
}
void CItem::SetGlowEffect( UI08 newValue )
{
	glow_effect = newValue;
}

void CItem::IncHP( SI16 newValue )
{
	hitpoints += newValue;
}

void CItem::IncDef( UI16 newValue )
{
	def += newValue;
}

void CItem::IncID( SI16 incAmount )
{
	id += incAmount;
	Dirty();
}

bool CItem::HoldItem( CItem *toHold )
{
	for( UI32 counter = 0; counter < Contains.size(); ++counter )
	{
		if( Contains[counter] == toHold )	// same pointer ie same thing
		{
			Console.Warning( 3, "%s(%X) already contains %s(%X)\n", GetName().c_str(), GetSerial(), toHold->GetName().c_str(), toHold->GetSerial() );
			return false;
		}
	}

	size_t index = Contains.size();
	Contains.resize( Contains.size() + 1 );
	Contains[index] = toHold;
	return true;
}
bool CItem::ReleaseItem( CItem *index )
{
	for( size_t counter = 0; counter < Contains.size(); ++counter )
	{
		if( Contains[counter] == index )
		{
			for( size_t counter2 = counter; counter2 < (Contains.size()-1); ++counter2 )
				Contains[counter2] = Contains[counter2+1];
			Contains.resize( Contains.size() - 1 );
			if( iCounter >= 0 && counter <= iCounter )	// if what we're deleting is in our list, and before the current item
				--iCounter;
			return true;
		}
	}
	return false;
}

CItem *CItem::FirstItem( void ) const
{
	iCounter = 0;
	if( static_cast<unsigned int>(iCounter) >= Contains.size() )
		return NULL;
	return Contains[iCounter];
}

CItem *CItem::NextItem( void ) const
{
	++iCounter;
	if( static_cast<unsigned int>(iCounter) >= Contains.size() )
		return NULL;
	return Contains[iCounter];
}

bool CItem::FinishedItems( void ) const
{
	return ( static_cast<unsigned int>(iCounter) >= Contains.size() );
}

size_t CItem::NumItems( void ) const
{
	return Contains.size();
}

CItem *CItem::GetItemObj( ITEM index ) const
{
	if( index >= Contains.size() )
		return NULL;
	return Contains[index];
}


bool CItem::WillDecay( void ) const
{
	return ( (priv&0x01) == 0x01 );
}

UI08 CItem::IsFieldSpell( void ) const
{
	switch( id )
	{
		case 0x3996:
		case 0x398C:	return 1;// fire
		case 0x3915:
		case 0x3920:	return 2;// poison
		case 0x3979:
		case 0x3967:	return 3;// paralyze
		case 0x3956:
		case 0x3946:	return 4;// energy
		default:		return 0;
	}
}
bool CItem::IsLockedDown( void ) const
{
	return ( movable == 3 );
}
bool CItem::IsShieldType( void ) const
{
	if( id >= 0x1B72 && id <= 0x1B7B )
		return true;
	if( id >= 0x1BC3 && id <= 0x1BC5 )
		return true;
	return false;
}

bool CItem::CanBeLockedDown( void ) const
{
	if( type == 12 || type == 13 )	// can't lock down a door
		return false;
	if( IsFieldSpell() )			// can't lock down a field spell
		return false;
	if( id == 0x0BD2 || type == 203 )	// house sign
		return false;
	if( IsLockedDown() )	// can't lock down if already locked down
		return false;
	return true;
}

void CItem::LockDown( void )
{
	movable = 3;
}

bool CItem::Save( std::ofstream &outStream )
{
	if( isFree() )
		return false;
	CMultiObj *multi	= NULL;
	cSocket *k			= NULL;

	if( WillDecay() && GetCont() == NULL )
	{
		if( GetDecayTime() == 0 )
			SetDecayTime( BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->SystemTimer( DECAY )) ) );
		else if( GetDecayTime() < cwmWorldState->GetUICurrentTime() )
		{
			if( isCorpse() )
			{	// Adjust this to work with the native pointer array
				for( CItem *j = FirstItem(); !FinishedItems(); j = NextItem() )
				{
					if( j != NULL && !j->isFree() )
					{
						if( j->GetLayer() != 0x0B && j->GetLayer() != 0x10 )
						{
							j->SetCont( NULL );
							j->SetLocation( this );
						}
						else
							Items->DeleItem( j );
					}
				}
			}
			else
			{
				if( GetMultiObj() == NULL )
				{
					multi = findMulti( this );
					if( multi == NULL )
					{
						Items->DeleItem( this );
						return false;
					}
				}
			}
		}
	}
	if( GetCont() != NULL || ( GetX() > 0 && GetX() < 6144 && GetY() < 4096 ) )
	{
		DumpHeader( outStream );
		DumpBody( outStream );

		DumpFooter( outStream );

		//Console << "Contains: " << (UI32)Contains.size() << myendl;
		for( UI32 tempCounter = 0; tempCounter < Contains.size(); ++tempCounter )
		{
			if( Contains[tempCounter] != NULL && Contains[tempCounter]->ShouldSave() )
				Contains[tempCounter]->Save( outStream );
		}
	}
	return true;
}

void CItem::RemoveSelfFromOwner( void )
{
	cBaseObject *oldOwner = GetOwnerObj();
	if( oldOwner != NULL )
	{
		if( oldOwner->GetObjType() == OT_CHAR )
			((CChar *)oldOwner)->RemoveOwnedItem( this );
	}
}
void CItem::AddSelfToOwner( void )
{
	cBaseObject *newOwner = GetOwnerObj();
	if( newOwner == NULL )
		return;
	if( newOwner->GetSerial() != GetSerial() )
	{
		if( newOwner->GetObjType() == OT_CHAR )
			((CChar *)newOwner)->AddOwnedItem( this );
	}
}

void CItem::RemoveSelfFromCont( void )
{
	try
	{
		RemoveFromSight();
		if( contObj != NULL )
		{
			if( contObj->GetObjType() == OT_CHAR )	// it's a char!
			{
				CChar *targChar = (CChar *)contObj;
				if( targChar != NULL )
				{
					Weight->subtractItemWeight( targChar, this );
					targChar->TakeOffItem( GetLayer() );
					//if( !targChar->TakeOffItem( GetLayer() ) )
					//Console << "Char " << targChar->GetName() << "(" << contserial << ") was never wearing item on layer " << (SI16)GetLayer() << myendl;
				}
			}
			else
			{
				CItem *targItem = (CItem *)contObj;;
				if( targItem != NULL )
				{
					Weight->subtractItemWeight( targItem, this );
					targItem->ReleaseItem( this );
					//if( !targItem->ReleaseItem( this ) )
					//Console.Error( "Error removing %s(%X) from %s(%X)\n", GetName(), GetSerial(), targItem->GetName(), contserial );
				}
			}
		}
		else
			MapRegion->RemoveItem( this );
	}
	catch( ... )
	{
	}
}

CItem * CItem::Dupe( void )
{
	ITEM targetOff;
	CItem *target = Items->MemItemFree( targetOff, true );
	if( target == NULL )
		return NULL;

	target->postLoaded = postLoaded;
	target->SetAmount( GetAmount() );
	target->SetArmourClass( GetArmourClass() );
	target->SetCarve( GetCarve() );
	target->SetColour( GetColour() );
	target->SetLayer( GetLayer() );
	target->SetCont( GetCont() );
	target->SetCreator( GetCreator() );
	target->SetCorpse( isCorpse() );
	target->SetDecayTime( GetDecayTime() );
	target->SetDesc( GetDesc() );
	target->SetDoorDir( GetDoorDir() );
	target->SetDexterity( GetDexterity() );
	target->Dexterity2( Dexterity2() );
	target->SetDef( GetDef() );
	target->SetDir( GetDir() );
	target->SetDisabled( IsDisabled() );
	target->SetDoorOpen( isDoorOpen() );
	target->SetDye( isDyeable() );
	target->SetFame( GetFame() );
	target->SetFree( isFree() );
	target->SetGateTime( GetGateTime() );
	target->SetGlow( GetGlow() );
	target->SetGlowColour( GetGlowColour() );
	target->SetGlowEffect( GetGlowEffect() );
	target->SetGood( GetGood() );
	target->SetHiDamage( GetHiDamage() );
	target->SetHP( GetHP() );
	target->SetID( GetID() );
	target->SetIntelligence( GetIntelligence() );
	target->Intelligence2( Intelligence2() );
	target->SetKarma( GetKarma() );
	target->SetKills( GetKills() );
	target->SetLocation( this );
	target->SetLoDamage( GetLoDamage() );
	target->SetMadeWith( GetMadeWith() );
	target->SetMovable( GetMovable() );
	target->SetMana( GetMana() );
	target->SetMaxHP( GetMaxHP() );
	target->SetMore( GetMore() );
	target->SetMoreX( GetMoreX() );
	target->SetMoreY( GetMoreY() );
	target->SetMoreZ( GetMoreZ() );
	target->SetMulti( GetMultiObj() );
	target->SetMurderTime( GetMurderTime() );
	target->SetName( GetName() );
	target->SetName2( GetName2() );
	target->SetOffSpell( GetOffSpell() );
	target->SetOwner( GetOwnerObj() );
	target->SetPileable( isPileable() );
	target->SetPoisoned( GetPoisoned() );
	target->SetRace( GetRace() );
	target->SetRank( GetRank() );
	target->SetRestock( GetRestock() );
	target->SetRndValueRate( GetRndValueRate() );
	target->SetScriptTrigger( GetScriptTrigger() );
	target->SetSpawn( GetSpawn(), targetOff );
	target->SetSpeed( GetSpeed() );
	target->SetStamina( GetStamina() );
	target->SetStrength( GetStrength() );
	target->Strength2( Strength2() );
	target->SetTitle( GetTitle() );
	target->SetType( GetType() );
	target->SetType2( GetType2() );
	target->SetBuyValue( GetBuyValue() );
	target->SetSellValue( GetSellValue() );
	target->SetVisible( GetVisible() );
	target->SetWeight( GetWeight() );
	target->SetWipeable( isWipeable() );
	target->SetPriv( GetPriv() );

	target->SetWeatherDamage( LIGHT, GetWeatherDamage( LIGHT ) );
	target->SetWeatherDamage( RAIN, GetWeatherDamage( RAIN ) );
	target->SetWeatherDamage( COLD, GetWeatherDamage( COLD ) );
	target->SetWeatherDamage( HEAT, GetWeatherDamage( HEAT ) );
	target->SetWeatherDamage( LIGHTNING, GetWeatherDamage( LIGHTNING ) );
	target->SetWeatherDamage( SNOW, GetWeatherDamage( SNOW ) );

	return target;
}

bool CItem::GetWeatherDamage( WeatherType effectNum ) const
{
	switch( effectNum )
	{
		case LIGHT:			return ( (weatherBools&0x01) == 0x01 );
		case RAIN:			return ( (weatherBools&0x02) == 0x02 );
		case COLD:			return ( (weatherBools&0x04) == 0x04 );
		case HEAT:			return ( (weatherBools&0x08) == 0x08 );
		case LIGHTNING:		return ( (weatherBools&0x10) == 0x10 );
		case SNOW:			return ( (weatherBools&0x20) == 0x20 );
		default:			return false;
	}
	return false;
}

void CItem::SetWeatherDamage( WeatherType effectNum, bool value )
{
	if( value )
	{
		switch( effectNum )
		{
			case LIGHT:			weatherBools |= 0x01;			break;
			case RAIN:			weatherBools |= 0x02;			break;
			case COLD:			weatherBools |= 0x04;			break;
			case HEAT:			weatherBools |= 0x08;			break;
			case LIGHTNING:		weatherBools |= 0x10;			break;
			case SNOW:			weatherBools |= 0x20;			break;
			default:			break;
		}
	}
	else
	{
		switch( effectNum )
		{
			case LIGHT:			weatherBools &= ~0x01;			break;
			case RAIN:			weatherBools &= ~0x02;			break;
			case COLD:			weatherBools &= ~0x04;			break;
			case HEAT:			weatherBools &= ~0x08;			break;
			case LIGHTNING:		weatherBools &= ~0x10;			break;
			case SNOW:			weatherBools &= ~0x20;			break;
			default:			break;
		}
	}
}

bool CItem::DumpHeader( std::ofstream &outStream ) const
{
	outStream << "[ITEM]" << std::endl;
	return true;
}


bool CItem::DumpBody( std::ofstream &outStream ) const
{
	std::string destination; 
	std::ostringstream dumping( destination ); 

	cBaseObject::DumpBody( outStream );
	dumping << "Layer=" << "0x" << std::hex << (SI16)GetLayer() << std::endl;
	dumping << "Cont=" << "0x" << GetContSerial() << std::endl;
	dumping << "More=" << "0x" << GetMore() << "," << "0x" << GetMoreB() << std::endl;
	dumping << "Name2=" << std::dec << GetName2() << std::endl;
	dumping << "Creator=" << "0x" << std::hex << GetCreator() << std::endl;
	dumping << "Desc=" << std::dec << GetDesc() << std::endl;
	dumping << "Type=" << (SI16)GetType() << "," << (SI16)GetType2() << std::endl;
	dumping << "Offspell=" << (SI16)GetOffSpell() << std::endl;
	dumping << "MoreXYZ=" << "0x" << std::hex << GetMoreX() << "," << "0x" << GetMoreY() << "," << "0x" << GetMoreZ() << std::endl;
	dumping << "Amount=" << std::dec;
	if( GetAmount() > MAX_STACK )
		dumping << MAX_STACK << std::endl;
	else
		dumping << GetAmount() << std::endl;
	dumping << "Doorflag=" << "0x" << std::hex << (SI16)GetDoorDir() << std::endl;
	dumping << "MaxHP=" << std::dec << GetMaxHP() << std::endl;
	dumping << "Speed=" << (SI16)GetSpeed() << std::endl;
	dumping << "Movable=" << (SI16)GetMovable() << std::endl;
	dumping << "Priv=" << (SI16)GetPriv() << std::endl;
	dumping << "Value=" << GetBuyValue() << "," << GetSellValue() << std::endl;
	dumping << "Restock=" << GetRestock() << std::endl;
	dumping << "AC=" << (SI16)GetArmourClass() << std::endl;
	dumping << "Rank=" << (SI16)GetRank() << std::endl;
	dumping << "Sk_Made=" << (SI16)GetMadeWith() << std::endl;
	dumping << "Enhanced=" << enhanced << std::endl;
	dumping << "Pileable=" << (SI16)(isPileable()?1:0) << std::endl;
	dumping << "Dyeable=" << (SI16)(isDyeable()?1:0) << std::endl;
	dumping << "Corpse=" << (SI16)(isCorpse()?1:0) << std::endl;
	dumping << "Wipe=" << (SI16)(isWipeable()?1:0) << std::endl;
	dumping << "Good=" << GetGood() << std::endl;
	dumping << "Glow=" << "0x" << std::hex << GetGlow() << std::endl;
	dumping << "GlowBC=" << "0x" << GetGlowColour() << std::endl;
	dumping << "GlowType=" << std::dec << (SI16)GetGlowEffect() << std::endl;
	dumping << "RaceDamage=" << (SI16)(GetWeatherDamage( LIGHT ) ? 1 : 0) << "," << (SI16)(GetWeatherDamage( RAIN ) ? 1 : 0) << ","
	<< (SI16)(GetWeatherDamage( HEAT ) ? 1 : 0) << "," << (SI16)(GetWeatherDamage( COLD ) ? 1 : 0) << ","
	<< (SI16)(GetWeatherDamage( SNOW ) ? 1 : 0) << "," << (SI16)(GetWeatherDamage( LIGHTNING ) ? 1 : 0) << std::endl;
	dumping << "EntryMadeFrom=" << EntryMadeFrom() << std::endl;

	//dumping << std::endl << std::endl;
	outStream << dumping.str();

	return true;
}

bool CItem::HandleLine( UString &tag, UString &data )
{
	bool rvalue = false;
	if( !( rvalue = cBaseObject::HandleLine( tag, data ) ) )
	{
		switch( (tag.data()[0]) )
		{
			case 'A':
				if( tag.upper() == "AMOUNT" )
				{
					amount = data.toUShort();
					rvalue = true;
				}
				else if( tag.upper() == "AC" )
				{
					SetArmourClass( data.toUByte() );
					rvalue = true;
				}
				break;
			case 'C':
				if( tag.upper() == "CONT" )
				{
					contObj = (cBaseObject *)data.toULong();
					rvalue = true;
				}
				else if( tag.upper() == "CREATOR" || tag.upper() == "CREATER" )
				{
					SetCreator( data.toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "CORPSE" )
				{
					SetCorpse( data.toUByte() == 1 );
					rvalue = true;
				}
				else if( tag.upper() == "COLD" )
				{
					SetWeatherDamage( COLD, data.toUByte() == 1 );
					rvalue = true;
				}
				break;
			case 'D':
				if( tag.upper() == "DESC" )
				{
					SetDesc( data.c_str() );
					rvalue = true;
				}
				else if( tag.upper() == "DOORFLAG" )
				{
					SetDoorDir( data.toUByte() );
					rvalue = true;
				}
				else if( tag.upper() == "DYEABLE" )
				{
					SetDye( data.toUByte() == 1 );
					rvalue = true;
				}
				break;
			case 'E':
				if( tag.upper() == "ENHANCED" )
				{
					enhanced = data.toULong();
					rvalue = true;
				}
				else if( tag.upper() == "ENTRYMADEFROM" )
				{
					EntryMadeFrom( data.toUShort() );
					rvalue = true;
				}
				break;
			case 'G':
				if( tag.upper() == "GLOWTYPE" )
				{
					SetGlowEffect( data.toUByte() );
					rvalue = true;
				}
				else if( tag.upper() == "GLOWBC" )
				{
					SetGlowColour( data.toUShort() );
					rvalue = true;
				}
				else if( tag.upper() == "GLOW" )
				{
					SetGlow( data.toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "GOOD" )
				{
					SetGood( data.toShort() );
					rvalue = true;
				}
				break;
			case 'H':
				if( tag.upper() == "HEAT" )
				{
					SetWeatherDamage( HEAT, data.toUByte() == 1 );
					rvalue = true;
				}
				break;
			case 'L':
				if( tag.upper() == "LAYER" )
				{
					layer = data.toUByte();
					rvalue = true;
				}
				else if( tag.upper() == "LIGHT" )
				{
					SetWeatherDamage( LIGHT, data.toUShort() == 1 );
					rvalue = true;
				}
				else if( tag.upper() == "LIGHTNING" )
				{
					SetWeatherDamage( LIGHTNING, data.toUShort() == 1 );
					rvalue = true;
				}
				break;
			case 'M':
				if( tag.upper() == "MORE" )
				{
					if( data.sectionCount( "," ) != 0 )
					{
						SetMore( data.section( ",", 0, 0 ).stripWhiteSpace().toULong() );
						SetMoreB( data.section( ",", 1, 1 ).stripWhiteSpace().toULong() );
					}
					else
						SetMore( data.toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "MORE2" )
				{
					SetMoreB( data.toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "MURDERER" )
					rvalue = true;
				else if( tag.upper() == "MOREXYZ" )
				{
					SetMoreX( data.section( ",", 0, 0 ).stripWhiteSpace().toULong() );
					SetMoreY( data.section( ",", 1, 1 ).stripWhiteSpace().toULong() );
					SetMoreZ( data.section( ",", 2, 2 ).stripWhiteSpace().toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "MOREX" )
				{
					SetMoreX( data.toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "MOREY" )
				{
					SetMoreY( data.toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "MOREZ" )
				{
					SetMoreZ( data.toULong() );
					rvalue = true;
				}
				else if( tag.upper() == "MOVABLE" )
				{
					SetMovable( data.toByte() );
					rvalue = true;
				}
				else if( tag.upper() == "MAXHP" )
				{
					SetMaxHP( data.toShort() );
					rvalue = true;
				}
				break;
			case 'N':
				if( tag.upper() == "NAME2" )
				{
					SetName2( data.c_str() );
					rvalue = true;
				}
				break;
			case 'O':
				if( tag.upper() == "OFFSPELL" )
				{
					SetOffSpell( data.toByte() );
					rvalue = true;
				}
				break;
			case 'P':
				if( tag.upper() == "PRIV" )
				{
					SetPriv( data.toUByte() );
					rvalue = true;
				}
				else if( tag.upper() == "PILEABLE" )
				{
					SetPileable( data.toUByte() == 1 );
					rvalue = true;
				}
				break;
			case 'R':
				if( tag.upper() == "RESTOCK" )
				{
					SetRestock( data.toUShort() );
					rvalue = true;
				}
				else if( tag.upper() == "RACEDAMAGE" )
				{
					SetWeatherDamage( LIGHT, data.section( ",", 0, 0 ).stripWhiteSpace().toUByte() == 1 );
					SetWeatherDamage( RAIN, data.section( ",", 1, 1 ).stripWhiteSpace().toUByte() == 1 );
					SetWeatherDamage( HEAT, data.section( ",", 2, 2 ).stripWhiteSpace().toUByte() == 1 );
					SetWeatherDamage( COLD, data.section( ",", 3, 3 ).stripWhiteSpace().toUByte() == 1 );
					SetWeatherDamage( SNOW, data.section( ",", 4, 4 ).stripWhiteSpace().toUByte() == 1 );
					SetWeatherDamage( LIGHTNING, data.section( ",", 5, 5 ).stripWhiteSpace().toUByte() == 1 );
					rvalue = true;	
				}
				else if( tag.upper() == "RANK" )
				{
					SetRank( data.toByte() );
					rvalue = true;
				}
				else if( tag.upper() == "RAIN" )
				{
					SetWeatherDamage( RAIN, data.toUByte() == 1 );
					rvalue = true;
				}
				break;
			case 'S':
				if( tag.upper() == "SPEED" )
				{
					SetSpeed( data.toUByte() );
					rvalue = true;
				}
				else if( tag.upper() == "SK_MADE" )
				{
					SetMadeWith( data.toByte() );
					rvalue = true;
				}
				else if( tag.upper() == "SNOW" )
				{
					SetWeatherDamage( SNOW, data.toUByte() == 1 );
					rvalue = true;
				}
				break;
			case 'T':
				if( tag.upper() == "TYPE" )
				{
					if( data.sectionCount( "," ) != 0 )
					{
						SetType( data.section( ",", 0, 0 ).stripWhiteSpace().toUByte() );
						SetType2( data.section( ",", 1, 1 ).stripWhiteSpace().toUByte() );
					}
					else
						SetType( data.toUByte() );
					rvalue = true;
				}
				else if( tag.upper() == "TYPE2" )
				{
					SetType2( data.toUByte() );
					rvalue = true;
				}
				break;
			case 'V':
				if( tag.upper() == "VALUE" )
				{
					if( data.sectionCount( "," ) != 0 )
					{
						SetBuyValue( data.section( ",", 0, 0 ).stripWhiteSpace().toLong() );
						SetSellValue( data.section( ",", 1, 1 ).stripWhiteSpace().toLong() );
					}
					else
					{
						SetBuyValue( data.toLong() );
						SetSellValue( (data.toLong() / 2) );
					}
					rvalue = true;
				}
				break;
			case 'W':
				if( tag.upper() == "WIPE" )
				{
					SetWipeable( data.toUByte() == 1 );
					rvalue = true;
				}
				break;
		}
	}
	return rvalue;
}


bool CItem::Load( std::ifstream &inStream, ITEM arrayOffset )
{
	cBaseObject::Load( inStream );
	return LoadRemnants( arrayOffset );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool LoadRemnants( int arrayOffset )
//|	Date			-	21st January, 2002
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	After handling data specific load, other parts go here
//o--------------------------------------------------------------------------
bool CItem::LoadRemnants( int arrayOffset )
{
	if( cwmWorldState->GetItemCount2() <= serial )
		cwmWorldState->SetItemCount2( serial + 1 );
	SetSerial( serial, arrayOffset );

	// Tauriel adding region pointers
	if( (SERIAL)contObj == INVALIDSERIAL )
	{
		MapRegion->AddItem( this );
		if( GetX() < 0 || GetY() < 0 || GetX() > 6144 || GetY() > 4096 )
			return false;
	}
	return true;
}

bool CItem::IsContType( void ) const
{
	if( GetLayer() == 0x15 || GetLayer() == 0x1D )
		return true;
	switch( GetType() )
	{
		case 1:
		case 8:
		case 63:
		case 64:
		case 65:
		case 87:	return true;
		default:	return false;
	}
	return false;
}

void CItem::PostLoadProcessing( UI32 index )
{
	cBaseObject::PostLoadProcessing( index );
	// Add item weight if item doesn't have it yet
	if( GetWeight() < 0 || GetWeight() > MAX_WEIGHT )
		SetWeight( Weight->calcWeight( this ) );

	SERIAL tempSerial	= (SERIAL)contObj;
	contObj				= NULL;
	if( tempSerial != INVALIDSERIAL )
	{
		if( tempSerial >= BASEITEMSERIAL )
			contObj = calcItemObjFromSer( tempSerial );
		else
			contObj	= calcCharObjFromSer( tempSerial );
	}
	SetCont( contObj );

	UI08 tempLayer = layer;
	layer = 0;
	SetLayer( tempLayer );

	Items->StoreItemRandomValue( this, 0xFF );
	CheckItemIntegrity();
	postLoaded = true;
}

void CItem::CheckItemIntegrity( void )
{
	SERIAL getSerial = GetSerial();
	if( getSerial == INVALIDSERIAL )
	{
		Console.Warning( 2, "Item (%s) has an invalid serial number, Deleting", GetName().c_str() );
		Items->DeleItem( this );
		return;
	}

	if( getSerial == GetContSerial() )
	{
		Console.Warning( 2, "Item %X (%s) has dangerous container value, Auto-Correcting", getSerial, GetName().c_str() );
		SetCont( NULL );
	}
	if( getSerial == GetOwner() )
	{
		Console.Warning( 2, "Item %X (%s) has dangerous owner value, Auto-Correcting", getSerial, GetName().c_str() );
		SetOwner( NULL );
	}
	if( getSerial == GetSpawn() )
	{
		Console.Warning( 2, "Item %X (%s) has dangerous spawner value, Auto-Correcting", getSerial, GetName().c_str() );
		SetSpawn( INVALIDSERIAL, calcItemFromSer( getSerial ) );
		nspawnsp.Remove( getSerial, calcItemFromSer( getSerial ) );
	}
}

bool CItem::isDecayable( void ) const
{
	return ( (priv&0x01) == 0x01 );
}
bool CItem::isNewbie( void ) const
{
	return ( (priv&0x02) == 0x02 );
}
bool CItem::isDispellable( void ) const
{
	return ( (priv&0x04) == 0x04 );
}

void CItem::SetDecayable( bool newValue )
{
	if( newValue )
		priv |= 0x01;
	else
		priv &= ~0x01;
}
void CItem::SetNewbie( bool newValue )
{
	if( newValue )
		priv |= 0x02;
	else
		priv &= ~0x02;
}
void CItem::SetDispellable( bool newValue )
{
	if( newValue )
		priv |= 0x04;
	else
		priv &= ~0x04;
}

//o--------------------------------------------------------------------------
//|	Function		-	UI16 EntryMadeFrom()
//|	Date			-	13 September, 2001
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the create.scp entry the item was made from
//o--------------------------------------------------------------------------
UI16 CItem::EntryMadeFrom( void ) const
{
	return entryMadeFrom;
}

//o--------------------------------------------------------------------------
//|	Function		-	void EntryMadeFrom( UI16 newValue )
//|	Date			-	13 September, 2001
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the create.scp entry the item was made from
//o--------------------------------------------------------------------------
void CItem::EntryMadeFrom( UI16 newValue )
{
	entryMadeFrom = newValue;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool IsMetalType( void ) const
//|	Date			-	13 November, 2001
//|	Programmer		-	Bel-CMC
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if it's a metal object
//o--------------------------------------------------------------------------
bool CItem::IsMetalType( void ) const
{
	if( id >= 0x1B72 && id <= 0x1B7B )
		return true;
	else if( id >= 0x1BC3 && id <= 0x1BC5 )
		return true;
	else if( id >= 0x13AF && id <= 0x13B0 )
		return true;
	else if( id >= 0x13B5 && id <= 0x13C4 )
		return true;
	else if( id >= 0x13E5 && id <= 0x13F2 )
		return true;
	else if( id >= 0x13FA && id <= 0x13FB )
		return true;
	else if( id >= 0x13FE && id <= 0x141A )
		return true;
	else if( id >= 0x1438 && id <= 0x1443 )
		return true;
	else if( id >= 0x0F43 && id <= 0x0F4E )
		return true;
	else if( id >= 0x0F51 && id <= 0x0F52 )
		return true;
	else if( id >= 0x0F5C && id <= 0x0F63 )
		return true;
	else
		return false;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool IsLeatherType( void ) const
//|	Date			-	13 November, 2001
//|	Programmer		-	Bel-CMC
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if it's a leather object
//o--------------------------------------------------------------------------
bool CItem::IsLeatherType( void ) const
{
	if( id >= 0x13C6 && id <= 0x13E2 )
		return true;
	if( id >= 0x144E && id <= 0x1457 )
		return true;
	return false;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool IsBowType( void ) const
//|	Date			-	13 November, 2001
//|	Programmer		-	Bel-CMC
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if it's a bow object
//o--------------------------------------------------------------------------
bool CItem::IsBowType( void ) const
{
	if( id >= 0x0F4F && id <= 0x0F50 )
		return true;
	if( id >= 0x13B1 && id <= 0x13B2 )
		return true;
	if( id >= 0x13FC && id <= 0x13FD )
		return true;
	return false;
}

int compare( const void *arg1, const void *arg2 )
{
	CItem *a = *( CItem ** )arg1;
	CItem *b = *( CItem ** )arg2;
	if( a == NULL )
		return 1;
	else if( b == NULL )
		return -1;
	if( a == b )
		return 0;
	if( a->GetSerial() < b->GetSerial() )
		return -1;
	else if( a->GetSerial() > b->GetSerial() )
		return 1;
	else
		return 0;
}

void CItem::Sort( void )
{
	if( Contains.size() < 2 )
		return;
	qsort( (void *)&Contains[0], (size_t)Contains.size(), sizeof( CItem * ), compare );
	UI16 baseY = 0, baseX = 0;
	switch( GetLayer() )
	{
		case 0x1A:			// buy layer
			break;
		case 0x1B:			// bought layer
			baseY = 100;
			break;
		default:	return;
	}
	for( UI16 i = 0; i < Contains.size(); ++i )
	{
		if( Contains[i] == NULL )
			continue;
		Contains[i]->SetX( baseX++ );
		Contains[i]->SetY( baseY );
		if( baseX == 200 )
		{
			baseX = 0;
			++baseY;
		}
	}
}

//o---------------------------------------------------------------------------o
//|		Function    :	void itemTalk( cSocket *s, SI32 dictEntry, R32 secsFromNow, UI16 Colour )
//|		Date        :	Unknown
//|		Programmer  :	UOX DevTeam
//o---------------------------------------------------------------------------o
//|		Purpose     :	Item "speech"
//o---------------------------------------------------------------------------o
void CItem::itemTalk( cSocket *s, SI32 dictEntry, R32 secsFromNow, UI16 Colour )
{
	if( s == NULL )
		return;
	
	std::string txt = Dictionary->GetEntry( dictEntry, s->Language() );
	if( txt.empty() )
		return;

	CSpeechEntry *toAdd = SpeechSys->Add();
	toAdd->Speech( txt.c_str() );
	toAdd->Font( FNT_NORMAL );
	toAdd->Speaker( GetSerial() );
	toAdd->SpokenTo( INVALIDSERIAL );
	toAdd->Type( TALK );
	toAdd->At( BuildTimeValue( secsFromNow ) );
	toAdd->TargType( SPTRG_BROADCASTPC );
	toAdd->Colour( Colour );
}


//o---------------------------------------------------------------------------o
//|	Function	-	void Refresh( cSocket *mSock )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Send this item to specified socket or all online people in range
//o---------------------------------------------------------------------------o
void CItem::Refresh( cSocket *mSock )
{
	if( GetCont() == this )
	{
		Console.Warning( 2, "Item %s(%X) has a dangerous container value, auto-correcting", GetName().c_str(), GetSerial() );
		SetCont( NULL );
	}

	cBaseObject *iCont = GetCont();
	if( GetGlow() != INVALIDSERIAL )
		Items->GlowItem( this );
	if( iCont == NULL )
	{
		if( mSock != NULL )
			SendToSocket( mSock );
		else
		{
			Network->PushConn();
			for( cSocket *aSock = Network->FirstSocket(); !Network->FinishedSockets(); aSock = Network->NextSocket() ) // send this item to all the sockets in range
			{
				SendToSocket( aSock );
			}
			Network->PopConn();
		}
		return;
	}
	else if( iCont->GetObjType() == OT_CHAR )
	{
		CChar *charCont = (CChar *)iCont;
		if( charCont != NULL )
		{
			CPWornItem toWear = (*this);
			if( mSock != NULL )
			{
				if( charInRange( mSock->CurrcharObj(), charCont ) )
					mSock->Send( &toWear );
			}
			else
			{
				Network->PushConn();
				for( cSocket *cSock = Network->FirstSocket(); !Network->FinishedSockets(); cSock = Network->NextSocket() )
				{
					if( charInRange( cSock->CurrcharObj(), charCont ) )
						cSock->Send( &toWear );
				}
				Network->PopConn();
			}
			return;
		}
	}
	else
	{
		CItem *itemCont = (CItem *)iCont;
		if( itemCont != NULL )
		{
			if( mSock != NULL )
				SendPackItemToSocket( mSock );
			else
			{
				Network->PushConn();
				for( cSocket *bSock = Network->FirstSocket(); !Network->FinishedSockets(); bSock = Network->NextSocket() )
					SendPackItemToSocket( bSock );
				Network->PopConn();
			}
			return;
		}
	}
	Console.Error( 2, " CItem::Refresh(%X): cannot determine container type!", GetSerial() );
}

//o---------------------------------------------------------------------------o
//|		Function    -	void SendToSocket( cSocket *mSock )
//|		Programmer  -	giwo
//|		Date        -	July 27, 2003
//o---------------------------------------------------------------------------o
//|		Purpose     -	Updates an item on the ground to specified socket
//o---------------------------------------------------------------------------o
void CItem::SendToSocket( cSocket *mSock )
{
	CChar *mChar = mSock->CurrcharObj();
	if( mChar != NULL )
	{
		if( objInRange( mChar, this, static_cast< UI16 >(mSock->Range() + Races->VisRange( mChar->GetRace() ) ) ) )
		{
			if( !mChar->IsGM() )
			{
				if( GetVisible() > 1 || ( GetVisible() > 0 && mChar != GetOwnerObj() ) )	// Not a GM, and not the Owner
					return;
			}
			CPObjectInfo toSend( (*this), (*mChar) );
			mSock->Send( &toSend );
			if( GetID() == 0x2006 )
			{
				CPCorpseClothing cpcc( this );
				mSock->Send( &cpcc );
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|		Function    -	void SendPackItemToSocket( cSocket *mSock )
//|		Programmer  -	giwo
//|		Date        -	July 27, 2003
//o---------------------------------------------------------------------------o
//|		Purpose     -	Updates an item contained in a pack to specified socket
//o---------------------------------------------------------------------------o
void CItem::SendPackItemToSocket( cSocket *mSock )
{
	CChar *mChar = mSock->CurrcharObj();
	if( mChar != NULL )
	{
		CPAddItemToCont itemSend = (*this);
		if( mChar->IsGM() && GetID() == 0x1647 )
		{
			itemSend.Model( 0x0A0F );
			itemSend.Colour( 0x00C6 );
		}
		CChar *c = getPackOwner( this );
		if( c == NULL )
		{
			CItem *j = getRootPack( this );
			if( j != NULL && itemInRange( mChar, j ) )
				mSock->Send( &itemSend );
		}
		else if( charInRange( mChar, c ) )
			mSock->Send( &itemSend );

	}
}
//o---------------------------------------------------------------------------o
//|		Function    :	void Dirty( void ) const
//|		Date        :	25 July, 2003
//|		Programmer  :	Maarc
//o---------------------------------------------------------------------------o
//|		Purpose     :	Forces the item onto the global refresh queue
//o---------------------------------------------------------------------------o
void CItem::Dirty( void ) const
{
	if( postLoaded )
		++(itemQueue[(cBaseObject *)this]);
}

//o---------------------------------------------------------------------------o
//|		Function    -	void CItem::RemoveFromSight
//|		Date        -	September 7th, 2003
//|		Programmer  -	giwo
//o---------------------------------------------------------------------------o
//|		Purpose     -	Loops through all online chars and removes the item from their sight
//o---------------------------------------------------------------------------o
void CItem::RemoveFromSight( void )
{
	CPRemoveItem toRemove	= (*this);
	cBaseObject *iCont		= GetCont();

	if( iCont == NULL || getRootPack( this ) != NULL )
	{
		CItem *rItem = NULL;
		if( iCont == NULL )
			rItem = this;
		else
			rItem = getRootPack( this );
		if( rItem != NULL )
		{
			CChar *tChar = NULL;
			Network->PushConn();
			for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
			{
				tChar = tSock->CurrcharObj();
				if( tChar != NULL )
				{
					if( objInRange( tChar, rItem, static_cast< UI16 >(tSock->Range() + Races->VisRange( tChar->GetRace() ) ) ) )
						tSock->Send( &toRemove );
				}
			}
			Network->PopConn();
		}
	}
	else if( iCont->GetObjType() == OT_CHAR || getPackOwner( (CItem *)iCont ) != NULL )
	{
		CChar *rChar = NULL;
		if( iCont->GetObjType() == OT_CHAR )
			rChar = (CChar *)iCont;
		else
			rChar = getPackOwner( (CItem *)iCont );
		if( rChar != NULL )
		{
			Network->PushConn();
			for( cSocket *cSock = Network->FirstSocket(); !Network->FinishedSockets(); cSock = Network->NextSocket() )
			{
				if( charInRange( cSock->CurrcharObj(), rChar ) )
					cSock->Send( &toRemove );
			}
			Network->PopConn();
		}
	}
	else
	{
			Network->PushConn();
			for( cSocket *mSock = Network->FirstSocket(); !Network->FinishedSockets(); mSock = Network->NextSocket() )
				mSock->Send( &toRemove );
			Network->PopConn();
	}
}