#include "uox3.h"
#include "fileio.h"
#include "classes.h"
#include "regions.h"

//	1.0		Abaddon		28th July, 2000
//			Very basically fleshed out
//			Owner, ban list and object inside support included
//			Note that lockdown stuff is NOT persistent yet, needs support later on
//			Best place would be world loading, any locked down objects flagged at time added to multi

//	1.1		Abaddon		28th August, 2000
//			Function declaration headers
//			Addition of SetMaxLockDowns

//	1.2		Abaddon		30th November, 2000
//			Changed lock downs to a vector, and full vector support added
//			Save needs modification

const UI16	DEFMULTI_MAXLOCKEDDOWN	= 256;

CMultiObj::CMultiObj( ITEM nItem, bool zeroSer ) : CItem( nItem, zeroSer ), maxLockedDown( DEFMULTI_MAXLOCKEDDOWN ), deed( "" )
{
	objType = OT_MULTI;
}

CMultiObj::~CMultiObj()
{
	Cleanup();
	charInMulti.clear();
	itemInMulti.clear();
	owners.clear();
	banList.clear();
}

//o--------------------------------------------------------------------------
//|	Function		-	AddToBanList( CChar *toban )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adds a character to the ban list, if he's not already on it
//o--------------------------------------------------------------------------
void CMultiObj::AddToBanList( CChar *toBan )
{
	if( !IsOnBanList( toBan ) )
		banList[toBan->GetSerial()] = toBan;
}

//o--------------------------------------------------------------------------
//|	Function		-	RemoveFromBanList( CChar *toRemove )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes a character from a ban list
//o--------------------------------------------------------------------------
void CMultiObj::RemoveFromBanList( CChar *toRemove )
{
	std::map< SERIAL, CChar *>::iterator p = banList.find( toRemove->GetSerial() );
	if( p != banList.end() )
		banList.erase( p );
}

//o--------------------------------------------------------------------------
//|	Function		-	AddAsOwner( CChar *newOwner )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adds a player as an owner, so long as they aren't already
//|						and aren't already on the ban list
//o--------------------------------------------------------------------------
void CMultiObj::AddAsOwner( CChar *newOwner )
{
	if( !IsOwner( newOwner ) && !IsOnBanList( newOwner ) )
		owners[newOwner->GetSerial()] = newOwner;
}

//o--------------------------------------------------------------------------
//|	Function		-	RemoveAsOwner( CChar *toRemove)
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes a character from the owner's list
//o--------------------------------------------------------------------------
void CMultiObj::RemoveAsOwner( CChar *toRemove )
{
	std::map< SERIAL, CChar *>::iterator p = owners.find( toRemove->GetSerial() );
	if( p != owners.end() )
		owners.erase( p );
}

//o--------------------------------------------------------------------------
//|	Function		-	ClearOwners( void )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Empties the owner list
//o--------------------------------------------------------------------------
void CMultiObj::ClearOwners( void )
{
	owners.clear();
}

bool CMultiObj::ItemIsInMulti( cBaseObject *toFind ) const
{
	baseMapConstIterator p = itemInMulti.find( toFind->GetSerial() );
	return ( p != itemInMulti.end() );
}

bool CMultiObj::CharIsInMulti( cBaseObject *toFind ) const
{
	baseMapConstIterator p = charInMulti.find( toFind->GetSerial() );
	return ( p != charInMulti.end() );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool IsInMulti( cBaseObject *toFind )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if an item or character is in the multi
//o--------------------------------------------------------------------------
bool CMultiObj::IsInMulti( cBaseObject *toFind ) const
{
	bool rvalue;
	if( toFind->GetObjType() == OT_CHAR )
		rvalue = CharIsInMulti( toFind );
	else
		rvalue = ItemIsInMulti( toFind );
	return rvalue;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool IsOnBanList( CChar *toBan )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if a character is on the ban list
//o--------------------------------------------------------------------------
bool CMultiObj::IsOnBanList( CChar *toBan ) const
{
	baseCharMapConstIterator p = banList.find( toBan->GetSerial() );
	return ( p != banList.end() );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool IsOwner( CChar *toFind )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if the character is an owner
//o--------------------------------------------------------------------------
bool CMultiObj::IsOwner( CChar *toFind ) const
{
	baseCharMapConstIterator p = owners.find( toFind->GetSerial() );
	return ( p != owners.end() );
}

void CMultiObj::AddCharToMulti( cBaseObject *toFind )
{
	charInMulti[toFind->GetSerial()] = toFind;
}
void CMultiObj::AddItemToMulti( cBaseObject *toFind )
{
	itemInMulti[toFind->GetSerial()] = toFind;
}

//o--------------------------------------------------------------------------
//|	Function		-	AddToMulti( cBaseObject *toAdd )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adds an item or character to the multi
//o--------------------------------------------------------------------------
void CMultiObj::AddToMulti( cBaseObject *toAdd )
{
	if( !IsInMulti( toAdd ) )
	{
		if( toAdd->GetObjType() == OT_CHAR )
			AddCharToMulti( toAdd );
		else
			AddItemToMulti( toAdd );
	}
}

void CMultiObj::RemoveCharFromMulti( cBaseObject *toFind )
{
	baseMapIterator p = charInMulti.find( toFind->GetSerial() );
	if( p != charInMulti.end() )
		charInMulti.erase( p );
}
void CMultiObj::RemoveItemFromMulti( cBaseObject *toFind )
{
	baseMapIterator p = itemInMulti.find( toFind->GetSerial() );
	if( p != itemInMulti.end() )
		itemInMulti.erase( p );
}

//o--------------------------------------------------------------------------
//|	Function		-	RemoveFromMulti( cBaseObject *toRemove )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes an item or character from the multi, if possible
//o--------------------------------------------------------------------------
void CMultiObj::RemoveFromMulti( cBaseObject *toRemove )
{
	if( toRemove->GetObjType() == OT_CHAR )
		RemoveCharFromMulti( toRemove );
	else
		RemoveItemFromMulti( toRemove );
}

//o--------------------------------------------------------------------------
//|	Function		-	UI16 GetLockDownCount( void )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns a count of the locked down items
//o--------------------------------------------------------------------------
size_t CMultiObj::GetLockDownCount( void ) const
{
	return lockedList.size();
}

//o--------------------------------------------------------------------------
//|	Function		-	UI16 GetMaxLockDowns( void )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the maximum number of possible lockdowns
//o--------------------------------------------------------------------------
UI32 CMultiObj::GetMaxLockDowns( void ) const
{
	return maxLockedDown;
}

//o--------------------------------------------------------------------------
//|	Function		-	LockDownItem( CItem *toLock )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Locks down an item toLock
//o--------------------------------------------------------------------------
void CMultiObj::LockDownItem( CItem *toLock )
{
	if( lockedList.size() < maxLockedDown )
	{
		UI32 i;
		for( i = 0; i < lockedList.size(); ++i )
		{
			if( toLock == lockedList[i] )
				break;
		}
		if( i == lockedList.size() )
		{
			toLock->LockDown();
			lockedList.push_back( toLock );
		}
	}
}

//o--------------------------------------------------------------------------
//|	Function		-	RemoveLockDown( CItem *toRemove )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Unlocks a locked down item
//o--------------------------------------------------------------------------
void CMultiObj::RemoveLockDown( CItem *toRemove )
{
	for( UI32 i = 0; i < lockedList.size(); ++i )
	{
		if( lockedList[i] == toRemove )
		{
			for( UI32 j = i; j < lockedList.size() - 1; ++j )
				lockedList[j] = lockedList[j+1];
			lockedList.resize( lockedList.size() - 1 );
			toRemove->SetMovable( 0 );
		}
	}
}

//o--------------------------------------------------------------------------
//|	Function		-	bool Save( ofstream &outStream )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Saves a multi out to disk
//|						outStream is the file to write to
//o--------------------------------------------------------------------------
bool CMultiObj::Save( std::ofstream &outStream )
{
	bool rvalue = false;
	if( !isFree() )
	{
		rvalue = true;
		typedef std::map< SERIAL, CChar *>::iterator iCounter;
		if( GetCont() != NULL || ( GetX() > 0 && GetX() < 6144 && GetY() < 4096 ) )
		{
			DumpHeader( outStream );
			DumpBody( outStream );
			DumpFooter( outStream );
		}
	}
	return rvalue;
}



//o--------------------------------------------------------------------------
//|	Function		-	bool Load( ifstream &instream )
//|	Date			-	28th July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Loads an item from disk, using instream as the source
//o--------------------------------------------------------------------------
bool CMultiObj::Load( std::ifstream &inStream, int arrayOffset )
{
	cBaseObject::Load( inStream );
	return LoadRemnants( arrayOffset );	
}

//o--------------------------------------------------------------------------
//|	Function		-	bool LoadRemnants( int arrayOffset )
//|	Date			-	20th January, 2002
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	After handling data specific load, other parts go here
//o--------------------------------------------------------------------------
bool CMultiObj::LoadRemnants( int arrayOffset )
{
	if( cwmWorldState->GetItemCount2() <= serial )
		cwmWorldState->SetItemCount2( serial + 1 );
	SetSerial( serial, arrayOffset );
	Items->StoreItemRandomValue( &items[arrayOffset], 0xFF ); // Magius(CHE) (2)


	// Add item weight if item doesn't have it yet
	if( GetWeight() <= 0 )
		SetWeight( 0 );

	// Tauriel adding region pointers
	bool rvalue = true;
	if( GetCont() == NULL )
	{
		MapRegion->AddItem( this );
		if( GetX() < 0 || GetY() < 0 )
			rvalue = false;
	}
	return rvalue;
}

//o--------------------------------------------------------------------------
//|	Function		-	SetMaxLockDowns( UI16 newValue )
//|	Date			-	28th August, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Sets the maximum number of lockdowns possible
//o--------------------------------------------------------------------------
void CMultiObj::SetMaxLockDowns( UI16 newValue )
{
	maxLockedDown = newValue;
}

//o--------------------------------------------------------------------------
//|	Function		-	CChar *FirstCharMulti()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the first character in the multi (if none, NULL)
//o--------------------------------------------------------------------------
CChar *CMultiObj::FirstCharMulti( void )
{
	CChar *rvalue		= NULL;
	charMultiIterator	= charInMulti.begin();
	if( !FinishedCharMulti() )
		rvalue = (CChar *)charMultiIterator->second;
	return rvalue;
}

//o--------------------------------------------------------------------------
//|	Function		-	CChar *NextCharMulti()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the next character in the multi
//o--------------------------------------------------------------------------
CChar *CMultiObj::NextCharMulti( void )
{
	CChar *rvalue = NULL;
	++charMultiIterator;
	if( !FinishedCharMulti() )
		rvalue = (CChar *)charMultiIterator->second;
	return rvalue;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool FinishedCharMulti()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if there are no more chars in the multi
//o--------------------------------------------------------------------------
bool CMultiObj::FinishedCharMulti( void ) const
{
	baseMapConstIterator t = charMultiIterator;
	return ( t == charInMulti.end() );
}

//o--------------------------------------------------------------------------
//|	Function		-	CItem *FirstItemMulti()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the first item in the multi
//o--------------------------------------------------------------------------
CItem *CMultiObj::FirstItemMulti( void )
{
	CItem *rvalue		= NULL;
	itemMultiIterator	= itemInMulti.begin();
	if( !FinishedItemMulti() )
		rvalue = (CItem *)itemMultiIterator->second;
	return rvalue;
}

//o--------------------------------------------------------------------------
//|	Function		-	CItem *NextItemMulti()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the next item (if any) in the multi
//o--------------------------------------------------------------------------
CItem *CMultiObj::NextItemMulti( void )
{
	CItem *rvalue = NULL;
	++itemMultiIterator;
	if( !FinishedItemMulti() )
		rvalue = (CItem *)itemMultiIterator->second;
	return rvalue;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool FinishedItemMulti()
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if there are no more items in the multi
//o--------------------------------------------------------------------------
bool CMultiObj::FinishedItemMulti( void ) const
{
	baseMapConstIterator t = itemMultiIterator;
	return ( t == itemInMulti.end() );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool DumpHeader( ofstream &outStream )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Dumps out the header for the CMultiObj
//o--------------------------------------------------------------------------
bool CMultiObj::DumpHeader( std::ofstream &outStream ) const
{
	outStream << "[HOUSE]" << std::endl;
	return true;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool DumpBody( ofstream &outStream )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Writes out all CMultiObj specific information to a world
//|						file.  Also prints out the Item and BaseObject portions
//o--------------------------------------------------------------------------
bool CMultiObj::DumpBody( std::ofstream &outStream ) const
{
	std::string destination; 
	std::ostringstream dumping( destination ); 
	baseCharMapConstIterator ban, owner;
	UI32 iCtr;

	CItem::DumpBody( outStream );

	for( ban = banList.begin(); ban != banList.end(); ++ban )
		dumping << "Banned=" << (ban->second)->GetSerial() << std::endl;

	for( owner = owners.begin(); owner != owners.end(); ++owner )
		dumping << "CoOwner=" << (owner->second)->GetSerial() << std::endl;

	for( iCtr = 0; iCtr < lockedList.size(); ++iCtr )
	{
		if( lockedList[iCtr] != NULL )
			dumping << "LockedItem=" << lockedList[iCtr]->GetSerial() << std::endl;
	}

	dumping << "MaxLockedDown=" << maxLockedDown << std::endl;
	// Add deedname= to the save :) We need a way to preserve the name of the item
	dumping << "DeedName=" << deed << std::endl;
	outStream << dumping.str();
	return true;
}


//o--------------------------------------------------------------------------
//|	Function		-	bool HandleLine( char *tag, char *data )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Processes a tag/data pair if it can.  If it can, returns
//|						true.  Otherwise, returns false.
//o--------------------------------------------------------------------------
bool CMultiObj::HandleLine( UString &tag, UString &data )
{
	bool rvalue = false;
	if( !( rvalue = CItem::HandleLine( tag, data ) ) )
	{
		switch( (tag.data()[0]) )
		{
			case 'B':
				if( tag.upper() == "BANNED" )
				{
					CChar *bList = calcCharObjFromSer( data.toULong() );
					if( bList != NULL )
						AddToBanList( bList );
					rvalue = true;
				}
				break;
			case 'C':
				if( tag.upper() == "COOWNER" )
				{
					CChar *cList = calcCharObjFromSer( data.toULong() );
					if( cList != NULL )
						AddAsOwner( cList );
					rvalue = true;
				}
				break;
			case 'D':
				{
					if( tag.upper() == "DEEDNAME" )
					{
						SetDeed( data.c_str() );
						rvalue = true;
					}
					break;
				}
				break;
			case 'L':
				if( tag.upper() == "LOCKEDITEM" )
				{
					CItem *iList = calcItemObjFromSer( data.toULong() );
					if( iList != NULL )
						LockDownItem( iList );
					rvalue = true;
				}
				break;
			case 'M':
				if( tag.upper() == "MAXLOCKEDDOWN" )
				{
					maxLockedDown = data.toUShort();
					rvalue = true;
				}
				break;
		}
	}
	return rvalue;
}

//o--------------------------------------------------------------------------
//|	Function		-	void PostLoadProcessing( UI32 index )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Does any post load processing required
//o--------------------------------------------------------------------------
void CMultiObj::PostLoadProcessing( UI32 index )
{
	CItem::PostLoadProcessing( index );
}


//o--------------------------------------------------------------------------
//|	Function		-	SetOwner( CChar *newOwner )
//|	Date			-	Unknown
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Ensures that newOwner is also on the house ownership
//|						list.
//o--------------------------------------------------------------------------
void CMultiObj::SetOwner( CChar *newOwner )
{
	cBaseObject::SetOwner( (cBaseObject *)newOwner );
	AddAsOwner( newOwner );
}

//o--------------------------------------------------------------------------
//|	Function		-	Cleanup( void )
//|	Date			-	26th September, 2001
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Makes sure that any items and chars inside the multi
//|						are removed
//o--------------------------------------------------------------------------
void CMultiObj::Cleanup( void )
{
	std::vector< cBaseObject * > toRemove;
	baseMapIterator iCleanup = itemInMulti.begin();
	while( iCleanup != itemInMulti.end() )
	{
		cBaseObject *iRemove = iCleanup->second;
		if( iRemove != NULL )
			toRemove.push_back( iRemove );
		++iCleanup;
	}
	baseMapIterator cCleanup = charInMulti.begin();
	while( cCleanup != charInMulti.end() )
	{
		cBaseObject *cRemove = cCleanup->second;
		if( cRemove != NULL )
			toRemove.push_back( cRemove );
		++cCleanup;
	}
	for( UI32 i = 0; i < toRemove.size(); ++i )
		toRemove[i]->SetMulti( (cBaseObject *)NULL );
}

std::string CMultiObj::GetDeed( void )
{
	return deed;
}

void CMultiObj::SetDeed( std::string newDeed )
{
	deed = newDeed;
}

