#ifndef __CMULTIOBJ_H__
#define __CMULTIOBJ_H__
#include "ustring.h"

typedef std::map< SERIAL, cBaseObject *> baseMap;
typedef baseMap::iterator			baseMapIterator;
typedef baseMap::const_iterator		baseMapConstIterator;

typedef std::map< SERIAL, CChar *>		baseCharMap;
typedef baseCharMap::iterator		baseCharMapIterator;
typedef baseCharMap::const_iterator	baseCharMapConstIterator;

class CMultiObj : public CItem
{
protected:
	std::string			deed;

	baseMap				itemInMulti;
	baseMap				charInMulti;
	baseCharMap			owners;
	baseCharMap			banList;
	ITEMLIST			lockedList;
	UI16				maxLockedDown;

	baseMapIterator		itemMultiIterator;
	baseMapIterator		charMultiIterator;

	bool				CharIsInMulti( cBaseObject *toFind ) const;
	bool				ItemIsInMulti( cBaseObject *toFind ) const;

	void				RemoveCharFromMulti( cBaseObject *toFind );
	void				RemoveItemFromMulti( cBaseObject *toFind );

	void				AddCharToMulti( cBaseObject *toFind );
	void				AddItemToMulti( cBaseObject *toFind );

	virtual void		Cleanup( void );
	virtual bool		LoadRemnants( int arrayOffset );

public:
						CMultiObj( ITEM nItem, bool zeroSer );
	virtual				~CMultiObj();

	void				AddToBanList( CChar *toBan );
	void				RemoveFromBanList( CChar *toRemove );

	void				AddAsOwner( CChar *newOwner );
	void				RemoveAsOwner( CChar *toRemove );
	void				ClearOwners( void );

	bool				IsInMulti( cBaseObject *toFind ) const;
	bool				IsOnBanList( CChar *toBan ) const;
	bool				IsOwner( CChar *toFind ) const;

	void				AddToMulti( cBaseObject *toAdd );
	void				RemoveFromMulti( cBaseObject *toRemove );

	size_t				GetLockDownCount( void ) const;
	UI32				GetMaxLockDowns( void ) const;
	void				SetMaxLockDowns( UI16 newValue );
	void				LockDownItem( CItem *toLock );
	void				RemoveLockDown( CItem *toRemove );

	virtual bool		Save( std::ofstream &outStream );
	virtual bool		DumpHeader( std::ofstream &outStream ) const;
	virtual bool		DumpBody( std::ofstream &outStream ) const;
	virtual bool		Load( std::ifstream &inStream, int arrayOffset );
	virtual bool		HandleLine( UString &tag, UString &data );
	virtual void		PostLoadProcessing( UI32 index );

	CChar *				FirstCharMulti( void );
	CChar *				NextCharMulti( void );
	bool				FinishedCharMulti( void ) const;

	CItem *				FirstItemMulti( void );
	CItem *				NextItemMulti( void );
	bool				FinishedItemMulti( void ) const;

	virtual void		SetOwner( CChar *newOwner );

	std::string			GetDeed( void );
	void				SetDeed( std::string newDeed );
};


#endif

