
#ifndef __CSERVERDATA__
#define __CSERVERDATA__

enum cSD_TID
{
	TIMER_ERROR = -1,
	CORPSE_DECAY = 0,
	LOOT_DECAY,
	WEATHER,
	SHOP_SPAWN,
	DECAY,
	INVISIBILITY,
	OBJECT_USAGE,
	GATE,
	POISON,
	LOGIN_TIMEOUT,
	HITPOINT_REGEN,
	STAMINA_REGEN,
	MANA_REGEN,
	ARMORAFFECTMANA_REGEN,
	BASE_FISHING,
	RANDOM_FISHING,
	SPIRIT_SPEAK,
	BEGGING_T,
	PLAYER_CORPSE,
	TIMER_COUNT
};

enum CSDDirectoryPaths
{
	CSDDP_ROOT = 0,
	CSDDP_DATA,
	CSDDP_DEFS,
	CSDDP_ACCESS,
	CSDDP_ACCOUNTS,
	CSDDP_SCRIPTS,
	CSDDP_BACKUP,
	CSDDP_MSGBOARD,
	CSDDP_SHARED,
	CSDDP_HTML,
	CSDDP_LOGS,
	CSDDP_DICTIONARIES,
	CSDDP_BOOKS,
	CSDDP_COUNT
};

class physicalServer
{
public:
  void setName(const std::string newName);
  void setDomain(const std::string newDomain);
  void setIP(const std::string newIP);
  void setPort(UI16 newPort);
  std::string getName(void);
  std::string getDomain(void);
  std::string getIP(void);
  UI16 getPort(void);

private:
	std::string name;
	std::string domain;
	std::string ip;
	UI16 port;
};

class CServerData 
{
protected:

	UI16		port;							//	Port number that the server listens on, for connections
	// ServerSystems
	std::vector< physicalServer > serverList;	//	Series of server entries for shard list
	UI08		consolelogenabled;				//	Various levels of legging 0 == none, 1 == normal, 2 == normal + all speech
	UI08		crashprotectionenabled;			//	Level of crash protection - number of crash restart attempts before stopping - NEVER ACTIVELY USED
	char		commandprefix;					//	Character that acts as the command prefix
	bool		announcesaves;					//	Determines if world saves are announced
	bool		joinpartmsgsenabled;			//	Determines if player join/part messages are enabled
	bool		mulcachingenabled;				//	Determines if the MULs are cached in memory
	bool		backupsenabled;					//	Determines if backups are enabled
	SI16		backupRatio;					//	Number of saves before a backup occurs
	UI32		serversavestimer;				//	Number of seconds between world saves

	// Skills & Stats
	UI16		skillcap;						//	A cap on the total of all a PC's skills
	UI08		skilldelay;						//	Number of seconds after a skill is used before another skill can be used
	UI16		statcap;						//	A cap on the total of a PC's stats
	SI16		maxstealthmovement;				//	Max number of steps allowed with stealth skill at 100.0
	SI16		maxstaminamovement;				//	Max number of steps allowed while running before stamina is reduced
	bool		snoopiscrime;					//	True if snooping is a crime, otherwise false

	// ServerTimers
	UI16		serverTimers[TIMER_COUNT];
	// Directories
	std::string serverDirectories[CSDDP_COUNT];

	// Settings
	bool		lootdecayswithcorpse;			//	True if loot decays with the corpse
	bool		guardsenabled;					//	True if guards are enabled on a global scheme
	bool		deathanimationenabled;			//	True if the death animation is played
	SI16		ambientsounds;					//	Ambient sounds - values from 1->10 - higher values indicate sounds occur less often
	bool		ambientfootsteps;				//	True if different tiles make different footstep sounds
	bool		internalaccountsenabled;		//	True if new accounts are made when a person logs in with an unknown username
	bool		showhiddennpcsenabled;			//	True if NPCs that are hidden are viewable by GMs
	bool		roguesenabled;					//	True if stealing is a useable skill
	bool		playerpersecution;				//	True if persecution is enabled (ghosts attacking living)
	SI16		htmlstatusenabled;				//	If > 0 then it's enabled - only used at PC char creation - use elsewhere? (was # of seconds between updates)
	bool		sellbynameenabled;				//	True if items can be sold by their name, not just ID/Colour
	SI16		sellmaxitems;					//	Maximum number of items that can be sold to a vendor
	bool		tradesystemenabled;				//	True if the trade system is enabled (uses the GOOD settings... does anyone use this?)
	bool		ranksystemenabled;				//	True if the rank system is enabled (allows for variable quality goods)
	SI16		cutscrollrequirement;			//	Should be a BOOL - If true, then skill requirements for casting spells is cut
	bool		npctraining;					//	True if NPCs can train PCs
	SI16		charhidewhilemounted;			//	Should be a BOOL - If true, then a character can hide/stealth while mounted

	// SpeedUp
	R64			checkitems;						//	How often (in seconds) items are checked for decay and other things
	R64			checkboats;						//	How often (in seconds) boats are checked for motion and so forth
	R64			checknpcai;						//	How often (in seconds) NPCs can execute an AI routine
	R64			checkspawnregions;				//	How often (in seconds) spawn regions are checked for new spawns
	R64			npcSpeed;						//	Speed at which NPCs move
	R64			flushTime;						//	How often (in minutes) online accounts are checked to see if they really ARE online

	// MessageBoards
	SI16		msgpostinglevel;				//	If not 0, then players can post
	SI16		msgremovallevel;				//	If not 0, then players can remove posts

	// Escorts
	bool		escortsenabled;					//	True if escorts are enabled
	UI16		escortinitexpire;				//	How long (in seconds) the escort is waiting for a PC before it expires
	UI16		escortactiveexpire;				//	How long (in seconds) the escort is following a PC before it expires
	UI16		escortdoneexpire;				//	How long (in seconds) the escort will stay around after it has been escorted

	// WorldLight
	LIGHTLEVEL	dungeonlightlevel;				//	Default light level for a dungeon, if not subject to a weather system
	LIGHTLEVEL	currentlightlevel;				//	Default current light level if not subject to a weather system
	LIGHTLEVEL	brightnesslightlevel;			//	Default brightest light level if not subject to a weather system
	LIGHTLEVEL	darknesslightlevel;				//	Default darkest light level if not subject to a weather system

	// WorldTimer								//	days/hours/minutes/seconds to it's own file?
	SI16		moon[2];						//	Moon current state
	SI16		days;							//	Number of days the world has been running (persistent)
	UI08		hours;							//	Number of hours the world has been running (persistent)
	UI08		minutes;						//	Number of minutes the world has been running (persistent)
	UI08		seconds;						//	Number of seconds the world has been running (persistent)
	bool		ampm;							//	Whether our current time is in the morning or afternoon

	// Begging
	SI16		beggingrange;					//	Change to RANGE type?  Distance at which an NPC/PC can beg

	// Tracking
	UI16		trackingbaserange;				//	Minimum range even a novice can track from
	UI16		trackingbasetimer;				//	Tracking timer - maximum time for a GM to track at
	UI08		trackingmaxtargets;				//	Maximum number of targets that can be tracked
	UI16		trackingmsgredisplaytimer;		//	How often (in seconds) the tracking message is redisplayed


	// Reputation
	UI16		murderdecaytimer;				//	TIMERVAL?  How long it takes before a murder is decayed
	UI16		criminaldecaytimer;				//	TIMERVAL?  How long it takes before becoming innocent again
	UI16		maxmurdersallowed;				//	Maximum number of kills before turning red

	// Resources
	UI08		minecheck;						//	Type of mining check performed - 0 anywhere 1 only mountains/floors 2 region based (not working)
	SI16		oreperarea;						//	TIMERVAL? Maximum number of ores in a resource area
	UI16		orerespawntimer;				//	Time at which ore is respawned (only 1 ore, not all)
	UI16		orerespawnarea;					//	Number of ore areas (ie Y x Y where Y == value)
	SI16		logsperarea;					//	Maximum number of logs in a resource area
	UI16		logsrespawntimer;				//	TIMERVAL? Time at which logs are respawned (only 1 log, not all)
	UI16		logsrespawnarea;				//	Number of log areas (ie Y x Y where Y == value)

	// Hunger & Food
	UI16		hungerrate;						//	TIMERVAL? Number of seconds between hunger checks
	UI16		hungerdamageratetimer;			//	TIMERVAL? Number of seconds between hunger damage checks
	SI16		hungerdamage;					//	Amount of damage applied if hungry and below threshold
	SI16		hungerthreshold;				//	The hunger level below which damage is sustained

	// Combat
	SI16		combatmaxrange;					//	RANGE?  Range at which combat can actually occur
	SI16		combatmaxspellrange;			//	RANGE?  Range at which spells can be cast
	bool		combatdisplayhitmessage;		//	True if hit messages are displayed in combat
	bool		combatmonstersvsanimals;		//	True if monsters can attack animals
	SI16		combatanimalattackchance;		//	Chance of animals being attacked (0-100)
	bool		combatanimalsguarded;			//	True if animals are guarded from attack
	SI16		combatnpcdamagerate;			//	NPC Damage divisor - PCs sustain less than NPCs.  If a PC, damage is 1/value
	SI16		combatnpcbasefleeat;			//	% of HP where an NPC will flee, if it's not defined for them
	SI16		combatnpcbasereattackat;		//	% of HP where an NPC will resume attacking
	SI16		combatattackstamina;			//	Amount of stamina lost when hitting an opponent
	R32			combatExplodeDelay;				//	Time from casting to actual explosion
	bool		shootonanimalback;				//	True if you can fire a bow from horseback

	// Start & Location Settings
	std::vector< STARTLOCATION >	startlocations;
	SI16		startgold;						//	Amount of gold created when a PC is made
	UI16		startprivs;						//	Starting privileges of characters

	// Anything under this comment is left here for symantics
	UI08 skilllevel;							//	Some skill value associated with the rank system
	SI16 buyThreshold;							//	Value above which money will be sourced from the bank rather than the player
	SI16 niceness;								//	How nicely the server "sleeps" in each thread

	// Gump stuff
	UI16	titleColour;						//	Default text colour for titles in gumps
	UI16	leftTextColour;						//	Default text colour for left text in gumps (2 column ones)
	UI16	rightTextColour;					//	Default text colour for right text in gumps
	UI16	buttonCancel;						//	Default Button ID for cancel button in gumps
	UI16	buttonLeft;							//	Default Button ID for left button in gumps
	UI16	buttonRight;						//	Default Button ID for right button in gumps
	UI16	backgroundPic;						//	Default Gump ID for background gump

	// Townstone stuff
	UI32 numSecsPollOpen;						//	Time (in seconds) for which a town voting poll is open
	UI32 numSecsAsMayor;						//	Time (in seconds) that a PC would be a mayor
	UI32 taxPeriod;								//	Time (in seconds) between periods of taxes for PCs 
	UI32 guardPayment;							//	Time (in seconds) between payments for guards

	void	PostLoadDefaults( void );

	SI16	potiondelay;						//	TIMERVAL?  Number of seconds delay on potion use

public:
	SI16		ServerMoon( SI16 slot );
	LIGHTLEVEL	WorldLightDarkLevel( void );
	LIGHTLEVEL	WorldLightBrightLevel( void );
	LIGHTLEVEL	WorldLightCurrentLevel( void );
	LIGHTLEVEL	DungeonLightLevel( void );
	UI16		ServerStartPrivs( void ) const;
	SI16		ServerStartGold( void ) const;

	void		ServerMoon( SI16 slot, SI16 value );
	void		WorldLightDarkLevel( LIGHTLEVEL value );
	void		WorldLightBrightLevel( LIGHTLEVEL value );
	void		WorldLightCurrentLevel( LIGHTLEVEL value );
	void		DungeonLightLevel( LIGHTLEVEL value );
	void		ServerStartPrivs( UI16 value );
	void		ServerStartGold( SI16 value );
	CServerData * ParseUox3Ini( std::string filename );

	virtual CServerData *load( void );
	virtual CServerData *load( std::string filename );
	virtual bool save( void );
	virtual bool save( std::string filename );

	void		ResetDefaults( void );

				CServerData( void );
	virtual		~CServerData();
	void		ServerName( std::string setname );
	void		ServerDomain( std::string setdomain );
	void		ServerIP( std::string setip );
	std::string ServerName( void );
	std::string ServerDomain( void );
	std::string ServerIP( void );
	void		ServerPort( UI16 setport );
	UI16		ServerPort( void );
	void		ServerConsoleLog( UI08 setting );
	UI08		ServerConsoleLogStatus( void );
	void		ServerCommandPrefix( char cmdvalue );
	char		ServerCommandPrefix( void );
	void		ServerAnnounceSaves( bool setting );
	bool		ServerAnnounceSavesStatus( void );
	void		ServerJoinPartAnnouncements( bool setting );
	bool		ServerJoinPartAnnouncementsStatus( void );
	void		ServerMulCaching( bool setting );
	bool		ServerMulCachingStatus( void );
	void		ServerBackups( bool setting );
	bool		ServerBackupStatus( void );
	void		ServerSavesTimer( UI32 timer );
	UI32		ServerSavesTimerStatus( void );
	void		ServerMainThreadTimer( UI32 threadtimer );
	UI32		ServerMainThreadTimerStatus( void );
	void		ServerSkillCap( UI16 cap );
	UI16		ServerSkillCapStatus( void );
	void		ServerSkillDelay( UI08 skdelay );
	UI08		ServerSkillDelayStatus( void );
	void		ServerStatCap( UI16 cap );
	UI16		ServerStatCapStatus( void );
	void		MaxStealthMovement( SI16 value );
	SI16		MaxStealthMovement( void );
	void		MaxStaminaMovement( SI16 value );
	SI16		MaxStaminaMovement( void );
	void		SystemTimer( cSD_TID timerid, UI16 value );
	UI16		SystemTimer( cSD_TID timerid );

	// Define all Path Get/Set's here please
	void		Directory( CSDDirectoryPaths dp, std::string value );
	std::string Directory( CSDDirectoryPaths dp );

	void		ServerCrashProtection( UI08 setting );
	UI08		ServerCrashProtectionStatus( void );

	void		CorpseLootDecay( bool value );
	bool		CorpseLootDecay( void );

	void		GuardStatus( bool value );
	bool		GuardsStatus( void );

	void		DeathAnimationStatus( bool value );
	bool		DeathAnimationStatus( void );

	void		WorldAmbientSounds( SI16 value );
	SI16		WorldAmbientSounds( void );

	void		AmbientFootsteps( bool value );
	bool		AmbientFootsteps( void );

	void		InternalAccountStatus( bool value );
	bool		InternalAccountStatus( void );

	void		ShowHiddenNpcStatus( bool value );
	bool		ShowHiddenNpcStatus( void );

	void		RogueStatus( bool value );
	bool		RogueStatus( void );

	void		SnoopIsCrime( bool value );
	bool		SnoopIsCrime( void );

	void		PlayerPersecutionStatus( bool value );
	bool		PlayerPersecutionStatus( void );

	void		HtmlStatsStatus( SI16 value );
	SI16		HtmlStatsStatus( void );

	void		SellByNameStatus( bool value );
	bool		SellByNameStatus( void );

	void		SellMaxItemsStatus( SI16 value );
	SI16		SellMaxItemsStatus( void );

	void		TradeSystemStatus( bool value );
	bool		TradeSystemStatus( void );

	void		RankSystemStatus( bool value );
	bool		RankSystemStatus( void );

	void		CutScrollRequirementStatus( SI16 value );
	SI16		CutScrollRequirementStatus( void );

	void		NPCTrainingStatus( bool setting );
	bool		NPCTrainingStatus( void );

	void		CheckItemsSpeed( R64 value );
	R64			CheckItemsSpeed( void );

	void		CheckBoatSpeed( R64 value );
	R64			CheckBoatSpeed( void );

	void		CheckNpcAISpeed( R64 value );
	R64			CheckNpcAISpeed( void );

	void		CheckSpawnRegionSpeed( R64 value );
	R64			CheckSpawnRegionSpeed( void );

	void		MsgBoardPostingLevel( SI16 value );
	SI16		MsgBoardPostingLevel( void );

	void		MsgBoardPostRemovalLevel( SI16 value );
	SI16		MsgBoardPostRemovalLevel( void );

	void		MineCheck( UI08 value );
	UI08		MineCheck( void );

	void		CombatDisplayHitMessage( bool value );
	bool		CombatDisplayHitMessage( void );

	void		CombatAttackStamina( SI16 value );
	SI16		CombatAttackStamina( void );

	void		CombatNPCDamageRate( SI16 value );
	SI16		CombatNPCDamageRate( void );

	UI08		SkillLevel( void );
	void		SkillLevel( UI08 value );

	void		EscortsEnabled( bool value );
	bool		EscortsEnabled( void );

	void		EscortInitExpire( UI16 value );
	UI16		EscortInitExpire( void );

	void		EscortActiveExpire( UI16 value );
	UI16		EscortActiveExpire( void );

	void		EscortDoneExpire( UI16 value );
	UI16		EscortDoneExpire( void );

	void		CombatMonstersVsAnimals( bool value );
	bool		CombatMonstersVsAnimals( void );

	void		CombatAnimalsAttackChance( SI16 value );
	SI16		CombatAnimalsAttackChance( void );

	void		HungerRate( UI16 value );
	UI16		HungerRate( void );

	void		HungerDamageRateTimer( UI16 value );
	UI16		HungerDamageRateTimer( void );

	void		HungerDamage( SI16 value );
	SI16		HungerDamage( void );

	void		HungerThreshold( SI16 value );
	SI16		HungerThreshold( void );

	void		PotionDelay( SI16 value );
	SI16		PotionDelay( void );

	void		BuyThreshold( SI16 value );
	SI16		BuyThreshold( void );

	void		BackupRatio( SI16 value );
	SI16		BackupRatio( void );

	void		CombatMaxRange( SI16 value );
	SI16		CombatMaxRange( void );

	void		CombatMaxSpellRange( SI16 value );
	SI16		CombatMaxSpellRange( void );

	void		CombatExplodeDelay( R32 value );
	R32			CombatExplodeDelay( void );

	void		CombatAnimalsGuarded( bool value );
	bool		CombatAnimalsGuarded( void );

	void		CombatNPCBaseFleeAt( SI16 value );
	SI16		CombatNPCBaseFleeAt( void );

	void		CombatNPCBaseReattackAt( SI16 value );
	SI16		CombatNPCBaseReattackAt( void );

	void		ShootOnAnimalBack( bool setting );
	bool		ShootOnAnimalBack( void );

	void		NPCSpeed( R64 value );
	R64			NPCSpeed( void );

	void		Niceness( SI16 value );
	SI16		Niceness( void );

	void		TitleColour( UI16 value );
	UI16		TitleColour( void );

	void		LeftTextColour( UI16 value );
	UI16		LeftTextColour( void );

	void		RightTextColour( UI16 value );
	UI16		RightTextColour( void );

	void		ButtonCancel( UI16 value );
	UI16		ButtonCancel( void );

	void		ButtonLeft( UI16 value );
	UI16		ButtonLeft( void );

	void		ButtonRight( UI16 value );
	UI16		ButtonRight( void );

	void		BackgroundPic( UI16 value );
	UI16		BackgroundPic( void );

	void		TownNumSecsPollOpen( UI32 value );
	UI32		TownNumSecsPollOpen( void );

	void		TownNumSecsAsMayor( UI32 value );
	UI32		TownNumSecsAsMayor( void );

	void		TownTaxPeriod( UI32 value );
	UI32		TownTaxPeriod( void );

	void		TownGuardPayment( UI32 value );
	UI32		TownGuardPayment( void );

	void		RepMurderDecay( UI16 value );
	UI16		RepMurderDecay( void );

	void		RepMaxKills( UI16 value );
	UI16		RepMaxKills( void );

	void		RepCrimTime( UI16 value );
	UI16		RepCrimTime( void );

	void		ResLogs( SI16 value );
	SI16		ResLogs( void );

	void		ResLogTime( UI16 value );
	UI16		ResLogTime( void );

	void		ResLogArea( UI16 value );
	UI16		ResLogArea( void );

	void		ResOre( SI16 value );
	SI16		ResOre( void );

	void		ResOreTime( UI16 value );
	UI16		ResOreTime( void );

	void		ResOreArea( UI16 value );
	UI16		ResOreArea( void );

	void		AccountFlushTimer( R64 value );
	R64			AccountFlushTimer( void );

	void		TrackingBaseRange( UI16 value );
	UI16		TrackingBaseRange( void );

	void		TrackingMaxTargets( UI08 value );
	UI08		TrackingMaxTargets( void );

	void		TrackingBaseTimer( UI16 value );
	UI16		TrackingBaseTimer( void );

	void		TrackingRedisplayTime( UI16 value );
	UI16		TrackingRedisplayTime( void );

	void		BeggingRange( SI16 value );
	SI16		BeggingRange( void );

	// Sept 22, 2002 - EviLDeD - Support for Xuri's HideWhileMounted fix.
	void		CharHideWhileMounted( SI16 value );
	SI16		CharHideWhileMounted( void );

	void		dumpLookup( int lookupid );
	void		dumpPaths( void );

	void			ServerLocation( std::string toSet );
	LPSTARTLOCATION ServerLocation( UI32 locNum );

	size_t			NumServerLocations( void ) const;

	SI16			ServerTimeDay( void ) const;
	UI08			ServerTimeHours( void ) const;
	UI08			ServerTimeMinutes( void ) const;
	UI08			ServerTimeSeconds( void ) const;
	bool			ServerTimeAMPM( void ) const;

	void			ServerTimeDay( SI16 nValue );
	void			ServerTimeHours( UI08 nValue );
	void			ServerTimeMinutes( UI08 nValue );
	void			ServerTimeSeconds( UI08 nValue );
	void			ServerTimeAMPM( bool nValue );

	// These functions return TRUE if it's a new day
	bool			incSecond( void );
	bool			incMinute( void );
	bool			incHour( void );
	bool			incDay( void );

	void			incMoon( int mNumber );

	physicalServer *ServerEntry( UI16 entryNum );
	UI16			ServerCount( void ) const;

private: 
	bool			resettingDefaults; 

};

#endif

