#include "uox3.h"
#include "packets.h"
#include "speech.h"
#include "cRaces.h"
#include "cGuild.h"
#include "targeting.h"
#include "commands.h"
#include "combat.h"

#if defined(__unix__)
	#include <sys/ioctl.h>
	#include <errno.h>
#endif

const UI32 DefaultClientVersion		= calcserial( 2, 0, 0, 0 );
const ClientTypes DefaultClientType = CV_NORMAL;


// These vectors are for dealing with packets that are larger than the buffer size
// While admittedly not thread friendly, the number of times these buffers are used
// should be very small and right now, is an implementation that will increase clieht
// compatability
std::vector< UI08 >	largeBuffer;
std::vector< UI08 >	largePackBuffer;

//	1.0		Abaddon		29th November, 2000
//			Initial implementation
//			Stores almost all information currently separated into different vars
//			Also has logging support, and non-blocking IO support
//			Makes use of a socket_error exception class

#if _DEBUG
const bool LOGDEFAULT = true;
#else
const bool LOGDEFAULT = false;
#endif

long socket_error::ErrorNumber( void ) const
{
	return errorNum;
}

const char *socket_error::what( void ) const throw()
{
	return runtime_error::what();
}

socket_error::socket_error( void ) : errorNum( -1 ), runtime_error( "" )
{
}

socket_error::socket_error( const std::string& what_arg ) : errorNum( -1 ), runtime_error( what_arg )
{
}

socket_error::socket_error( const long errorNumber ) : errorNum( errorNumber ), runtime_error( "" )
{
}

//o---------------------------------------------------------------------------o
//|   Function    -  int cSocket::CliSocket( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the socket identifier for our socket
//o---------------------------------------------------------------------------o
int cSocket::CliSocket( void ) const
{
	return cliSocket;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::CliSocket( int newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Assigns a new socket value
//o---------------------------------------------------------------------------o
void cSocket::CliSocket( int newValue )
{
	cliSocket = newValue;
	UI32 mode = 1;
	// set the socket to nonblocking
	ioctlsocket( cliSocket, FIONBIO, &mode );
}

//o---------------------------------------------------------------------------o
//|   Function    -  bool cSocket::CryptClient( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the true if the socket is set to crypt mode
//o---------------------------------------------------------------------------o
bool cSocket::CryptClient( void ) const
{
	return cryptclient;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::CryptClient( bool newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the value of the socket's crypt mode
//o---------------------------------------------------------------------------o
void cSocket::CryptClient( bool newValue )
{
	cryptclient = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  char *cSocket::XText( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the socket's xtext buffer
//o---------------------------------------------------------------------------o
char *cSocket::XText( void )
{
	return &xtext[0];
}

void cSocket::XText( const char *newValue )
{
	strncpy( xtext, newValue, 64 );
	xtext[64] = 0;
}

//o---------------------------------------------------------------------------o
//|   Function    -  CItem *cSocket::AddMItem( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the socket's addmitem
//o---------------------------------------------------------------------------o
CItem *cSocket::AddMItem( void ) const
{
	return addmitem;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddMItem( CItem *newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the socket's addmitem
//o---------------------------------------------------------------------------o
void cSocket::AddMItem( CItem *newValue )
{
	addmitem = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  SI08 cSocket::AddZ( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Return's the socket's addz
//o---------------------------------------------------------------------------o
SI08 cSocket::AddZ( void ) const
{
	return addz;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddZ( SI08 newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the socket's addz
//o---------------------------------------------------------------------------o
void cSocket::AddZ( SI08 newValue )
{
	addz = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  int cSocket::AddX( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the addx associated with the socket
//o---------------------------------------------------------------------------o
SI32 cSocket::AddX( void ) const
{
	return addx[0];
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddX( int newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the socket's addx value
//o---------------------------------------------------------------------------o
void cSocket::AddX( SI32 newValue )
{
	addx[0] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  int cSocket::AddY( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the addy associated with the socket
//o---------------------------------------------------------------------------o
SI32 cSocket::AddY( void ) const
{
	return addy[0];
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddY( int newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the socket's addy value
//o---------------------------------------------------------------------------o
void cSocket::AddY( SI32 newValue )
{
	addy[0] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  int cSocket::AddX2( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the second addx associated with the socket
//o---------------------------------------------------------------------------o
SI32 cSocket::AddX2( void ) const
{
	return addx[1];
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddX2( int newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the socket's second addx value
//o---------------------------------------------------------------------------o
void cSocket::AddX2( SI32 newValue )
{
	addx[1] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  int cSocket::AddY2( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the socket's second addy value
//o---------------------------------------------------------------------------o
SI32 cSocket::AddY2( void ) const
{
	return addy[1];
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddY2( int newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the socket's second addy value
//o---------------------------------------------------------------------------o
void cSocket::AddY2( SI32 newValue )
{
	addy[1] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  UI08 cSocket::AddID1( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the first addid associated with the socket
//o---------------------------------------------------------------------------o
UI08 cSocket::AddID1( void ) const
{
	return addid[0];
}

//o---------------------------------------------------------------------------o
//|   Function    -  UI08 cSocket::AddID2( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the second addid associated with the socket
//o---------------------------------------------------------------------------o
UI08 cSocket::AddID2( void ) const
{
	return addid[1];
}

//o---------------------------------------------------------------------------o
//|   Function    -  UI08 cSocket::AddID3( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the third addid associated with the socket
//o---------------------------------------------------------------------------o
UI08 cSocket::AddID3( void ) const
{
	return addid[2];
}

//o---------------------------------------------------------------------------o
//|   Function    -  UI08 cSocket::AddID4( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the fourth addid associated with the socket
//o---------------------------------------------------------------------------o
UI08 cSocket::AddID4( void ) const
{
	return addid[3];
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddID1( UI08 newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the first addid associated with the socket
//o---------------------------------------------------------------------------o
void cSocket::AddID1( UI08 newValue )
{
	addid[0] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddID2( UI08 newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the second addid associated with the socket
//o---------------------------------------------------------------------------o
void cSocket::AddID2( UI08 newValue )
{
	addid[1] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddID3( UI08 newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the third addid associated with the socket
//o---------------------------------------------------------------------------o
void cSocket::AddID3( UI08 newValue )
{
	addid[2] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::AddID4( UI08 newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the fourth addid associated with the socket
//o---------------------------------------------------------------------------o
void cSocket::AddID4( UI08 newValue )
{
	addid[3] = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  char cSocket::DyeAll( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the dyeall status of a socket
//o---------------------------------------------------------------------------o
char cSocket::DyeAll( void ) const
{
	return dyeall;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::DyeAll( char newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the dyeall status of a socket
//o---------------------------------------------------------------------------o
void cSocket::DyeAll( char newValue )
{
	dyeall = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::CloseSocket( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Closes the open socket
//o---------------------------------------------------------------------------o
void cSocket::CloseSocket( void )
{
	closesocket( cliSocket );
}

//o---------------------------------------------------------------------------o
//|   Function    -  bool cSocket::FirstPacket( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns true if it's the first packet received
//o---------------------------------------------------------------------------o
bool cSocket::FirstPacket( void ) const
{
	return firstPacket;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::FirstPacket( bool newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets whether the socket has received it's first packet yet
//o---------------------------------------------------------------------------o
void cSocket::FirstPacket( bool newValue )
{
	firstPacket = newValue;
}

//o---------------------------------------------------------------------------o
//|   Function    -  SI32 cSocket::IdleTimeout( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the time point at which the char times out
//o---------------------------------------------------------------------------o
SI32 cSocket::IdleTimeout( void ) const
{
	return idleTimeout;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::IdleTimeout( long newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the time point at which the char times out
//o---------------------------------------------------------------------------o
void cSocket::IdleTimeout( SI32 newValue )
{
	idleTimeout = newValue;
	wasIdleWarned = false;
}

bool cSocket::WasIdleWarned( void ) const
{
	return wasIdleWarned;
}

void cSocket::WasIdleWarned( bool value )
{
	wasIdleWarned = value;
}

//o---------------------------------------------------------------------------o
//|   Function    -  UI08 *cSocket::Buffer( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns a pointer to the buffer of the socket
//o---------------------------------------------------------------------------o
UI08 *cSocket::Buffer( void )
{
	return &buffer[0];
}

//o---------------------------------------------------------------------------o
//|   Function    -  UI08 *cSocket::OutBuffer( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns a pointer to the outgoing buffer of the socket
//o---------------------------------------------------------------------------o
UI08 *cSocket::OutBuffer( void )
{
	return &outbuffer[0];
}

//o---------------------------------------------------------------------------o
//|   Function    -  UI08 *cSocket::TBuffer( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns a pointer to the socket's TBuffer
//o---------------------------------------------------------------------------o
UI08 *cSocket::TBuffer( void )
{
	return &tbuffer[0];
}

//o---------------------------------------------------------------------------o
//|   Function    -  SI16 cSocket::WalkSequence( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the point in the walk sequence of the socket
//o---------------------------------------------------------------------------o
SI16 cSocket::WalkSequence( void ) const
{
	return walkSequence;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::WalkSequence( SI16 newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the walk sequence value of the socket
//o---------------------------------------------------------------------------o
void cSocket::WalkSequence( SI16 newValue )
{
	walkSequence = newValue;
}

//o---------------------------------------------------------------------------o
//|		Function    -	UI16 cSocket::AcctNo( void )
//|		Date        -	November 29th, 2000
//|		Programmer  -	Abaddon
//|		Modified	-	Maarc, February 3, 2003 - reduced to UI16 to deal with
//|						accounts changes
//o---------------------------------------------------------------------------o
//|		Purpose     -	Returns the ID of the account number socket belongs to
//o---------------------------------------------------------------------------o
UI16 cSocket::AcctNo( void ) const
{
	return wAccountID;
}

//o---------------------------------------------------------------------------o
//|		Function    -	void cSocket::AcctNo( UI16 newValue )
//|		Date        -	November 29th, 2000
//|		Programmer  -	Abaddon
//|		Modified	-	Maarc, February 3, 2003 - reduced to UI16 to deal with
//|						accounts changes
//o---------------------------------------------------------------------------o
//|		Purpose     -	Sets the ID of the account number the socket belongs to
//o---------------------------------------------------------------------------o
void cSocket::AcctNo( UI16 newValue )
{
	if( !Accounts->GetAccountByID( newValue, (ACCOUNTSBLOCK&)actbAccount ) )
	{
		wAccountID = AB_INVALID_ID;
		actbAccount.wAccountIndex = AB_INVALID_ID;
		return;
	}
	wAccountID = actbAccount.wAccountIndex;
}

//o---------------------------------------------------------------------------o
//|   Function    -  int cSocket::TempInt( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the tempint of the socket
//o---------------------------------------------------------------------------o
SI32 cSocket::TempInt( void ) const
{
	return tempint;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::TempInt( int newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the tempint value of the socket
//o---------------------------------------------------------------------------o
void cSocket::TempInt( SI32 newValue )
{
	tempint = newValue;
}

CChar *					DEFSOCK_CURRCHAROBJ				= NULL;
const SI32				DEFSOCK_IDLETIMEOUT				= -1;
const UI16				DEFSOCK_ACCOUNTID				= AB_INVALID_ID;
const SI32				DEFSOCK_TEMPINT					= 0;
const char				DEFSOCK_DYEALL					= 0;
const SI08				DEFSOCK_ADDZ					= 0;
CItem *					DEFSOCK_ADDMITEM				= NULL;
const bool				DEFSOCK_NEWCLIENT				= true;
const bool				DEFSOCK_FIRSTPACKET				= true;
const UI08				DEFSOCK_RANGE					= 15;
const bool				DEFSOCK_CRYPTCLIENT				= false;
const int				DEFSOCK_CLISOCKET				= INVALID_SOCKET;
const SI16				DEFSOCK_WALKSEQUENCE			= -1;
const char				DEFSOCK_CURSPELLTYPE			= 0;
const int				DEFSOCK_OUTLENGTH				= 0;
const int				DEFSOCK_INLENGTH				= 0;
const bool				DEFSOCK_LOGGING					= LOGDEFAULT;
const int				DEFSOCK_POSTACKCOUNT			= 0;
const int				DEFSOCK_POSTCOUNT				= 0;
const PickupLocations	DEFSOCK_PSPOT					= PL_NOWHERE;
const SERIAL			DEFSOCK_PFROM					= INVALIDSERIAL;
const SI16				DEFSOCK_PX						= 0;
const SI16				DEFSOCK_PY						= 0;
const SI08				DEFSOCK_PZ						= 0;
const UnicodeTypes		DEFSOCK_LANG					= UT_ENU;
const ClientTypes		DEFSOCK_CLITYPE					= DefaultClientType;
const UI32				DEFSOCK_CLIENTVERSION			= DefaultClientVersion;
const UI32				DEFSOCK_BYTESSENT				= 0;
const UI32				DEFSOCK_BYTESRECEIVED			= 0;
const bool				DEFSOCK_RECEIVEDVERSION			= false;

cSocket::cSocket() : currCharObj( DEFSOCK_CURRCHAROBJ )/*, actbAccount()*/, idleTimeout( DEFSOCK_IDLETIMEOUT ), wAccountID( DEFSOCK_ACCOUNTID ),
tempint( DEFSOCK_TEMPINT ), dyeall( DEFSOCK_DYEALL ), addz( DEFSOCK_ADDZ ), addmitem( DEFSOCK_ADDMITEM ), newClient( DEFSOCK_NEWCLIENT ), 
firstPacket( DEFSOCK_FIRSTPACKET ), range( DEFSOCK_RANGE ), cryptclient( DEFSOCK_CRYPTCLIENT ), cliSocket( DEFSOCK_CLISOCKET ), 
walkSequence( DEFSOCK_WALKSEQUENCE ), currentSpellType( DEFSOCK_CURSPELLTYPE ), outlength( DEFSOCK_OUTLENGTH ), inlength( DEFSOCK_INLENGTH ), 
logging( DEFSOCK_LOGGING ), postAckCount( DEFSOCK_POSTACKCOUNT ), postCount( DEFSOCK_POSTCOUNT ), pSpot( DEFSOCK_PSPOT ), pFrom( DEFSOCK_PFROM ),
pX( DEFSOCK_PX ), pY( DEFSOCK_PY ), pZ( DEFSOCK_PZ ), lang( DEFSOCK_LANG ), cliType( DEFSOCK_CLITYPE ), clientVersion( DEFSOCK_CLIENTVERSION ), 
bytesReceived( DEFSOCK_BYTESRECEIVED ), bytesSent( DEFSOCK_BYTESSENT ), receivedVersion( DEFSOCK_RECEIVEDVERSION )
{
	InternalReset();
	actbAccount.wAccountIndex = AB_INVALID_ID;
}

cSocket::~cSocket()
{
	closesocket( cliSocket );
}

//o---------------------------------------------------------------------------o
//|   Function    -  char cSocket::CurrentSpellType( void )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Returns the current spell type of the socket
//o---------------------------------------------------------------------------o
char cSocket::CurrentSpellType( void ) const
{
	return currentSpellType;
}

//o---------------------------------------------------------------------------o
//|   Function    -  void cSocket::CurrentSpellType( char newValue )
//|   Date        -  November 29th, 2000
//|   Programmer  -  Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -  Sets the spell type of the socket
//o---------------------------------------------------------------------------o
void cSocket::CurrentSpellType( char newValue )
{
	currentSpellType = newValue;
}

bool cSocket::FlushBuffer( bool doLog )
{
	if( outlength > 0 )
	{
		int len;
		if( cryptclient )
		{
			char xoutbuffer[MAXBUFFER*2];
			len = Pack( outbuffer, xoutbuffer, outlength );
			send( cliSocket, xoutbuffer, len, 0 );
		}
		else
		{
			//char xoutbuffer[MAXBUFFER*2];
			//len = Pack( outbuffer,xoutbuffer,outlength);
			send( cliSocket, (char *)&outbuffer[0], outlength, 0 );
		}
		if( Logging() && doLog )
		{
			FILE *fp;
			SERIAL toPrint;
			if( currCharObj == NULL )
				toPrint = INVALIDSERIAL;
			else
				toPrint = currCharObj->GetSerial();
			std::string filename = cwmWorldState->ServerData()->Directory( CSDDP_LOGS ) + UString::number( toPrint ) + ".snd";
			char qbuffer[8];
			int j = 0;
			memset( qbuffer, 0x00, 8 );
			if( ( fp = fopen( filename.c_str(), "a" ) ) != NULL )
			{
				fprintf( fp, "[SEND]Packet: 0x%2X --> Length: %li\n", outbuffer[0], outlength );
				for( int i = 0; i < outlength; ++i )
				{
					qbuffer[j++] = outbuffer[i];
					fprintf( fp, "0x%2X ", outbuffer[i] );
					if( j >= 7 )
					{
						fprintf( fp, "  %s\n", qbuffer );
						j = 0;
					}
				}
				fprintf( fp, "  %s\n\n", qbuffer );
				fclose( fp );
			}
			else
			{
				Console.Error( 1, "Failed to open socket log %s", filename.c_str() );
			}
			bytesSent += outlength;
		}
		outlength = 0;
		return true;
	}
	else
		return false;
}

bool cSocket::FlushLargeBuffer( bool doLog )
{
	if( outlength > 0 )
	{
		if( cryptclient )
		{
			largePackBuffer.resize( outlength * 2 );
			int len = Pack( &largeBuffer[0], &largePackBuffer[0], outlength );
			send( cliSocket, (char *)&largePackBuffer[0], len, 0 );
		}
		else
			send( cliSocket, (char *)&largeBuffer[0], outlength, 0 );

		if( Logging() && doLog )
		{
			FILE *fp = NULL;
			SERIAL toPrint;
			if( currCharObj == NULL )
				toPrint = INVALIDSERIAL;
			else
				toPrint = currCharObj->GetSerial();
			std::string filename = cwmWorldState->ServerData()->Directory( CSDDP_LOGS ) + UString::number( toPrint ) + ".snd";
			char qbuffer[8];
			int j = 0;
			memset( qbuffer, 0x00, 8 );
			if( ( fp = fopen( filename.c_str(), "a" ) ) != NULL )
			{
				fprintf( fp, "[SEND]Packet: 0x%2X --> Length: %li\n", outbuffer[0], outlength );
				for( int i = 0; i < outlength; ++i )
				{
					qbuffer[j++] = largeBuffer[i];
					fprintf( fp, "0x%2X ", largeBuffer[i] );
					if( j >= 7 )
					{
						fprintf( fp, "  %s\n", qbuffer );
						j = 0;
					}
				}
				fprintf( fp, "  %s\n\n", qbuffer );
				fclose( fp );
			}
			else
				Console.Error( 1, "Failed to open socket log %s", filename.c_str() );
			bytesSent += outlength;
		}
		outlength = 0;
		return true;
	}
	else
		return false;
}

static UI32 bit_table[257][2] =
{
	{0x02, 0x00}, 	{0x05, 0x1F}, 	{0x06, 0x22}, 	{0x07, 0x34}, 	{0x07, 0x75}, 	{0x06, 0x28}, 	{0x06, 0x3B}, 	{0x07, 0x32}, 
	{0x08, 0xE0}, 	{0x08, 0x62}, 	{0x07, 0x56}, 	{0x08, 0x79}, 	{0x09, 0x19D},	{0x08, 0x97}, 	{0x06, 0x2A}, 	{0x07, 0x57}, 
	{0x08, 0x71}, 	{0x08, 0x5B}, 	{0x09, 0x1CC},	{0x08, 0xA7}, 	{0x07, 0x25}, 	{0x07, 0x4F}, 	{0x08, 0x66}, 	{0x08, 0x7D}, 
	{0x09, 0x191},	{0x09, 0x1CE}, 	{0x07, 0x3F}, 	{0x09, 0x90}, 	{0x08, 0x59}, 	{0x08, 0x7B}, 	{0x08, 0x91}, 	{0x08, 0xC6}, 
	{0x06, 0x2D}, 	{0x09, 0x186}, 	{0x08, 0x6F}, 	{0x09, 0x93}, 	{0x0A, 0x1CC},	{0x08, 0x5A}, 	{0x0A, 0x1AE},	{0x0A, 0x1C0}, 
	{0x09, 0x148},	{0x09, 0x14A}, 	{0x09, 0x82}, 	{0x0A, 0x19F}, 	{0x09, 0x171},	{0x09, 0x120}, 	{0x09, 0xE7}, 	{0x0A, 0x1F3}, 
	{0x09, 0x14B},	{0x09, 0x100},	{0x09, 0x190},	{0x06, 0x13}, 	{0x09, 0x161},	{0x09, 0x125},	{0x09, 0x133},	{0x09, 0x195}, 
	{0x09, 0x173},	{0x09, 0x1CA},	{0x09, 0x86}, 	{0x09, 0x1E9}, 	{0x09, 0xDB}, 	{0x09, 0x1EC},	{0x09, 0x8B}, 	{0x09, 0x85}, 
	{0x05, 0x0A}, 	{0x08, 0x96}, 	{0x08, 0x9C}, 	{0x09, 0x1C3}, 	{0x09, 0x19C},	{0x09, 0x8F}, 	{0x09, 0x18F},	{0x09, 0x91}, 
	{0x09, 0x87}, 	{0x09, 0xC6}, 	{0x09, 0x177},	{0x09, 0x89}, 	{0x09, 0xD6}, 	{0x09, 0x8C}, 	{0x09, 0x1EE},	{0x09, 0x1EB}, 
	{0x09, 0x84}, 	{0x09, 0x164}, 	{0x09, 0x175},	{0x09, 0x1CD}, 	{0x08, 0x5E}, 	{0x09, 0x88}, 	{0x09, 0x12B},	{0x09, 0x172}, 
	{0x09, 0x10A},	{0x09, 0x8D}, 	{0x09, 0x13A},	{0x09, 0x11C}, 	{0x0A, 0x1E1},	{0x0A, 0x1E0}, 	{0x09, 0x187},	{0x0A, 0x1DC}, 
	{0x0A, 0x1DF},	{0x07, 0x74}, 	{0x09, 0x19F},	{0x08, 0x8D},	{0x08, 0xE4}, 	{0x07, 0x79}, 	{0x09, 0xEA}, 	{0x09, 0xE1}, 
	{0x08, 0x40}, 	{0x07, 0x41}, 	{0x09, 0x10B},	{0x09, 0xB0}, 	{0x08, 0x6A}, 	{0x08, 0xC1}, 	{0x07, 0x71}, 	{0x07, 0x78}, 
	{0x08, 0xB1}, 	{0x09, 0x14C}, 	{0x07, 0x43}, 	{0x08, 0x76}, 	{0x07, 0x66}, 	{0x07, 0x4D}, 	{0x09, 0x8A}, 	{0x06, 0x2F}, 
	{0x08, 0xC9},	{0x09, 0xCE}, 	{0x09, 0x149},	{0x09, 0x160}, 	{0x0A, 0x1BA}, 	{0x0A, 0x19E}, 	{0x0A, 0x39F}, 	{0x09, 0xE5}, 
	{0x09, 0x194}, 	{0x09, 0x184}, 	{0x09, 0x126}, 	{0x07, 0x30}, 	{0x08, 0x6C}, 	{0x09, 0x121}, 	{0x09, 0x1E8}, 	{0x0A, 0x1C1}, 
	{0x0A, 0x11D}, 	{0x0A, 0x163}, 	{0x0A, 0x385}, 	{0x0A, 0x3DB}, 	{0x0A, 0x17D}, 	{0x0A, 0x106}, 	{0x0A, 0x397}, 	{0x0A, 0x24E}, 
	{0x07, 0x2E}, 	{0x08, 0x98}, 	{0x0A, 0x33C}, 	{0x0A, 0x32E}, 	{0x0A, 0x1E9}, 	{0x09, 0xBF}, 	{0x0A, 0x3DF}, 	{0x0A, 0x1DD}, 
	{0x0A, 0x32D}, 	{0x0A, 0x2ED}, 	{0x0A, 0x30B}, 	{0x0A, 0x107}, 	{0x0A, 0x2E8}, 	{0x0A, 0x3DE}, 	{0x0A, 0x125}, 	{0x0A, 0x1E8}, 
	{0x09, 0xE9}, 	{0x0A, 0x1CD}, 	{0x0A, 0x1B5}, 	{0x09, 0x165}, 	{0x0A, 0x232}, 	{0x0A, 0x2E1}, 	{0x0B, 0x3AE}, 	{0x0B, 0x3C6}, 
	{0x0B, 0x3E2}, 	{0x0A, 0x205}, 	{0x0A, 0x29A}, 	{0x0A, 0x248}, 	{0x0A, 0x2CD}, 	{0x0A, 0x23B}, 	{0x0B, 0x3C5}, 	{0x0A, 0x251}, 
	{0x0A, 0x2E9}, 	{0x0A, 0x252}, 	{0x09, 0x1EA}, 	{0x0B, 0x3A0}, 	{0x0B, 0x391}, 	{0x0A, 0x23C}, 	{0x0B, 0x392}, 	{0x0B, 0x3D5}, 
	{0x0A, 0x233}, 	{0x0A, 0x2CC}, 	{0x0B, 0x390}, 	{0x0A, 0x1BB}, 	{0x0B, 0x3A1}, 	{0x0B, 0x3C4}, 	{0x0A, 0x211}, 	{0x0A, 0x203}, 
	{0x09, 0x12A}, 	{0x0A, 0x231}, 	{0x0B, 0x3E0}, 	{0x0A, 0x29B}, 	{0x0B, 0x3D7}, 	{0x0A, 0x202}, 	{0x0B, 0x3AD}, 	{0x0A, 0x213}, 
	{0x0A, 0x253}, 	{0x0A, 0x32C}, 	{0x0A, 0x23D}, 	{0x0A, 0x23F}, 	{0x0A, 0x32F}, 	{0x0A, 0x11C}, 	{0x0A, 0x384}, 	{0x0A, 0x31C}, 
	{0x0A, 0x17C}, 	{0x0A, 0x30A}, 	{0x0A, 0x2E0}, 	{0x0A, 0x276}, 	{0x0A, 0x250}, 	{0x0B, 0x3E3}, 	{0x0A, 0x396}, 	{0x0A, 0x18F}, 
	{0x0A, 0x204}, 	{0x0A, 0x206}, 	{0x0A, 0x230}, 	{0x0A, 0x265}, 	{0x0A, 0x212}, 	{0x0A, 0x23E}, 	{0x0B, 0x3AC}, 	{0x0B, 0x393}, 
	{0x0B, 0x3E1}, 	{0x0A, 0x1DE}, 	{0x0B, 0x3D6}, 	{0x0A, 0x31D}, 	{0x0B, 0x3E5}, 	{0x0B, 0x3E4}, 	{0x0A, 0x207}, 	{0x0B, 0x3C7}, 
	{0x0A, 0x277}, 	{0x0B, 0x3D4}, 	{0x08, 0xC0},	{0x0A, 0x162}, 	{0x0A, 0x3DA}, 	{0x0A, 0x124}, 	{0x0A, 0x1B4}, 	{0x0A, 0x264}, 
	{0x0A, 0x33D}, 	{0x0A, 0x1D1}, 	{0x0A, 0x1AF}, 	{0x0A, 0x39E}, 	{0x0A, 0x24F}, 	{0x0B, 0x373}, 	{0x0A, 0x249}, 	{0x0B, 0x372}, 
	{0x09, 0x167}, 	{0x0A, 0x210}, 	{0x0A, 0x23A}, 	{0x0A, 0x1B8}, 	{0x0B, 0x3AF}, 	{0x0A, 0x18E}, 	{0x0A, 0x2EC}, 	{0x07, 0x62}, 
	{0x04, 0x0D}
};

int cSocket::Pack( void *pvIn, void *pvOut, int len )
{
	UI08 *pIn = (UI08 *)pvIn;
	UI08 *pOut = (UI08 *)pvOut;

	int actByte = 0, bitByte = 0;
	int nrBits;
	UI32 value;

	while( len-- )
	{
		nrBits = bit_table[*pIn][0];
		value = bit_table[*pIn++][1];

		while( nrBits-- )
		{
			pOut[actByte] = (pOut[actByte] << 1) | (UI08)((value >> nrBits) & 0x1);

			bitByte = (bitByte + 1) & 0x07;
			if( !bitByte )
				++actByte;
		}
	}

	nrBits = bit_table[256][0];
	value = bit_table[256][1];

	while( nrBits-- )
	{
		pOut[actByte] = (pOut[actByte] << 1) | (UI08)((value >> nrBits) & 0x1);

		bitByte = (bitByte + 1) & 0x07;
		if( !bitByte )
			++actByte;
	}

	if( bitByte )
	{
		while( bitByte < 8 )
		{
			pOut[actByte] <<= 1;
			++bitByte;
		}
		++actByte;
	}
	return actByte;
}

void cSocket::Send( const void *point, int length ) // Buffering send function
{
	if( outlength + length > MAXBUFFER )
		FlushBuffer();
	if( outlength > 0 )
		Console.Print( "Fragmented packet! [packet: %i]\n", outbuffer[0] );
	if( GetAccount().bPackPacket )
	{
		char packbuff[MAXBUFFER+2];
		int count = Pack( (void*)point, packbuff, length );
		memcpy( &outbuffer[outlength], packbuff, count );
	}
	else
	{
		// sometimes we send enormous packets... oh what fun
		if( length > MAXBUFFER )
		{
#ifdef _DEBUG
			Console.Print( "Large packet found [%i]\n", outbuffer[0] );
#endif
			largeBuffer.resize( length );
			memcpy( &largeBuffer[0], point, length );
			outlength = length;
			FlushLargeBuffer();
			return;
		}
		else
			memcpy( &outbuffer[outlength], point, length );
	}
	outlength += length;
}

#if defined(__unix__)
int GrabLastError( void )
{
	return errno;
}
#else
int GrabLastError( void )
{
	return WSAGetLastError();
}
#endif

void cSocket::FlushIncoming( void )
{
	int count = 0;
	do
	{
		count = recv( cliSocket, (char *)&buffer[inlength], 1, 0 );
	} while( count > 0 );
}

void cSocket::ReceiveLogging( cPInputBuffer *toLog )
{
	if( Logging() )
	{
		FILE *fp = NULL;
		SERIAL toPrint;
		if( currCharObj == NULL )
			toPrint = INVALIDSERIAL;
		else
			toPrint = currCharObj->GetSerial();
			std::string filename = cwmWorldState->ServerData()->Directory( CSDDP_LOGS ) + UString::number( toPrint ) + ".rcv";
		fp = fopen( filename.c_str(), "a" );

		if( toLog != NULL )
			toLog->Log( fp );
		else
		{
			char qbuffer[8];	
			memset( qbuffer, 0x00, 8 );
			int j = 0;
			if( fp != NULL )
			{
				fprintf( fp, "[RECV]Packet: 0x%2X --> Length: %li\n\n", buffer[0], inlength );
				for( int counter = 0; counter < inlength; ++counter )
				{
					qbuffer[j++] = buffer[counter];
					fprintf(fp, "0x%2X ", buffer[counter] );
					if( j >= 7 )
					{
						fprintf( fp, "  %s\n", qbuffer );
						j = 0;
					}
				}
				fprintf( fp, "  %s\n\n", qbuffer );
				fclose( fp );
			}
			else
				Console.Error( 1, "Failed to open socket log %s", filename.c_str() );
		}
		if( fp != NULL )
		{
			fflush( fp );
			fclose( fp );
		}
	}
}
int cSocket::Receive( int x, bool doLog )
{
	int count			= 0;
	UI08 recvAttempts	= 0;
	long curTime		= getclock();
	long nexTime		= curTime;
	do
	{
		count = recv( cliSocket, (char *)&buffer[inlength], x - inlength, 0 );
		if( count > 0 )
		{
			inlength += count;
		}
		else if( count == SOCKET_ERROR )
		{
			int lastError = GrabLastError();
#if defined(__unix__)
			if( lastError != EWOULDBLOCK )
#else
			if( lastError != WSAEWOULDBLOCK )
#endif
				throw socket_error( lastError );
			UOXSleep( 20 );
		}
		++recvAttempts;
		nexTime = getclock();
		if( recvAttempts == 24 || (nexTime - curTime) > 2 * 1000 )	// looks like we're not going to get it!
			throw socket_error( "Socket receive error" );
	}
	while( x != 2560 && x != inlength );
	if( doLog )
		ReceiveLogging( NULL );
	bytesReceived += count;
	return count;
}

void cSocket::InternalReset( void )
{
//	memset( &itemmake, 0, sizeof( make_st ) );
	memset( buffer, 0, MAXBUFFER );
	memset( outbuffer, 0, MAXBUFFER );
	memset( tbuffer, 0, MAXBUFFER );
	memset( xtext, 0, 65 );
	ClearAuthor();
	ClearTitle();
	ClearPage();
	addx[0] = addx[1] = 0;
	addy[0] = addy[1] = 0;
	addid[0] = addid[1] = addid[2] = addid[3] = 0;
	clientip[0] = clientip[1] = clientip[2] = clientip[3] = 0;
	// set the socket to nonblocking
	UI32 mode = 1;
	ioctlsocket( cliSocket, FIONBIO, &mode );
}
cSocket::cSocket( int sockNum ) : currCharObj( NULL )/*, actbAccount()*/, idleTimeout( -1 ), wAccountID(AB_INVALID_ID),
tempint( 0 ), dyeall( 0 ), addz( 0 ), addmitem( NULL ), newClient( true ), firstPacket( true ), range( 15 ),
cryptclient( false ), cliSocket( sockNum ), walkSequence( -1 ), currentSpellType( 0 ),
outlength( 0 ), inlength( 0 ), logging( LOGDEFAULT ), postCount( 0 ), postAckCount( 0 ), pSpot( PL_NOWHERE ), pFrom( INVALIDSERIAL ),
pX( 0 ), pY( 0 ), pZ( 0 ), lang( UT_ENU ), cliType( DefaultClientType ), clientVersion( DefaultClientVersion ), bytesReceived( 0 ), bytesSent( 0 ),
receivedVersion( false )
{
	InternalReset();
}

void cSocket::ClearAuthor( void )
{
	memset( authorbuffer, '~', 32 );
}
void cSocket::ClearTitle( void )
{
	memset( titlebuffer, '~', 62 );
}
void cSocket::ClearPage( void )
{
	memset( pagebuffer, '~', 512 );
}

void cSocket::OutLength( int newValue )
{
	outlength = newValue;
}
void cSocket::InLength( int newValue )
{
	inlength = newValue;
}
int cSocket::OutLength( void ) const
{
	return outlength;
}
int cSocket::InLength( void ) const
{
	return inlength;
}

bool cSocket::Logging( void ) const
{
	return logging;
}

void cSocket::Logging( bool newValue )
{
	logging = newValue;
}

CChar *cSocket::CurrcharObj( void ) const
{
	return currCharObj;
}

void cSocket::CurrcharObj( CChar *newValue )
{
	currCharObj = newValue;
}

//o--------------------------------------------------------------------------o
//|	Function			-	ACCOUNTSBLOCK &cSocket::GetAccount(void)
//|	Date					-	1/17/2003 6:21:59 AM
//|	Developers		-	EviLDeD
//|	Organization	-	UOX3 DevTeam
//|	Status				-	Currently under development
//o--------------------------------------------------------------------------o
//|	Description		-	Return to the calling function this objects accounts 
//|									referance.
//o--------------------------------------------------------------------------o
//| Modifications	-	
//o--------------------------------------------------------------------------o
ACCOUNTSBLOCK &cSocket::GetAccount(void)
{
	return actbAccount;
}

//o--------------------------------------------------------------------------o
//|	Function			-	void cSocket::SetAccount(ACCOUNTSBLOCK &actbBlock)
//|	Date					-	1/17/2003 7:01:23 AM
//|	Developers		-	EviLDeD
//|	Organization	-	UOX3 DevTeam
//|	Status				-	Currently under development
//o--------------------------------------------------------------------------o
//|	Description		-	
//o--------------------------------------------------------------------------o
//| Modifications	-	
//o--------------------------------------------------------------------------o
void cSocket::SetAccount(ACCOUNTSBLOCK &actbBlock)
{
	if( actbBlock.wAccountIndex==AB_INVALID_ID )
	{
		actbAccount.wAccountIndex=AB_INVALID_ID;
		return;
	}
	actbAccount=actbBlock;
	wAccountID=actbAccount.wAccountIndex;
}
//
void cSocket::SetAccount(std::string sUsername)
{
	if( !Accounts->GetAccountByName(sUsername,(ACCOUNTSBLOCK&)actbAccount) )
	{
		// Ok there was an error setting an account to this character.
		wAccountID=AB_INVALID_ID;
		actbAccount.wAccountIndex=wAccountID;
		return;
	}
	wAccountID = actbAccount.wAccountIndex;
}
//
void cSocket::SetAccount(UI16 wNewAccountID)
{
	if( !Accounts->GetAccountByID(wNewAccountID,(ACCOUNTSBLOCK&)actbAccount) )
	{
		// Ok there was an error setting an account to this character.
		wAccountID=AB_INVALID_ID;
		actbAccount.wAccountIndex=wAccountID;
		return;
	}
	wAccountID = actbAccount.wAccountIndex;
}
//


UI08 cSocket::ClientIP1( void ) const
{
	return clientip[0];
}
UI08 cSocket::ClientIP2( void ) const
{
	return clientip[1];
}
UI08 cSocket::ClientIP3( void ) const
{
	return clientip[2];
}
UI08 cSocket::ClientIP4( void ) const
{
	return clientip[3];
}

void cSocket::ClientIP1( UI08 newValue )
{
	clientip[0] = newValue;
}
void cSocket::ClientIP2( UI08 newValue )
{
	clientip[1] = newValue;
}
void cSocket::ClientIP3( UI08 newValue )
{
	clientip[2] = newValue;
}
void cSocket::ClientIP4( UI08 newValue )
{
	clientip[3] = newValue;
}

bool cSocket::NewClient( void ) const
{
	return newClient;
}

void cSocket::NewClient( bool newValue )
{
	newClient = newValue;
}

UI32 cSocket::GetDWord( size_t offset )
{
	return calcserial( buffer[offset], buffer[offset+1], buffer[offset+2], buffer[offset+3] );
}

UI16 cSocket::GetWord( size_t offset )
{
	return (UI16)((buffer[offset]<<8) + buffer[offset+1]);
}

UI08 cSocket::GetByte( size_t offset )
{
	return buffer[offset];
}

void cSocket::SetDWord( size_t offset, UI32 newValue )
{
	buffer[offset]   = (UI08)( newValue>>24 );
	buffer[offset+1] = (UI08)( newValue>>16 );
	buffer[offset+2] = (UI08)( newValue>>8 );
	buffer[offset+3] = (UI08)( newValue%256 );
}
void cSocket::SetWord( size_t offset, UI16 newValue ) 
{
	buffer[offset]   = (UI08)( newValue>>8 );
	buffer[offset+1] = (UI08)( newValue%256 );
}
void cSocket::SetByte( size_t offset, UI08 newValue )
{
	buffer[offset] = newValue;
}

void cSocket::ClientIP( UI32 newValue )
{
	clientip[0] = (UI08)( newValue>>24 );
	clientip[1] = (UI08)( newValue>>16 );
	clientip[2] = (UI08)( newValue>>8 );
	clientip[3] = (UI08)( newValue%256 );
}

void cSocket::TargetOK( bool newValue )
{
	targetok = newValue;
}

bool cSocket::TargetOK( void ) const
{
	return targetok;
}

void cSocket::AddID( UI32 newValue )
{
	addid[0] = (UI08)( newValue>>24 );
	addid[1] = (UI08)( newValue>>16 );
	addid[2] = (UI08)( newValue>>8 );
	addid[3] = (UI08)( newValue%256 );
}

UI32 cSocket::AddID( void ) const
{
	return calcserial( addid[0], addid[1], addid[2], addid[3] );
}

void cSocket::ClickX( SI16 newValue )
{
	clickx = newValue;
}
void cSocket::ClickY( SI16 newValue )
{
	clicky = newValue;
}
SI16 cSocket::ClickX( void ) const
{
	return clickx;
}
SI16 cSocket::ClickY( void ) const
{
	return clicky;
}

char cSocket::PostAcked( int x, int y ) const
{
	return postAcked[x][y];
}
int cSocket::PostCount( void ) const
{
	return postCount;
}
int cSocket::PostAckCount( void ) const
{
	return postAckCount;
}

void cSocket::PostAcked( int x, int y, char newValue )
{
	postAcked[x][y] = newValue;
}
void cSocket::PostCount( int newValue )
{
	postCount = newValue;
}
void cSocket::PostAckCount( int newValue )
{
	postAckCount = newValue;
}

void cSocket::Send( cPBaseBuffer *toSend )
{
	if( toSend == NULL )
		return;

	UI32 len = 0;
	if( cryptclient )
	{
		len = toSend->Pack();
		send( cliSocket, (char *)toSend->PackedPointer(), len, 0 );
	}
	else
	{
		len = toSend->Length();
		send( cliSocket, (char *)toSend->Pointer(), len, 0 );
	}

	bytesSent += len;

	if( Logging() )
	{
		FILE *fp = NULL;
		SERIAL toPrint;
		if( currCharObj == NULL )
			toPrint = INVALIDSERIAL;
		else
			toPrint = currCharObj->GetSerial();
		std::string filename = cwmWorldState->ServerData()->Directory( CSDDP_LOGS ) + UString::number( toPrint ) + ".snd";
		fp = fopen( filename.c_str(), "a" );
		toSend->Log( fp );
		if( fp != NULL )
		{
			fflush( fp );
			fclose( fp );
		}
	}
}

void cSocket::PickupSpot( PickupLocations newValue )
{
	pSpot = newValue;
}
PickupLocations	cSocket::PickupSpot( void ) const
{
	return pSpot;
}
SERIAL cSocket::PickupSerial( void ) const
{
	return pFrom;
}
void cSocket::PickupSerial( SERIAL pickupSerial )
{
	pFrom = pickupSerial;
}

SI16 cSocket::PickupX( void ) const
{
	return pX;
}

SI16 cSocket::PickupY( void ) const
{
	return pY;
}

SI08 cSocket::PickupZ( void ) const
{
	return pZ;
}

void cSocket::PickupLocation( SI16 x, SI16 y, SI08 z )
{
	pX = x;
	pY = y;
	pZ = z;
}

void cSocket::PickupX( SI16 x )
{
	pX = x;
}

void cSocket::PickupY( SI16 y )
{
	pY = y;
}

void cSocket::PickupZ( SI08 z )
{
	pZ = z;
}

char *cSocket::AuthorBuffer( void )
{
	return authorbuffer;
}
char *cSocket::TitleBuffer( void )
{
	return titlebuffer;
}
char *cSocket::PageBuffer( void )
{
	return pagebuffer;
}
void cSocket::AuthorBuffer( const char *newValue )
{
	strcpy( authorbuffer, newValue );
}
void cSocket::PageBuffer( const char *newValue )
{
	strcpy( pagebuffer, newValue );
}
void cSocket::TitleBuffer( const char *newValue )
{
	strcpy( titlebuffer, newValue );
}


// Helper function to help us packet data into an array
template< class T >
void PackInto( UI08 *toPack, size_t offset, T& value )
{
	size_t sizeVal		= sizeof( T );
	UI08 *recastedPtr	= reinterpret_cast< UI08 * >(&value);
	for( size_t i = 0; i < sizeVal; ++i )
	{
		toPack[offset + i] = recastedPtr[i];
	}
}


// cPBaseBuffer class implementation
size_t cPBaseBuffer::Length( void ) const
{
	return internalBuffer.size();
}

UI32 cPBaseBuffer::PackedLength( void ) const
{
	return packedLength;
}

void cPBaseBuffer::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	char qbuffer[8];
	char j = 0;
	if( fullHeader )
		fprintf( fp, "[SEND]Packet: 0x%2X --> Length: %li\n", internalBuffer[0], internalBuffer.size() );
	for( UI32 i = 0; i < internalBuffer.size(); ++i )
	{
		qbuffer[j++] = internalBuffer[i];
		if( internalBuffer[i] < 10 )
			fprintf( fp, "0x0%X ", internalBuffer[i] );
		else
			fprintf( fp, "0x%2X ", internalBuffer[i] );
		if( j >= 7 )
		{
			fprintf( fp, "  %s\n", qbuffer );
			j = 0;
		}
	}
	fprintf( fp, "  %s\n\n", qbuffer );
}

UI08& cPBaseBuffer::operator [] ( size_t Num )
{
	if( Num > Length() )
		throw std::runtime_error( "out of bounds" );
	return internalBuffer[Num];
}

void cPBaseBuffer::InternalReset( void )
{
	internalBuffer.resize( 0 );
	packedBuffer.resize( 0 );
	isPacked		=	false;
	packedLength	=	0;
	internalBufferOffset = 0xFFFFFFFF;	// we aren't positioned yet
}
cPBaseBuffer::cPBaseBuffer()
{
	InternalReset();
}

cPBaseBuffer::cPBaseBuffer( char *initBuffer, size_t len )
{
	InternalReset();
	internalBuffer.resize( len );
	for( size_t i = 0; i < len; ++i )
		internalBuffer[i] = initBuffer[i];
	internalBufferOffset = len;
}

cPBaseBuffer::cPBaseBuffer( cPBaseBuffer *initBuffer )
{
	InternalReset();
	internalBuffer.resize( initBuffer->Length() );
	for( size_t i = 0; i < initBuffer->Length(); ++i )
		internalBuffer[i] = (*initBuffer)[i];
}

cPBaseBuffer::~cPBaseBuffer()
{
	internalBuffer.resize( 0 );
	packedBuffer.resize( 0 );
}

void cPBaseBuffer::Resize( size_t newLen )
{
	internalBuffer.resize( newLen );
}

const UI08 *cPBaseBuffer::Pointer( void ) const
{
	return (const UI08 *)&internalBuffer[0];
}

const UI08 *cPBaseBuffer::PackedPointer( void ) const
{
	return (const UI08 *)&packedBuffer[0];
}

cPBaseBuffer& cPBaseBuffer::operator<<( const UI08 value )
{
	if( internalBufferOffset == 0xFFFFFFFF )
	{
		internalBuffer.resize( 1 );
		internalBufferOffset = 0;
	}
	else
		internalBuffer.resize( internalBufferOffset + 1 );
	PackInto( &internalBuffer[0], internalBufferOffset, (UI08)value );
	return (*this);
}
cPBaseBuffer& cPBaseBuffer::operator<<( const UI16 value )
{
	if( internalBufferOffset == 0xFFFFFFFF )
	{
		internalBuffer.resize( 2 );
		internalBufferOffset = 0;
	}
	else
		internalBuffer.resize( internalBufferOffset + 2 );
	PackInto( &internalBuffer[0], internalBufferOffset, (UI16)value );
	return (*this);
}
cPBaseBuffer& cPBaseBuffer::operator<<( const UI32 value )
{
	if( internalBufferOffset == 0xFFFFFFFF )
	{
		internalBuffer.resize( 4 );
		internalBufferOffset = 0;
	}
	else
		internalBuffer.resize( internalBufferOffset + 4 );
	PackInto( &internalBuffer[0], internalBufferOffset, (UI32)value );
	return (*this);
}
cPBaseBuffer& cPBaseBuffer::operator<<( const SI08 value )
{
	return ( (*this) << static_cast< UI08 >(value) );
}
cPBaseBuffer& cPBaseBuffer::operator<<( const SI16 value )
{
	return ( (*this) << static_cast< UI16 >(value) );
}
cPBaseBuffer& cPBaseBuffer::operator<<( const SI32 value )
{
	return ( (*this) << static_cast< UI32 >(value) );
}
cPBaseBuffer& cPBaseBuffer::operator<<( const std::string& value )
{
	std::string::const_iterator i = value.begin();
	while( i != value.end() )
	{
		(*this) << static_cast< UI08 >(*i);
		++i;
	}
	return (*this);
}

/*UI32 cPBaseBuffer::Pack( void )
{
}
*/
cPUOXBuffer::cPUOXBuffer() : cPBaseBuffer()
{
}
cPUOXBuffer::cPUOXBuffer( char *initBuffer, size_t len ) : cPBaseBuffer( initBuffer, len )
{
}
cPUOXBuffer::~cPUOXBuffer()
{
}
cPUOXBuffer::cPUOXBuffer( cPBaseBuffer *initBuffer ) : cPBaseBuffer( initBuffer )
{
}
UI32 cPUOXBuffer::Pack( void )
{
	if( isPacked )
		return packedLength;

	packedBuffer.resize( internalBuffer.size() * 2 );
	size_t len	= Length();
	UI08 *pIn	= (UI08 *)&internalBuffer[0];
	UI08 *pOut	= (UI08 *)&packedBuffer[0];

	packedLength	= 0;
	int bitByte		= 0;
	int nrBits;
	UI32 value;

	while( len-- )
	{
		nrBits	= bit_table[*pIn][0];
		value	= bit_table[*pIn++][1];

		while( nrBits-- )
		{
			pOut[packedLength] = (pOut[packedLength] << 1) | (UI08)((value >> nrBits) & 0x1);

			bitByte = (bitByte + 1) & 0x07;
			if( !bitByte )
				++packedLength;
		}
	}

	nrBits	= bit_table[256][0];
	value	= bit_table[256][1];

	while( nrBits-- )
	{
		pOut[packedLength] = (pOut[packedLength] << 1) | (UI08)((value >> nrBits) & 0x1);

		bitByte = (bitByte + 1) & 0x07;
		if( !bitByte )
			++packedLength;
	}

	if( bitByte )
	{
		while( bitByte < 8 )
		{
			pOut[packedLength] <<= 1;
			++bitByte;
		}
		++packedLength;
	}
	isPacked	= true;
	return packedLength;
}


cPXGMBuffer::cPXGMBuffer() : cPBaseBuffer()
{
}
cPXGMBuffer::cPXGMBuffer( char *initBuffer, size_t len ) : cPBaseBuffer( initBuffer, len )
{
}
cPXGMBuffer::~cPXGMBuffer()
{
}
cPXGMBuffer::cPXGMBuffer( cPBaseBuffer *initBuffer ) : cPBaseBuffer( initBuffer )
{
}
UI32 cPXGMBuffer::Pack( void )
{
	return Length();
}
UI32 cPXGMBuffer::PackedLength( void ) const
{
	return Length();
}
const UI08 *cPXGMBuffer::PackedPointer( void ) const
{
	// we don't pack with xGM
	return Pointer();
}



cPInputBuffer::cPInputBuffer() : tSock( NULL )
{
}
cPInputBuffer::cPInputBuffer( cSocket *input )
{
	SetSocket( input );
}
void cPInputBuffer::Receive( void )
{
}
UI08& cPInputBuffer::operator[] ( size_t num )
{
	return internalBuffer[num];
}
size_t cPInputBuffer::Length( void )
{
	return internalBuffer.size();
}
UI08 *cPInputBuffer::Pointer( void )
{
	return &internalBuffer[0];
}
void cPInputBuffer::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	char qbuffer[8];
	char j			= 0;
	UI08 *buffer	= tSock->Buffer();
	const UI32 len	= tSock->InLength();
	UI08 actByte	= 0;
	if( fullHeader )
		fprintf( fp, "[RECV]Packet Class Generic: 0x%2X --> Length: %i\n", buffer[0], len );
	for( size_t i = 0; i < len; ++i )
	{
		actByte			= tSock->GetByte( i );
		qbuffer[j++]	= actByte;
		if( actByte < 10 )
			fprintf( fp, "0x0%X ", actByte );
		else
			fprintf( fp, "0x%2X ", actByte );
		if( j >= 7 )
		{
			fprintf( fp, "  %s\n", qbuffer );
			j = 0;
		}
	}
	fprintf( fp, "  %s\n\n", qbuffer );
}

long cPInputBuffer::DWord( size_t offset )
{
	return ( (internalBuffer[offset]<<24) + (internalBuffer[offset+1]<<16) + (internalBuffer[offset+2]<<8) + internalBuffer[offset+3] );
}

SI32 cPInputBuffer::Word( size_t offset )
{
	return ( (internalBuffer[offset]<<8) + internalBuffer[offset+1] );
}

UI08 cPInputBuffer::Byte( size_t offset )
{
	return internalBuffer[offset];
}


bool cPInputBuffer::Handle( void )
{
	return false;
}

void cPInputBuffer::SetSocket( cSocket *toSet )
{
	tSock = toSet;
}

cSocket *cPInputBuffer::GetSocket( void ) const
{
	return tSock;
}

UnicodeTypes cSocket::Language( void ) const
{
	return lang;
}

void cSocket::Language( UnicodeTypes newVal )
{
	lang = newVal;
}

UI32 cSocket::ClientVersion( void ) const
{
	return clientVersion;
}
void cSocket::ClientVersion( UI32 newVer )
{
	clientVersion = newVer;
}
void cSocket::ClientVersion( UI08 major, UI08 minor, UI08 sub, UI08 letter )
{
	ClientVersion( calcserial( major, minor, sub, letter ) );
}

ClientTypes cSocket::ClientType( void ) const
{
	return cliType;
}
void cSocket::ClientType( ClientTypes newVer )
{
	cliType = newVer;
}


UI08 cSocket::ClientVersionMajor( void ) const
{
	return (UI08)(clientVersion>>24);
}

UI08 cSocket::ClientVersionMinor( void ) const
{
	return (UI08)(clientVersion>>16);
}

UI08 cSocket::ClientVersionSub( void ) const
{
	return (UI08)(clientVersion>>8);
}

UI08 cSocket::ClientVersionLetter( void ) const
{
	return (UI08)(clientVersion%256);
}

void cSocket::ClientVersionMajor( UI08 value )
{
	ClientVersion( value, ClientVersionMinor(), ClientVersionSub(), ClientVersionLetter() );
}

void cSocket::ClientVersionMinor( UI08 value )
{
	ClientVersion( ClientVersionMajor(), value, ClientVersionSub(), ClientVersionLetter() );
}

void cSocket::ClientVersionSub( UI08 value )
{
	ClientVersion( ClientVersionMajor(), ClientVersionMinor(), value, ClientVersionLetter() );
}

void cSocket::ClientVersionLetter( UI08 value )
{
	ClientVersion( ClientVersionMajor(), ClientVersionMinor(), ClientVersionSub(), value );
}

void cSocket::Range( UI08 value )
{
	range = value;
}

UI08 cSocket::Range( void ) const
{
	return range;
}

void cSocket::sysmessage( const char *txt, ...) // System message (In lower left corner)
{
	va_list argptr;
	if( txt == NULL )
		return;

	CChar *mChar = CurrcharObj();
	if( mChar == NULL )
		return;

	char msg[512];
	va_start( argptr, txt );
	vsprintf( msg, txt, argptr );
	va_end( argptr );
	
	CSpeechEntry *toAdd = SpeechSys->Add();
	toAdd->Speech( msg );
	toAdd->Font( FNT_NORMAL );
	toAdd->Speaker( INVALIDSERIAL );
	toAdd->SpokenTo( mChar->GetSerial() );
	toAdd->Colour( 0x0040 );
	toAdd->Type( SYSTEM );
	toAdd->At( cwmWorldState->GetUICurrentTime() );
	toAdd->TargType( SPTRG_INDIVIDUAL );
}

void cSocket::sysmessage( SI32 dictEntry, ... )	// System message (based on dictionary entry)
{
	CChar *mChar = CurrcharObj();
	if( mChar == NULL )
		return;

	va_list argptr;
	std::string txt = Dictionary->GetEntry( dictEntry, Language() );
	if( txt.empty() )
		return;
	char msg[512];
	va_start( argptr, dictEntry );
	vsprintf( msg, txt.c_str(), argptr );
	va_end( argptr );

	CSpeechEntry *toAdd = SpeechSys->Add();
	toAdd->Speech( msg );
	toAdd->Font( FNT_NORMAL );
	toAdd->Speaker( INVALIDSERIAL );
	toAdd->SpokenTo( mChar->GetSerial() );
	toAdd->Colour( 0x0040 );
	toAdd->Type( SYSTEM );
	toAdd->At( cwmWorldState->GetUICurrentTime() );
	toAdd->TargType( SPTRG_INDIVIDUAL );
}

//o---------------------------------------------------------------------------o
//|		Function    :	void objMessage( SI32 dictEntry, cBaseObject *getObj, R32 secsFromNow, UI16 Colour, ... )
//|		Date        :	2/11/2003
//|		Programmer  :	Zane
//o---------------------------------------------------------------------------o
//|		Purpose     :	Shows information on items when clicked or guild info (if any) for players
//o---------------------------------------------------------------------------o
void cSocket::objMessage( SI32 dictEntry, cBaseObject *getObj, R32 secsFromNow, UI16 Colour, ... )
{
	if( getObj == NULL )
		return;

	std::string txt = Dictionary->GetEntry( dictEntry, Language() );
	if( txt.empty() )
		return;

	va_list argptr;

	char msg[512];
	va_start( argptr, Colour );
	vsprintf( msg, txt.c_str(), argptr );
	va_end( argptr );
	objMessage( msg, getObj, secsFromNow, Colour );
}

//o---------------------------------------------------------------------------o
//|		Function    :	void objMessage( const char *txt, cBaseObject *getObj, R32 secsFromNow, UI16 Colour )
//|		Date        :	2/11/2003
//|		Programmer  :	Zane
//o---------------------------------------------------------------------------o
//|		Purpose     :	Shows information on items when clicked or guild info (if any) for players
//o---------------------------------------------------------------------------o
void cSocket::objMessage( const char *txt, cBaseObject *getObj, R32 secsFromNow, UI16 Colour )
{
	if( txt == NULL )
		return;
	CChar *mChar		= CurrcharObj();
	CSpeechEntry *toAdd = SpeechSys->Add();
	toAdd->Speech( txt );
	toAdd->Font( FNT_NORMAL );
	toAdd->Speaker( getObj->GetSerial() );
	toAdd->SpokenTo( mChar->GetSerial() );
	toAdd->Type( SYSTEM );
	toAdd->At( BuildTimeValue( secsFromNow ) );
	toAdd->TargType( SPTRG_INDIVIDUAL );

	if( getObj->GetObjType() == OT_ITEM )
	{
		CItem *getItem = static_cast< CItem *>(getObj);
		if( getItem->isCorpse() )
		{
			UI16 targColour;
			CChar *targChar = (CChar *)getItem->GetOwnerObj();
			if( targChar != NULL )
				targColour = GetFlagColour( mChar, targChar );
			else
			{
				switch( getItem->GetMoreZ() )
				{
					case 0x01:	targColour = 0x0026;	break;	//red
					case 0x02:	targColour = 0x03B2;	break;	// gray
					case 0x08:	targColour = 0x0049;	break;	// green
					case 0x10:	targColour = 0x0030;	break;	// orange
					default:
					case 0x04:	targColour = 0x005A;	break;	// blue
				}
			}
			toAdd->Colour( targColour );
		}
	}
	else
		toAdd->Colour( Colour );
	if( toAdd->Colour() == 0 )
		toAdd->Colour( 0x03B2 );
}

void cSocket::ShowCharName( CChar *i, char b ) // Singleclick text for a character
{
	UI08 a1 = i->GetSerial( 1 );
	UI08 a2 = i->GetSerial( 2 );
	UI08 a3 = i->GetSerial( 3 );
	UI08 a4 = i->GetSerial( 4 );
    if( i->GetSquelched() )
		objMessage( " [squelched]", i );
	CChar *mChar = CurrcharObj();
	char temp[1024];
	if( mChar->GetSingClickSer() || b )
		sprintf( temp, "%s [%x %x %x %x]", i->GetName().c_str(), a1, a2, a3, a4 );
	else
	{
		if( !i->IsNpc() )
		{
			if( mChar->IsGM() )
			{
				sprintf( temp, "[%x %x %x %x]", a1, a2, a3, a4 );
				objMessage( temp, mChar );
			}
			if( !isOnline( i ) )
				sprintf( temp, "%s (OFF)", i->GetName().c_str() );
			else 
				strcpy( temp, i->GetName().c_str() );
		}
		else
		{
			// :Terrin: Just to replace '_'s? How about this (faster/cleaner):
			strcpy( temp, i->GetName().c_str() );
			for( char *cp = strchr( temp, '_' ); cp; cp = strchr( cp, '_' ) )
			{
				*cp = ' ';
			}
		}
	}
	textFlags( i, temp );
}

void cSocket::textFlags( CChar *i, const char *name )
{
	char name2[150];
	char temp2[150];
	CChar *mChar = CurrcharObj();
	strcpy( name2, name );
	if( !i->IsNpc() && !( i->GetCommandLevel() >= CNS_CMDLEVEL ) && (i->GetFame() >= 10000 ) ) // Morollan, only normal players have titles now
	{
		if( i->GetID( 2 ) == 0x91 )
			sprintf( name2, "Lady %s", name ); //Morrolan, added Lord/Lady to title overhead
		else if( i->GetID( 1 ) == 0x90 )
			sprintf( name2, "Lord %s", name );
	}
	
	if( i->IsInvulnerable() )
		strcat( name2, " (invulnerable)" );
	if( i->IsFrozen() )
		strcat( name2, " (frozen) ");
	if( i->IsGuarded() )
		strcat( name2, " (guarded)" );
	if( i->IsTamed() && i->IsNpc() && i->GetOwnerObj() != NULL && i->GetNPCAiType() != aiPLAYERVENDOR )
		strcat( name2, " (tame) ");
	if( i->GetTownPriv() == 2 )
	{
		sprintf( temp2, "Mayor %s", name2 );
		strcpy( name2, temp2 );
	}
	if( !i->IsNpc() && i->GetRace() != 0 && i->GetRace() != 65535 )	// need to check for placeholder race (Abaddon)
		sprintf( name2, "%s (%s)", name2, Races->Name( i->GetRace() ) );
	if( i->GetKills() > cwmWorldState->ServerData()->RepMaxKills() )
		strcat( name2, " [Murderer]" );
	if( i->GetGuildNumber() != -1 && !i->IsIncognito() )
		GuildSys->DisplayTitle( this, i );
	
	CSpeechEntry *toAdd = SpeechSys->Add();
	toAdd->Speech( name2 );
	toAdd->Font( FNT_NORMAL );
	toAdd->Speaker( i->GetSerial() );
	toAdd->SpokenTo( mChar->GetSerial() );
	toAdd->Colour( GetFlagColour( mChar, i ) );
	toAdd->Type( SYSTEM );
	toAdd->At( cwmWorldState->GetUICurrentTime() );
	toAdd->TargType( SPTRG_INDIVIDUAL );
}

UI16 cSocket::GetFlagColour( CChar *src, CChar *trg )
{
	GUILDRELATION gRel;
	if( trg->IsIncognito() )
		gRel = GR_UNKNOWN;
	else
		gRel = GuildSys->Compare( src, trg );
	SI08 race = Races->Compare( src, trg );

	if( !trg->IsNpc() && trg->GetCommandLevel() > 0 && ( trg->GetID() == 0x03DB ) )
		return Commands->GetColourByLevel( trg->GetCommandLevel() );//get their command level color if they look like a gm ONLY
	else if( trg->GetKills() > cwmWorldState->ServerData()->RepMaxKills() || trg->IsMurderer() )
		return 0x0026;//Red
	else if( gRel == GR_SAME || gRel == GR_ALLY || race > 0 )
		return 0x0043;//Same guild (Green)
	else if( gRel == GR_WAR || race < 0 )
		return 0x0030;//enemy guild (orange)
	else if( trg->IsCriminal() )
		return 0x03B2;//grey
	else 
		return 0x0058;//blue

	return 0;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void target()
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Send targeting cursor to client
//o---------------------------------------------------------------------------o
void cSocket::target( UI08 targType, UI08 targID, const char *txt )
{
	target( calcserial( 0, 1, targType, targID ), txt );
}
void cSocket::target( UI08 targType, UI08 targID, SI32 dictEntry )
{
	target( calcserial( 0, 1, targType, targID ), Dictionary->GetEntry( dictEntry, Language() ).c_str() );
}
void cSocket::target( SERIAL ser, const char *txt )
{
	CPTargetCursor toSend;
	toSend.ID( ser );
	toSend.Type( 1 );
	TargetOK( true );
	sysmessage( txt );
	Send( &toSend );
}

void cSocket::mtarget( SERIAL ser, UI16 itemID, const char *txt )
{
	CPMultiPlacementView toSend( ser );
	toSend.MultiModel( itemID );
	toSend.RequestType( 1 );

	sysmessage( txt );
	TargetOK( true );
	Send( &toSend );
}

void cSocket::mtarget( UI08 a1, UI08 a2, UI08 a3, UI08 a4, UI08 b1, UI08 b2, const char *txt )
{
	mtarget( calcserial( a1, a2, a3, a4 ), (UI16)((b1<<8) + b2), txt );
}

bool cSocket::ReceivedVersion( void ) const
{
	return receivedVersion;
}
void cSocket::ReceivedVersion( bool value )
{
	receivedVersion = value;
}

UI32 cSocket::BytesSent( void ) const
{
	return bytesSent;
}

UI32 cSocket::BytesReceived( void ) const
{
	return bytesReceived;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void statwindow( CChar *i )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Opens the status window
//o---------------------------------------------------------------------------o
void cSocket::statwindow( CChar *i )
{
	if( i == NULL )
		return;

	CPStatWindow toSend( (*i), (*this) );
	
	CChar *mChar = CurrcharObj();
	//Zippy 9/17/01 : fixed bug of your name on your own stat window
	toSend.NameChange( mChar != i && ( mChar->IsGM() || i->GetOwnerObj() == mChar ) );
	toSend.Gold( calcGold( i ) );
	toSend.AC( Combat->calcDef( i, 0, false ) );
	toSend.Weight( (UI16)(i->GetWeight() / 100) );
	Send( &toSend );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void updateskill( UI08 skillnum )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Update a certain skill
//o---------------------------------------------------------------------------o
void cSocket::updateskill( UI08 skillnum )
{
	CChar *mChar = CurrcharObj();
	CPUpdIndSkill toSend( (*mChar), (UI08)skillnum );
	Send( &toSend );
}

