//	 cmdtable.h - Crackerjack <crackerjack@crackerjack.net> July 24/99
//	This code is an attempt to clean up the messy "if/then/else" routines
//	currently in use for GM commands, as well as adding more functionality
//	and more potential for functionality.
//	
//	Current features:
//	- Actual table of commands to execute, what perms are required, dialog
//	  messages for target commands, etc handled by a central system
//	
//

#ifndef __CMDTABLE_H
#define __CMDTABLE_H

// Types of commands
enum commandTypes
{
	CMD_ITEMMENU = 1,	// Open item menu specified in cmd_extra
	CMD_TARGET,			// Call target struct specified in cmd_extra
	CMD_FUNC,			// Call function specified in cmd_extra must be of type GMFUNC
	CMD_SOCKFUNC,		// Call function specified in cmd_extra with a socket
	CMD_TARGETX,		// target with addx[] argument
	CMD_TARGETXY,		// target with addx & y [] arguments
	CMD_TARGETXYZ,		// target with addx & y & z [] arguments
	CMD_TARGETID1,		// target with 1 addid #
	CMD_TARGETID2,		// target with 2 addid #s
	CMD_TARGETID3,		// target with 3 addid #s
	CMD_TARGETID4,		// target with 4 addid #s
	CMD_TARGETTMP,		// target with tempint
};

struct cmdtable_entry
{
	const char *	cmd_name;
	UI08			cmdLevelReq;
	UI08			cmdType;
	void			(*cmd_extra) ();	// executable function
};
struct cmdtable_mapentry
{
	UI08	cmdLevelReq;
	UI08	cmdType;
	void	(*cmd_extra) ();	// executable function
	cmdtable_mapentry() : cmdLevelReq( 0 ), cmdType( CMD_SOCKFUNC ), cmd_extra( NULL )
	{
	}
	cmdtable_mapentry( UI08 cLR, UI08 cT, void (*ce)() ) : cmdLevelReq( cLR ), cmdType( cT ), cmd_extra( ce )
	{
	}
};

struct targtable_entry
{
    const char		*name;
	UI08			cmdLevelReq;
	UI08			cmdType;
    TargetIDs		targID;
    SI32			dictEntry;
};
struct targtable_mapentry
{
	UI08			cmdLevelReq;
	UI08			cmdType;
    TargetIDs		targID;
    SI32			dictEntry;
	targtable_mapentry() : cmdLevelReq( 0 ), cmdType( CMD_TARGET ), targID( TARGET_NOFUNC ), dictEntry( 0 )
	{
	}
	targtable_mapentry( UI08 cLR, UI08 cT, TargetIDs tID, SI32 dE ) : cmdLevelReq( cLR ), cmdType( cT ), targID( tID ), dictEntry( dE )
	{
	}
};

typedef std::map< std::string, cmdtable_mapentry >	CmdTableDataType;
typedef CmdTableDataType::iterator			CmdTableIterator;
extern	CmdTableDataType					cmd_table;

#define CMD_EXEC		void (*) ( void )
#define CMD_SOCKEXEC	void (*) ( cSocket * )
#define CMD_DEFINE		void (*)()

typedef std::map< std::string, targtable_mapentry >	TargetTableDataType;
typedef TargetTableDataType::iterator			TargetTableIterator;
extern	TargetTableDataType					targ_table;

// Defined commands that are just being mapped to internal functions
#define command_time telltime

typedef void (CommandFunc)( void );
typedef void (CommandSockFunc)( cSocket *s );
// All defined commands
// A
CommandSockFunc command_add;
CommandSockFunc command_addx;
CommandSockFunc command_allmoveon;
CommandSockFunc command_allmoveoff;
CommandSockFunc command_additem;
CommandSockFunc command_areaCommand;
CommandSockFunc command_action;
CommandFunc		command_announceon;
CommandFunc		command_announceoff;
CommandSockFunc command_addnpc;
CommandSockFunc command_addaccount;
// B
CommandSockFunc command_brightlight;
// C
CommandSockFunc command_cleanup;
CommandSockFunc command_clear;
CommandSockFunc command_command;
CommandSockFunc command_cq;
CommandSockFunc command_cnext;
CommandSockFunc command_cclear;
CommandSockFunc command_cachestats;
CommandSockFunc command_cy;
// D
CommandSockFunc command_dye;
CommandSockFunc command_dupe;
CommandFunc		command_disconnect;
CommandSockFunc command_dungeonlight;
CommandSockFunc command_darklight;
CommandSockFunc command_decay;
CommandSockFunc command_delid;
// E
// F
CommandSockFunc command_fix;
CommandSockFunc command_forcewho;
// G
CommandSockFunc command_gms;
CommandSockFunc command_gmtransfer;
CommandSockFunc command_gotocur;
CommandSockFunc command_gpost;
CommandSockFunc command_goplace;
CommandSockFunc command_gochar;
CommandSockFunc command_go;
CommandFunc		command_gcollect;
CommandSockFunc command_gmmenu;
CommandSockFunc command_gumpmenu;
CommandSockFunc command_gmopen;
CommandSockFunc command_gumpopen;
CommandFunc		command_guardson;
CommandFunc		command_guardsoff;
CommandSockFunc command_getlight;
CommandSockFunc command_gy;
// H
CommandSockFunc command_hidehs;
CommandSockFunc command_howto;
// I
CommandSockFunc command_iwipe;
CommandSockFunc command_itemmenu;
CommandSockFunc command_invul;
// J
// K
CommandSockFunc command_killall;
// L
CommandSockFunc command_lpost;
CommandFunc		command_loaddefaults;
// M
CommandSockFunc command_midi;
CommandFunc		command_minecheck;
CommandSockFunc command_make;
// N
CommandSockFunc command_next;
CommandSockFunc command_noinvul;
CommandSockFunc command_nodecay;
CommandSockFunc command_npcrect;
CommandSockFunc command_npccircle;
CommandSockFunc command_npcwander;
CommandSockFunc command_nacct;
// O
// P
CommandSockFunc command_post;
CommandSockFunc command_poly;
CommandSockFunc command_pdump;
// Q
CommandSockFunc command_q;
// R
CommandSockFunc command_regspawnall;
CommandSockFunc command_resend;
CommandSockFunc command_rpost;
CommandSockFunc command_rename;
CommandSockFunc command_restock;
CommandSockFunc command_restockall;
CommandFunc		command_respawn;
CommandSockFunc command_regspawnmax;
CommandSockFunc command_regspawn;
CommandSockFunc command_reloadserver;
CommandSockFunc command_reloadaccounts;
CommandSockFunc command_readspawnregions;
CommandSockFunc command_reportbug;
CommandSockFunc command_readini;
CommandSockFunc command_reloaddefs;
CommandSockFunc command_rename2;
// S
CommandSockFunc command_status;
CommandSockFunc command_showids;
CommandSockFunc command_skin;
CommandFunc		command_save;
CommandSockFunc command_showtime;
CommandSockFunc command_setpriv;
CommandFunc		command_settime;
CommandFunc		command_shutdown;
CommandSockFunc command_setshoprestockrate;
CommandSockFunc command_showhs;
CommandSockFunc command_set;
CommandSockFunc command_squelch;
CommandSockFunc command_spawnkill;
CommandSockFunc command_setrace;
CommandSockFunc command_sfx;
CommandSockFunc command_secondsperuominute;
CommandSockFunc command_sgy;
CommandSockFunc command_setspawnobj;
CommandSockFunc command_setspawnobjlist;
// T
CommandSockFunc command_teleport;
CommandSockFunc command_tile;
CommandSockFunc command_time;
CommandSockFunc command_title;
CommandSockFunc command_tell;
CommandSockFunc command_tilew;
CommandSockFunc command_temp;
// U
// V
CommandSockFunc command_validcmd;
// W
CommandSockFunc command_who;
CommandSockFunc command_wipenpcs;
CommandSockFunc command_where;
CommandSockFunc command_wipe;
CommandSockFunc command_wholist;
CommandSockFunc command_wf;
// X
CommandSockFunc command_xgoplace;
CommandSockFunc command_xtele;
CommandSockFunc command_xgate;
// Y
// Z
CommandSockFunc command_zerokills;

void HandleHowTo( cSocket *sock, int cmdNumber );

#endif // __CMDTABLE_H
