#include <algorithm>
#include "uox3.h"
#include "cGuild.h"
#include "combat.h"
#include "townregion.h"
#include "cRaces.h"
#include "skills.h"
#include "cMagic.h"
#include "cEffects.h"
#include "packets.h"
#include "classes.h"
#include "regions.h"
#include "magic.h"

#undef DBGFILE
#define DBGFILE "combat.cpp"
#define SWINGAT (UI32)1.75 * 1000

const UI08 BODYPERCENT = 0;
const UI08 ARMSPERCENT = 1;
const UI08 HEADPERCENT = 2;
const UI08 LEGSPERCENT = 3;
const UI08 NECKPERCENT = 4;
const UI08 OTHERPERCENT = 5;
const UI08 TOTALTARGETSPOTS = 6;

const UI08 LOCPERCENTAGES[TOTALTARGETSPOTS] = { 44, 14, 14, 14, 7, 7 };

//o---------------------------------------------------------------------------o
//|	Function	-	UI08 cCombat::getBowType( CChar *i )
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find if character is holding a Bow or XBow
//o---------------------------------------------------------------------------o
UI08 cCombat::getBowType( CChar *i )
{
	if( i == NULL )
		return 0;

	CItem *getBow = i->GetItemAtLayer( 1 );
	if( getBow == NULL )
		getBow = i->GetItemAtLayer( 2 );

	if( getBow == NULL )
		return 0;

	switch( getWeaponType( getBow ) )
	{
		case BOWS:
			return BOWS;
		case XBOWS:
			return XBOWS;
		default:
			return 0;
	}
}
/*
//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::ItemCastSpell( cSocket *mSock, CChar *c, CItem *i )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle items that cast spells during combat
//o---------------------------------------------------------------------------o
void cCombat::ItemCastSpell( cSocket *mSock, CChar *c, CItem *i )
{
	if( c == NULL || i == NULL || mSock == NULL ) 
		return;

	CChar *ourChar = mSock->CurrcharObj();
	UI16 spellnum = static_cast<UI16>(((i->GetMoreX()*8)-8) + i->GetMoreY());

	if( i->GetType() != 15 || i->GetMoreZ() <= 0 ) 
		return;
	
	switch( spellnum )
	{
		case 1:		
		case 3:		
		case 5:		
		case 8:		
		case 18:	
		case 22:	
		case 27:	
		case 30:	
		case 37:	
		case 38:	
		case 42:	
		case 43:	
		case 51:
			CastSpell( ourChar, c, spellnum );
			break;
		default:
			Effects->staticeffect( ourChar, 0x3735, 0, 30 );
			Effects->PlaySound( ourChar, 0x005C );
			break;
	}

	i->SetMoreZ( i->GetMoreZ() - 1 );
	if( i->GetMoreZ() == 0 )
		mSock->sysmessage( (SI32)0 );
}
*/
//o--------------------------------------------------------------------------
//|	Function		-	InvalidateAttacker( CChar *attack )
//|	Date			-	3rd July, 2001
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Resets the attacker attack so that it cancels attack 
//|						setup.
//o--------------------------------------------------------------------------
void cCombat::InvalidateAttacker( CChar *attack )
{
	if( attack == NULL )
		return;

	if( attack->IsNpc() && attack->GetNPCAiType() == aiGUARD )
	{
		attack->SetTimer( tNPC_SUMMONTIME, BuildTimeValue( 20 ) );
		attack->SetNpcWander( 2 );
		attack->SetTimer( tNPC_MOVETIME, BuildTimeValue(static_cast<R32>( cwmWorldState->ServerData()->NPCSpeed() )) );
		attack->talkAll( 281, false );
	}

	attack->SetTarg( INVALIDSERIAL );
	CChar *attAttacker = attack->GetAttacker();
	if( attAttacker != NULL )
	{
		attAttacker->SetAttackFirst( false );
		attAttacker->SetAttacker( NULL );
	}
	attack->SetAttacker( NULL );
	attack->SetAttackFirst( false );
	if( attack->IsAtWar() && attack->IsNpc() && !attack->IsDead() && attack->GetNPCAiType() != aiPLAYERVENDOR )
		npcToggleCombat( attack );
}

//o--------------------------------------------------------------------------
//|	Function		-	void AdjustRaceDamage( CChar *defend, CItem *weapon, SI16 &bDamage )
//|	Date			-	3rd July, 2001
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adjusts the damage dealt to defend by weapon based on
//|						race and weather weaknesses
//o--------------------------------------------------------------------------
void cCombat::AdjustRaceDamage( CChar *defend, CItem *weapon, SI16 &bDamage )
{
	if( defend == NULL || weapon == NULL )
		return;

	if( weapon->GetRace() == defend->GetRace() ) 
		bDamage *= 2;
	CRace *rPtr = Races->Race( defend->GetRace() );
	if( rPtr != NULL )
	{
		if( weapon->GetWeatherDamage( LIGHT ) && rPtr->AffectedByLight() )
			bDamage *= 2;
		if( weapon->GetWeatherDamage( RAIN ) && rPtr->AffectedByRain() )
			bDamage *= 2;
		if( weapon->GetWeatherDamage( COLD ) && rPtr->AffectedByCold() )
			bDamage *= 2;
		if( weapon->GetWeatherDamage( HEAT ) && rPtr->AffectedByHeat() )
			bDamage *= 2;
		if( weapon->GetWeatherDamage( LIGHTNING ) && rPtr->AffectedByLightning() )
			bDamage *= 2;
		if( weapon->GetWeatherDamage( SNOW ) && rPtr->AffectedBySnow() )
			bDamage *= 2;
	}
}

//o--------------------------------------------------------------------------
//|	Function		-	SI08 DoHitMessage( CChar *defend, CChar *attack, cSocket *mSock, SI16 damage )
//|	Date			-	3rd July, 2001
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Prints out the hit message (if enabled) and calculates
//|						where on the body the person was hit and returns that
//o--------------------------------------------------------------------------
SI08 cCombat::DoHitMessage( CChar *defend, CChar *attack, cSocket *mSock, SI16 damage )
{
	if( defend == NULL || attack == NULL || defend == attack )
		return 0;

	SI08 hitPos = RandomNum( 0, 99 ); // Determine area of Body Hit
	for( UI08 t = BODYPERCENT; t < TOTALTARGETSPOTS; ++t )
	{
		hitPos -= LOCPERCENTAGES[t];
		if( hitPos < 0 ) 
		{
			hitPos = t + 1;
			break;
		}
	}
	if( cwmWorldState->ServerData()->CombatDisplayHitMessage() )
	{
		if( !defend->IsNpc() && mSock != NULL && damage > 1 )
		{
			UI08 randHit = RandomNum( 0, 2 );
			switch( hitPos )
			{
				case 1:	// Body
					switch ( randHit )
					{
						case 1:
							if( damage < 10  ) 
								mSock->sysmessage( 284, attack->GetName().c_str() );
							else
								mSock->sysmessage( 285, attack->GetName().c_str() );
							break;
						case 2:
							if( damage < 10 ) 
								mSock->sysmessage( 286, attack->GetName().c_str() );
							else
								mSock->sysmessage( 287, attack->GetName().c_str() );
							break;
						default:
							if( damage < 10 ) 
								mSock->sysmessage( 288, attack->GetName().c_str() );
							else
								mSock->sysmessage( 289, attack->GetName().c_str() );
							break;
						}
					break;
				case 2:	// Arms
					switch( randHit )
					{
						case 1:		mSock->sysmessage( 290, attack->GetName().c_str() );		break;
						case 2:		mSock->sysmessage( 291, attack->GetName().c_str() );		break;
						default:	mSock->sysmessage( 292, attack->GetName().c_str() );		break;
					}
					break;
				case 3:	// Head
					switch( randHit )
					{
						case 1:
							if( damage <  10 ) 
								mSock->sysmessage( 293, attack->GetName().c_str() );
							else
								mSock->sysmessage( 294, attack->GetName().c_str() );
							break;
						case 2:
							if( damage <  10 ) 
								mSock->sysmessage( 295, attack->GetName().c_str() );
							else 
								mSock->sysmessage( 296, attack->GetName().c_str() );
							break;
						default:
							if( damage <  10 ) 
								mSock->sysmessage( 297, attack->GetName().c_str() );
							else
								mSock->sysmessage( 298, attack->GetName().c_str() );
							break;
					}
					break;
				case 4:	// Legs
					switch( randHit )
					{
						case 1:		mSock->sysmessage( 299, attack->GetName().c_str() );		break;
						case 2:		mSock->sysmessage( 300, attack->GetName().c_str() );		break;
						default:	mSock->sysmessage( 301, attack->GetName().c_str() );		break;
					}
					break;
				case 5:	// Neck
					mSock->sysmessage( 302, attack->GetName().c_str() );
					break;
				case 6:	// Hands
					switch( randHit )
					{
						case 1:		mSock->sysmessage( 303, attack->GetName().c_str() );		break;
						case 2:		mSock->sysmessage( 304, attack->GetName().c_str() );		break;
						default:	mSock->sysmessage( 305, attack->GetName().c_str() );		break;
					}
					break;
			}
		}
	}
	return hitPos;
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::CombatHit( CChar *attack, CChar *defend, SI32 weaponType )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle combat hit (or miss), and calculate damage
//|					PARAM WARNING: weaponType is an unreferenced parameters
//|					Weapontype was meant for race and other weapon weaknesses
//o---------------------------------------------------------------------------o
void cCombat::CombatHit( CChar *attack, CChar *defend, SI32 weaponType )
{
	if( defend == NULL || attack == NULL || defend == attack ) 
		return;

	attack->SetSwingTarg( INVALIDSERIAL );
	if( defend->IsDead() || defend->IsInvulnerable() || defend->GetNPCAiType() == aiPLAYERVENDOR )
	{
		InvalidateAttacker( attack );
		return;
	}

	cSocket *aSock = calcSocketObjFromChar( attack );
	cSocket *dSock = calcSocketObjFromChar( defend );

 	UI08 getFightSkill = getCombatSkill( attack );

	bool los = false;
	if( getDist( attack, defend ) < 2 )
		los = true;
	else if( getFightSkill == ARCHERY )
		los = LineOfSight( aSock, attack, defend->GetX(), defend->GetY(), defend->GetZ(), WALLS_CHIMNEYS + DOORS + FLOORS_FLAT_ROOFING );

	if( !los )
		return;

	UI08 bowType = getBowType( attack );
	UI16 attSkill = attack->GetSkill( getFightSkill );

	bool skillPassed = false;
	if( Skills->CheckSkill( attack, getFightSkill, 0, 1000 ) )
	{
		UI16 getAttackSkill = ( attSkill + attack->GetSkill( TACTICS ) ) / 2;
		UI16 getDefSkill = defend->GetSkill( TACTICS );
		skillPassed = ( !RandomNum( 0, 5 ) || RandomNum( static_cast< UI16 >(getAttackSkill / 1.25 ), getAttackSkill ) >= RandomNum( static_cast< UI16 >( getDefSkill / 1.25), getDefSkill ) );
	}
	if( !skillPassed )
	{
		if( getFightSkill == ARCHERY && !RandomNum( 0, 2 ) )
		{
			CItem *c = Items->SpawnItem( NULL, attack, 1, "#", true, 0x0E75, 0, false, false );
			if( c != NULL )
			{
				if( bowType == BOWS )
					c->SetID( 0x0F3F );
				else 
					c->SetID( 0x1BFB );

				c->SetLocation( defend );
			}
		}
		doMissedSoundEffect( attack );
		if( defend->GetTarg() == INVALIDSERIAL )
		{//if the defender is swung at, and they don't have a target already, set this as their target
			StartAttack( defend, attack );
			attack->SetTimer( tCHAR_TIMEOUT, cwmWorldState->GetUICurrentTime() + ((( 100 - UOX_MIN( attack->GetDexterity(), static_cast<SI16>(100) ) ) * 1000 ) /25 ) + ( 1 * 1000 ) );
		}
	} 
	else 
	{
		Skills->CheckSkill( attack, TACTICS, 0, 1000 );

		if( defend->GetID() == 0x0191 )
			Effects->PlaySound( defend, 0x014B );
		else if( defend->GetID() == 0x0190 ) 
			Effects->PlaySound( defend, 0x0156 );
		else
			Effects->playMonsterSound( defend, defend->GetID(), SND_DEFEND );

		if( attack->GetPoisonStrength() && defend->GetPoisonStrength() < attack->GetPoisonStrength() )
		{
			if( ( getFightSkill == FENCING || getFightSkill == SWORDSMANSHIP ) && !RandomNum( 0, 2 ) )
			{
				defend->SetPoisoned( attack->GetPoisonStrength() );
				defend->SetTimer( tCHAR_POISONTIME, BuildTimeValue(static_cast<R32> (40 / defend->GetPoisoned() )) ); // a lev.1 poison takes effect after 40 secs, a deadly pois.(lev.4) takes 40/4 secs - AntiChrist
				defend->SetTimer( tCHAR_POISONWEAROFF, defend->GetTimer( tCHAR_POISONTIME ) + ( 1000 * cwmWorldState->ServerData()->SystemTimer( POISON ) ) ); //wear off starts after poison takes effect - AntiChrist
				if( dSock != NULL )
				{
					defend->SendToSocket( dSock, true, defend );
					dSock->sysmessage( 282 );
				}
			}
		}			

		if( getFightSkill != WRESTLING )
			ItemSpell( attack, defend );

		CItem *weapon = getWeapon( attack );
		SI16 damage = calcDamage( attack, defend, dSock, weapon, getFightSkill );

//		if( !attack->IsNpc() && weapon != NULL )
//			ItemCastSpell( aSock, defend, weapon );

		if( damage > 0 )
		{
			// Interrupt Spells
			if( damage > 1 && !defend->IsNpc() )
			{
				if( defend->GetCasting() && dSock->CurrentSpellType() == 0 )
				{
					defend->StopSpell();
					defend->SetFrozen( false );
					dSock->sysmessage( 306 );
				}
			}
			// Reactive Armor
			if( defend->GetReactiveArmour() )
			{
				int damage1 = (int)( damage * ( defend->GetSkill( MAGERY ) / 2000.0 ) );
				defend->IncHP( -( damage - damage1 ) );
				if( defend->IsNpc() ) 
					damage1 *= cwmWorldState->ServerData()->CombatNPCDamageRate();
				attack->IncHP( -damage1 );
				Effects->staticeffect( defend, 0x374A, 0, 15 );
			}
			else 
				defend->IncHP( -damage );
			// Splitting NPC's
			if( defend->IsNpc() )			// Splitting NPC's
				doSplit( defend );
		}

		if( attack->isHuman() )
			doSoundEffect( attack, weapon );

		if( defend->GetHP() < 0 ) 
			defend->SetHP( 0 );
		updateStats( defend, 0 );
		if( defend->GetID() >= 0x0190 )
		{
			if( !defend->IsOnHorse() ) 
				npcAction( defend, 0x14 );
		}
	}
}

SI16 cCombat::calcDamage( CChar *cAttack, CChar *cDefend, cSocket *dSock, CItem *weapon, UI08 getFightSkill )
{
	SI16 BaseDamage = calcAtt( cAttack );
	if( BaseDamage == -1 )  // No damage if weapon breaks
		return 0;
	// Race Dmg Modification: Bonus percentage.
	SI32 RaceDamage = Races->DamageFromSkill( getFightSkill, cAttack->GetRace() );
	if( RaceDamage != 0 )
		BaseDamage += (SI16)( (R32)BaseDamage * ( (R32)RaceDamage / 1000 ) );

	// Check to see if weapons affect defender's race.
	if( getFightSkill != WRESTLING )
		AdjustRaceDamage( cDefend, weapon, BaseDamage );

	R32 multiplier;
	SI16 damage = 0;

	// Strength Damage Bonus, +20% Damage
	multiplier = static_cast<R32>( ( ( UOX_MIN( cAttack->GetStrength(), static_cast<SI16>(200) ) * 20 ) / 100 ) / 100 ) + 1;
	damage = (SI16)(BaseDamage * multiplier);

	// Tactics Damage Bonus, % = ( Tactics + 50 )
	multiplier = static_cast<R32>( ( cAttack->GetSkill( TACTICS ) + 500 ) / 1000 );
	damage += (SI16)(BaseDamage * multiplier);

	if( cDefend->IsNpc() ) // Anatomy PvM damage Bonus, % = ( Anat / 5 )
		multiplier = static_cast<R32>( ( cAttack->GetSkill( ANATOMY ) / 5 ) / 1000 );
	else // Anatomy PvP damage Bonus, % = ( Anat / 2.5 )
		multiplier = static_cast<R32>( ( cAttack->GetSkill( ANATOMY ) / 2.5 ) / 1000 );
	damage += (SI16)(BaseDamage * multiplier);

	// Lumberjacking Bonus ( 30% Damage at GM Skill )
	if( cAttack->GetSkill( LUMBERJACKING ) >= 1000 )
		damage += (SI16)(BaseDamage * .3);

  	// Defender Tactics Damage Modifier, -20% Damage
	multiplier = static_cast<R32>(1.0 - ( ( ( cDefend->GetSkill( TACTICS ) * 20 ) / 1000 ) / 100 ));
	damage = (SI16)(damage * multiplier);

	if( cDefend->GetTarg() == INVALIDSERIAL )
	{//if the defender is swung at, and they don't have a target already, set this as their target
		StartAttack( cDefend, cAttack );
		cAttack->SetTimer( tCHAR_TIMEOUT, cwmWorldState->GetUICurrentTime() + ((( 100 - UOX_MIN( cAttack->GetDexterity(), (static_cast<SI16>(100)) ) ) * 1000 ) /25 ) + ( 1 * 1000 ) );
	}

	if( damage < 1 )
	{
		doMissedSoundEffect( cAttack );
		return 0;
	}

	UI16 attSkill = cAttack->GetSkill( getFightSkill );

	// Check Shield Defense
	CItem *shield = getShield( cDefend );
	if( shield != NULL )
	{
		Skills->CheckSkill( cDefend, PARRYING, 0, 1000 );
		// Chance to block with Shield ( % = Skill / 2 ) 
		UI16 defendParry = cDefend->GetSkill( PARRYING );
		if( HalfRandomNum( defendParry ) >= HalfRandomNum( attSkill ) )
		{
			damage -= HalfRandomNum( shield->GetDef() );
			if( !RandomNum( 0, 5 ) ) 
				shield->IncHP( -1 );
			if( shield->GetHP() <= 0 )
			{
				if( dSock != NULL )
					dSock->sysmessage( 283 );
				Items->DeleItem( shield );
			}
		}
	}

	// Armor / Skin Absorbtion
	UI16 getDef;
	SI08 hitMsg = DoHitMessage( cDefend, cAttack, dSock, damage );
	if( cDefend->isHuman() )
	{
		getDef = calcDef( cDefend, hitMsg, true );
		getDef = HalfRandomNum( getDef );
	}
	else if( cDefend->GetDef() > 0 )
		getDef = cDefend->GetDef();
	else
		getDef = 20;

	// Damage based on Attack Skill (Armor defends less against low-skill characters)
	damage -= (SI16)( ( getDef * attSkill ) / 750 );

	// Give them one last chance to do Damage
	if( damage <= 0 ) 
		damage = RandomNum( 0, 4 );

	if( !cDefend->IsNpc() ) 
		damage /= cwmWorldState->ServerData()->CombatNPCDamageRate(); // Rate damage against other players

	return damage;
}

void cCombat::doSplit( CChar *toSplit )
{
	UI08 splitnum;
	if( toSplit->GetSplit() > 0 && toSplit->GetHP() >= 1 )
	{
		if( RandomNum( 0, 100 ) <= toSplit->GetSplitChance() )
		{
			if( toSplit->GetSplit() == 1 ) 
				splitnum = 1;
			else 
				splitnum = RandomNum( static_cast< UI08 >(1), toSplit->GetSplit() );

			for( UI08 splitcount = 0; splitcount < splitnum; ++splitcount )
				Npcs->Split( toSplit );
		}
	}
}
//o---------------------------------------------------------------------------o
//|	Function	-	void cCombat::CombatLoop( CChar *attack )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle combat
//o---------------------------------------------------------------------------o
void cCombat::CombatLoop( CChar *attack )
{
	if( attack == NULL )
	{
		Console.Log( "CombatLoop() - Bad Attacker\n", "combat.log" );
		return;
	}
	if( attack->isFree() || attack->IsDead() || attack->IsInvulnerable() )
		return;

	if( attack->GetSwingTarg() != INVALIDSERIAL )
	{
		if( TimerOk( attack ) )
			CombatHit( attack, &chars[attack->GetSwingTarg()] );
		return;
	}

	if( attack->GetTarg() == INVALIDSERIAL )
		return;

	CChar *defend = &chars[attack->GetTarg()];
	if( defend == NULL )
	{
		if( attack->IsAtWar() )
		{
			npcToggleCombat( attack );
			attack->SetAttacker( NULL );
			attack->SetAttackFirst( false );
		}
		return;
	}

	if( ( defend->isFree() || defend->IsDead() || defend->IsInvulnerable() || defend->GetNPCAiType() == aiPLAYERVENDOR ) ||
		( !attack->IsNpc() && !isOnline( attack ) ) || ( !defend->IsNpc() && !isOnline( defend ) ) )
	{
		InvalidateAttacker( attack );
		return;
	}

	UI16 playerDistance = getDist( attack, defend );
	if( playerDistance > 1 || abs( attack->GetZ() - defend->GetZ() ) <= MAX_Z_STEP )
	{
		if( !LineOfSight( NULL, attack, defend->GetX(), defend->GetY(), defend->GetZ(), WALLS_CHIMNEYS + DOORS + FLOORS_FLAT_ROOFING ) )
			return;

		if( !charInRange( attack, defend ) )
		{
			if( attack->GetNPCAiType() == aiGUARD && region[attack->GetRegion()]->IsGuarded() )
			{
				attack->SetLocation( defend );
				attack->Teleport();
				Effects->PlaySound( attack, 0x01FE );
				Effects->staticeffect( attack, 0x372A, 0x09, 0x06 );
				attack->talkAll( 1616, true );
				AttackTarget( attack, defend );
			}
			else
				InvalidateAttacker( attack );
			return;
		}
	}

	if( TimerOk( attack ) )
		DoAttack( attack, defend, playerDistance );	

	if( attack->IsNpc() )
		DoNpcSpAttack( attack, defend, playerDistance );

	try
	{
		if( defend->GetHP() < 1 )
			Kill( attack, defend );
	}
	catch( ... )
	{
		Console << "EXCEPTION: CombatLoop() - Unable to access defender. Pointer not valid." << myendl;
	}
}

void cCombat::DoAttack( CChar *cAttack, CChar *cDefend, UI16 playerDistance )
{
	UI08 bowType = 0;
	UI08 getFightSkill = getCombatSkill( cAttack );
	bool doHit = false;
	if( getFightSkill == ARCHERY )
	{
		cSocket *aSock = calcSocketObjFromChar( cAttack );
		if( playerDistance < 2 )
			return;
		bowType = getBowType( cAttack );
		if( cAttack->IsOnHorse() && !cwmWorldState->ServerData()->ShootOnAnimalBack() )
		{//Why can't we fire while mounted?!?! That's retarded
			if( aSock != NULL )
				aSock->sysmessage( 308 );
			cAttack->SetWar( false );
			cAttack->SetAttacker( NULL );
			cAttack->SetAttackFirst( false );
			return;
		}
		UI32 getArrowAmt;
		if( bowType == BOWS ) 
			getArrowAmt = GetAmount( cAttack, 0x0F3F );
		else 
			getArrowAmt = GetAmount( cAttack, 0x1BFB );
	
		if( getArrowAmt > 0 || cAttack->IsNpc() ) 
			doHit = true;
		else if( aSock != NULL ) 
			aSock->sysmessage( 309 );
	}
	else if( playerDistance < 2 ) 
		doHit = true;
	if( doHit )
	{
		if( cAttack->IsNpc() )
		{
			if( cAttack->GetNpcWander() != 5 )
			{
				UI08 charDir = getCharDir( cAttack, cDefend->GetX(), cDefend->GetY() );
				if( cAttack->GetDir() != charDir && charDir < 8 )
				{
					cAttack->SetDir( charDir );
					cAttack->Teleport();
				}
			}
		}
		if( abs( cwmWorldState->ServerData()->CombatAttackStamina() ) > 0 && ( !cAttack->IsGM() && !cAttack->IsCounselor() ) )
		{
			cAttack->SetStamina( cAttack->GetStamina() + cwmWorldState->ServerData()->CombatAttackStamina() );
			if( cAttack->GetStamina() > cAttack->GetMaxStam() ) 
				cAttack->SetStamina( cAttack->GetMaxStam() );
			if( cAttack->GetStamina() < 0 ) 
				cAttack->SetStamina( 0 );
    			updateStats( cAttack, 2 );
		}
		UI16 charID = cAttack->GetID();
		if( charID < 0x0190 )
		{
		UI08 aa = RandomNum( 4, 6 ); // some creatures dont have animation #4
			if( creatures[charID].AntiBlink() )
			{
				++aa;
				if( charID == 5 ) // eagles need special treatment
				{
					switch( RandomNum( 0, 2 ) )
					{
						case 0: aa = 0x1;  break;
						case 1: aa = 0x14; break;
						case 2: aa = 0x4;  break;
					}
				}
			}
			npcAction( cAttack, aa ); 
			if( RandomNum( 0, 4 ) ) 
				Effects->playMonsterSound( cAttack, cAttack->GetID(), SND_ATTACK );
		}
		else if( cAttack->IsOnHorse() )
			CombatOnHorse( cAttack );
		else
			CombatOnFoot( cAttack );
	
		/*if( ( playerDistance < 2 || getFightSkill == ARCHERY ) &&  !cAttack->GetNPCAiType() == aiGUARD )
		{
			if( cDefend->GetTarg() == INVALIDSERIAL )
				StartAttack( cDefend, cAttack  );
		}*/

		if( getFightSkill == ARCHERY )
		{
			if( bowType == BOWS )
			{
				DeleteQuantity( cAttack, 0x0F3F, 1 );
				Effects->movingeffect( cAttack, cDefend, 0x0F42, 0x08, 0x00, 0x00 );
			}
			else
			{
				DeleteQuantity( cAttack, 0x1BFB, 1 );
				Effects->movingeffect( cAttack, cDefend, 0x1BFE, 0x08, 0x00, 0x00 );
			}
			CombatHit( cAttack, cDefend );
		}
		else 
			cAttack->SetSwingTarg( calcCharFromSer( cDefend->GetSerial() ) ); 
  	}
	// attack speed should be determined here.
	// attack speed = 15000 / ((DEX+100) * weapon speed)
	R32 getDelay = 0;

	CItem *weapon = getWeapon( cAttack );
	if( weapon != NULL )
	{
	    if( weapon->GetSpeed() == 0 ) 
			weapon->SetSpeed( 35 );
		getDelay = (15000 / ((UOX_MIN( cAttack->GetDexterity(), static_cast<SI16>(100) ) + 100 ) * weapon->GetSpeed())); //Calculate combat delay
	}
	else
	{
		int getOffset;
		if( cAttack->GetSkill( WRESTLING ) < 800 )
			getOffset = (((int)(cAttack->GetSkill( WRESTLING ) / 200 )) * 5) + 30;
		else
			getOffset = 50;
		getDelay = (15000 / ((UOX_MIN( cAttack->GetDexterity(), static_cast<SI16>(100) ) + 100) * getOffset));
	}
	cAttack->SetTimer( tCHAR_TIMEOUT, BuildTimeValue( getDelay ) );
	if( getFightSkill != ARCHERY ) 
		CombatHit( cAttack, cDefend );
}

void cCombat::DoNpcSpAttack( CChar *npcAttack, CChar *cDefend, UI16 playerDistance )
{
	if( npcAttack->GetMana() > 0 && npcAttack->GetTimer( tNPC_SPATIMER ) <= cwmWorldState->GetUICurrentTime() )
	{
		if( !cDefend->IsDead() && playerDistance < cwmWorldState->ServerData()->CombatMaxSpellRange() )
		{
			SI16 spattacks = npcAttack->GetSpAttack();
			if( spattacks <= 0 )
				return;
			if( spattacks > 8 )
				spattacks = 8;

			npcAttack->StopSpell();
			switch( ( RandomNum( static_cast< SI16 >(0), spattacks ) + 1 ) )
			{
				case 1:
					switch( RandomNum( 1, 6 ) ) 
					{
						case 1:		CastSpell( npcAttack, cDefend, 1 );			break;	// Clumsy
						case 2:		CastSpell( npcAttack, cDefend, 3 );			break;	// Feeble Mind
						case 3:		CastSpell( npcAttack, cDefend, 4 );			break;	// Heal (personal...switch target temporarily to self and then back after casting)
						case 4:		CastSpell( npcAttack, cDefend, 5 );			break;	// Magic Arrow
						case 5:		CastSpell( npcAttack, cDefend, 7 );			break;	// Reactive Armor (personal...switch target temporarily to self and then back after casting)
						case 6:			// Check if poisoned and cast cure (switch to self) - Hanse
							if( !CastSpell( npcAttack, cDefend, 11 ) )					// cure
								CastSpell( npcAttack, cDefend, 8 );						// weaken
							break;
					}
					break;
				case 2:			CastSpell( npcAttack, cDefend, 12 );		break;	// Harm
				case 3:
					switch( RandomNum( 1, 2 ) ) 
					{
						case 1:		CastSpell( npcAttack, cDefend, 18 );		break;	// Fireball
						case 2:		CastSpell( npcAttack, cDefend, 20 );		break;	// Poison
					}
					break;
				case 4:
					switch( RandomNum( 1, 4 ) ) 
					{
						case 1:		CastSpell( npcAttack, cDefend, 27 );		break;	// Curse
						case 2:		CastSpell( npcAttack, cDefend, 29 );		break;	// Greater Healing (personal...switch target temporarily to self and then back after casting)
						case 3:		CastSpell( npcAttack, cDefend, 30 );		break;	// Lightning
						case 4:		CastSpell( npcAttack, cDefend, 31 );		break;	// Mana Drain
					}
					break;
				case 5:
					switch( RandomNum( 1, 4 ) ) 
					{
						case 1:		CastSpell( npcAttack, cDefend, 33 );		break;	// Blade Spirits
						case 2:		CastSpell( npcAttack, cDefend, 36 );		break;	// Magic Reflection (personal...no target switching required)
						case 3:		CastSpell( npcAttack, cDefend, 37 );		break;	// Mind Blast
						case 4:		CastSpell( npcAttack, cDefend, 38 );		break;	// Paralyze
					}
					break;
				case 6:	// If NPC is fighting summoned NPC, cast dispel...else cast other spells - Hanse
					if( cDefend->IsNpc() )
					{
						if( cDefend->GetTimer( tNPC_SUMMONTIME ) > 0 && cDefend->GetNPCAiType() != aiGUARD ) 
						{
							CastSpell( npcAttack, cDefend, 41 );
							break;
						}
					}
					switch( RandomNum( 1, 3 ) ) 
					{
						case 1:		CastSpell( npcAttack, cDefend, 42 );		break;	// Energy Bolt
						case 2:		CastSpell( npcAttack, cDefend, 43 );		break;	// Explosion
						case 3:		CastSpell( npcAttack, cDefend, 46 );		break;	// Mass Curse
					}
					break;
				case 7:
					switch( RandomNum( 1, 4 ) )
					{
						case 1:		CastSpell( npcAttack, cDefend, 49 );		break;	// Chain Lightning
						case 2:		CastSpell( npcAttack, cDefend, 51 );		break;	// Flamestrike
						case 3:		CastSpell( npcAttack, cDefend, 53 );		break;	// Mana Vampire
						case 4:		CastSpell( npcAttack, cDefend, 55 );		break;	// Meteor Swarm
					}
					break;
				case 8:
					switch( RandomNum( 1, 2 ) ) 
					{
						case 1:		CastSpell( npcAttack, cDefend, 57 );		break;	// Earthquake
						case 2:		CastSpell( npcAttack, cDefend, 58 );		break;	// Energy Vortex
					}
					break;
				// This is where dragon attacks go eventually when the npc.scp is fixed... - Hanse
				/*
				case 9:
				case 10:
				case 11:
				*/
			default:
				break;
			}
			if( npcAttack->GetSpellCast() != -1 )
				Magic->CastSpell( NULL, npcAttack );
			npcAttack->SetTimer( tNPC_SPATIMER, BuildTimeValue( npcAttack->GetSpDelay() ) );
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	inline void QuickSwitch( CChar *attack, CChar *defend, SI16 spellNum )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Allows beneficial spells to be cast on self during combat
//o---------------------------------------------------------------------------o
inline void QuickSwitch( CChar *attack, CChar *defend, SI16 spellNum )
{
	if( attack == NULL || defend == NULL || attack == defend )
		return;

	attack->SetSpellCast( static_cast<SI08>(spellNum ));
	attack->SetTarg( calcCharFromSer( attack->GetSerial() ) );
	Magic->CastSpell( NULL, attack );
	attack->SetTarg( calcCharFromSer( defend->GetSerial() ) );
	attack->StopSpell();
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool cCombat::CastSpell( CChar *attack, CChar *defend, SI16 spellNum )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handles spellcasting during combat
//o---------------------------------------------------------------------------o
bool cCombat::CastSpell( CChar *attack, CChar *defend, SI16 spellNum )
{
	if( attack == NULL || defend == NULL || attack == defend )
		return false;

	if( attack->GetMana() <= spells[spellNum].Mana() )
		return false;
	switch( spellNum )
	{
		case 7:
			if( !attack->GetReactiveArmour() )
				attack->SetSpellCast( static_cast<SI08>(spellNum) );
			else
				return false;
			break;
		case 11:
			if( attack->GetPoisoned() > 0 )
				QuickSwitch( attack, defend, spellNum );
			else
				return false;
			break;
		case 4:
		case 29:
			if( attack->GetStrength() > attack->GetHP() ) 
				QuickSwitch( attack, defend, spellNum );
			else
				return false;
			break;
		case 36:
			if( !attack->IsPermReflected() )
				attack->SetSpellCast( static_cast<SI08>(spellNum ));
			else
				return false;
			break;
		case 1:
		case 3:
		case 5:
		case 18:
		case 20:
		case 27:
		case 30:
		case 31:
		case 33:
		case 37:
		case 38:
		case 41:
		case 42:
		case 43:
		case 46:
		case 49:
		case 51:
		case 53:
		case 55:
		case 57:
		case 58:
			attack->SetSpellCast( static_cast<SI08>(spellNum) );
			break;
	}
	return true;
}

//o---------------------------------------------------------------------------o
//|   Function    :  UI08 cCombat::getCombatSkill( CChar *i )
//|   Date        :  Unknown
//|   Programmer  :  Zane
//o---------------------------------------------------------------------------o
//|   Purpose     :  Gets your combat skill based on the weapon in your hand (if any)
//o---------------------------------------------------------------------------o
UI08 cCombat::getCombatSkill( CChar *i )
{
	if( i == NULL )
		return WRESTLING;

	CItem *j = getWeapon( i );
	switch( getWeaponType( j ) )
	{
		case DEF_SWORDS:
		case SLASH_SWORDS:
		case ONEHND_LG_SWORDS:
		case TWOHND_LG_SWORDS:
		case ONEHND_AXES:
		case TWOHND_AXES:
		case BARDICHE:
			return SWORDSMANSHIP;
		case DEF_MACES:
		case LG_MACES:
			return MACEFIGHTING;
		case DEF_FENCING:
		case TWOHND_FENCING:
			return FENCING;
		case BOWS:
		case XBOWS:
			return ARCHERY;
		case WRESTLING:
		default:
			return WRESTLING;
	}
}

//o---------------------------------------------------------------------------o
//|   Function    :  CItem * cCombat::getShield( CChar *i )
//|   Date        :  Unknown
//|   Programmer  :  Zane
//o---------------------------------------------------------------------------o
//|   Purpose     :  Check players hands for a shield
//o---------------------------------------------------------------------------o
CItem * cCombat::getShield( CChar *i )
{
	if( i != NULL )
	{
		CItem *shield = i->GetItemAtLayer( 2 );
		if( shield != NULL && shield->IsShieldType() )
			return shield;
	}
	return NULL;
}

//o---------------------------------------------------------------------------o
//|   Function    :  SI16 cCombat::calcAtt( CChar *p )
//|   Date        :  Unknown
//|   Programmer  :  Zane
//o---------------------------------------------------------------------------o
//|   Purpose     :  Calculate total attack power and do damage to the weapon
//o---------------------------------------------------------------------------o
SI16 cCombat::calcAtt( CChar *p )
{
	if( p == NULL )
		return 0;

	SI16 getDamage = 0;

	CItem *weapon = getWeapon( p );
	if( weapon != NULL )
	{
		if( weapon->GetLoDamage() > 0 && weapon->GetHiDamage() > 0 )
		{
			if( weapon->GetLoDamage() >= weapon->GetHiDamage() ) 
				getDamage += weapon->GetLoDamage();
			else
				getDamage += RandomNum( weapon->GetLoDamage(), weapon->GetHiDamage() );

			if( !p->IsNpc() && !RandomNum( 0, 5 ) )
			{
				weapon->IncHP( -1 );
				if( weapon->GetHP() <= 0 )
				{
					cSocket *mSock = calcSocketObjFromChar( p );
					if( mSock != NULL )
					{
						std::string name;
						getTileName( weapon, name );
						mSock->sysmessage( 311, name.c_str() );
					}
					Items->DeleItem( weapon );
					return -1;
				}
			}
		}
	}
	else if( p->IsNpc() ) 
	{
		if( p->GetLoDamage() >= p->GetHiDamage() )
			getDamage = p->GetLoDamage();
		else if( p->GetHiDamage() > 2 )
			getDamage = RandomNum( p->GetLoDamage(), p->GetHiDamage() );
	}
	else
	{
		UI16 getWrestSkill = ( p->GetSkill( WRESTLING ) / 65 );
		if( getWrestSkill > 0 )
			getDamage = HalfRandomNum( getWrestSkill );
		else
			getDamage = RandomNum( 1, 2 );
	}
	if( getDamage < 1 )
		getDamage = 1;
	return getDamage;
}

/*
	New Defense Calculations
	Programmer: Zane (giwo)
	Date: 3/03/2003

	Purpose:
		To emulate OSI's armor calculation methods for specific hit locations and total AR level

	How we do this:
		We need to first determine what we are calculating, total AR (hitLoc 0) or the armor level of a specific locations (hitLoc 1-6).
		Next we need to loop through the items a player has equipped searching for items that match the following criteria:
			Defence > 0
			On a specific layer (depending on the hit location)
			The item with the most AR covering that location
		Lastly we have to return this information. If we are returning the AR of a specific hit location, then we return the actual Def value of the item
			if we are returning the total AR of a character, we need to find the AR / the divisor used for that specific location.

		Divisors based upon location / Armor (as best I can match it to OSI)
			Torso (hitLoc 1):	AR / 2.8
			Arms (hitLoc 2):	AR / 6.8
			Head (hitLoc 3):	AR / 7.3
			Legs (hitLoc 4):	AR / 4.5
			Neck (hitLoc 5):	AR / 14.5
			Hands (hitLoc 6):	AR / 14.5

		Functions:
			checkDef( CItem *checkItem, CItem& currItem, SI32 &currDef )
			getArmorDef( CChar *mChar, CItem& defItem, UI08 bodyLoc, bool findTotal )
			calcDef( CChar *mChar, UI08 hitLoc, bool doDamage )

	All information on armor defence values, divisors, and combat calculations gleaned from uo.stratics.com
	http://uo.stratics.com/content/arms-armor/armor.shtml
	http://uo.stratics.com/content/arms-armor/combat.shtml
	http://uo.stratics.com/content/arms-armor/clothes.shtml
	http://uo.stratics.com/content/aos/combatchanges.shtml
*/

//o--------------------------------------------------------------------------o
//|	Function		-	UI16 cCombat::calcDef( CChar *mChar, UI08 hitLoc, bool doDamage )
//|	Date			-	3/03/2003
//|	Developers		-	Zane
//|	Organization	-	UOX3 DevTeam
//o--------------------------------------------------------------------------o
//|	Description		-	Finds the defense value of a specific location or the entire character based on hitLoc
//o--------------------------------------------------------------------------o
UI16 cCombat::calcDef( CChar *mChar, UI08 hitLoc, bool doDamage )
{
	if( mChar == NULL )
		return 0;

	SI32 total = mChar->GetDef();
	if( !mChar->IsNpc() || mChar->isHuman() )	// Polymorphed Characters and GM's can still wear armor
	{
		CItem *defendItem = NULL;
		if( hitLoc == 0 )
		{
			for( UI08 getLoc = 1; getLoc < 7; ++getLoc )
				getArmorDef( mChar, total, getLoc, true );
			total = (total / 100);
		}
		else
			defendItem = getArmorDef( mChar, total, hitLoc );

		if( doDamage && defendItem != NULL && !mChar->IsNpc() )
		{
			SI08 armorDamage = 0;	// Based on OSI standards, each successful hit does 0 to 2 damage to armor hit
			armorDamage -= RandomNum( 0, 2 );
			defendItem->IncHP( armorDamage );

			if( defendItem->GetHP() <= 0 )
			{
				cSocket *mSock = calcSocketObjFromChar( mChar );
				if( mSock != NULL )
				{
					std::string name;
					getTileName( defendItem, name );
					mSock->sysmessage( 311, name.c_str() );
					mSock->statwindow( mChar );
				}
				Items->DeleItem( defendItem );
			}
		}
	}

	if( total < 2 )
		total = 2;
	return (UI16)total;
}

//o--------------------------------------------------------------------------o
//|	Function		-	CItem * cCombat::getArmorDef( CChar *mChar, SI32 &totalDef, UI08 bodyLoc, bool findTotal )
//|	Date			-	3/03/2003
//|	Developers		-	Zane
//|	Organization	-	UOX3 DevTeam
//o--------------------------------------------------------------------------o
//|	Description		-	Finds the item covering the location bodyLoc with the greatest AR and
//|							returns it along with its def value
//o--------------------------------------------------------------------------o
CItem * cCombat::getArmorDef( CChar *mChar, SI32 &totalDef, UI08 bodyLoc, bool findTotal )
{
	SI32 armorDef = 0;
	CItem *currItem = NULL;
	switch( bodyLoc )
	{
		case 1:		// Torso
			currItem = checkDef( mChar->GetItemAtLayer( 5 ), currItem, armorDef );	// Shirt
			currItem = checkDef( mChar->GetItemAtLayer( 13 ), currItem, armorDef );	// Torso (Inner - Chest Armor)
			currItem = checkDef( mChar->GetItemAtLayer( 17 ), currItem, armorDef );	// Torso (Middle - Tunic, etc)
			currItem = checkDef( mChar->GetItemAtLayer( 20 ), currItem, armorDef );	// Back (Cloak)
			currItem = checkDef( mChar->GetItemAtLayer( 22 ), currItem, armorDef );	// Torso (Outer - Robe)
			if( findTotal )
				armorDef = (SI32)(100 * (R32)( armorDef / 2.8 ));
			break;
		case 2:		// Arms
			currItem = checkDef( mChar->GetItemAtLayer( 19 ), currItem, armorDef );	// Arms
			if( findTotal )
				armorDef = (SI32)(100 * (R32)( armorDef / 6.8 ));
			break;
		case 3:		// Head
			currItem = checkDef( mChar->GetItemAtLayer( 6 ), currItem, armorDef );	// Head
			if( findTotal )
				armorDef = (SI32)(100 * (R32)( armorDef / 7.3 ));
			break;
		case 4:		// Legs
			currItem = checkDef( mChar->GetItemAtLayer( 3 ), currItem, armorDef );	// Shoes
			currItem = checkDef( mChar->GetItemAtLayer( 4 ), currItem, armorDef );	// Pants
			currItem = checkDef( mChar->GetItemAtLayer( 12 ), currItem, armorDef );	// Waist (Half Apron)
			currItem = checkDef( mChar->GetItemAtLayer( 23 ), currItem, armorDef );	// Legs (Outer - Skirt, Kilt)
			currItem = checkDef( mChar->GetItemAtLayer( 24 ), currItem, armorDef );	// Legs (Inner - Leg Armor)
			if( findTotal )
				armorDef = (SI32)(100 * (R32)( armorDef / 4.5 ));
			break;
		case 5:		// Neck
			currItem = checkDef( mChar->GetItemAtLayer( 10 ), currItem, armorDef );	// Neck
			if( findTotal )
				armorDef = (SI32)(100 * (R32)( armorDef / 14.5 ));
			break;
		case 6:		// Hands
			currItem = checkDef( mChar->GetItemAtLayer( 7 ), currItem, armorDef );	// Gloves
			if( findTotal )
				armorDef = (SI32)(100 * (R32)( armorDef / 14.5 ));
			break;
		default:
			break;
	}
	if( findTotal )
		totalDef += armorDef;
	else
		totalDef = armorDef;

	return currItem;
}

//o--------------------------------------------------------------------------o
//|	Function		-	CItem *cCombat::checkDef( CItem *checkItem, CItem& currItem, SI32 &currDef )
//|	Date			-	3/03/2003
//|	Developers		-	Zane
//|	Organization	-	UOX3 DevTeam
//o--------------------------------------------------------------------------o
//|	Description		-	Checks the defense of checkItem vs the defense of currItem and returns
//|							the item with the greater Def and its def value
//o--------------------------------------------------------------------------o
CItem *cCombat::checkDef( CItem *checkItem, CItem *currItem, SI32 &currDef )
{
	if( checkItem != NULL && checkItem->GetDef() > currDef )
	{
		currDef = checkItem->GetDef();
		return checkItem;
	}
	return currItem;
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool cCombat::TimerOk( CChar *c )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check combat timer (for next swing)
//o---------------------------------------------------------------------------o
bool cCombat::TimerOk( CChar *c )
{
	if( c == NULL )
		return false;

	bool retVal = false;
	if( c->GetTimer( tCHAR_TIMEOUT ) < cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) 
		retVal = true;
	return retVal;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem * cCombat::getWeapon( CChar *i )
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find what weapon (if any) character is holding
//o---------------------------------------------------------------------------o
CItem * cCombat::getWeapon( CChar *i )
{
	if( i == NULL )
		return NULL;

	CItem *j = i->GetItemAtLayer( 1 );
	if( j != NULL )
	{
		if( j->GetType() == 9 )	// spell books aren't weapons
			return NULL;
		return j;
	}
	j = i->GetItemAtLayer( 2 );
	if( j != NULL && !j->IsShieldType() )
		return j;
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::CombatOnHorse( CChar *i )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Play animations for combat while mounted
//o---------------------------------------------------------------------------o
void cCombat::CombatOnHorse( CChar *i )
{
	if( i == NULL )
		return;

	CItem *j = getWeapon( i );
	switch( getWeaponType( j ) )
	{
		case BOWS:
			npcAction( i, 0x1B );
			return;
		case XBOWS:
			npcAction( i, 0x1C);
			return;
		case DEF_SWORDS:
		case SLASH_SWORDS:
		case ONEHND_LG_SWORDS:
		case ONEHND_AXES:
		case DEF_MACES:
		case DEF_FENCING:
			npcAction( i, 0x1A );
			return;
		case TWOHND_FENCING:
		case TWOHND_LG_SWORDS:
		case BARDICHE:
		case TWOHND_AXES:
		case LG_MACES:
			npcAction( i, 0x1D );
			return;
		case WRESTLING:
		default:
			npcAction( i, 0x1A );
			return;
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::CombatOnFoot( CChar *i )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Play animations for combat on foot
//o---------------------------------------------------------------------------o
void cCombat::CombatOnFoot( CChar *i )
{
	if( i == NULL )
		return;

	CItem *j = getWeapon( i );
	switch( getWeaponType( j ) )
	{
		case BOWS:
			npcAction( i, 0x12 );
			return;
		case XBOWS:
			npcAction( i, 0x13 );
			return;
		case DEF_SWORDS:
		case SLASH_SWORDS:
		case ONEHND_LG_SWORDS:
		case ONEHND_AXES:
			if( RandomNum( 0, 1 ) )
				npcAction( i, 0x09 );
			else
				npcAction( i, 0x0A );
			return;
		case DEF_MACES:
			if( RandomNum( 0, 1 ) )
				npcAction( i, 0x09 );
			else
				npcAction( i, 0x0B );
			return;
		case LG_MACES:
		case TWOHND_LG_SWORDS:
		case BARDICHE:
		case TWOHND_AXES:
			if( RandomNum( 0, 1 ) )
				npcAction( i, 0x0C );
			else
				npcAction( i, 0x0D );
			return;
		case DEF_FENCING:
			npcAction( i, 0x0A );
			return;
		case TWOHND_FENCING:
			npcAction( i, 0x0E );
			return;
		case WRESTLING:
		default:
			switch( RandomNum( 0, 2 ) )
			{
				case 0:		npcAction( i, 0x0A );	return;  //fist straight-punch
				case 1:		npcAction( i, 0x09 );	return;  //fist top-down
				default:	npcAction( i, 0x1F );	return;  //fist over-head
			}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	UI08 cCombat::getWeaponType( CItem *i )
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find what weapon type a character is holding (based on its ID)
//o---------------------------------------------------------------------------o
UI08 cCombat::getWeaponType( CItem *i )
{
	if( i == NULL )
		return WRESTLING;

	switch( i->GetID() )
	{
		// Slashing Swords
		case 0x0EC4: //skinning knife
		case 0x0EC5: //skinning knife
		case 0x0F60: //longsword
		case 0x0F61: //longsword
		case 0x13B7: //long sword
		case 0x13B8: //long sword
		case 0x1440: //cutlass
		case 0x1441: //cutlass
		case 0x26BB: //bone harvester - AoS
		case 0x26C5: //bone harvester - AoS
		case 0x2573: //magic sword - LBR
		case 0x2574: //magic sword - LBR
		case 0x2575: //magic sword - LBR
		case 0x2576: //magic sword - LBR
		case 0x257E: //short sword - LBR
			return DEF_SWORDS;
		// Default Swords
		case 0x13F6: //butcher knife
		case 0x13F7: //butcher knife
		case 0x13FE: //katana
		case 0x13FF: //katana
		case 0x13B5: //scimitar
		case 0x13B6: //scimitar
		case 0x13B9: //viking sword
		case 0x13BA: //viking sword
		case 0x255E: //ratman sword - LBR
		case 0x2560: //skeleton scimitar - LBR
			return SLASH_SWORDS;
		// One-Handed Lg. Swords
		case 0x0F5E: //broadsword
		case 0x0F5F: //broadsword
		case 0x257D: //longsword - LBR
		case 0x26CE: //paladin sword - AoS
		case 0x26CF: //paladin sword - AoS
		case 0x2554: //daemon sword - LBR
			return ONEHND_LG_SWORDS;
		// Two-Handed Lg. Swords
		case 0x143E: //halberd
		case 0x143F: //halberd
		case 0x26BD: //bladed staff - AoS
		case 0x26C7: //bladed staff - AoS
		case 0x26C1: //crescent blade - AoS
		case 0x26CB: //crescent blade - AoS
		case 0x2578: //no-dachi - LBR
			return TWOHND_LG_SWORDS;
		// Bardiche
		case 0x0F4D: //bardiche
		case 0x0F4E: //bardiche
		case 0x26BA: //scythe - AoS
		case 0x26C4: //scythe - AoS
		case 0x255B: //ophidian bardiche - LBR
		case 0x2577: //naginata - LBR
			return BARDICHE;
		// One-Handed Axes
		case 0x0EC2: //cleaver
		case 0x0EC3: //cleaver
		case 0x0E85: //pickaxe
		case 0x0E86: //pickaxe
		case 0x2567: //orc lord battleaxe - LBR
		case 0x2579: //pick - LBR
			return ONEHND_AXES;
		// Two-Handed Axes
		case 0x13FA: //large battle axe
		case 0x13FB: //large battle axe
		case 0x1442: //two handed axe
		case 0x1443: //two handed axe
		case 0x0F43: //hatchet
		case 0x0F44: //hatchet
		case 0x0F45: //executioner's axe
		case 0x0F46: //executioner's axe
		case 0x0F47: //battle axe
		case 0x0F48: //battle axe
		case 0x0F49: //axe
		case 0x0F4A: //axe
		case 0x0F4B: //double axe
		case 0x0F4C: //double axe
		case 0x255F: //skeleton axe - LBR
		case 0x255D: //ratman axe - LBR
		case 0x2564: //troll axe - LBR
		case 0x2570: //hatchet - LBR
			return TWOHND_AXES;
		// Default Maces
		case 0x13E3: //smith's hammer
		case 0x13E4: //smith's hammer
		case 0x13B3: //club
		case 0x13B4: //club
		case 0x0FB4: //sledge hammer
		case 0x0FB5: //sledge hammer
		case 0x0F5C: //mace
		case 0x0F5D: //mace
		case 0x1406: //war mace
		case 0x1407: //war mace
		case 0x143C: //hammer pick
		case 0x143D: //hammer pick
		case 0x13AF: //war axe
		case 0x13B0: //war axe
		case 0x143A: //maul
		case 0x143B: //maul
		case 0x26BC: //scepter - AoS
		case 0x26C6: //scepter - AoS
		case 0x2557: //lizardman's mace - LBR
		case 0x255C: //orc club - LBR
		case 0x256F: //smyth's hammer - LBR
		case 0x257F: //war mace - LBR
			return DEF_MACES;
		// Large Maces
		case 0x13F4: //crook
		case 0x13F5: //crook
		case 0x13F8: //gnarled staff
		case 0x13F9: //gnarled staff
		case 0x0DF0: //black staff
		case 0x0DF1: //black staff
		case 0x0E81: //shepherd's crook
		case 0x0E82: //shepherd's crook
		case 0x0E89: //quarter staff
		case 0x0E8A: //quarter staff
		case 0x1438: //war hammer
		case 0x1439: //war hammer
		case 0x2555: //ettin hammer - LBR
		case 0x2556: //liche's staff - LBR
		case 0x2559: //ogre's club - LBR
		case 0x255A: //ophidian staff - LBR
		case 0x2561: //terathan staff - LBR
		case 0x2565: //troll maul - LBR
		case 0x2566: //frost troll club - LBR
		case 0x2568: //orc mage staff - LBR
		case 0x2569: //bone mage staff - LBR
		case 0x256B: //magic staff - LBR
		case 0x256C: //magic staff - LBR
		case 0x256D: //magic staff - LBR
		case 0x256E: //magic staff - LBR
			return LG_MACES;
		// Bows
		case 0x13B1: //bow
		case 0x13B2: //bow
		case 0x26C2: //composite bow - AoS
		case 0x26CC: //composite bow - AoS
		case 0x2571: //horseman's bow - LBR
			return BOWS;
		// Crossbows
		case 0x0F4F: //crossbow
		case 0x0F50: //crossbow
		case 0x13FC: //heavy crossbow
		case 0x13FD: //heavy crossbow
		case 0x26C3: //repeating crossbow - AoS
		case 0x26CD: //repeating crossbow - AoS
			return XBOWS;
		// Normal Fencing Weapons
		case 0x0F51: //dagger
		case 0x0F52: //dagger
		case 0x1400: //kryss
		case 0x1401: //kryss
		case 0x1402: //short spear
		case 0x1403: //short spear
		case 0x1404: //war fork
		case 0x1405: //war fork
		case 0x257C: //sword(rapier) - LBR
			return DEF_FENCING;
		// Stabbing Fencing Weapons
		case 0x0E87: //pitchfork
		case 0x0E88: //pitchfork
		case 0x0F62: //spear
		case 0x0F63: //spear
		case 0x26BE: //pike - AoS
		case 0x26C8: //pike - AoS
		case 0x26BF: //double bladed staff - AoS
		case 0x26C9: //double bladed staff - AoS
		case 0x26C0: //lance - AoS
		case 0x26CA: //lance - AoS
		case 0x2558: //Lizardman's spear - LBR
		case 0x2562: //terathan spear - LBR
		case 0x2572: //horseman's javelin - LBR
		case 0x257A: //spear - LBR
		case 0x257B: //spear - LBR
			return TWOHND_FENCING;
		default: // Wrestling
			return WRESTLING;
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::ItemSpell( CChar *attacker, CChar *defender )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle weapons that cast spells
//o---------------------------------------------------------------------------o
void cCombat::ItemSpell( CChar *attacker, CChar *defender )
{
	if( attacker == NULL || defender == NULL || attacker == defender )
		return;

	cSocket *mSock = calcSocketObjFromChar( attacker );
	if( mSock != NULL )
		mSock->CurrentSpellType( 2 );

	CItem *wand = getWeapon( attacker );
	if( wand != NULL )
	{
		if( wand->GetOffSpell() && wand->GetHiDamage() && wand->GetType() == 15 )
		{
			switch( wand->GetOffSpell() )
			{
				case 1:		CastSpell( attacker, defender,  1 );	break;	// Clumsy
				case 2:		CastSpell( attacker, defender,  3 );	break;	// Feeblemind
				case 3:		CastSpell( attacker, defender,  5 );	break;	// Magic Arrow
				case 4:		CastSpell( attacker, defender,  8 );	break;	// Weaken
				case 5:		CastSpell( attacker, defender, 12 );	break;	// Harm
				case 6:		CastSpell( attacker, defender, 18 );	break;	// Fireball
				case 8:		CastSpell( attacker, defender, 27 );	break;	// Curse
				case 9:		CastSpell( attacker, defender, 30 );	break;	// Lightning
				case 11:	CastSpell( attacker, defender, 37 );	break;	// Mind Blast
				case 12:	CastSpell( attacker, defender, 38 );	break;	// Paralyze
				case 14:	CastSpell( attacker, defender, 43 );	break;	// Explosion
				case 15:	CastSpell( attacker, defender, 51 );	break;	// Flame strike
			}

			wand->SetMoreZ( wand->GetMoreZ() - 1 );
			if( wand->GetMoreZ() == 0 )
			{
				wand->SetType( wand->GetType2() );
				wand->SetMoreX( 0 );
				wand->SetMoreY( 0 );
				wand->SetOffSpell( 0 );
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::doSoundEffect( CChar *p, CItem *weapon )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Do the "Hit" Sound Effect
//o---------------------------------------------------------------------------o
void cCombat::doSoundEffect( CChar *p, CItem *weapon )
{
	if( p == NULL || weapon == NULL )
		return;

	switch( getWeaponType( weapon ) )
	{
		case ONEHND_AXES:
		case TWOHND_AXES:
		case DEF_MACES:
		case LG_MACES:
			Effects->PlaySound( p, 0x0232 ); // Whoosh Weapons
			break;
		case DEF_SWORDS:
		case DEF_FENCING:
		case TWOHND_FENCING:
			Effects->PlaySound( p, 0x023C ); // Stabbing Weapons
			break;
		case BARDICHE:
			Effects->PlaySound( p, 0x0236 ); // Bardiche
			break;
		case SLASH_SWORDS:
			Effects->PlaySound( p, 0x023B ); // Slashing Weapons
			break;
		case ONEHND_LG_SWORDS:
		case TWOHND_LG_SWORDS:
			Effects->PlaySound( p, 0x0237 ); // Large Swords
			break;
		case BOWS:
		case XBOWS:
			Effects->PlaySound( p, 0x0223 ); // Bows
			break;
		case WRESTLING:
		default:
			switch( RandomNum( 0, 3 ) ) // Wrestling
			{
				case 0:		Effects->PlaySound( p, 0x0135 );	break;
				case 1:		Effects->PlaySound( p, 0x0137 );	break;
				case 2:		Effects->PlaySound( p, 0x013D );	break;
				default:	Effects->PlaySound( p, 0x013B );	break;
			}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::doMissedSoundEffect( CChar *p )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Do the "Missed" Sound Effect
//o---------------------------------------------------------------------------o
void cCombat::doMissedSoundEffect( CChar *p )
{
	if( p == NULL )
		return;

	switch( RandomNum( 0, 2 ) )
	{
		case 0:	Effects->PlaySound( p, 0x0238 );	break;
		case 1:	Effects->PlaySound( p, 0x0239 );	break;
		default: Effects->PlaySound( p, 0x023A );	break;
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::Kill( CChar *attack, CChar *defend )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle death during combat
//o---------------------------------------------------------------------------o
void cCombat::Kill( CChar *attack, CChar *defend )
{
	if( attack == NULL || defend == NULL || attack == defend )
		return;

	Karma( attack, defend, ( 0 - ( defend->GetKarma() ) ) );
	Fame( attack, defend->GetFame() );
	if( attack->GetNPCAiType() == aiGUARD && defend->IsNpc() )
	{
		npcAction( defend, 0x15 );
		Effects->playDeathSound( defend );

		Npcs->DeleteChar( defend ); // Guards, don't give body
		npcToggleCombat( attack );
		return;
	}

	// Add murder counts
	if( !attack->IsNpc() && !defend->IsNpc() )
	{
		if( attack->DidAttackFirst() && defend->IsInnocent() && GuildSys->ResultInCriminal( attack, defend ) && Races->Compare( attack, defend ) == 0 )
		{
			attack->SetKills( attack->GetKills() + 1 );
			cSocket *aSock = calcSocketObjFromChar( attack );
			if( aSock != NULL )
			{
				aSock->sysmessage( 314, attack->GetKills() );
				if( attack->GetKills() == cwmWorldState->ServerData()->RepMaxKills() + 1 )
					aSock->sysmessage( 315 );
			}
		}
		Console.Log( Dictionary->GetEntry( 1617 ).c_str(), "PvP.log", defend->GetName().c_str(), attack->GetName().c_str() );
	}
	if( attack->IsNpc() )
		npcToggleCombat( attack );
	doDeathStuff( defend );
}

//o---------------------------------------------------------------------------o
//|	Function	-	cCombat::SpawnGuard( CChar *mChar, CChar *targChar, SI16 x, SI16 y, SI08 z )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle spawning a guard in guarded areas
//|					NEED TO REWORK FOR REGIONAL GUARD STUFF
//o---------------------------------------------------------------------------o
void cCombat::SpawnGuard( CChar *mChar, CChar *targChar, SI16 x, SI16 y, SI08 z )
{
	if( mChar == NULL || targChar == NULL )
		return;

	if( targChar->IsDead() || targChar->IsInvulnerable() )
		return;

	UI08 targRegion = mChar->GetRegion(); 

	if( !region[targRegion]->IsGuarded() || !cwmWorldState->ServerData()->GuardsStatus() )
		return;

	bool reUseGuard = false;

	CHARACTER trgChar = calcCharFromSer( targChar->GetSerial() );
	if( trgChar == INVALIDSERIAL )
		return;

	CChar *getGuard = NULL;

	SI16 xOffset = MapRegion->GetGridX( mChar->GetX() );
	SI16 yOffset = MapRegion->GetGridY( mChar->GetY() );

	SubRegion *toCheck = MapRegion->GetGrid( xOffset, yOffset, mChar->WorldNumber() );
	if( toCheck != NULL )
	{
		for( getGuard = toCheck->FirstChar(); !toCheck->FinishedChars(); getGuard = toCheck->GetNextChar() )
		{
			if( getGuard == NULL )
				break;

			if( !getGuard->IsNpc() || getGuard->GetNPCAiType() != aiGUARD )
				continue;

			if( getGuard->GetTarg() == INVALIDSERIAL || getGuard->GetTarg() == trgChar  )
			{
				if( charInRange( getGuard, targChar ) )
					reUseGuard = true;
			}
			else if( getGuard->GetTarg() == trgChar )
				return;
		}
	}
	// 1/13/2003 - Maarc - Fix for JSE NocSpawner
	if( !reUseGuard )
	{
		int t = region[targRegion]->GetRandomGuard();
#if defined( _MSC_VER )
#pragma note( "DEPENDENT ON NUMERIC NPC SECTION" )
#endif
		UString tempStuff	= UString::number( t );
		getGuard			= Npcs->AddNPCxyz( NULL, tempStuff, x, y, z, mChar->WorldNumber() ); 
		if( getGuard == NULL )
			return;

		getGuard->SetNPCAiType( aiGUARD );
	}
	//
	if( getGuard != NULL )
	{
		getGuard->SetAttackFirst( true );
		getGuard->SetAttacker( targChar );
		getGuard->SetTarg( trgChar );
		getGuard->SetNpcWander( 2 );
		if( !getGuard->IsAtWar() )
			npcToggleCombat( getGuard );

		if( reUseGuard )
		{
			getGuard->SetLocation( targChar );
			getGuard->Teleport();
		}
		else
		{
			getGuard->SetTimer( tNPC_MOVETIME, BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->NPCSpeed() )) );
			getGuard->SetTimer( tNPC_SUMMONTIME, BuildTimeValue( 25 ) );

			Effects->PlaySound( getGuard, 0x01FE );
			Effects->staticeffect( getGuard, 0x372A, 0x09, 0x06 );
			
			getGuard->Update();
			getGuard->talkAll( 313, true );
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool WillResultInCriminal( CChar *attack, CChar *targ )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check flagging, race, and guild info to find if character
//|					should be flagged criminal (returns true if so)
//o---------------------------------------------------------------------------o
bool cCombat::WillResultInCriminal( CChar *attack, CChar *targ )
{
	if( attack == NULL || targ == NULL ) 
		return false;
	else if( !GuildSys->ResultInCriminal( attack, targ ) || Races->Compare( attack, targ ) != 0 ) 
		return false;
	else if( targ->GetFlag() == 0x04 && attack->GetFlag() != 0x01 )
		return true;
	else 
		return false;
}

//o---------------------------------------------------------------------------o
//|	Function	-	PlayerAttack( cSocket *s )
//|	Programmer	-	UOX DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle player attacking (Double-clicking whilst in war mode)
//o---------------------------------------------------------------------------o
void cCombat::PlayerAttack( cSocket *s )
{
	if( s == NULL )
		return;

	CChar *ourChar = s->CurrcharObj();
	if( ourChar == NULL )
		return;

	SERIAL serial = s->GetDWord( 1 );
	if( serial == INVALIDSERIAL ) 
	{
		ourChar->SetTarg( INVALIDSERIAL );
		return;
	}
	CChar *i = calcCharObjFromSer( serial );
	if( i == NULL )
	{
		ourChar->SetTarg( INVALIDSERIAL );
		return;
	}
	if( ourChar->IsDead() )
	{
		if( i->IsNpc() )
		{
			switch( i->GetNPCAiType() )
			{
				case aiHEALER_G:	// Good Healer
					if( !ourChar->IsCriminal() && !ourChar->IsMurderer() )
					{ // Character is innocent
						if( objInRange( i, ourChar, DIST_NEARBY ) )
						{	//let's resurrect him!
							npcAction( i, 0x10 );
							Targ->NpcResurrectTarget( ourChar );
							Effects->staticeffect( ourChar, 0x376A, 0x09, 0x06 );
							i->talkAll( ( 316 + RandomNum( 0, 4 ) ), false );
						} 
						else
							i->talkAll( 321, true );
					} 
					else // Character is criminal or murderer
						i->talkAll( 322, true );
					break;
				case aiHEALER_E: // Evil Healer
					if( ourChar->IsMurderer() )
					{
						if( objInRange( i, ourChar, DIST_NEARBY ) )	// let's resurrect him
						{
							npcAction( i, 0x10 );
							Targ->NpcResurrectTarget( ourChar );
							Effects->staticeffect( ourChar, 0x3709, 0x09, 0x19 ); //Flamestrike effect
							i->talkAll( ( 323 + RandomNum( 0, 4 ) ), false );
						} 
						else
							i->talkAll( 328, true );
					} 
					else
						i->talkAll( 329, true );
					break;
				default:
					s->sysmessage( 330 );
					break;
			}
			return;
		} //if isNpc
		else
		{
			if( cwmWorldState->ServerData()->PlayerPersecutionStatus() )
			{
				ourChar->SetTarg( calcCharFromSer( i->GetSerial() ) );
				Skills->Persecute( s );
				return;
			} 
			else
			{
				s->sysmessage( 330 );
				return;
			}
		}//if isNpc
	}//if isDead
	else if( ourChar->GetTarg() != calcCharFromSer( serial ) )
	{
		ourChar->SetTarg( calcCharFromSer( i->GetSerial() ) );
		if( ourChar->GetHidden() && !ourChar->IsPermHidden() )
			ourChar->ExposeToView();
		if( i->IsDead() || i->GetHP() <= 0 )
		{
			s->sysmessage( 331 );
			return;
		}
		if( i->GetNPCAiType() == aiPLAYERVENDOR ) // PlayerVendors
		{
			s->sysmessage( 332, i->GetName().c_str() );
			return;
		}
		if( i->IsInvulnerable() )
		{
			s->sysmessage( 333 );
			return;
		}
		
		//flag them FIRST so that anything attacking them as a result of this is not flagged.
		if( WillResultInCriminal( ourChar, i ) ) //REPSYS
		{
			bool regionGuarded = ( region[i->GetRegion()]->IsGuarded() );
			if( cwmWorldState->ServerData()->GuardsStatus() && regionGuarded && i->IsNpc() && i->GetNPCAiType() != aiGUARD && i->isHuman() )
				i->talkAll( 335, true );
			criminal( ourChar );
		}

		if( i->IsGuarded() )
			petGuardAttack( ourChar, i, i->GetSerial() );

		ourChar->emoteAll( 334, true, ourChar->GetName().c_str(), i->GetName().c_str() );	// Attacker emotes "You see attacker attacking target" to all nearby

		if( !i->IsNpc() )
		{
			cSocket *iSock = calcSocketObjFromChar( i );
			if( iSock != NULL )
				i->emote( iSock, 1281, true, ourChar->GetName().c_str() ); // "Attacker is attacking you!" sent to target socket only
		}

		// keep the target highlighted
		CPAttackOK toSend = (*i);
		s->Send( &toSend );
		
		if( i->GetNPCAiType() != aiGUARD && i->GetTarg() == INVALIDSERIAL )
		{
			i->SetAttacker( ourChar );
			i->SetAttackFirst( false );
		}
		ourChar->SetAttackFirst( true );
		ourChar->SetAttacker( i );
		AttackTarget( i, ourChar );
	}
}

bool cCombat::StartAttack( CChar *cAttack, CChar *cTarget )
{
	if( cAttack == NULL || cTarget == NULL || cAttack == cTarget )
		return false;
	if( cAttack->IsDead() || cTarget->IsDead() )
		return false;
	if( !LineOfSight( NULL, cAttack, cTarget->GetX(), cTarget->GetY(), cTarget->GetZ(), WALLS_CHIMNEYS + DOORS + FLOORS_FLAT_ROOFING ) )
		return false;

	CHARACTER attack = calcCharFromSer( cAttack->GetSerial() );
	CHARACTER target = calcCharFromSer( cTarget->GetSerial() );
	bool returningAttack = false;

	if( cAttack->GetTarg() == target )
		return false;

	cAttack->SetTarg( target );
	cAttack->SetAttacker( cTarget );
	cAttack->SetAttackFirst( ( cTarget->GetTarg() != attack ) );
	if( !cTarget->IsInvulnerable() && cTarget->GetTarg() == INVALIDSERIAL )	// Only invuln don't fight back
	{
		cTarget->SetTarg( attack );
		cTarget->SetAttacker( cAttack );
		cTarget->SetAttackFirst( false );
		returningAttack = true;
	}

	if( cAttack->GetHidden() && !cAttack->IsPermHidden() )
		cAttack->ExposeToView();

	if( !cAttack->IsNpc() )
		cAttack->BreakConcentration( calcSocketObjFromChar( cAttack ) );
	else
	{
		Effects->playMonsterSound( cAttack, cAttack->GetID(), SND_STARTATTACK );

		// if the source is an npc, make sure they're in war mode and reset their movement time
		if( !cAttack->IsAtWar() ) 
			npcToggleCombat( cAttack );
		cAttack->SetTimer( tNPC_MOVETIME, BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->NPCSpeed() )) );
	}

	// Only unhide the defender, if they're going to return the attack (otherwise they're doing nothing!)
	if( returningAttack )
	{
		if( cTarget->GetHidden() && !cTarget->IsPermHidden() )
			cTarget->ExposeToView();

		if( !cTarget->IsNpc() )
		cTarget->BreakConcentration( calcSocketObjFromChar( cTarget ) );

		// if the target is an npc, and not a guard, make sure they're in war mode and update their movement time
		// ONLY IF THEY'VE CHANGED ATTACKER
		if( cTarget->IsNpc() && cTarget->GetNPCAiType() != aiGUARD )
		{
			if( !cTarget->IsAtWar() )
				npcToggleCombat( cTarget );
			cTarget->SetTimer( tNPC_MOVETIME, BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->NPCSpeed() )) );
		}
	}
	return true;
}
void cCombat::AttackTarget( CChar *cAttack, CChar *cTarget )
{
	if( !StartAttack( cAttack, cTarget ) )
		return;

	bool gCompare = GuildSys->ResultInCriminal( cAttack, cTarget );
	SI08 rCompare = Races->Compare( cAttack, cTarget );

	// If the target is an innocent, not a racial or guild ally/enemy, then flag them as criminal
	// and, of course, call the guards ;>
	if( cTarget->IsInnocent() && gCompare && rCompare == 0 )
	{
		bool regionGuarded = ( region[cTarget->GetRegion()]->IsGuarded() );
		if( cwmWorldState->ServerData()->GuardsStatus() && regionGuarded )
		{
			if( cTarget->IsNpc() && cTarget->GetNPCAiType() != aiGUARD && cTarget->isHuman() )
				cTarget->talkAll( 1282, true );
			criminal( cAttack );
#ifdef DEBUG
			Console.Print( "DEBUG: [npcAttackTarget)] %s is being set to criminal", cAttack->GetName() );
#endif
		}
	}
	if( cAttack->DidAttackFirst() )
	{
		cAttack->emoteAll( 334, true, cAttack->GetName().c_str(), cTarget->GetName().c_str() );  // NPC should emote "Source is attacking Target" to all nearby - Zane
		if( !cTarget->IsNpc() )
		{
			cSocket *iSock = calcSocketObjFromChar( cTarget );
			if( iSock != NULL )
				cTarget->emote( iSock, 1281, true, cAttack->GetName().c_str() );	// Target should get an emote only to his socket "Target is attacking you!" - Zane
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	petGuardAttack( CChar *mChar, CChar *owner, SERIAL guarded )
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get the pet guarding an item / character and have him attack
//|					the person using / attacking the item / character
//o---------------------------------------------------------------------------o
void cCombat::petGuardAttack( CChar *mChar, CChar *owner, SERIAL guarded )
{
	if( mChar == NULL || owner == NULL || guarded == INVALIDSERIAL )
		return;

	if( mChar->GetSerial() == owner->GetSerial() || mChar->IsCounselor() || mChar->IsGM() )
		return;

	CChar *petGuard = Npcs->getGuardingPet( owner, guarded );
	if( petGuard != NULL && objInRange( mChar, petGuard, cwmWorldState->ServerData()->CombatMaxRange() ) )
	{
		if( !Npcs->checkPetFriend( mChar, petGuard ) )
			AttackTarget( petGuard, mChar );
		else
		{
			cSocket *oSock = calcSocketObjFromChar( owner );
			if( oSock != NULL )
				oSock->sysmessage( 1629 );
		}
	}
}
