#include "uox3.h"
#include "targeting.h"
#include "commands.h"
#include "cSpawnRegion.h"
#include "cServerDefinitions.h"
#include "PageVector.h"
#include "ssection.h"
#include "gump.h"
#include "trigger.h"
#include "mapstuff.h"
#include "cScript.h"
#include "cEffects.h"
#include "network.h"
#include "classes.h"
#include "cSpeech.h"
#include "packets.h"

#undef DBGFILE
#define DBGFILE "commands.cpp"

cCommands::cCommands()
{
	CommandReset();
}

//o---------------------------------------------------------------------------o
//|	Function	-	static inline void doTarget( cSocket *s, UI08 targID, SI32 dictEntry )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Do targeting stuff
//o---------------------------------------------------------------------------o
static inline void doTarget( cSocket *s, UI08 targID, SI32 dictEntry )
{
	s->target( 0, targID, dictEntry );
}

//o---------------------------------------------------------------------------o
//|	Function		-	UI08 NumArguments( void )
//|	Date			-	3/12/2003
//|	Programmer		-	Zane
//|	Modified		-	4/2/2003 - Reduced to a UI08 - Zane
//o---------------------------------------------------------------------------o
//|	Purpose			-	Number of arguments in a command
//o---------------------------------------------------------------------------o
UI08 cCommands::GetNumArguments( void )
{
	return numArguments;
}
void cCommands::SetNumArguments( UI08 newVal )
{
	numArguments = newVal;
}

//o---------------------------------------------------------------------------o
//|	Function	-	SI32 cCommands::GetArgument( UI08 argNum )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Grabs argument argNum and converts it to an integer
//o---------------------------------------------------------------------------o
SI32 cCommands::GetArgument( UI08 argNum )
{
	if( GetComm( argNum ) == NULL )
		return 0;
	return UString( GetComm( argNum ) ).toLong();
}

//o---------------------------------------------------------------------------o
//|	Function	-	char * cCommands::Comm( UI08 part )
//|	Date		-	4/02/2003
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Gets/Sets the Comm value
//o---------------------------------------------------------------------------o
char * cCommands::GetComm( UI08 part )
{
	if( part > CMAX )
		return NULL;
	return comm[part];
}
void cCommands::SetComm( UI08 part, char *value )
{
	if( part <= CMAX )
		comm[part] = value;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::splitline( char *toSplit )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Puts single words of cline into a comm array
//o---------------------------------------------------------------------------o
void cCommands::splitline( char *toSplit )
{
	UI08 i = 0;
	char *d = " ";
	char *s = strtok( toSplit, d );
	while( s != NULL )
	{
		if( i > CMAX )
			break;
		SetComm( i, s );
		++i;
		s = strtok( NULL, d );
	}
	Commands->SetNumArguments( i );
}

//o--------------------------------------------------------------------------o
//|	Function		-	void cCommands::Command( cSocket *s )
//|	Date			-	
//|	Developers		-	EviLDeD
//|	Organization	-	UOX3 DevTeam
//|	Status			-	Currently under development
//o--------------------------------------------------------------------------o
//|	Description		-	Handles commands sent from client
//o--------------------------------------------------------------------------o
//| Modifications	-	25 June, 2003
//|						Made it accept a CPITalkRequest, allowing to remove
//|						the need for Offset and unicode decoding
//o--------------------------------------------------------------------------o
void cCommands::Command( CPITalkRequest *pckTalk )
{
	cSocket *s		= pckTalk->GetSocket();
	CChar *mChar	= s->CurrcharObj();
	char *text		= &(pckTalk->Text()[1]);

	TCommand( s, mChar, text );
}

void cCommands::TCommand( cSocket *s, CChar *mChar, char* text )
{
	memset( s->TBuffer(), 0, MAXBUFFER );
	char *tbuffer	= (char *)s->TBuffer();
	size_t i		= 9;
	size_t sLen		= strlen( text );

	for( i = 0; i < sLen; ++i )
	{
		tbuffer[i] = text[i];
		text[i] = toupper( text[i] );
	}
	splitline( text );
	if( GetNumArguments() < 1 )
		return;
	char *comm = text;	// discard the leading '

	TargetTableIterator findTarg = targ_table.find( comm );
	if( findTarg != targ_table.end() )
	{
		bool plClearance = ( mChar->GetCommandLevel() >= findTarg->second.cmdLevelReq || mChar->GetAccount().wAccountIndex == 0 );
		if( !plClearance )
		{
			Log( comm, mChar, NULL, "Insufficient clearance" );
			s->sysmessage( 337 );
			return;
		}
		Log( comm, mChar, NULL, "Cleared" );
		switch( findTarg->second.cmdType )
		{
			case CMD_TARGET:
				doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				break;
			case CMD_TARGETX:
				if( GetNumArguments() == 2 )
				{
					s->AddX( Commands->GetArgument( 1 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 338 );
				break;
			case CMD_TARGETXY:
				if( GetNumArguments() == 3 )
				{
					s->AddX( Commands->GetArgument( 1 ) );
					s->AddY( Commands->GetArgument( 2 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 339 );
				break;
			case CMD_TARGETXYZ:
				if( GetNumArguments() == 4 )
				{
					s->AddX( Commands->GetArgument( 1 ) );
					s->AddY( Commands->GetArgument( 2 ) );
					s->AddZ( Commands->GetArgument( 3 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 340 );
				break;
			case CMD_TARGETID1:
				if( GetNumArguments() == 2 )
				{
					s->AddID1( Commands->GetArgument( 1 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 338 );
				break;
			case CMD_TARGETID2:
				if( GetNumArguments() == 3 )
				{
					s->AddID1( Commands->GetArgument( 1 ) );
					s->AddID2( Commands->GetArgument( 2 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 339 );
				break;
			case CMD_TARGETID3:
				if( GetNumArguments() == 4 )
				{
					s->AddID1( Commands->GetArgument( 1 ) );
					s->AddID2( Commands->GetArgument( 2 ) );
					s->AddID3( Commands->GetArgument( 3 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 340 );
				break;
			case CMD_TARGETID4:
				if( GetNumArguments() == 5 )
				{
					s->AddID1( Commands->GetArgument( 1 ) );
					s->AddID2( Commands->GetArgument( 2 ) );
					s->AddID3( Commands->GetArgument( 3 ) );
					s->AddID4( Commands->GetArgument( 4 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 344 );
				break;
			case CMD_TARGETTMP:
				if( GetNumArguments() == 2 )
				{
					s->TempInt( Commands->GetArgument( 1 ) );
					doTarget( s, findTarg->second.targID, findTarg->second.dictEntry );
				}
				else
					s->sysmessage( 338 );
				break;
		}
		return;
	}
	else
	{
		CmdTableIterator toFind = cmd_table.find( comm );
		if( toFind == cmd_table.end() )
		{
			cScript *toGrab = Trigger->GetScript( mChar->GetScriptTrigger() );
			if( toGrab == NULL || !toGrab->OnCommand( s ) )
				s->sysmessage( 336 );
			return;
		}
		else
		{
			bool plClearance = ( mChar->GetCommandLevel() >= toFind->second.cmdLevelReq || mChar->GetAccount().wAccountIndex == 0 );
			// from now on, account 0 ALWAYS has admin access, regardless of command level
			if( !plClearance )
			{
				Log( comm, mChar, NULL, "Insufficient clearance" );
				s->sysmessage( 337 );
				return;
			}
			Log( comm, mChar, NULL, "Cleared" );

			switch( toFind->second.cmdType )
			{
				case CMD_FUNC:
					(*((CMD_EXEC)toFind->second.cmd_extra)) ();
					break;
				case CMD_SOCKFUNC:
					(*((CMD_SOCKEXEC)toFind->second.cmd_extra)) (s);
					break;
				case CMD_ITEMMENU:
					NewAddMenu( s, (int)toFind->second.cmd_extra );
					break;
				default:
					s->sysmessage( 346 );
					break;
			}
			return;
		}
	}
	s->sysmessage( "BUG: Should never reach end of command() function!" );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::MakeShop( CChar *c )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Turn an NPC into a shopkeeper
//o---------------------------------------------------------------------------o
void cCommands::MakeShop( CChar *c )
{
	if( c == NULL )
		return;
	c->SetShop( true );
	CItem *tPack = NULL;
	for( UI08 i = 0x1A; i <= 0x1C; ++i )
	{
		tPack = c->GetItemAtLayer( i );
		if( tPack == NULL )
		{
			tPack = Items->SpawnItem( NULL, c, 1, "#", false, 0x2AF8, 0, false, false );
			if( tPack != NULL )
			{
				tPack->SetLayer( i );
				if( !tPack->SetCont( c ) )
					Items->DeleItem( tPack );
				else
				{
					tPack->SetType( 1 );
					tPack->SetNewbie( true );
				}
			}
		}
	}
	c->Teleport();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::showQue( cSocket *s, bool isGM )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Shows next unhandled call in the queue
//o---------------------------------------------------------------------------o
void cCommands::showQue( cSocket *s, bool isGM )
{
	if( isGM )
		GMQueue->SendAsGump( s );
	else
		CounselorQueue->SendAsGump( s );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::nextCall( cSocket *s, bool isGM )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Send GM/Counsellor to next call in the que
//o---------------------------------------------------------------------------o
void cCommands::nextCall( cSocket *s, bool isGM )
{
	CChar *j = NULL;
	int x = 0;
	CChar *mChar = s->CurrcharObj();
	if( mChar->GetCallNum() != 0 )
		closeCall( s, isGM );
	HelpRequest *tempPage = NULL;
	if( isGM )
	{
		for( tempPage = GMQueue->First(); !GMQueue->AtEnd(); tempPage = GMQueue->Next() )
		{
			if( !tempPage->IsHandled() )
			{
				j = calcCharObjFromSer( tempPage->WhoPaging() );
				if( j != NULL )
				{
					GumpDisplay GMNext( s, 300, 200 );
					GMNext.AddData( "Pager: ", j->GetName().c_str() );
					GMNext.AddData( "Problem: ", tempPage->Reason() );
					GMNext.AddData( "Serial number ", tempPage->WhoPaging(), 3 );
					GMNext.AddData( "Paged at: ", tempPage->TimeOfPage() );
					GMNext.Send( 4, false, INVALIDSERIAL );
					tempPage->IsHandled( true );
					mChar->SetLocation( j );
					mChar->SetCallNum( static_cast<SI16>(tempPage->RequestID() ));
					mChar->Teleport();
					++x;
				}
				if( x > 0 )
					break;
			}
		}
		if( x == 0 )
			s->sysmessage( 347 );
	}
	else //Player is a counselor
	{
		x = 0;
		for( tempPage = CounselorQueue->First(); !CounselorQueue->AtEnd(); tempPage = CounselorQueue->Next() )
		{
			if( !tempPage->IsHandled() )
			{
				j = calcCharObjFromSer( tempPage->WhoPaging() );
				if( j != NULL )
				{
					GumpDisplay CNext( s, 300, 200 );
					CNext.AddData( "Pager: ", j->GetName().c_str() );
					CNext.AddData( "Problem: ", tempPage->Reason() );
					CNext.AddData( "Serial number ", tempPage->WhoPaging(), 3 );
					CNext.AddData( "Paged at: ", tempPage->TimeOfPage() );
					CNext.Send( 4, false, INVALIDSERIAL );
					tempPage->IsHandled( true );
					mChar->SetCallNum( static_cast<SI16>(tempPage->RequestID()) );
					mChar->SetLocation( j );
					mChar->Teleport();
					++x;
					break;
				}
			}
			if( x > 0 )
				break;
		}
		if( x == 0 )
			s->sysmessage( 348 );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::closeCall( cSocket *s, bool isGM )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Closes an open call in the Que
//o---------------------------------------------------------------------------o
void cCommands::closeCall( cSocket *s, bool isGM )
{
	CChar *mChar = s->CurrcharObj();
	if( mChar->GetCallNum() != 0 )
	{
		if( isGM )
		{
			if( GMQueue->GotoPos( GMQueue->FindCallNum( mChar->GetCallNum() ) ) )
			{
				GMQueue->Remove();
				mChar->SetCallNum( 0 );
				s->sysmessage( 1285 );
			}
		}
		else
		{
			if( CounselorQueue->GotoPos( CounselorQueue->FindCallNum( mChar->GetCallNum() ) ) )
			{
				CounselorQueue->Remove();
				mChar->SetCallNum( 0 );
				s->sysmessage( 1286 );
			}
		}
	}
	else
		s->sysmessage( 1287 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::KillSpawn( cSocket *s, SI32 r )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Kill spawns in a spawnregion
//o---------------------------------------------------------------------------o
void cCommands::KillSpawn( cSocket *s, SI32 r )
{
	int killed = 0;

	s->sysmessage( 349 );
	for( CHARACTER i = 0; i < cwmWorldState->GetCharCount(); ++i )
	{
		if( chars[i].GetSpawn( 1 ) < 0x40 && chars[i].GetSpawn( 3 ) == r && chars[i].GetSpawn( 2 ) == 1 ) // spawn2 == 1 if spawned by region
		{
			CChar *iDead = &chars[i];
			Effects->bolteffect( iDead );
			Effects->PlaySound( iDead, 0x0029 );
			Npcs->DeleteChar( iDead );
			++killed;
		}
	}
	doGCollect();
	s->sysmessage( "Done." );
	s->sysmessage( 350, killed, r );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::RegSpawnMax( cSocket *s, cSpawnRegion *spawnReg )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Spawn a region to its maximum capacity
//o---------------------------------------------------------------------------o
void cCommands::RegSpawnMax( cSocket *s, cSpawnRegion *spawnReg )
{
	SI32 spawn = ( spawnReg->GetMaxSpawn() - spawnReg->GetCurrent() );
	if( spawn > 250 )
	{
		s->sysmessage( 351 );
		spawn = 250;
	}
	char temps[MAX_REGIONNAME + 150];
	sprintf( temps, Dictionary->GetEntry( 352 ).c_str(), spawnReg->GetName().c_str(), spawnReg->GetRegionNum(), spawn );
	Speech->sysbroadcast( temps );

	for( SI32 i = 1; i < spawn; ++i )
		spawnReg->doRegionSpawn();
	spawnReg->SetNextTime( BuildTimeValue( static_cast<R32>(60 * RandomNum( spawnReg->GetMinTime(), spawnReg->GetMaxTime() )) ) );

	s->sysmessage( 353, spawn, spawnReg->GetName().c_str(), spawnReg->GetRegionNum() );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::RegSpawnNum( cSocket *s, cSpawnRegion *spawnReg, SI32 n)
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Do a certain number of spawns in a region
//o---------------------------------------------------------------------------o
void cCommands::RegSpawnNum( cSocket *s, cSpawnRegion *spawnReg, SI32 n)
{
	if( n > 250 )
	{
		s->sysmessage( 354 );
		return;
	}
	else
	{
		SI32 spawn = ( spawnReg->GetMaxSpawn() - spawnReg->GetCurrent() );
		char temps[MAX_REGIONNAME + 150];
		if( spawn < n )
			s->sysmessage( 355, n, spawnReg->GetName().c_str(), spawnReg->GetRegionNum(), spawn, spawnReg->GetMaxSpawn() );
		else
			spawn = n;
		sprintf( temps, Dictionary->GetEntry( 356 ).c_str(), spawnReg->GetName().c_str(), spawnReg->GetRegionNum(), spawn);
		Speech->sysbroadcast( temps );
		for( SI32 i = 1; i < spawn; ++i )
			spawnReg->doRegionSpawn();

		spawnReg->SetNextTime( BuildTimeValue(static_cast<R32>( 60 * RandomNum( spawnReg->GetMinTime(), spawnReg->GetMaxTime() ) )) );
		s->sysmessage( 357, spawn, spawnReg->GetName().c_str(), spawnReg->GetRegionNum() );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::KillAll( cSocket *s, SI32 percent, char* sysmsg)
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Kill all NPC's in a world
//o---------------------------------------------------------------------------o
void cCommands::KillAll( cSocket *s, SI32 percent, const char* sysmsg )
{
	s->sysmessage( 358 );
	Speech->sysbroadcast( sysmsg );
	for( CHARACTER i = 0; i < cwmWorldState->GetCharCount(); ++i )
	{
		if( chars[i].IsNpc() )
		{
			if( RandomNum( 0, 99 ) + 1 <= percent )
			{
				CChar *iDead = &chars[i];
				Effects->bolteffect( iDead );
				Effects->PlaySound( iDead, 0x0029 );
				doDeathStuff( iDead );
			}
		}
	}
	s->sysmessage( "Done." );
}

//o---------------------------------------------------------------------------o
//|   Function	  -  void CPage( int s, char *reason )
//|   Date		  -  Unknown
//|   Programmer  -  Unknown  (Touched tabstops by EviLDeD Dec 23, 1998)
//o---------------------------------------------------------------------------o
//|   Purpose     -  Used when player pages a counselor
//o---------------------------------------------------------------------------o
void cCommands::CPage( cSocket *s, const char *reason )
{
	CChar *mChar = s->CurrcharObj();
	bool x = false;
	UI08 a1 = mChar->GetSerial( 1 );
	UI08 a2 = mChar->GetSerial( 2 );
	UI08 a3 = mChar->GetSerial( 3 );
	UI08 a4 = mChar->GetSerial( 4 );
	
	HelpRequest pageToAdd;
	pageToAdd.Reason( reason );
	pageToAdd.WhoPaging( mChar->GetSerial() );
	pageToAdd.IsHandled( false );
	pageToAdd.TimeOfPage( time( NULL ) );

	int callNum = CounselorQueue->Add( &pageToAdd );
	if( callNum != -1 )
	{
		mChar->SetPlayerCallNum( callNum );
		if( !strcmp( reason, "OTHER" ) )
		{
			mChar->SetSpeechMode( 2 );
			s->sysmessage( 359 );
		}
		else
		{
			char temp[1024];
			sprintf( temp, "Counselor Page from %s [%x %x %x %x]: %s", mChar->GetName().c_str(), a1, a2, a3, a4, reason );
			Network->PushConn();
			for( cSocket *iSock = Network->FirstSocket(); !Network->FinishedSockets(); iSock = Network->NextSocket() )
			{
				CChar *iChar = iSock->CurrcharObj();
				if( iChar->IsGMPageable() || iChar->IsCounselor() )
				{
					x = true;
					iSock->sysmessage( temp );
				}
			}
			Network->PopConn();
			if( x )
				s->sysmessage( 360 );
			else
				s->sysmessage( 361 );
		}
	}
}

//o---------------------------------------------------------------------------o
//|   Function	  :  void GMPage( cSocket *s, char *reason )
//|   Date		  :  Unknown
//|   Programmer  :  Unknown
//o---------------------------------------------------------------------------o
//|   Purpose     :  Used when player calls a GM
//o---------------------------------------------------------------------------o
void cCommands::GMPage( cSocket *s, const char *reason )
{
	bool x = false;
	CChar *mChar = s->CurrcharObj();
	UI08 a1 = mChar->GetSerial( 1 );
	UI08 a2 = mChar->GetSerial( 2 );
	UI08 a3 = mChar->GetSerial( 3 );
	UI08 a4 = mChar->GetSerial( 4 );
	
	HelpRequest pageToAdd;
	pageToAdd.Reason( reason );
	pageToAdd.WhoPaging( mChar->GetSerial() );
	pageToAdd.IsHandled( false );
	pageToAdd.TimeOfPage( time( NULL ) );
	int callNum = GMQueue->Add( &pageToAdd );
	if( callNum != -1 )
	{
		mChar->SetPlayerCallNum( callNum );
		if( !strcmp( reason, "OTHER" ) )
		{
			mChar->SetSpeechMode( 1 );
			s->sysmessage( 362 );
		}
		else
		{
			char temp[1024];
			sprintf( temp, "Page from %s [%x %x %x %x]: %s", mChar->GetName().c_str(), a1, a2, a3, a4, reason );
			Network->PushConn();
			for( cSocket *iSock = Network->FirstSocket(); !Network->FinishedSockets(); iSock = Network->NextSocket() )
			{
				CChar *iChar = iSock->CurrcharObj();
				if( iChar == NULL )
					continue;
				if( iChar->IsGMPageable() )
				{
					x = true;
					iSock->sysmessage( temp );
				}
			}
			Network->PopConn();
			if( x )
				s->sysmessage( 363 );
			else
				s->sysmessage( 364 );
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::DyeItem( cSocket *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Change hue of the target
//o---------------------------------------------------------------------------o
void cCommands::DyeTarget( cSocket *s )
{
	CItem *i = calcItemObjFromSer( s->GetDWord( 1 ) );
	if( i != NULL )
	{
		UI16 colour = s->GetWord( 7 );
           
		if( !s->DyeAll() )
		{
			if( colour < 0x0002 || colour > 0x03E9 )
				colour = 0x03E9;
		}
	
		i->SetColour( colour );
		
		Effects->PlaySound( s, 0x023E, true );
		return;
	}

	CChar *mChar = s->CurrcharObj();
	CChar *c = calcCharObjFromSer( s->GetDWord( 1 ) );
	if( c != NULL )
	{
		if( !mChar->IsGM() ) 
			return;

		UI16 k = s->GetWord( 7 );
		UI16 body = c->GetID();
		SI32 b = k&0x4000; 

		if( ( ( k>>8 ) < 0x80 ) && body >= 0x0190 && body <= 0x0193 ) 
			k += 0x8000;

		if( b == 16384 && (body >= 0x0190 && body <= 0x03e1 ) ) 
			k = 0xF000; // but assigning the only "transparent" value that works, namly semi-trasnparency.

		if( k != 0x8000 )
		{
			c->SetSkin( k );
			c->SetOrgSkin( k );
			c->Teleport();
		} 
	}
	Effects->PlaySound( s, 0x023E, true );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::AddHere( cSocket *s, SI08 z )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Add an item at players location
//o---------------------------------------------------------------------------o
void cCommands::AddHere( cSocket *s, SI08 z )
{
	CTile tile;
	UI16 itemID = ((s->AddID1())<<8) + s->AddID2();
	
	Map->SeekTile( itemID, &tile );
	bool pileable = tile.Stackable();
	CChar *mChar = s->CurrcharObj();
	CItem *c = Items->SpawnItem( NULL, mChar, 1, "#", pileable, itemID, 0, false, false );
	if( c != NULL )
	{
		c->SetLocation( mChar->GetX(), mChar->GetY(), z );
		c->SetDoorDir( 0 );
		c->SetPriv( 0 );
	}
	s->AddID1( 0 );
	s->AddID2( 0 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::MakePlace( cSocket *s, SI32 i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Change teleport location to x,y,z
//o---------------------------------------------------------------------------o
void cCommands::MakePlace( cSocket *s, SI32 i )
{
	SI16 x = 0, y = 0;
	SI08 z = 0;
	char temp[1024];
	sprintf( temp, "LOCATION %i", i );
	ScriptSection *Location = FileLookup->FindEntry( temp, location_def );
	if( Location == NULL )
		return;
	for( UString tag = Location->First(); !Location->AtEnd(); tag = Location->Next() )
	{
		UString data = Location->GrabData();
		if( tag.upper() == "X" )
			x = data.toShort();
		else if( tag.upper() == "Y" )
			y = data.toShort();
		else if( tag.upper() == "Z" )
			z = data.toByte();
	}
	s->AddX( x );
	s->AddY( y );
	s->AddZ( z );
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *cCommands::DupeItem( cSocket *s, CItem *i, UI32 amount )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Dupe selected item
//o---------------------------------------------------------------------------o
CItem *cCommands::DupeItem( cSocket *s, CItem *i, UI32 amount )
{
	CChar *mChar = s->CurrcharObj();
	CItem *p = getRootPack( i ), *cp = mChar->GetPackItem();

	if( ( p == NULL && cp == NULL ) || mChar == NULL || i->isCorpse() )
		return NULL;
	
	CItem *c = i->Dupe();
	if( c == NULL )
		return NULL;

	c->IncX(2);
	c->IncY(2);
	c->IncZ(1);

	if( p == NULL && cp != NULL )
		c->SetCont( cp );

	if( amount > MAX_STACK )
	{
		amount -= MAX_STACK;
		DupeItem( s, i, amount );
	}
	c->SetAmount( amount );
	if( c->GetSpawnObj() != NULL )
		nspawnsp.AddSerial( c->GetSpawn(), calcItemFromSer( c->GetSerial() ) );

	return c;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::Wipe( cSocket *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Wipes all items in a world
//o---------------------------------------------------------------------------o
void cCommands::Wipe( cSocket *s )
{
	CChar *mChar = s->CurrcharObj();
	Console << mChar->GetName() << " has initiated an item wipe" << myendl;
	
	for( ITEM k = 0; k <= cwmWorldState->GetItemCount(); ++k )
	{
		if( items[k].GetCont() == NULL && !items[k].isWipeable() )
			Items->DeleItem( &items[k] );
	}
	Speech->sysbroadcast( Dictionary->GetEntry( 365 ) ); 
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::Load( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Load the command table
//o---------------------------------------------------------------------------o
void cCommands::Load( void )
{
	SI32 tablePos;
	SI16 commandCount = 0;
	ScriptSection *commands = FileLookup->FindEntry( "COMMAND_OVERRIDE", command_def );
	if( commands == NULL )
	{
		InitClearance();
		return;
	}

	UString tag;
	UString data;

	STRINGLIST	badCommands;
	for( tag = commands->First(); !commands->AtEnd(); tag = commands->Next() )
	{
		data = commands->GrabData();
		CmdTableIterator toFind = cmd_table.find( tag );
		TargetTableIterator findTarg = targ_table.find( tag );
		if( toFind == cmd_table.end() && findTarg == targ_table.end() )
			badCommands.push_back( tag ); // make sure we don't index into array at -1
		else
		{
			++commandCount;
			if( toFind != cmd_table.end() )
				toFind->second.cmdLevelReq		= data.toUByte();
			else if( findTarg != targ_table.end() )
				findTarg->second.cmdLevelReq	= data.toUByte();
//			if( commandCount % 10 == 0 )
//				Console << ".";
		}
		// check for commands here
	}
	if( badCommands.size() > 0 )
	{
		Console << myendl;
		for( tablePos = 0; static_cast<unsigned int>(tablePos) < badCommands.size(); ++tablePos )
			Console << "Invalid command '" << badCommands[tablePos].c_str() << "' found in commands.dfn!" << myendl;
	}

	ScriptSection *cmdClearance = FileLookup->FindEntry( "COMMANDLEVELS", command_def );
	if( cmdClearance == NULL )
	{
		InitClearance();
		return;
	}

	size_t currentWorking;
	for( tag = cmdClearance->First(); !cmdClearance->AtEnd(); tag = cmdClearance->Next() )
	{
		data			= cmdClearance->GrabData();
		currentWorking	= clearance.size();
		clearance.push_back( new commandLevel_st );
		strcpy( clearance[currentWorking]->name, tag.c_str() );
		clearance[currentWorking]->commandLevel = data.toUByte();
	}
	for( UI32 tempCounter = 0; tempCounter < clearance.size(); ++tempCounter )
	{
		cmdClearance = FileLookup->FindEntry( clearance[tempCounter]->name, command_def );
		if( cmdClearance == NULL )
			continue;
		for( tag = cmdClearance->First(); !cmdClearance->AtEnd(); tag = cmdClearance->Next() )
		{
			data = cmdClearance->GrabData();
			if( tag.upper() == "NICKCOLOUR" )
				clearance[tempCounter]->nickColour = data.toUShort();
			else if( tag.upper() == "DEFAULTPRIV" )
				clearance[tempCounter]->defaultPriv = data.toUShort();
			else if( tag.upper() == "BODYID" )
				clearance[tempCounter]->targBody = data.toUShort();
			else if( tag.upper() == "ALLSKILL" )
				clearance[tempCounter]->allSkillVals = data.toUShort();
			else if( tag.upper() == "BODYCOLOUR" )
				clearance[tempCounter]->bodyColour = data.toUShort();
			else if( tag.upper() == "STRIPOFF" )
				clearance[tempCounter]->stripOff = ( data.toUShort() != 0 );
			else
				Console << myendl << "Unknown tag in " << clearance[tempCounter]->name << ": " << tag << " with data of " << data << myendl;
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::Log( std::string command, CChar *player1, CChar *player2, std::string extraInfo )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Writes toLog to a file
//o---------------------------------------------------------------------------o
void cCommands::Log( std::string command, CChar *player1, CChar *player2, std::string extraInfo )
{
	std::string logName	= cwmWorldState->ServerData()->Directory( CSDDP_LOGS ) + "command.log";
	FILE *commandLog	= fopen( logName.c_str(), "a+" );
	if( commandLog == NULL )
	{
		Console.Error( 1, "Unable to open command log file %s!", logName.c_str() );
		return;
	}
	char dateTime[1024];
	RealTime( dateTime );

	if( player2 != NULL )
		fprintf( commandLog, "[%s] %s (serial: %X ) used command <%s> on player %s (serial: %X ) Extra Info: %s\n", dateTime, player1->GetName().c_str(), player1->GetSerial(), command.c_str(), player2->GetName().c_str(), player2->GetSerial(), extraInfo.c_str() );
	else
		fprintf( commandLog, "[%s] %s (serial: %X ) used command <%s> Extra Info: %s\n", dateTime, player1->GetName().c_str(), player1->GetSerial(), command.c_str(), extraInfo.c_str() );
	fclose( commandLog );
}

//o---------------------------------------------------------------------------o
//|	Function	-	commandLevel_st *cCommands::GetClearance( const char *clearName )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get the command level of a character
//o---------------------------------------------------------------------------o
commandLevel_st *cCommands::GetClearance( const char *clearName )
{
	size_t clearanceSize = clearance.size();
	if( clearanceSize == 0 )
		return NULL;
	char *nameBackup = new char[strlen( clearName ) + 1];
	strcpy( nameBackup, clearName );
	nameBackup = strupr( nameBackup );
	for( UI32 counter = 0; counter < clearanceSize; ++counter )
	{
		if( !strcmp( nameBackup, clearance[counter]->name ) )
		{
			delete [] nameBackup;
			return clearance[counter];
		}
	}
	delete [] nameBackup;
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	UI16 cCommands::GetColourByLevel( UI08 commandLevel )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get a players nick color based on his command level
//o---------------------------------------------------------------------------o
UI16 cCommands::GetColourByLevel( UI08 commandLevel )
{
	size_t clearanceSize = clearance.size();
	if( clearanceSize == 0 )
		return 0x005A;
	if( commandLevel > clearance[0]->commandLevel )
		return clearance[0]->nickColour;

	for( size_t counter = 0; counter < ( clearanceSize - 1 ); ++counter )
	{
		if( commandLevel <= clearance[counter]->commandLevel && commandLevel > clearance[counter+1]->commandLevel )
			return clearance[counter]->nickColour;
	}
	return clearance[clearanceSize - 1]->nickColour;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCommands::InitClearance( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Initialize command levels
//o---------------------------------------------------------------------------o
void cCommands::InitClearance( void )
{
	clearance.push_back( new commandLevel_st );	// 0 -> 2
	clearance.push_back( new commandLevel_st );
	clearance.push_back( new commandLevel_st );

	strcpy( clearance[0]->name, "GM" );
	strcpy( clearance[1]->name, "COUNSELOR" );
	strcpy( clearance[2]->name, "PLAYER" );

	clearance[0]->commandLevel = 2;
	clearance[1]->commandLevel = 1;
	clearance[2]->commandLevel = 0;

	clearance[0]->targBody = 0x03DB;
	clearance[1]->targBody = 0x03DB;
	clearance[2]->targBody = 0;

	clearance[0]->bodyColour = 0x8021;
	clearance[1]->bodyColour = 0x8002;

	clearance[0]->defaultPriv = 0xFD;
	clearance[1]->defaultPriv = 0x8DB6;
	clearance[2]->defaultPriv = 0;

	clearance[0]->nickColour = 0x03;
	clearance[1]->nickColour = 0x03;
	clearance[2]->nickColour = 0x005A;

	clearance[0]->allSkillVals = 1000;
	clearance[1]->allSkillVals = 0;
	clearance[2]->allSkillVals = 0;
}

//o---------------------------------------------------------------------------o
//|	Function	-	commandLevel_st *cCommands::GetClearance( UI08 commandLevel )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Geta characters command level
//o---------------------------------------------------------------------------o
commandLevel_st *cCommands::GetClearance( UI08 commandLevel )
{
	size_t clearanceSize = clearance.size();
	if( clearanceSize == 0 )
		return NULL;
	if( commandLevel > clearance[0]->commandLevel )
		return clearance[0];

	for( size_t counter = 0; counter < ( clearanceSize - 1 ); ++counter )
	{
		if( commandLevel <= clearance[counter]->commandLevel && commandLevel > clearance[counter+1]->commandLevel )
			return clearance[counter];
	}
	return clearance[clearanceSize - 1];
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool cCommands::CommandExists( const char *cmdName )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check if a command is valid
//o---------------------------------------------------------------------------o
bool cCommands::CommandExists( const char *cmdName )
{
	CmdTableIterator toFind = cmd_table.find( cmdName );
	return ( toFind != cmd_table.end() );
}

//o---------------------------------------------------------------------------o
//|	Function	-	const char * cCommands::FirstCommand( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get first command in cmd_table
//o---------------------------------------------------------------------------o
const char * cCommands::FirstCommand( void )
{
	cmdPointer = cmd_table.begin();
	if( FinishedCommandList() )
		return NULL;
	return cmdPointer->first.c_str();
}

//o---------------------------------------------------------------------------o
//|	Function	-	const char * cCommands::NextCommand( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get next command in cmd_table
//o---------------------------------------------------------------------------o
const char * cCommands::NextCommand( void )
{
	if( FinishedCommandList() )
		return NULL;
	++cmdPointer;
	if( FinishedCommandList() )
		return NULL;
	return cmdPointer->first.c_str();
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool cCommands::FinishedCommandList( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get end of cmd_table
//o---------------------------------------------------------------------------o
bool cCommands::FinishedCommandList( void )
{
	return ( cmdPointer == cmd_table.end() );
}

//o---------------------------------------------------------------------------o
//|	Function	-	cmdtable_mapentry *cCommands::CommandDetails( const char *cmdName )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get command info
//o---------------------------------------------------------------------------o
cmdtable_mapentry *cCommands::CommandDetails( const char *cmdName )
{
	if( !CommandExists( cmdName ) )
		return NULL;
	CmdTableIterator toFind = cmd_table.find( cmdName );
	return &(toFind->second);
}
