// All funcs in this file are used for item/char distance related situations
// There's a chance that a number of these could become inline
#include "uox3.h"
#include "cRaces.h"
#include "regions.h"

//o---------------------------------------------------------------------------o
//|	Function	-	bool checkItemRange( CChar *mChar, CItem *i, UI16 distance )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Checks if an item is within a certain distance (based on
//|					its container and location)
//o---------------------------------------------------------------------------o
bool checkItemRange( CChar *mChar, CItem *i, UI16 distance )
{
	CChar *itemOwner = NULL;

	bool checkRange = false;

	if( i->GetCont() != NULL ) // It's inside another container, we need root container to calculate distance
	{
		itemOwner = getPackOwner( i );
		if( itemOwner == NULL ) // In a pack on the ground!
		{
			CItem *rootContainer = getRootPack( i );
			if( rootContainer == NULL )
				Console.Error( 2, "Pack with serial %li has an invalid container chain!", i->GetSerial() );
			else
				checkRange = objInRange( mChar, rootContainer, distance );
		}
	}
	else  // It's on the ground
		checkRange = objInRange( mChar, i, distance );

	bool isGM = mChar->IsGM();
	bool isCounselor = mChar->IsCounselor();
	bool isPackOwner = ( itemOwner == mChar );
	bool isPetOwner = ( itemOwner != NULL && itemOwner != mChar && (CChar *)(itemOwner->GetOwnerObj()) == mChar );

	if( isGM || isCounselor || isPackOwner || isPetOwner )
		return true;
	else
		return checkRange;
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool objInRange( cSocket *mSock, cBaseObject *obj, UI16 distance )
//|	Date		-	2/12/2003
//| Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check if BaseObject obj is within a certain distance
//o---------------------------------------------------------------------------o
bool objInRange( cSocket *mSock, cBaseObject *obj, UI16 distance )
{
	CChar *mChar = mSock->CurrcharObj();
	if( mChar == NULL )
		return false;
	if( mChar == obj )
		return true;
	if( mChar->WorldNumber() != obj->WorldNumber() )
		return false;
	point3 difference = mChar->GetLocation() - obj->GetLocation();
	return ( difference.MagSquared() <= ( distance * distance ) );
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool objInRange( cBaseObject *a, cBaseObject *b, UI16 distance )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check if an object is within a certain distance of another
//|					object
//o---------------------------------------------------------------------------o
bool objInRange( cBaseObject *a, cBaseObject *b, UI16 distance )
{
	if( a == b )
		return true;
	if( a->WorldNumber() != b->WorldNumber() )
		return false;
	point3 difference = a->GetLocation() - b->GetLocation();
	return ( difference.MagSquared() <= ( distance * distance ) );
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool itemInRange( cSocket *s, CItem *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check if item i in visual range
//o---------------------------------------------------------------------------o
bool itemInRange( CChar *mChar, CItem *i )
{
	if( mChar == NULL || i == NULL )
		return false;
	SI08 vr;
	if( i->GetID( 1 ) >= 0x40 ) 
		vr = DIST_BUILDRANGE;
	else
		vr = MAX_VISRANGE + Races->VisRange( mChar->GetRace() );
	point3 difference = mChar->GetLocation() - i->GetLocation();
	return ( difference.MagSquared() <= ( vr * vr ) );
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool charInRange( CChar *a, CChar *b )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check if characters a and b are in visual range
//o---------------------------------------------------------------------------o
bool charInRange( CChar *a, CChar *b )
{
	if( a == NULL || b == NULL )
		return false;
	if( a == b )
		return true;
	if( a->WorldNumber() != b->WorldNumber() )
		return false;
	SI16 visRange		= MAX_VISRANGE + Races->VisRange( a->GetRace() );
	point3 difference	= a->GetLocation() - b->GetLocation();
	return ( difference.MagSquared() <= ( visRange * visRange ) );
}

//o---------------------------------------------------------------------------o
//|	Function	-	UI16 getDist( cBaseObject *a, cBaseObject *b )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get the distance between two objects
//o---------------------------------------------------------------------------o
UI16 getDist( cBaseObject *a, cBaseObject *b )
{
	if( a == NULL || b == NULL ) 
		return 0xFFFF;
	if( a->WorldNumber() != b->WorldNumber() )
		return 0xFFFF;
	point3 difference = a->GetLocation() - b->GetLocation();
	return (SI16)difference.Mag();
}

