#include "uox3.h"
#include "regions.h"

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *FindItem( CChar *toFind, UI08 type )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Look for items of a certain type in character's pack
//o---------------------------------------------------------------------------o
CItem *FindItemOfType( CChar *toFind, UI08 type )
{
	for( CItem *toCheck = toFind->FirstItem(); !toFind->FinishedItems(); toCheck = toFind->NextItem() )
	{
		if( toCheck != NULL )
		{
			if( toCheck->GetType() == type )	// it's in our hand
				return toCheck;					// we've found the first occurance on the person!
			else if( toCheck->GetLayer() == 0x15 )	// could use packitem, but we're already in the same type of loop, so we'll check it ourselves
			{
				CItem *packSearchResult = SearchSubPackForItemOfType( toCheck, type );
				if( packSearchResult != NULL )
					return packSearchResult;
			}
		}
	}
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *SearchSubPackForItem( CItem *toSearch, UI08 type )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Search character's subpacks for items of specific type
//o---------------------------------------------------------------------------o
CItem *SearchSubPackForItemOfType( CItem *toSearch, UI08 type )
{
	for( CItem *toCheck = toSearch->FirstItem(); !toSearch->FinishedItems(); toCheck = toSearch->NextItem() )
	{
		if( toCheck != NULL )
		{
			if( toCheck->GetType() == type )	// it's in our hand
				return toCheck;					// we've found the first occurance on the person!
			else if( toCheck->GetType() == 1 || toCheck->GetType() == 8 )	// search any subpacks, specifically pack and locked containers
			{ 
				CItem *packSearchResult = SearchSubPackForItem( toCheck, type );
				if( packSearchResult != NULL )
					return packSearchResult;
			}
		}
	}
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *FindItem( CChar *toFind, UI16 itemID )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Look for items of a certain ID in character's pack
//o---------------------------------------------------------------------------o
CItem *FindItem( CChar *toFind, UI16 itemID )
{
	for( CItem *toCheck = toFind->FirstItem(); !toFind->FinishedItems(); toCheck = toFind->NextItem() )
	{
		if( toCheck != NULL )
		{
			if( toCheck->GetID() == itemID )	// it's in our hand
				return toCheck;					// we've found the first occurance on the person!
			else if( toCheck->GetLayer() == 0x15 )	// could use packitem, but we're already in the same type of loop, so we'll check it ourselves
			{
				CItem *packSearchResult = SearchSubPackForItem( toCheck, itemID );
				if( packSearchResult != NULL )
					return packSearchResult;
			}
		}
	}
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	ITEM SearchSubPackForItem( CItem *toSearch, UI16 itemID )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Search character's subpacks for items of specific ID
//o---------------------------------------------------------------------------o
CItem *SearchSubPackForItem( CItem *toSearch, UI16 itemID )
{
	for( CItem *toCheck = toSearch->FirstItem(); !toSearch->FinishedItems(); toCheck = toSearch->NextItem() )
	{
		if( toCheck != NULL )
		{
			if( toCheck->GetID() == itemID )	// it's in our hand
				return toCheck;					// we've found the first occurance on the person!
			else if( toCheck->GetType() == 1 || toCheck->GetType() == 8 )	// search any subpacks, specifically pack and locked containers
			{ 
				CItem *packSearchResult = SearchSubPackForItem( toCheck, itemID );
				if( packSearchResult != NULL )
					return packSearchResult;
			}
		}
	}
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool inMulti( SI16 x, SI16 y, SI08 z, CItem *m )
//|	Programmer	-	Zippy
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check if item is in a multi
//o---------------------------------------------------------------------------o
bool inMulti( SI16 x, SI16 y, SI08 z, CItem *m )
{
	if( m == NULL )
		return false;
	SI32 length;
	st_multi *multi = NULL;
	UI16 multiID = (UI16)(( m->GetID() ) - 0x4000);
	Map->SeekMulti( multiID, &length );

	if( length == -1 || length >= 17000000)
	{
		Console << "inmulti() - Bad length in multi file, avoiding stall. Item Name: " << m->GetName() << " " << m->GetSerial() << myendl;
		length = 0;
	}
	for( SI32 j = 0; j < length; ++j )
	{
		multi = Map->SeekIntoMulti( multiID, j );

		if( multi->visible && ( m->GetX() + multi->x == x ) && ( m->GetY() + multi->y == y ) && ( m->GetZ() + multi->z == z ) )
			return true;
	}
	return false;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CMultiObj *findMulti( SI16 x, SI16 y, SI08 z, UI08 worldNumber )
//|	Programmer	-	Zippy
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find a multi at x,y,z
//o---------------------------------------------------------------------------o
CMultiObj *findMulti( cBaseObject *i )
{
	if( i == NULL )
		return NULL;
	return findMulti( i->GetX(), i->GetY(), i->GetZ(), i->WorldNumber() );
}
CMultiObj *findMulti( SI16 x, SI16 y, SI08 z, UI08 worldNumber )
{
	SI32 lastdist = 30;
	CMultiObj *multi = NULL;
	SI32 ret, dx, dy;

	SI16 xOffset = MapRegion->GetGridX( x );
	SI16 yOffset = MapRegion->GetGridY( y );
	for( SI08 counter1 = -1; counter1 <= 1; ++counter1 )
	{
		for( SI08 counter2 = -1; counter2 <= 1; ++counter2 )
		{
			SubRegion *toCheck = MapRegion->GetGrid( xOffset + counter1, yOffset + counter2, worldNumber );
			if( toCheck == NULL )
				continue;
			toCheck->PushItem();
			// Handle the STL here
			ITEMLIST_ITERATOR I;
			//for( CItem *itemCheck = toCheck->FirstItem(); !toCheck->FinishedItems(); itemCheck = toCheck->GetNextItem() )
			for( I=toCheck->itemData.begin(); I!=toCheck->itemData.end(); ++I )
			{
				CItem *itemCheck;
				itemCheck=(*I);
				if( itemCheck == NULL || I == toCheck->itemData.end())
					continue;
				if( itemCheck->GetID( 1 ) >= 0x40 )
				{
					dx = abs( x - itemCheck->GetX() );
					dy = abs( y - itemCheck->GetY() );
					ret = (SI32)( hypot( dx, dy ) );
					if( ret <= lastdist )
					{
						lastdist = ret;
						if( inMulti( x, y, z, itemCheck ) )
						{
							multi = static_cast<CMultiObj *>(itemCheck);
							toCheck->PopItem();
							return multi;
						}
					}
				}
			}
			toCheck->PopItem();
		}
	}
	return multi;
}

