// House code for deed creation by Tal Strake, revised by Cironian

#include "uox3.h"
#include "boats.h"
#include "movement.h"
#include "cServerDefinitions.h"
#include "ssection.h"
#include "packets.h"
#include "classes.h"
#include "cSpeech.h"
#include "regions.h"
#include "targeting.h"

#undef DBGFILE
#define DBGFILE "house.cpp"

//o---------------------------------------------------------------------------o
//|   Function    -  void buildHouse( cSocket *s, UI08 i )
//|   Date        -  UnKnown - Rewrite Date 1/24/99
//|   Programmer  -  UnKnown - Rewrite by Zippy (onlynow@earthlink.net)
//o---------------------------------------------------------------------------o
//|   Purpose     -  Triggered by double clicking a deed-> the deed's moreX is read
//|						for the house section in house.scp. Extra items can be added
//|						using HOUSE ITEM, (this includes all doors!) and locked "LOCK"
//|						Space around the house with SPACEX/Y and CHAR offset CHARX/Y/Z
//o---------------------------------------------------------------------------o
void buildHouse( cSocket *s, UI08 i )
{
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;

	SI16 sx = 0, sy = 0, cx = 0, cy = 0;
	SI08 cz = 8;
	std::vector< const char * > houseItems;
	char sect[512];
	bool itemsWillDecay = false;
	static bool shouldBuild = false;
	bool boat = false;
	UString hdeed;
	UString tag;
	UString data;
	UI16 houseID = 0;
	houseItems.resize( 0 );
	if( i )
	{
		sprintf( sect, "HOUSE %u", i );
		ScriptSection *House = FileLookup->FindEntry( sect, house_def );
		if( House == NULL )
			return;
		for( tag = House->First(); !House->AtEnd(); tag = House->Next() )
		{
			data = House->GrabData();
			if( tag.upper() == "ID" )
				houseID = data.toUShort();
			else if( tag.upper() == "SPACEX" )
				sx = static_cast<SI16>(data.toShort() + 1);
			else if( tag.upper() == "SPACEY" )
				sy = static_cast<SI16>(data.toShort() + 1);
			else if( tag.upper() == "CHARX" )
				cx = data.toShort();
			else if( tag.upper() == "CHARY" )
				cy = data.toShort();
			else if( tag.upper() == "CHARZ" )
				cz = data.toByte();
			else if( tag.upper() == "ITEMSDECAY" )
				itemsWillDecay = ( data.toShort() == 1 );
			else if( tag.upper() == "HOUSE_ITEM" )
				houseItems.push_back( data.c_str() );
			else if( tag.upper() == "HOUSE_DEED" )
				hdeed = data;
			else if( tag.upper() == "BOAT" ) 
				boat = true;	//Boats
		}
		if( !houseID )
		{
			Console.Error( 1, "Bad house script # %i!\n",i );
			return;
		}
	}
	if( !shouldBuild )
	{
		if( i )
		{
			s->AddID1( 0x40 );
			s->AddID2( 100 );
			if( houseID >= 0x4000 )
				s->mtarget( calcserial( 0, 1, 0, 0 ), houseID - 0x4000, Dictionary->GetEntry( 576, s->Language() ).c_str() );
			else
				s->target( 0, TARGET_ADD, 576 );
		}
		else
			s->mtarget( 0, 1, 0, 0, s->AddID1() - 0x40, s->AddID2(), Dictionary->GetEntry( 576, s->Language() ).c_str() );

		shouldBuild = true;//for when we come back after they target something
		return;
	}

	CChar *mChar = s->CurrcharObj();
	if( mChar == NULL )
		return;

	UI08 worldNumber = mChar->WorldNumber();
	shouldBuild = false;
	SI16 x = s->GetWord( 11 );
	SI16 y = s->GetWord( 13 );
	SI08 z = static_cast<SI08>(s->GetByte( 16 ) + Map->TileHeight( s->GetWord( 17 ) ));
	if( !mChar->IsGM() )
	{
		if( x > 6200 || y > 4200 )
		{
			s->sysmessage( 577 );
			return;
		}

		for( int k = -sx; k < sx; ++k ) //check the SPACEX and SPACEY to make sure they are valid locations....
		{
			for( int l = -sy; l < sy; ++l )
			{
				if( ( !Movement->validNPCMove( x+k, y+l, z, mChar ) ) &&
					( ( mChar->GetX() != x+k ) && ( mChar->GetY() != y+l ) ) )
//				This will take the char making the house out of the space check, be careful 
//				you don't build a house on top of your self..... this had to be done So you 
//				could extra space around houses, (12+) and they would still be buildable.
				{
					s->sysmessage( 577 );
					return;
				}
			}
		}
	}
	char temp[1024];
	CMultiObj *house = NULL;
	CItem *fakeHouse = NULL;
	if( houseID >= 0x4000 )
	{
		if( (houseID%256) >= 18 ) 
			sprintf( temp, "%s's house", mChar->GetName().c_str() ); //This will make the little deed item you see when you have showhs on say the person's name, thought it might be helpful for GMs.
		else 
			strcpy( temp, "a mast" );
		house = Items->SpawnMulti( mChar, temp, houseID );
		if( house == NULL )
			return;

		house->SetLocation( x, y, z );
		house->SetPriv( 0 );
		house->SetDecayable( itemsWillDecay );
		house->SetDeed( hdeed.c_str() ); // crackerjack 8/9/99 - for converting back *into* deeds
		house->SetOwner( mChar );
	}
	else
	{
		strcpy( temp, "#" );
		fakeHouse = Items->SpawnItem( s, mChar, 1, temp, false, houseID, 0, false, true );
		if( fakeHouse == NULL )
			return;
		fakeHouse->SetLocation( x, y, z );
		fakeHouse->SetPriv( 0 );
		fakeHouse->SetOwner( (cBaseObject *)mChar );
		fakeHouse->SetDecayable( false );
	}

	if( houseItems.size() == 0 && !boat )
	{
		mChar->Teleport();
		return;//If there's no extra items, we don't really need a key, or anything else do we? ;-)
	}

	if( boat ) //Boats
	{
		if( !Boats->CreateBoat( s, house, (houseID%256), i ) )
		{
			Items->DeleItem( house );
			return;
		} 
	}

	if( i )//Boats->.. Moved from up there ^
		Items->DeleItem( calcItemObjFromSer( mChar->GetSpeechItem() ) );

	mChar->SetSpeechItem( INVALIDSERIAL );

	//Key...
	CItem *key = NULL;
	if( houseID >= 0x4000 )
	{
		if( (houseID%256) >= 112 && (houseID%256) <= 115 ) 
			key = Items->SpawnItem( s, mChar, 1, "a tent key", false, 0x1010, 0, true, true );//iron key for tents
		else if( (houseID%256) <= 0x18 ) 
			key = Items->SpawnItem( s, mChar,1,"a ship key", false, 0x1013, 0, true, true );//Boats -Rusty Iron Key
		else 
			key = Items->SpawnItem( s, mChar, 1, "a house key", false, 0x100F, 0, true, true );//gold key for everything else

		if( key != NULL )
		{
			key->SetMore( house->GetSerial() );
			key->SetType( 7 );
		}
	}
	ScriptSection *HouseItem = NULL;
	CItem *hItem = NULL;
	for( UI32 iCounter = 0; iCounter < houseItems.size(); ++iCounter )//Loop through the HOUSE_ITEMs
	{
		sprintf( sect, "HOUSE ITEM %s", houseItems[iCounter] );
		HouseItem = FileLookup->FindEntry( sect, house_def );
		if( HouseItem != NULL )
		{
			hItem = NULL;	// clear it out
			for( tag = HouseItem->First(); !HouseItem->AtEnd(); tag = HouseItem->Next() )
			{
				data = HouseItem->GrabData();
				if( tag.upper() == "ITEM" )
				{
					hItem = Items->CreateScriptItem( s, data.c_str(), false, worldNumber );
					if( hItem == NULL )
					{
						Console << "Error in house creation, item " << data << " could not be made" << myendl;
						break;
					}
					else
					{
						hItem->SetMovable( 2 );//Non-Moveable by default
						hItem->SetPriv( 0 );//since even things in houses decay, no-decay by default
						hItem->SetLocation( house );
						hItem->SetOwner( (cBaseObject *)mChar );
						if( houseID >= 0x4000 )
							hItem->SetMulti( house );
					}
				}
				else if( tag.upper() == "DECAY" )
					hItem->SetDecayable( true );
				else if( tag.upper() == "NODECAY" )
					hItem->SetDecayable( false );
				else if( tag.upper() == "PACK" )//put the item in the Builder's Backpack
				{
					CItem *pack = getPack( mChar );
					hItem->SetCont( pack );
					hItem->SetX( RandomNum( 31, 120 ) );
					hItem->SetY( RandomNum( 31, 120 ) );
					hItem->SetZ( 9 );
				}
				else if( tag.upper() == "MOVEABLE" )
					hItem->SetMovable( 1 );
				else if( tag.upper() == "LOCK" && houseID >= 0x4000 )//lock it with the house key
					hItem->SetMore( house->GetSerial() );
				else if( tag.upper() == "X" )//offset + or - from the center of the house:
					hItem->SetX( static_cast<SI16>(x + data.toShort()) );
				else if( tag.upper() == "Y" )
					hItem->SetY( static_cast<SI16>(y + data.toShort()) );
				else if( tag.upper() == "Z" )
					hItem->SetZ( static_cast<SI08>(z + data.toShort()) );
			}
			if( hItem != NULL && hItem->GetCont() == NULL )
				MapRegion->AddItem( hItem );
		}
	}
	// map regions not done here... hmmm... not good!!!  Not a safe assumption we're in same region
	mChar->SetLocation( x + cx, y + cy, z + cz );
	mChar->Teleport();
}

// turn a house into a deed if possible. - crackerjack 8/9/99
// s = socket of player
// i = house item # in items[]
// morex on the house item must be set to deed item # in items.scp.
void deedHouse( cSocket *s, CMultiObj *i )
{
	if( i == NULL ) 
		return;

	CChar *mChar = s->CurrcharObj();

	if( i->GetOwnerObj() == mChar || mChar->IsGM() )
	{
		CItem *pvDeed = NULL;
		char temp[1024];

		CItem *ii = Items->SpawnItemToPack( s, mChar, i->GetDeed(), false );	// need to make before delete
		if( ii == NULL ) 
			return;
        
		s->sysmessage( 578, i->GetName().c_str() );
		s->sysmessage( 579, ii->GetName().c_str() );

		SI16 xOffset = MapRegion->GetGridX( i->GetX() );
		SI16 yOffset = MapRegion->GetGridY( i->GetY() );
		UI08 worldNumber = i->WorldNumber();
		for( SI08 counter1 = -1; counter1 <= 1; ++counter1 )
		{
			for( SI08 counter2 = -1; counter2 <= 1; ++counter2 )
			{
				SubRegion *toCheck = MapRegion->GetGrid( xOffset + counter1, yOffset + counter2, worldNumber );
				if( toCheck == NULL )
					continue;

				toCheck->PushItem();
				// Do the STL stuff instead
				ITEMLIST_ITERATOR I;
				//for( CItem *itemCheck = toCheck->FirstItem(); !toCheck->FinishedItems(); itemCheck = toCheck->GetNextItem() )
				for( I = toCheck->itemData.begin();I!=toCheck->itemData.end();++I)
				{
					CItem *itemCheck;
					itemCheck=(*I);
					if( itemCheck == NULL || I == toCheck->itemData.end() )		
						continue;
					if( itemCheck->GetType() == 203 || "Brass sign" == itemCheck->GetName() )
					{
						Console << "Found house sign." << myendl;
					}
					if( itemCheck->GetMultiObj() == i )
					{
						// Delete Doors & Signs
						if( itemCheck->GetType() == 203 || itemCheck->GetType() == 12 || itemCheck->GetType() == 13 )
							Items->DeleItem( itemCheck );
						else
							itemCheck->SetMulti( INVALIDSERIAL );
					}
				}
				//Items->DeleItem(toCheck->AddChar
				toCheck->PopItem();
				toCheck->PushChar();
				for( CChar *charCheck = toCheck->FirstChar(); !toCheck->FinishedChars(); charCheck = toCheck->GetNextChar() )
				{
					if( charCheck == NULL )
						continue;
					if( charCheck->GetMultiObj() == i )
					{
						// Delete Player Vendors
						if( charCheck->GetNPCAiType() == aiPLAYERVENDOR ) // player vendor in right place
						{
							sprintf( temp, Dictionary->GetEntry( 580 ).c_str(), charCheck->GetName().c_str() );
							pvDeed = Items->SpawnItem( NULL, mChar, 1, temp, false, 0x14F0, 0, true, false );
							if( pvDeed != NULL )
							{
								pvDeed->SetType( 217 );
								pvDeed->SetBuyValue( 2000 );
							}
							s->sysmessage( 581, charCheck->GetName().c_str() );
							Npcs->DeleteChar( charCheck );
						}
						else
							charCheck->SetMulti( INVALIDSERIAL );
					}
				}
				toCheck->PopChar();
			}
		}
		UI16 keyCount = 0;
		SERIAL serialToMatch = i->GetSerial();
		s->sysmessage( 582 );
		for( ITEM a = 0; a < cwmWorldState->GetItemCount(); ++a )		// we HAVE to have an itemcount, because when you replace the house, the serial will be different
		{
			// so we have to remove all keys that used that serial
			if( items[a].GetMore() == serialToMatch && items[a].GetType() == 7 )
			{
				Items->DeleItem( &items[a] );
				++keyCount;
			}
		}
		s->sysmessage( 583, keyCount );
		Items->DeleItem( i );
		return;
	}
}

void killKeys( SERIAL targSerial )
// This function is rather CPU-expensive, but AFAIK there is no
// better way to find all keys than to do it this way.. :/
{
	for( ITEM i = 0; i < cwmWorldState->GetItemCount(); ++i ) 
	{
		if( items[i].GetType() == 7 && items[i].GetMore() == targSerial ) // make key uselss
			items[i].SetMore( 0 );
	}
}

bool OnHouseList( CMultiObj *house, CChar *toCheck )
{
	if( house == NULL || toCheck == NULL )
		return false;
	return house->IsOwner( toCheck );
}
UI08 AddToHouse( CMultiObj *house, CChar *toAdd, UI08 mode )
{
	if( house == NULL || toAdd == NULL )
		return 0;
	const UI08 OWNER = 0;
	const UI08 BAN = 1;
	if( house->IsOwner( toAdd ) || house->IsOnBanList( toAdd ) )
		return 2;

	SI16 sx, sy, ex, ey;
	Map->MultiArea( house, sx, sy, ex, ey );
	// Make an object with the character's serial & the list type
	// and put it "inside" the house item.
	if( toAdd->GetX() >= sx && toAdd->GetY() >= sy && toAdd->GetX() <= ex && toAdd->GetY() <= ey )
	{
		switch( mode )
		{
			default:
			case OWNER:		house->AddAsOwner( toAdd );		break;
			case BAN:		house->AddToBanList( toAdd );	break;
		}
		return 1;
	}
	else
		return 3;
}
bool DeleteFromHouseList( CMultiObj *house, CChar *toDelete, UI08 mode )
{
	if( house == NULL || toDelete == NULL )
		return false;
	switch( mode )
	{
		default:
		case 0:	// owner
			house->RemoveAsOwner( toDelete );
			break;
		case 1:	// ban
			house->RemoveFromBanList( toDelete );
			break;
	}
	return true;
}
void cSpeech::HouseSpeech( cSocket *mSock, std::string talk )
{
	if( mSock == NULL ) 
		return;

	UnicodeTypes sLang = mSock->Language();

	CChar *mChar			= mSock->CurrcharObj();
	CMultiObj *realHouse	= findMulti( mChar );
	if( realHouse == NULL ) 
		return; // not in a house, so we don't care.
	if( realHouse->GetObjType() != OT_MULTI )
		return;

	UString msg( talk );
	msg = msg.upper();
	if( msg.find( Dictionary->GetEntry( 584, sLang ) ) != std::string::npos ) 
	{ // realHouse ban
		mSock->AddID( realHouse->GetSerial() );
		mSock->target( 0, TARGET_HOUSEBAN, 585 );
		return;
	}
	if( msg.find( Dictionary->GetEntry( 586, sLang ) ) != std::string::npos ) 
	{ // kick out of realHouse
		mSock->AddID( realHouse->GetSerial() );
		mSock->target( 0, TARGET_HOUSEEJECT, 587 );
		return;
	}
	if( msg.find( Dictionary->GetEntry( 588, sLang ) ) != std::string::npos )
	{ // lock down item
		if( realHouse->GetLockDownCount() < realHouse->GetMaxLockDowns() )
		{
			mSock->AddID( realHouse->GetSerial() );
			mSock->target( 0, TARGET_HOUSELOCKDOWN, 589 );
		}
		return;
	}
	if( msg.find( Dictionary->GetEntry( 590, sLang ) ) != std::string::npos )
	{ // lock down item
		if( realHouse->GetLockDownCount() > 0 )
		{
			mSock->AddID( realHouse->GetSerial() );
			mSock->target( 0, TARGET_HOUSERELEASE, 591 );
		}
		return;
	}
}
