#include "uox3.h"
#include "townregion.h"
#include "cServerDefinitions.h"
#include "cSpawnRegion.h"
#include "ssection.h"
#include "trigger.h"
#include "scriptc.h"
#include "cScript.h"
#include "cEffects.h"
#include "packets.h"
#include "classes.h"
#include "regions.h"

#undef DBGFILE
#define DBGFILE "items.cpp"

//o---------------------------------------------------------------------------o
//|	Function	-	CItem * cItem::MemItemFree( bool zeroSer, UI08 itemType )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find a free item slot, checking freeitemmem[] first
//o---------------------------------------------------------------------------o
CItem *cItem::MemItemFree( ITEM& offset, bool zeroSer, UI08 itemType )
{
	char memerr = 0;

	offset = items.New( zeroSer, itemType );

	int slots = 8192;
					// careful with this number.....
					// if it too low => slow and can cause crashes under
					// w95/98.  This is because w95/98 can only handle 8196
					// subsequent realloc calls ( crappy thing...)
					// free() calls DON'T help !!!, btw
					// so we have to make that number real big
					// under nt and unix this limit doesn't exist
	if( cwmWorldState->GetItemCount() >= cwmWorldState->GetIMem() ) //theres no more free sluts.. er slots, so get more memory
	{
		ITEM *Copy_loscache = NULL;
		UI16 *Copy_itemids = NULL;

		// create new objects to copy original data to
		if( ( Copy_loscache = new ITEM[cwmWorldState->GetIMem()] ) == NULL )
			memerr = 1;
		else if( ( Copy_itemids = new UI16[cwmWorldState->GetIMem()] ) == NULL )
			memerr = 1;

		// make sure nothing bad happened
		if( !memerr )
		{
			// make a copy - I prefer memcpy although a copy constructor wouldn't hurt =)
			memcpy( Copy_loscache, loscache, sizeof( ITEM ) * cwmWorldState->GetIMem() );
			memcpy( Copy_itemids, itemids, sizeof( UI16 ) * cwmWorldState->GetIMem() );

			// delete the old objects
			delete [] itemids;
			delete [] loscache;

			// initialize to NULL so if an error has occurred we can properly
			// free them by checking for NULL
			loscache = NULL;
			itemids = NULL;

			// create new objects with more room for more items, etc.
			if( ( loscache = new ITEM[cwmWorldState->GetIMem() + slots] ) == NULL )
				memerr = 2;
			else if( ( itemids = new UI16[cwmWorldState->GetIMem() + slots] ) == NULL )
				memerr = 2;

			if( !memerr )
			{
				// restore copy to new objects
				memcpy( loscache, Copy_loscache, sizeof( ITEM ) * cwmWorldState->GetIMem() );
				memcpy( itemids, Copy_itemids, sizeof( UI16 ) * cwmWorldState->GetIMem() );

				// delete copies
				delete [] Copy_itemids;
				delete [] Copy_loscache;
			}
		}
		if( memerr )
		{
			// cleanup if neccessary
			if( memerr >= 1 )
			{
				if( Copy_itemids )
					delete [] Copy_itemids;
				if( Copy_loscache )
					delete [] Copy_loscache;

				// cleanup if neccessary
				if( memerr == 2 )
				{
					if( itemids )
						delete [] Copy_itemids;
					if( loscache )
						delete [] Copy_loscache;
				}
			}
			Console.Error( 3, "Could not reallocate item memory after %i. No more items will be created.", cwmWorldState->GetIMem() );
			Console.Warning( 3, " UOX may become unstable." );
		} 
		else 
			cwmWorldState->SetIMem( cwmWorldState->GetIMem() + slots );
	}
	if( offset == INVALIDSERIAL )
		return NULL;
	return &items[offset];
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::DeleItem( CItem *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Remove an item from the array
//o---------------------------------------------------------------------------o
void cItem::DeleItem( CItem *i )
{
	if( i == NULL )
		return;
	if( i->isFree() )
		return;

	ITEM calcedItem = calcItemFromSer( i->GetSerial() );

	// if we delete an item we should delete it from spawnregions
	// this will fix several crashes
	if( i->IsSpawned() )
	{
		cSpawnRegion *spawnReg = NULL;

		for( UI16 k = 1; k < cwmWorldState->GetTotalSpawnRegions(); ++k )
		{
			spawnReg = spawnregion[k];
			if( spawnReg == NULL )
				continue;
			else
				spawnReg->deleteSpawnedItem( i );
		}
	}

	i->SetCont( NULL );

	UI16 scpNum			= i->GetScriptTrigger();
	cScript *tScript	= Trigger->GetScript( scpNum );
	if( tScript != NULL )
		tScript->OnDelete( i );

	i->RemoveFromSight();
	if( i->GetGlow() != INVALIDSERIAL )
	{
		CItem *j = calcItemObjFromSer( i->GetGlow() );
		if( j != NULL )
			DeleItem( j );  // LB glow stuff, deletes the glower of a glowing stuff automatically
	}

	size_t ci = INVALIDSERIAL;
	if( i->GetType() == 61 )
	{
		HashBucketMulti< ITEM > *hashBucketI = nspawnsp.GetBucket( (i->GetSerial())%HASHMAX );
		for( size_t j = 0; j < hashBucketI->NumEntries(); ++j )
		{
			ci = hashBucketI->GetEntry( j );
			if( ci != INVALIDSERIAL )
			{
				if( items[ci].isFree() )
					continue;
				if( i == items[ci].GetSpawnObj() )
				{
					if( i != &items[ci] )
						items[ci].SetSpawn( INVALIDSERIAL, ci );
				}
			}
		}
	}

	if( i->GetType() == 62 || i->GetType() == 69 || i->GetType() == 125 )
	{
		HashBucketMulti< CHARACTER > *hashBucketC = ncspawnsp.GetBucket( (i->GetSerial())%HASHMAX );
		for( size_t l = 0; l < hashBucketC->NumEntries(); ++l )
		{
			ci = hashBucketC->GetEntry( l );
			if( ci != INVALIDSERIAL )
			{
				if( chars[ci].isFree() )
					continue;
				if( chars[ci].GetSpawnObj() == i )
					chars[ci].SetSpawn( INVALIDSERIAL, ci );
			}
		}
	}

	if( i->GetSpawnObj() != NULL ) 
		nspawnsp.Remove( i->GetSpawn(), calcedItem );

	for( CItem *tItem = i->FirstItem(); !i->FinishedItems(); tItem = i->NextItem() )
	{
		if( tItem != NULL )
			Items->DeleItem( tItem );
	}

	if( i->isGuarded() )
	{
		CChar *owner = NULL;
		CMultiObj *multi = findMulti( i );
		if( multi != NULL )
			owner = (CChar *)multi->GetOwnerObj();
		if( owner == NULL )
			owner = getPackOwner( i );
		if( owner != NULL )
		{
			CChar *petGuard = Npcs->getGuardingPet( owner, i->GetSerial() );
			if( petGuard != NULL )
				petGuard->SetGuarding( INVALIDSERIAL );
			i->SetGuarded( false );
		}
	}

	QUEUEMAP_ITERATOR toFind = itemQueue.find( (cBaseObject *)i );
	if( toFind != itemQueue.end() )
		itemQueue.erase( toFind );

	nitemsp.Remove( i->GetSerial() );
	items.Delete( calcedItem );
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool ApplyItemSection( CItem *applyTo, ScriptSection *toApply )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Load items from the script sections to the array
//o---------------------------------------------------------------------------o
bool ApplyItemSection( CItem *applyTo, ScriptSection *toApply )
{
	if( toApply == NULL || applyTo == NULL )
		return false;
	DFNTAGS tag = DFNTAG_COUNTOFTAGS;
	UString cdata;
	UI32 ndata = INVALIDSERIAL, odata = INVALIDSERIAL;
	for( tag = toApply->FirstTag(); !toApply->AtEndTags(); tag = toApply->NextTag() )
	{
		cdata = toApply->GrabData( ndata, odata );

		switch( tag )
		{
			case DFNTAG_AMOUNT:			applyTo->SetAmount( ndata );					break;
			case DFNTAG_ATT:			applyTo->SetLoDamage( (SI16)ndata ); 
										applyTo->SetHiDamage( (SI16)odata ); 
										break;
			case DFNTAG_AC:				applyTo->SetArmourClass( (UI08)ndata );			break;
			case DFNTAG_CREATOR:		applyTo->SetCreator( ndata );					break;
			case DFNTAG_COLOUR:			applyTo->SetColour( (UI16)ndata );				break;
			case DFNTAG_COLOURLIST:		applyTo->SetColour( addRandomColor( cdata.c_str() ) );	break;
			case DFNTAG_CORPSE:			applyTo->SetCorpse( ndata != 0 );				break;
			case DFNTAG_COLD:			applyTo->SetWeatherDamage( COLD, ndata != 0 );	break;
			case DFNTAG_DAMAGE:			applyTo->SetLoDamage( (SI16)ndata );
										applyTo->SetHiDamage( (SI16)odata );
										break;
			case DFNTAG_DEF:			applyTo->SetDef( (UI16)RandomNum( ndata, odata ) );	break;
			case DFNTAG_DEX:			applyTo->SetDexterity( (SI16)RandomNum( ndata, odata ) );	break;
			case DFNTAG_DEXADD:			applyTo->Dexterity2( (SI16)ndata );					break;
			case DFNTAG_DIR:			if( cdata.upper() == "NE" )
											applyTo->SetDir( 1 );
										else if( cdata.upper() == "E" )
											applyTo->SetDir( 2 );
										else if( cdata.upper() == "SE" )
											applyTo->SetDir( 3 );
										else if( cdata.upper() == "S" )
											applyTo->SetDir( 4 );
										else if( cdata.upper() == "SW" )
											applyTo->SetDir( 5 );
										else if( cdata.upper() == "W" )
											applyTo->SetDir( 6 );
										else if( cdata.upper() == "NW" )
											applyTo->SetDir( 7 );
										else if( cdata.upper() == "N" )
											applyTo->SetDir( 0 );
										break;
			case DFNTAG_DYE:			applyTo->SetDye( ndata != 0 );				break;
			case DFNTAG_DECAY:			
										if( ndata == 1 )
											applyTo->SetDecayable( true );
										else
											applyTo->SetDecayable( false );
										break;
			case DFNTAG_DISPELLABLE:	applyTo->SetDispellable( true );			break;
			case DFNTAG_DISABLED:		applyTo->SetDisabled( ndata != 0 );			break;
			case DFNTAG_DOORFLAG:		applyTo->SetDoorDir( (UI08)ndata );			break;
			case DFNTAG_ENHANCED:		applyTo->enhanced = ndata;					break;
			case DFNTAG_FAME:			applyTo->SetFame( (SI16)ndata );			break;
			case DFNTAG_GOOD:			applyTo->SetGood( static_cast< SI16 >(ndata) );					break;
			case DFNTAG_GLOW:			applyTo->SetGlow( ndata );					break;
			case DFNTAG_GLOWBC:			applyTo->SetGlowColour( (UI16)ndata );		break;
			case DFNTAG_GLOWTYPE:		applyTo->SetGlowEffect( (UI08)ndata );		break;
			case DFNTAG_GET:
			{
										ScriptSection *toFind = FileLookup->FindEntry( cdata.c_str(), items_def );
										ApplyItemSection( applyTo, toFind );
			}
										break;
			case DFNTAG_HP:				applyTo->SetHP( (SI16)ndata );				break;
			case DFNTAG_HIDAMAGE:		applyTo->SetHiDamage( (SI16)ndata );		break;
			case DFNTAG_HEAT:			applyTo->SetWeatherDamage( HEAT, ndata != 0 );		break;
			case DFNTAG_ID:				applyTo->SetID( (UI16)ndata );				break;
			case DFNTAG_INTELLIGENCE:	applyTo->SetIntelligence( (SI16)ndata );	break;
			case DFNTAG_INTADD:			applyTo->Intelligence2( (SI16)ndata );		break;
			case DFNTAG_LODAMAGE:		applyTo->SetLoDamage( (SI16)ndata );		break;
			case DFNTAG_LAYER:			applyTo->SetLayer( (SI08)ndata );			break;
			case DFNTAG_LIGHT:			applyTo->SetWeatherDamage( LIGHT, ndata != 0 );		break;
			case DFNTAG_LIGHTNING:		applyTo->SetWeatherDamage( LIGHTNING, ndata != 0 );	break;
			case DFNTAG_MAXHP:			applyTo->SetMaxHP( (SI16)ndata );			break;
			case DFNTAG_MOVABLE:		applyTo->SetMovable( (SI08)ndata );			break;
			case DFNTAG_MORE:			applyTo->SetMore( ndata );					break;
			case DFNTAG_MORE2:			applyTo->SetMoreB( ndata );					break;
			case DFNTAG_MOREX:			applyTo->SetMoreX( ndata );					break;
			case DFNTAG_MOREY:			applyTo->SetMoreY( ndata );					break;
			case DFNTAG_MOREZ:			applyTo->SetMoreZ( ndata );					break;
			case DFNTAG_NAME:			applyTo->SetName( cdata.c_str() );			break;
			case DFNTAG_NAME2:			applyTo->SetName2( cdata.c_str() );			break;
			case DFNTAG_NEWBIE:			applyTo->SetNewbie( true );					break;
			case DFNTAG_OFFSPELL:		applyTo->SetOffSpell( (SI08)ndata );		break;
			case DFNTAG_POISONED:		applyTo->SetPoisoned( (UI08)ndata );		break;
			case DFNTAG_PILEABLE:		applyTo->SetPileable( ndata != 0 );			break;
			case DFNTAG_PRIV:			applyTo->SetPriv( (UI08)ndata );			break;
			case DFNTAG_RANK:
										applyTo->SetRank( (SI08)ndata );
										if( applyTo->GetRank() <= 0 ) 
											applyTo->SetRank( 10 );
										break;
			case DFNTAG_RACE:			applyTo->SetRace( (UI16)ndata );			break;
			case DFNTAG_RESTOCK:		applyTo->SetRestock( (UI16)ndata );			break;
			case DFNTAG_RAIN:			applyTo->SetWeatherDamage( RAIN, ndata != 0 );		break;
			case DFNTAG_SK_MADE:		applyTo->SetMadeWith( (SI08)ndata );		break;
			case DFNTAG_SPD:			applyTo->SetSpeed( (UI08)ndata );			break;
			case DFNTAG_STRENGTH:		applyTo->SetStrength( (SI16)ndata );		break;
			case DFNTAG_STRADD:			applyTo->Strength2( (SI16)ndata );			break;
			case DFNTAG_SNOW:			applyTo->SetWeatherDamage( SNOW, ndata != 0 );		break;
			case DFNTAG_SCRIPT:			applyTo->SetScriptTrigger( (UI16)ndata );	break;
			case DFNTAG_SPAWNOBJ:		applyTo->SetDesc( cdata.c_str() );			break;
			case DFNTAG_SPAWNOBJLIST:
										applyTo->SetDesc( cdata.c_str() );
										applyTo->SetSpawnerList( true );
										break;
			case DFNTAG_TYPE:			applyTo->SetType( (UI08)ndata );			break;
			case DFNTAG_TYPE2:			applyTo->SetType2( (UI08)ndata );			break;
			case DFNTAG_VISIBLE:		applyTo->SetVisible( (SI08)ndata );			break;
			case DFNTAG_VALUE:
#if defined( _MSC_VER )
#pragma note( "Verify that the ustring stuff works well here" )
#endif
										if( !cdata.empty() )
										{
											if( cdata.sectionCount( " " ) != 0 )
											{
												applyTo->SetBuyValue(  cdata.section( " ", 0, 0 ).stripWhiteSpace().toShort() );
												applyTo->SetSellValue( cdata.section( " ", 1, 1 ).stripWhiteSpace().toShort() );
												break;
											}
										}
										applyTo->SetBuyValue( ndata );
										applyTo->SetSellValue( (ndata / 2) );
										break;
			case DFNTAG_WEIGHT:			applyTo->SetWeight( ndata );				break;
			case DFNTAG_WIPE:			applyTo->SetWipeable( ndata != 0 );			break;
			case DFNTAG_NOTES:
			case DFNTAG_CATEGORY:
				break;
			default:					Console.Warning( 2, "Unknown items dfn tag %i %s %i %i ", tag, cdata.c_str(), ndata, odata );	break;
		}
	}
	return true;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *CreateItem( cSocket *s, string name, UI08 worldNumber )
//|	Programmer	-	Abaddon
//| Date		-	28th September, 2001
//o---------------------------------------------------------------------------o
//|	Purpose		-	Does the minimum required to spawn an item from the scripts
//|					assigning a world number and setting the serial
//o---------------------------------------------------------------------------o
CItem *cItem::CreateItem( cSocket *s, std::string name, UI08 worldNumber )
{
	CTile tile;
	ScriptSection *itemCreate = FileLookup->FindEntry( name, items_def );
	if( itemCreate == NULL )
		return NULL;

	// Scan through for an itemlist, if we make an itemlist, then we'll call our random item func and return instantly
	// This way, we skip over some code we double up like an itemcount and itemcount2 increase

	if( itemCreate->ItemListExist() )
	{
		CItem *iListMade = CreateRandomItem( s, itemCreate->ItemListData(), true, worldNumber );
		if( iListMade != NULL )
			iListMade->WorldNumber( worldNumber );
		return iListMade;
	}
	ITEM iMadeOff;
	CItem *iMade = MemItemFree( iMadeOff, true );
	if( iMade == NULL )
		return NULL;

	iMade->SetID( 0x0915 );
	iMade->SetWipeable( false );
	iMade->SetAmount( 1 );		// obviously some things have no default val, and 0 is not a good number

	if( !ApplyItemSection( iMade, itemCreate ) )
		Console.Error( 2, "Trying to apply an item section failed" );

	Map->SeekTile( iMade->GetID(), &tile );
	if( tile.Stackable() )
		iMade->SetPileable( true );
	if( !iMade->GetMaxHP() && iMade->GetHP() ) 
		iMade->SetMaxHP( iMade->GetHP() );

	iMade->WorldNumber( worldNumber );
	return iMade;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *CreateScriptItem( cSocket *s, string name, bool nSpawned, UI08 worldNumber )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Create an item based on its script entries
//|									
//|	Modification	-	04/22/2002	-	Added a check here to make sure that there 
//|									are no trailign spaces.
//o---------------------------------------------------------------------------o
CItem * cItem::CreateScriptItem( cSocket *s, std::string name, bool nSpawned, UI08 worldNumber )
{
	UString ourName( name );
	ourName = ourName.stripWhiteSpace();
	CItem *iMade = CreateItem( s, ourName, worldNumber );
	if( iMade == NULL )
	{
		Console.Error( 2, "CreateScriptItem(): Bad script item %s (Item Not Found).\n", name.c_str() );
		return NULL;
	}
 
	if( s != NULL && !nSpawned )
		iMade->SetLocation( s->GetWord( 11 ), s->GetWord( 13 ), s->GetByte( 16 ) + Map->TileHeight( s->GetWord( 17 ) ) );

	if( iMade->GetCont() == NULL ) 
		MapRegion->AddItem( iMade );
	cScript *toGrab = Trigger->GetScript( iMade->GetScriptTrigger() );
	if( toGrab != NULL )
		toGrab->OnCreate( iMade );
	return iMade;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *cItem::CreateRandomItem( cSocket *s, std::string sItemList, bool nSpawned )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Create a random item from the scripts
//o---------------------------------------------------------------------------o
CItem *cItem::CreateRandomItem( cSocket *s, std::string sItemList, bool nSpawned, UI08 worldNumber )
{
	UString sect	= "ITEMLIST " + sItemList;
	sect			= sect.stripWhiteSpace();
	ScriptSection *ItemList = FileLookup->FindEntry( sect, items_def );
	if( ItemList == NULL )
		return NULL;
	size_t i = ItemList->NumEntries();
	if( i == 0 )
		return NULL;
	UString k = ItemList->MoveTo( RandomNum( static_cast< size_t >(0), i - 1 ) );
	if( !k.empty() )
	{
		CItem *iMade = CreateScriptItem( s, k.c_str(), nSpawned, worldNumber );
		return iMade;
	}
	return NULL;
}

//o--------------------------------------------------------------------------o
//|	Function	-	CItem * cItem::SpawnItem( cSocket *csSocket, CChar *ccPlayer, UI32 dwAmount, std::string szNameItem, bool bToStack, UI16 wItemID, UI16 wItemColor, bool bInPack, bool bBroadcast )
//|	Date		-	5/24/2003 12:58:02 PM
//|	Developers	-	EviLDeD
//|	Organization-	UOX3 DevTeam
//|	Status		-	Currently under development
//o--------------------------------------------------------------------------o
//|	Description	-	Spawn an item.  I have seen this primarily used for the JSE
//|					I am assuming that these are the functions that the JSE
//|					docs refer to.  There are 2 versions of this functions.  Both
//|					are 9 argument versions that differ only in the way that
//|					the item ID is specified. IE: 0x09D0 or "0x09D0".
//|
//|					This function performs an autostacking check, and then
//|					creates the items in question.  This function was re-written
//|					to operate more effectively, and preserve member information
//|					which the previous code was not doing.
//o--------------------------------------------------------------------------o
CItem * cItem::SpawnItem( cSocket *nSocket, CChar *ch, UI32 nAmount, std::string cName, bool nStackable, UI16 realItemID, UI16 realColour, bool nPack, bool nSend )
{
	if( nAmount == 0 )
		return NULL;
	UI08 worldNumber	= ch->WorldNumber();
	CItem *i			= NULL;
	bool inpack			= false;
	CItem *p			= getPack( ch );
	//Auto-Stack code!
	if( nPack && nStackable && p != NULL )
	{
		for( i = p->FirstItem(); !p->FinishedItems(); i = p->NextItem() )
		{
			if( i != NULL && ( i->GetID() == realItemID ) && i->GetColour() == realColour )
			{
				if( i->GetAmount() + nAmount <= MAX_STACK )
				{
					i->SetAmount( i->GetAmount() + nAmount );
					inpack = true;
					break;
				}
			}
		}
	}

	if( !nPack || ( nPack && !inpack ) )
	{
		ITEM iOff;
		i = MemItemFree( iOff, true );
		if( i == NULL ) 
			return NULL;

		i->WorldNumber( worldNumber );
		if( !cName.empty() )
			i->SetName( cName );
		i->SetID( realItemID );
		i->SetColour( realColour );
		i->SetAmount( nAmount );
		i->SetPileable( nStackable );
		i->SetDecayable( true );
		if( nPack )
		{
			if( p != NULL )
			{
				i->SetCont( p );
				i->SetX( ( 50 + RandomNum( 0, 79 ) ) );
				i->SetY( ( 50 + RandomNum( 0, 79 ) ) );
				i->SetZ( 9 );
			}
			else
				i->SetLocation( ch );
		}
		GetScriptItemSetting( i );
	}
	
	if( nSend && nSocket != NULL ) 
	{ 
	   // added to fix weight bug - sereg 
	   //as i supposed the weight of the total package had been added before
	   UI16 sum_nAmount = i->GetAmount(); 
	   i->SetAmount( nAmount ); 
	   nSocket->statwindow( ch ); 
	   i->SetAmount( sum_nAmount ); 
	} 

	cScript *toGrab = Trigger->GetScript( i->GetScriptTrigger() );
	if( toGrab != NULL )
		toGrab->OnCreate( i );
	return i;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *cItem::SpawnMulti( CChar *ch, std::string cName, UI16 realItemId )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Spawn a Multi
//o---------------------------------------------------------------------------o
CMultiObj *cItem::SpawnMulti( CChar *ch, std::string cName, UI16 realItemID )
{
	ITEM cOff;
	CItem *c = MemItemFree( cOff, true, 1 );
	if( c == NULL ) 
		return NULL;

	CMultiObj *i = static_cast< CMultiObj * >(c);
	i->WorldNumber( ch->WorldNumber() );
	if( !cName.empty() )
		i->SetName( cName );
	i->SetID( realItemID );
	GetScriptItemSetting( i );
	return i;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::GetScriptItemSetting( CItem *ciScriptItem )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get item settings from scripts
//o---------------------------------------------------------------------------o
void cItem::GetScriptItemSetting( CItem *ciScriptItem )
{
	if( ciScriptItem != NULL )
	{
		UString sbuf = "0x" + UString::number( ciScriptItem->GetID(), 16 );
		ScriptSection *toFind = FileLookup->FindEntrySubStr( sbuf, hard_items_def );
		if( toFind != NULL )
			ApplyItemSection( ciScriptItem, toFind );
	}
}

CItem *autoStack( cSocket *mSock, CItem *iToStack, CItem *iPack );

//o---------------------------------------------------------------------------o
//|	Function	-	CItem * cItem::SpawnItemToPack( cSocket *s, CChar *ch, string name, bool nDigging, bool autoStack )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Spawn an item inside a pack
//o---------------------------------------------------------------------------o
CItem * cItem::SpawnItemToPack( cSocket *sock, CChar *mChar, std::string name, bool nDigging, bool bAutoStack )
{
	CItem *iPack = getPack( mChar );
	if( iPack == NULL ) 
	{
		Console.Warning( 2, "SpawnItemToPack(): Character %s(%X) has no pack.  Item is not created.\n", mChar->GetName().c_str(), mChar->GetSerial() );
		return NULL;
	}

	CItem *iCreated = CreateScriptItem( sock, name, false, iPack->WorldNumber() );

	if( iCreated == NULL )
		return NULL;

	if( nDigging ) 
	{
		if( iCreated->GetBuyValue() != 0 )
		{
			iCreated->SetBuyValue( RandomNum( static_cast< SI32 >(1), iCreated->GetBuyValue() ) );
			iCreated->SetSellValue( (SI32)iCreated->GetBuyValue() / 2 );
		}
		if( iCreated->GetHP() != 0 ) 
			iCreated->SetHP( RandomNum( static_cast< SI16 >(1), iCreated->GetHP() ) );
	}
	
	GetScriptItemSetting( iCreated );
	if( bAutoStack )
		iCreated = autoStack( sock, iCreated, iPack );	// if it didn't stack, it's iCreated... if it did, then it's the stack!
	else
	{
		iCreated->SetCont( iPack );
		iCreated->SetX( 50 + RandomNum( 0, 79 ) );
		iCreated->SetY( 50 + RandomNum( 0, 79 ) );
		iCreated->SetZ( 9 );
	}

	if( sock != NULL )
		sock->statwindow( mChar );
	return iCreated;
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool cItem::DecayItem( CItem *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Cause items to decay when left on the ground
//o---------------------------------------------------------------------------o
bool cItem::DecayItem( CItem *i ) 
{
	if( i == NULL || i->isFree() )
		return false;
	if( !i->WillDecay() )
		return false;

	if( i->GetDecayTime() > cwmWorldState->GetUICurrentTime() && !cwmWorldState->GetOverflow() )
		return false;

	bool retVal = false;
	if( i->GetCont() == NULL )	// Are we on the ground?
	{
		// decaytime = 5 minutes, * 60 secs per min, * 1000
		UI32 decayTimer = BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->SystemTimer( DECAY ) ));
		if( i->GetDecayTime() == 0 ) 
		{
			i->SetDecayTime( decayTimer );
			return false;
		}

		bool isCorpse = i->isCorpse();
		
		// Multis
		if( !i->IsFieldSpell() && !isCorpse ) // Gives fieldspells a chance to decay in multis
		{
			if( i->GetMultiObj() == NULL )
			{
				CMultiObj *multi = findMulti( i );
				if( multi != NULL )
				{
					if( multi->GetMore( 4 ) == 0 )
					{
						i->SetDecayTime( decayTimer );
						i->SetMulti( multi );
						return false;
					}
				}
			} 
			else	// in a house, therefore... no decay
			{					
				i->SetDecayTime( decayTimer );
				return false;
			}
		}

		if( isCorpse && i->GetOwnerObj() != NULL && i->GetMore( 4 ) )
		{
			UI32 preservebody = 0;
			for( CItem *j = i->FirstItem(); !i->FinishedItems(); j = i->NextItem() )
			{
				if( j != NULL )
					++preservebody;
			}
			if( preservebody > 1  )
			{
				i->SetMore( i->GetMore( 4 ) - 1, 4 );
				i->SetDecayTime( decayTimer );
				return false;
			}
		}
#if defined( _MSC_VER )
#pragma note( "Type == 1?  Or IsContType()?" )
#endif
		if( ( !isCorpse && i->GetType() == 1 ) || ( isCorpse && ( i->GetOwnerObj() != NULL || !cwmWorldState->ServerData()->CorpseLootDecay() ) ) )
		{
			for( CItem *io = i->FirstItem(); !i->FinishedItems(); io = i->NextItem() )
			{
                if( io != NULL )
				{
					io->SetCont( NULL );
					io->SetLocation( i );

					io->SetDecayTime( decayTimer );
				}
			}
		}
		DeleItem( i );
		retVal = true;
	}
	return retVal;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::RespawnItem( ITEM i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Respawn an item
//o---------------------------------------------------------------------------o
void cItem::RespawnItem( CItem *i )
{
	try
	{
		if( i == NULL || i->isFree() )
		return;
	}
	catch( ... )
	{
		Console << "EXCEPTION: cItem::RespawnItem() - Invalid pointer provided, ignored!" << myendl;
		return;
	}

	switch( i->GetType() )
	{
		case 61:
		case 62:
		case 63:
		case 64:
		case 65:
		case 69:
		case 125:
			break;
		default:
			return;
	}

	int j, m, k;
	ITEM ci;

	HashBucketMulti< ITEM > * hashBucketI = NULL;
	HashBucketMulti< CHARACTER > * hashBucketC = NULL;
	for( int c = 0; c < i->GetAmount(); ++c )
	{
		if( i->GetGateTime() + ( c*i->GetMoreZ() * 1000) <= cwmWorldState->GetUICurrentTime() )
		{
			m = 0;
			switch( i->GetType() )
			{
				case 61:
					k = 0;
					hashBucketI = nspawnsp.GetBucket( (i->GetSerial())%HASHMAX );
					for( j = 0; j < static_cast<int>(hashBucketI->NumEntries()); ++j )
					{
						ci = hashBucketI->GetEntry( j );
						if( ci != INVALIDSERIAL )
						{
							if( items[ci].isFree() )
								continue;
							if( i == items[ci].GetSpawnObj() )
							{
								if( i != &items[ci] && items[ci].GetX() == i->GetX() && items[ci].GetY() == i->GetY() && items[ci].GetZ() == i->GetZ() )
								{
									k = 1;
									break;
								}
							}
						}
					}

					if( k == 0 )
					{
						if( i->GetGateTime() == 0 )
						{
							i->SetGateTime( BuildTimeValue( static_cast<R32>(RandomNum( i->GetMoreY() * 60, i->GetMoreZ() * 60 ) )) );
						}
						if( i->GetGateTime() <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
						{
							const char *lDesc = i->GetDesc();
							if( lDesc[0] != 0 )	// not NULL terminated, so we can do something!
							{
								AddRespawnItem( i, lDesc, false, i->isSpawnerList() );
							}
							else if( i->GetMoreX() != 0 )
							{
								AddRespawnItem( i, i->GetMoreX(), false );
							}
							else
								break;
							i->SetGateTime( 0 );
						}
					}
					break;
				case 62:
				case 69:
				case 125:
					k = 0;

					hashBucketC = ncspawnsp.GetBucket( (i->GetSerial())%HASHMAX );
					for( j = 0; j < static_cast<int>(hashBucketC->NumEntries()); ++j )
					{
						ci = hashBucketC->GetEntry( j );
						if( ci != INVALIDSERIAL )
						{
							if( chars[ci].isFree() )
								continue;
							if( chars[ci].GetSpawnObj() == i )
								++k;
						}
					}

					if( k < i->GetAmount() )
					{
						if( i->GetGateTime() == 0 )
						{
							i->SetGateTime( BuildTimeValue( static_cast<R32>(RandomNum( i->GetMoreY() * 60, i->GetMoreZ() * 60 )) ) );
						}
						if( i->GetGateTime() <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
						{
							const char *mDesc = i->GetDesc();
							if( mDesc[0] != 0 )	// not NULL terminated, so we can do something!
							{
								Npcs->SpawnNPC( i, mDesc, i->WorldNumber(), i->isSpawnerList() );
								i->SetGateTime( 0 );
							}
							else if( i->GetMoreX() != 0 )
							{
								char temp[512];
								sprintf( temp, "%u", i->GetMoreX() );
								Npcs->SpawnNPC( i, temp, i->WorldNumber(), false );
								i->SetGateTime( 0 );
							}
						}
					}
					break;
				case 63:
				case 64:
				case 65:
					CItem *getItem;
					getItem = NULL;
					for( getItem = i->FirstItem(); !i->FinishedItems(); getItem = i->NextItem() )
					{
						if( getItem != NULL )
						{
							if( !getItem->isFree() )
								++m;
						}
					}
					if( m < i->GetAmount() )
					{
						if( i->GetGateTime() == 0 )
						{
							i->SetGateTime( BuildTimeValue( static_cast<R32>(RandomNum( i->GetMoreY() * 60, i->GetMoreZ() * 60 )) ) );
						}
						if( i->GetGateTime() <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
						{
							const char *nDesc = i->GetDesc();
							if( i->GetType() == 63 )
								i->SetType( 64 ); // Lock the container
							if( nDesc[0] != 0 )	// not NULL terminated, so we can do something!
							{
								AddRespawnItem( i, nDesc, true, i->isSpawnerList() );
								i->SetGateTime( 0 );
							}
							else if( i->GetMoreX() != 0 )
							{
								AddRespawnItem( i, i->GetMoreX(), true );
								i->SetGateTime( 0 );
							}
						}
					}
					break;
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	UI08 cItem::PackType( UI08 id1, UI08 id2 )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get the packtype based on the packs ID
//o---------------------------------------------------------------------------o
UI08 cItem::PackType( UI16 id )
{
	switch( id )
	{
		case 0x09A8: 
		case 0x09AA:
		case 0x09B0:	return 1;
		case 0x09A9:	return 2;
		case 0x09AB:	return 3;
		case 0x09B2:	return 4;
		case 0x0E3C:
		case 0x0E3D:
		case 0x0E3E:
		case 0x0E3F:
		case 0x0E7E:
		case 0x0E78:	return 2;
		case 0x0E76:
		case 0x0E79:
		case 0x0E7D:
		case 0x0E80:
		case 0x0E7A:	return 1;
		case 0x0E40:
		case 0x0E41:
		case 0x0E42:
		case 0x0E7C:
		case 0x0E43:	return 3;
		case 0x0E75:
		case 0x0E7F:
		case 0x0E83:
		case 0x0E77:	return 4;
		default:		return 0xFF;
	}
	return 0xFF;		// safety catch all (avoids compiler warnings)
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::AddRespawnItem( CItem *s, UI32 x, bool inCont )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Item spawning stuff
//o---------------------------------------------------------------------------o
void cItem::AddRespawnItem( CItem *s, UI32 x, bool inCont )
{
	if( s == NULL )
		return;
	UString temp = UString::number( x );
	AddRespawnItem( s, temp, inCont, false );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::AddRespawnItem( CItem *s, const char *x, bool inCont )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Item spawning stuff
//o---------------------------------------------------------------------------o
void cItem::AddRespawnItem( CItem *s, std::string x, bool inCont, bool randomItem = false )
{
	if( s == NULL || x.empty() )
		return;
	CItem *c = NULL;
	if( randomItem )
		c = CreateRandomItem( NULL, x, true, s->WorldNumber() );
	else
		c = CreateScriptItem( NULL, x, true, s->WorldNumber() );
	if( c == NULL )
		return;

	if( inCont )
		c->SetCont( s );
	else
		c->SetLocation( s );
	c->SetSpawn( s->GetSerial(), calcItemFromSer( c->GetSerial() ) );
	
	if( inCont )
	{
		CItem *z = NULL;
		if( c->GetSpawnObj() != NULL )
			z = (CItem *)c->GetSpawnObj();
		if( z != NULL )
		{
			UI08 k = PackType( z->GetID() );
			if( k == 0xFF )
			{
				Console.Warning( 2, " A non-container item was set as container spawner" );
				k = 1;
			}
			c->SetX( RandomNum( 0, 99 ) + 18 );
			c->SetZ( 9 );
			switch( k )
			{
				case 1:	c->SetY( ( RandomNum( 0, 49 ) ) + 50 );		break;
				case 2:	c->SetY( ( RandomNum( 0, 49 ) ) + 30 );		break;
				case 3:	c->SetY( ( RandomNum( 0, 39 ) ) + 100 );	break;
				case 4:
					c->SetY( ( RandomNum( 0, 79 ) ) + 60 );
					c->SetX( ( RandomNum( 0, 79 ) ) + 60 );
					break;
				default:	c->SetY( ( RandomNum( 0, 49 ) ) + 30 );	break;
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|		Function	-	menuAddItem( cSocket *s, std::string item, UI16 amount )
//|		Programmer	-	Zane
//|		Date		-	January 31, 2003
//o---------------------------------------------------------------------------o
//|		Purpose		-	Create an item selected from the Add menu
//o---------------------------------------------------------------------------o
void cItem::menuAddItem( cSocket *s, std::string item, UI16 amount )
{
	CChar *mChar = s->CurrcharObj();
	CItem *c = Items->SpawnItemToPack( s, mChar, item, false );
	if( c == NULL )
		return;
	c->SetName2( c->GetName().c_str() );
	c->SetCreator( INVALIDSERIAL );
	c->SetMovable( 1 );
	if( amount > 0 && c->isPileable() )
		c->SetAmount( amount );
	s->statwindow( mChar );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::GlowItem( CItem *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle glowing items
//o---------------------------------------------------------------------------o
void cItem::GlowItem( CItem *i )
{
	if( i->GetGlow() != INVALIDSERIAL )
	{
		CItem *j = calcItemObjFromSer( i->GetGlow() );
		if( j == NULL )
			return;

		//j->SetLayer( i->GetLayer() ); // copy layer information of the glowing item to the invisible light emitting object

		cBaseObject *getCont = i->GetCont();
		if( getCont == NULL ) // On the ground
		{
			j->SetCont( NULL );
			j->SetDir( 29 );
			j->SetLocation( i );
		}
		else if( getCont->GetObjType() == OT_ITEM ) // In a pack
		{
			MapRegion->RemoveItem( j );
			j->SetCont( getCont );
			j->SetDir( 99 );  // gives no light in backpacks
			j->SetX( i->GetX() );
			j->SetY( i->GetY() );
			j->SetZ( i->GetZ() );
		}
		else // Equipped
		{
			CChar *s = (CChar *)getCont;
			if( s != NULL )
			{
				MapRegion->RemoveItem( j );
				j->SetCont( getCont );
				j->SetX( s->GetX() );
				j->SetY( s->GetY() );
				j->SetZ( s->GetZ()+4 );
				if( isOnline( s ) )
					j->SetDir( 29 );
				else
					j->SetDir( 99 );
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::CheckEquipment( CChar *p )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check equipment of character
//o---------------------------------------------------------------------------o
void cItem::CheckEquipment( CChar *p )
{
	if( p != NULL ) 
	{
		cSocket *pSock = calcSocketObjFromChar( p );
		if( pSock == NULL ) 
			return;

		const SI16 StrengthToCompare = p->GetStrength();
		for( CItem *i = p->FirstItem(); !p->FinishedItems(); i = p->NextItem() )
		{
			if( i != NULL )
			{
				if( i->GetStrength() > StrengthToCompare )//if strength required > character's strength
				{
					std::string itemname;
					if( i->GetName() == "#" ) 
						getTileName( i, itemname );
					else 
						itemname = i->GetName();

					i->SetCont( NULL );
					i->SetLocation( p );
									
					Network->PushConn();
					for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
					{
						if( charInRange( p, tSock->CurrcharObj() ) )
							p->SendWornItems( tSock );
					}
					Network->PopConn();
					pSock->sysmessage( 1604, itemname.c_str() );
					Effects->itemSound( pSock, i );
				}
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cItem::StoreItemRandomValue( CItem *i, UI08 tmpreg )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Remember an items value
//o---------------------------------------------------------------------------o
void cItem::StoreItemRandomValue( CItem *i, UI08 tmpreg )
{
	if( i->GetGood() < 0 ) 
		return;
	if( tmpreg == 0xFF )
	{
		cBaseObject *getLastCont = i->GetCont();
		if( getLastCont != NULL )
			tmpreg = calcRegionFromXY( getLastCont->GetX(), getLastCont->GetY(), getLastCont->WorldNumber() );
		if( tmpreg == 0xFF )
			return;
	}
	
	SI32 min = region[tmpreg]->GetGoodRnd1( static_cast<UI08>(i->GetGood()) );
	SI32 max = region[tmpreg]->GetGoodRnd2( static_cast<UI08>(i->GetGood()) );
	
	if( max != 0 || min != 0 )
		i->SetRndValueRate( RandomNum( min, max ) );
}

//o---------------------------------------------------------------------------o
//|	Function	-	cItem::addGold( cSocket *s, UI32 amt )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Adds gold to characters total gold
//o---------------------------------------------------------------------------o
void cItem::addGold( cSocket *s, UI32 amt )
{
	SpawnItem( s, s->CurrcharObj(), amt, "#", true, 0x0EED, 0, true, true );
}

//o--------------------------------------------------------------------------
//|	Function		-	CItem *cItem::SpawnRandomItem( cSocket *nCharID, DEFINITIONCATEGORIES sourceDFN, char *cList, char *cItemID )
//|	Date			-	Unknown
//|	Programmer		-	Unknown
//|	Modified		-	Changed to FileLookup based and return CItem * 
//|						instead of creature ID (Abaddon)
//o--------------------------------------------------------------------------
//|	Purpose			-	Gets the random item number from the script and list specified
//|						Creates the item and returns it
//o--------------------------------------------------------------------------
CItem *cItem::SpawnRandomItem( cSocket *nCharID, DEFINITIONCATEGORIES sourceDFN, char* cList, char* cItemID )
{
	char sect[512];
	sprintf( sect, "%s %s", cList, cItemID );
	ScriptSection *randData = NULL;
	randData = FileLookup->FindEntry( sect, sourceDFN );
		if( randData == NULL )
			return NULL;

	size_t numEntries = randData->NumEntries();
	if( numEntries > 0 )
	{
		size_t i		= rand()%(numEntries);
		UString targID	= randData->MoveTo( i );
		if( !targID.empty() )
			return SpawnItemToPack( nCharID, nCharID->CurrcharObj(), targID.c_str(), true );
	}
	return NULL;
}
