#include "uox3.h"
#include "msgboard.h"
#include "movement.h"
#include "cServerDefinitions.h"
#include "cSpawnRegion.h"
#include "skills.h"
#include "commands.h"
#include "ssection.h"
#include "trigger.h"
#include "cScript.h"
#include "teffect.h"
#include "packets.h"
#include "cEffects.h"
#include "classes.h"
#include "regions.h"

#undef DBGFILE
#define DBGFILE "npcs.cpp"

//o---------------------------------------------------------------------------o
//|	Function	-	void DeleteChar( CChar *k )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Delete character
//o---------------------------------------------------------------------------o
void cCharStuff::DeleteChar( CChar *k )
{
	if( k == NULL )
		return;
	CHARACTER kChar = calcCharFromSer( k->GetSerial() );
	MapRegion->RemoveChar( k );

	// If we delete a NPC we should delete his tempeffects as well
	for( teffect_st *Effect = TEffects->First(); !TEffects->AtEnd(); Effect = TEffects->Next() )
	{
		if( Effect->Destination() == k->GetSerial() )
			Effect->Destination( INVALIDSERIAL );
		
		if( Effect->Source() == k->GetSerial() )
			Effect->Source( INVALIDSERIAL );
	}

	// if we delete a NPC we should delete him from spawnregions
	// this will fix several crashes
	if( k->IsNpc() && k->IsSpawned() )
	{
		cSpawnRegion *spawnReg = NULL;

		for( UI16 i = 1; i < cwmWorldState->GetTotalSpawnRegions(); ++i )
		{
			spawnReg = spawnregion[i];

			if( spawnReg == NULL )
				continue;
			else
				spawnReg->deleteSpawnedChar( k );
		}
	}

	for( CItem *tItem = k->FirstItem(); !k->FinishedItems(); tItem = k->NextItem() )
	{
		if( tItem != NULL )
			Items->DeleItem( tItem );
	}
	
	if( !k->IsNpc() )
	{
		ACCOUNTSBLOCK mAcct = k->GetAccount();
		if( mAcct.wAccountIndex != AB_INVALID_ID )
		{
			for( UI08 actr = 0; actr < 5; ++actr )
			{
				if( mAcct.lpCharacters[actr] != NULL && mAcct.lpCharacters[actr]->GetSerial() == k->GetSerial() )
				{
					Accounts->DelCharacter( mAcct.wAccountIndex, actr );
					break;
				}
			}
		}
	}

	UI16 scpNum = k->GetScriptTrigger();
	cScript *tScript = Trigger->GetScript( scpNum );
	if( tScript != NULL )
		tScript->OnDelete( k );

	if( k->GetSpawnObj() != NULL )
		ncspawnsp.Remove( k->GetSpawn(), kChar );

	if( k->IsGuarded() )
	{
		CChar *petGuard = Npcs->getGuardingPet( k, k->GetSerial() );
		if( petGuard != NULL )
			petGuard->SetGuarding( INVALIDSERIAL );
		k->SetGuarded( false );
	}
	stopPetGuarding( k );

	k->RemoveFromSight();

	ncharsp.Remove( k->GetSerial() );
	chars.Delete( kChar );
}

//o---------------------------------------------------------------------------o
//|	Function	-	CChar *MemCharFree( CHARACTER& offset, bool zeroSer )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find a place in memory to hold a new NPC
//o---------------------------------------------------------------------------o
CChar * cCharStuff::MemCharFree( CHARACTER& offset, bool zeroSer )
{
	offset = chars.New( zeroSer );
	if( cwmWorldState->GetCharCount() >= cwmWorldState->GetCMem() ) //Lets ASSUME theres no more memory left instead of acctually checking all the items to find a free one.
		cwmWorldState->IncCMem();
	if( offset == INVALIDSERIAL )
		return NULL;
	return &chars[offset];
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *addRandomLoot( CItem *s, char *lootlist )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Add loot to monsters packs
//o---------------------------------------------------------------------------o
CItem *cCharStuff::addRandomLoot( CItem *s, const char *lootlist )
{
	char sect[512];
	CItem *retitem = NULL;
	sprintf( sect, "LOOTLIST %s", lootlist );
	ScriptSection *LootList = FileLookup->FindEntry( sect, npc_def );
	if( LootList == NULL )
		return NULL;
	size_t i = LootList->NumEntries();
	if( i > 0 )
	{
		i = RandomNum( static_cast< size_t >(0), i - 1 );
		UString tag = LootList->MoveTo( i );
		if( tag.empty() )
			return NULL;
		retitem = Items->CreateScriptItem( NULL, tag.c_str(), false, s->WorldNumber() );
		if( retitem != NULL )
		{
			retitem->SetX( 50 + RandomNum( 0, 79 ) );
			retitem->SetY( 50 + RandomNum( 0, 79 ) );
			retitem->SetZ( 9 );
			retitem->SetCont( s );
		}
	}
	return retitem;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CHARACTER CreateRandomNpc( char * npclist, UI08 worldNumber )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Adds a random NPC
//o---------------------------------------------------------------------------o
CChar *cCharStuff::CreateRandomNpc( const char * npcList, UI08 worldNumber )
{
	char sect[512];
	sprintf( sect, "NPCLIST %s", npcList );
	ScriptSection *NpcList = FileLookup->FindEntry( sect, npc_def );
	if( NpcList == NULL )
		return NULL;
	size_t i = NpcList->NumEntries();
	if( i == 0 )
		return NULL;
	UString k = NpcList->MoveTo( RandomNum( static_cast< size_t >(0), i - 1 ) );

	if( !k.empty() )
		return CreateScriptNpc( k.c_str(), worldNumber );
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CChar *SpawnNPC( CItem *i, int npcNum, UI08 worldNumber )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Spawn an NPC
//o---------------------------------------------------------------------------o
CChar *cCharStuff::SpawnNPC( CItem *i, std::string npcNum, UI08 worldNumber, bool randomNPC = false )
{
	// If the spawner type is 125 and escort quests are not active then abort
	if( i != NULL )
	{
		if( i->GetType() == 125 && !cwmWorldState->ServerData()->EscortsEnabled() )
		  return NULL;
	}
	CChar *c = NULL;
	if( randomNPC )
		c = CreateRandomNpc( npcNum.c_str(), worldNumber );
	else
		c = CreateScriptNpc( npcNum, worldNumber );
	if( c == NULL )
		return NULL;
	CHARACTER mChar = calcCharFromSer( c->GetSerial() );
	c->SetSpawn( INVALIDSERIAL, mChar );
	// see if we are item spawning or region spawning
	if( i != NULL )
	{
		SI16 awayX = 0, awayY = 0;
		if( i->GetType() == 69 && i->GetCont() == NULL )
			awayX = awayY = 10;
		else if( i->GetType() == 125 && i->GetCont() == NULL )
		{
			awayX = i->GetMore( 3 );
			awayY = i->GetMore( 4 );
		}
		FindSpotForNPC( c, i->GetX(), i->GetY(), awayX, awayY, i->GetZ() );
		c->SetSpawn( i->GetSerial(), mChar );
	}
	PostSpawnUpdate( c );
	if( i != NULL )
	{
		if( i->GetType() == 125 )
			MsgBoardQuestEscortCreate( c );
	}
	return c;
}
//o---------------------------------------------------------------------------o
//|	Function	-	CChar *SpawnNPC( cSpawnRegion *spawnReg, int npcNum, UI08 worldNumber )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Spawn an NPC
//o---------------------------------------------------------------------------o
CChar *cCharStuff::SpawnNPC( cSpawnRegion *spawnReg, std::string npcNum, UI08 worldNumber )
{
	CChar *c = CreateScriptNpc( npcNum, worldNumber );
	if( c == NULL )
		return NULL;
	if( spawnReg != NULL )
		c->SetSpawn( calcserial( 0, 1, static_cast<UI08>(spawnReg->GetRegionNum()), 0 ), calcCharFromSer( c->GetSerial() ) );
	else
		c->SetSpawn( INVALIDSERIAL, calcCharFromSer( c->GetSerial() ) );
	// see if we are item spawning or region spawning
	if( spawnReg != NULL )
	{
		SI16 xAway	= ( spawnReg->GetX2() - spawnReg->GetX1() ) / 2;
		SI16 yAway	= ( spawnReg->GetY2() - spawnReg->GetY1() ) / 2;
		SI16 originX	= spawnReg->GetX1() + xAway;
		SI16 originY	= spawnReg->GetY1() + yAway;
		if( xAway <= 0 )
		{
			Console.Error( 2, " Spawn region %d: x1 (%d) must be at least 2 less than x2(%d)", 
				       spawnReg->GetRegionNum() - 1,
				       spawnReg->GetX1(), spawnReg->GetX2());
			xAway = 10;
		}
		else if( yAway <= 0 )
		{
			Console.Error( 2, " Spawn region %d: y1 (%d) must be at least 2 less than y2(%d)", 
				       spawnReg->GetRegionNum() - 1,
				       spawnReg->GetY1(), spawnReg->GetY2());

			yAway = 10;
		}
		FindSpotForNPC( c, originX, originY, xAway, yAway, ILLEGAL_Z );
	}
	PostSpawnUpdate( c );
	return c;
}

void cCharStuff::PostSpawnUpdate( CChar *c )
{
	c->SetRegion( calcRegionFromXY( c->GetX(), c->GetY(), c->WorldNumber() ) );

	for( UI08 z = 0; z < ALLSKILLS; ++z )
		Skills->updateSkillLevel( c, z );

	c->Update();
	MapRegion->AddChar( c );
	setcharflag( c );
	cScript *toGrab = Trigger->GetScript( c->GetScriptTrigger() );
	if( toGrab != NULL )
		toGrab->OnCreate( c );
}
//o---------------------------------------------------------------------------o
//|	Function	-	CChar *AddNPC( cSocket *s, cSpawnRegion *spawnReg, int npcNum, UI08 worldNumber )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Spawn an NPC and place him in the region or on the item that
//|					spawned him.
//o---------------------------------------------------------------------------o
CChar *cCharStuff::AddNPC( cSocket *s, cSpawnRegion *spawnReg, std::string npcNum, UI08 worldNumber )
{
	CChar *c = CreateScriptNpc( npcNum, worldNumber );
	if( c == NULL )
		return NULL;
	if( spawnReg != NULL )
		c->SetSpawn( calcserial( 0, 1, static_cast<UI08>(spawnReg->GetRegionNum()), 0 ), calcCharFromSer( c->GetSerial() ) );
	else
		c->SetSpawn( INVALIDSERIAL, calcCharFromSer( c->GetSerial() ) );
	if( s != NULL )
	{
		const SI16 coreX = s->GetWord( 11 );
		const SI16 coreY = s->GetWord( 13 );
		const SI08 coreZ = s->Buffer()[16] + Map->TileHeight( s->GetWord( 17 ) );
		if( c->GetFx( 1 ) == -1 )
		{
			c->SetFx( coreX, 1 );
			c->SetFy( coreY, 1 );
			if( c->GetFz() != -1 )
				c->SetFz( coreZ );
		}
		c->SetLocation( coreX, coreY, coreZ );
	}
	PostSpawnUpdate( c );
	return c;
}

//o---------------------------------------------------------------------------o
//|	Function	-	SI16 getRadius( CChar *c )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Generate a sensible radius given the values from npc.scp
//o---------------------------------------------------------------------------o
SI16 getRadius( CChar *c )
{
	// see if they supplied a 'radius'
	if( c->GetFx( 2 ) > 0)
	{
		// if they were supplying a bounding area, use the radius from that
		if( c->GetFx( 1 ) > 0)
			c->SetFx( abs(c->GetFx( 1 ) - c->GetFx( 2 )), 2 );
	}
	// ensure its not something bogus
	if( c->GetFx( 2 ) <= 0 || c->GetFx( 2 ) > 100)
		c->SetFx( 10, 2 );
	return c->GetFx( 2 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void InitializeWanderArea( CChar *c, SI16 xAway, SI16 yAway )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Setup the wander area if the npcwander is rect or circle
//o---------------------------------------------------------------------------o
void InitializeWanderArea( CChar *c, SI16 xAway, SI16 yAway )
{
	// compute the rectangular bounding area
	if( 3 == c->GetNpcWander() )
	{
		// if they provided a legal rectangle and
		// ensure the bounding rect contains the current location
		// if it doesn't the monster will never move!
		if( c->GetFx( 1 ) >= 0 && c->GetFy( 1 ) >= 0 && c->GetFy( 2 ) >= 0 && c->GetFx( 2 ) >= 0 &&
			checkBoundingBox( c->GetX(), c->GetY(), c->GetFx( 1 ), c->GetFy( 1 ), c->GetFz(), c->GetFx( 2 ), c->GetFy( 2 ), c->WorldNumber() ) )
		{
			// don't do anything to use what they specified in npc.scp		
		}
		else
		{
			// if they provided a 'radius' in the npc.scp use that
			if( c->GetFx( 2 ) > 0)
				xAway = yAway = getRadius( c );
			// setup info for rectangular areas
			c->SetFx( c->GetX() - xAway, 1 );
			c->SetFy( c->GetY() - yAway, 1 );
			c->SetFx( c->GetX() + xAway, 2 );
			c->SetFy( c->GetX() + yAway, 2 );
		}
	}
	else if( 4 == c->GetNpcWander() )
	{
		// if they provided a legal circle and
		// ensure the bounding circle contains the current location
		// if it doesn't the monster will never move!
		if( c->GetFx( 1 ) >= 0 && c->GetFy( 1 ) >= 0 && c->GetFx( 2 ) >= 0 &&
			checkBoundingCircle( c->GetX(), c->GetY(), c->GetFx( 1 ), c->GetFy( 1 ), c->GetFz(), c->GetFx( 2 ), c->WorldNumber() ) )
		{
			// don't do anything to use what they specified in npc.scp		
		}
		else
		{
			// if they provided a 'radius' in the npc.scp use that
			if( c->GetFx( 2 ) > 0)
				xAway = yAway = getRadius( c );
			// setup info for circular areas
			c->SetFx( c->GetX(), 1 );
			c->SetFy( c->GetY(), 1 );
			c->SetFx( xAway, 2 );
			c->SetFy( -1, 2 );
		}
	}
	// setting fz1 actually makes it check against the height and slows the system down a lot
	// does anyone really need to constrain the height at which a monster can move?? - fur
	//chars[c].fz1 = chars[c].z;
	
#ifdef DEBUG_SPAWN
	printf("Bounding area for this monster is fx1: %d, fx2: %d, fy1: %d, fy2: %d\n", c->GetFx( 1 ), c->GetFx( 2 ), c->GetFy( 1 ), c->GetFy( 2 ) );
#endif
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCharStuff::FindSpotForNPC( CHARACTER c, SI16 originX, SI16 originY, SI16 xAway, SI16 yAway, SI08 z )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find a valid spot to drop an NPC near the spawners location
//|									
//|	Modification	-	04/20/2002	-	Zippy -  (Type 69) xos and yos (X OffSet, Y OffSet)
//|									are used to find a random number that is then added to the spawner's 
//|									x and y (Using the spawner's z) and then place the NPC anywhere in 
//|									a square around the spawner. This square is random anywhere from -10 
//|									to +10 from the spawner's location (for x and y) If the place chosen 
//|									is not a valid position (the NPC can't walk there) then a new place 
//|									will be chosen, if a valid place cannot be found in a certain # of 
//|									tries (50),the NPC will be placed directly on the spawner and the 
//|									server op will be warned. 
//o---------------------------------------------------------------------------o
void cCharStuff::FindSpotForNPC( CChar *c, SI16 originX, SI16 originY, SI16 xAway, SI16 yAway, SI08 z )
{

#ifdef DEBUG_SPAWN
	Console.Print( "Going to spawn at (%d,%d) within %d by %d\n", originX, originY, xAway, yAway );
#endif
	
	if( c == NULL )
		return;
	SI32 k = xAway * yAway / 2;
	SI16 xos = 0, yos = 0;
	bool foundSpot = false;
	if( k > 50 )
		k = 50;
	
	UI08 worldNumber = c->WorldNumber();
	while( !foundSpot )
	{
		if( --k < 0 ) //this CAN be a bit laggy. adjust as nessicary
		{
			if( xAway > 0 && yAway > 0 )
				Console << "Problem area spawner found, NPC placed at default location" << myendl;
			xos = originX;
			yos = originY;
			foundSpot = true;
			break;
		}
	    
		xos = originX + RandomNum( static_cast< SI16 >(-xAway), xAway );
		yos = originY + RandomNum( static_cast< SI16 >(-yAway), yAway );
	    
		if( xos >= 1 && yos >= 1 )
			foundSpot = Map->CanMonsterMoveHere( xos, yos, z, worldNumber );
	}
	
	// should we not add and remove from mapregions here????
	SI08 targZ = 0;
	if( ILLEGAL_Z == z )
		targZ = Map->Height( xos, yos, 0, worldNumber );
	else
		targZ = z;

	c->SetLocation( xos, yos, targZ );
	InitializeWanderArea( c, xAway, yAway );
}


//o---------------------------------------------------------------------------o
//|	Function	-	CHARACTER cCharStuff::AddNPCxyz( cSocket *s, string npcNum, SI16 x1, SI16 y1, SI08 z1, UI08 worldNumber )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Add NPC at a given location
//o---------------------------------------------------------------------------o
CChar * cCharStuff::AddNPCxyz( cSocket *s, std::string npcNum, SI16 x1, SI16 y1, SI08 z1, UI08 worldNumber )
{
	CChar *c = CreateScriptNpc( npcNum, worldNumber );
	if( c == NULL )
		return NULL;
	c->SetLocation( x1, y1, z1 );

	if( s != NULL )
	{
		if( c->GetFx( 1 ) == -1 )
		{
			c->SetFx( s->GetWord( 11 ), 1 );
			c->SetFy( s->GetWord( 13 ), 1 );
			if( c->GetFz() != -1 )
				c->SetFz( s->Buffer()[16] + Map->TileHeight( s->GetWord( 17 ) ) );
		}
	}

	c->SetRegion( calcRegionFromXY(c->GetX(), c->GetY(), worldNumber ) );

	for( UI08 z = 0; z < ALLSKILLS; ++z )
		Skills->updateSkillLevel( c, z );

	c->Update();
	setcharflag( c );
	cScript *toGrab = Trigger->GetScript( c->GetScriptTrigger() );
	if( toGrab != NULL )
		toGrab->OnCreate( c );
	return c;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCharStuff::Split( CChar *k )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Cause NPC to split during combat
//o---------------------------------------------------------------------------o
void cCharStuff::Split( CChar *k )
{
	CChar *c = k->Dupe();
	if( c == NULL )
		return;

	SERIAL serial = c->GetSerial();
	c->SetSerial( serial, calcCharFromSer( c->GetSerial() ) );
	c->SetFTarg( INVALIDSERIAL );
	c->SetLocation( k->GetX() + 1, k->GetY(), k->GetZ() );
	c->SetKills( 0 );
	c->SetHP( k->GetMaxHP() );
	c->SetStamina( k->GetMaxStam() );
	c->SetMana( k->GetMaxMana() );
	if( RandomNum( 0, 34 ) == 5 )
		c->SetSplit( 1 );
	else
		c->SetSplit( 0 );
	c->Update();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCharStuff::LoadShopList( char *list, CChar *c )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Loads the shopping list pointed to by data in items.scp
//o---------------------------------------------------------------------------o
void cCharStuff::LoadShopList( const char *list, CChar *c )
{
	CItem *buyLayer		= c->GetItemAtLayer( 0x1A );
	CItem *boughtLayer	= c->GetItemAtLayer( 0x1B );
	CItem *sellLayer	= c->GetItemAtLayer( 0x1C );

	char sect[512];
	sprintf( sect, "SHOPLIST %s", list );
	ScriptSection *ShoppingList = FileLookup->FindEntry( sect, items_def );
	if( ShoppingList == NULL )
		return;

	DFNTAGS tag = DFNTAG_COUNTOFTAGS;
	UString cdata;
	UI32 ndata = INVALIDSERIAL, odata = INVALIDSERIAL;
	CItem *retitem = NULL;

	for( tag = ShoppingList->FirstTag(); !ShoppingList->AtEndTags(); tag = ShoppingList->NextTag() )
	{
		cdata = ShoppingList->GrabData( ndata, odata );
		switch( tag )
		{
			case DFNTAG_RSHOPITEM:
				if( buyLayer != NULL )
				{
					retitem = Items->CreateScriptItem( NULL, cdata.c_str(), false, c->WorldNumber() );
					if( retitem != NULL )
					{
						retitem->SetCont( buyLayer );
						retitem->SetX( 50 + ( RandomNum( 0, 79 ) ) );
						retitem->SetY( 50 + ( RandomNum( 0, 79 ) ) );
						retitem->SetZ( 9 );
						if( retitem->GetName2()[0] && ( strcmp( retitem->GetName2(), "#" ) ) )
							retitem->SetName( retitem->GetName2() ); // Item identified! -- by Magius(CHE)
					}
				}
				else
					Console << "Warning: Bad Shopping List " << list << " with no Vendor Buy Pack for NPC " << c << " (serial: " << c->GetSerial() << myendl;
				break;
			case DFNTAG_SELLITEM:
				if( sellLayer != NULL )
				{
					retitem = Items->CreateScriptItem( NULL, cdata.c_str(), false, c->WorldNumber() );
					if( retitem != NULL )
					{
						retitem->SetCont( sellLayer );
						retitem->SetSellValue( retitem->GetBuyValue() / 2 );
						retitem->SetX( 50 + ( RandomNum( 0, 79 ) ) );
						retitem->SetY( 50 + ( RandomNum( 0, 79 ) ) );
						retitem->SetZ( 9 );
						if( retitem->GetName2()[0] && ( strcmp( retitem->GetName2(), "#" ) ) )
							retitem->SetName( retitem->GetName2() );
					}
				}
				else
					Console << "Warning: Bad Shopping List " << list << " with no Vendor Sell Pack for NPC " << c << " (serial: " << c->GetSerial() << myendl;
				break;
			case DFNTAG_SHOPITEM:
				if( boughtLayer != NULL )
				{
					retitem = Items->CreateScriptItem( NULL, cdata.c_str(), false, c->WorldNumber() );
					if( retitem != NULL )
					{
						retitem->SetCont( boughtLayer );
						retitem->SetX( 50 + ( RandomNum( 0, 79 ) ) );
						retitem->SetY( 50 + ( RandomNum( 0, 79 ) ) );
						retitem->SetZ( 9 );
						if( retitem->GetName2()[0] && ( strcmp( retitem->GetName2(), "#" ) ) )
							retitem->SetName( retitem->GetName2() );
					}
				}
				else
					Console << "Warning: Bad Shopping List " << list << " with no Vendor Bought Pack for NPC " << c << " (serial: " << c->GetSerial() << myendl;
				break;
			case DFNTAG_VALUE:
#if defined( _MSC_VER )
#pragma note( "Verify that the ustring stuff works well here" )
#endif
				if( retitem != NULL )
				{
					if( !cdata.empty() )
					{
						if( cdata.sectionCount( " " ) != 0 )
						{
							retitem->SetBuyValue(  cdata.section( " ", 0, 0 ).stripWhiteSpace().toShort() );
							retitem->SetSellValue( cdata.section( " ", 1, 1 ).stripWhiteSpace().toShort() );
							break;
						}
					}
					retitem->SetBuyValue( ndata );
					retitem->SetSellValue( (ndata / 2) );
				}
				break;
			default:
				break;
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool cCharStuff::ApplyNpcSection( CChar *applyTo, ScriptSection *NpcCreation )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Apply Npc.scp sections to an NPC
//o---------------------------------------------------------------------------o
bool cCharStuff::ApplyNpcSection( CChar *applyTo, ScriptSection *NpcCreation )
{
	if( NpcCreation == NULL || applyTo == NULL )
		return false;

	UI16 haircolor	= INVALIDCOLOUR;
	UI16 storeval	= 0xFFFF;

	CItem *buyPack = NULL, *boughtPack = NULL, *sellPack = NULL;
	CItem *retitem = NULL, *mypack = NULL;

	DFNTAGS tag = DFNTAG_COUNTOFTAGS;
	UString cdata;
	UI32 ndata = INVALIDSERIAL, odata = INVALIDSERIAL;
	UI08 skillToSet = 0;
	for( tag = NpcCreation->FirstTag(); !NpcCreation->AtEndTags(); tag = NpcCreation->NextTag() )
	{
		cdata = NpcCreation->GrabData( ndata, odata );
		switch( tag )
		{
			case DFNTAG_ALCHEMY:			skillToSet = ALCHEMY;				break;
			case DFNTAG_ANATOMY:			skillToSet = ANATOMY;				break;
			case DFNTAG_ANIMALLORE: 		skillToSet = ANIMALLORE;			break;
			case DFNTAG_ARMSLORE:			skillToSet = ARMSLORE;				break;
			case DFNTAG_ARCHERY:			skillToSet = ARCHERY;				break;
			case DFNTAG_DAMAGE:
			case DFNTAG_ATT:
											applyTo->SetLoDamage( (SI16)ndata );
											applyTo->SetHiDamage( (SI16)odata );
											break;
			case DFNTAG_BACKPACK:
											if( mypack == NULL )
												mypack = getPack( applyTo );
											if( mypack == NULL )
											{
												mypack = Items->SpawnItem( NULL, applyTo, 1, "Backpack", false, 0x0E75, 0, false, false );
												if( mypack != NULL )
												{
													applyTo->SetPackItem( mypack );
													mypack->SetX( 0 );
													mypack->SetY( 0 );
													mypack->SetZ( 0 );
													mypack->SetLayer( 0x15 );
													if( !mypack->SetCont( applyTo ) )
														mypack = NULL;
													else
													{
														mypack->SetType( 1 );
														mypack->SetDye( true );
													}
												}
											}
											break;
			case DFNTAG_BEGGING:			skillToSet = BEGGING;				break;
			case DFNTAG_BLACKSMITHING:		skillToSet = BLACKSMITHING;			break;
			case DFNTAG_BOWCRAFT:			skillToSet = BOWCRAFT;				break;
			case DFNTAG_COLOUR:
											if( retitem != NULL )
												retitem->SetColour( (UI16)ndata );
											break;
			case DFNTAG_CAMPING:			skillToSet = CAMPING;				break;
			case DFNTAG_CARPENTRY:			skillToSet = CARPENTRY;				break;
			case DFNTAG_CARTOGRAPHY:		skillToSet = CARTOGRAPHY;			break;
	#ifdef __NEW_SKILLS__
			case DFNTAG_CHIVALRY:			skillToSet = CHIVALRY;				break;
	#endif
			case DFNTAG_COOKING:			skillToSet = COOKING;				break;
			case DFNTAG_COLOURMATCHHAIR:
											if( retitem != NULL )
												retitem->SetColour( (UI16)haircolor );
											break;
			case DFNTAG_COLOURLIST:
											storeval = addRandomColor( cdata.c_str() );
											if( retitem != NULL )
												retitem->SetColour( storeval );
											break;
			case DFNTAG_CARVE:
											applyTo->SetCarve( static_cast<SI16>(ndata) );	break;
			case DFNTAG_DIR:
											if( cdata.upper() == "NE" )
												applyTo->SetDir( 1 );
											else if( cdata.upper() == "E" )
												applyTo->SetDir( 2 );
											else if( cdata.upper() == "SE" )
												applyTo->SetDir( 3 );
											else if( cdata.upper() == "S" )
												applyTo->SetDir( 4 );
											else if( cdata.upper() == "SW" )
												applyTo->SetDir( 5 );
											else if( cdata.upper() == "W" )
												applyTo->SetDir( 6 );
											else if( cdata.upper() == "NW" )
												applyTo->SetDir( 7 );
											else if( cdata.upper() == "N" )
												applyTo->SetDir( 0 );
											break;
			case DFNTAG_DEX:
											applyTo->SetDexterity( (SI16)RandomNum( ndata, odata ) );
											applyTo->SetStamina( applyTo->GetMaxStam() );
											break;
			case DFNTAG_DETECTINGHIDDEN:	skillToSet = DETECTINGHIDDEN;			break;
			case DFNTAG_DEF:				applyTo->SetDef( (UI16)RandomNum( ndata, odata ) ); break;
			case DFNTAG_EMOTECOLOUR:		applyTo->SetEmoteColour( (UI16)ndata ); break;
			case DFNTAG_ENTICEMENT:			skillToSet = ENTICEMENT;				break;
			case DFNTAG_EVALUATINGINTEL:	skillToSet = EVALUATINGINTEL;			break;
			case DFNTAG_FAME:				applyTo->SetFame( (SI16)ndata );		break;
			case DFNTAG_FX1:				applyTo->SetFx( (SI16)ndata, 1 );		break;
			case DFNTAG_FX2:				applyTo->SetFx( (SI16)ndata, 2 );		break;
			case DFNTAG_FY1:				applyTo->SetFy( (SI16)ndata, 1 );		break;
			case DFNTAG_FY2:				applyTo->SetFy( (SI16)ndata, 2 );		break;
			case DFNTAG_FZ1:				applyTo->SetFz( (SI08)ndata );			break;
			case DFNTAG_FLEEAT:				applyTo->SetFleeAt( (SI16)ndata );		break;
			case DFNTAG_FISHING:			skillToSet = FISHING;					break;
			case DFNTAG_FORENSICS:			skillToSet = FORENSICS;					break;
			case DFNTAG_FENCING:			skillToSet = FENCING;					break;
	#ifdef __NEW_SKILLS__
			case DFNTAG_FOCUS:				skillToSet = FOCUS;						break;
	#endif
			case DFNTAG_GOLD:
											if( mypack == NULL )
												mypack = getPack( applyTo );
											if( mypack != NULL )
											{
												retitem = Items->SpawnItem( NULL, applyTo, 1, "#", true, 0x0EED, 0, true, false );
												if( retitem != NULL )
												{
													retitem->SetDecayable( true );
													retitem->SetAmount( (UI16)( RandomNum( (int)(ndata), (int)(odata) ) ) );
													retitem->SetCont( mypack );
												}
											}
											else
												Console.Warning( 2, "Bad NPC Script with problem no backpack for gold" );
											break;
			case DFNTAG_GET:
			{
											ScriptSection *toFind = FileLookup->FindEntry( cdata.c_str(), npc_def );
											ApplyNpcSection( applyTo, toFind );
			}
											break;
			case DFNTAG_HEALING:			skillToSet = HEALING;					break;
			case DFNTAG_HERDING:			skillToSet = HERDING;					break;
			case DFNTAG_HIDING:				skillToSet = HIDING;					break;
			case DFNTAG_HAIRCOLOUR:
											haircolor = addRandomColor( cdata.c_str() );
											if( retitem != NULL )
												retitem->SetColour( haircolor );
											break;
			case DFNTAG_HIDAMAGE:			applyTo->SetHiDamage( (SI16)ndata );	break;
			case DFNTAG_HP:					applyTo->SetHP( (SI16)ndata );			break;
			case DFNTAG_ID:
											applyTo->SetID( (UI16)ndata );
											applyTo->SetOrgID( (UI16)ndata );
											break;
			case DFNTAG_EQUIPITEM:
											retitem = Items->CreateScriptItem( NULL, cdata.c_str(), false, applyTo->WorldNumber() );
											if( retitem != NULL )
											{
												if( !retitem->SetCont( applyTo ) )
													retitem = NULL;
												else if( retitem->GetLayer() == 0 )
													Console << "Warning: Bad NPC Script with problem item " << cdata << " executed!" << myendl;
											}
											break;
			case DFNTAG_INTELLIGENCE:
											applyTo->SetIntelligence( (SI16)RandomNum( ndata, odata ) );
											applyTo->SetMana( applyTo->GetMaxMana() );
											break;
			case DFNTAG_ITEMID:				skillToSet = ITEMID;					break;
			case DFNTAG_INSCRIPTION:		skillToSet = INSCRIPTION;				break;
			case DFNTAG_KARMA:				applyTo->SetKarma( (SI16)ndata );		break;
			case DFNTAG_LOOT:
											if( mypack == NULL )
												mypack = getPack( applyTo );
											if( mypack != NULL )
												retitem = addRandomLoot( mypack, cdata.c_str() );
											else
												Console.Warning( 2, "Bad NPC Script with problem no backpack for loot" );
											break;
			case DFNTAG_LOCKPICKING:		skillToSet = LOCKPICKING;				break;
			case DFNTAG_LUMBERJACKING:		skillToSet = LUMBERJACKING;				break;
			case DFNTAG_LODAMAGE:			applyTo->SetLoDamage( (SI16)ndata );	break;
			case DFNTAG_MAGERY:				skillToSet = MAGERY;					break;
			case DFNTAG_MAGICRESISTANCE:	skillToSet = MAGICRESISTANCE;			break;
			case DFNTAG_MUSICIANSHIP:		skillToSet = MUSICIANSHIP;				break;
			case DFNTAG_MACEFIGHTING:		skillToSet = MACEFIGHTING;				break;
			case DFNTAG_MANA:				applyTo->SetMana( (UI16)ndata );		break;
			case DFNTAG_MINING:				skillToSet = MINING;					break;
			case DFNTAG_MEDITATION:			skillToSet = MEDITATION;				break;
			case DFNTAG_NAME:				applyTo->SetName( cdata.c_str() );		break;
			case DFNTAG_NAMELIST:			setRandomName( applyTo, cdata.c_str() );		break;
	#ifdef __NEW_SKILLS__
			case DFNTAG_NECROMANCY:			skillToSet = NECROMANCY;				break;
	#endif
			case DFNTAG_NPCWANDER:			applyTo->SetNpcWander( (SI08)ndata );	break;
			case DFNTAG_NPCAI:				applyTo->SetNPCAiType( (SI16)ndata );	break;
			case DFNTAG_NOTRAIN:			applyTo->SetCanTrain( false );			break;
			case DFNTAG_PACKITEM:
											if( mypack == NULL )
												mypack = getPack( applyTo );
											if( mypack != NULL )
											{
#if defined( _MSC_VER )
#pragma note( "Verify that the ustring stuff works well here" )
#endif
												if( !cdata.empty() )
												{
													if( cdata.sectionCount( "," ) != 0 )
													{
														retitem = Items->SpawnItemToPack( NULL, applyTo, cdata.section( ",", 0, 0 ).stripWhiteSpace(), false );
														if( retitem != NULL )
														{
															if( retitem->isPileable() )
																retitem->SetAmount( cdata.section( ",", 1, 1 ).stripWhiteSpace().toUShort() );
														}
													}
													else
														retitem = Items->SpawnItemToPack( NULL, applyTo, cdata.c_str(), false );
												}
											}
											else
												Console << "Warning: Bad NPC Script with problem no backpack for packitem" << myendl;
											break;
			case DFNTAG_POISONSTRENGTH:		applyTo->SetPoisonStrength( (UI08)ndata ); break;
			case DFNTAG_PRIV1:				applyTo->SetPriv( static_cast<UI16>(ndata) ); break;
			case DFNTAG_PRIV2:														break;
			case DFNTAG_PARRYING:			skillToSet = PARRYING;					break;
			case DFNTAG_PEACEMAKING:		skillToSet = PEACEMAKING;				break;
			case DFNTAG_PROVOCATION:		skillToSet = PROVOCATION;				break;
			case DFNTAG_POISONING:			skillToSet = POISONING;					break;
			case DFNTAG_RSHOPITEM:
											if( buyPack == NULL )
												buyPack = applyTo->GetItemAtLayer( 0x1A );
											if( buyPack != NULL )
											{
												retitem = Items->CreateScriptItem( NULL, cdata, false, applyTo->WorldNumber() );
												if( retitem != NULL )
												{
													retitem->SetCont( buyPack );
													retitem->SetX( 50 + RandomNum( 0, 79 ) );
													retitem->SetY( 50 + RandomNum( 0, 79 ) );
													retitem->SetZ( 9 );
													if( retitem->GetName2()[0] && ( strcmp( retitem->GetName2(), "#" )))
														retitem->SetName( retitem->GetName2() ); // Item identified! -- by Magius(CHE)
												}
											}
											else
												Console.Warning( 2, "Bad NPC Script with no Vendor Buy Pack for item" );
											break;
			case DFNTAG_REATTACKAT:			applyTo->SetReattackAt( (SI16)ndata );	break;
			case DFNTAG_REMOVETRAPS:		skillToSet = REMOVETRAPS;				break;
			case DFNTAG_RACE:				applyTo->SetRace( (UI16)ndata );		break;
			case DFNTAG_RUNS:				applyTo->SetRun( true );				break;
			case DFNTAG_SKIN:				applyTo->SetSkin( (UI16)ndata );		break;
			case DFNTAG_SHOPKEEPER:			Commands->MakeShop( applyTo );			break;
			case DFNTAG_SHOPLIST:			LoadShopList( cdata.c_str(), applyTo );	break;
			case DFNTAG_SELLITEM:
											if( sellPack == NULL )
												sellPack = applyTo->GetItemAtLayer( 0x1C );
											if( sellPack != NULL )
											{
												retitem = Items->CreateScriptItem( NULL, cdata, false, applyTo->WorldNumber() );
												if( retitem != NULL )
												{
													retitem->SetCont( sellPack );
													//retitem->SetValue( retitem->GetValue() / 2 );
													retitem->SetX( 50 + RandomNum( 0, 79 ) );
													retitem->SetY( 50 + RandomNum( 0, 79 ) );
													retitem->SetZ( 9 );
													if( retitem->GetName2()[0] && ( strcmp( retitem->GetName2(), "#" ) ) )
														retitem->SetName( retitem->GetName2() );
												}
											}
											else
												Console.Warning( 2, "Bad NPC Script with no Vendor SellPack for item" );
											break;
			case DFNTAG_SHOPITEM:
											if( boughtPack == NULL )
												boughtPack = applyTo->GetItemAtLayer( 0x1B );
											if( boughtPack != NULL )
											{
												retitem = Items->CreateScriptItem( NULL, cdata, false, applyTo->WorldNumber() );
												if( retitem != NULL )
												{
													retitem->SetCont( boughtPack );
													retitem->SetX( 50 + RandomNum( 0, 79 ) );
													retitem->SetY( 50 + RandomNum( 0, 79 ) );
													retitem->SetZ( 9 );
													if( retitem->GetName2()[0] && ( strcmp( retitem->GetName2(), "#" ) ) )
														retitem->SetName( retitem->GetName2() );
												}
											}
											else
												Console.Warning( 2, "Bad NPC Script with no Vendor Bought Pack for item" );
											break;
			case DFNTAG_SAYCOLOUR:			applyTo->SetSayColour( (UI16)ndata );	break;
			case DFNTAG_SPATTACK:			applyTo->SetSpAttack( (SI16)ndata );	break;
			case DFNTAG_SPADELAY:			applyTo->SetSpDelay( (SI08)ndata );		break;
			case DFNTAG_SPLIT:				applyTo->SetSplit( (UI08)ndata );		break;
			case DFNTAG_SPLITCHANCE:		applyTo->SetSplitChance( (UI08)ndata ); break;
			case DFNTAG_SNOOPING:			skillToSet = SNOOPING;					break;
			case DFNTAG_SPIRITSPEAK:		skillToSet = SPIRITSPEAK;				break;
			case DFNTAG_STEALING:			skillToSet = STEALING;					break;
			case DFNTAG_STRENGTH:
											applyTo->SetStrength( (SI16)RandomNum( ndata, odata ) );
											applyTo->SetHP( applyTo->GetMaxHP() );
											break;
			case DFNTAG_SWORDSMANSHIP:		skillToSet = SWORDSMANSHIP;				break;
			case DFNTAG_STAMINA:			applyTo->SetStamina( (UI16)ndata );		break;
			case DFNTAG_STEALTH:			skillToSet = STEALTH;					break;
			case DFNTAG_SKINLIST:
											storeval = addRandomColor( cdata.c_str() );
											applyTo->SetSkin( storeval );
											break;
			case DFNTAG_SKILL:				applyTo->SetBaseSkill( (UI16)odata, (UI08)ndata ); break;
			case DFNTAG_SCRIPT:				applyTo->SetScriptTrigger( (UI16)ndata ); break;
			case DFNTAG_TITLE:				applyTo->SetTitle( cdata.c_str() );		break;
			case DFNTAG_TOTAME:				applyTo->SetTaming( (SI16)ndata );		break;
			case DFNTAG_TACTICS:			skillToSet = TACTICS;					break;
			case DFNTAG_TAILORING:			skillToSet = TAILORING;					break;
			case DFNTAG_TAMING:				skillToSet = TAMING;					break;
			case DFNTAG_TASTEID:			skillToSet = TASTEID;					break;
			case DFNTAG_TINKERING:			skillToSet = TINKERING;					break;
			case DFNTAG_TRACKING:			skillToSet = TRACKING;					break;
			case DFNTAG_VETERINARY:			skillToSet = VETERINARY;				break;
			case DFNTAG_WRESTLING:			skillToSet = WRESTLING;					break;
			case DFNTAG_ALIGNMENT:
			case DFNTAG_MOD1:
			case DFNTAG_MOD2:
			case DFNTAG_MOD3:
											break;
			case DFNTAG_NOTES:
			case DFNTAG_CATEGORY:
											break;
			default:						Console << "Unknown tag in ApplyNpcSection(): " << (SI32)tag << myendl; break;
		}
		if( skillToSet != 0 )
		{
			applyTo->SetBaseSkill( static_cast<UI16>(RandomNum( ndata, odata )), skillToSet );
			skillToSet = 0;
		}
	}
	return true;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CChar *CreateScriptNpc( string targNPC, UI08 worldNumber )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Create an NPC and assign their variables based on npc.scp entrys
//o---------------------------------------------------------------------------o
CChar *cCharStuff::CreateScriptNpc( std::string targNPC, UI08 worldNumber )
{
	ScriptSection *NpcCreation = FileLookup->FindEntry( targNPC.c_str(), npc_def );
	if( NpcCreation == NULL )
		return NULL;

	if( NpcCreation->NpcListExist() )
		return CreateRandomNpc( NpcCreation->NpcListData().c_str(), worldNumber );

	CHARACTER npcNumOff;
	CChar *npcNum = MemCharFree( npcNumOff );
	if( npcNum == NULL )
		return NULL;

	npcNum->SetSkillTitles( true );
	npcNum->SetNpc( true );
	npcNum->SetLoDamage( 1 );
	npcNum->SetHiDamage( 1 );
	npcNum->SetDef( 1 );
	npcNum->SetSpawn( INVALIDSERIAL, npcNumOff );
	npcNum->WorldNumber( worldNumber );

	if( !ApplyNpcSection( npcNum, NpcCreation ) )
		std::cout << "Failed to apply NPC script settings for npc " << npcNum->GetSerial() << std::endl;
	
	return npcNum;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void npcAction( CChar *npc, SI16 x )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	NPC does a certain action
//o---------------------------------------------------------------------------o
void npcAction( CChar *npc, SI16 x )
{
	CPCharacterAnimation toSend = (*npc);
	toSend.Action( x );
	Network->PushConn();
	for( cSocket *tSock = Network->FirstSocket(); 
	     !Network->FinishedSockets(); tSock = Network->NextSocket() )
	{
		if( charInRange( tSock->CurrcharObj(), npc ) )
			tSock->Send( &toSend );
	}
	Network->PopConn();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void npcAct( cSocket *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	NPC Action
//o---------------------------------------------------------------------------o
void npcAct( cSocket *s )
{
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		npcAction( i, s->AddID1() );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void npcToggleCombat( CChar *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	NPC Toggles combat mode
//o---------------------------------------------------------------------------o
void npcToggleCombat( CChar *s )
{
	s->SetWar( !s->IsAtWar() );
	Movement->CombatWalk( s );
}


//o---------------------------------------------------------------------------o
//|	Function	-	CChar * cCharStuff::getGuardingPet( CChar *mChar, SERIAL guarded )
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get the pet guarding an item / character
//o---------------------------------------------------------------------------o
CChar * cCharStuff::getGuardingPet( CChar *mChar, SERIAL guarded )
{
	if( mChar == NULL || guarded == INVALIDSERIAL )
		return NULL;

	CHARLIST *myPets = mChar->GetPetList();
	// DO the STL
	CHARLIST_ITERATOR I;
	for(I=myPets->begin();I!=myPets->end();++I)
	{
		CChar * pet;
		pet=(*I);
	//if( myPets != NULL )
	//{
	//	for( UI32 j = 0; j < myPets->size(); ++j )
	//	{
	//		CChar *pet = (*myPets)[j];
	//		if( pet != NULL )
	//		{
				if( !pet->IsMounted() && pet->GetNPCAiType() == aiPET_GUARD && 
					pet->GetGuarding() == guarded && pet->GetOwnerObj() == mChar )
					return pet;
	//		}
	//	}
	}
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool cCharStuff::checkPetFriend( CChar *mChar, CChar *pet )
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Search a pets friends list to check if a character is a friend
//o---------------------------------------------------------------------------o
bool cCharStuff::checkPetFriend( CChar *mChar, CChar *pet )
{
	CHARLIST *petFriends = pet->GetFriendList();
	// DO the stl
	CHARLIST_ITERATOR I;
	for(I=petFriends->begin();I!=petFriends->end();++I)
	{
		CChar * getFriend;
		getFriend=(*I);

	//if( petFriends != NULL )
	//{
	//	for( UI32 j = 0; j < petFriends->size(); ++j )
	//	{
	//		CChar *getFriend = (*petFriends)[j];
			if( getFriend != NULL && getFriend == mChar )
				return true;
	//	}
	}
	return false;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cCharStuff::stopPetGuarding( CChar *pet )
//|	Programmer	-	Zane
//o---------------------------------------------------------------------------o
//|	Purpose		-	Find the item/char pet is guarding and set it to not guarded
//o---------------------------------------------------------------------------o
void cCharStuff::stopPetGuarding( CChar *pet )
{
	SERIAL petGuarding = pet->GetGuarding();
	if( petGuarding == INVALIDSERIAL )
		return;

	CItem *itemGuard = calcItemObjFromSer( petGuarding );
	if( itemGuard != NULL )
		itemGuard->SetGuarded( false );
	else
	{
		CChar *charGuard = calcCharObjFromSer( petGuarding );
		if( charGuard != NULL )
			charGuard->SetGuarded( false );
	}
	pet->SetGuarding( INVALIDSERIAL );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void setRandomName( CChar *s, char *namelist )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Sets a character with a random name from NPC.scp namelist
//o---------------------------------------------------------------------------o
void setRandomName( CChar *s, const char *namelist )
{
	char sect[512];
	
	sprintf( sect, "RANDOMNAME %s", namelist );
	
	char tempName[512];

	ScriptSection *RandomName = FileLookup->FindEntry( sect, npc_def );
	if( RandomName == NULL )
	{
		sprintf( tempName, "Error Namelist %s Not Found", namelist );
		s->SetName( tempName );
		return;
	}
	
	size_t i = RandomName->NumEntries();

	sprintf( tempName, "namecount %i", i );
	s->SetName( tempName );
	if( i > 0 )
	{
		i = rand()%(i);
		UString tag = RandomName->MoveTo( (SI16)i );
		s->SetName( tag.c_str() );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	int addRandomColor( CChar *s, char *colorlist )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Randomly colors character
//o---------------------------------------------------------------------------o
UI16 addRandomColor( const char *colorlist )
{
	char sect[512];
	sprintf( sect, "RANDOMCOLOR %s", colorlist );
	ScriptSection *RandomColours = FileLookup->FindEntry( sect, colors_def );
	if( RandomColours == NULL )
	{
		Console.Warning( 2, "Error Colorlist %s Not Found", colorlist );
		return 0;
	}
	size_t i = RandomColours->NumEntries();
	if( i > 0 )
	{
		i = rand()%i;
		UString tag = RandomColours->MoveTo( i );
		return tag.toUShort();
	}
	return 0;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void MonsterGate( CChar *s, SI32 x )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle monster gates (polymorphs players into monster bodies)
//o---------------------------------------------------------------------------o
void MonsterGate( CChar *s, SI32 x )
{
	CItem *mypack = NULL, *retitem = NULL;
	if( s->IsNpc() ) 
		return;

	UString sect = UString::number( x );
	ScriptSection *Monster = FileLookup->FindEntry( sect, npc_def );
	if( Monster == NULL )
		return;
	
	s->SetTitle( "\0" );

	CItem *n;
	for( CItem *z = s->FirstItem(); !s->FinishedItems(); z = s->NextItem() )
	{
		if( z != NULL )
		{
			if( z->isFree() )
				continue;
			if( z->GetLayer() != 0x15 && z->GetLayer() != 0x1D && z->GetLayer() != 0x10 && z->GetLayer() != 0x0B )
			{
				if( mypack == NULL )
					mypack = getPack( s );
				if( mypack == NULL )
				{
					n = Items->SpawnItem( NULL, s, 1, "#", false, 0x0E75, 0, false, false );
					s->SetPackItem( n );
					if( n == NULL ) 
						return;
					n->SetLayer( 0x15 );
					if( n->SetCont( s ) )
					{
						n->SetType( 1 );
						n->SetDye( true );
						mypack = n;
						retitem = n;
					}
				}
				z->SetX( (SI16)( ( RandomNum( 0, 79 ) ) + 50 ) );
				z->SetY( (SI16)( ( RandomNum( 0, 79 ) ) + 50 ) );
				z->SetZ( 9 );
				z->SetCont( mypack );
				
			}
			else if( z->GetLayer() == 0x0B || z->GetLayer() == 0x10 )
				Items->DeleItem( z );
		}
	}
	
	DFNTAGS tag = DFNTAG_COUNTOFTAGS;
	UString cdata;
	UI32 ndata = INVALIDSERIAL, odata = INVALIDSERIAL;
	UI08 skillToSet = 0;
	for( tag = Monster->FirstTag(); !Monster->AtEndTags(); tag = Monster->NextTag() )
	{
		cdata = Monster->GrabData( ndata, odata );
		switch( tag )
		{
			case DFNTAG_ALCHEMY:			skillToSet = ALCHEMY;				break;
			case DFNTAG_ANATOMY:			skillToSet = ANATOMY;				break;
			case DFNTAG_ANIMALLORE:			skillToSet = ANIMALLORE;			break;
			case DFNTAG_ARMSLORE:			skillToSet = ARMSLORE;				break;
			case DFNTAG_ARCHERY:			skillToSet = ARCHERY;				break;
			case DFNTAG_BEGGING:			skillToSet = BEGGING;				break;
			case DFNTAG_BLACKSMITHING:		skillToSet = BLACKSMITHING;			break;
			case DFNTAG_BOWCRAFT:			skillToSet = BOWCRAFT;				break;
			case DFNTAG_DAMAGE:
			case DFNTAG_ATT:				s->SetLoDamage( static_cast<SI16>(ndata) );	s->SetHiDamage( static_cast<SI16>(odata ));		break;
			case DFNTAG_COLOUR:				if( retitem != NULL )
												retitem->SetColour( static_cast<UI16>(ndata ));
											break;
			case DFNTAG_CAMPING:			skillToSet =  CAMPING;				break;
			case DFNTAG_CARPENTRY:			skillToSet =  CARPENTRY;			break;
			case DFNTAG_CARTOGRAPHY:		skillToSet =  CARTOGRAPHY;			break;
	#ifdef __NEW_SKILLS__
			case DFNTAG_CHIVALRY:			skillToSet =  CHIVALRY;				break;
	#endif
			case DFNTAG_COOKING:			skillToSet =  COOKING;				break;
			case DFNTAG_DEX:				s->SetDexterity( static_cast< SI16 >(RandomNum( ndata, odata )) );
											s->SetStamina( (SI16)s->GetMaxStam() );
											break;
			case DFNTAG_DEF:				s->SetDef( static_cast< UI16 >(RandomNum( ndata, odata )) );	break;
			case DFNTAG_DETECTINGHIDDEN:	skillToSet =  DETECTINGHIDDEN;		break;
			case DFNTAG_ENTICEMENT:			skillToSet =  ENTICEMENT;			break;
			case DFNTAG_EVALUATINGINTEL:	skillToSet =  EVALUATINGINTEL;		break;
			case DFNTAG_FAME:				s->SetFame( static_cast<SI16>(ndata) );	break;
			case DFNTAG_FISHING:			skillToSet =  FISHING;				break;
			case DFNTAG_FORENSICS:			skillToSet =  FORENSICS;			break;
			case DFNTAG_FENCING:			skillToSet =  FENCING;				break;
	#ifdef __NEW_SKILLS__
			case DFNTAG_FOCUS:				skillToSet =  FOCUS;				break;
	#endif
			case DFNTAG_GOLD:				retitem = n = Items->SpawnItem( NULL, s, 1, "#", true, 0x0EED, 0, true, false );
											if( n == NULL ) 
												break;
											n->SetAmount( (UI16)RandomNum( ndata, odata ) );
											break;
			case DFNTAG_HIDAMAGE:			s->SetHiDamage( static_cast<SI16>(ndata) );										break;
			case DFNTAG_HEALING:			skillToSet =  HEALING;				break;
			case DFNTAG_HERDING:			skillToSet =  HERDING;				break;
			case DFNTAG_HIDING:				skillToSet =  HIDING;				break;
			case DFNTAG_HP:					s->SetHP( static_cast<SI16>(ndata) );
			case DFNTAG_ID:					s->SetID( static_cast<UI16>(ndata) );
											s->SetOrgID( static_cast<UI16>(ndata) );
											break;
			case DFNTAG_EQUIPITEM:			retitem = Items->CreateScriptItem( NULL, cdata.c_str(), false, s->WorldNumber() );
											if( retitem != NULL )
											{
												if( !retitem->SetCont( s ) )
													retitem = NULL;
												else if( retitem->GetLayer() == 0 )
													Console << "Warning: Bad NPC Script " << x << " with problem item " << cdata << " executed!" << myendl;
											}
											break;
			case DFNTAG_INTELLIGENCE:		s->SetIntelligence( static_cast< SI16 >(RandomNum( ndata, odata )) );
											s->SetMana( s->GetMaxMana() );
											break;
			case DFNTAG_ITEMID:				skillToSet =  ITEMID;				break;
			case DFNTAG_INSCRIPTION:		skillToSet =  INSCRIPTION;			break;
			case DFNTAG_KARMA:				s->SetKarma( static_cast<SI16>(ndata) );
			case DFNTAG_LOOT:				retitem = Npcs->addRandomLoot( mypack, cdata.c_str() );
											break;
			case DFNTAG_LODAMAGE:			s->SetLoDamage( static_cast<SI16>(ndata) );		break;
			case DFNTAG_LOCKPICKING:		skillToSet =  LOCKPICKING;			break;
			case DFNTAG_LUMBERJACKING:		skillToSet =  LUMBERJACKING;		break;
			case DFNTAG_MACEFIGHTING:		skillToSet =  MACEFIGHTING;			break;
			case DFNTAG_MINING:				skillToSet =  MINING;				break;
			case DFNTAG_MEDITATION:			skillToSet =  MEDITATION;			break;
			case DFNTAG_MAGERY:				skillToSet =  MAGERY;				break;
			case DFNTAG_MAGICRESISTANCE:	skillToSet =  MAGICRESISTANCE;		break;
			case DFNTAG_MUSICIANSHIP:		skillToSet =  MUSICIANSHIP;			break;
			case DFNTAG_NAME:				s->SetName( cdata.c_str() );		break;
			case DFNTAG_NAMELIST:			setRandomName( s, cdata.c_str() );	break;
	#ifdef __NEW_SKILLS__
			case DFNTAG_NECROMANCY:			skillToSet = NECROMANCY;			break;
	#endif
			case DFNTAG_PACKITEM:			
#if defined( _MSC_VER )
#pragma note( "Verify that the ustring stuff works well here" )
#endif
											if( !cdata.empty() )
											{
												if( cdata.sectionCount( "," ) != 0 )
												{
													retitem = Items->SpawnItemToPack( NULL, s, cdata.section( ",", 0, 0 ).stripWhiteSpace(), false );
													if( retitem != NULL )
													{
														if( retitem->isPileable() )
															retitem->SetAmount( cdata.section( ",", 1, 1 ).stripWhiteSpace().toUShort() );
														else
														{
															std::string mitem	= cdata.section( ",", 0, 0 ).stripWhiteSpace();
															int mtotal			= cdata.section( ",", 1, 1 ).stripWhiteSpace().toUShort() - 1;	// we all ready created one
															// We have to create a series of individual items, for it doesn't stack
															for( int mindex = 0; mindex < mtotal; ++mindex )
															{
																retitem = Items->SpawnItemToPack( NULL, s, mitem, false );
															}
														}
													}
												}
												else
													retitem = Items->SpawnItemToPack( NULL, s, cdata.c_str(), false );
											}
											break;
			case DFNTAG_PARRYING:			skillToSet =  PARRYING;				break;
			case DFNTAG_PEACEMAKING:		skillToSet =  PEACEMAKING;			break;
			case DFNTAG_PROVOCATION:		skillToSet =  PROVOCATION;			break;
			case DFNTAG_POISONING:			skillToSet =  POISONING;			break;
			case DFNTAG_REMOVETRAPS:		skillToSet =  REMOVETRAPS;			break;
			case DFNTAG_SKIN:				s->SetSkin( static_cast<UI16>(ndata ));		break;
			case DFNTAG_STRENGTH:			s->SetStrength( static_cast< SI16 >(RandomNum( ndata, odata )) );
											s->SetHP( s->GetMaxHP() );
											break;
			case DFNTAG_SKILL:				s->SetBaseSkill( static_cast<UI16>(odata), static_cast<UI08>(ndata) );					break;
			case DFNTAG_SNOOPING:			skillToSet =  SNOOPING;				break;
			case DFNTAG_SPIRITSPEAK:		skillToSet =  SPIRITSPEAK;			break;
			case DFNTAG_STEALING:			skillToSet =  STEALING;				break;
			case DFNTAG_SWORDSMANSHIP:		skillToSet =  SWORDSMANSHIP;		break;
			case DFNTAG_STEALTH:			skillToSet =  STEALTH;				break;
			case DFNTAG_TITLE:				s->SetTitle( cdata.c_str() );		break;
			case DFNTAG_TAILORING:			skillToSet =  TAILORING;			break;
			case DFNTAG_TAMING:				skillToSet =  TAMING;				break;
			case DFNTAG_TASTEID:			skillToSet =  TASTEID;				break;
			case DFNTAG_TINKERING:			skillToSet =  TINKERING;			break;
			case DFNTAG_TRACKING:			skillToSet =  TRACKING;				break;
			case DFNTAG_TACTICS:			skillToSet =  TACTICS;				break;
			case DFNTAG_VETERINARY:			skillToSet =  VETERINARY;			break;
			case DFNTAG_WRESTLING:			skillToSet =  WRESTLING;			break;
			case DFNTAG_STAMINA:			s->SetStamina(static_cast<SI16>(ndata));	break;
			case DFNTAG_MANA:				s->SetMana(static_cast<SI16>(ndata));	break;
			case DFNTAG_POISONSTRENGTH:		s->SetPoisonStrength(static_cast<UI08>(ndata));	break;
			default:						break;
		}
		if( skillToSet != 0 )
		{
			s->SetBaseSkill( static_cast<UI16>(RandomNum( ndata, odata )), skillToSet );
			skillToSet = 0;
		}
	}
 
	//Now find real 'skill' based on 'baseskill' (stat modifiers)
	for( UI08 j = 0; j < ALLSKILLS; ++j )
		Skills->updateSkillLevel( s, j );
	s->Teleport();
	Effects->staticeffect( s, 0x373A, 0, 15 );
	Effects->PlaySound( s, 0x01E9 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void Karma( CChar *nCharID, CChar *nKilledID, SI16 nKarma )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle karma addition/subtraction when character kills
//|					another Character / NPC
//o---------------------------------------------------------------------------o
void Karma( CChar *nCharID, CChar *nKilledID, SI16 nKarma )
{	// nEffect = 1 positive karma effect
	SI16 nChange = 0;
	bool nEffect = false;
	//
	SI16 nCurKarma = nCharID->GetKarma();
	if( nCurKarma > 10000 )
		nCharID->SetKarma( 10000 );
	else if( nCurKarma < -10000 ) 
		nCharID->SetKarma( -10000 );
	//
	if( nCurKarma < nKarma && nKarma > 0 )
	{
		nChange = ( ( nKarma - nCurKarma ) / 75 );
		nCharID->SetKarma( (SI16)( nCurKarma + nChange ) );
		nEffect = true;
	}
	if( nCurKarma > nKarma && ( nKilledID == NULL || nKilledID->GetKarma() > 0 ) )
	{
		nChange = ( ( nCurKarma - nKarma ) / 50 );
		nCharID->SetKarma( (SI16)( nCurKarma - nChange ) );
		nEffect = false;
	}
	if( nChange == 0 )	// NPCs CAN gain/lose karma
		return;

	cSocket *mSock = calcSocketObjFromChar( nCharID );
	if( nCharID->IsNpc() || mSock == NULL )
		return;
	if( nChange <= 25 )
	{
		if( nEffect )
			mSock->sysmessage( 1367 );
		else
			mSock->sysmessage( 1368 );
		return;
	}
	if( nChange <= 50 )
	{
		if( nEffect )
			mSock->sysmessage( 1369 );
		else
			mSock->sysmessage( 1370 );
		return;
	}
	if( nEffect )
		mSock->sysmessage( 1371 );
	else
		mSock->sysmessage( 1372 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void Fame( CChar *nCharID, SI16 nFame )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle fame addition when character kills another
//|					Character / NPC
//o---------------------------------------------------------------------------o
void Fame( CChar *nCharID, SI16 nFame )
{
	SI16 nChange = 0;
	bool nEffect = false;
	SI16 nCurFame = nCharID->GetFame();
	if( nCharID->IsDead() )
	{
		if( nCurFame <= 0 )
			nCharID->SetFame( 0 );
		else
		{
			nChange = ( nCurFame - 0 ) / 25;
			nCharID->SetFame( (SI16)( nCurFame - nChange ) );
		}
		nCharID->SetDeaths( (UI16)( nCharID->GetDeaths() + 1 ) );
		nEffect = false;
	}
	else if( nCurFame <= nFame )
	{
		nChange = ( nFame - nCurFame ) / 75;
		nCharID->SetFame( (SI16)( nCurFame + nChange ) );
		nEffect = true;
		if( nCharID->GetFame() > 10000 )
			nCharID->SetFame( 10000 );
	}
	else
		return;	// current fame is greater than target fame, and we're not dead
	if( nChange == 0 )
		return;
	cSocket *mSock = calcSocketObjFromChar( nCharID );
	if( mSock == NULL || nCharID->IsNpc() )
		return;
	if( nChange <= 25 )
	{
		if( nEffect )
			mSock->sysmessage( 1373 );
		else
			mSock->sysmessage( 1374 );
	}
	else if( nChange <= 50 )
	{
		if( nEffect )
			mSock->sysmessage( 1375 );
		else
			mSock->sysmessage( 1376 );
	}
	else
	{
		if( nEffect )
			mSock->sysmessage( 1377 );
		else
			mSock->sysmessage( 1378 );
	}
}

//o--------------------------------------------------------------------------
//|	Function		-	CChar *cCharStuff::SpawnRandomMonster( cSocket *nCharID, DEFINITIONCATEGORIES sourceDFN, char *cList, char *cNpcID )
//|	Date			-	Unknown
//|	Programmer		-	Unknown
//|	Modified		-	Changed to FileLookup based and return CChar * 
//|						instead of creature ID (Abaddon)
//o--------------------------------------------------------------------------
//|	Purpose			-	Gets the random monster number from the script and list specified
//|						Creates the creature and returns it
//o--------------------------------------------------------------------------
CChar *cCharStuff::SpawnRandomMonster( cSocket *nCharID, DEFINITIONCATEGORIES sourceDFN, char* cList, char* cNpcID )
{
	char sect[512];
 	sprintf( sect, "%s %s", cList, cNpcID );
	ScriptSection *targData = NULL;
	targData = FileLookup->FindEntry( sect, sourceDFN );
	if( targData == NULL )
		return NULL;
	size_t i		= targData->NumEntries();
	CChar *tChar	= nCharID->CurrcharObj();
	if( i > 0 )
	{
		i = rand()%(i);
		UString targID = targData->MoveTo( i );
		if( !targID.empty() )
			return AddNPC( nCharID, NULL, targID.c_str(), tChar->WorldNumber() );
	}
	return NULL;
}


