#include <algorithm>
#include "uox3.h"
#include "packets.h"
#include "movement.h"
#include "cServerDefinitions.h"
#include "targeting.h"
#include "speech.h"
#include "ssection.h"
#include "gump.h"
#include "cThreadQueue.h"
#include "combat.h"
#include "classes.h"
#include "trigger.h"
#include "cScript.h"
#include "PageVector.h"
#include "cSpeech.h"

//#define __USING_BOOST_TEST__

#ifdef __USING_BOOST_TEST__
#include <boost/tokenizer.hpp>
using namespace boost;
#endif


void PackString( UI08 *toPack, int offset, std::string value, UI32 maxLen )
{
	int toWrite	= UOX_MIN( maxLen, static_cast<UI32>(value.length()) );
	for( int i = 0; i < toWrite; ++i )
		toPack[offset+i] = (UI08)value[i];
}
std::string UnpackString( UI08 *toPack, int offset, std::string &value, UI32 maxLen )
{
	value = "";
	for( UI32 i = 0; i < maxLen; ++i )
	{
		if( toPack[offset + i] != 0 )
			value += (char)(toPack[offset + i]);
	}
	return value;
}
void PackLong( UI08 *toPack, int offset, SI32 value )
{
	toPack[offset+0] = (UI08)(value>>24);
	toPack[offset+1] = (UI08)(value>>16);
	toPack[offset+2] = (UI08)(value>>8);
	toPack[offset+3] = (UI08)(value%256);
}
SI32 UnpackSLong( UI08 *toPack, int offset )
{
	return (toPack[offset+0]<<24) + (toPack[offset+1]<<16) + (toPack[offset+2]<<8) + toPack[offset+3];
}
UI32 UnpackULong( UI08 *toPack, int offset )
{
	return (UI32)((toPack[offset+0]<<24) + (toPack[offset+1]<<16) + (toPack[offset+2]<<8) + toPack[offset+3]);
}
UI16 UnpackUShort( UI08 *toPack, int offset )
{
	return (UI16)((toPack[offset+0]<<8) + toPack[offset+1]);
}
SI16 UnpackSShort( UI08 *toPack, int offset )
{
	return (SI16)((toPack[offset+0]<<8) + toPack[offset+1]);
}
void PackLong( UI08 *toPack, int offset, UI32 value )
{
	toPack[offset+0] = (UI08)((value&0xFF000000)>>24);
	toPack[offset+1] = (UI08)((value&0x00FF0000)>>16);
	toPack[offset+2] = (UI08)((value&0x0000FF00)>>8);
	toPack[offset+3] = (UI08)((value&0x000000FF)%256);
}

void PackShort( UI08 *toPack, int offset, UI16 value )
{
	toPack[offset+0] = (UI08)((value&0xFF00)>>8);
	toPack[offset+1] = (UI08)((value&0x00FF)%256);
}

void pSplit( const char *pass0, char *pass1, char *pass2 ) // Split login password into UOX password and UO password
{
#ifdef __USING_BOOST_TEST__
	typedef tokenizer<char_separator<char> > chartokenizer;
	char_separator<char> sep( "/" );
	std::string line( pass0 );
	chartokenizer tok( line, sep );

	chartokenizer::iterator beg = tok.begin(); 
	if( beg != tok.end() )
		strcpy( pass1, (*beg).c_str() );
	++beg;
	if( beg != tok.end() )
		strcpy( pass2, (*beg).c_str() );
#else
	int i = 0;
	pass1[0] = 0;
	while( pass0[i] != '/' && pass0[i] != 0 ) 
		++i;
	strncpy( pass1, pass0, i );
	pass1[i] = 0;
	if( pass0[i] != 0 ) 
		strcpy( pass2, pass0 + i + 1 );
#endif
}

// Unknown bytes
// 5->8
// 18->27
// 30->36

//0x1B Packet
//Last Modified on Friday, 20-Nov-1998
//Char Location and body type (37 bytes) 
//	BYTE cmd 
//	BYTE[4] player id 
//	BYTE[4] unknown1 
//	BYTE[2] bodyType 
//	BYTE[2] xLoc 
//	BYTE[2] yLoc 
//	BYTE[2] zLoc 
//	BYTE direction 
//	BYTE[2] unknown2 
//	BYTE[4] unknown3 (usually has FF somewhere in it) 
//	BYTE[4] unknown4 
//	BYTE flag byte 
//	BYTE highlight color 
//	BYTE[7] unknown5 
//Note: Only send once after login. Its mandatory to send it once.
//Note2: OSI calls this packet Login confirm

void CPCharLocBody::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[SEND]Packet     : CPCharLocBody 0x1B --> Length: %i\n", internalBuffer.size() );
	fprintf( fp, "PlayerID         : %x\n", UnpackULong(  &internalBuffer[0], 1  ) );
	fprintf( fp, "Unknown1         : %i\n", UnpackULong(  &internalBuffer[0], 5  ) );
	fprintf( fp, "BodyType         : %x\n", UnpackUShort( &internalBuffer[0], 9  ) );
	fprintf( fp, "X Loc            : %i\n", UnpackUShort( &internalBuffer[0], 11 ) );
	fprintf( fp, "Y Loc            : %i\n", UnpackUShort( &internalBuffer[0], 13 ) );
	fprintf( fp, "Z Loc            : %i\n", UnpackUShort( &internalBuffer[0], 15 ) );
	fprintf( fp, "Direction        : %i\n", internalBuffer[17] );
	fprintf( fp, "Unknown2         : %i\n", UnpackUShort( &internalBuffer[0], 18 ) );
	fprintf( fp, "Unknown3         : %i\n", UnpackULong(  &internalBuffer[0], 20 ) );
	fprintf( fp, "Unknown4         : %i\n", UnpackULong(  &internalBuffer[0], 24 ) );
	fprintf( fp, "Flag Byte        : %i\n", internalBuffer[28] );
	fprintf( fp, "Highlight Colour : %i\n", internalBuffer[29] );
	fprintf( fp, "  Raw dump     :\n" );
	cPUOXBuffer::Log( fp, false );
}

void CPCharLocBody::InternalReset( void )
{
	internalBuffer.resize( 37 );
	internalBuffer[0] = 0x1B;
	for( UI08 k = 5; k < 9; ++k )
		internalBuffer[k] = 0;
	for( UI08 i = 18; i < 28; ++i )
		internalBuffer[i] = 0;
	for( UI08 j = 30; j < 37; ++j )
		internalBuffer[j] = 0;
	HighlightColour( 0 );
}
CPCharLocBody::CPCharLocBody()
{
	InternalReset();
}

CPCharLocBody::CPCharLocBody( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}

void CPCharLocBody::CopyData( CChar &toCopy )
{
	PlayerID( toCopy.GetSerial() );
	BodyType( toCopy.GetID() );
	X( toCopy.GetX() );
	Y( toCopy.GetY() );
	Z( toCopy.GetZ() );
	Direction( toCopy.GetDir() );
}

CPCharLocBody::~CPCharLocBody()
{
}

CPCharLocBody &CPCharLocBody::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}
void CPCharLocBody::PlayerID( SERIAL toPut )
{
	PackLong( &internalBuffer[0], 1, toPut );
}
void CPCharLocBody::BodyType( UI16 toPut )
{
	PackShort( &internalBuffer[0], 9, toPut );
}
void CPCharLocBody::X( UI16 toPut )
{
	PackShort( &internalBuffer[0], 11, toPut );
}
void CPCharLocBody::Y( UI16 toPut )
{
	PackShort( &internalBuffer[0], 13, toPut );
}
void CPCharLocBody::Z( UI16 toPut )
{
	PackShort( &internalBuffer[0], 15, toPut );
}
void CPCharLocBody::Direction( UI08 toPut )
{
	internalBuffer[17] = toPut;
}
void CPCharLocBody::Flag( UI08 toPut )
{
	internalBuffer[28] = toPut;
}
void CPCharLocBody::HighlightColour( UI08 color )
{
	internalBuffer[29] = color;
}

//0x1C Packet
//Last Modified on Friday, 20-Apr-1998 
//Send Speech (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[4] itemID (FF FF FF FF = system) 
//	BYTE[2] model (item hex # - FF FF = system) 
//	BYTE Type 
//	BYTE[2] Text Color 
//	BYTE[2] Font 
//	BYTE[30] Name 
//	BYTE[?] Null-Terminated Message (? = blockSize - 44) 

CPacketSpeech::CPacketSpeech( CSpeechEntry &toCopy )
{
	internalBuffer.resize( 44 );
	internalBuffer[0] = 0x1C;
	CopyData( toCopy );
}

void CPacketSpeech::CopyData( CSpeechEntry &toCopy )
{
	Speech( toCopy.Speech() );
	SpeakerSerial( toCopy.Speaker() );
	Colour( toCopy.Colour() );
	Font( toCopy.Font() );
	Language( toCopy.Language() );
	Unicode( toCopy.Unicode() );
	Type( toCopy.Type() );
	if( strlen( toCopy.SpeakerName() ) != 0 )
		SpeakerName( toCopy.SpeakerName() );
	switch( toCopy.SpkrType() )
	{
		case SPK_UNKNOWN:
		case SPK_SYSTEM:
			SpeakerName( "System" );
			SpeakerModel( INVALIDID );
			if( toCopy.Colour() == 0 )
				Colour( 0x0040 );
			break;
		case SPK_CHARACTER:
			CChar *ourChar;
			ourChar = calcCharObjFromSer( toCopy.Speaker() );
			if( ourChar != NULL )
			{
				SpeakerName( ourChar->GetName().c_str() );
				SpeakerModel( ourChar->GetID() );
			}
			else
				SpeakerModel( INVALIDID );
			break;
		case SPK_ITEM:
			CItem *ourItem;
			ourItem = calcItemObjFromSer( toCopy.Speaker() );
			if( ourItem != NULL )
			{
				SpeakerName( ourItem->GetName().c_str() );
				SpeakerModel( ourItem->GetID() );
			}
			else
				SpeakerModel( INVALIDID );
			break;
	}
}

CPacketSpeech &CPacketSpeech::operator=( CSpeechEntry &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

CPacketSpeech &CPacketSpeech::operator=( CPacketSpeech &toCopy )
{
	internalBuffer.resize( toCopy.Length() );
	for( UI32 i = 0; i < internalBuffer.size(); ++i )
		internalBuffer[i] = toCopy[i];
	return (*this);
}

CPacketSpeech::~CPacketSpeech()
{
}

CPacketSpeech::CPacketSpeech() : isUnicode( false )
{
	internalBuffer.resize( 44 );	// Minimum packet size
	internalBuffer[0] = 0x1C;
	internalBuffer[1] = 0;
	internalBuffer[2] = 44;
	internalBuffer[3] = 0xFF;
	internalBuffer[4] = 0xFF;
	internalBuffer[5] = 0xFF;
	internalBuffer[6] = 0xFF;
	internalBuffer[7] = 0xFF;
	internalBuffer[8] = 0xFF;

	internalBuffer[14] = 'S';
	internalBuffer[15] = 'y';
	internalBuffer[16] = 's';
	internalBuffer[17] = 't';
	internalBuffer[18] = 'e';
	internalBuffer[19] = 'm';
	internalBuffer[20] = 0;
}

void CPacketSpeech::SpeakerName( const char *toPut )
{
	size_t len = strlen( toPut );
	if( len >= 30 )
	{
		strncpy( (char *)&internalBuffer[14], toPut, 29 );
		internalBuffer[43] = 0;
	}
	else
	{
		strcpy( (char *)&internalBuffer[14], toPut );
	}
}
void CPacketSpeech::SpeakerSerial( SERIAL toPut )
{
	PackLong( &internalBuffer[0], 3, toPut );
}
void CPacketSpeech::SpeakerModel( UI16 toPut )
{
	PackShort( &internalBuffer[0], 7, toPut );
}
void CPacketSpeech::Colour( COLOUR toPut )
{
	PackShort( &internalBuffer[0], 10, toPut );
}
void CPacketSpeech::Font( FontType toPut )
{
	internalBuffer[12] = 0;
	internalBuffer[13] = (UI08)toPut;
}
#if defined( _MSC_VER )
#pragma note( "Function Warning: CPacketSpeech::Language(), does nothing" )
#endif
void CPacketSpeech::Language( UnicodeTypes toPut )
{

}
void CPacketSpeech::Unicode( bool toPut )
{
	isUnicode = toPut;
}
void CPacketSpeech::Type( SpeechType toPut )
{
	internalBuffer[9] = (UI08)toPut;
}
void CPacketSpeech::Speech( const char *toPut )
{
	size_t len			= strlen( toPut );
	const size_t newLen	= 44 + len + 1;
	internalBuffer.resize( 44 + len + 1 );
	strcpy( (char *)&internalBuffer[44], toPut );
	PackShort( &internalBuffer[0], 1, static_cast<UI16>(newLen) );
}

//0x21 Packet
//Last Modified on Wednesday, 06-May-1998 23:30:37 EDT 
//Character Move Reject (8 bytes) 
//	BYTE cmd 
//	BYTE sequence # 
//	BYTE[2] xLoc 
//	BYTE[2] yLoc 
//	BYTE direction 
//	BYTE zLoc 

CPWalkDeny::CPWalkDeny()
{
	internalBuffer.resize( 8 );
	internalBuffer[0] = 0x21;
}
void CPWalkDeny::SequenceNumber( char newValue )
{
	internalBuffer[1] = newValue;
}
void CPWalkDeny::X( SI16 newValue )
{
	PackShort( &internalBuffer[0], 2, newValue );
}
void CPWalkDeny::Y( SI16 newValue )
{
	PackShort( &internalBuffer[0], 4, newValue );
}
void CPWalkDeny::Z( SI08 newValue )
{
	internalBuffer[7] = newValue;
}
void CPWalkDeny::Direction( char newValue )
{
	internalBuffer[6] = newValue;
}

//0x22 Packet
//Last Modified on Wednesday, 11-Nov-1998 
//Character Move ACK/ Resync Request(3 bytes) 
//	BYTE cmd 
//	BYTE sequence (matches sent sequence) 
//	BYTE (0x00) 

CPWalkOK::CPWalkOK()
{
	internalBuffer.resize( 3 );
	internalBuffer[0] = 0x22;
}
void CPWalkOK::SequenceNumber( char newValue )
{
	internalBuffer[1] = newValue;
}
void CPWalkOK::OtherByte( char newValue )
{
	internalBuffer[2] = newValue;
}

//0x77 Packet
//Last Modified on Sunday, 13-Feb-2000 
//Update Player (17 bytes) 
//	BYTE cmd 
//	BYTE[4] player id 
//	BYTE[2] model
//	BYTE[2] xLoc 
//	BYTE[2] yLoc 
//	BYTE zLoc 
//	BYTE direction 
//	BYTE[2] hue/skin color 
//	BYTE flag (bit field) 
//	BYTE highlight color

CPExtMove::CPExtMove()
{
	internalBuffer.resize( 17 );
	internalBuffer[0] = 0x77;
}

CPExtMove::CPExtMove( CChar &toCopy )
{
	internalBuffer.resize( 17 );
	internalBuffer[0] = 0x77;
	CopyData( toCopy );
}

CPExtMove &CPExtMove::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

void CPExtMove::Serial( SERIAL newSerial )
{
	PackLong( &internalBuffer[0], 1, newSerial );
}
void CPExtMove::ID( UI16 bodyID )
{
	PackShort( &internalBuffer[0], 5, bodyID );
}
void CPExtMove::X( SI16 newValue )
{
	PackShort( &internalBuffer[0], 7, newValue );
}
void CPExtMove::Y( SI16 newValue )
{
	PackShort( &internalBuffer[0], 9, newValue );
}
void CPExtMove::Z( SI08 newValue )
{
	internalBuffer[11] = newValue;
}
void CPExtMove::Direction( char newValue )
{
	internalBuffer[12] = newValue;
}
void CPExtMove::Colour( UI16 colourID )
{
	PackShort( &internalBuffer[0], 13, colourID );
}
void CPExtMove::FlagColour( char newValue )
{
	internalBuffer[16] = newValue;
}
void CPExtMove::Flag( char newValue )
{
	internalBuffer[15] = newValue;
}

void CPExtMove::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
	ID( toCopy.GetID() );
	X( toCopy.GetX() );
	Y( toCopy.GetY() );
	Z( toCopy.GetDispZ() );
	Direction( toCopy.GetDir() );
	Colour( toCopy.GetSkin() );

	UI08 flag = 0;
	if( toCopy.IsAtWar() ) 
		flag |= 0x40;
	else if( toCopy.IsDead() )
		flag |= 0x80;
	if( toCopy.GetHidden() ) 
		flag |= 0x80;
	if( toCopy.GetPoisoned() ) 
		flag |= 0x04;
	if( toCopy.GetID() == 0x0191 )
		flag |= 0x02;
#if defined( _MSC_VER )
#pragma note( "Get confirmation here" )
#endif
	// turn it yellow on full health?
	if( toCopy.GetHP() == toCopy.GetMaxHP() )
		flag |= 0x08;
	Flag( flag );
}

//0xAA Packet
//Last Modified on Thursday, 16-Apr-1998 19:52:14 EDT 
//OK / Not OK To Attack (5 bytes) 
//	BYTE cmd 
//	BYTE[4] CharID being attacked 
//ID is set to 00 00 00 00 when attack is refused.

void CPAttackOK::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
}
CPAttackOK::CPAttackOK()
{
	internalBuffer.resize( 5 );
	internalBuffer[0] = 0xAA;
}
CPAttackOK::CPAttackOK( CChar &toCopy )
{
	internalBuffer.resize( 5 );
	internalBuffer[0] = 0xAA;
	CopyData( toCopy );
}
void CPAttackOK::Serial( SERIAL newSerial )
{
	PackLong( &internalBuffer[0], 1, newSerial );
}
CPAttackOK &CPAttackOK::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

//0x1D Packet
//Last Modified on Monday, 13-Apr-1998 17:06:02 EDT 
//Delete object (5 bytes) 
//	BYTE cmd 
//	BYTE[4] item/char id 
void CPRemoveItem::CopyData( cBaseObject &toCopy )
{
	Serial( toCopy.GetSerial() );
}
CPRemoveItem::CPRemoveItem()
{
	internalBuffer.resize( 5 );
	internalBuffer[0] = 0x1D;
}
CPRemoveItem::CPRemoveItem( cBaseObject &toCopy )
{
	internalBuffer.resize( 5 );
	internalBuffer[0] = 0x1D;
	CopyData( toCopy );
}
void CPRemoveItem::Serial( SERIAL newSerial )
{
	PackLong( &internalBuffer[0], 1, newSerial );
}
CPRemoveItem &CPRemoveItem::operator=( cBaseObject &toCopy )
{
	CopyData( toCopy );
	return (*this);
}


//0xBC Packet
//Last Modified on Friday, 19-May-2000  
//Seasonal Information(3 bytes) 
//	BYTE cmd 
//	BYTE id1
//	BYTE id2
//Note: Server message
//Note: if id2 = 1, then this is a season change.
//Note: if season change, then id1 = (0=spring, 1=summer, 2=fall, 3=winter, 4 = desolation)
CPWorldChange::CPWorldChange()
{
	internalBuffer.resize( 3 );
	internalBuffer[0] = 0xBC;
}
CPWorldChange::CPWorldChange( WorldType newSeason, UI08 newCursor )
{
	internalBuffer.resize( 3 );
	internalBuffer[0] = 0xBC;
	Season( newSeason );
	Cursor( newCursor );
}
void CPWorldChange::Season( WorldType newSeason )
{
	internalBuffer[1] = (UI08)newSeason;
}
void CPWorldChange::Cursor( UI08 newCursor )
{
	internalBuffer[2] = newCursor;
}


//0x4F Packet
//Last Modified on Saturday, 14-Nov-1998 
//Overall Light Level (2 bytes) 
//	BYTE cmd 
//	BYTE level 
//		0x00 - day 
//		0x09 - OSI night
//		0x1F - Black 
//		Max normal val = 0x1F 
CPLightLevel::CPLightLevel()
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x4F;
}
CPLightLevel::CPLightLevel( LIGHTLEVEL level )
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x4F;
	Level( level );
}

void CPLightLevel::Level( LIGHTLEVEL level )
{
	internalBuffer[1] = level;
}

void CPUpdIndSkill::InternalReset( void )
{
	internalBuffer.resize( 11 );
	internalBuffer[0] = 0x3A;
	internalBuffer[1] = 0x00; // Length of message
	internalBuffer[2] = 0x0B; // Length of message
	internalBuffer[3] = 0xFF; // single entry
	internalBuffer[4] = 0x00;
}
void CPUpdIndSkill::CopyData( CChar& i, UI08 sNum )
{
	SkillNum( sNum );
	Skill( i.GetSkill( sNum ) );
	BaseSkill( i.GetBaseSkill( sNum ) );
	Lock( i.GetSkillLock( sNum ) );
}
CPUpdIndSkill::CPUpdIndSkill()
{
	InternalReset();
}
CPUpdIndSkill::CPUpdIndSkill( CChar& i, UI08 sNum )
{
	InternalReset();
	CopyData( i, sNum );
}
void CPUpdIndSkill::Character( CChar& i, UI08 sNum )
{
	CopyData( i, sNum );
}
void CPUpdIndSkill::SkillNum( UI08 sNum )
{
	internalBuffer[5] = sNum;
}
void CPUpdIndSkill::Skill( SI16 skillval )
{
	PackShort( &internalBuffer[0], 6, skillval );
}
void CPUpdIndSkill::BaseSkill( SI16 skillval )
{
	PackShort( &internalBuffer[0], 8, skillval );
}
void CPUpdIndSkill::Lock( UI08 lockVal )
{
	internalBuffer[10] = lockVal;
}

//0x3B Packet
//Last Modified on Wednesday, 06-May-1998 23:30:41 EDT 
//Buy Item(s) (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[4] vendorID 
//	BYTE flag 
//		0x00 - no items following 
//		0x02 - items following 
//	For each item 
//		BYTE (0x1A) 
//		BYTE[4] itemID (from 3C packet) 
//		BYTE[2] # bought 

void CPBuyItem::CopyData( cBaseObject &i )
{
	Serial( i.GetSerial() );
}
CPBuyItem::CPBuyItem()
{
	InternalReset();
}
CPBuyItem::CPBuyItem( cBaseObject &i )
{
	InternalReset();
	CopyData( i );
}
CPBuyItem &CPBuyItem::operator=( cBaseObject &toCopy )
{
	CopyData( toCopy );
	return (*this);
}
void CPBuyItem::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 3, toSet );
}

void CPBuyItem::InternalReset( void )
{
	internalBuffer.resize( 8 );
	internalBuffer[0] = 0x3B;
	internalBuffer[1] = 0x00;
	internalBuffer[2] = 0x08;
	internalBuffer[7] = 0x00;
}

const long loopbackIP = (127<<24) + 1;

//0x8C Packet
//Last Modified on Monday, 13-Apr-1998 17:06:42 EDT 
//Connect to Game Server (11 bytes) 
//	BYTE cmd 
//	BYTE[4] gameServer IP 
//	BYTE[2] gameServer port 
//	BYTE[4] new key 

CPRelay::CPRelay()
{
	InternalReset();
}
CPRelay::CPRelay( long newIP )
{
	InternalReset();
	ServerIP( newIP );
}
CPRelay::CPRelay( long newIP, UI16 newPort )
{
	InternalReset();
	ServerIP( newIP );
	Port( newPort );
}
void CPRelay::ServerIP( long newIP )
{
	PackLong( &internalBuffer[0], 1, newIP );
}
void CPRelay::Port( UI16 newPort )
{
	PackShort( &internalBuffer[0], 5, newPort );
}
void CPRelay::SeedIP( long newIP )
{
	PackLong( &internalBuffer[0], 7, newIP );
}
void CPRelay::InternalReset( void )
{
	internalBuffer.resize( 11 );
	internalBuffer[0] = 0x8C;
	SeedIP( loopbackIP );
}


//0x2E Packet
//Last Modified on Thursday, 19-Nov-1998 
//Worn Item (15 bytes) 
//	BYTE cmd 
//	BYTE[4] itemid (always starts 0x40 in my data) 
//	BYTE[2] model (item hex #) 
//	BYTE (0x00) 
//	BYTE layer
//	BYTE[4] playerID 
//	BYTE[2] color/hue 

CPWornItem::CPWornItem()
{
	internalBuffer.resize( 15 );
	internalBuffer[0] = 0x2E;
	internalBuffer[7] = 0x00;
}
void CPWornItem::ItemSerial( SERIAL itemSer )
{
	PackLong( &internalBuffer[0], 1, itemSer );
}
void CPWornItem::Model( SI16 newModel )
{
	PackShort( &internalBuffer[0], 5, newModel );
}
void CPWornItem::Layer( UI08 layer )
{
	internalBuffer[8] = layer;
}
void CPWornItem::CharSerial( SERIAL chSer )
{
	PackLong( &internalBuffer[0], 9, chSer );
}
void CPWornItem::Colour( SI16 newColour )
{
	PackShort( &internalBuffer[0], 13, newColour );
}

CPWornItem::CPWornItem( CItem &toCopy )
{
	internalBuffer.resize( 15 );
	internalBuffer[0] = 0x2E;
	internalBuffer[7] = 0x00;
	CopyData( toCopy );
}

void CPWornItem::CopyData( CItem &toCopy )
{
	ItemSerial( toCopy.GetSerial() );
	Colour( toCopy.GetColour() );
	Model( toCopy.GetID() );
	Layer( toCopy.GetLayer() );
	CharSerial( toCopy.GetContSerial() );
}
CPWornItem &CPWornItem::operator=( CItem &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

//0x6E Packet
//Last Modified on Monday, 19-Apr-1999 
//Character Animation (14 bytes) 
//	BYTE cmd 
//	BYTE[4] item/char ID 
//	BYTE[2] movement model 
//		0x00 = walk 
//		0x01 = walk faster 
//		0x02 = run 
//		0x03 = run (faster?) 
//		0x04 = nothing 
//		0x05 = shift shoulders 
//		0x06 = hands on hips 
//		0x07 = attack stance (short) 
//		0x08 = attack stance (longer) 
//		0x09 = swing (attack with knife) 
//		0x0a = stab (underhanded) 
//		0x0b = swing (attack overhand with sword) 
//		0x0c = swing (attack with sword over and side) 
//		0x0d = swing (attack with sword side) 
//		0x0e = stab with point of sword 
//		0x0f = ready stance 
//		0x10 = magic (butter churn!) 
//		0x11 = hands over head (balerina) 
//		0x12 = bow shot 
//		0x13 = crossbow 
//		0x14 = get hit 
//		0x15 = fall down and die (backwards) 
//		0x16 = fall down and die (forwards) 
//		0x17 = ride horse (long) 
//		0x18 = ride horse (medium) 
//		0x19 = ride horse (short) 
//		0x1a = swing sword from horse 
//		0x1b = normal bow shot on horse 
//		0x1c = crossbow shot 
//		0x1d = block #2 on horse with shield 
//		0x1e = block on ground with shield 
//		0x1f = swing, and get hit in middle 
//		0x20 = bow (deep) 
//		0x21 = salute 
//		0x22 = scratch head 
//		0x23 = 1 foot forward for 2 secs 
//		0x24 = same 
//	BYTE unknown1 (0x00) 
//	BYTE direction 
//	BYTE[2] repeat (1 = once / 2 = twice / 0 = repeat forever) 
//	BYTE forward/backwards(0x00=forward, 0x01=backwards) 
//	BYTE repeat Flag (0 - Don't repeat / 1 repeat) 
//	BYTE frame Delay (0x00 - fastest / 0xFF - Too slow to watch) 

void CPCharacterAnimation::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
}
CPCharacterAnimation::CPCharacterAnimation()
{
	InternalReset();
}
CPCharacterAnimation::CPCharacterAnimation( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPCharacterAnimation::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 1, toSet );
}
void CPCharacterAnimation::Action( UI16 model )
{
	PackShort( &internalBuffer[0], 5, model );
}
void CPCharacterAnimation::Direction( UI08 dir )
{
	internalBuffer[8] = dir;
}
void CPCharacterAnimation::Repeat( SI16 repeatValue )
{
	PackShort( &internalBuffer[0], 9, repeatValue );
}
void CPCharacterAnimation::DoBackwards( bool newValue )
{
	internalBuffer[11] = (UI08)((newValue?1:0));
}
void CPCharacterAnimation::RepeatFlag( bool newValue )
{
	internalBuffer[12] = (UI08)((newValue?1:0));
}
void CPCharacterAnimation::FrameDelay( UI08 delay )
{
	internalBuffer[13] = delay;
}
CPCharacterAnimation &CPCharacterAnimation::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}
void CPCharacterAnimation::InternalReset( void )
{
	internalBuffer.resize( 14 );
	internalBuffer[0] = 0x6E;
	internalBuffer[7] = 0;
	Repeat( 1 );
	DoBackwards( false );
	RepeatFlag( false );
	FrameDelay( 0 );
}

//0x20 Packet
//Last Modified on Thursday, 19-Nov-1998 
//Draw Game Player (19 bytes) 
//	BYTE cmd 
//	BYTE[4] creature id 
//	BYTE[2] bodyType 
//	BYTE unknown1 (0) 
//	BYTE[2] skin color / hue 
//	BYTE flag byte 
//	BYTE[2] xLoc 
//	BYTE[2] yLoc 
//	BYTE[2] unknown2 (0)
//	BYTE direction 
//	BYTE zLoc 
//Note: Only used with the character being played by the client.

void CPDrawGamePlayer::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
	Model( toCopy.GetID() );
	Colour( toCopy.GetColour() );
	X( toCopy.GetX() );
	Y( toCopy.GetY() );
	Z( toCopy.GetZ() );
	Direction( toCopy.GetDir() );
	UI08 flag = 0;
	if( toCopy.GetPoisoned() )
		flag |= 0x04;
	if( toCopy.GetHidden() )
		flag |= 0x80;
	Flag( flag );
}
void CPDrawGamePlayer::InternalReset( void )
{
	internalBuffer.resize( 19 );
	internalBuffer[0] = 0x20;
	internalBuffer[7] = 0;
	internalBuffer[15] = 0;
	internalBuffer[16] = 0;
}
CPDrawGamePlayer::CPDrawGamePlayer()
{
	InternalReset();
}
CPDrawGamePlayer::CPDrawGamePlayer( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPDrawGamePlayer::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 1, toSet );
}
void CPDrawGamePlayer::Model( SI16 toSet )
{
	PackShort( &internalBuffer[0], 5, toSet );
}
void CPDrawGamePlayer::Colour( SI16 toSet )
{
	PackShort( &internalBuffer[0], 8, toSet );
}
void CPDrawGamePlayer::Flag( UI08 toSet )
{
	internalBuffer[10] = toSet;
}
void CPDrawGamePlayer::X( SI16 toSet )
{
	PackShort( &internalBuffer[0], 11, toSet );
}
void CPDrawGamePlayer::Y( SI16 toSet )
{
	PackShort( &internalBuffer[0], 13, toSet );
}
void CPDrawGamePlayer::Direction( UI08 toSet )
{
	internalBuffer[17] = toSet;
}
void CPDrawGamePlayer::Z( SI08 toSet )
{
	internalBuffer[18] = toSet;
}
CPDrawGamePlayer &CPDrawGamePlayer::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

//0x4E Packet
//Last Modified on Sunday, 17-May-1998 13:33:55 EDT 
//Personal Light Level (6 bytes) 
//	BYTE cmd 
//	BYTE[4] creature id 
//	BYTE level 

void CPPersonalLightLevel::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
}
void CPPersonalLightLevel::InternalReset( void )
{
	internalBuffer.resize( 6 );
	internalBuffer[0] = 0x4E;
}
CPPersonalLightLevel::CPPersonalLightLevel()
{
	InternalReset();
}
CPPersonalLightLevel::CPPersonalLightLevel( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPPersonalLightLevel::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 1, toSet );
}
void CPPersonalLightLevel::Level( LIGHTLEVEL lightLevel )
{
	internalBuffer[5] = lightLevel;
}

CPPersonalLightLevel &CPPersonalLightLevel::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

//0x54 Packet
//Last Modified on Sunday, 13-Feb-2000 
//Play Sound Effect (12 bytes) 
//	BYTE cmd 
//	BYTE mode (0x00=quiet, repeating, 0x01=single normally played sound effect) 
//	BYTE[2] SoundModel 
//	BYTE[2] unknown3 (speed/volume modifier? Line of sight stuff?) 
//	BYTE[2] xLoc 
//	BYTE[2] yLoc 
//	BYTE[2] zLoc 
CPPlaySoundEffect::CPPlaySoundEffect()
{
	InternalReset();
}

void CPPlaySoundEffect::Mode( UI08 mode )
{
	internalBuffer[1] = mode;
}
void CPPlaySoundEffect::Model( UI16 newModel )
{
	PackShort( &internalBuffer[0], 2, newModel );
}
void CPPlaySoundEffect::X( SI16 xLoc )
{
	PackShort( &internalBuffer[0], 6, xLoc );
}
void CPPlaySoundEffect::Y( SI16 yLoc )
{
	PackShort( &internalBuffer[0], 8, yLoc );
}
void CPPlaySoundEffect::Z( SI16 zLoc )
{
	PackShort( &internalBuffer[0], 10, zLoc );
}
void CPPlaySoundEffect::InternalReset( void )
{
	internalBuffer.resize( 12 );
	internalBuffer[0] = 0x54;
	internalBuffer[1] = 1;
	internalBuffer[4] = 0;
	internalBuffer[5] = 0;
}
CPPlaySoundEffect &CPPlaySoundEffect::operator=( cBaseObject &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

void CPPlaySoundEffect::CopyData( cBaseObject &toCopy )
{
	X( toCopy.GetX() );
	Y( toCopy.GetY() );
	Z( toCopy.GetZ() );
}

CPPlaySoundEffect::CPPlaySoundEffect( cBaseObject &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}

//0x88 Packet
//Last Modified on Saturday, 18-Apr-1998 17:49:39 EDT 
//Open Paperdoll (66 bytes) 
//	BYTE cmd 
//	BYTE[4] charid 
//	BYTE[60] text 
//	BYTE flag byte
void CPPaperdoll::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
}
void CPPaperdoll::InternalReset( void )
{
	internalBuffer.resize( 66 );
	internalBuffer[0] = 0x88;
}
CPPaperdoll::CPPaperdoll()
{
	InternalReset();
}
CPPaperdoll::CPPaperdoll( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPPaperdoll::Serial( SERIAL tSerial )
{
	PackLong( &internalBuffer[0], 1, tSerial );
}
void CPPaperdoll::FlagByte( UI08 fVal )
{
	internalBuffer[65] = fVal;
}
void CPPaperdoll::Text( const char *toPut )
{
	if( strlen( toPut ) > 60 )
	{
		strncpy( (char *)&internalBuffer[5], toPut, 59 );
		internalBuffer[64] = 0;
	}
	else
		strcpy( (char *)&internalBuffer[5], toPut );
}
CPPaperdoll &CPPaperdoll::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

//0x65 Packet
//Last Modified on Wednesday, 24-May-2000 
//Set Weather (4 bytes) 
//	BYTE cmd 
//	BYTE type (0x00  It starts to rain, 0x01  A fierce storm approaches., 0x02  It begins to snow, 0x03 - A storm is brewing., 0xFF  None (turns off sound effects), 0xFE (no effect?? Set temperature?) 
//	BYTE num (number of weather effects on screen)
//	BYTE temperature 
//Note: Server Message
//Note: Temperature has no effect at present.
//Note: maximum number of weather effects on screen is 70.
//Note: If it is raining, you can add snow by setting the num to the num of rain currently going, plus the number of snow you want. 
//Note: Weather messages are only displayed when weather starts.
//Note: Weather will end automatically after 6 minutes without any weather change packets.
//Note: You can totally end weather (to display a new message) by teleporting.  I think its either the 0x78 or 0x20 messages that reset it, though I havent checked to be sure (other possibilities, 0x4F or 0x4E)
void CPWeather::InternalReset( void )
{
	internalBuffer.resize( 4 );
	internalBuffer[0] = 0x65;
	internalBuffer[1] = 0xFF;
	internalBuffer[2] = 0x40;
	internalBuffer[3] = 0x10;
}
CPWeather::CPWeather()
{
	InternalReset();
}
CPWeather::CPWeather( UI08 nType )
{
	InternalReset();
	Type( nType );
}
CPWeather::CPWeather( UI08 nType, UI08 nParts )
{
	InternalReset();
	Type( nType );
	Particles( nParts );
}
CPWeather::CPWeather( UI08 nType, UI08 nParts, UI08 nTemp )
{
	InternalReset();
	Type( nType );
	Particles( nParts );
	Temperature( nTemp );
}
void CPWeather::Type( UI08 nType )
{
	internalBuffer[1] = nType;
}
void CPWeather::Particles( UI08 nParts )
{
	internalBuffer[2] = nParts;
}
void CPWeather::Temperature( UI08 nTemp )
{
	internalBuffer[3] = nTemp;
}

//0x70 Packet
//Last Modified on Friday, 20-Nov-1998 
//Graphical Effect (28 bytes) 
//	BYTE cmd 
//	BYTE direction type 
//		00 = go from source to dest 
//		01 = lightning strike at source 
//		02 = stay at current x,y,z 
//		03 = stay with current source character id 
//	BYTE[4] character id 
//	BYTE[4] target id
//	BYTE[2] model of the first frame of the effect
//	BYTE[2] xLoc 
//	BYTE[2] yLoc 
//	BYTE zLoc 
//	BYTE[2] xLoc of target 
//	BYTE[2] yLoc of target 
//	BYTE zLoc of target 
//	BYTE speed of the animation
//	BYTE duration (0=really long, 1= shortest)
//	BYTE[2] unknown2 (0 works)
//	BYTE adjust direction during animation (1=no)
//	BYTE explode on impact  
void CPGraphicalEffect::InternalReset( void )
{
	internalBuffer.resize( 28 );
	internalBuffer[0] = 0x70;
	internalBuffer[24] = 0;
	internalBuffer[25] = 0;
}
CPGraphicalEffect::CPGraphicalEffect( UI08 effectType )
{
	InternalReset();
	Effect( effectType );
}
void CPGraphicalEffect::Effect( UI08 effectType )
{
	internalBuffer[1] = effectType;
}
void CPGraphicalEffect::SourceSerial( cBaseObject &toSet )
{
	SourceSerial( toSet.GetSerial() );
}
void CPGraphicalEffect::SourceSerial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 2, toSet );
}
void CPGraphicalEffect::TargetSerial( cBaseObject &toSet )
{
	TargetSerial( toSet.GetSerial() );
}
void CPGraphicalEffect::TargetSerial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 6, toSet );
}
void CPGraphicalEffect::Model( SI16 nModel )
{
	PackShort( &internalBuffer[0], 10, nModel );
}
void CPGraphicalEffect::X( SI16 nX )
{
	PackShort( &internalBuffer[0], 12, nX );
}
void CPGraphicalEffect::Y( SI16 nY )
{
	PackShort( &internalBuffer[0], 14, nY );
}
void CPGraphicalEffect::Z( SI08 nZ )
{
	internalBuffer[16] = nZ;
}
void CPGraphicalEffect::XTrg( SI16 nX )
{
	PackShort( &internalBuffer[0], 17, nX );
}
void CPGraphicalEffect::YTrg( SI16 nY )
{
	PackShort( &internalBuffer[0], 19, nY );
}
void CPGraphicalEffect::ZTrg( SI08 nZ )
{
	internalBuffer[21] = nZ;
}
void CPGraphicalEffect::Speed( UI08 nSpeed )
{
	internalBuffer[22] = nSpeed;
}
void CPGraphicalEffect::Duration( UI08 nDuration )
{
	internalBuffer[23] = nDuration;
}
void CPGraphicalEffect::AdjustDir( bool nValue )
{
	internalBuffer[26] = (UI08)((nValue?0:1));
}
void CPGraphicalEffect::ExplodeOnImpact( bool nValue )
{
	internalBuffer[27] = (UI08)((nValue?1:0));
}

CPGraphicalEffect::CPGraphicalEffect( UI08 effectType, cBaseObject &src, cBaseObject &trg )
{
	InternalReset();
	Effect( effectType );
	SourceSerial( src );
	TargetSerial( trg );
}
CPGraphicalEffect::CPGraphicalEffect( UI08 effectType, cBaseObject &src )
{
	InternalReset();
	Effect( effectType );
	SourceSerial( src );
}

void CPGraphicalEffect::SourceLocation( cBaseObject& toSet )
{
	SourceLocation( toSet.GetX(), toSet.GetY(), toSet.GetZ() );
}
void CPGraphicalEffect::TargetLocation( cBaseObject& toSet )
{
	TargetLocation( toSet.GetX(), toSet.GetY(), toSet.GetZ() );
}

void CPGraphicalEffect::SourceLocation( SI16 x, SI16 y, SI08 z )
{
	X( x );
	Y( y );
	Z( z );
}
void CPGraphicalEffect::TargetLocation( SI16 x, SI16 y, SI08 z )
{
	XTrg( x );
	YTrg( y );
	ZTrg( z );
}


//0xA1 Packet
//Last Modified on Tuesday, 20-Apr-1999 
//Update Current Health (9 bytes) 
//	BYTE cmd 
//	BYTE[4] playerID 
//	BYTE[2] maxHealth 
//	BYTE[2] currentHealth
//0xA2 Packet
//Last Modified on Tuesday, 21-Apr-1998 20:45:30 EDT 
//Update Current Mana (9 bytes) 
//	BYTE cmd 
//	BYTE[4] playerID 
//	BYTE[2] maxMana 
//	BYTE[2] currentMana 
//0xA3 Packet
//Last Modified on Tuesday, 21-Apr-1998 20:45:32 EDT 
//Update Current Stamina (9 bytes) 
//	BYTE cmd 
//	BYTE[4] playerID 
//	BYTE[2] maxStamina 
//	BYTE[2] currentStamina 
void CPUpdateStat::InternalReset( void )
{
	internalBuffer.resize( 9 );
	internalBuffer[0] = 0xA1;
}
CPUpdateStat::CPUpdateStat( CChar &toUpdate, UI08 statNum )
{
	InternalReset();
	Serial( toUpdate.GetSerial() );
	switch( statNum )
	{
		case 0:	MaxVal( toUpdate.GetStrength() );
				CurVal( toUpdate.GetHP() );
				break;
		case 2:	MaxVal( toUpdate.GetDexterity() );
				CurVal( toUpdate.GetStamina() );
				break;
		case 1:	MaxVal( toUpdate.GetIntelligence() );
				CurVal( toUpdate.GetMana() );
				break;
	}
	internalBuffer[0] += (UI08)(statNum);
}
void CPUpdateStat::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 1, toSet );
}
void CPUpdateStat::MaxVal( SI16 maxVal )
{
	PackShort( &internalBuffer[0], 5, maxVal );
}
void CPUpdateStat::CurVal( SI16 curVal )
{
	PackShort( &internalBuffer[0], 7, curVal );
}

//0xAF Packet
//Last Modified on Tuesday, 20-Apr-1999 
//Display Death Action (13 bytes) 
//	BYTE cmd 
//	BYTE[4] player id
//	BYTE[4] corpse id
//	BYTE[4] unknown (all 0)
void CPDeathAction::InternalReset( void )
{
	internalBuffer.resize( 13 );
	internalBuffer[0] = 0xAF;
	internalBuffer[9] = 0;
	internalBuffer[10] = 0;
	internalBuffer[11] = 0;
}
CPDeathAction::CPDeathAction( CChar &dying, CItem &corpse )
{
	InternalReset();
	Player( dying.GetSerial() );
	Corpse( corpse.GetSerial() );
}
CPDeathAction::CPDeathAction()
{
	InternalReset();
}
void CPDeathAction::Player( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 1, toSet );
}
void CPDeathAction::Corpse( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 5, toSet );
}
void CPDeathAction::FallDirection( UI08 toFall )
{
	internalBuffer[12] = toFall;
}
CPDeathAction &CPDeathAction::operator=( CChar &dying )
{
	Player( dying.GetSerial() );
	return (*this);
}
CPDeathAction &CPDeathAction::operator=( CItem &corpse )
{
	Corpse( corpse.GetSerial() );
	return (*this);
}

//0x6D Packet
//Last Modified on Sunday, 17-May-1998 13:33:59 EDT 
//Play Midi Music (3 bytes) 
//	BYTE cmd 
//	BYTE[2] musicID 
void CPPlayMusic::InternalReset( void )
{
	internalBuffer.resize( 3 );
	internalBuffer[0] = 0x6D;
}
CPPlayMusic::CPPlayMusic( SI16 musicID )
{
	InternalReset();
	MusicID( musicID );
}
CPPlayMusic::CPPlayMusic()
{
	InternalReset();
}
void CPPlayMusic::MusicID( SI16 musicID )
{
	internalBuffer[1] = (UI08)(musicID>>8);
	internalBuffer[2] = (UI08)(musicID%256);
}

//0x24 Packet
//Last Modified on Tuesday, 14-Apr-1998 20:53:33 EDT 
//Draw Container (7 bytes) 
//	BYTE cmd 
//	BYTE[4] item id 
//	BYTE[2] model-Gump 
//		0x003c = backpack 
void CPDrawContainer::InternalReset( void )
{
	internalBuffer.resize( 7 );
	internalBuffer[0] = 0x24;
}
CPDrawContainer::CPDrawContainer()
{
	InternalReset();
}
CPDrawContainer::CPDrawContainer( CItem &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPDrawContainer::Model( UI16 newModel )
{
	internalBuffer[5] = (UI08)(newModel>>8);
	internalBuffer[6] = (UI08)(newModel%256);
}
CPDrawContainer &CPDrawContainer::operator=( CItem &toCopy )
{
	CopyData( toCopy );
	return (*this);
}
void CPDrawContainer::CopyData( CItem &toCopy )
{
	Serial( toCopy.GetSerial() );
}
void CPDrawContainer::Serial( SERIAL toSet )
{
	internalBuffer[1] = (UI08)(toSet>>24);
	internalBuffer[2] = (UI08)(toSet>>16);
	internalBuffer[3] = (UI08)(toSet>>8);
	internalBuffer[4] = (UI08)(toSet%256);
}

//	0x7C Packet
//	Last Modified on Thursday, 03-Sep-2002
//	Open Dialog Box (Variable # of bytes) 
//	BYTE cmd													0 
//	BYTE[2] blockSize											1
//	BYTE[4] dialogID (echo'd back to the server in 7d)			3
//	BYTE[2] menuid (echo'd back to server in 7d)				7
//	BYTE length of question										9
//	BYTE[length of question] question text						10
//	BYTE # of responses 
//	Then for each response: 
//		BYTE[2] model id # of shown item (if grey menu -- then always 0x00 as msb) 
//		BYTE[2] color of shown item
//		BYTE response text length 
//		BYTE[response text length] response text 

void CPOpenGump::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[SEND]Packet     : CPOpenGump 0x7C --> Length: %i\n", internalBuffer.size() );
	fprintf( fp, "DialogID         : %x\n", UnpackULong(  &internalBuffer[0], 3  ) );
	fprintf( fp, "MenuID           : %x\n", UnpackUShort( &internalBuffer[0], 7  ) );
	fprintf( fp, "Question Length  : %i\n", internalBuffer[9] );
	fprintf( fp, "Question         : " );

	for( UI32 i = 0; i < internalBuffer[9]; ++i )
		fprintf( fp, "%c", internalBuffer[10+i] );
	fprintf( fp, "\n" );

	fprintf( fp, "# Responses      : %i\n", internalBuffer[responseBaseOffset] );
	size_t offsetCount = responseBaseOffset + 1;
	for( UI32 j = 0; j < internalBuffer[responseBaseOffset]; ++j )
	{
		fprintf( fp, "    Response %i\n", j );
		fprintf( fp, "      Model  : %x\n", UnpackUShort( &internalBuffer[0], static_cast< UI32 >(offsetCount + 0) ) );
		fprintf( fp, "      Colour : %x\n", UnpackUShort( &internalBuffer[0], static_cast< UI32 >(offsetCount + 2) ) );
		fprintf( fp, "      Length : %i\n", internalBuffer[offsetCount + 4] );
		fprintf( fp, "      Text   : " );

		for( UI32 k = 0; k < internalBuffer[offsetCount + 4]; ++k )
			fprintf( fp, "%c", internalBuffer[offsetCount + 5 + k] );
		fprintf( fp, "\n" );
		offsetCount += ( 5 + internalBuffer[offsetCount + 4] );
	}
	fprintf( fp, "  Raw dump     :\n" );
	cPUOXBuffer::Log( fp, false );
}
void CPOpenGump::Question( std::string toAdd )
{
	internalBuffer.resize( 10 + toAdd.length() + 2 );	// 10 for start of string, length of string + NULL, plus spot for # responses
	for( UI32 i = 0; i < toAdd.length(); ++i )
		internalBuffer[10 + i] = toAdd[i];
#ifdef _DEBUG
	if( toAdd.length() >= 255 )
		Console.Error( 1, "CPOpenGump::Question toAdd.length() is too long (%i)", toAdd.length() );
#endif
	internalBuffer[9]	= static_cast< UI08 >(toAdd.length() + 1);
	responseBaseOffset	= (internalBuffer.size() - 1);
	responseOffset		= responseBaseOffset + 1;
}
void CPOpenGump::AddResponse( UI16 modelNum, UI16 colour, std::string responseText )
{
	++internalBuffer[responseBaseOffset]; // increment number of responses
#ifdef _DEBUG
	if( responseText.length() >= 255 )
		Console.Error( 1, "CPOpenGump::AddResponse responseText is too long (%i)", responseText.length() );
#endif
	UI16 toAdd = static_cast< UI16 >(5 + responseText.length());
	internalBuffer.resize( internalBuffer.size() + toAdd );
	PackShort(  &internalBuffer[0], static_cast< UI32 >(responseOffset + 0), modelNum );
	PackShort(  &internalBuffer[0], static_cast< UI32 >(responseOffset + 2), colour   );
	internalBuffer[responseOffset + 4] = static_cast< UI08 >(responseText.length());
	PackString( &internalBuffer[0], static_cast< UI32 >(responseOffset + 5), responseText, static_cast< UI32 >(responseText.length()) );
	responseOffset += toAdd;
}
void CPOpenGump::Finalize( void )
{
	PackShort( &internalBuffer[0], 1, static_cast< UI16 >(internalBuffer.size()) );
}
void CPOpenGump::InternalReset( void )
{
	responseOffset		= 0xFFFFFFFF;
	responseBaseOffset	= 0xFFFFFFFF;
	internalBuffer.resize( 9 );
	internalBuffer[0] = 0x7C;
	internalBuffer[1] = 0x00;
	internalBuffer[2] = 0x00;
	internalBuffer[3] = 0x01;
	internalBuffer[4] = 0x02;
	internalBuffer[5] = 0x03;
	internalBuffer[6] = 0x04;
	internalBuffer[7] = 0x00;
	internalBuffer[8] = 0x64;
}
CPOpenGump::CPOpenGump()
{
	InternalReset();
}
CPOpenGump::CPOpenGump( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPOpenGump::Length( int totalLines )
{
	PackShort( &internalBuffer[0], 1, totalLines );
}
void CPOpenGump::GumpIndex( int index )
{
	PackShort( &internalBuffer[0], 7, index );
}
CPOpenGump &CPOpenGump::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}
void CPOpenGump::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
}
void CPOpenGump::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 3, toSet );
}




//0x6C Packet
//Last Modified on Sunday, 13-Feb-2000 
//Targeting Cursor Commands (19 bytes) 
//	BYTE cmd 
//	BYTE type 
//		0x00 = Select Object
//		0x01 = Select X, Y, Z 
//	BYTE[4] cursorID 
//	BYTE Cursor Type 
//		Always 0 now  
//	The following are always sent but are only valid if sent by client 
//	BYTE[4] Clicked On ID 
//	BYTE[2] click xLoc 
//	BYTE[2] click yLoc 
//	BYTE unknown (0x00) 
//	BYTE click zLoc 
//	BYTE[2] model # (if a static tile, 0 if a map/landscape tile)
//Note: the model # shouldnt be trusted.
CPTargetCursor::CPTargetCursor()
{
	internalBuffer.resize( 19 );
	internalBuffer[0] = 0x6C;
	CursorType( 0 );
}
void CPTargetCursor::Type( UI08 nType )
{
	internalBuffer[1] = nType;
}
void CPTargetCursor::ID( SERIAL toSet )
{
	internalBuffer[2] = (UI08)(toSet>>24);
	internalBuffer[3] = (UI08)(toSet>>16);
	internalBuffer[4] = (UI08)(toSet>>8);
	internalBuffer[5] = (UI08)(toSet%256);
}
void CPTargetCursor::CursorType( UI08 nType )
{
	internalBuffer[6] = nType;
}

//0x11 Packet
//Last Modified on Sunday, 28-Jul-2002
//Stat window info 
//	BYTE cmd 
//	BYTE[2] packet length 
//	BYTE[4] player id 
//	BYTE[30] playerName 
//	BYTE[2] currentHitpoints 
//	BYTE[2] maxHitpoints 
//	BYTE[1] name change flag (0x1 = allowed, 0 = not allowed)
//	BYTE[1] flag 
//		(0x00 no more data following (end of packet here).  
//		0x01 more data after this flag following, 
//		0x03: like 1, with extended info)
//	BYTE sex (0=male, 1=female) 
//	BYTE[2] str 
//	BYTE[2] dex 
//	BYTE[2] int 
//	BYTE[2] currentStamina 
//	BYTE[2] maxStamina 
//	BYTE[2] currentMana 
//	BYTE[2] maxMana 
//	BYTE[4] gold 
//	BYTE[2] armor class 
//	BYTE[2] weight 
//	If (flag == 3) (fill new statusbar data)
// 	BYTE[2]  statcap
//	BYTE  pets current
//	BYTE pets max
//	If (flag == 4) 
//	BYTE[2]	fireresist
//	BYTE[2]	coldresist
//	BYTE[2]	poisonresist
//	BYTE[2]	energyresist
//	BYTE[2]	luck
//	BYTE[2]	damage max 
//	BYTE[2]	damage min
//	BYTE[4]	unknown

//Note: Server Message
//Note: For characters other than the player, currentHitpoints and maxHitpoints are not the actual values.  MaxHitpoints is a fixed value, and currentHitpoints works like a percentage.
void CPStatWindow::SetCharacter( CChar &toCopy, cSocket &target )
{
	if( target.ReceivedVersion() )
	{
		if( target.ClientVersionMajor() >= 4 )
		{
			extended3 = true;
			extended4 = true;
			internalBuffer.resize( 88 );
			internalBuffer[2] = 88;
			Flag( 4 );
		}
		else if( target.ClientVersionMajor() >= 3 )
		{
			extended3 = true;
			internalBuffer.resize( 70 );
			internalBuffer[2] = 70;
			Flag( 3 );
		}
	}
	else
		Flag( 1 );
	Serial( toCopy.GetSerial() );
	Name( toCopy.GetName().c_str() );
	bool isDead = toCopy.IsDead();
	if( isDead )
	{
		CurrentHP( 0 );
		MaxHP( 0 );
	}
	else
	{
		CurrentHP( toCopy.GetHP() );
		MaxHP( toCopy.GetStrength() );
	}
	NameChange( false );
	if( toCopy.GetID() == 0x0190 || toCopy.GetID() == 0x0192 )
		Sex( 0 );
	else
		Sex( 1 );
	if( isDead )
	{
		Strength( 0 );
		Dexterity( 0 );
		Intelligence( 0 );
		Stamina( 0 );
		MaxStamina( 0 );
		Mana( 0 );
		MaxMana( 0 );
	}
	else
	{
		Strength( toCopy.GetStrength() );
		Dexterity( toCopy.GetDexterity() );
		Intelligence( toCopy.GetIntelligence() );
		Stamina( toCopy.GetStamina() );
		MaxStamina( toCopy.GetDexterity() );
		Mana( toCopy.GetMana() );
		MaxMana( toCopy.GetIntelligence() );
	}
	if( extended3 )
	{
		StatCap( cwmWorldState->ServerData()->ServerStatCapStatus() );
		CurrentPets( toCopy.GetPetList()->size() );
		MaxPets( 0xFF );
	}
	if( extended4 )
	{
		FireResist( 0 );
		ColdResist( 0 );
		EnergyResist( 0 );
		PoisonResist( 0 );
		Luck( 0 );
		Unknown( 0 );
		DamageMin( 0 );
		DamageMax( Combat->calcAtt( &toCopy ) );
	}
}
void CPStatWindow::InternalReset( void )
{
	internalBuffer.resize( 66 );
	internalBuffer[0] = 0x11;
	internalBuffer[1] = 0;
	internalBuffer[2] = 66;
	extended3 = false;
	extended4 = false;
	Flag( 0 );
}
CPStatWindow::CPStatWindow()
{
	InternalReset();
}
CPStatWindow::CPStatWindow( CChar &toCopy, cSocket &target )
{
	InternalReset();
	SetCharacter( toCopy, target );
}
void CPStatWindow::Serial( SERIAL toSet )
{
	internalBuffer[3] = (UI08)(toSet>>24);
	internalBuffer[4] = (UI08)(toSet>>16);
	internalBuffer[5] = (UI08)(toSet>>8);
	internalBuffer[6] = (UI08)(toSet%256);
}
void CPStatWindow::Name( const char *nName )
{
	if( strlen( nName ) >= 30 )
	{
		strncpy( (char *)&internalBuffer[7], nName, 29 );
		internalBuffer[36] = 0;
	}
	else
		strcpy( (char*)&internalBuffer[7], nName );
}
void CPStatWindow::CurrentHP( SI16 nValue )
{
	internalBuffer[37] = (UI08)(nValue>>8);
	internalBuffer[38] = (UI08)(nValue%256);
}
void CPStatWindow::MaxHP( SI16 nValue )
{
	internalBuffer[39] = (UI08)(nValue>>8);
	internalBuffer[40] = (UI08)(nValue%256);
}
void CPStatWindow::NameChange( bool nValue )
{
	internalBuffer[41] = (UI08)((nValue?0xFF:0));
}
void CPStatWindow::Flag( UI08 nValue )
{
	internalBuffer[42] = nValue;
}
void CPStatWindow::Sex( UI08 nValue )
{
	internalBuffer[43] = nValue;
}
void CPStatWindow::Strength( SI16 nValue )
{
	internalBuffer[44] = (UI08)(nValue>>8);
	internalBuffer[45] = (UI08)(nValue%256);
}
void CPStatWindow::Dexterity( SI16 nValue )
{
	internalBuffer[46] = (UI08)(nValue>>8);
	internalBuffer[47] = (UI08)(nValue%256);
}
void CPStatWindow::Intelligence( SI16 nValue )
{
	internalBuffer[48] = (UI08)(nValue>>8);
	internalBuffer[49] = (UI08)(nValue%256);
}
void CPStatWindow::Stamina( SI16 nValue )
{
	internalBuffer[50] = (UI08)(nValue>>8);
	internalBuffer[51] = (UI08)(nValue%256);
}
void CPStatWindow::MaxStamina( SI16 nValue )
{
	internalBuffer[52] = (UI08)(nValue>>8);
	internalBuffer[53] = (UI08)(nValue%256);
}
void CPStatWindow::Mana( SI16 nValue )
{
	internalBuffer[54] = (UI08)(nValue>>8);
	internalBuffer[55] = (UI08)(nValue%256);
}
void CPStatWindow::MaxMana( SI16 nValue )
{
	internalBuffer[56] = (UI08)(nValue>>8);
	internalBuffer[57] = (UI08)(nValue%256);
}
void CPStatWindow::Gold( UI32 gValue )
{
	internalBuffer[58] = (UI08)(gValue>>24);
	internalBuffer[59] = (UI08)(gValue>>16);
	internalBuffer[60] = (UI08)(gValue>>8);
	internalBuffer[61] = (UI08)(gValue%256);
}
void CPStatWindow::AC( UI16 nValue )
{
	internalBuffer[62] = (UI08)(nValue>>8);
	internalBuffer[63] = (UI08)(nValue%256);
}
void CPStatWindow::Weight( UI16 nValue )
{
	internalBuffer[64] = (UI08)(nValue>>8);
	internalBuffer[65] = (UI08)(nValue%256);
}

void CPStatWindow::StatCap( UI16 value )
{
	if( extended3 )
	{
		internalBuffer[66] = (UI08)(value>>8);
		internalBuffer[67] = (UI08)(value%256);
	}
}
void CPStatWindow::CurrentPets( UI08 value )
{
	if( extended3 )
	{
		internalBuffer[68] = value;
	}
}
void CPStatWindow::MaxPets( UI08 value )
{
	if( extended3 )
	{
		internalBuffer[69] = value;
	}
}
void CPStatWindow::FireResist( UI16 value )
{
	if( extended4 )
	{
		internalBuffer[70] = (UI08)(value>>8);
		internalBuffer[71] = (UI08)(value%256);
	}
}
void CPStatWindow::ColdResist( UI16 value )
{
	if( extended4 )
	{
		internalBuffer[72] = (UI08)(value>>8);
		internalBuffer[73] = (UI08)(value%256);
	}
}
void CPStatWindow::PoisonResist( UI16 value )
{
	if( extended4 )
	{
		internalBuffer[74] = (UI08)(value>>8);
		internalBuffer[75] = (UI08)(value%256);
	}
}
void CPStatWindow::EnergyResist( UI16 value )
{
	if( extended4 )
	{
		internalBuffer[76] = (UI08)(value>>8);
		internalBuffer[77] = (UI08)(value%256);
	}
}
void CPStatWindow::Luck( UI16 value )
{
	if( extended4 )
	{
		internalBuffer[78] = (UI08)(value>>8);
		internalBuffer[79] = (UI08)(value%256);
	}
}
void CPStatWindow::DamageMax( UI16 value )
{
	if( extended4 )
	{
		internalBuffer[80] = (UI08)(value>>8);
		internalBuffer[81] = (UI08)(value%256);
	}
}
void CPStatWindow::DamageMin( UI16 value )
{
	if( extended4 )
	{
		internalBuffer[82] = (UI08)(value>>8);
		internalBuffer[83] = (UI08)(value%256);
	}
}
void CPStatWindow::Unknown( UI32 value )
{
	if( extended4 )
	{
		internalBuffer[84] = (UI08)(value>>24);
		internalBuffer[85] = (UI08)(value>>16);
		internalBuffer[86] = (UI08)(value>>8);
		internalBuffer[87] = (UI08)(value%256);
	}
}

//0x53 Packet
//Last Modified on Sunday, 13-Feb-2000 
//Idle Warning(2 bytes) 
//	BYTE cmd 
//	BYTE value 
//		(0x07 idle
//		0x05 another character is online
//Another character from this account is currently online in this world.  You must either log in as that character or wait for it to time out.

void CPIdleWarning::InternalReset( void )
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x53;
}
CPIdleWarning::CPIdleWarning()
{
	InternalReset();
}
CPIdleWarning::CPIdleWarning( UI08 errorNum )
{
	InternalReset();
	Error( errorNum );
}
void CPIdleWarning::Error( UI08 errorNum )
{
	internalBuffer[1] = errorNum;
}


//0x5B Packet
//Last Modified on Saturday, 18-Apr-1998 17:49:28 EDT 
//Time (4 bytes) 
//	BYTE cmd 
//	BYTE hour 
//	BYTE minute 
//	BYTE second 
CPTime::CPTime()
{
	InternalReset();
}
void CPTime::Hour( UI08 hour )
{
	internalBuffer[1] = hour;
}
void CPTime::Minute( UI08 minute )
{
	internalBuffer[2] = minute;
}
void CPTime::Second( UI08 second )
{
	internalBuffer[3] = second;
}
void CPTime::InternalReset( void )
{
	internalBuffer.resize( 4 );
	internalBuffer[0] = 0x5B;
}
CPTime::CPTime( UI08 hour, UI08 minute, UI08 second )
{
	InternalReset();
	Hour( hour );
	Minute( minute );
	Second( second );
}

//0x55 Packet
//Last Modified on Monday, 26-Oct-1998 
//Login Complete, Start Game (1 byte) 
//	BYTE cmd 
CPLoginComplete::CPLoginComplete()
{
	internalBuffer.resize( 1 );
	internalBuffer[0] = 0x55;
}


//0x69 Packet
//Last Modified on Thursday, 23-Apr-1998 19:26:05 EDT 
//Change Text/Emote Color (5 bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[2] unknown1 
//The client sends two of these independent of the color chosen. It sends two of them in quick succession as part of the "same" packet. The unknown1 is 0x00 0x01 in the first and 0x00 0x02 in the second.
//Note, this message has been removed.  It is no longer used.
CPTextEmoteColour::CPTextEmoteColour()
{
	BlockSize( 5 );
}
void CPTextEmoteColour::BlockSize( SI16 newValue )
{
	internalBuffer.resize( newValue );
	internalBuffer[0] = 0x69;
	internalBuffer[1] = (UI08)(newValue>>8);
	internalBuffer[2] = (UI08)(newValue%256);
}
void CPTextEmoteColour::Unknown( SI16 newValue )
{
	internalBuffer[3] = (UI08)(newValue>>8);
	internalBuffer[4] = (UI08)(newValue%256);
}


//0x72 Packet
//Last Modified on Saturday, 14-Nov-1998 
//Request War Mode Change/Send War Mode status (5 bytes) 
//	BYTE cmd 
//	BYTE flag 
//		0x00 - Normal 
//		0x01 - Fighting 
//	BYTE[3] unknown1 (always 00 32 00 in testing) 
//Server replies with 0x77 packet
CPWarMode::CPWarMode()
{
	InternalReset();
}

void CPWarMode::Flag( UI08 nFlag )
{
	internalBuffer[1] = nFlag;
}
void CPWarMode::InternalReset( void )
{
	internalBuffer.resize( 5 );
	internalBuffer[0] = 0x72;
	internalBuffer[2] = 0x00;
	internalBuffer[3] = 0x32;
	internalBuffer[4] = 0x00;
}
CPWarMode::CPWarMode( UI08 nFlag )
{
	InternalReset();
	Flag( nFlag );
}

//0x33 Packet
//Last Modified on Thursday, 19-Nov-1998 
//Pause/Resume Client (2 bytes)
//	BYTE cmd 
//	BYTE pause/resume (0=pause, 1=resume)
void CPPauseResume::InternalReset( void )
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x33;
}
CPPauseResume::CPPauseResume()
{
	InternalReset();
}
CPPauseResume::CPPauseResume( UI08 mode )
{
	InternalReset();
	Mode( mode );
}
void CPPauseResume::Mode( UI08 mode )
{
	internalBuffer[1] = mode;
}

//0xA5 Packet
//Last Modified on Wednesday, 06-May-1998 23:30:48 EDT 
//Open Web Browser (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[blockSize-3] null terminated full web address 
void CPWebLaunch::InternalReset( void )
{
	internalBuffer.resize( 4 );
	internalBuffer[0] = 0xA5;
}
CPWebLaunch::CPWebLaunch()
{
	InternalReset();
}
CPWebLaunch::CPWebLaunch( const char *txt )
{
	InternalReset();
	Text( txt );
}
void CPWebLaunch::Text( const char *txt )
{
	const SI16 tLen = (SI16)strlen( txt )+4;
	SetSize( tLen );
	strcpy( (char *)&internalBuffer[3], txt );
}

void CPWebLaunch::SetSize( SI16 newSize )
{
	internalBuffer.resize( newSize );
	internalBuffer[1] = (UI08)(newSize>>8);
	internalBuffer[2] = (UI08)(newSize%256);
}

//0xBA Packet
//Last Modified on Sunday, 13-Feb-2000  
//Quest Arrow (6 bytes) 
//	BYTE cmd
//	BYTE active (1=on, 0=off)
//	BYTE[2] xLoc 
//	BYTE[2] yLoc
//Server Message
void CPTrackingArrow::InternalReset( void )
{
	internalBuffer.resize( 6 );
	internalBuffer[0] = 0xBA;
}
CPTrackingArrow::CPTrackingArrow()
{
	InternalReset();
}
CPTrackingArrow::CPTrackingArrow( SI16 x, SI16 y )
{
	InternalReset();
	Location( x, y );
}
void CPTrackingArrow::Location( SI16 x, SI16 y )
{
	internalBuffer[2] = (UI08)(x>>8);
	internalBuffer[3] = (UI08)(x%256);
	internalBuffer[4] = (UI08)(y>>8);
	internalBuffer[5] = (UI08)(y%256);
}
CPTrackingArrow &CPTrackingArrow::operator=( cBaseObject &toCopy )
{
	Location( toCopy.GetX(), toCopy.GetY() );
	return (*this);
}
void CPTrackingArrow::Active( UI08 value )
{
	internalBuffer[1] = value;
}

CPTrackingArrow::CPTrackingArrow( cBaseObject &toCopy )
{
	InternalReset();
	Location( toCopy.GetX(), toCopy.GetY() );
}

//0x27 Packet
//Last Modified on Friday, 19-May-2000
//Reject Request to Move Items (2 bytes) 
//	BYTE cmd 
//	BYTE unknown1 (0x00) 
//Note: Server Message
void CPBounce::InternalReset( void )
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x27;
}
CPBounce::CPBounce()
{
	InternalReset();
}
CPBounce::CPBounce( UI08 mode )
{
	InternalReset();
	Mode( mode );
}
void CPBounce::Mode( UI08 mode )
{
	internalBuffer[1] = mode;
}

//0x95 Packet
//Last Modified on Thursday, 30-Apr-1998 18:34:08 EDT 
//Dye Window (9 bytes) 
//	BYTE cmd 
//	BYTE[4] itemID of dye target
//	BYTE[2] ignored on send, model on return
//	BYTE[2] model on send, color on return  (default on server send is 0x0FAB) 
//NOTE: This packet is sent by both the server and client
void CPDyeVat::InternalReset( void )
{
	internalBuffer.resize( 9 );
	internalBuffer[0] = 0x95;
	internalBuffer[5] = 0;
	internalBuffer[6] = 0;
}
void CPDyeVat::CopyData( cBaseObject &target )
{
	Serial( target.GetSerial() );
	Model( target.GetID() );
}
CPDyeVat::CPDyeVat()
{
	InternalReset();
}
CPDyeVat::CPDyeVat( cBaseObject &target )
{
	InternalReset();
	CopyData( target );
}
void CPDyeVat::Serial( SERIAL toSet )
{
	internalBuffer[1] = (UI08)(toSet>>24);
	internalBuffer[2] = (UI08)(toSet>>16);
	internalBuffer[3] = (UI08)(toSet>>8);
	internalBuffer[4] = (UI08)(toSet%256);
}
void CPDyeVat::Model( SI16 toSet )
{
	internalBuffer[7] = (UI08)(toSet>>8);
	internalBuffer[8] = (UI08)(toSet%256);
}
CPDyeVat &CPDyeVat::operator=( cBaseObject &target )
{
	CopyData( target );
	return (*this);
}

//0x99 Packet
//Last Modified on Friday, 20-Nov-1998 
//Bring Up House/Boat Placement View (26 bytes) 
//	BYTE cmd 
//	BYTE request (0x01 from server, 0x00 from client) 
//	BYTE[4] ID of deed 
//	BYTE[12] unknown (all 0)
//	BYTE[2] multi model (item model - 0x4000)
//	BYTE[6] unknown (all 0)
void CPMultiPlacementView::InternalReset( void )
{
	internalBuffer.resize( 26 );
	internalBuffer[0] = 0x99;
	for( UI08 i = 6; i < 18; ++i )
		internalBuffer[i] = 0;
	for( UI08 k = 20; k < 26; ++k )
		internalBuffer[k] = 0;
}
void CPMultiPlacementView::CopyData( CItem &target )
{
	DeedSerial( target.GetSerial() );
}
CPMultiPlacementView::CPMultiPlacementView()
{
	InternalReset();
}
CPMultiPlacementView::CPMultiPlacementView( CItem &target )
{
	InternalReset();
	CopyData( target );
}
void CPMultiPlacementView::RequestType( UI08 rType )
{
	internalBuffer[1] = rType;
}
void CPMultiPlacementView::DeedSerial( SERIAL toSet )
{
	internalBuffer[2] = (UI08)(toSet>>24);
	internalBuffer[3] = (UI08)(toSet>>16);
	internalBuffer[4] = (UI08)(toSet>>8);
	internalBuffer[5] = (UI08)(toSet%256);
}
void CPMultiPlacementView::MultiModel( SI16 toSet )
{
	internalBuffer[18] = (UI08)(toSet>>8);
	internalBuffer[19] = (UI08)(toSet%256);
}
CPMultiPlacementView &CPMultiPlacementView::operator=( CItem &target )
{
	CopyData( target );
	return (*this);
}

CPMultiPlacementView::CPMultiPlacementView( SERIAL toSet )
{
	InternalReset();
	DeedSerial( toSet );
}

CPEnableClientFeatures::CPEnableClientFeatures()
{
//Enable locked client features (3 bytes) 
//        BYTE cmd 
//        BYTE[2] feature#
//Bit# 1 T2A upgrade, enables chatbutton, 
//Bit# 2 enables LBR update.  (of course LBR installation is required)
//(plays MP3 instead of midis, 2D LBR client shows new LBR monsters,)
//Bit# 3: unknown, never seen it set
//Bit# 4: unknown, set on OSI servers that have AOS code - no matter of account status (doesnt seem to unlock/lock anything on client side)
//Bit# 5: enables AOS update (necro/paladin skills for all clients, malas map/AOS monsters if AOS installation present)
//Bit# 15: since client 4.0 this bit has to be set, otherwise bits 3..14 are ignored.
//Thus 0: neither T2A NOR LBR, equal to not sending it at all, 
//1 is T2A, chatbutton, 
//2 is LBR without chatbutton, 
//3 is LBR with chatbutton
//8013, LBR + chatbutton + AOS enabled
//Note1: this message is send immediately after login.
//Note2: on OSI  servers this controls features OSI enables/disables via upgrade codes.
//Note3: a 3 doesnt seem to hurt older (NON LBR) clients.

	internalBuffer.resize( 3 );
	internalBuffer[0] = 0xB9;
	internalBuffer[1] = 0x80;		// 0x00
	internalBuffer[2] = 0x1F;		// 0x11
}

//0x25 Packet
//Last Modified on Saturday, 02-May-1998 16:05:35 EDT 
//Add Item to Container (20 bytes) 
//	BYTE cmd 
//	BYTE[4] item id to add 
//	BYTE[2] model 
//	BYTE unknown1 (0)
//	BYTE[2] # of items 
//	BYTE[2] xLoc in container 
//	BYTE[2] yLoc in container 
//	BYTE[4] itemID of container 
//	BYTE[2] color 
void CPAddItemToCont::InternalReset( void )
{
	internalBuffer.resize( 20 );
	internalBuffer[0] = 0x25;
	internalBuffer[7] = 0;
}
void CPAddItemToCont::CopyData( CItem &toCopy )
{
	Serial( toCopy.GetSerial() );
	Model( toCopy.GetID() );
	NumItems( toCopy.GetAmount() );
	X( toCopy.GetX() );
	Y( toCopy.GetY() );
	Container( toCopy.GetContSerial() );
	Colour( toCopy.GetColour() );
}
CPAddItemToCont::CPAddItemToCont()
{
	InternalReset();
}
CPAddItemToCont::CPAddItemToCont( CItem &toAdd )
{
	InternalReset();
	CopyData( toAdd );
}
void CPAddItemToCont::Serial( SERIAL toSet )
{
	internalBuffer[1] = (UI08)(toSet>>24);
	internalBuffer[2] = (UI08)(toSet>>16);
	internalBuffer[3] = (UI08)(toSet>>8);
	internalBuffer[4] = (UI08)(toSet%256);
}
void CPAddItemToCont::Model( SI16 toSet )
{
	internalBuffer[5] = (UI08)(toSet>>8);
	internalBuffer[6] = (UI08)(toSet%256);
}
void CPAddItemToCont::NumItems( SI16 toSet )
{
	internalBuffer[8] = (UI08)(toSet>>8);
	internalBuffer[9] = (UI08)(toSet%256);
}
void CPAddItemToCont::X( SI16 x )
{
	internalBuffer[10] = (UI08)(x>>8);
	internalBuffer[11] = (UI08)(x%256);
}
void CPAddItemToCont::Y( SI16 y )
{
	internalBuffer[12] = (UI08)(y>>8);
	internalBuffer[13] = (UI08)(y%256);
}
void CPAddItemToCont::Container( SERIAL toAdd )
{
	internalBuffer[14] = (UI08)(toAdd>>24);
	internalBuffer[15] = (UI08)(toAdd>>16);
	internalBuffer[16] = (UI08)(toAdd>>8);
	internalBuffer[17] = (UI08)(toAdd%256);
}
void CPAddItemToCont::Colour( SI16 toSet )
{
	internalBuffer[18] = (UI08)(toSet>>8);
	internalBuffer[19] = (UI08)(toSet%256);
}
CPAddItemToCont &CPAddItemToCont::operator=( CItem &toAdd )
{
	CopyData( toAdd );
	return (*this);
}

//0x26 Packet
//Last Modified on Friday, 19-May-2000
//Kick Player (5 bytes) 
//	BYTE cmd 
//	BYTE[4] ID of GM who issued kick? 
//Note: Server Message
void CPKickPlayer::InternalReset( void )
{
	internalBuffer.resize( 5 );
	internalBuffer[0] = 0x26;
}
void CPKickPlayer::CopyData( CChar &toCopy )
{
	Serial( toCopy.GetSerial() );
}
CPKickPlayer::CPKickPlayer()
{
	InternalReset();
}
CPKickPlayer::CPKickPlayer( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPKickPlayer::Serial( SERIAL toSet )
{
	internalBuffer[1] = (UI08)(toSet>>24);
	internalBuffer[2] = (UI08)(toSet>>16);
	internalBuffer[3] = (UI08)(toSet>>8);
	internalBuffer[4] = (UI08)(toSet%256);
}
CPKickPlayer &CPKickPlayer::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

//0x2C Packet
//Last Modified on Friday, 19-May-2000
//Resurrection Menu Choice (2 bytes) 
//	BYTE cmd 
//	BYTE action (2=ghost, 1=resurrect, 0=from server)
//Note: Client and Server Message
//Note: Resurrection menu has been removed from UO.
void CPResurrectMenu::InternalReset( void )
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x2C;
}
CPResurrectMenu::CPResurrectMenu()
{
	InternalReset();
}
CPResurrectMenu::CPResurrectMenu( UI08 action )
{
	InternalReset();
	Action( action );
}
void CPResurrectMenu::Action( UI08 action )
{	// valid values 0 == from server, 1 == resurrect, 2 == ghost (server/client job)
	internalBuffer[1] = action;
}

//0x2F Packet
//Last Modified on Saturday, 1-May-1999
//Fight Occurring (10 bytes) 
//	BYTE cmd 
//	BYTE unknown1 (0)
//	BYTE[4] ID of attacker 
//	BYTE[4] ID of attacked 
//This packet is sent when there is a fight going on somewhere on screen.
void CPFightOccurring::InternalReset( void )
{
	internalBuffer.resize( 10 );
	internalBuffer[0] = 0x2F;
	internalBuffer[1] = 0;
}
// Sent when fight occuring somewhere on the screen
CPFightOccurring::CPFightOccurring()
{
}
CPFightOccurring::CPFightOccurring( CChar &attacker, CChar &defender )
{
	Attacker( attacker );
	Defender( defender );
}
void CPFightOccurring::Attacker( SERIAL toSet )
{
	internalBuffer[2] = (UI08)(toSet>>24);
	internalBuffer[3] = (UI08)(toSet>>16);
	internalBuffer[4] = (UI08)(toSet>>8);
	internalBuffer[5] = (UI08)(toSet%256);
}
void CPFightOccurring::Attacker( CChar &attacker )
{
	Attacker( attacker.GetSerial() );
}
void CPFightOccurring::Defender( SERIAL toSet )
{
	internalBuffer[6] = (UI08)(toSet>>24);
	internalBuffer[7] = (UI08)(toSet>>16);
	internalBuffer[8] = (UI08)(toSet>>8);
	internalBuffer[9] = (UI08)(toSet%256);
}
void CPFightOccurring::Defender( CChar &defender )
{
	Defender( defender.GetSerial() );
}

void CPSkillsValues::InternalReset( void )
{
	internalBuffer.resize( 1 );
	internalBuffer[0] = 0x3A;
}
void CPSkillsValues::CopyData( CChar &toCopy )
{
	NumSkills( ALLSKILLS );
	SetCharacter( toCopy );
}

void CPSkillsValues::SetCharacter( CChar &toCopy )
{
	for( SI08 i = 0; i < NumSkills(); ++i )
		SkillEntry( i, toCopy.GetSkill( i ), toCopy.GetBaseSkill( i ), toCopy.GetSkillLock( i ) );
}

void CPSkillsValues::BlockSize( SI16 newValue )
{
	internalBuffer.resize( newValue );
	internalBuffer[1] = (UI08)(newValue>>8);
	internalBuffer[2] = (UI08)(newValue%256);
	internalBuffer[3] = 0;	// full list
	internalBuffer[4] = 0x00;
	internalBuffer[newValue-1] = 0;	// finish off with a double NULL
	internalBuffer[newValue-2] = 0;
}
CPSkillsValues::CPSkillsValues()
{
	InternalReset();
}
CPSkillsValues::CPSkillsValues( CChar &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
void CPSkillsValues::NumSkills( UI08 numSkills )
{
	BlockSize( numSkills * 7 + 6 );
}
UI08 CPSkillsValues::NumSkills( void )
{
	int size = ( (internalBuffer[1]) << 8 ) + internalBuffer[2];
	size -= 6;
	size /= 7;
	return (UI08)size;
}

void CPSkillsValues::SkillEntry( SI16 skillID, SI16 skillVal, SI16 baseSkillVal, UI08 skillLock )
{
	int offset = skillID * 7 + 5;
	internalBuffer[offset] = skillID + 1;
	WriteShort( offset + 1, skillVal );
	WriteShort( offset + 3, baseSkillVal );
	internalBuffer[offset + 5] = skillLock;
}
CPSkillsValues &CPSkillsValues::operator=( CChar &toCopy )
{
	CopyData( toCopy );
	return (*this);
}

void CPSkillsValues::WriteShort( SI16 offset, SI16 value )
{
	internalBuffer[offset] = (UI08)(value>>8);
	internalBuffer[offset+1] = (UI08)(value%256);
}

//0x90 Packet
//Last Modified on Sunday, 2-May-1999 
//Map message(19 bytes) 
//	BYTE cmd 
//	BYTE[4] key used 
//	BYTE[2] gump art id (0x139D)
//	BYTE[2] upper left x location
//	BYTE[2] upper left y location
//	BYTE[2] lower right x location
//	BYTE[2] lower right y location
//	BYTE[2] gump width in pixels
//	BYTE[2] gump height in pixels 
CPMapMessage::CPMapMessage()
{
	internalBuffer.resize( 19 );
	internalBuffer[0] = 0x90;
	GumpArt( 0x139D );
}
void CPMapMessage::UpperLeft( SI16 x, SI16 y )
{
	internalBuffer[7] = (UI08)(x>>8);
	internalBuffer[8] = (UI08)(x%256);
	internalBuffer[9] = (UI08)(y>>8);
	internalBuffer[10] = (UI08)(y%256);
}
void CPMapMessage::LowerRight( SI16 x, SI16 y )
{
	internalBuffer[11] = (UI08)(x>>8);
	internalBuffer[12] = (UI08)(x%256);
	internalBuffer[13] = (UI08)(y>>8);
	internalBuffer[14] = (UI08)(y%256);
}
void CPMapMessage::Dimensions( SI16 width, SI16 height )
{
	internalBuffer[15] = (UI08)(width>>8);
	internalBuffer[16] = (UI08)(width%256);
	internalBuffer[17] = (UI08)(height>>8);
	internalBuffer[18] = (UI08)(height%256);
}
void CPMapMessage::GumpArt( SI16 newArt )
{
	internalBuffer[5] = (UI08)(newArt>>8);
	internalBuffer[6] = (UI08)(newArt%256);
}
void CPMapMessage::KeyUsed( long key )
{
	internalBuffer[1] = (UI08)(key>>24);
	internalBuffer[2] = (UI08)(key>>16);
	internalBuffer[3] = (UI08)(key>>8);
	internalBuffer[4] = (UI08)(key%256);
}


//0x93 Packet
//Last Modified on Wednesday, 2-Feb-2000 
//Books - Title Page (99 bytes) 
//	BYTE cmd 
//	BYTE[4] bookID 
//	BYTE write flag 
//		0x00 - non-writable 
//		0x01 - writable 
//	BYTE new flag
//	BYTE[2] # of pages 
//	BYTE[60] title 
//	BYTE[30] author 
//Client sends a 0x93 message on book close Look into this. J
//Client Ver:
//Books - Update Title Page (99 bytes)
//	BYTE cmd
//	BYTE[4] bookID
//	BYTE[4] unknown (0)
//	BYTE[60] title (zero terminated, garbage after terminator)
//	BYTE[30] author (zero terminated, garbage after terminator)
CPBookTitlePage::CPBookTitlePage()
{
	internalBuffer.resize( 99 );
	internalBuffer[0] = 0x93;
}
void CPBookTitlePage::Serial( SERIAL toSet )
{
	internalBuffer[1] = (UI08)(toSet>>24);
	internalBuffer[2] = (UI08)(toSet>>16);
	internalBuffer[3] = (UI08)(toSet>>8);
	internalBuffer[4] = (UI08)(toSet%256);
}
void CPBookTitlePage::WriteFlag( UI08 flag )
{
	internalBuffer[5] = flag;
}
void CPBookTitlePage::NewFlag( UI08 flag )
{
	internalBuffer[6] = flag;
}
void CPBookTitlePage::Pages( SI16 pages )
{
	internalBuffer[7] = (UI08)(pages>>8);
	internalBuffer[8] = (UI08)(pages%256);
}
void CPBookTitlePage::Title( const char *txt )
{
	if( strlen( txt ) >= 60 )
	{
		strncpy( (char *)&internalBuffer[9], txt, 59 );
		internalBuffer[68] = 0;
	}
	else
		strcpy( (char *)&internalBuffer[9], txt );
}
void CPBookTitlePage::Author( const char *txt )
{
	if( strlen( txt ) >= 30 )
	{
		strncpy( (char *)&internalBuffer[69], txt, 29 );
		internalBuffer[98] = 0;
	}
	else
		strcpy( (char *)&internalBuffer[69], txt );
}


//0xBB Packet
//Last Modified on Sunday, 30-Jan-2000  
//Ultima Messenger  (9 bytes) 
//	BYTE cmd 
//	BYTE[4] id1
//	BYTE[4] id2
//Note: This is both a client and server message.
CPUltimaMessenger::CPUltimaMessenger()
{
	internalBuffer.resize( 9 );
	internalBuffer[0] = 0xBB;
}
void CPUltimaMessenger::ID1( SERIAL toSet )
{
	internalBuffer[1] = (UI08)(toSet>>24);
	internalBuffer[2] = (UI08)(toSet>>16);
	internalBuffer[3] = (UI08)(toSet>>8);
	internalBuffer[4] = (UI08)(toSet%256);
}
void CPUltimaMessenger::ID2( SERIAL toSet )
{
	internalBuffer[5] = (UI08)(toSet>>24);
	internalBuffer[6] = (UI08)(toSet>>16);
	internalBuffer[7] = (UI08)(toSet>>8);
	internalBuffer[8] = (UI08)(toSet%256);
}

cPInputBuffer *WhichLoginPacket( UI08 packetID, cSocket *s )
{
	switch( packetID )
	{
		case 0x00:	return NULL;	// Character Create
		case 0x01:	return NULL;	// Main Menu on the character select screen
		case 0x04:	return ( new CPIGodModeToggle( s )		);
		case 0x5D:	return NULL;	// Character Select
		case 0x73:	return ( new CPIKeepAlive( s )			);
		case 0x80:	return ( new CPIFirstLogin( s )			);
		case 0x83:	return NULL;	// Character Delete
		case 0x91:	return ( new CPISecondLogin( s )		);
		case 0xA0:	return ( new CPIServerSelect( s )		);
		case 0xA4:	return ( new CPISpy( s )				);
		case 0xBB:	return NULL;	// No idea
		case 0xBD:	return ( new CPIClientVersion( s )		);
		case 0xBF:	return NULL;	
		default:	break;
	}
	throw socket_error( "Bad packet request" );
	return NULL;
}

cPInputBuffer *WhichPacket( UI08 packetID, cSocket *s )
{
	switch( packetID )
	{
		case 0x02:	return ( new CPIMoveRequest( s )		);
		case 0x03:	return ( new CPITalkRequestAscii( s )	);
		case 0x04:	return ( new CPIGodModeToggle( s )		);
		case 0x05:	return ( new CPIAttack( s )				);
		case 0x06:	return ( new CPIDblClick( s )			);
		case 0x07:	return ( new CPIGetItem( s )			);
		case 0x08:	return ( new CPIDropItem( s )			);
		case 0x09:	return ( new CPISingleClick( s )		);
		case 0x13:	return ( new CPIEquipItem( s )			);
		case 0x22:	return ( new CPIResyncReq( s )			);
		case 0x2C:	return ( new CPIResMenuChoice( s )		);
		case 0x34:	return ( new CPIStatusRequest( s )		);
		case 0x6C:	return ( new CPITargetCursor( s )		);
		case 0x73:	return ( new CPIKeepAlive( s )			);
		case 0x75:	return ( new CPIRename( s )				);
		case 0x80:	return ( new CPIFirstLogin( s )			);
		case 0x91:	return ( new CPISecondLogin( s )		);
		case 0x98:	return ( new CPIAllNames3D( s )			);
		case 0xA0:	return ( new CPIServerSelect( s )		);
		case 0xA4:	return ( new CPISpy( s )				);
		case 0xA7:	return ( new CPITips( s )				);
		case 0xAD:	return ( new CPITalkRequestUnicode( s ) );
		case 0xB1:	return ( new CPIGumpMenuSelect( s )		);
		case 0xBD:	return ( new CPIClientVersion( s )		);
		case 0xC8:	return ( new CPIUpdateRangeChange( s )	);
		default:	return NULL;
	}
	return NULL;
}

//	0x80 Packet
//	Last Modified on Monday, 13-Apr-1998 17:06:30 EDT 
//	Login Request (62 bytes) 
//	BYTE cmd 
//	BYTE[30] userid 
//	BYTE[30] password 
//	BYTE unknown1 (not usually 0x00 - so not NULL) 

void CPIFirstLogin::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[RECV]Packet   : CPIFirstLogin 0x80 --> Length: 62\n" );
	fprintf( fp, "UserID         : %s\n", Name() );
	fprintf( fp, "Password       : %s\n", Pass() );
	fprintf( fp, "Unknown        : %i\n", Unknown() );
	fprintf( fp, "  Raw dump     :\n" );
	cPInputBuffer::Log( fp, false );
}
bool CPIFirstLogin::Handle( void )
{
	tSock->AcctNo( AB_INVALID_ID );
	LoginDenyReason t = LDR_NODENY;
	ACCOUNTSBLOCK actbTemp;

	if( Accounts->GetAccountByName( (char *)&tSock->Buffer()[1], actbTemp ) )
		tSock->SetAccount( actbTemp );

	char *username = (char *)&tSock->Buffer()[1];
	char pass1[256], pass2[256];
	memset( pass1, 0, 256 );
	memset( pass2, 0, 256 );
	pSplit( (char *)&tSock->Buffer()[31], pass1, pass2 );

	// Need auto account creation code here
	if( tSock->AcctNo() == AB_INVALID_ID )
	{
		if( cwmWorldState->ServerData()->InternalAccountStatus() )
		{
			Accounts->AddAccount( username, pass1, "" );
			if( Accounts->GetAccountByName( username, actbTemp ) )	// grab our account number
				tSock->SetAccount( actbTemp );
		}
	}
	if( tSock->AcctNo() != AB_INVALID_ID )
	{
		if( actbTemp.wFlags&AB_FLAGS_BANNED )
			t = LDR_ACCOUNTDISABLED;
		else if( strcmp( actbTemp.sPassword.c_str(), pass1 ) )
			t = LDR_BADPASSWORD;
	}
	else
		t = LDR_UNKNOWNUSER;
	if( t == LDR_NODENY && ( actbTemp.wFlags&AB_FLAGS_ONLINE ) )
		t = LDR_ACCOUNTINUSE;
	if( t != LDR_NODENY )
	{
		CPLoginDeny pckDeny( t );
		tSock->Send( &pckDeny );
		tSock->AcctNo( AB_INVALID_ID );
		Network->LoginDisconnect( tSock );
		return true;
	}
	if( tSock->AcctNo() != AB_INVALID_ID )
	{
		char temp[1024];
		actbTemp.dwLastIP = calcserial( tSock->ClientIP4(), tSock->ClientIP3(), tSock->ClientIP2(), tSock->ClientIP1() );
		sprintf( temp, "Client [%i.%i.%i.%i] connected using Account '%s'.", tSock->ClientIP4(), tSock->ClientIP3(), tSock->ClientIP2(), tSock->ClientIP1(), username );
		Console.Log( temp, "server.log" );
		messageLoop << temp;

		actbTemp.wFlags |= AB_FLAGS_ONLINE;

		UI16 servcount = cwmWorldState->ServerData()->ServerCount();
		CPGameServerList toSend( servcount );
		for( UI16 i = 0; i < servcount; ++i )
		{
			physicalServer *sData = cwmWorldState->ServerData()->ServerEntry( i );
			toSend.AddServer( i, sData );
		}
		tSock->Send( &toSend );
	}

	return true;
}

CPIFirstLogin::CPIFirstLogin()
{
	InternalReset();
}
CPIFirstLogin::CPIFirstLogin( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
	Receive();
}
void CPIFirstLogin::Receive( void )
{
	tSock->FirstPacket( false );
	tSock->Receive( 62, false );
	memcpy( &internalBuffer[0], tSock->Buffer(), 62 );
}
void CPIFirstLogin::InternalReset( void )
{
	internalBuffer.resize( 62 );
	internalBuffer[0] = 0x80;
}

const char *CPIFirstLogin::Name( void )
{
	return (const char *)&internalBuffer[1];
}
const char *CPIFirstLogin::Pass( void )
{
	return (const char *)&internalBuffer[31];
}
UI08 CPIFirstLogin::Unknown( void )
{
	return internalBuffer[61];
}

//	0xA0 Packet
//	Last Modified on Monday, 13-Apr-1998 17:06:46 EDT 
//	Select Server (3 bytes) 
//	BYTE cmd 
//	BYTE[2] server # chosen 
//	0x80 Packet
void CPIServerSelect::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[RECV]Packet   : CPIServerSelect 0xA0 --> Length: 03\n" );
	fprintf( fp, "Server         : %i\n", ServerNum() );
	fprintf( fp, "  Raw dump     :\n" );
	cPInputBuffer::Log( fp, false );
}

void CPIServerSelect::InternalReset( void )
{
	internalBuffer.resize( 3 );
	internalBuffer[0] = 0xA0;
}
CPIServerSelect::CPIServerSelect()
{
	InternalReset();
}
CPIServerSelect::CPIServerSelect( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
	Receive();
}
void CPIServerSelect::Receive( void )
{
	tSock->Receive( 3, false );
	memcpy( &internalBuffer[0], tSock->Buffer(), 3 );
}
SI16 CPIServerSelect::ServerNum( void )
{
	// EviLDeD - Sept 19, 2002
	// Someone said that there was an issue with False logins that request server 0. Default to server 1.
	SI16 temp = ( (internalBuffer[1]<<8) + internalBuffer[2] );
	if( temp == 0 )
		return 1;
	else
		return temp;
}

bool CPIServerSelect::Handle( void )
{
	UI16 selectedServer		= ServerNum() - 1;
	physicalServer *tServer = cwmWorldState->ServerData()->ServerEntry( selectedServer );
	UI32 ip = htonl( inet_addr( tServer->getIP().c_str() ) );
	CPRelay toSend( ip, tServer->getPort() );
	tSock->Send( &toSend );
	// Mark packet as sent. No we need to change how we network.
	tSock->GetAccount().bPackPacket=true;
	return true;
}

//	0x91 Packet
//	Last Modified on Monday, 13-Apr-1998 17:06:45 EDT 
//	Game Server Login (65 bytes) 
//	BYTE cmd 
//	BYTE[4] key used 
//	BYTE[30] sid 
//	BYTE[30] password 
void CPISecondLogin::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[RECV]Packet   : CPISecondLogin 0x80 --> Length: 65\n" );
	fprintf( fp, "Key Used       : %i\n", Account() );
	fprintf( fp, "SID            : %s\n", Name() );
	fprintf( fp, "Password       : %s\n", Pass() );
	fprintf( fp, "  Raw dump     :\n" );
	cPInputBuffer::Log( fp, false );
}
void CPISecondLogin::InternalReset( void )
{
	internalBuffer.resize( 65 );
	internalBuffer[0] = 0x91;
}
CPISecondLogin::CPISecondLogin()
{
	InternalReset();
}
CPISecondLogin::CPISecondLogin( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
	Receive();
}
void CPISecondLogin::Receive( void )
{
	tSock->FirstPacket( false );
	tSock->Receive( 65, false );
	tSock->CryptClient( true );
	memcpy( &internalBuffer[0], tSock->Buffer(), 65 );
}
long CPISecondLogin::Account( void )
{
	return DWord( 1 );
}
const char *CPISecondLogin::Name( void )
{
	return (const char *)&internalBuffer[5];
}
const char *CPISecondLogin::Pass( void )
{
	return (const char *)&internalBuffer[35];
}

bool CPISecondLogin::Handle( void )
{
	LoginDenyReason t = LDR_NODENY;
	tSock->AcctNo( AB_INVALID_ID );
	ACCOUNTSBLOCK actbTemp;
	if( Accounts->GetAccountByName( Name(), actbTemp ) )
		tSock->SetAccount( actbTemp );

	char pass1[256], pass2[256];
	memset( pass1, 0, 256 );
	memset( pass2, 0, 256 );
	pSplit( Pass(), pass1, pass2 );

	if( tSock->AcctNo() != AB_INVALID_ID )
	{
		if( actbTemp.wFlags&AB_FLAGS_BANNED )
			t = LDR_ACCOUNTDISABLED;
		else if( strcmp( pass1, actbTemp.sPassword.c_str() ) )
			t = LDR_UNKNOWNUSER;
	}
	else
		t = LDR_UNKNOWNUSER;
	if( t != LDR_NODENY )
	{
		tSock->AcctNo( AB_INVALID_ID );
		CPLoginDeny pckDeny( t );
		tSock->Send( &pckDeny );
		Network->LoginDisconnect( tSock );
		return true;
	}
	else
	{
		UI08 charCount = 0;
		for( UI32 i = 0; i < 5; ++i )
		{
			if( actbTemp.dwCharacters[i] != INVALIDSERIAL )
				++charCount;
		}
		CServerData *sd		= cwmWorldState->ServerData();
		size_t serverCount	= sd->NumServerLocations();
		CPCharAndStartLoc toSend( actbTemp, charCount, static_cast<UI08>(serverCount) );
		for( size_t j = 0; j < serverCount; ++j )
		{
			toSend.AddStartLocation( sd->ServerLocation( j ), static_cast<UI08>(j) );
		}
		tSock->Send( &toSend );
	}
	return true;
}

void CPGumpTextEntry::InternalReset( void )
{
	BlockSize( 3 );
	internalBuffer[0] = 0xAB;
}
CPGumpTextEntry::CPGumpTextEntry()
{
	InternalReset();
}
CPGumpTextEntry::CPGumpTextEntry( const char *text )
{
	InternalReset();
	Text1( text );
}
CPGumpTextEntry::CPGumpTextEntry( const char *text1, const char *text2 )
{
	InternalReset();
	Text1( text1 );
	Text2( text2 );
}
void CPGumpTextEntry::Serial( SERIAL id )
{
	internalBuffer[3] = (UI08)(id>>24);
	internalBuffer[4] = (UI08)(id>>16);
	internalBuffer[5] = (UI08)(id>>8);
	internalBuffer[6] = (UI08)(id%256);
}
void CPGumpTextEntry::ParentID( UI08 newVal )
{
	internalBuffer[7] = newVal;
}
void CPGumpTextEntry::ButtonID( UI08 newVal )
{
	internalBuffer[8] = newVal;
}
void CPGumpTextEntry::Cancel( UI08 newVal )
{
	SI16 t1Len = Text1Len();
	internalBuffer[t1Len + 11] = newVal;
}
void CPGumpTextEntry::Style( UI08 newVal )
{
	SI16 t1Len = Text1Len();
	internalBuffer[t1Len + 12] = newVal;
}
void CPGumpTextEntry::Format( SERIAL id )
{
	SI16 t1Len = Text1Len();
	internalBuffer[t1Len + 13] = (UI08)(id>>24);
	internalBuffer[t1Len + 14] = (UI08)(id>>16);
	internalBuffer[t1Len + 15] = (UI08)(id>>8);
	internalBuffer[t1Len + 16] = (UI08)(id%256);
}
void CPGumpTextEntry::Text1( const char *txt )
{
	size_t sLen = strlen( txt );
	BlockSize( 20 + sLen );	// 11 + 1 + 8
	Text1Len( sLen + 1 );
	strcpy( (char *)&internalBuffer[11], txt );
}
void CPGumpTextEntry::Text2( const char *txt )
{
	size_t sLen			= strlen( txt );
	SI16 currentSize	= CurrentSize();
	BlockSize( static_cast< UI16 >(currentSize + sLen + 1) );
	Text2Len( static_cast< SI16 >(sLen + 1) );
	strcpy( (char *)&internalBuffer[currentSize], txt );
}
void CPGumpTextEntry::BlockSize( SI16 newVal )
{
	internalBuffer.resize( newVal );
	internalBuffer[0] = 0xAB;
	internalBuffer[1] = (UI08)(newVal>>8);
	internalBuffer[2] = (UI08)(newVal%256);
}

SI16 CPGumpTextEntry::CurrentSize( void )
{
	return (SI16)((internalBuffer[1]<<8) + internalBuffer[2]);
}

SI16 CPGumpTextEntry::Text1Len( void )
{
	return (SI16)((internalBuffer[9]<<8) + internalBuffer[10]);
}

void CPGumpTextEntry::Text1Len( SI16 newVal )
{
	internalBuffer[9] = (UI08)(newVal>>8);
	internalBuffer[10] = (UI08)(newVal%256);
}

SI16 CPGumpTextEntry::Text2Len( void )
{
	SI16 t1Len = Text1Len();
	return (SI16)(( internalBuffer[t1Len + 17]<<8 ) + internalBuffer[t1Len + 18]);
}

void CPGumpTextEntry::Text2Len( SI16 newVal )
{
	SI16 t1Len = Text1Len();
	internalBuffer[t1Len + 17] = (UI08)(newVal>>8);
	internalBuffer[t1Len + 18] = (UI08)(newVal%256);
}

CPGodModeToggle::CPGodModeToggle()
{
	InternalReset();
}
void CPGodModeToggle::InternalReset( void )
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x2B;
}

void CPGodModeToggle::CopyData( cSocket *s )
{
	internalBuffer[1] = s->GetByte( 1 );
}

CPGodModeToggle::CPGodModeToggle( cSocket *s )
{
	InternalReset();
	CopyData( s );
}

CPGodModeToggle& CPGodModeToggle::operator=( cSocket *s )
{
	CopyData( s );
	return (*this);
}

void CPGodModeToggle::ToggleStatus( bool toSet )
{
	if( toSet )
		internalBuffer[1] = 1;
	else
		internalBuffer[1] = 0;
}

void CPLoginDeny::InternalReset( void )
{
	internalBuffer.resize( 2 );
	internalBuffer[0] = 0x82;
}
CPLoginDeny::CPLoginDeny()
{
	InternalReset();
}
CPLoginDeny::CPLoginDeny( LoginDenyReason reason )
{
	InternalReset();
	DenyReason( reason );
}
void CPLoginDeny::DenyReason( LoginDenyReason reason )
{
	internalBuffer[1] = reason;
}

//	0xBD Packet
//	Last Modified on Wednesday, 28-Aug-2002
//	Client Version Message (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] len
//	If (client-version of packet) 
//		BYTE[len-3] string stating the client version (0 terminated)  (like: 1.26.4)
//	Note: Client + Server Message
//	Client version : client sends its version string (e.g 3.0.8j)
//	Server version : 0xbd 0x0 0x3 (client replies with client version of this packet)
//	Note2:  clients sends a client version of this packet ONCE at login (without server request.)
void CPIClientVersion::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[RECV]Packet   : CPIClientVersion 0xBD --> Length: %i\n", tSock->GetWord( 1 ) );
	fprintf( fp, "Version        : %s\n", Offset() );
	fprintf( fp, "  Raw dump     :\n" );
	cPInputBuffer::Log( fp, false );
}
void CPIClientVersion::InternalReset( void )
{
	internalBuffer.resize( 3 );
	internalBuffer[0] = 0xBD;
	len = 0;
}
CPIClientVersion::CPIClientVersion()
{
	InternalReset();
}
CPIClientVersion::CPIClientVersion( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
	Receive();
}
void CPIClientVersion::Receive( void )
{
	tSock->Receive( 3, false );
	len = tSock->GetWord( 1 );
	tSock->Receive( len, false );
}

//o---------------------------------------------------------------------------o
//|   Function    -	UI08 ShiftValue( UI08 toShift, UI08 base, UI08 upper, bool extra)
//|   Date        -	21st November, 2001
//|   Programmer  -	Abaddon
//o---------------------------------------------------------------------------o
//|   Purpose     -	If the value is between base and upper (inclusive)
//|					it shifts the data down by base (if extra) or base - 1 (if not extra)
//o---------------------------------------------------------------------------o
UI08 ShiftValue( UI08 toShift, UI08 base, UI08 upper, bool extra )
{
	if( extra )
	{
		if( toShift >= base && toShift <= upper )
			toShift -= base;
	}
	else
	{
		if( toShift >= base && toShift <= upper )
			toShift = toShift - base + 1;
	}
	return toShift;
}
char *CPIClientVersion::Offset( void )
{
	return (char *)&(tSock->Buffer()[3]);
}
bool CPIClientVersion::Handle( void )
{
	char *verString	= Offset();
	verString[len]	= 0;
	UI08 major, minor, sub, letter;

	std::string s( verString );
	std::istringstream ss( s );
	char period;
	ss >> major >> period;
	ss >> minor >> period;
	ss >> sub >> letter;

	major	= ShiftValue( major,  '0', '9', true );
	minor	= ShiftValue( minor,  '0', '9', true );
	sub		= ShiftValue( sub,    '0', '9', true );
	letter	= ShiftValue( letter, 'a', 'z', false );
	letter	= ShiftValue( letter, 'A', 'Z', false );

	tSock->ClientVersion( major, minor, sub, letter );
	if( strstr( verString, "Dawn" ) )
		tSock->ClientType( CV_UO3D );
	else if( strstr( verString, "Krrios" ) )
		tSock->ClientType( CV_KRRIOS );
	tSock->ReceivedVersion( true );
	return true;
}

//	0xC8 Packet
//	Last Modified on Monday, 4th-Sep-2002  
//	Client view range (2 Bytes)
//	BYTE cmd 
//	BYTE range 
//	Note: client+server packet.
//	Range: how far away client wants to see (=get send) items/npcs
//	Update: since client 3.0.8o it actually got activated.(in a useful way)
//	When increase/decrease macro send, client sends a 0xc8.
//	If and only if server *relays* the packet (sending back the same data) range stuff gets activated client side.
//	"Greying" has no packet, purely client internal.
//	Minimal value:5, maximal: 18
void CPIUpdateRangeChange::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[RECV]Packet   : CPIUpdateRangeChange 0xC8 --> Length: %i\n", internalBuffer.size() );
	fprintf( fp, "Range			 : %i\n", internalBuffer[1] );
	fprintf( fp, "  Raw dump     :\n" );
	cPInputBuffer::Log( fp, false );
}
void CPIUpdateRangeChange::InternalReset( void )
{
	internalBuffer.resize( 2 );
}
CPIUpdateRangeChange::CPIUpdateRangeChange()
{
	InternalReset();
}
CPIUpdateRangeChange::CPIUpdateRangeChange( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
	Receive();
}
void CPIUpdateRangeChange::Receive( void )
{
	tSock->Receive( 2, false );
	internalBuffer[0] = tSock->GetByte( 0 );
	internalBuffer[1] = tSock->GetByte( 1 );
}
bool CPIUpdateRangeChange::Handle( void )
{
	// Byte 0 == 0xC8
	// Byte 1 == 0x0F (by default)  This is the distance, we believe

	// SURPRISE! This value, if used by the server, WILL NOT SHOW THE ITEMS IN A REAL CLIENT!
	// Krrios' client behaves (go Krrios!), but the UO ones don't
	switch( tSock->ClientType() )
	{
		case CV_NORMAL:
		case CV_T2A:
		case CV_UO3D:
			tSock->Range( tSock->GetByte( 1 ) - 3 );
			break;
		default:
			tSock->Range( tSock->GetByte( 1 ) );
			break;
	}
	tSock->Send( &(internalBuffer[0]), 2 );	// we want to echo it back to the client
#if defined( _MSC_VER )
#pragma note( "Flush location" )
#endif
	tSock->FlushBuffer();
	return true;
}

//	Subcommand 8: Set cursor hue / Set MAP 
//	BYTE hue  (0 = Felucca, unhued / BRITANNIA map.  1 = Trammel, hued gold / BRITANNIA map, 2 = (switch to) ILSHENAR map)
//	Note: Server Message

void CPMapChange::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[SEND]Packet   : CPMapChange 0xBF Subcommand 8 --> Length: %i\n", internalBuffer.size() );
	fprintf( fp, "Hue            : %i", internalBuffer[5] );
	switch( internalBuffer[5] )
	{
		case 0:		fprintf( fp, " (Felucca)\n" );	break;
		case 1:		fprintf( fp, " (Trammel)\n" );	break;
		case 2:		fprintf( fp, " (Ilshenar)\n" );	break;
		case 3:		fprintf( fp, " (Malas)\n" );	break;
		default:	fprintf( fp, " (Unknown)\n" );	break;
	}
	fprintf( fp, "  Raw dump     :\n" );
	cPUOXBuffer::Log( fp, false );
}
void CPMapChange::InternalReset( void )
{
	internalBuffer.resize( 6 );
	internalBuffer[0] = 0xBF;
	internalBuffer[1] = 0x00;
	internalBuffer[2] = 0x06;
	internalBuffer[3] = 0x00;
	internalBuffer[4] = 0x08;	// set client hue
}
CPMapChange::CPMapChange()
{
	InternalReset();
}
CPMapChange::CPMapChange( UI08 newMap )
{
	InternalReset();
	SetMap( newMap );
}
CPMapChange::CPMapChange( cBaseObject *moving )
{
	InternalReset();
	if( moving != NULL )
		SetMap( moving->WorldNumber() );
}
CPMapChange::~CPMapChange()
{
}
void CPMapChange::SetMap( UI08 newMap )
{
	internalBuffer[5] = newMap;
}
CPMapChange& CPMapChange::operator=( cBaseObject& moving )
{
	SetMap( moving.WorldNumber() );
	return (*this);
}

void CPItemsInContainer::InternalReset( void )
{
	internalBuffer.resize( 5 );
	internalBuffer[0]	= 0x3C;
	isVendor			= false;
	vendorSerial		= INVALIDSERIAL;
}
CPItemsInContainer::CPItemsInContainer()
{
	InternalReset();
}
CPItemsInContainer::CPItemsInContainer( CItem *container, CChar *vendor )
{
	InternalReset();
	if( vendor != NULL )
	{
		isVendor		= true;
		vendorSerial	= vendor->GetSerial() | BASEITEMSERIAL;
	}
	if( container != NULL )
		CopyData( (*container ) );
}
CPItemsInContainer::CPItemsInContainer( CItem *container )
{
	InternalReset();
	if( container != NULL )
		CopyData( (*container) );
}

UI16 CPItemsInContainer::NumberOfItems( void ) const
{
	return (UI16)( (internalBuffer[3]<<8) + internalBuffer[4] );
}

void CPItemsInContainer::NumberOfItems( UI16 numItems )
{
	// set the number of items
	internalBuffer[3]	= (UI08)(numItems>>8);
	internalBuffer[4]	= (UI08)(numItems%256);
	// knowing the number of items, set the packet size
	UI16 packetSize		= (UI16)((numItems * 19) + 5);
	internalBuffer.resize( packetSize );
	internalBuffer[1]	= (UI08)(packetSize>>8);
	internalBuffer[2]	= (UI08)(packetSize%256);
}
void CPItemsInContainer::AddItem( CItem *toAdd, UI16 itemNum )
{
	UI16 baseOffset = (UI16)(5 + itemNum * 19);
	PackLong(  &internalBuffer[0], baseOffset +  0, toAdd->GetSerial() );
	PackShort( &internalBuffer[0], baseOffset +  4, toAdd->GetID() );
	PackShort( &internalBuffer[0], baseOffset +  7, toAdd->GetAmount() );
	PackShort( &internalBuffer[0], baseOffset +  9, toAdd->GetX() );
	PackShort( &internalBuffer[0], baseOffset + 11, toAdd->GetY() );

	if( isVendor )
		PackLong( &internalBuffer[0], baseOffset + 13, vendorSerial );
	else
		PackLong( &internalBuffer[0], baseOffset + 13, toAdd->GetContSerial() );

	PackShort( &internalBuffer[0], baseOffset + 17, toAdd->GetColour() );

	toAdd->SetDecayTime( 0 );
}

void CPItemsInContainer::Add( UI16 itemNum, SERIAL toAdd, SERIAL cont, UI08 amount )
{
	UI16 baseOffset = (UI16)(5 + itemNum * 19);

	PackLong(  &internalBuffer[0], baseOffset,		toAdd );
	PackShort( &internalBuffer[0], baseOffset + 7,	amount );
	PackLong(  &internalBuffer[0], baseOffset + 13, cont );
}
CPItemsInContainer& CPItemsInContainer::operator=( CItem& container )
{
	CopyData( container );
	return (*this);
}

void CPItemsInContainer::CopyData( CItem& toCopy )
{
	UI16 itemCount	= 0;
	bool isCorpse	= toCopy.isCorpse();

	for( CItem *ctr = toCopy.FirstItem(); !toCopy.FinishedItems(); ctr = toCopy.NextItem() )
	{
		if( ctr != NULL && ( !isCorpse || ( isCorpse && ctr->GetLayer() != 0 ) ) )
		{
			if( !ctr->isFree() )
			{
				internalBuffer.resize( internalBuffer.size() + 19 );
				AddItem( ctr, itemCount );
				++itemCount;
			}
		}
	}

	NumberOfItems( itemCount );
}


//0x74 Packet
//Last Modified on Sunday, 03-May-1998 22:52:07 EDT 
//Open Buy Window (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[4] (vendorID | 0x40000000) 
//	BYTE # of items 
//	# of items worth of item segments 
//		BYTE[4] price 
//		BYTE length of text description 
//		BYTE[text length] item description 
//NOTE: This packet is always preceded by a describe contents packet (0x3c) with the container id as the (vendorID | 0x40000000) and then an open container packet (0x24?) with the vendorID only and a model number of 0x0030 (probably the model # for the buy screen)
void CPOpenBuyWindow::InternalReset( void )
{
	internalBuffer.resize( 16384 );	// start big, and work back down
	internalBuffer[0] = 0x74;
	internalBuffer[1] = 0;
	internalBuffer[2] = 0;
	internalBuffer[3] = 0;
	internalBuffer[4] = 0;
	internalBuffer[5] = 0;
	internalBuffer[6] = 0;
	internalBuffer[7] = 0;
}
CPOpenBuyWindow::CPOpenBuyWindow()
{
	InternalReset();
}
CPOpenBuyWindow::CPOpenBuyWindow( CItem *container, CChar *p )
{
	InternalReset();
	if( p != NULL )
		VendorID( p->GetSerial() | BASEITEMSERIAL );
	else
		VendorID( INVALIDSERIAL );
	if( container != NULL )
		CopyData( (*container), p );
}

UI08 CPOpenBuyWindow::NumberOfItems( void ) const
{
	return internalBuffer[7];
}

void CPOpenBuyWindow::NumberOfItems( UI08 numItems )
{
	// set the number of items
	internalBuffer[7] = numItems;
}
void CPOpenBuyWindow::VendorID( SERIAL ser )
{
	PackLong( &internalBuffer[0], 3, ser );
}
SI16 CPOpenBuyWindow::AddItem( CItem *toAdd, CChar *p, UI16 baseOffset )
{
	UI32 value = calcValue( toAdd, (UI32)toAdd->GetBuyValue() );
	if( cwmWorldState->ServerData()->TradeSystemStatus() )
		value = calcGoodValue( p, toAdd, value, false );
	std::string itemname;
	itemname.reserve( 30 );
	size_t sLen = getTileName( toAdd, itemname ); // Item name length, don't strip the NULL (3D client doesn't like it)

	internalBuffer.resize( baseOffset + 5 + sLen );
	PackLong( &internalBuffer[0], baseOffset + 0, value );
	internalBuffer[baseOffset +  4] = static_cast< UI08 >(sLen);

	for( UI08 k = 0; k < sLen; ++k )
		internalBuffer[baseOffset + 5 + k] = itemname[k];
	return (SI16)(5 + sLen);
}
void CPOpenBuyWindow::Container( CItem *toAdd, CChar *p )
{
	if( p != NULL )
		VendorID( p->GetSerial() );
	else
		VendorID( INVALIDSERIAL );
	CopyData( (*toAdd), p );
}

void CPOpenBuyWindow::CopyData( CItem& toCopy, CChar *p )
{
	UI08 itemCount	= 0;
	UI16 length		= 8;
	for( CItem *ctr = toCopy.FirstItem(); !toCopy.FinishedItems(); ctr = toCopy.NextItem() )
	{
		if( ctr != NULL )
		{
			if( !ctr->isFree() )
			{
				length += AddItem( ctr, p, length );
				++itemCount;
			}
		}
	}
	NumberOfItems( itemCount );
	internalBuffer.resize( length );
	PackShort( &internalBuffer[0], 1, length );
}

void CPOpenBuyWindow::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
		fprintf( fp, "[SEND]Packet   : CPOpenBuyWindow 0x74 --> Length: %li\n", internalBuffer.size() );
	fprintf( fp, "Block size     : %i\n", UnpackUShort( &internalBuffer[0], 1 ) );
	fprintf( fp, "Vendor ID      : %x\n", UnpackULong( &internalBuffer[0], 3 ) );
	fprintf( fp, "Number of Items: %i\n", internalBuffer[7] );
	int baseOffset = 8;
	for( UI32 x = 0; x < internalBuffer[7]; ++x )
	{
		fprintf( fp, "  ITEM %i\n      Price: %i\n", x, UnpackULong( &internalBuffer[0], baseOffset ) );
		baseOffset += 4;
		fprintf( fp, "      Len  : %i\n", internalBuffer[baseOffset] );
		fprintf( fp, "      Name : " );
		for( UI32 y = 0; y < internalBuffer[baseOffset]; ++y )
			fprintf( fp, "%c", internalBuffer[baseOffset + 1 + y] );
		baseOffset += internalBuffer[baseOffset] + 1;
		fprintf( fp, "\n" );
	}

	fprintf( fp, "  Raw dump       :\n" );

	cPUOXBuffer::Log( fp, false );
}

//0xA9 Packet
//Last Modified on Sunday, 28-Jul-2002
//Characters / Starting Locations (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE # of characters 
//	Following repeated 5 times 
//		BYTE[30] character name 
//		BYTE[30] character password 
//	BYTE number of starting locations 
//	Following for as many locations as you have 
//		BYTE locationIndex (0-based) 
//		BYTE[31] town (general name) 
//		BYTE[31] exact name 
//	BYTE Flags 
//		0x02 = send config/req logout (IGR?)
//		0x04 = single character (siege)
//		0x08 = enable npcpopup menus
//		0x10 = unknown, 
//		0x20 = enable common AOS features (tooltip thing/fight system book, but not AOS monsters/map/skills)
void CPCharAndStartLoc::Log( FILE *fp, bool fullHeader )
{
	if( fp == NULL )
		return;
	if( fullHeader )
	fprintf( fp, "[SEND]Packet   : CPCharAndStartLoc 0xA9 --> Length: %i\n", internalBuffer.size() );
	fprintf( fp, "# Chars		 : %i\n", internalBuffer[3] );
	fprintf( fp, "Characters --\n" );
	for( UI32 i = 0; i < 5; ++i )
	{
		UI32 baseOffset = 4 + i * 60;
		fprintf( fp, "    Character %i\n", i );
		fprintf( fp, "      Name: " );
		for( UI32 j = 0; j < 30; ++j )
		{
			if( internalBuffer[baseOffset+j] != 0 )
				fprintf( fp, "%c", internalBuffer[baseOffset+j] );
			else
				break;
		}
		fprintf( fp, "\n      Pass: " );
		for( UI32 k = 0; k < 30; ++k )
		{
			if( internalBuffer[baseOffset+k+30] != 0 )
				fprintf( fp, "%c", internalBuffer[baseOffset+k+30] );
			else
				break;
		}
		fprintf( fp, "\n" );
	}
	fprintf( fp, "# Starts       : %i\n", internalBuffer[304] );
	fprintf( fp, "Starting locations --\n" );
	for( UI32 l = 0; l < internalBuffer[304]; ++l )
	{
		UI32 baseOffset = 305 + l * 63;
		fprintf( fp, "    Start %i\n", l );
		fprintf( fp, "      Index : %i\n", internalBuffer[baseOffset] );
		fprintf( fp, "      General Name: " );
		++baseOffset;
		for( UI32 m = 0; m < 31; ++m )
		{
			if( internalBuffer[baseOffset+m] != 0 )
				fprintf( fp, "%c", internalBuffer[baseOffset+m] );
			else
				break;
		}
		fprintf( fp, "\n      Exact Name  : " );
		baseOffset += 31;
		for( UI32 n = 0; n < 31; ++n )
		{
			if( internalBuffer[baseOffset+n] != 0 )
				fprintf( fp, "%c", internalBuffer[baseOffset+n] );
			else
				break;
		}
		fprintf( fp, "\n" );
	}
	UI32 lastByte = internalBuffer[internalBuffer.size() - 1];
	fprintf( fp, "Flags  		 : %i\n", lastByte );
	if( (lastByte&0x02) == 0x02 )
		fprintf( fp, "       		 : Send config/request logout\n" );
	if( (lastByte&0x04) == 0x04 )
		fprintf( fp, "       		 : Single character\n" );
	if( (lastByte&0x08) == 0x08 )
		fprintf( fp, "       		 : Enable NPC Popup menus\n" );
	if( (lastByte&0x10) == 0x10 )
		fprintf( fp, "       		 : Unknown\n" );
	if( (lastByte&0x20) == 0x20 )
		fprintf( fp, "       		 : Enable Common AoS features\n" );
	fprintf( fp, "  Raw dump     :\n" );
	cPUOXBuffer::Log( fp, false );
}
void CPCharAndStartLoc::InternalReset( void )
{
	internalBuffer.resize( 4 );
	internalBuffer[0] = 0xA9;
}
void CPCharAndStartLoc::CopyData(ACCOUNTSBLOCK& toCopy )
{
	for( UI08 i = 0; i < 5; ++i )
	{
		if( toCopy.lpCharacters[i] != NULL )
			AddCharacter( toCopy.lpCharacters[i], i );
	}
}

CPCharAndStartLoc::CPCharAndStartLoc()
{
	InternalReset();
}
CPCharAndStartLoc::CPCharAndStartLoc(ACCOUNTSBLOCK& actbBlock, UI08 numCharacters, UI08 numLocations )
{
	InternalReset();
	NumberOfCharacters( numCharacters );
	NumberOfLocations( numLocations );
	CopyData(actbBlock);
}
void CPCharAndStartLoc::NumberOfLocations( UI08 numLocations )
{
	// was 305 +, now 309 +
	UI16 packetSize = (UI16)(309 + 63 * numLocations);
	internalBuffer.resize( packetSize );
	internalBuffer[1] = (UI08)(packetSize>>8);
	internalBuffer[2] = (UI08)(packetSize%256);
	internalBuffer[304] = numLocations;
	// turn on /*send config,*/ npcpopup menus and common AOS features
//	internalBuffer[packetSize - 1] = ( 0x20 | 0x08 /*| 0x02*/ );
	internalBuffer[packetSize - 1] = 0x08;
}
void CPCharAndStartLoc::NumberOfCharacters( UI08 numCharacters )
{
	internalBuffer[3] = numCharacters;
}
void CPCharAndStartLoc::AddCharacter( CChar *toAdd, UI08 charOffset )
{
	if( toAdd == NULL )
		return;
	UI16 baseOffset = (UI16)(4 + charOffset * 60);
	strncpy( (char *)&internalBuffer[baseOffset], toAdd->GetName().c_str(), 29 );
	// extra 30 bytes unused, as we don't use a character password
}
void CPCharAndStartLoc::AddStartLocation( LPSTARTLOCATION sLoc, UI08 locOffset )
{
	if( sLoc == NULL )
		return;
	UI16 baseOffset				= (UI16)(305 + locOffset * 63);
	internalBuffer[baseOffset]	= locOffset;
	size_t townLen				= strlen( sLoc->town );
	size_t descLen				= strlen( sLoc->description );
	for( UI32 i = 0; i < townLen; ++i )
		internalBuffer[baseOffset+i+1] = sLoc->town[i];
	for( UI32 j = 0; j < descLen; ++j )
		internalBuffer[baseOffset+j+31] = sLoc->town[j];
}

CPCharAndStartLoc& CPCharAndStartLoc::operator=(ACCOUNTSBLOCK& actbBlock )
{
	CopyData(actbBlock);
	return (*this);
}

//0xF0 Packet
//Last Modified on Tuesday, 26-Feb-2002  
//Custom client packet
//	BYTE cmd 
//	BYTE[2] len
//	BYTE subcmd 
//	BYTE[len - 4] data 

//Subcommand 0: Acknowledge login (1 byte (for 5 total))
//	BYTE ack  (0 = Rejected, unauthorized.  1 = Accepted, GM priviledges. 2 = Accepted, player priviledges)
//Note: Server Message
//Note: GM priviledges unlock movement rate configurations

CPKAccept::CPKAccept( UI08 Response )
{
	internalBuffer.resize( 5 );
	internalBuffer[0] = 0xF0;
	internalBuffer[1] = 0x00;
	internalBuffer[2] = 0x05; // Length... 5 bytes
	internalBuffer[3] = 0x00; // Client response

	// Ack
	internalBuffer[4] = Response;
}

//0xA7 Packet
//Last Modified on Wednesday, 06-May-1998 23:30:53 EDT 
//Request Tips/Notice (4 bytes) 
//	BYTE cmd 
//	BYTE[2] last tip # 
//	BYTE flag 
//		0x00 - tips window 
//		0x01 - notice window 
void CPITips::InternalReset( void )
{
}
CPITips::CPITips()
{
	InternalReset();
}
CPITips::CPITips( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
	Receive();
}
void CPITips::Receive( void )
{
	tSock->Receive( 4, false );
}
bool CPITips::Handle( void )
{
	ScriptSection *Tips = FileLookup->FindEntry( "TIPS", misc_def );
	if( Tips == NULL )
		return true;

	UI16 i = static_cast<UI16>(tSock->GetWord( 1 ) + 1);
	if( i == 0 ) 
		i = 1;

	int x = i;
	UString tag;
	UString data;
	char temp[1024];
	for( tag = Tips->First(); !Tips->AtEnd(); tag = Tips->Next() )
	{
		if( !tag.empty() && tag.upper() == "TIP" )
			--x;
		if( x <= 0 )
			break;
	}
	if( x > 0 )
		tag = Tips->Prev();
	data = Tips->GrabData();
	sprintf(temp, "TIP %i", data.toLong() );
	Tips = FileLookup->FindEntry( temp, misc_def );
	if( Tips == NULL )
		return true;
	x = -1;
	char tipData[2048];
	tipData[0] = 0;
	CPUpdScroll toSend( 0, (UI08)i );
	for( tag = Tips->First(); !Tips->AtEnd(); tag = Tips->Next() )
	{
		data = Tips->GrabData();
		sprintf( tipData, "%s%s %s ", tipData, tag.c_str(), data.c_str() );
	}

	std::string toAdd;
	toSend.AddString( tipData );
	toSend.Finalize();
	tSock->Send( &toSend );

	return true;
}

//0xA6 Packet
//Last Modified on Wednesday, 06-May-1998 23:30:49 EDT 
//Tips/Notice window (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE flag 
//		0x00 - tips window 
//		0x01 - notice window 
//	BYTE[2] unknown1 
//	BYTE[2] tip # 
//	BYTE[2] msgSize 
//	BYTE[?] message (? = blockSize - 10) 
//*Null terminated I think (Gimli)
void CPUpdScroll::InternalReset( void )
{
	internalBuffer.resize( 10 );	// 10, not 11
	internalBuffer[0] = 0xA6;
	internalBuffer[1] = 0x01;
	internalBuffer[2] = 0x02;
	internalBuffer[3] = 0x02;
	internalBuffer[4] = 0x00;
	internalBuffer[5] = 0x00;
	internalBuffer[6] = 0x00;
	internalBuffer[7] = 0x00;
	internalBuffer[8] = 0x01;
	internalBuffer[9] = 0x02;
	memset( tipData, 0, 2048 );
}
CPUpdScroll::CPUpdScroll()
{
	InternalReset();
}
CPUpdScroll::CPUpdScroll( UI08 tType )
{
	InternalReset();
	TipType( tType );
}
CPUpdScroll::CPUpdScroll( UI08 tType, UI08 tNum )
{
	InternalReset();
	TipType( tType );
	TipNumber( tNum );
}
CPUpdScroll::~CPUpdScroll()
{
}
void CPUpdScroll::AddString( const char *toAdd )
{
	strcat( tipData, toAdd );
}
void CPUpdScroll::AddStrings( const char *tag, const char *data )
{
	char temp[1024];
	sprintf( temp, "%s %s", tag, data );
	AddString( temp );
}
void CPUpdScroll::TipType( UI08 tType )
{
	internalBuffer[3] = tType;
}
void CPUpdScroll::TipNumber( UI08 tipNum )
{
	internalBuffer[7] = tipNum;
}
void CPUpdScroll::Finalize( void )
{
	size_t y = strlen( tipData ) + 10;
	SetLength( static_cast< UI16 >(y) );
	memcpy( &internalBuffer[10], tipData, y - 10 );
}

void CPUpdScroll::SetLength( UI16 len )
{
	internalBuffer.resize( len );
	internalBuffer[1] = (UI08)(len>>8);
	internalBuffer[2] = (UI08)(len%256);
	internalBuffer[8] = (UI08)((len - 10)>>8);
	internalBuffer[9] = (UI08)((len - 10)%256);
}

//
//0x75 Packet
//Last Modified on Sunday, 13-Feb-2000 
//Rename Character (35 bytes) 
//	BYTE cmd 
//	BYTE[4] id 
//	BYTE[30] new name 

void CPIRename::InternalReset( void )
{
}
CPIRename::CPIRename()
{
	InternalReset();
}
CPIRename::CPIRename( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
	Receive();
}
void CPIRename::Receive( void )
{
	tSock->Receive( 0x23, false );
}
bool CPIRename::Handle( void )
{
	SERIAL serial = tSock->GetDWord( 1 );
	if( serial == INVALIDSERIAL ) 
		return true;
	CChar *c = calcCharObjFromSer( serial );
	if( c != NULL )
		c->SetName( (char *)&tSock->Buffer()[5] );
	return true;
}

//0xC0 Packet
//Last Modified on Friday, 26-Jul-2002  
//(yet another) Graphical Effect (36 Bytes)
//	BYTE cmd 
//	BYTE type 
//	BYTE[4] sourceSerial 
//	BYTE[4] targetSerial 
//	BYTE[2] itemID 
//	BYTE[2] xSource 
//	BYTES[2] ySource 
//	BYTE zSource 
//	BYTE[2] xTarget 
//	BYTE[2] yTarget 
//	BYTE zTarget 
//	BYTE speed 
//	BYTE duration 
//	BYTE[2] unk // On OSI, flamestrikes are 0x0100 
//	BYTE fixedDirection 
//	BYTE explodes 
//	BYTE[4] hue 
//	BYTE[4] renderMode 
//Server message
void CPGraphicalEffect2::InternalReset( void )
{
	internalBuffer.resize( 36 );
	internalBuffer[0] = 0xC0;
	internalBuffer[24] = 0;
	internalBuffer[25] = 0;
}
CPGraphicalEffect2::CPGraphicalEffect2( UI08 effectType ) : CPGraphicalEffect( effectType )
{
	InternalReset();
}
CPGraphicalEffect2::CPGraphicalEffect2( UI08 effectType, cBaseObject &src, cBaseObject &trg ) : CPGraphicalEffect( effectType, src, trg )
{
	InternalReset();
	Effect( effectType );
	SourceSerial( src );
	TargetSerial( trg );
}
CPGraphicalEffect2::CPGraphicalEffect2( UI08 effectType, cBaseObject &src ) : CPGraphicalEffect( effectType, src )
{
	InternalReset();
	Effect( effectType );
	SourceSerial( src );
}
void CPGraphicalEffect2::Hue( UI32 hue )
{
	PackLong( &internalBuffer[0], 28, hue );
}
void CPGraphicalEffect2::RenderMode( UI32 mode )
{
	PackLong( &internalBuffer[0], 32, mode );
}

//0x73 Packet
//Last Modified on Friday, 20-Nov-1998 
//Ping message (2 bytes) 
//	BYTE cmd 
//	BYTE seq 
CPIKeepAlive::CPIKeepAlive()
{
}
CPIKeepAlive::CPIKeepAlive( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIKeepAlive::Receive( void )
{
	tSock->Receive( 2, false );
}
bool CPIKeepAlive::Handle( void )
{
	tSock->Send( tSock->Buffer(), 2 );
#if defined( _MSC_VER )
#pragma note( "Flush location" )
#endif
	tSock->FlushBuffer();
	return true;
}

//0x34 Packet
//Last modified on Thursday, 19-Nov-1998 
//Get Player Status (10 bytes) 
//	BYTE cmd 
//	BYTE[4] pattern (0xedededed) 
//	BYTE getType 
//		0x04 - Basic Stats (Packet 0x11 Response) 
//		0x05 = Request Skills (Packet 0x3A Response) 
//	BYTE[4] playerID 
CPIStatusRequest::CPIStatusRequest()
{
}
CPIStatusRequest::CPIStatusRequest( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIStatusRequest::Receive( void )
{
	tSock->Receive( 10, false );
}
bool CPIStatusRequest::Handle( void )
{
	if( tSock->GetByte( 5 ) == 4 )
		tSock->statwindow( calcCharObjFromSer( tSock->GetDWord( 6 ) ) );
	if( tSock->GetByte( 5 ) == 5 )
	{
		CPSkillsValues toSend;
//		if( tSock->ReceivedVersion() )
//		{
//			if( tSock->ClientVersionMajor() >= 4 && tSock->ClientVersionSub() >= 0 && tSock->ClientVersionLetter() >= 6 )
				toSend.NumSkills( ALLSKILLS );
//			else
//				toSend.NumSkills( ALLSKILLS_COMPAT );
			toSend.SetCharacter( *(tSock->CurrcharObj()) );
			tSock->Send( &toSend );
//		}
	}
	return true;
}

//0xA4 Packet
//Last Modified on Friday, 15-May-2000 
//Spy on Client (149 bytes) 
//	BYTE cmd 
//	BYTE[148] Unknown (previously, this has had info such as your graphics card name, free HD space, number of processors, etc.)
CPISpy::CPISpy()
{
}
CPISpy::CPISpy( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPISpy::Receive( void )
{
	tSock->Receive( 0x95, false );
}
bool CPISpy::Handle( void )
{
	return true;
}

//0x04 Packet
//Last Modified on Friday, 19-May-2000 
//Request (2 bytes) 
//	BYTE cmd 
//	BYTE mode (0=off, 1=on)  
//Note: Client Message
CPIGodModeToggle::CPIGodModeToggle()
{
}
CPIGodModeToggle::CPIGodModeToggle( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIGodModeToggle::Receive( void )
{
	tSock->Receive( 2, false );
}
bool CPIGodModeToggle::Handle( void )
{
	CChar *ourChar = tSock->CurrcharObj();
	if( ourChar->IsGM() )
	{
		CPGodModeToggle cpgmt = tSock;
		tSock->Send( &cpgmt );
	}
	else
	{
		tSock->sysmessage( 1641 );
		Console << "Godclient detected - Account[" << ourChar->GetAccount().wAccountIndex << "] Username[" << ourChar->GetAccount().sUsername << ". Client disconnected!" << myendl;
		Network->Disconnect( tSock );
	}
	return true;
}

//0x56 Packet
//Last Modified on Friday, 26-Jul-2002
//Map Related(11 bytes) 
//	BYTE cmd																				0
//	BYTE[4] id																				1
//	BYTE command																			5
//		1 = add map point, 
//		2 = add new pin with pin number. (insertion. other pins after the number are pushed back.) 
//		3 = change pin
//		4 = remove pin
//		5 = remove all pins on the map 
//		6 = toggle the 'editable' state of the map. 
//		7 = return msg from the server to the request 6 of the client. 
//	BYTE plotting state (1=on, 0=off, valid only if command 7)								6
//	BYTE[2] x location (relative to upper left corner of the map, in pixels, for points)	7
//	BYTE[2] y location (relative to upper left corner of the map, in pixels, for points)	9
CPMapRelated::CPMapRelated()
{
	internalBuffer.resize( 11 );
	internalBuffer[0] = 0x56;
}
void CPMapRelated::PlotState( UI08 pState )
{
	internalBuffer[6] = pState;
}
void CPMapRelated::Location( SI16 x, SI16 y )
{
	PackShort( &internalBuffer[0], 7, x );
	PackShort( &internalBuffer[0], 9, y );
}
void CPMapRelated::Command( UI08 cmd )
{
	internalBuffer[5] = cmd;
}
void CPMapRelated::ID( SERIAL key )
{
	PackLong( &internalBuffer[0], 1, key );
}

//0x06 Packet
//Last Modified on Friday, 19-May-2000 
//Double click (5 bytes) 
//	BYTE cmd 
//	BYTE[4] ID of double clicked object 
//Note: Client Message
CPIDblClick::CPIDblClick()
{
}
CPIDblClick::CPIDblClick( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIDblClick::Receive( void )
{
	tSock->Receive( 5, false );
}

// bool CPIDblClick::Handle() is implemented in cplayeraction.cpp


//0x09 Packet
//Last Modified on Friday, 19-May-2000 
//Single click (5 bytes) 
//	BYTE cmd 
//	BYTE[4] ID of single clicked object 
//Note: Client Message
CPISingleClick::CPISingleClick()
{
}
CPISingleClick::CPISingleClick( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPISingleClick::Receive( void )
{
	tSock->Receive( 5, false );
}
// bool CPISingleClick::Handle() is implemented in cplayeraction.cpp

//0x02 Packet
//Last Modified on Friday, 19-May-2000
//Move Request (7 bytes) 
//	BYTE cmd 
//	BYTE direction 
//	BYTE sequence number 
//	BYTE[4] fastwalk prevention key
//Note: Client Message
//Note: sequence number starts at 0, after a reset.  However, if 255 is reached, the next seq # is 1, not 0.
//Fastwalk prevention notes: each 0x02 pops the top element from fastwalk key stack. (0xbf sub1 init. fastwalk stack, 0xbf sub2 pushes an element to stack)
//If stack is empty key value is 0. ( never set keys to 0 in 0xbf sub 1/2)
//Because client sometimes sends bursts of 0x02s DONT check for a certain top stack value.
//The only safe way to detect fastwalk: push a key after EACH x021, 0x22, (=send 0xbf sub 2) check in 0x02 for stack emptyness.
//If empty -> fastwalk alert.
//Note that actual key values are irrelevant. (just dont use 0)
//Of course without perfect 0x02/0x21/0x22 synch (serverside) its useless to use fastwalk detection.
//Last but not least: fastwalk detection adds 9 bytes per step and player !
CPIMoveRequest::CPIMoveRequest()
{
}
CPIMoveRequest::CPIMoveRequest( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIMoveRequest::Receive( void )
{
	tSock->Receive( 7, false );
}
bool CPIMoveRequest::Handle( void )
{
	CChar *ourChar = tSock->CurrcharObj();
	Movement->Walking( ourChar, tSock->GetByte( 1 ), tSock->GetByte( 2 ) );
	ourChar->BreakConcentration( tSock );
	return true;
}

//0x22 Packet
//Last Modified on Wednesday, 11-Nov-1998 
//Character Move ACK/ Resync Request(3 bytes) 
//	BYTE cmd 
//	BYTE sequence (matches sent sequence) 
//	BYTE (0x00) 
CPIResyncReq::CPIResyncReq()
{
}
CPIResyncReq::CPIResyncReq( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIResyncReq::Receive( void )
{
	tSock->Receive( 3, false );
}
bool CPIResyncReq::Handle( void )
{
	return true;
}

//0x2C Packet
//Last Modified on Friday, 19-May-2000
//Resurrection Menu Choice (2 bytes) 
//	BYTE cmd 
//	BYTE action (2=ghost, 1=resurrect, 0=from server)
//Note: Client and Server Message
//Note: Resurrection menu has been removed from UO.
CPIResMenuChoice::CPIResMenuChoice()
{
}
CPIResMenuChoice::CPIResMenuChoice( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIResMenuChoice::Receive( void )
{
	tSock->Receive( 2, false );
}
bool CPIResMenuChoice::Handle( void )
{
	UI08 cmd = tSock->GetByte( 1 );
	if( cmd == 0x02 ) 
		tSock->sysmessage( 766 );
	if( cmd == 0x01 ) 
		tSock->sysmessage( 767 );
	return true;
}

//0x05 Packet
//Last Modified on Friday, 19-May-2000 
//Attack Request (5 bytes) 
//	BYTE cmd 
//	BYTE[4] ID to be attacked 
//Note: Client Message
CPIAttack::CPIAttack()
{
}
CPIAttack::CPIAttack( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIAttack::Receive( void )
{
	tSock->Receive( 5, false );
}
bool CPIAttack::Handle( void )
{
	CChar *ourChar = tSock->CurrcharObj();
	ourChar->BreakConcentration( tSock );
	Combat->PlayerAttack( tSock );
	return true;
}

//0x6C Packet
//Last Modified on Sunday, 13-Feb-2000 
//Targeting Cursor Commands (19 bytes) 
//	BYTE cmd 
//	BYTE type 
//		0x00 = Select Object
//		0x01 = Select X, Y, Z 
//	BYTE[4] cursorID 
//	BYTE Cursor Type 
//	Always 0 now  
//	The following are always sent but are only valid if sent by client 
//	BYTE[4] Clicked On ID 
//	BYTE[2] click xLoc 
//	BYTE[2] click yLoc 
//	BYTE unknown (0x00) 
//	BYTE click zLoc 
//	BYTE[2] model # (if a static tile, 0 if a map/landscape tile)
//Note: the model # shouldnt be trusted.
CPITargetCursor::CPITargetCursor()
{
}
CPITargetCursor::CPITargetCursor( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPITargetCursor::Receive( void )
{
	tSock->Receive( 19, false );
}
bool CPITargetCursor::Handle( void )
{
	CChar *ourChar = tSock->CurrcharObj();
	if( tSock->TargetOK() )
		Targ->MultiTarget( tSock );
	ourChar->BreakConcentration( tSock );
	return true;
}

//0x13 Packet
//Last Modified on Thursday, 19-Nov-1998 
//Drop->Wear Item (10 bytes) 
//	BYTE cmd 
//	BYTE[4] itemid 
//	BYTE layer (see layer list at top) 
//	BYTE[4] playerID 
//Note: The layer byte should not be trusted.  
CPIEquipItem::CPIEquipItem()
{
}
CPIEquipItem::CPIEquipItem( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIEquipItem::Receive( void )
{
	tSock->Receive( 10, false );
}

// bool CPIEquipItem::Handle() implemented in cplayeraction.cpp

//0x07 Packet
//Last Modified on Friday, 19-May-2000
//Pick Up Item(s) (7 bytes) 
//	BYTE cmd 
//	BYTE[4] item id 
//	BYTE[2] # of items in stack 
//Note: Client Message
CPIGetItem::CPIGetItem()
{
}
CPIGetItem::CPIGetItem( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIGetItem::Receive( void )
{
	tSock->Receive( 7, false );
}

// bool CPIGetItem::Handle() implemented in cplayeraction.cpp

//0x08 Packet
//Last Modified on Friday, 19-May-2000 
//Drop Item(s) (14 bytes) 
//	BYTE cmd 
//	BYTE[4] item id 
//	BYTE[2] X Location 
//	BYTE[2] Y Location
//	BYTE Z Location
//	BYTE[4] Move Into Container ID (FF FF FF FF if normal world) 
//Note: Client Message
//Note: 3D clients sometimes sends 2 of them (bursts) for ONE drop action. 
//The last one having 1s in X/Y locs.
//Be very careful how to handle this odd bursts server side, neither always process, nor always skipping is correct.
CPIDropItem::CPIDropItem()
{
}
CPIDropItem::CPIDropItem( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIDropItem::Receive( void )
{
	tSock->Receive( 14, false );
}

// bool CPIDropItem::Handle() implemented in cplayeraction.cpp

//0xB1 Packet
//Last Modified on Sunday, 28-Jul-2002
//Gump Menu Selection (Variable # of bytes) 
//	BYTE cmd													0
//	BYTE[2] blockSize											1
//	BYTE[4] id (first Id in 0xb0)								3
//	BYTE[4] gumpId (second Id in 0xb0)							7
//	BYTE[4] buttonId (which button perssed ? 0 if closed)		11
//	BYTE[4] switchcount  (response info for radio buttons and checkboxes, any switches listed here are switched on)	15
//	For each switch
//		BYTE[4] SwitchId
//	BYTE[4] textcount  (response info for textentries)			19 + switchcount * 4
//	For each textentry
//		BYTE[2] textId
//		BYTE[2] textlength
//		BYTE[length*2] Unicode text (not nullterminated)
CPIGumpMenuSelect::CPIGumpMenuSelect()
{
}
CPIGumpMenuSelect::CPIGumpMenuSelect( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIGumpMenuSelect::Receive( void )
{
	tSock->Receive( 3, false );
	tSock->Receive( tSock->GetWord( 1 ), false );
}
bool CPIGumpMenuSelect::Handle( void )
{
	id			= tSock->GetDWord( 3 );
	gumpID		= tSock->GetDWord( 7 );
	buttonID	= tSock->GetDWord( 11 );
	switchCount	= tSock->GetDWord( 15 );
	textOffset	= 19 + 4 * switchCount;
	textCount	= tSock->GetDWord( textOffset );
	
	BuildTextLocations();
	Gumps->Button( this );
	return true;
}

SERIAL CPIGumpMenuSelect::ButtonID( void ) const
{
	return buttonID;
}
SERIAL CPIGumpMenuSelect::GumpID( void ) const
{
	return gumpID;
}
SERIAL CPIGumpMenuSelect::ID( void ) const
{
	return id;
}
UI32 CPIGumpMenuSelect::SwitchCount( void ) const
{
	return switchCount;
}
UI32 CPIGumpMenuSelect::TextCount( void ) const
{
	return textCount;
}

UI32 CPIGumpMenuSelect::SwitchValue( UI32 index ) const
{
	if( index >= switchCount )
		return INVALIDSERIAL;
	// we use a 0 based counter, which means that the first switch is at 19, not 15
	return tSock->GetDWord( 19 + 4 * index );
}

UI16 CPIGumpMenuSelect::GetTextID( UI08 number ) const
{
	if( number >= textCount )
		return 0xFFFF;
	return tSock->GetWord( textLocationOffsets[number] );
}
UI16 CPIGumpMenuSelect::GetTextLength( UI08 number ) const
{
	if( number >= textCount )
		return 0xFFFF;
	return tSock->GetWord( textLocationOffsets[number] + 2 );
}
std::string CPIGumpMenuSelect::GetTextString( UI08 number ) const
{
	if( number >= textCount )
		return "";
	UI08 bufferLen		= GetTextLength( number ) * 2 + 1;
	UI08 *buffer		= new UI08[bufferLen];
	UI16 bufferOffset	= textLocationOffsets[number] + 4;
	strncpy( (char *)buffer, (const char *)&(tSock->Buffer()[bufferOffset]), bufferLen );
	buffer[bufferLen-1] = 0;
	std::string toReturn = (char *)buffer;
	delete [] buffer;
	return toReturn;

}
std::string CPIGumpMenuSelect::GetTextUString( UI08 number ) const
{
	if( number >= textCount )
		return "";
	std::string toReturn = "";
	UI16 offset = textLocationOffsets[number] + 4;
	for( int counter = 0; counter < GetTextLength( number ); ++counter )
		toReturn += tSock->GetByte( offset + counter * 2 + 1 );
	return toReturn;
}
void CPIGumpMenuSelect::BuildTextLocations( void )
{
	size_t i = textOffset + 4;	// first is textOffset + 4, to walk past the number of text strings
	textLocationOffsets.resize( textCount );
	for( size_t j = 0; j < textCount; ++j )
	{
		textLocationOffsets[j] = static_cast<wchar_t>(i);
		i += 2;	// skip the text ID
		UI16 textLen = tSock->GetWord( i );
		i += ( 2 * textLen + 2 );	// we need to add the + 2 for the text len field
	}
}


//0x78 Packet
//Draw object (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[4] itemID/playerID 
//	BYTE[2] model (item hex #) 
//	if (itemID & 0x80000000) 
//		BYTE[2] amount/Corpse Model Num 
//	BYTE[2] xLoc (only 15 lsb) 
//	BYTE[2] yLoc 
//	if (xLoc & 0x8000) 
//		BYTE direction 
//	BYTE zLoc 
//	BYTE direction 
//	BYTE[2] dye/skin color 
//	BYTE flag 
//	BYTE notoriety (2's complement signed) 
//	if (BYTE[4] == 0x00 0x00 0x00 0x00) 
//		DONE 
//	else loop this until above if statement is satisified
//		BYTE[4] itemID 
//		BYTE[2] model (item hex # - only 15 lsb) 
//		BYTE layer 
//		if (model & 0x8000) 
//			BYTE[2] hue 
//
void CPDrawObject::InternalReset( void )
{
	SetLength( 19 );
	internalBuffer[0] = 0x78;
}
void CPDrawObject::SetLength( UI16 len )
{
	internalBuffer.resize( len );
	internalBuffer[1] = (UI08)(len>>8);
	internalBuffer[2] = (UI08)(len%256);
	curLen = len;
}
CPDrawObject::CPDrawObject()
{
	InternalReset();
}
CPDrawObject::CPDrawObject( CChar &mChar )
{
	InternalReset();
	CopyData( mChar );
}
CPDrawObject::~CPDrawObject()
{
}
void CPDrawObject::Finalize( void )
{
	int cPos = curLen;
	SetLength( curLen + 4 );
	PackLong( &internalBuffer[0], cPos, (UI32)0 );
}
void CPDrawObject::AddItem( CItem *toAdd )
{
	int cPos = curLen;
	bool bColour = ( toAdd->GetColour() != 0 );
	if( bColour )
		SetLength( curLen + 9 );
	else
		SetLength( curLen + 7 );

	PackLong(  &internalBuffer[0], cPos, toAdd->GetSerial() );
	PackShort( &internalBuffer[0], cPos + 4, toAdd->GetID() );
	internalBuffer[cPos + 6] = toAdd->GetLayer();

	if( bColour )
	{
		cPos += 7;
		internalBuffer[cPos - 3] |= 0x80;
		PackShort( &internalBuffer[0], cPos, toAdd->GetColour() );
	}
}
void CPDrawObject::SetRepFlag( UI08 value )
{
	internalBuffer[18] = value;
}
void CPDrawObject::SetCharFlag( UI08 value )
{
	internalBuffer[17] = value;
}
CPDrawObject& CPDrawObject::operator=( CChar& mChar )
{
	CopyData( mChar );
	return (*this);
}
void CPDrawObject::CopyData( CChar& mChar )
{
	PackLong(  &internalBuffer[0],  3, mChar.GetSerial() );
	
	PackShort( &internalBuffer[0],  7, mChar.GetID() );
	PackShort( &internalBuffer[0],  9, mChar.GetX() );
	PackShort( &internalBuffer[0], 11, mChar.GetY() );
	PackShort( &internalBuffer[0], 15, mChar.GetSkin() );

	SetZ( mChar.GetZ() );
	internalBuffer[14] = mChar.GetDir();
}

void CPDrawObject::SetZ( UI08 value )
{
	internalBuffer[13] = value;
}


//0x89 Packet
//Corpse Clothing (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize
//	BYTE[4] corpseID
//		BYTE itemLayer 
//		BYTE[4] itemID
//	BYTE terminator (0x00)
//Followed by a 0x3C message with the contents.

void CPCorpseClothing::InternalReset( void )
{
	internalBuffer.resize( 7 );
	internalBuffer[0] = 0x89;
}
CPCorpseClothing::CPCorpseClothing()
{
	InternalReset();
}
CPCorpseClothing::CPCorpseClothing( CItem *corpse )
{
	InternalReset();
	if( corpse != NULL )
		CopyData( (*corpse) );
}

void CPCorpseClothing::NumberOfItems( UI16 numItems )
{
	// knowing the number of items, set the packet size
	UI16 packetSize = (UI16)((numItems * 5) + 8);		// 7 for lead in, 1 for lead out
	internalBuffer.resize( packetSize );
	internalBuffer[1] = (UI08)(packetSize>>8);
	internalBuffer[2] = (UI08)(packetSize%256);
	internalBuffer[packetSize - 1] = 0;
}
void CPCorpseClothing::AddItem( CItem *toAdd, UI16 itemNum )
{
	UI16 baseOffset = (UI16)(7 + itemNum * 5);
	internalBuffer[baseOffset]	= toAdd->GetLayer();
	PackLong( &internalBuffer[0], baseOffset + 1, toAdd->GetSerial() );
}
CPCorpseClothing& CPCorpseClothing::operator=( CItem& corpse )
{
	CopyData( corpse );
	return (*this);
}

void CPCorpseClothing::CopyData( CItem& toCopy )
{
	UI16 itemCount = 0;
	for( CItem *ctr = toCopy.FirstItem(); !toCopy.FinishedItems(); ctr = toCopy.NextItem() )
	{
		if( ctr != NULL )
		{
			if( !ctr->isFree() && ctr->GetLayer() != 0 )
				++itemCount;
		}
	}
	NumberOfItems( itemCount );
	UI16 iCount = 0;
	for( CItem *ci = toCopy.FirstItem(); !toCopy.FinishedItems(); ci = toCopy.NextItem() )
	{
		if( ci != NULL )
		{
			if( !ci->isFree() && ci->GetLayer() != 0 )
			{
				AddItem( ci, iCount );
				++iCount;
			}
		}
	}
}

//0x1A Packet
//Last Modified on Saturday, 13-Apr-1999 
//Object Information (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[4] itemID 
//	BYTE[2] model # 
//	if (itemID & 0x80000000) 
//		BYTE[2] item count (or model # for corpses) 
//	if (model & 0x8000) 
//		BYTE Incr Counter (increment model by this #) 
//	BYTE[2] xLoc (only use lowest significant 15 bits) 
//	BYTE[2] yLoc 
//	if (xLoc & 0x8000) 
//		BYTE direction 
//	BYTE zLoc 
//	if (yLoc & 0x8000) 
//		BYTE[2] dye 
//	if (yLoc & 0x4000) 
//		BYTE flag byte (See top)
void CPObjectInfo::InternalReset( void )
{
	internalBuffer.resize( 19 );
	internalBuffer[0] = 0x1A;
	internalBuffer[1] = 0;
	internalBuffer[2] = 19;
}
void CPObjectInfo::CopyData( CItem& mItem, CChar& mChar )
{
	PackLong( &internalBuffer[0], 3, mItem.GetSerial() );
	internalBuffer[3] += 0x80;	// Enable piles

	// if player is a gm, this item
	// is shown like a candle (so that he can move it),
	// ....if not, the item is a normal
	// invisible light source!
	if( mChar.IsGM() && mItem.GetID() == 0x1647 )
		PackShort( &internalBuffer[0], 7, 0x0A0F );
	else
		PackShort( &internalBuffer[0], 7, mItem.GetID() );

	PackShort( &internalBuffer[0],  9, mItem.GetAmount() );
	PackShort( &internalBuffer[0], 11, mItem.GetX() );
	PackShort( &internalBuffer[0], 13, mItem.GetY() + 0xC000 );	// Enable Dye and Move
	internalBuffer[15] = mItem.GetZ();

	if( mChar.IsGM() && mItem.GetID() == 0x1647 )
		PackShort( &internalBuffer[0], 16, 0x00C6 );
	else
		PackShort( &internalBuffer[0], 16, mItem.GetColour() );
	internalBuffer[18] = 0;
	if( mItem.GetVisible() > 0 )
		internalBuffer[18] |= 0x80;

	if( mItem.GetMovable() == 1 || mChar.AllMove() || ( mItem.IsLockedDown() && &mChar == mItem.GetOwnerObj() ) ) 
		internalBuffer[18] += 0x20;
	if( mChar.ViewHouseAsIcon() && mItem.GetID() >= 0x4000 )
		PackShort( &internalBuffer[0], 7, 0x14F0 );

	if( mItem.GetDir() )
	{
		internalBuffer.resize( 20 );
		for( int i = 19; i > 15; --i )
			internalBuffer[i] = internalBuffer[i-1];
		
		internalBuffer[15]	= mItem.GetDir();
		++internalBuffer[2];
		internalBuffer[11]	+= 0x80;	// Enable direction
	}
}
CPObjectInfo::CPObjectInfo()
{
	InternalReset();
}
CPObjectInfo::CPObjectInfo( CItem& mItem, CChar& mChar )
{
	InternalReset();
	CopyData( mItem, mChar );
}
CPObjectInfo::~CPObjectInfo()
{
}
void CPObjectInfo::Objects( CItem& mItem, CChar& mChar )
{
	CopyData( mItem, mChar );
}


//	COLOUR				textColour;
//	UI16				strLen;
//	UI16				fontUsed;
//	char				txtSaid[4096];
CPITalkRequest::CPITalkRequest()
{
	InternalReset();
}
CPITalkRequest::CPITalkRequest( cSocket *s ) : cPInputBuffer( s )
{
	InternalReset();
}
void CPITalkRequest::InternalReset( void )
{
	textColour	= 0x3B2;
	strLen		= 0;
	fontUsed	= 0;
	typeUsed	= UNKNOWN;
	memset( txtSaid, 0, 4096 );
	isUnicode	= false;
}

COLOUR CPITalkRequest::TextColour( void ) const
{
	return textColour;
}
UI16 CPITalkRequest::Length( void ) const
{
	return strLen;
}
UI16 CPITalkRequest::Font( void ) const
{
	return fontUsed;
}

SpeechType CPITalkRequest::Type( void ) const
{
	return typeUsed;
}

bool CPITalkRequest::IsUnicode( void ) const
{
	return isUnicode;
}

std::string CPITalkRequest::TextString( void )	const
{
	return txtSaid;
}
char * CPITalkRequest::Text( void ) const
{
	return (char *)&txtSaid[0];
}

bool CPITalkRequest::HandleCommon( void )
{
	CChar *ourChar = tSock->CurrcharObj();
	ourChar->BreakConcentration( tSock );
	ourChar->setUnicode( false );
	CItem *speechItem = calcItemObjFromSer( ourChar->GetSpeechItem() );

	SI32 j = 0;
	char temp[1024];

	cSocket *tmpSock = NULL;

	switch( ourChar->GetSpeechMode() )
	{
		case 3: // Player vendor item pricing
			j = UString( Text() ).toLong();
			if( j > 0 )
			{
				speechItem->SetBuyValue( j );
				tSock->sysmessage( 753, j );
			} 
			else 
				tSock->sysmessage( 754, speechItem->GetBuyValue() );
			tSock->sysmessage( 755 );
			ourChar->SetSpeechMode( 4 );
			break;
		case 4: // Player vendor item describing
			speechItem->SetDesc( Text() );
			tSock->sysmessage( 756, Text() );
			ourChar->SetSpeechMode( 0 );
			break;
		case 7: // Rune renaming
			char tempname[512];
			sprintf( tempname, Dictionary->GetEntry( 684 ).c_str(), Text() );
			speechItem->SetName( tempname );
			tSock->sysmessage( 757, Text() );
			ourChar->SetSpeechMode( 0 );
			break;
		case 6: // Name deed
			ourChar->SetName( Text() );
			tSock->sysmessage( 758, Text() );
			ourChar->SetSpeechMode( 0 );
			Items->DeleItem( speechItem );
			ourChar->SetSpeechItem( INVALIDSERIAL );
			break;
		case 5:	// Key renaming
		case 8: // Sign renaming
			speechItem->SetName( Text() );
			tSock->sysmessage( 759, Text() );
			ourChar->SetSpeechMode( 0 );
			break;
		case 9: // JavaScript speech
		{
			UI16 scpTrig		= ourChar->GetSpeechCallback();
			cScript *myScript	= Trigger->GetScript( scpTrig );
			
			if( myScript != NULL )
				myScript->OnSpeechInput( ourChar, speechItem, Text() );

			ourChar->SetSpeechMode( 0 );
			break;
		}
		case 1: // GM Page
			UI08 a1, a2, a3, a4;
			if( GMQueue->GotoPos( GMQueue->FindCallNum( ourChar->GetPlayerCallNum() ) ) )
			{
				HelpRequest *tempPage = NULL;
				tempPage = GMQueue->Current();
				a1 = ourChar->GetSerial( 1 );
				a2 = ourChar->GetSerial( 2 );
				a3 = ourChar->GetSerial( 3 );
				a4 = ourChar->GetSerial( 4 );

				tempPage->Reason( Text() );
				tempPage->WhoPaging( ourChar->GetSerial() );
				sprintf( temp, "GM Page from %s [%x %x %x %x]: %s", ourChar->GetName().c_str(), a1, a2, a3, a4, tempPage->Reason() );
				bool x = false;
				Network->PushConn();
				for( tmpSock = Network->FirstSocket(); !Network->FinishedSockets(); tmpSock = Network->NextSocket() )
				{
					CChar *tChar = tmpSock->CurrcharObj();
					if( tChar == NULL )
						continue;

					if( tChar->IsGMPageable() )
					{
						x = true;
						tmpSock->sysmessage( temp );
					}
				}
				Network->PopConn();
				if( x )
					tSock->sysmessage( 363 );
				else 
					tSock->sysmessage( 364 );
			}
			ourChar->SetSpeechMode( 0 );
			break;
		case 2: // Counselor Page
			a1 = ourChar->GetSerial( 1 );
			a2 = ourChar->GetSerial( 2 );
			a3 = ourChar->GetSerial( 3 );
			a4 = ourChar->GetSerial( 4 );

			HelpRequest *tempPageCns;
			if( CounselorQueue->GotoPos( CounselorQueue->FindCallNum( ourChar->GetPlayerCallNum() ) ) )
			{
				tempPageCns = CounselorQueue->Current();
				tempPageCns->Reason( Text() );
				tempPageCns->WhoPaging( ourChar->GetSerial() );
				sprintf( temp, "Counselor Page from %s [%x %x %x %x]: %s", ourChar->GetName().c_str(), a1, a2, a3, a4, tempPageCns->Reason() );
				bool x = false;
				Network->PushConn();
				for( tmpSock = Network->FirstSocket(); !Network->FinishedSockets(); tmpSock = Network->NextSocket() )
				{
					CChar *tChar = tmpSock->CurrcharObj();
					if( tChar == NULL )
						continue;

					if( tChar->IsGMPageable() )
					{
						x = true;
						tmpSock->sysmessage( temp );
					}
				}
				Network->PopConn();
				if( x )
					tSock->sysmessage( 360 );
				else 
					tSock->sysmessage( 361 );
			}
			ourChar->SetSpeechMode( 0 );
			break;
		case 0: // normal speech
		default:
			if( ourChar->GetSquelched() )
				tSock->sysmessage( 760 );
			else 
				return false;
	//			Speech->talking( this );
			break;
	}
	return true;
}


//Talk Request (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE SpeechType 
//	BYTE[2] Color 
//	BYTE[2] SpeechFont 
//	BYTE[?] msg - Null Terminated (blockSize - 8)
CPITalkRequestAscii::CPITalkRequestAscii() : CPITalkRequest()
{
}
CPITalkRequestAscii::CPITalkRequestAscii( cSocket *s ) : CPITalkRequest( s )
{
	Receive();
}
void CPITalkRequestAscii::Receive( void )
{
	tSock->Receive( 3, false );
	UI16 blockLen	= tSock->GetWord( 1 );
	tSock->Receive( blockLen, false );

	strLen			= blockLen - 8;
	typeUsed		= (SpeechType)tSock->GetByte( 3 );
	textColour		= tSock->GetWord( 4 );
	fontUsed		= tSock->GetWord( 6 );
	
	strcpy( txtSaid, (char *)&(tSock->Buffer()[8]) );
}
bool CPITalkRequestAscii::Handle( void )
{
	if( !HandleCommon() )
		Speech->generictalking( this );
	return true;
}

//0xAD Packet
//Last Modified on Monday, 18-Feb-2002 
//Unicode/Ascii speech request (Variable # of bytes)
//BYTE cmd												0
//	BYTE[2] blockSize									1
//	BYTE Type											3
//	BYTE[2] Color										4
//	BYTE[2] Font										6
//	BYTE[4] Language (Null Terminated)					8
//		enu - United States English
//		des - German Swiss
//		dea - German Austria
//		deu - German Germany
//		... for a complete list see langcode.iff
//	if (Type & 0xc0) 
//		BYTE[1,5] Number of distinct Trigger words (NUMWORDS).
//			12 Bit number, Byte #13  = Bit 114 of NUMWORDS, Hi-Nibble of Byte #14 (Bit 74)  = Bit  03 of NUMWORDS
//		BYTE[1,5] Index to speech.mul
//			12 Bit number, Low Nibble of Byte #14 (Bits 30) = Bits 11..8 of Index, Byte #15  = Bits 70 of Index
//		UNKNOWNS = ( (NUMWORDS div 2) *3 ) + (NUMWORDS % 2)   1
//			div = Integeger division, % = modulo operation, NUMWORDS >= 1
//			examples: UNKNOWNS(1)=0, UNKNOWNS(2)=2, UNKNOWNS(3)=3, UNKNOWNS(4)=5, UNKNOWNS(5)=6, 
//			UNKNOWNS(6)=8, UNKNOWNS(7)=9, UNKNOWNS(8)=11, UNKNOWNS(9)=12 
//		BYTE[UNKNOWNS]
//			Idea behind this is getting speech parsing load client side.
//			Thus this contains data OSI server use for easier parsing.
//			Its client side hardcoded and exact details are unkown.
//		BYTE[?] Ascii Msg  Null Terminated (blockSize  (15+UNKNOWNS) )
//	else 
//		BYTE[?][2] Unicode Msg - Null Terminated (blockSize - 12)
//	Note1:  For pre 2.0.7 clients Type is always < 0xc0
//	Note2: Uox based emus convert post 2.0.7 data of this packet to pre 2.0.7 data if Type >=0xc0
//	Note3: (different view of it)
//	If Mode&0xc0 then there are keywords (from speech.mul) present. 
//	Keywords:
//	The first 12 bits = the number of keywords present. The keywords are included right after this, each one is 12 bits also. 
//	The keywords are padded to the closest byte. For example, if there are 2 keywords, it will take up 5 bytes. 12bits for the number, and 12 bits for each keyword. 12+12+12=36. Which will be padded 4 bits to 40 bits or 5 bytes.

void UpdateLanguage( cSocket *s );

CPITalkRequestUnicode::CPITalkRequestUnicode()
{
	isUnicode = true;
	memset( unicodeTxt, 0, 8192 );
}
CPITalkRequestUnicode::CPITalkRequestUnicode( cSocket *s ) : CPITalkRequest( s )
{
	isUnicode = true;
	memset( unicodeTxt, 0, 8192 );
	Receive();
}
void CPITalkRequestUnicode::Receive( void )
{
	tSock->Receive( 3, false );
	UI16 blockLen	= tSock->GetWord( 1 );
	tSock->Receive( blockLen, false );

//	strLen			= blockLen - 8;
	typeUsed		= (SpeechType)tSock->GetByte( 3 );
	textColour		= tSock->GetWord( 4 );
	fontUsed		= tSock->GetWord( 6 );

	UpdateLanguage( tSock );

	CChar *mChar	= tSock->CurrcharObj();
	mChar->setUnicode( true );

	// Check for command word versions of this packet
	int myoffset	= 13;
	int myincrement = 2;
	int myj			= 12;
	UI08 *buffer	= tSock->Buffer();
	UI16 j			= 0;

	for( j = 8; j <= 10; ++j )
		langCode[j - 8] = tSock->GetByte( j );

	langCode[3] = 0;

	if( typeUsed == 0xC0 || typeUsed == 0xC9 || typeUsed == 0xC1 || typeUsed == 0xC2 || typeUsed == 0xC6 || typeUsed == 0xC8 )
	{
		myincrement = 1;
//		buffer[3] &= 0x0F;	// set to normal to send it back
		switch( tSock->GetByte( 13 ) & 0xF0 )
		{		// Copy the buffer up, and convert it to unicode
			case 0x10:	myoffset = 15;		break;
			case 0x20:	myoffset = 17;		break;
			case 0x30:	myoffset = 18;		break;
			case 0x40:	myoffset = 20;		break;
		}
		//
		myj = 12;
		int mysize = blockLen - myoffset;
		for( j = 0; j < mysize; ++j )
			unicodeTxt[j] = buffer[j + myoffset];
		for( j = 0; j < mysize; ++j )
		{	// we would overwrite our buffer, so we need to catch it before we do that.
			buffer[myj++] = 0;
			buffer[myj++] = unicodeTxt[j];
		}
		buffer[myj-1] = 0;
		blockLen = ((blockLen - myoffset) * 2) + 12;
		PackShort( tSock->Buffer(), 1, blockLen );
		// update our unicode text
		memcpy( unicodeTxt, &(tSock->Buffer()[12]), blockLen - 12 );
	}
	else		// NOT a weird configuration, it is unicode however!
	{
		// starts at 12
		memcpy( unicodeTxt, &(tSock->Buffer()[12]), blockLen - 12 );
	}
	strLen = 0;
	for( j = 13; j < blockLen; j += 2 )
	{
		txtSaid[(j-13)/2] = buffer[j];
		++strLen;
	}
}

bool CPITalkRequestUnicode::Handle( void )
{
	if( !HandleCommon() )
		Speech->generictalking( this );
	return true;
}

std::string CPITalkRequestUnicode::UnicodeTextString( void ) const
{
	return unicodeTxt;
}
char * CPITalkRequestUnicode::UnicodeText( void ) const
{
	return (char *)unicodeTxt;
}

char * CPITalkRequestUnicode::Language( void ) const
{
	return (char *)langCode;
}


//0x1C Packet
//Last Modified on Friday, 20-Apr-1998 
//Send Speech (Variable # of bytes) 
//	BYTE cmd										0
//	BYTE[2] blockSize								1
//	BYTE[4] itemID (FF FF FF FF = system)			3
//	BYTE[2] model (item hex # - FF FF = system)		7
//	BYTE Type										9
//	BYTE[2] Text Color								10
//	BYTE[2] Font									12
//	BYTE[30] Name									14
//	BYTE[?] Null-Terminated Message (? = blockSize - 44) 
void CPSpeech::Object( cBaseObject &talking )
{
	CopyData( talking );
}
void CPSpeech::Object( CPITalkRequest &tSaid )
{
	CopyData( tSaid );
}
void CPSpeech::Type( UI08 value )
{
	internalBuffer[9] = value;
}
void CPSpeech::Colour( COLOUR value )
{
	// Ideally one would track down where the value is getting set to 0, 
	// a color of 0 will crash 3d client
	if( value == 0 )
	{
		value = 0x3B2;
	}
	PackShort( &internalBuffer[0], 10, value );
}
void CPSpeech::Font( UI16 value )
{
	PackShort( &internalBuffer[0], 12, value );
}
void CPSpeech::Name( char *value )
{
	strncpy( (char *)&internalBuffer[14], value, 30 );
}
void CPSpeech::Message( char *value )
{
	size_t length = strlen( value );
	SetLength( static_cast< UI16 >(44 + length + 1) );
	strncpy( (char *)&internalBuffer[44], value, length );
}
void CPSpeech::SetLength( UI16 value )
{
	internalBuffer.resize( value );
	PackShort( &internalBuffer[0], 1, value );
}

void CPSpeech::InternalReset( void )
{
	SetLength( 44 );
	internalBuffer[0] = 0x1C;
	Colour( 0x3B2 );
}

CPSpeech::CPSpeech()
{
	InternalReset();
}
CPSpeech::CPSpeech( cBaseObject &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
CPSpeech::CPSpeech( CPITalkRequest &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
CPSpeech &CPSpeech::operator=( cBaseObject &toCopy )
{
	CopyData( toCopy );
	return (*this);
}
void CPSpeech::CopyData( cBaseObject &toCopy )
{
	Serial( toCopy.GetSerial() );
	ID( toCopy.GetID() );
	Name( (char *)toCopy.GetName().c_str() );
//	internalBuffer[9] = 1;
}
void CPSpeech::CopyData( CPITalkRequest &talking )
{
	Colour( talking.TextColour() );
	Font( talking.Font() );
	Type( talking.Type() );
	Message( talking.Text() );
}
void CPSpeech::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 3, toSet );
}
void CPSpeech::ID( UI16 toSet )
{
	PackShort( &internalBuffer[0], 7, toSet );
}
void CPSpeech::GrabSpeech( cSocket *mSock, CChar *mChar )
{
	internalBuffer[10] = mSock->GetByte( 4 );
	internalBuffer[11] = mSock->GetByte( 5 );
	internalBuffer[12] = mSock->GetByte( 6 );
	internalBuffer[13] = mChar->GetFontType();
}

void CPSpeech::GhostIt( UI08 method )
{
	// Method ignored currently
	// Designed with the idea that you can garble text in more than one way
	// eg 0 == ghost, 1 == racial, 2 == magical, etc etc
	for( UI16 j = 44; j < internalBuffer.size() - 1; ++j )
	{
		if( internalBuffer[j] != 32 )
			internalBuffer[j] = ( RandomNum( 0, 1 ) == 0 ? 'O' : 'o' );
	}
}
CPSpeech &CPSpeech::operator=( CPSpeech &copyFrom )
{
	internalBuffer.resize( copyFrom.internalBuffer.size() );
	for( UI32 i = 0; i < copyFrom.internalBuffer.size(); ++i )
		internalBuffer[i] = copyFrom.internalBuffer[i];
	return (*this);
}

//	0xAE Packet
//	Last Modified on Wednesday, 11-Nov-1998 
//	Unicode Speech message(Variable # of bytes) 
//	
//	BYTE cmd 
//	BYTE[2] blockSize
//	BYTE[4] ID
//	BYTE[2] Model
//	BYTE Type
//	BYTE[2] Color
//	BYTE[2] Font
//	BYTE[4] Language
//	BYTE[30] Name
//	BYTE[?][2] Msg  Null Terminated (blockSize - 48)
void CPUnicodeSpeech::Object( cBaseObject &talking )
{
	CopyData( talking );
}
void CPUnicodeSpeech::Object( CPITalkRequestAscii &tSaid )
{
	CopyData( tSaid );
}
void CPUnicodeSpeech::Object( CPITalkRequestUnicode &tSaid )
{
	CopyData( tSaid );
}
void CPUnicodeSpeech::Type( UI08 value )
{
	internalBuffer[9] = ( value & 0x0F );
}
void CPUnicodeSpeech::Colour( COLOUR value )
{
	PackShort( &internalBuffer[0], 10, value );
}
void CPUnicodeSpeech::Font( UI16 value )
{
	PackShort( &internalBuffer[0], 12, value );
}
void CPUnicodeSpeech::Language( char *value )
{
	strncpy( (char *)&internalBuffer[14], value, 4 );
}
void CPUnicodeSpeech::Name( char *value )
{
	strncpy( (char *)&internalBuffer[18], value, 30 );
}
void CPUnicodeSpeech::Message( char *value )
{
	size_t length = strlen( value );
	SetLength( static_cast< UI16 >(48 + 2 * length + 2) );
	for( UI16 i = 0; i < length; ++i )
		internalBuffer[49 + i * 2] = value[i];
}
void CPUnicodeSpeech::SetLength( UI16 value )
{
	internalBuffer.resize( value );
	PackShort( &internalBuffer[0], 1, value );
}

void CPUnicodeSpeech::InternalReset( void )
{
	SetLength( 48 );
	internalBuffer[0] = 0xAE;
	Language( "ENU" );
}

CPUnicodeSpeech::CPUnicodeSpeech()
{
	InternalReset();
}
CPUnicodeSpeech::CPUnicodeSpeech( cBaseObject &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
CPUnicodeSpeech::CPUnicodeSpeech( CPITalkRequestAscii &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
CPUnicodeSpeech::CPUnicodeSpeech( CPITalkRequestUnicode &toCopy )
{
	InternalReset();
	CopyData( toCopy );
}
CPUnicodeSpeech &CPUnicodeSpeech::operator=( cBaseObject &toCopy )
{
	CopyData( toCopy );
	return (*this);
}
void CPUnicodeSpeech::CopyData( cBaseObject &toCopy )
{
	Serial( toCopy.GetSerial() );
	ID( toCopy.GetID() );
	Name( (char *)toCopy.GetName().c_str() );
//	internalBuffer[9] = 1;
}
void CPUnicodeSpeech::CopyData( CPITalkRequestAscii &talking )
{
	Colour( talking.TextColour() );
	Font( talking.Font() );
	Type( talking.Type() );
	Language( "ENU" );
	Message( talking.Text() );
}
void CPUnicodeSpeech::CopyData( CPITalkRequestUnicode &talking )
{
	Colour( talking.TextColour() );
	Font( talking.Font() );
	Type( talking.Type() );
	Language( talking.Language() );
//	Message( talking.Text() );

	UI16 length = talking.Length();
	char *uniTxt = talking.UnicodeText();
	
	SetLength( 48 + 2 * length + 2 );
	for( UI16 i = 0; i < (2 * length); ++i )
		internalBuffer[48 + i] = uniTxt[i];
}
void CPUnicodeSpeech::Serial( SERIAL toSet )
{
	PackLong( &internalBuffer[0], 3, toSet );
}
void CPUnicodeSpeech::ID( UI16 toSet )
{
	PackShort( &internalBuffer[0], 7, toSet );
}
void CPUnicodeSpeech::GrabSpeech( cSocket *mSock, CChar *mChar )
{
	internalBuffer[10] = mSock->GetByte( 4 );
	internalBuffer[11] = mSock->GetByte( 5 );
	internalBuffer[12] = mSock->GetByte( 6 );
	internalBuffer[13] = mChar->GetFontType();
}

void CPUnicodeSpeech::GhostIt( UI08 method )
{
	// Method ignored currently
	// Designed with the idea that you can garble text in more than one way
	// eg 0 == ghost, 1 == racial, 2 == magical, etc etc
	for( UI16 j = 49; j < internalBuffer.size() - 1; j += 2 )
	{
		if( internalBuffer[j] != 32 )
			internalBuffer[j] = ( RandomNum( 0, 1 ) == 0 ? 'O' : 'o' );
	}
}
CPUnicodeSpeech &CPUnicodeSpeech::operator=( CPUnicodeSpeech &copyFrom )
{
	internalBuffer.resize( copyFrom.internalBuffer.size() );
	for( UI32 i = 0; i < copyFrom.internalBuffer.size(); ++i )
		internalBuffer[i] = copyFrom.internalBuffer[i];
	return (*this);
}

//	0xA8 Packet
//	Last Modified on Monday, 13-Apr-1998 17:06:50 EDT 
//	Game Server List (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE System Info Flag 
//		0xCC - Don't send 
//		0x64 - Send Video card 
//		?? - 
//	BYTE[2] # of servers 
//	Then each server -- 
//		BYTE[2] serverIndex (0-based) 
//		BYTE[32] serverName 
//		BYTE percentFull 
//		BYTE timezone 
//		BYTE[4] pingIP 
void CPGameServerList::InternalReset( void )
{
	internalBuffer.resize( 6 );
	internalBuffer[0] = 0xA8;
	internalBuffer[2] = 6;
	internalBuffer[3] = 0xFF;
}
CPGameServerList::CPGameServerList()
{
	InternalReset();
}
CPGameServerList::CPGameServerList( UI16 numServers )
{
	InternalReset();
	NumberOfServers( numServers );
}
void CPGameServerList::NumberOfServers( UI16 numItems )
{
	UI32 tlen = 6 + ( numItems * 40 );
	internalBuffer.resize( tlen );
	PackShort( &internalBuffer[0], 1, static_cast< UI16 >(tlen) );
	PackShort( &internalBuffer[0], 4, numItems );
}
void CPGameServerList::AddServer( UI16 servNum, physicalServer *data )
{
	UI32 baseOffset = 6 + servNum * 40;
	PackShort( &internalBuffer[0], baseOffset, servNum + 1 );
	strcpy( (char *)&internalBuffer[baseOffset + 2], data->getName().c_str() );
	UI32 ip = htonl( inet_addr( data->getIP().c_str() ) );
	PackLong( &internalBuffer[0], baseOffset + 36, ip );
}

//	0x6F Packet
//	Last Modified on Friday, 20-Nov-1998 
//	Secure Trading (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE action
//	BYTE[4] id1
//	BYTE[4] id2
//	BYTE[4] id3
//	BYTE nameFollowing (0 or 1)
//	If (nameFollowing = 1) 
//	BYTE[?] charName 
void CPSecureTrading::InternalReset( void )
{
	internalBuffer.resize( 17 );
	internalBuffer[0] = 0x6F;
	internalBuffer[2] = 17;
}
void CPSecureTrading::CopyData( cBaseObject& mItem, cBaseObject& mItem2, cBaseObject& mItem3 )
{
	PackLong( &internalBuffer[0], 4,  mItem.GetSerial()  );
	PackLong( &internalBuffer[0], 8,  mItem2.GetSerial() );
	PackLong( &internalBuffer[0], 12, mItem3.GetSerial() );
}
void CPSecureTrading::CopyData( cBaseObject& mItem, SERIAL mItem2, SERIAL mItem3 )
{
	PackLong( &internalBuffer[0], 4,  mItem.GetSerial() );
	PackLong( &internalBuffer[0], 8,  mItem2			);
	PackLong( &internalBuffer[0], 12, mItem3			);
}
CPSecureTrading::CPSecureTrading()
{
	InternalReset();
}
CPSecureTrading::CPSecureTrading( cBaseObject& mItem, cBaseObject& mItem2, cBaseObject& mItem3 )
{
	InternalReset();
	CopyData( mItem, mItem2, mItem3 );
}
CPSecureTrading::CPSecureTrading( cBaseObject& mItem, SERIAL mItem2, SERIAL mItem3 )
{
	InternalReset();
	CopyData( mItem, mItem2, mItem3 );
}
CPSecureTrading::~CPSecureTrading()
{
}
void CPSecureTrading::Objects( cBaseObject& mItem, cBaseObject& mItem2, cBaseObject& mItem3 )
{
	CopyData( mItem, mItem2, mItem3 );
}
void CPSecureTrading::Objects( cBaseObject& mItem, SERIAL mItem2, SERIAL mItem3 )
{
	CopyData( mItem, mItem2, mItem3 );
}
void CPSecureTrading::Action( UI08 value )
{
	internalBuffer[3] = value;
}
void CPSecureTrading::Name( const char *nameFollowing )
{
	internalBuffer.resize( 47 );
	internalBuffer[16] = 1;
	strcpy( (char *)&internalBuffer[17], nameFollowing );
}

//	0x98 Packet
//	Last Modified on Monday, 04-Aug-2002 18:34:08 EDT 
//	All-names 3D (Variable # of Bytes)
//	BYTE cmd 
//	BYTE[2] blocksize
//	BYTE[4] ID
//	If (server-reply) BYTE[30] name (0 terminated)

//	NOTE: Only 3D clients send this packet 
//	Server and client packet.

//	Client asks for name of object with ID x.
//	Server has to reply with ID + name
//	Client automatically knows names of items.
//	Hence it only asks only for NPC/Player names nearby, but shows bars of items plus NPCs.

//	Client request has 7 bytes, server-reply 37
//	Triggered by Crtl + Shift. 
void CPAllNames3D::InternalReset( void )
{
	internalBuffer.resize( 37 );
	internalBuffer[0] = 0x98;
	internalBuffer[2] = 37;
}
void CPAllNames3D::CopyData( cBaseObject& obj )
{
	PackLong( &internalBuffer[0], 3, obj.GetSerial() );
	strcpy( (char *)&internalBuffer[7], obj.GetName().c_str() );
}
CPAllNames3D::CPAllNames3D()
{
	InternalReset();
}
CPAllNames3D::CPAllNames3D( cBaseObject& obj )
{
	InternalReset();
	CopyData( obj );
}
void CPAllNames3D::Object( cBaseObject& obj )
{
	CopyData( obj );
}


CPIAllNames3D::CPIAllNames3D() : cPInputBuffer()
{
}
CPIAllNames3D::CPIAllNames3D( cSocket *s ) : cPInputBuffer( s )
{
	Receive();
}
void CPIAllNames3D::Receive( void )
{
	tSock->Receive( 3, false );
	tSock->Receive( tSock->GetWord( 1 ), false );
}
bool CPIAllNames3D::Handle( void )
{
	SERIAL objSer		= tSock->GetDWord( 3 );
	cBaseObject *toName = NULL;
	
	if( objSer < BASEITEMSERIAL )
		toName = calcItemObjFromSer( objSer );
	else if( objSer != INVALIDSERIAL )
		toName = calcCharObjFromSer( objSer );

	if( toName != NULL )
	{
		CPAllNames3D toSend( (*toName) );
		tSock->Send( &toSend );
	}
	return true;
}

//	0x66 Packet
//	Last Modified on Monday, 19th-Feb-2002 
//	Books - Page (Variable # of bytes) 
//	BYTE cmd 
//	BYTE[2] blockSize 
//	BYTE[4] bookID 
//	BYTE[2] # of pages in this packet 
//	For each page: 
//		BYTE[2] page # 
//		BYTE[2] # of lines on page 
//		Repeated for each line: 
//			BYTE[var] null terminated line 
//	Note: 
//	server side: # of pages equals value given in 0x93/0xd4
//	EACH page # given. If empty: # lines: 0 + terminator (=3 0s)
//	client side:  # of pages always 1. if 2 pages changed, client generates 2 packets. 

void CPBookPage::IncLength( UI08 amount )
{
	bookLength += amount;
	internalBuffer.resize( bookLength );
}
void CPBookPage::InternalReset( void )
{
	internalBuffer.resize( 9 );
	bookLength			= 9;
	pageCount			= 0;
	internalBuffer[0]	= 0x66;
}
void CPBookPage::CopyData( CItem& obj )
{
	Serial( obj.GetSerial() );
}
CPBookPage::CPBookPage()
{
	InternalReset();
}
CPBookPage::CPBookPage( CItem& obj )
{
	InternalReset();
	CopyData( obj );
}
void CPBookPage::Object( CItem& obj )
{
	CopyData( obj );
}
void CPBookPage::NewPage( SI16 pNum )
{
	++pageCount;	// 1 based counter
	UI16 baseOffset = bookLength;
	IncLength( 4 );
	if( pNum == -1 )
		PackShort( &internalBuffer[0], baseOffset, pageCount );
	else
		PackShort( &internalBuffer[0], baseOffset, pNum );
	internalBuffer[baseOffset + 3] = 8;	// 8 lines per page
}
void CPBookPage::AddLine( const char *line )
{
	UI16 baseOffset = bookLength;
	size_t strLen	= strlen( line ) + 1;
	IncLength( static_cast< UI08 >(strLen) );
	strcpy( (char *)&internalBuffer[baseOffset], line );
}
void CPBookPage::Finalize( void )
{
	PackShort( &internalBuffer[0], 1, bookLength );
	PackShort( &internalBuffer[0], 7, pageCount );
}
void CPBookPage::Serial( SERIAL value )
{
	PackLong( &internalBuffer[0], 3, value );
}

//	0xB0 Packet
//	Last Modified on Tuesday, 20-Apr-1999 
//	Send Gump Menu Dialog (Variable # of bytes) 
//	BYTE cmd											0
//	BYTE[2] blockSize									1
//	BYTE[4] id											3
//	BYTE[4] gumpid										7
//	BYTE[4] x											11
//	BYTE[4] y											15
//	BYTE[2] command section length						19
//	BYTE[?] commands (zero terminated)					21 + ??
//	BYTE[2] numTextLines
//	BYTE[2] text length (in unicode (2 byte) characters.)
//	BYTE[?] text (in unicode)

//	STRINGLIST		commands, text;
CPSendGumpMenu::CPSendGumpMenu()
{
	internalBuffer.resize( 21 );
	internalBuffer[0] = 0xB0;		// command byte
	internalBuffer[14] = 0x6E;		// default x
	internalBuffer[18] = 0x46;		// default y
}
void CPSendGumpMenu::UserID( SERIAL value )
{
	PackLong( &internalBuffer[0], 3, value );
}
void CPSendGumpMenu::GumpID( SERIAL value )
{
	PackLong( &internalBuffer[0], 7, value );
}
void CPSendGumpMenu::X( UI32 value )
{
	PackLong( &internalBuffer[0], 11, value );
}
void CPSendGumpMenu::Y( UI32 value )
{
	PackLong( &internalBuffer[0], 15, value );
}
void CPSendGumpMenu::AddCommand( const char *actualCommand, ... )
{
	va_list argptr;
	char msg[512];
#ifdef __NONANSI_VASTART__
	va_start( argptr );
#else
	va_start( argptr, actualCommand );
#endif
	vsprintf( msg, actualCommand, argptr );
	va_end( argptr );

	commands.push_back( msg );
}
void CPSendGumpMenu::AddText( const char *actualText, ... )
{
	va_list argptr;
	char msg[512];
#ifdef __NONANSI_VASTART__
	va_start( argptr );
#else
	va_start( argptr, actualText );
#endif
	vsprintf( msg, actualText, argptr );
	va_end( argptr );

	text.push_back( msg );
}
void CPSendGumpMenu::Finalize( void )
{
	UI32 length		= 21;
	UI32 length2	= 1;
	UI16 increment	= 0;
	UI32 line		= 0;

	std::string cmdString;

	for( line = 0; line < commands.size(); ++line )
	{
		if( commands[line].length() == 0 )
			break;
		increment = static_cast< UI16 >(commands[line].length() + 4);
		internalBuffer.resize( length + increment );
		cmdString = "{ " + commands[line] + " }";
		PackString( &internalBuffer[0], length, cmdString, increment );
		length	+= increment;
		length2	+= increment;
	}

	if( length2 > 65536 )
		throw std::runtime_error( "Packet 0xB0 is far too large" );

	PackShort( &internalBuffer[0], 19, static_cast<UI16>(length2) );

	UI32 tlOff	= length + 1;
	length		+= 3;
	UI32 tlines = 0;

	internalBuffer.resize( length );	// match the 3 byte increase
	UI16 lineLen = 0;

	for( line = 0; line < text.size(); ++line )
	{
		if( text[line].length() == 0 )
			break;
		// Unfortunately, unicode strings are... different
		// so we can't use PackString
		lineLen		= static_cast< UI16 >(text[line].length());
		increment	= lineLen * 2 + 2;
		internalBuffer.resize( length + increment );
		PackShort( &internalBuffer[0], length, lineLen );
		for( UI32 i = 0; i < lineLen; ++i )
			internalBuffer[length + 3 + i*2] = (text[line])[i];
		length += increment;
		++tlines;
	}
	
	if( length > 65536 )
		throw std::runtime_error( "Packet 0xB0 is far too large" );

	PackShort( &internalBuffer[0], 1, static_cast< UI16 >(length) );
	PackShort( &internalBuffer[0], tlOff, static_cast< UI16 >(tlines) );
}
