#include "uox3.h"
#include "classes.h"
#include "regions.h"
#include "fileio.h"

SI16 upperArrayX[UOMT_COUNT];
SI16 upperArrayY[UOMT_COUNT];

void readline( std::ifstream &toRead, char *buffer, int bufferLen )
{
	bool valid = false;
	do 
	{
		toRead.getline( buffer, bufferLen );
#if defined(__unix__)
		trimWindowsText( buffer );
#endif

		if( toRead.eof() || toRead.fail() )
		{
			strcpy( buffer, "---REGION---" );
			valid = true;
		}
		else if( buffer[0] == '/' || buffer[0] == ' ' || buffer[0] == 10 || buffer[0] == 13 || buffer[0] == 0 )
			valid = false;
		else
			valid = true;
	} while( !valid );
}

void LoadChar( std::ifstream& readDestination )
{
	CHARACTER xOff;
	CChar *x = Npcs->MemCharFree( xOff, true );
	if( x == NULL ) 
		return;
	if( !x->Load( readDestination, xOff ) ) // if no load, DELETE
		Npcs->DeleteChar( x );
}
void LoadItem( std::ifstream& readDestination )
{
	ITEM xOff;
	CItem *x = Items->MemItemFree( xOff, false );
	if( x == NULL ) 
		return;
	if( !x->Load( readDestination, xOff ) ) // if no load, DELETE
		Items->DeleItem( x );
}
void LoadMulti( std::ifstream& readDestination )
{
	ITEM xOff;
	CItem *x = Items->MemItemFree( xOff, false, 1 );
	if( x == NULL ) 
		return;
	CMultiObj *ourHouse = static_cast<CMultiObj *>(x);
	if( !ourHouse->Load( readDestination, xOff ) )	// if no load, DELETE
		Items->DeleItem( x );
}

typedef UI32 vIterator;
//o--------------------------------------------------------------------------
//|	Function		-	SubRegion constructor
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	This function is does basically what the name implies
//o--------------------------------------------------------------------------
SubRegion::SubRegion() : charCounter( INVALIDSERIAL ), itemCounter( INVALIDSERIAL ) //constructor
{
}

//o--------------------------------------------------------------------------
//|	Function		-	SubRegion destructor
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes all map entries
//o--------------------------------------------------------------------------
SubRegion::~SubRegion()
{
	itemData.clear();
	charData.clear();
}

//o--------------------------------------------------------------------------
//|	Function		-	CItem *GetCurrentItem()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns a CItem * to the current referenced item
//o--------------------------------------------------------------------------
CItem *SubRegion::GetCurrentItem( void )
{
	return itemData[itemCounter];
}

//o--------------------------------------------------------------------------
//|	Function		-	CChar *GetCurrentChar()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns a CChar * to the current referenced char
//o--------------------------------------------------------------------------
CChar *SubRegion::GetCurrentChar( void )
{
	return charData[charCounter];
}

//o--------------------------------------------------------------------------
//|	Function		-	CItem *FirstItem()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Moves iterator to start and returns the first CItem *
//o--------------------------------------------------------------------------
CItem *SubRegion::FirstItem( void )
{
	itemCounter = 0;
	if( itemCounter >= itemData.size() )
		return NULL;
	else
		return itemData[itemCounter];
}

//o--------------------------------------------------------------------------
//|	Function		-	CChar *FirstChar()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Moves iterator to start and returns the first CChar *
//o--------------------------------------------------------------------------
CChar *SubRegion::FirstChar( void )
{
	charCounter = 0;
	if( charCounter >= charData.size() )
		return NULL;
	else
		return charData[charCounter];
}

//o--------------------------------------------------------------------------
//|	Function		-	CItem *GetNextItem()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Moves along the iterator, returning the next CItem *
//o--------------------------------------------------------------------------
CItem *SubRegion::GetNextItem( void )
{
	if( ++itemCounter >= itemData.size() )
		return NULL;
	else
		return itemData[itemCounter];
}

//o--------------------------------------------------------------------------
//|	Function		-	CChar *GetNextChar()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Moves along the iterator, returning the next CChar *
//o--------------------------------------------------------------------------
CChar *SubRegion::GetNextChar( void )
{
	if( ++charCounter >= charData.size() )
		return NULL;
	else
		return charData[charCounter];
}


//o--------------------------------------------------------------------------
//|	Function		-	bool FinishedItems()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	returns true if we hit the end of the iterator
//o--------------------------------------------------------------------------
bool SubRegion::FinishedItems( void )
{
	return ( itemCounter >= itemData.size() );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool FinishedChars()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns true if we hit the end of the iterator
//o--------------------------------------------------------------------------
bool SubRegion::FinishedChars( void )
{
	return ( charCounter >= charData.size() );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool AddItem( CItem *toAdd )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adds toAdd to the map if it doesn't already exist
//o--------------------------------------------------------------------------
bool SubRegion::AddItem( CItem *toAdd )
{
	for( vIterator p = 0; p < itemData.size(); ++p )
	{
		if( itemData[p] == toAdd )
			return false;
	}
	itemData.push_back( toAdd );
	return true;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool AddChar( CChar *toAdd )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adds toAdd to the map if it doesn't already exist
//o--------------------------------------------------------------------------
bool SubRegion::AddChar( CChar *toAdd )
{
	for( vIterator p = 0; p < charData.size(); ++p )
	{
		if( charData[p] == toAdd )
			return false;
	}
	charData.push_back( toAdd );
	return true;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool RemoveItem( CItem *toRemove )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes toRemove from the map if it is in there
//o--------------------------------------------------------------------------
bool SubRegion::RemoveItem( CItem *toRemove )
{
	for( vIterator p = 0; p < itemData.size(); ++p )
	{
		if( itemData[p] == toRemove )
		{
			for( unsigned int r = p; r < itemData.size() - 1; ++r )
				itemData[r] = itemData[r+1];
			itemData.resize( itemData.size() - 1 );
			for( unsigned int q = 0; q < itemIteratorBackup.size(); ++q )
			{
				if( itemIteratorBackup[q] >= p )
					--itemIteratorBackup[q];
			}
			return true;
		}
	}
	return false;
}

//o--------------------------------------------------------------------------
//|	Function		-	bool RemoveChar( CChar *toRemove )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes toRemove from the map if it is in there
//o--------------------------------------------------------------------------
bool SubRegion::RemoveChar( CChar *toRemove )
{
	for( vIterator p = 0; p < charData.size(); ++p )
	{
		if( charData[p] == toRemove )
		{
			for( unsigned int r = p; r < charData.size() - 1; ++r )
				charData[r] = charData[r+1];
			charData.resize( charData.size() - 1 );
			for( unsigned int q = 0; q < charIteratorBackup.size(); ++q )
			{
				if( charIteratorBackup[q] >= p )
					--charIteratorBackup[q];
			}
			return true;
		}
	}
	return false;
}

void SubRegion::PopItem( void )
{
	itemCounter = itemIteratorBackup.back();
	itemIteratorBackup.pop_back();
}
void SubRegion::PopChar( void )
{
	charCounter = charIteratorBackup.back();
	charIteratorBackup.pop_back();
}
void SubRegion::PushItem( void )
{
	itemIteratorBackup.push_back( itemCounter );
}
void SubRegion::PushChar( void )
{
	charIteratorBackup.push_back( charCounter );
}

void SubRegion::SaveToDisk( std::ofstream& writeDestination, std::ofstream &houseDestination )
{
	// reworked SaveChar from WorldMain to deal with pointer based stuff in region rather than index based stuff in array
	// Also saves out all data regardless (in preparation for a simple binary save)

	// Let's start by writing out our characters
	for( CChar *charToWrite = FirstChar(); !FinishedChars(); charToWrite = GetNextChar() )
	{
		//if( !charToWrite->IsNpc() && charToWrite->GetAccount() != -1 ) {
#if defined( _MSC_VER )
		#pragma note( "PlayerHTML Dumping needs to be reimplemented" )
#endif
		//DumpPlayerHTML( charToWrite );
		if( charToWrite->ShouldSave() ) 
		{
			//Console << "Saving char " << charToWrite->GetName() << myendl;
			charToWrite->Save( writeDestination );
		}
	}
	// Handle the STL
	//ITEMLIST_ITERATOR I;
	for( CItem *itemToWrite = FirstItem(); !FinishedItems(); itemToWrite = GetNextItem() )
	//for( I=itemData.begin(); I!=itemData.end(); ++I )
	{
		//CItem *itemToWrite;
		//itemToWrite=(*I);
		if( itemToWrite==NULL )//|| I == itemData.end())
			continue;
		if( itemToWrite->ShouldSave() )
		{
			if( itemToWrite->GetObjType() == OT_MULTI )
			{
				CMultiObj *iMulti = static_cast< CMultiObj * >(itemToWrite);
				iMulti->Save( houseDestination );
			}
			else
				itemToWrite->Save( writeDestination );
		}
	}
}

//o--------------------------------------------------------------------------
//|	Function		-	LoadFromDisk( char *filename )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Loads in items/NPCs from disk in filename
//o--------------------------------------------------------------------------
void SubRegion::LoadFromDisk( std::ifstream& filename )
{
	char line[1024];
	while( !filename.eof() && !filename.fail() )
	{
		filename.getline( line, 1024 );
		UString sLine( line );
		sLine = sLine.removeComment().stripWhiteSpace();
		if( sLine.substr( 0, 1 ) == "[" )	// in a section
		{
			sLine = sLine.substr( 1, sLine.size() - 2 );
			sLine = sLine.upper().stripWhiteSpace();
			if( sLine == "CHARACTER" )
				LoadChar( filename );
			else if( sLine == "ITEM" )
				LoadItem( filename );
			else if( sLine == "HOUSE" )
				LoadMulti( filename );
		}
		else if( sLine == "---REGION---" )	// end of region
			return;
	}
}

//o--------------------------------------------------------------------------
//|	Function		-	cMapRegion constructor
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	This function is does basically what the name implies
//o--------------------------------------------------------------------------
cMapRegion::cMapRegion()
{
	for( UI08 i = 0; i < (UI08)UOMT_COUNT; ++i )
	{
		upperArrayX[i] = MapWidths[i] / MapColSize;
		upperArrayY[i] = MapHeights[i] / MapRowSize;
	}
}; //constructor

//o--------------------------------------------------------------------------
//|	Function		-	cMapRegion constructor
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	This function is does basically what the name implies
//o--------------------------------------------------------------------------
cMapRegion::~cMapRegion()
{
	
}; //destructor

//o--------------------------------------------------------------------------
//|	Function		-	SubRegion *GetCell( SI16 x, SI16 y )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Returns the subregion that x,y is in
//o--------------------------------------------------------------------------
SubRegion *cMapRegion::GetCell( SI16 x, SI16 y, UI08 worldNumber )
{
	SI16 targX = GetGridX( x ), targY = GetGridY( y );
	if( worldNumber >= NumberOfWorlds || targX < 0 || targY < 0 || targX >= upperArrayX[worldNumber] || targY >= upperArrayY[worldNumber] )
		return &overFlow;
	return &internalRegions[targX][targY][worldNumber];
}

//o--------------------------------------------------------------------------
//|	Function		-	bool AddItem( CItem *nItem )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adds nItem to the proper SubRegion
//o--------------------------------------------------------------------------
bool cMapRegion::AddItem( CItem *nItem )
{
	if( nItem == NULL )
		return false;
	UI08 worldNumber = nItem->WorldNumber();
	SI16 x = GetGridX( nItem->GetX() );
	SI16 y = GetGridY( nItem->GetY() );
	if( x < 0 || y < 0 || x >= upperArrayX[worldNumber] || y >= upperArrayY[worldNumber] )
	{
		overFlow.AddItem( nItem );
		return false;
	}
	return internalRegions[x][y][worldNumber].AddItem( nItem );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool RemoveItem( CItem *nItem )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes nItem from it's CURRENT SubRegion
//|					-	Do this before adjusting the location
//o--------------------------------------------------------------------------
bool cMapRegion::RemoveItem( CItem *nItem )
{
	if( nItem == NULL )
		return false;
	SI16 x = GetGridX( nItem->GetX() );
	SI16 y = GetGridY( nItem->GetY() );
	UI08 worldNumber = nItem->WorldNumber();
	if( x < 0 || y < 0 || x >= upperArrayX[worldNumber] || y >= upperArrayY[worldNumber] )
	{
		overFlow.RemoveItem( nItem );
		return false;
	}
	return internalRegions[x][y][worldNumber].RemoveItem( nItem );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool AddChar( CChar *toAdd )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Adds toAdd to the proper SubRegion
//o--------------------------------------------------------------------------
bool cMapRegion::AddChar( CChar *toAdd )
{
	if( toAdd == NULL )
		return false;
	UI08 worldNumber = toAdd->WorldNumber();
	SI16 x = GetGridX( toAdd->GetX() );
	SI16 y = GetGridY( toAdd->GetY() );
	if( x < 0 || y < 0 || x >= upperArrayX[worldNumber] || y >= upperArrayY[worldNumber] )
	{
		overFlow.AddChar( toAdd );
		return false;
	}
	return internalRegions[x][y][worldNumber].AddChar( toAdd );
}

//o--------------------------------------------------------------------------
//|	Function		-	bool RemoveChar( CChar *toRemove )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Removes toRemove from it's CURRENT SubRegion
//|					-	Do this before adjusting the location
//o--------------------------------------------------------------------------
bool cMapRegion::RemoveChar( CChar *toRemove )
{
	if( toRemove == NULL )
		return false;
	UI08 worldNumber = toRemove->WorldNumber();
	SI16 x = GetGridX( toRemove->GetX() );
	SI16 y = GetGridY( toRemove->GetY() );
	if( x < 0 || y < 0 || x >= upperArrayX[worldNumber] || y >= upperArrayY[worldNumber] )
	{
		overFlow.RemoveChar( toRemove );
		return false;
	}
	return internalRegions[x][y][worldNumber].RemoveChar( toRemove );
}

//o--------------------------------------------------------------------------
//|	Function		-	SI16 GetGridX( SI16 x )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Find the Column of SubRegion we want based on location
//o--------------------------------------------------------------------------
SI16 cMapRegion::GetGridX( SI16 x )
{
	return static_cast<SI16>(x / MapColSize);
}

//o--------------------------------------------------------------------------
//|	Function		-	SI16 GetGridY( SI16 y )
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Finds the Row of SubRegion we want based on location
//o--------------------------------------------------------------------------
SI16 cMapRegion::GetGridY( SI16 y )
{
	return static_cast<SI16>(y / MapRowSize);
}

//o--------------------------------------------------------------------------
//|	Function		-	Save()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Saves out each SubRegion
//o--------------------------------------------------------------------------
void cMapRegion::Save( void )
{
	const long AreaX				= UpperX / 8;	// we're storing 8x8 grid arrays together
	const long AreaY				= UpperY / 8;
	std::ofstream writeDestination, houseDestination;
	const int onePercent			= (int)((float)(AreaX*AreaY*8*8*NumberOfWorlds)/100.0f);
	const char blockDiscriminator[] = "\n\n---REGION---\n\n";
	const char binBlockDisc			= (char)0xFF;
	int count						= 0;
	const UI08 Version				= 3;
	int s_t							= getclock();

	Console << "Saving Character and Item Map Region data...   ";
	Console.TurnYellow();
	Console << "0%";

	std::string basePath = cwmWorldState->ServerData()->Directory( CSDDP_SHARED );
	std::string filename = basePath + "house.wsc";
	
	houseDestination.open( filename.c_str() );

	for( SI32 counter1 = 0; counter1 < AreaX; ++counter1 )	// move left->right
	{
		long baseX = counter1 * 8;
		for( SI32 counter2 = 0; counter2 < AreaY; ++counter2 )	// move up->down
		{
			long baseY	= counter2 * 8;								// calculate x grid offset
			filename	= basePath + UString::number( counter1 ) + "." + UString::number( counter2 ) + ".wsc";;	// let's name our file
			writeDestination.open( filename.c_str() );

			if( !writeDestination ) 
			{
				Console.Error( 1, "Failed to open %s for writing", filename.c_str() );
				continue;
			}

			for( UI08 xCnt = 0; xCnt < 8; ++xCnt )					// walk through each part of the 8x8 grid, left->right
			{
				for( UI08 yCnt = 0; yCnt < 8; ++yCnt )				// walk the row
				{
					for( UI08 wCnt = 0; wCnt < NumberOfWorlds; ++wCnt )
					{
						++count;
						if( count%onePercent == 0 )
						{
							if( count/onePercent <= 10 )
								Console << "\b\b" << (UI32)(count/onePercent) << "%";
							else if( count/onePercent <= 100 )
								Console << "\b\b\b" << (UI32)(count/onePercent) << "%";
						}
						internalRegions[baseX + xCnt][baseY + yCnt][wCnt].SaveToDisk( writeDestination, houseDestination );

						writeDestination << blockDiscriminator;
					}
				}
			}
			writeDestination.close();
		}
	}
	houseDestination.close();

	filename = basePath + "overflow.wsc";
	writeDestination.open( filename.c_str() );

	if( writeDestination.is_open() )
	{
		overFlow.SaveToDisk( writeDestination, writeDestination );
		writeDestination.close();
	}
	else
	{
		Console.Error( 1, "Failed to open %s for writing", filename.c_str() );
		return;
	}
	
	Console << "\b\b\b\b";
	Console.PrintDone();

	int e_t = getclock();
	Console.Print( "World saved in %.02fsec\n", ((float)(e_t-s_t))/1000.0f );
}

//o--------------------------------------------------------------------------
//|	Function		-	Load()
//|	Date			-	23 July, 2000
//|	Programmer		-	Abaddon
//|	Modified		-
//o--------------------------------------------------------------------------
//|	Purpose			-	Loads in each SubRegion
//o--------------------------------------------------------------------------
void cMapRegion::Load( void )
{
	const long AreaX		= UpperX / 8;	// we're storing 8x8 grid arrays together
	const long AreaY		= UpperY / 8;
	const int onePercent	= (int)((float)(AreaX*AreaY*8*8*NumberOfWorlds)/100.0f);
	int count				= 0;
	std::ifstream readDestination;
	UI08 Version			= 0;

	Console.TurnYellow();
	Console << "0%";
	int s_t					= getclock();
	std::string basePath	= cwmWorldState->ServerData()->Directory( CSDDP_SHARED );
	std::string filename;
	for( int counter1 = 0; counter1 < AreaX; ++counter1 )	// move left->right
	{
		long baseX = counter1 * 8;
		for( int counter2 = 0; counter2 < AreaY; ++counter2 )	// move up->down
		{
			long baseY	= counter2 * 8;								// calculate x grid offset
			filename	= basePath + UString::number( counter1 ) + "." + UString::number( counter2 ) + ".wsc";	// let's name our file
			readDestination.open( filename.c_str() );					// let's open it 

			if( readDestination.eof() || readDestination.fail() )
			{
				readDestination.close();
				readDestination.clear();
				continue;
			}
			for( UI08 xCnt = 0; xCnt < 8; ++xCnt )					// walk through each part of the 8x8 grid, left->right
			{
				long writeLong = baseX + xCnt;						// we know which column we're on
				for( UI08 yCnt = 0; yCnt < 8; ++yCnt )				// walk the row
				{
					for( UI08 wCnt = 0; wCnt < NumberOfWorlds; ++wCnt )
					{
						++count;
						if( count%onePercent == 0 )
						{
							if( count/onePercent <= 10 )
								Console << "\b\b" << (UI32)(count/onePercent) << "%";
							else if( count/onePercent <= 100 )
								Console << "\b\b\b" << (UI32)(count/onePercent) << "%";
						}
						internalRegions[writeLong][baseY + yCnt][wCnt].LoadFromDisk( readDestination );	// load it in as ASCII
					}
				}
			}
			readDestination.close();
		}
	}

	Console.TurnNormal();
	Console << "\b\b\b";
	Console.PrintDone();

	filename = basePath + "overflow.wsc";
	std::ifstream flowDestination( filename.c_str() );
	overFlow.LoadFromDisk( flowDestination );
	flowDestination.close();

	filename = basePath + "house.wsc";
	std::ifstream houseDestination( filename.c_str() );
	LoadHouseMulti( houseDestination );
	for( UI32 cCounter = 0; cCounter < cwmWorldState->GetCharCount(); ++cCounter )
	{
		if( !chars[cCounter].isFree() )
			chars[cCounter].PostLoadProcessing( cCounter );
	}
	for( UI32 iCounter = 0; iCounter < cwmWorldState->GetItemCount(); ++iCounter )
	{
		if( !items[iCounter].isFree() )
			items[iCounter].PostLoadProcessing( iCounter );
	}
	houseDestination.close();

	int e_t = getclock();
	Console.Print( "ASCII world loaded in %.02fsec\n", ((float)(e_t-s_t))/1000.0f );
}

int cMapRegion::GetGridIndex( SI16 x, SI16 y )
{
	SI16 trgX = GetGridX( x );
	SI16 trgY = GetGridY( y );
	return ( trgX + trgY * UpperX );	// x offset + y * num cols per row
}

SubRegion *cMapRegion::GetGrid( int xOffset, int yOffset, UI08 worldNumber )
{
	if( xOffset < 0 || xOffset >= upperArrayX[worldNumber] || yOffset < 0 || yOffset >= upperArrayY[worldNumber] )
		return &overFlow;
	return &internalRegions[xOffset][yOffset][worldNumber];
}

void cMapRegion::LoadHouseMulti( std::ifstream &houseDestination )
{
	char line[1024];
	while( !houseDestination.eof() && !houseDestination.fail() )
	{
		houseDestination.getline( line, 1024 );
		UString sLine( line );
		sLine = sLine.removeComment().stripWhiteSpace();
		if( sLine.substr( 0, 1 ) == "[" )	// in a section
		{
			sLine = sLine.substr( 1, sLine.size() - 2 );
			sLine = sLine.upper().stripWhiteSpace();
			if( sLine == "CHARACTER" )
				LoadChar( houseDestination );
			else if( sLine == "ITEM" )
				LoadItem( houseDestination );
			else if( sLine == "HOUSE" )
				LoadMulti( houseDestination );
		}
		else if( sLine == "---REGION---" )	// end of region
			break;
	}
}

bool cMapRegion::Add( cBaseObject *toAdd )
{
	if( toAdd == NULL )
		return false;
	if( toAdd->GetObjType() == OT_CHAR )
		return AddChar( static_cast< CChar *>(toAdd) );
	else
		return AddItem( (CItem *)(toAdd) );
}
bool cMapRegion::Remove( cBaseObject *toRemove )
{
	if( toRemove == NULL )
		return false;
	if( toRemove->GetObjType() == OT_CHAR )
		return RemoveChar( static_cast< CChar *>(toRemove) );
	else
		return RemoveItem( (CItem *)(toRemove) );
}
