// This file houses things like broadcast, textflags, and other speech related functions

#include "uox3.h"
#include "speech.h"
#include "cVersionClass.h"
#include "boats.h"
#include "cGuild.h"
#include "msgboard.h"
#include "townregion.h"
#include "cRaces.h"
#include "targeting.h"
#include "commands.h"
#include "skills.h"
#include "trigger.h"
#include "cScript.h"
#include "cEffects.h"
#include "packets.h"
#include "classes.h"
#include "cSpeech.h"
#include "regions.h"

#ifndef va_start
	#include <cstdarg>
//	using namespace std;
#endif

std::map< std::string, UnicodeTypes > codeLookup;

void InitializeLookup( void )
{
	for( int i = (int)ZERO; i < (int)TOTAL_LANGUAGES; ++i )
		codeLookup[LanguageCodes[(UnicodeTypes)i]] = (UnicodeTypes)i;
}

UnicodeTypes FindLanguage( char *lang )
{
	std::map< std::string, UnicodeTypes >::iterator p = codeLookup.find( lang );
	if( p != codeLookup.end() )
		return p->second;
	else
		return TOTAL_LANGUAGES;
}
void UpdateLanguage( cSocket *s )
{
	if( s == NULL )
		return;
	char langCode[4];

	langCode[0] = s->GetByte( 8 );
	langCode[1] = s->GetByte( 9 );
	langCode[2] = s->GetByte( 10 );
	langCode[3] = 0;

	strupr( langCode );	// Convert to uppercase
	UnicodeTypes cLang = s->Language();
	if( LanguageCodes[cLang] != langCode )
	{
		UnicodeTypes newLang = FindLanguage( langCode );
		if( newLang == TOTAL_LANGUAGES )
			Console.Error( 0, "Unknown language type \"%s\".  PLEASE report this on www.sourceforge.net/projects/uox3 in the bugtracker!", langCode );
		else
			s->Language( newLang );
	}
}

//o-------------------------------------------------------------------------
//|	Function	-	consolebroadcast( std::string txt )
//|	Date		-	Unknown
//|	Programmer	-	UOX DevTeam
//|	Modification-	EviLDeD(February 27, 2000)
//o-------------------------------------------------------------------------
//|	Purpose			-	This function was adapted to be used with the new code
//|						in the console thread that allows text to be entered on
//|						the console and it be shipped out to all logged in players.
//o-------------------------------------------------------------------------
void cSpeech::consolebroadcast( std::string txt )
{
	if( !txt.empty() )
	{
		std::string temp	= "System: " + txt;
		CSpeechEntry *toAdd	= SpeechSys->Add();
		toAdd->Speech( temp.c_str() );
		toAdd->Font( FNT_NORMAL );
		toAdd->Speaker( INVALIDSERIAL );
		toAdd->SpokenTo( INVALIDSERIAL );
		toAdd->Type( SYSTEM );
		toAdd->At( cwmWorldState->GetUICurrentTime() );
		toAdd->TargType( SPTRG_BROADCASTPC );
		toAdd->Colour( 0x084D );
		toAdd->Font( FNT_BOLD );
		toAdd->SpeakerName( "" );
	}
}

void cSpeech::sysbroadcast( std::string txt ) // System broadcast in bold text
{
	if( !txt.empty() )
	{
		CSpeechEntry *toAdd = SpeechSys->Add();
		toAdd->Speech( txt.c_str() );
		toAdd->Font( FNT_NORMAL );
		toAdd->Speaker( INVALIDSERIAL );
		toAdd->SpokenTo( INVALIDSERIAL );
		toAdd->Type( SYSTEM );
		toAdd->At( cwmWorldState->GetUICurrentTime() );
		toAdd->TargType( SPTRG_BROADCASTPC );
		toAdd->Colour( 0x084D );
		toAdd->Font( FNT_BOLD );
		toAdd->SpeakerName( "System: " );
	}
}

void cSpeech::broadcast( cSocket *mSock ) // GM Broadcast (Done if a GM yells something)
{
	CChar *mChar = mSock->CurrcharObj();
	CPSpeech toSend( *mChar );
	toSend.Type( 1 );
	toSend.GrabSpeech( mSock, mChar );

	if( !mChar->isUnicode() )
		toSend.Message( (char *)&(mSock->Buffer()[8]) );
	else
	{
		char nonuni[512]; 
		for( UI16 i = 13; i < mSock->GetWord( 1 ); i += 2 )
			nonuni[(i-13)/2] = mSock->GetByte( i );
		toSend.Message( &nonuni[0] );
	}

	Network->PushConn();
	for( cSocket *bSock = Network->FirstSocket(); !Network->FinishedSockets(); bSock = Network->NextSocket() )
	{
		bSock->Send( &toSend );
	}
	Network->PopConn();
}

/*
Unicode speech format
byte=char, short=char[2], int=char[4], wchar=char[2]=unicode character

  Message Sent By Client:
  0xAD - Unicode Speech Request
  UI08 cmd (0xAD)
  short msgsize 1,2
  byte type (0=say, 2=emote, 8=whisper, 9=yell) 3
  short color 4,5
  short font 6,7
  UI08[4] lang (null terminated, "enu " for US english.) 8,9,10,11
  wchar[?] text (null terminated, ?=(msgsize-12)/2) 13
  
	Message Sent By Server:
	Unicode Speech message(Variable # of bytes) 
	BYTE cmd											0
	BYTE[2] blockSize									1-2
	BYTE[4] ID											3-6
	BYTE[2] Model										7-8
	BYTE Type											9
	BYTE[2] Color										10-11
	BYTE[2] Font										12-13
	BYTE[4] Language									14-17 (3 byte unicode language, mirrored from client)
	BYTE[30] Name										18-47
	BYTE[?][2] Msg - Null Terminated (blockSize - 48)	48+?
*/

void cSpeech::generictalking( CPITalkRequest *pckTalk ) // PC speech
{
	cSocket *mSock	= pckTalk->GetSocket();
	CChar *mChar	= mSock->CurrcharObj();
	if( mChar == NULL )
		return;

	char *asciiText		= pckTalk->Text();
	cScript *myScript	= Trigger->GetScript( mChar->GetScriptTrigger() );
	if( myScript != NULL )
	{
		if( !myScript->OnTalk( mChar, asciiText ) )
			return;
	}
	
	if( ( asciiText[0] == cwmWorldState->ServerData()->ServerCommandPrefix() ) || ( ( asciiText[0] == '.' ) && ( asciiText[1] != '.' ) ) ) 
		Commands->Command( pckTalk );
	else
	{
		if( mChar->GetHidden() && !mChar->IsPermHidden() )
			mChar->ExposeToView();
		
		bool resp = response( pckTalk );
		if( resp )
			return;  //Vendor responded already
		if( pckTalk->Type() == 0x09 && mChar->CanBroadcast() )
			broadcast( mSock );
		else
		{
			UString text( asciiText );
			text = text.upper();
			if( text == Dictionary->GetEntry( 1614, mSock->Language() ) )
				GuildSys->Resign( mSock );
			if( text.find( "DEVTEAM033070" ) != std::string::npos )
			{
				std::string temp3 = "RBuild: " + cVersionClass::GetRealBuild() + " PBuild: " + cVersionClass::GetBuild() + " --> Version: " + cVersionClass::GetVersion();
				mSock->sysmessage( temp3.c_str() );
			}
		
			if( pckTalk->Type() == 0 )
				mChar->SetSayColour( pckTalk->TextColour() );
			if( pckTalk->Type() == 2 )
				mChar->SetEmoteColour( pckTalk->TextColour() );
			if( cwmWorldState->ServerData()->ServerConsoleLogStatus() == 2 ) //Logging -- Zippy
			{
				char temp[1024], temp2[512];
				sprintf( temp2, "%s.log", mChar->GetName().c_str() );
				sprintf( temp, "%s [%x %x %x %x] [%i]: %s\n", mChar->GetName().c_str(), mChar->GetSerial( 1 ), mChar->GetSerial( 2 ), mChar->GetSerial( 3 ), mChar->GetSerial( 4 ), mChar->GetAccount().wAccountIndex, asciiText );
				Console.Log( temp, temp2 );
			}
			
			std::string search1	= Dictionary->GetEntry( 1660 );
			bool response1		= (text.find( search1 ) != std::string::npos);
			if( response1 )
				callGuards( mChar );
			
            Boats->BoatSpeech( mSock, asciiText );//Boats
			HouseSpeech( mSock, asciiText );		

			cPUOXBuffer *txtToSend			= NULL;
			cPUOXBuffer *ghostedText		= NULL;
			CPUnicodeSpeech *uniTxtToSend	= NULL;
			CPUnicodeSpeech *uniGhostedText = NULL;
			CPSpeech *asciiTxtToSend		= NULL;
			CPSpeech *asciiGhostedText		= NULL;

			if( pckTalk->IsUnicode() )
			{
				uniTxtToSend = new CPUnicodeSpeech();
				uniTxtToSend->Object( *((CPITalkRequestUnicode *)pckTalk) );
				uniTxtToSend->Object( *mChar );
				txtToSend = uniTxtToSend;

				uniGhostedText = new CPUnicodeSpeech();
				(*uniGhostedText) = (*uniTxtToSend);
				uniGhostedText->GhostIt( 0 );

				ghostedText = uniGhostedText;
			}
			else
			{
				asciiTxtToSend = new CPSpeech();
				asciiTxtToSend->Object( *((CPITalkRequestAscii *)pckTalk) );
				asciiTxtToSend->Object( *mChar );
				txtToSend = asciiTxtToSend;

				asciiGhostedText = new CPSpeech();
				(*asciiGhostedText) = (*asciiTxtToSend);
				asciiGhostedText->GhostIt( 0 );

				ghostedText = asciiGhostedText;
			}
			mSock->Send( txtToSend );

			Network->PushConn();
			for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
			{
				CChar *tChar = tSock->CurrcharObj();
				if( mChar != tChar && charInRange( tChar, mChar ) )
				{
					if( !tChar->IsGM() && !tChar->IsCounselor() && tChar->GetTimer( tPC_SPIRITSPEAK ) == 0 )// GM/Counselors can see ghosts talking always Seers?
					{
						if( mChar->IsDead() && !tChar->IsDead() )  // Ghost can talk normally to other ghosts
							tSock->Send( ghostedText );
						else
							tSock->Send( txtToSend );
					}
					else if( tChar->GetRace() != mChar->GetRace() && !tChar->IsGM() )
					{
						if( Skills->CheckSkill( tChar, SPIRITSPEAK, Races->LanguageMin( mChar->GetRace() ), 1000 ) != 1 )
							tSock->Send( ghostedText );
						else
							tSock->Send( txtToSend );
					}
					else
						tSock->Send( txtToSend );
				}
			}
			Network->PopConn();

			delete uniTxtToSend;
			delete asciiTxtToSend;
			delete uniGhostedText;
			delete asciiGhostedText;

			if( mChar->IsDead() )  // this makes it so npcs do not respond to dead people
				return;
			checkForVendorResponse( pckTalk, mChar );			
		}
	}
}

//o--------------------------------------------------------------------------o
//|	Function/Class	-	void cSpeech::checkForVendorResponse( cSocket *mSock, CChar *mChar )
//|	Date			-	23 June, 2003
//|	Developer(s)	-	Maarc
//|	Company/Team	-	UOX3 DevTeam
//|	Status			-	Complete
//o--------------------------------------------------------------------------o
//|	Description		-	Moved into its own function, this was common to both
//|						talking and unicodetalking
//|						Makes talking/unicodetalking very similar
//o--------------------------------------------------------------------------o
//|	Returns			-
//o--------------------------------------------------------------------------o	
void cSpeech::checkForVendorResponse( CPITalkRequest *pckTalk, CChar *mChar )
{
	cSocket *mSock		= pckTalk->GetSocket();
	int i				= 0;
	bool found			= false;
	SI16 x1				= mChar->GetX();
	SI16 y1				= mChar->GetY();
	SI08 z1				= mChar->GetZ();
	SI16 xOffset		= MapRegion->GetGridX( x1 );
	SI16 yOffset		= MapRegion->GetGridY( y1 );
	UI08 worldNumber	= mChar->WorldNumber();

	for( SI08 counter1 = -1; counter1 <= 1; ++counter1 )
	{
		for( SI08 counter2 = -1; counter2 <= 1; ++counter2 )
		{
			SubRegion *MapArea = MapRegion->GetGrid( xOffset + counter1, yOffset + counter2, worldNumber );
			if( MapArea == NULL )	// no valid region
				continue;
			MapArea->PushChar();
			for( CChar *tempChar = MapArea->FirstChar(); !MapArea->FinishedChars() && !found; tempChar = MapArea->GetNextChar() )
			{
				if( tempChar == NULL )
					continue;
				if( tempChar != mChar && tempChar->IsNpc() )
				{
					SI16 x2 = tempChar->GetX();
					SI16 y2 = tempChar->GetY();
					SI08 z2 = tempChar->GetZ();
					if( abs( x1 - x2 ) <= 5 && abs( y1 - y2 ) <= 5 && abs( z1 - z2 ) <= 9 ) 
					{
						found = true;		// we never actually use the person we found!
						break;
					}
				}
			}
			MapArea->PopChar();
			if( found )
				break;
		}
	}
	if( found )
		responsevendor( pckTalk );
}

void cSpeech::tellmessage( cSocket *i, cSocket *s, std::string txt )
{
	if( i == NULL || s == NULL || txt.empty() )
		return;
	CChar *mChar		= i->CurrcharObj();
	CChar *tChar		= s->CurrcharObj();
	std::string temp	= mChar->GetName() + " tells " + tChar->GetName() + ": " + txt;
	CSpeechEntry *toAdd	= SpeechSys->Add();

	toAdd->Font( (FontType)mChar->GetFontType() );

	toAdd->Speech( temp.c_str() );
	toAdd->Speaker( mChar->GetSerial() );
	toAdd->SpokenTo( tChar->GetSerial() );
	toAdd->Type( TALK );
	toAdd->At( cwmWorldState->GetUICurrentTime() );
	toAdd->TargType( SPTRG_INDIVIDUAL );
	toAdd->Colour( mChar->GetSayColour() );
}

CSpeechQueue::CSpeechQueue( void ) : pollTime( 100 ), runAsThread( false )
{
	InitializeLookup();
}
CSpeechQueue::~CSpeechQueue()
{
	speechList.resize( 0 );
}

void CSpeechQueue::SayIt( SPEECHLIST_ITERATOR toSay )
{
	CPacketSpeech toSend	= (*toSay);
	CChar *thisChar			= NULL;
	CItem *thisItem			= NULL;
	cBaseObject *thisObj	= NULL;
	if( toSay->Speaker() > BASEITEMSERIAL )
	{
		thisItem	= calcItemObjFromSer( toSay->Speaker() );
		thisObj		= thisItem;
	}
	else
	{
		thisChar	= calcCharObjFromSer( toSay->Speaker() );
		thisObj		= thisChar;
	}
	CChar *sChar	= NULL;
	cSocket *mSock	= NULL;
	switch( toSay->TargType() )
	{
		case SPTRG_INDIVIDUAL:		// aimed at individual person
			sChar = calcCharObjFromSer( toSay->SpokenTo() );
			if( sChar != NULL )
			{
				mSock = calcSocketObjFromChar( sChar );
				if( mSock != NULL )
					mSock->Send( &toSend );
			}
			break;
		case SPTRG_PCNPC:				// all NPCs and PCs in range
		case SPTRG_PCS:					// all PCs in range
			if( thisObj == NULL )
				break;
			if( thisItem != NULL && thisItem->GetCont() != NULL )	// not on ground, can't guarantee speech
				break;
			Network->PushConn();
			for( mSock = Network->FirstSocket(); !Network->FinishedSockets(); mSock = Network->NextSocket() )
			{
				if( mSock != NULL )
				{
					CChar *mChar = mSock->CurrcharObj();
					if( mChar != NULL && objInRange( mChar, thisObj, mSock->Range() ) )
					{
						if( mChar->GetCommandLevel() >= toSay->CmdLevel() )
							mSock->Send( &toSend );
					}
				}
			}
			Network->PopConn();
			break;
		case SPTRG_BROADCASTPC:			// ALL PCs everywhere + NPCs in range
		case SPTRG_BROADCASTALL:
			Network->PushConn();
			for( mSock = Network->FirstSocket(); !Network->FinishedSockets(); mSock = Network->NextSocket() )
			{
				if( mSock != NULL )
				{
					CChar *mChar = mSock->CurrcharObj();
					if( mChar != NULL )
					{
						if( mChar->GetCommandLevel() >= toSay->CmdLevel() )
							mSock->Send( &toSend );
					}
				}
			}
			Network->PopConn();
			break;
	};
}

//okay.. we cannot use iterators for this, because any time something is erased from the vector, the iterator will no onger be valid... which could cause crashes and will cause weird things to happen.
bool CSpeechQueue::InternalPoll( void )		// Send out any pending speech, returning true if entries were sent
{
	bool retVal = false;
	SPEECHLIST toKeep;

	// Iterators work just great, just make sure to test them for valididty, and stop using []access vectors wont sort.
	SPEECHLIST_ITERATOR I;
	for( I = speechList.begin(); I != speechList.end(); ++I )
	{
		CSpeechEntry tempSpeechList = (*I);
		if( tempSpeechList.At() == -1 || static_cast<UI32>(tempSpeechList.At()) <= cwmWorldState->GetUICurrentTime() )
		{
			retVal = true;
			// 1/13/2003 - Maarc - Quick fix for more strict gcc 3.2 compliance.
			SayIt( (SPEECHLIST_ITERATOR) (&tempSpeechList) );
		}
		else	// We want to keep the entries that have been unsaid
			toKeep.push_back( tempSpeechList );
	}
	speechList.clear();
	// Copy back the unsaid speech for checking next time around
	for( I = toKeep.begin(); I != toKeep.end(); ++I )
	{
		CSpeechEntry toCopy = (*I);
		speechList.push_back( toCopy );
	}
	toKeep.clear();
	return retVal;
}

bool CSpeechQueue::Poll( void )
{
	if( RunAsThread() )
	{
		while( RunAsThread() )
		{
			InternalPoll();
			UOXSleep( pollTime );	// so that it's never 0 (and number of milliseconds)
		}
		return true;
	}
	else
		return InternalPoll();
}

CSpeechEntry * CSpeechQueue::Add( void )		// Make space in queue, and return pointer to new entry
{
	size_t iSize = speechList.size();
	speechList.resize( iSize + 1 );
	return &speechList[iSize];
}

int CSpeechQueue::PollTime( void ) const
{
	return pollTime;
}

void CSpeechQueue::PollTime( int value )
{
	pollTime = value;
}

bool CSpeechQueue::RunAsThread( void ) const
{
	return runAsThread;
}

void CSpeechQueue::RunAsThread( bool value )
{
	runAsThread = value;
}

void CSpeechQueue::DumpInFile( void )
{
	FILE *toWriteTo = fopen( "speechdump.txt", "w" );
	if( toWriteTo != NULL )
	{
		for( SPEECHLIST_ITERATOR I = speechList.begin(); I != speechList.end(); ++I )
		{
			fprintf( toWriteTo, "Time: %i\nColour: %i\nFont: %i\nLanguage: %i\nSpeech: %s\nSpeaker: %s\n\n", I->At(), I->Colour(), I->Font(), I->Language(), I->Speech(), I->SpeakerName() );
		}
		fclose( toWriteTo );
	}
	else
	{
		Console.Error( 1, "Failed to open speechdump.txt for writing" );
	}
}

void DictionarySpeech( cBaseObject &speaker, cBaseObject *spokenTo, SI32 spEntry, bool antiSpam, SpeechType sType, COLOUR sColour = 0x005A, FontType fType = FNT_NORMAL, SpeechTarget spTrg = SPTRG_INDIVIDUAL, UnicodeTypes spLang = UT_ENU, ... )
{
	if( antiSpam && speaker.GetObjType() == OT_CHAR )
	{
		if( static_cast< CChar & >(speaker).GetTimer( tCHAR_ANTISPAM ) < cwmWorldState->GetUICurrentTime() )
			static_cast< CChar & >(speaker).SetTimer( tCHAR_ANTISPAM, BuildTimeValue( 10 ) ); 
		else
			return;
	}
	std::string txtEntry = Dictionary->GetEntry( spEntry, spLang );
	if( txtEntry.empty() )
		return;
	char msg[512];

	va_list argptr;
	va_start( argptr, spLang );
	vsprintf( msg, txtEntry.c_str(), argptr );
	va_end( argptr );

	CSpeechEntry *toAdd = SpeechSys->Add();
	toAdd->Language( spLang );
	toAdd->Font( fType );
	if( spokenTo != NULL )
		toAdd->SpokenTo( spokenTo->GetSerial() );
	toAdd->Speech( msg );
	toAdd->Speaker( speaker.GetSerial() );
	toAdd->Type( sType );
	toAdd->At( cwmWorldState->GetUICurrentTime() );
	toAdd->TargType( spTrg );
	toAdd->Colour( sColour );
}

bool cSpeech::response( CPITalkRequest *pckTalk )
{
	int i, j;
	int skill		= -1;
	cSocket *mSock	= pckTalk->GetSocket();
	char *response1 = NULL;
	char *response2 = NULL;
	char *response3 = NULL;

	char *asciiText = pckTalk->Text();
	char upperAscii[256];

	char search1[50];
	char search2[50];
	char search3[50];
	
	char temp[512];
	char temp2[512];

	CChar *mChar = mSock->CurrcharObj();

	strcpy( upperAscii, asciiText );
	strupr( upperAscii );
	strcpy( search1, " KILLS" ); 
	response1 = (strstr( upperAscii, search1 ) );
	if( response1 && isOnline( mChar ) && !mChar->IsDead() )
	{
		i = mChar->GetKills() ;
		if( i == 0 )
			mSock->sysmessage( 1288 );
		else if( i > cwmWorldState->ServerData()->RepMaxKills() )
			mSock->sysmessage( 1289, i );
		else
		{
			mSock->sysmessage( 1290, i );
			return true;
		}
	}
	
	SI16 x				= mChar->GetX();
	SI16 y				= mChar->GetY();
	bool retval			= false;
	CHARACTER FTarg		= 0;
	UI08 worldNumber	= mChar->WorldNumber();
	SI16 xOffset		= MapRegion->GetGridX( x );
	SI16 yOffset		= MapRegion->GetGridY( y );
	SI08 dx, dy;
	
	for( dx = -1; dx <= 1; ++dx )
	{
		for( dy = -1; dy <= 1; ++dy )
		{
			SubRegion *CellResponse = MapRegion->GetGrid( xOffset+dx, yOffset+dy, worldNumber );
			if( CellResponse == NULL )
				return false;

			CellResponse->PushChar();
			for( CChar *Npc = CellResponse->FirstChar(); !CellResponse->FinishedChars(); Npc = CellResponse->GetNextChar() )
			{
				SI16 diffX = mChar->GetX() - Npc->GetX();
				SI16 diffY = mChar->GetY() - Npc->GetY();
				SI08 diffZ = mChar->GetZ() - Npc->GetZ();
				if( Npc == NULL || Npc == mChar )
					continue;
				if( abs( diffX ) > 30 || abs( diffY ) > 30 || abs( diffZ ) > 30 )
					continue;
				if( abs( diffX ) <= 5 && abs( diffY ) <= 5 && abs( diffZ ) <= 9 )	// originally 2, 2, 5
				{
					UI16 speechTrig = Npc->GetScriptTrigger();
					cScript *toExecute = Trigger->GetScript( speechTrig );
					if( toExecute != NULL )
					{
//|						-1	=> No such function or bad call
//|						0	=> Let other NPCs and PCs see it
//|						1	=> Let other PCs see it
//|						2	=> Let no one else see it
						SI08 rVal = -1;
						if( Npc->IsDisabled() )
							Npc->talkAll( 1291, false );
						else if( !mChar->IsDead() )
							rVal = toExecute->OnSpeech( upperAscii, mChar, Npc );
						switch( rVal )
						{
							case 1:		// No other NPCs to see it, but PCs should
								CellResponse->PopChar();
								return false;
							case 2:		// no one else to see it
								CellResponse->PopChar();
								return true;
							case 0:		// Other NPCs and PCs to see it
							case -1:	// no function, so do nothing... NOT handled!
							default:
								break;
						}
					}
				}

				// Dupois - Added Dec 20, 1999
				// Escort text matches
				sprintf( search1, Dictionary->GetEntry( 1292 ).c_str() );
				sprintf( search2, Dictionary->GetEntry( 1293 ).c_str() );
				response1 = ( strstr( upperAscii, search1 ) );
				response2 = ( strstr( upperAscii, search2 ) );

				// If either of the above responses match, then find the NPC we are talking to
				if( response1 || response2 )
				{
					// If the PC is dead then break out, The dead cannot accept quests
					if( mChar->IsDead() ) 
					{
						CellResponse->PopChar();
						return false;
					}
					if( Npc->IsNpc() )	// If this is an NPC then check it 
					{
						// I WILL TAKE THEE
						// If this is a request for hire and the PC is within range of the NPC and the NPC is waiting for an ESCORT
						if( objInRange( Npc, mChar, DIST_INRANGE ) )
						{
							if( response1 && Npc->GetQuestType() == ESCORTQUEST  && Npc->GetFTarg() == INVALIDSERIAL )
							{
								Npc->SetFTarg( calcCharFromSer( mChar->GetSerial() ) );			// Set the NPC to follow the PC
								Npc->SetNpcWander( 1 );			// Set the NPC to wander freely
								Npc->SetTimer( tNPC_SUMMONTIME, BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->EscortActiveExpire() )) );			// Set the expire time if nobody excepts the quest

								// Send out the rant about accepting the escort
								Npc->talkAll( 1294, false, region[Npc->GetQuestDestRegion()]->GetName().c_str() );

								// Remove post from message board (Mark for deletion only - will be cleaned during cleanup)
								MsgBoardQuestEscortRemovePost( Npc );
								// Return 1 so that we indicate that we handled the message
								CellResponse->PopChar();
								return true;
							}
							if( response1 && Npc->GetQuestType() == ESCORTQUEST )
								response2 = response1;	// If the current NPC already has an ftarg then respond to query for quest

							// DESTINATION
							// If this is a request to find out where a NPC wants to go and the PC is within range of the NPC and the NPC is waiting for an ESCORT
							if( response2 && Npc->GetQuestType() == ESCORTQUEST )
							{
								if( Npc->GetFTarg() == calcCharFromSer( mChar->GetSerial() ) )
									Npc->talkAll( 1295, false, region[Npc->GetQuestDestRegion()]->GetName().c_str() );	// Send out the rant about accepting the escort
								else if( Npc->GetFTarg() == INVALIDSERIAL )  // If nobody has been accepted for the quest yet
									Npc->talkAll( 1296, false, region[Npc->GetQuestDestRegion()]->GetName().c_str() );	// Send out the rant about accepting the escort
								else // The must be enroute
									Npc->talkAll( 1297, false, region[Npc->GetQuestDestRegion()]->GetName().c_str(), chars[Npc->GetFTarg()].GetName().c_str() );	// Send out a message saying we are already being escorted
								// Return success ( we handled the message )
								CellResponse->PopChar();
								return true;
							}
						}
					}
				}

				if( Npc->GetNPCAiType() == aiBANKER && !mChar->IsDead() && objInRange( Npc, mChar, DIST_INRANGE ) )	// if he's a banker and we're close!
				{
					strcpy( search1, "BANK" );
					strcpy( search2, "BALANCE" );
					response1 = strstr( upperAscii, search1 );
					response2 = strstr( upperAscii, search2 );
					if( response1 )
						openBank( mSock, mChar );
					if( response2 )	
					{
						UI32 goldCount = GetBankCount( mChar, 0x0EED );
						Npc->talk( mSock, 1298, true, mChar->GetName().c_str(), goldCount );
					}
				}

				// This training code is by Anthracks (fred1117@tiac.net) and really psychotic
				// if it doesn't work or you can't decipher it, you know who to blame
				strcpy( search1, Dictionary->GetEntry( 1299 ).c_str() );
				strcpy( search2, Dictionary->GetEntry( 1300 ).c_str() );
				strcpy( search3, Dictionary->GetEntry( 1301 ).c_str() );
				response1 = strstr( upperAscii, search1 );
				response2 = strstr( upperAscii, search2 );
				response3 = strstr( upperAscii, search3 );
				if( cwmWorldState->ServerData()->NPCTrainingStatus() && (response1 || response2 || response3) ) //if the player wants to train
				{
					// Stop the NPC from moving for a minute while talking with the player
					Npc->SetTimer( tNPC_MOVETIME, BuildTimeValue( 60 ) );
					mChar->SetTrainer( INVALIDSERIAL ); //this is to prevent errors when a player says "train <skill>" then doesn't pay the npc
					skill = -1;
					for( i = 0; i < ALLSKILLS; ++i )
					{
						if( strstr( upperAscii, skillname[i] ) )
						{
							skill = i;  //Leviathan fix
							break;
						}
					}
					if( skill == -1 ) // Didn't ask to be trained in a specific skill - Leviathan fix
					{
						if( mChar->GetTrainer() == INVALIDSERIAL ) //not being trained, asking what skills they can train in
						{
							if( Npc->IsNpc() )
							{
								if( objInRange( Npc, mChar, DIST_NEARBY ) && Npc->isHuman() )
								{
									if( !Npc->CanTrain() )
									{
										Npc->talk( mSock, 1302, false );
										CellResponse->PopChar();
										return true;
									}
									Npc->SetTrainingPlayerIn( 255 ); // Like above, this is to prevent  errors when a player says "train <skill>" then doesn't pay the npc
									strcpy( temp, Dictionary->GetEntry( 1303 ).c_str() );
									for( j = 0; j < ALLSKILLS; ++j )
									{
										if( Npc->GetBaseSkill( j ) > 10 )
										{
											sprintf( temp2, "%s, ", strlwr( skillname[j] ) );
											strupr( skillname[j] ); // I found out strlwr changes the actual  string permanently, so this undoes that
											if( !y ) 
												temp2[0] = toupper( temp2[0] ); // If it's the first skill,  capitalize it.
											strcat( temp, temp2 );
											++y;
										}
									}
									if( y )
									{
										temp[strlen( temp ) - 2] = '.'; // Make last character a . not a ,  just to look nicer
										Npc->talk( mSock, temp, false );
									}
									else
										Npc->talk( mSock, 1302, false );

									CellResponse->PopChar();
									return true;
								}
							}
						} 
					}
					else // They do want to learn a specific skill
					{
						if( Npc->IsNpc() )
						{
							if( objInRange( Npc, mChar, DIST_NEARBY ) && ( Npc->GetID() == 0x0190 || Npc->GetID() == 0x0191 ) )
							{
								if( !Npc->CanTrain() )
								{
									Npc->talk( mSock, 1302, false );
									CellResponse->PopChar();
									return true;
								}
								if( Npc->GetBaseSkill( skill ) > 10 )
								{
									sprintf( temp, Dictionary->GetEntry( 1304 ).c_str(), strlwr( skillname[skill] ) );
									strupr( skillname[skill] ); // I found out strlwr changes the actual string permanently, so this undoes that
									if( mChar->GetBaseSkill( skill ) >= 250 )
										strcat( temp, Dictionary->GetEntry( 1305 ).c_str() );
									else
									{
										if( Npc->GetBaseSkill( skill ) <= 250)
											sprintf( temp2, Dictionary->GetEntry( 1306 ).c_str(),(int)( Npc->GetBaseSkill( skill ) / 2 / 10 ),(int)( Npc->GetBaseSkill( skill ) / 2 ) - mChar->GetBaseSkill( skill ) );
										else
											sprintf( temp2, Dictionary->GetEntry( 1306 ).c_str(), 25, 250 - mChar->GetBaseSkill( skill ) );
										strcat( temp, temp2 );
										mChar->SetTrainer( Npc->GetSerial() );
										Npc->SetTrainingPlayerIn( skill );
									}
									Npc->talk( mSock, temp, false );
									CellResponse->PopChar();
									return true;
								}
								else
								{
									Npc->talk( mSock, 1307, false ); 
									CellResponse->PopChar();
									return true; 
								}
							}
						}
					}
				}

				if( ( Npc->GetOwnerObj() != NULL ) && ( Npc->GetNPCAiType() != aiPLAYERVENDOR ) && 
				( ( abs( diffX ) <= 7 ) && ( abs( diffY ) <= 7 ) && ( abs( diffZ ) <= 20 ) ) )
				{
					bool isFriend = Npcs->checkPetFriend( mChar, Npc );
					if( Npc->GetOwnerObj() == mChar || isFriend || mChar->IsGM() )
					{
						char search4[50];
						strcpy( search4, "ALL" );
						strcpy( search3, Npc->GetName().c_str() );
						strupr( search3 );
						response3		= strstr( upperAscii, search3 );
						char *response4 = strstr( upperAscii, search4 );
						if( response3 || response4 ) // If petname or ALL is in the line
						{
							strcpy( search1, Dictionary->GetEntry( 1308 ).c_str() ); // Follow
							strcpy( search2, Dictionary->GetEntry( 1309 ).c_str() ); // Me
							response1 = strstr( upperAscii, search1 );
							response2 = strstr( upperAscii, search2 );
							if( response1 )
							{
								Npcs->stopPetGuarding( Npc );
								mChar->SetGuarded( false );
								if( response2 ) //if me is in
								{
									Npc->SetFTarg( calcCharFromSer( mChar->GetSerial() ) );
									Npc->SetNpcWander( 1 );
									Effects->playMonsterSound( Npc, Npc->GetID(), SND_STARTATTACK );
									retval = true;
								}
								else
								{	// add pet follow code here, look for "all" command
									if( ( FTarg == 0 ) || ( FTarg == INVALIDSERIAL ) )
									{
										mSock->AddID( Npc->GetSerial() );
										mSock->target( 0, TARGET_FOLLOW, 1310 );
										FTarg = Npc->GetFTarg();
									}
									else 
									{
										Npc->SetFTarg( FTarg );
										Npc->SetNpcWander( 1 );
										Effects->playMonsterSound( Npc, Npc->GetID(), SND_STARTATTACK );
									}
									retval = true;
								}
								// check if "all" was said... if not => return
								if( !response4 )
								{
									CellResponse->PopChar();
									return true;
								}
							}

							strcpy( search1, Dictionary->GetEntry( 1311 ).c_str() ); // Kill
							strcpy( search2, Dictionary->GetEntry( 1312 ).c_str() ); // Attack
							response1 = strstr( upperAscii, search1 );
							response2 = strstr( upperAscii, search2 );
							if( response1 || response2 )
							{
								Npcs->stopPetGuarding( Npc );
								if( !isFriend )
								{
									mSock->AddID( Npc->GetSerial() );
									//pet kill code here
									mSock->target( 0, TARGET_ATTACK, 1313 );
									CellResponse->PopChar();
									return true;
								}
							}

							strcpy( search1, Dictionary->GetEntry( 1314 ).c_str() ); // Fetch
							strcpy( search2, Dictionary->GetEntry( 1315 ).c_str() ); // Get
							response1 = strstr( upperAscii, search1 );
							response2 = strstr( upperAscii, search2 );
							if( response1 || response2 )
							{
								Npcs->stopPetGuarding( Npc );
								mSock->AddID( Npc->GetSerial() );
								//pet fetch code here
								mSock->target( 0, TARGET_GUARD, 1316 );
								CellResponse->PopChar();
								return true;
							}

							strcpy( search1, Dictionary->GetEntry( 1317 ).c_str() ); // Come
							response1 = strstr( upperAscii, search1 );
							if( response1 )
							{
								Npcs->stopPetGuarding( Npc );
								Npc->SetFTarg( calcCharFromSer( mChar->GetSerial() ) );
								Npc->SetNpcWander( 1 );
								mSock->sysmessage( 1318 );
								retval = true;
								if( !response4 )
								{
									CellResponse->PopChar();
									return true;
								}
							}

							strcpy( search1, Dictionary->GetEntry( 1319 ).c_str() ); // Guard
							strcpy( search2, Dictionary->GetEntry( 1320 ).c_str() ); // Me
							response1 = strstr( upperAscii, search1 );
							response2 = strstr( upperAscii, search2 );
							if( response1 )
							{
								if( !isFriend )
								{
									Npcs->stopPetGuarding( Npc );
									if( response2 ) //if me is in
									{
										mSock->sysmessage( 1321 );
										Npc->SetNPCAiType( aiPET_GUARD );									// 32 is guard mode
										mChar->SetGuarded( true );
										Npc->SetFTarg( calcCharFromSer( mChar->GetSerial() ) );
										Npc->SetNpcWander( 1 );
										//add pet guard me code here
										CellResponse->PopChar();
										return true;
									}
									else
									{
										mSock->AddID( Npc->GetSerial() );
										// Guard target
										mSock->target( 0, TARGET_GUARD, 1104 );
										CellResponse->PopChar();
										return true;
									}
								}
							}

							strcpy( search1, Dictionary->GetEntry( 1619 ).c_str() ); // Friend
							response1 = strstr( upperAscii, search1 );
							if( response1 )
							{
								if( !isFriend )
								{
									mSock->AddID( Npc->GetSerial() );
									// Friend target
									mSock->target( 0, TARGET_FRIEND, 1620 );
									CellResponse->PopChar();
									return true;
								}
							}

							strcpy( search1, Dictionary->GetEntry( 1626 ).c_str() ); // Stop
							strcpy( search2, Dictionary->GetEntry( 1627 ).c_str() ); // Stay
							response1 = strstr( upperAscii, search1 );
							response2 = strstr( upperAscii, search2 );
							if( response1 || response2 )
							{
								Npcs->stopPetGuarding( Npc );
								//pet stop code here
								Npc->SetFTarg( INVALIDSERIAL );
								Npc->SetTarg( INVALIDSERIAL );
								if( Npc->IsAtWar() ) 
									npcToggleCombat( Npc );
								Npc->SetNpcWander( 0 );
								retval = true;
								if( !response4 )
								{
									CellResponse->PopChar();
									return true;
								}
							}

							strcpy( search1, Dictionary->GetEntry( 1322 ).c_str() ); // Transfer
							response1 = strstr( upperAscii, search1 );
							if( response1 && !response4 )
							{
								Npcs->stopPetGuarding( Npc );
								//pet transfer code here
								mSock->AddID( Npc->GetSerial() );
								mSock->target( 0, TARGET_TRANSFER, 1323 );
								CellResponse->PopChar();
								return true;
							}

							strcpy( search1, Dictionary->GetEntry( 1324 ).c_str() ); // Release
							response1 = strstr( upperAscii, search1 );
							if( response1 && !response4 )
							{
								Npcs->stopPetGuarding( Npc );
								//pet release code here
								Npc->SetFTarg( INVALIDSERIAL );
								Npc->SetNpcWander( 2 );
								Npc->SetOwner( NULL );
								Npc->talkAll( 1325, false, Npc->GetName().c_str() );
								if( Npc->GetTimer( tNPC_SUMMONTIME ) )
								{
									Effects->PlaySound( &chars[i], 0x01FE );
									Npcs->DeleteChar( Npc );
								}
								CellResponse->PopChar();
								return true;
							}
						}
					}
				}
			}
			CellResponse->PopChar();
		}
	}
	return retval;
}

void cSpeech::responsevendor( CPITalkRequest *pckTalk )
{
	char *response1 = NULL;
	char *response2 = NULL;
	char *response3 = NULL;
	char *response4 = NULL;
	
	char search1[50];
	char search2[50];
	char search3[50];
	char search4[50];
	
	cSocket *mSock	= pckTalk->GetSocket();
	CChar *mChar	= mSock->CurrcharObj();
	UI08 *tbuffer	= mSock->TBuffer();
	char *asciiText	= pckTalk->Text();
	SI16 x = mChar->GetX(), y = mChar->GetY();
	char upperAscii[256];

	strcpy( upperAscii, asciiText );
	strupr( upperAscii );
	
	SubRegion *toSearch	= NULL;
	CChar *NpcSearch	= NULL;
	
	strcpy( search1, Dictionary->GetEntry( 1330 ).c_str() );
	strcpy( search2, Dictionary->GetEntry( 1331 ).c_str() );
	strcpy( search4, Dictionary->GetEntry( 1332 ).c_str() );
	response1 = strstr( upperAscii, search1 );
	response2 = strstr( upperAscii, search2 );
	response4 = strstr( upperAscii, search4 );
	
	UI08 worldNumber = mChar->WorldNumber();
	if( response4 )
	{
		toSearch = MapRegion->GetCell( x, y, worldNumber );
		if( !toSearch ) 
			return;
		toSearch->PushChar();
		for( NpcSearch = toSearch->FirstChar(); !toSearch->FinishedChars(); NpcSearch = toSearch->GetNextChar() )
		{
			if( mChar == NpcSearch )
				continue;
			if( !NpcSearch->IsNpc() && !isOnline( NpcSearch ) )
				continue;
			NpcSearch->SetTimer( tNPC_MOVETIME, BuildTimeValue( 60 ) );
			strcpy( search3, NpcSearch->GetName().c_str() );
			strupr( search3 );
			response3 = strstr( upperAscii, search3 );
			if( response3 || response2 || response1 )
			{
				if( abs( mChar->GetX() - NpcSearch->GetX() ) <= 3 &&
					abs( mChar->GetY() - NpcSearch->GetY() ) <= 3 &&
					abs( mChar->GetZ() - NpcSearch->GetZ() ) <= 5 )
				{ //PlayerVendors
					if( NpcSearch->GetNPCAiType() == aiPLAYERVENDOR )
					{
						mSock->AddX( calcCharFromSer( NpcSearch->GetSerial() ) );
						NpcSearch->talk( mSock, 1333, false );
						mSock->target( 0, TARGET_PLVBUY, " ");
						break;
					} 
					else if( Targ->BuyShop( mSock, NpcSearch ) ) 
						break;
				}
			}
		}
		toSearch->PopChar();
	}
	
	
	//PlayerVendors
	response4 = 0;
	strcpy( search4, Dictionary->GetEntry( 1334 ).c_str() );
	response4 = strstr( upperAscii, search4 );
	if( response4 )//AntiChrist
	{
		toSearch = MapRegion->GetCell( x, y, worldNumber );
		if( !toSearch ) 
			return;
		toSearch->PushChar();
		for( NpcSearch = toSearch->FirstChar(); !toSearch->FinishedChars(); NpcSearch = toSearch->GetNextChar() )
		{
			if( mChar == NpcSearch )
				continue;
			if( !NpcSearch->IsNpc() && !isOnline( NpcSearch ) )
				continue;
			NpcSearch->SetTimer( tNPC_MOVETIME, BuildTimeValue( 60 ) );
			strcpy( search3, NpcSearch->GetName().c_str() );
			strupr( search3 );
			response3 = strstr( upperAscii, search3 );
			if( response3 || response2 || response1 )
			{
				if( abs( mChar->GetX() - NpcSearch->GetX() ) <= 3 &&
					abs( mChar->GetY() - NpcSearch->GetY() ) <= 3 &&
					abs( mChar->GetZ() - NpcSearch->GetZ() ) <= 5 )
				{ //PlayerVendors
					if( NpcSearch->GetNPCAiType() == aiPLAYERVENDOR )
					{
						PlVGetgold( mSock, NpcSearch );
						toSearch->PopChar();
						break;
					}
				}
			}
		}
		toSearch->PopChar();
	}
	//end PlayerVendors --^
	
	response4 = 0;
	strcpy( search4, Dictionary->GetEntry( 1335 ).c_str() );
	response4 = strstr( upperAscii, search4 );
	if( response4 )//AntiChrist
	{
		toSearch = MapRegion->GetCell( x, y, worldNumber );
		if( !toSearch ) 
			return;
		toSearch->PushChar();
		for( NpcSearch = toSearch->FirstChar(); !toSearch->FinishedChars(); NpcSearch = toSearch->GetNextChar() )
		{
			if( mChar == NpcSearch )
				continue;
			if( !NpcSearch->IsNpc() && !isOnline( NpcSearch ) )
				continue;
			NpcSearch->SetTimer( tNPC_MOVETIME, BuildTimeValue( 60 ) );
			strcpy( search3, NpcSearch->GetName().c_str() );
			strupr( search3 );
			response3 = strstr( upperAscii, search3 );
			if( response3 || response2 || response1 )
			{
				if( abs( mChar->GetX() - NpcSearch->GetX() ) <= 3 &&
					abs( mChar->GetY() - NpcSearch->GetY() ) <= 3 &&
					abs( mChar->GetZ() - NpcSearch->GetZ() ) <= 5 )
				{ //PlayerVendors
					// Stop the NPC from moving for a minute while talking with the player
					NpcSearch->SetTimer( tNPC_MOVETIME, BuildTimeValue( 60 ) );
					sendSellStuff( mSock, NpcSearch );
					break;
				}
			}
		}
		toSearch->PopChar();
	}
	return;
}


