#include "uox3.h"
#include "cdice.h"
#include "skills.h"
#include "boats.h"
#include "cGuild.h"
#include "townregion.h"
#include "cRaces.h"
#include "cServerDefinitions.h"
#include "commands.h"
#include "cMagic.h"
#include "ssection.h"
#include "gump.h"
#include "trigger.h"
#include "cScript.h"
#include "cEffects.h"
#include "packets.h"
#include "classes.h"
#include "regions.h"
#include "combat.h"
#include "magic.h"

#undef DBGFILE
#define DBGFILE "targeting.cpp"

#if defined( _DEBUG )
#define VALIDATESOCKET( s ) if( s == NULL ) \
	{ \
		Console.Print( "Socket failure at %s", __FILE__LINE__ );	\
		return;	\
	}
#else
#define VALIDATESOCKET( s ) if( s == NULL ) \
	return;
#endif


void cTargets::PlVBuy( cSocket *s )//PlayerVendors
{
	VALIDATESOCKET( s );

	CHARACTER v = s->AddX();
	if( v == INVALIDSERIAL || v > cwmWorldState->GetCMem() ) 
		return;
	if( chars[v].isFree() ) 
		return;

	CChar *mChar = s->CurrcharObj();
	UI32 gleft = calcGold( mChar );

	CItem *p = getPack( mChar );
	if( p == NULL ) 
	{
		s->sysmessage( 773 ); 
		return; 
	}

	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL ) 
		return;	
	if( i->GetCont() == NULL ) 
		return;
	
	CItem *np = (CItem *)i->GetCont();
	CChar *npc = getPackOwner( np );
	if( npc == NULL || npc != &chars[v] ) 
		return;
	if( npc->GetNPCAiType() != aiPLAYERVENDOR )
		return;
	if( mChar == npc->GetOwnerObj() )
	{
		npc->talk( s, 999, false );
		return;
	}

	if( gleft < static_cast<UI32>(i->GetBuyValue()) ) 
	{
		npc->talk( s, 1000, false );
		return;
	} 
	else 
	{
		UI32 tAmount = DeleteQuantity( mChar, 0x0EED, (UI32)i->GetBuyValue() );
		// tAmount > 0 indicates there wasn't enough money...
		// could be expanded to take money from bank too...
	}
	
	npc->talk( s, 1001, false );
	npc->SetHoldG( npc->GetHoldG() + i->GetBuyValue() );

	i->SetCont( p );	// move containers
	s->statwindow( mChar );
}

void cTargets::HandleGuildTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *trgChar	= NULL;
	CChar *mChar	= s->CurrcharObj();
	CGuild *mGuild	= NULL, *tGuild = NULL;
	switch( s->GetByte( 5 ) )
	{
		case 0:	// recruit character
			trgChar = calcCharObjFromSer( s->GetDWord( 7 ) );
			if( trgChar != NULL )
			{
				if( trgChar->GetGuildNumber() == -1 )	// no existing guild
				{
					mGuild = GuildSys->Guild( mChar->GetGuildNumber() );
					if( mGuild != NULL )
						mGuild->NewRecruit( (*trgChar) );
				}
				else
					s->sysmessage( 1002 );
			}
			break;
		case 1:		// declare fealty
			trgChar = calcCharObjFromSer( s->GetDWord( 7 ) );
			if( trgChar != NULL )
			{
				if( trgChar->GetGuildNumber() == mChar->GetGuildNumber() )	// same guild
					mChar->SetGuildFealty( trgChar->GetSerial() );
				else
					s->sysmessage( 1003 );
			}
			break;
		case 2:	// declare war
			trgChar = calcCharObjFromSer( s->GetDWord( 7 ) );
			if( trgChar != NULL )
			{
				if( trgChar->GetGuildNumber() != mChar->GetGuildNumber() )
				{
					if( trgChar->GetGuildNumber() == -1 )
						s->sysmessage( 1004 );
					else
					{
						mGuild = GuildSys->Guild(mChar->GetGuildNumber() );
						if( mGuild != NULL )
						{
							mGuild->SetGuildRelation( trgChar->GetGuildNumber(), GR_WAR );
							tGuild = GuildSys->Guild( trgChar->GetGuildNumber() );
							if( tGuild != NULL )
								tGuild->TellMembers( 1005, mGuild->Name() );
						}
					}
				}
				else
					s->sysmessage( 1006 );
			}
			break;
		case 3:	// declare ally
			trgChar = calcCharObjFromSer( s->GetDWord( 7 ) );
			if( trgChar != NULL )
			{
				if( trgChar->GetGuildNumber() != mChar->GetGuildNumber() )
				{
					if( trgChar->GetGuildNumber() == -1 )
						s->sysmessage( 1004 );
					else
					{
						mGuild = GuildSys->Guild( mChar->GetGuildNumber() );
						if( mGuild != NULL )
						{
							mGuild->SetGuildRelation( trgChar->GetGuildNumber(), GR_ALLY );
							tGuild = GuildSys->Guild( trgChar->GetGuildNumber() );
							if( tGuild != NULL )
								tGuild->TellMembers( 1007, mGuild->Name() );
						}
					}
				}
				else
					s->sysmessage( 1006 );
			}
			break;
	}

}

void cTargets::HandleSetScpTrig( cSocket *s )
{
	VALIDATESOCKET( s );
	UI16 targTrig		= (UI16)s->AddX();
	cScript *toExecute	= Trigger->GetScript( targTrig );
	if( toExecute == NULL )
	{
		s->sysmessage( "No such script is known to the server" );
		return;
	}

	SERIAL ser = s->GetDWord( 7 );
	if( ser < BASEITEMSERIAL )
	{	// character
		CChar *targChar = calcCharObjFromSer( ser );
		if( targChar != NULL )
		{
			targChar->SetScriptTrigger( targTrig );
			s->sysmessage( 1653 );
		}
	}
	else
	{	// item
		CItem *targetItem = calcItemObjFromSer( ser );
		if( targetItem != NULL )
		{
			s->sysmessage( 1652 );
			targetItem->SetScriptTrigger( targTrig );
		}
	}
}

//o--------------------------------------------------------------------------o
//|	Function		-	void cTargets::MultiTarget( cSocket *s )
//|	Date			-	Unknown
//|	Developers		-	Unknown
//|	Organization	-	UOX3 DevTeam
//o--------------------------------------------------------------------------o
//|	Description		-	Runs various commands based upon the target ID we sent to the socket
//o--------------------------------------------------------------------------o
//| Modifications	-	Overhauled to use an enum allowing simple modification (Zane)
//o--------------------------------------------------------------------------o
void cTargets::MultiTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *mChar = s->CurrcharObj();
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
	{
		if( mChar->GetSpellCast() != -1 )	// need to stop casting if we don't target right
			mChar->StopSpell();
		return; // do nothing if user cancels, avoids CRASH! - Morrolan
	}
	
	UI08 a1 = s->GetByte( 2 );
	UI08 a2 = s->GetByte( 3 );
	UI08 a3 = s->GetByte( 4 );
	TargetIDs targetID = (TargetIDs)s->GetByte( 5 );
	s->TargetOK( false );
	if( mChar->IsDead() && !mChar->IsGM() && mChar->GetAccount().wAccountIndex != 0 )
	{
		s->sysmessage( 1008 );
		if( mChar->GetSpellCast() != -1 )	// need to stop casting if we don't target right
			mChar->StopSpell();
		return;
	}
	if( a1 == 0 && a2 == 1 )
	{
		if( a3 == 2 )	// Guilds
		{
			HandleGuildTarget( s );
			return;
		}
		if( a3 == 1 )	// CustomTarget
		{
			UI16 scriptNum = (UI16)s->AddX();
			cScript *tScript = Trigger->GetScript( scriptNum );
			if( tScript != NULL )
				tScript->DoCallback( s, s->GetDWord( 7 ), static_cast<UI08>(targetID) );
			return;
		}
 		if( a3 == 0 )
		{
			CItem *targetItem;
			switch( targetID )
			{
				case TARGET_ADD:			AddTarget( s );						break;
				case TARGET_RENAME:			RenameTarget( s );					break;
				case TARGET_TELE:			TeleTarget( s );					break;	
				case TARGET_REMOVE:			RemoveTarget( s );					break;
				case TARGET_DYE:			DyeTarget( s );						break;
				case TARGET_TYPE:			TypeTarget( s );					break;
				case TARGET_ID:				IDtarget( s );						break;
				case TARGET_XGO:			XgoTarget( s );						break;
				case TARGET_PRIV:			PrivTarget( s );					break;
				case TARGET_MORE:			MoreTarget( s );					break;
				case TARGET_KEY:			KeyTarget( s );						break;
				case TARGET_ISTATS:			IstatsTarget( s );					break;
				case TARGET_CSTATS:			CstatsTarget( s );					break;
				case TARGET_KILLHAIR:		KillTarget( s, 0x0B );				break;
				case TARGET_KILLBEARD:		KillTarget( s, 0x10 );				break;
				case TARGET_KILLPACK:		KillTarget( s, 0x15 );				break;
				case TARGET_FONT:			FontTarget( s );					break;
				case TARGET_KILL:			GhostTarget( s );					break;
				case TARGET_RESURRECT:		ResurrectionTarget( s );			break;
				case TARGET_BOLT:			BoltTarget( s );					break;
				case TARGET_AMOUNT:			AmountTarget( s );					break;
				case TARGET_KICK:			CloseTarget( s );					break;
				case TARGET_ADDMENU:		AddMenuTarget( s, true, s->XText() ); break;
				case TARGET_NPCMENU:		NpcMenuTarget( s );					break;
				case TARGET_MOVABLE:		MovableTarget( s );					break;
				case TARGET_OWNER:			OwnerTarget( s );					break;
				case TARGET_DYEALL:			ColorsTarget( s );					break;
				case TARGET_DVAT:			DvatTarget( s );					break;
				case TARGET_ADDNPC:			AddNpcTarget( s );					break;
				case TARGET_FREEZE:			FreezeTarget( s );					break;
				case TARGET_UNFREEZE:		UnfreezeTarget( s );				break;
				case TARGET_ALLSET:			AllSetTarget( s );					break;
				case TARGET_INFO:			InfoTarget( s );					break;
				case TARGET_TITLE:			TitleTarget( s );					break;
				case TARGET_SHOWDETAIL:		ShowDetail( s );					break;
				case TARGET_NPCACT:			npcAct( s );						break;
				case TARGET_WSTATS:			WstatsTarget( s );					break;
				case TARGET_NPCFOLLOW:		NpcTarget( s );						break;
				case TARGET_NPCFOLLOW2:		NpcTarget2( s );					break;
				case TARGET_NPCRESURRECT:	NpcResurrectTarget( mChar );		break;
				case TARGET_NPCCIRCLE:		NpcCircleTarget( s );				break;
				case TARGET_NPCWANDER:		NpcWanderTarget( s );				break;
				case TARGET_VISIBLE:		VisibleTarget( s );					break;
				case TARGET_TWEAK:			TweakTarget( s );					break;
				case TARGET_MOREX:			MoreXTarget( s );					break;
				case TARGET_MOREY:			MoreYTarget( s );					break;
				case TARGET_MOREZ:			MoreZTarget( s );					break;
				case TARGET_MOREXYZ:		MoreXYZTarget( s );					break;
				case TARGET_NPCRECT:		NpcRectTarget( s );					break;
				case TARGET_DIVINELOCK:
											targetItem = calcItemObjFromSer( s->GetDWord( 7 ) );
											if( targetItem != NULL )
											{
												s->sysmessage( 1010 );
												targetItem->enhanced ^= 0x0001;	// Set DevineLock bit in items
											}
											break;
				case TARGET_DIVINEUNLOCK:
											targetItem = calcItemObjFromSer( s->GetDWord( 7 ) );
											if( targetItem != NULL )
											{
												s->sysmessage( 1011 );
												targetItem->enhanced &= 0xFFFE;	// ReSet DevineLock bit in items
											}
											break;
				case TARGET_MAKESTATUS:		MakeStatusTarget( s );				break;
				case TARGET_SETSCPTRIG:		HandleSetScpTrig( s );				break;
				case TARGET_DELETECHAR:		DeleteCharTarget( s );				break;
				case TARGET_AXE:			AxeTarget( s );						break;
				case TARGET_SWORD:			SwordTarget( s );					break;
				case TARGET_SETDIR:			SetDirTarget( s );					break;
				case TARGET_SETPRIVS:		ObjPrivTarget( s );					break;
				case TARGET_AREACOMMAND:	AreaCommand( s );					break;
				case TARGET_SETCOMMANDLEVEL:		
											if( mChar->GetAccount().wAccountIndex == 0 )
												CommandLevel( s ); 
											else
												s->sysmessage( 1009 );
												break;
				case TARGET_NPCAI:			NpcAITarget( s );					break;
				case TARGET_XBANK:			xBankTarget( s );					break;
				case TARGET_DUPE:			DupeTarget( s );					break;
				case TARGET_MOVETOBAG:		MoveToBagTarget( s );				break;
				case TARGET_MANA:			ManaTarget( s );					break;
				case TARGET_STAMINA:		StaminaTarget( s );					break;
				case TARGET_GMOPEN:			GmOpenTarget( s );					break;
				case TARGET_JAIL:			JailTarget( s, s->GetDWord( 7 ) );		break;
				case TARGET_RELEASE:		ReleaseTarget( s, s->GetDWord( 7 ) );	break;
				case TARGET_PERMHIDE:		permHideTarget( s );				break;
				case TARGET_UNHIDE:			unHideTarget( s );					break;
				case TARGET_WIPABLE:		SetWipeTarget( s );					break;
				case TARGET_XTELEPORT:		XTeleport( s, 0 );					break;
				case TARGET_SETSPATTACK:	SetSpAttackTarget( s );				break;
				case TARGET_FULLSTATS:		FullStatsTarget( s );				break;
				case TARGET_LOADCANNON:		LoadCannon( s );					break;
				case TARGET_SETPOISON:		SetPoisonTarget( s );				break;
				case TARGET_SETPOISONED:	SetPoisonedTarget( s );				break;
				case TARGET_SETSPDELAY:		SetSpaDelayTarget( s );				break;
				case TARGET_SETADVOBJ:		SetAdvObjTarget( s );				break;
				case TARGET_SETINVULNFLAG:	SetInvulFlag( s );					break;
				case TARGET_VIAL:			vialtarget( s );					break;
				case TARGET_TILING:			Tiling( s );						break;
				case TARGET_WIPING:			Wiping( s );						break;
				case TARGET_CANTRAIN:		CanTrainTarget( s );				break;
				case TARGET_SETSPLIT:		SetSplitTarget( s );				break;
				case TARGET_SETSPLITCHANCE: SetSplitChanceTarget( s );			break;
				case TARGET_POSSESS:		PossessTarget( s );					break;
				case TARGET_TELESTUFF:		TeleStuff( s );						break;        
				case TARGET_SQUELCH:		SquelchTarg( s );					break;
				case TARGET_BAN:			BanTarg( s );						break;
				case TARGET_SHOWSKILLS:		ShowSkillTarget( s );				break;
				case TARGET_GLOW:			GlowTarget( s );					break;
				case TARGET_UNGLOW:			UnglowTarget( s );					break;
				case TARGET_NEWX:			NewXTarget( s );					break;
				case TARGET_NEWY:			NewYTarget( s );					break;
				case TARGET_NEWZ:			NewzTarget( s );					break;
				case TARGET_INCX:			IncXTarget( s );					break;
				case TARGET_INCY:			IncYTarget( s );					break;
				case TARGET_INCZ:			IncZTarget( s );					break;
				// Vendors
				case TARGET_MAKESHOP:		MakeShopTarget( s );				break;
				case TARGET_REMOVESHOP:		RemoveShopTarget( s );				break;
				case TARGET_BUYSHOP:		BuyShopTarget( s );					break;
				case TARGET_SELL:			SellStuffTarget( s );				break;
				case TARGET_SETBUYVALUE:	SetBuyValueTarget( s );				break;
				case TARGET_SETSELLVALUE:	SetSellValueTarget( s );			break;
				case TARGET_SETRESTOCK:		SetRestockTarget( s );				break;
				case TARGET_PLVBUY:			PlVBuy( s );						break;
				// Town Stuff
				case TARGET_TOWNALLY:		MakeTownAlly( s );					break;
				case TARGET_VOTEFORMAYOR:	region[mChar->GetTown()]->VoteForMayor( s ); break;
				// House Functions
				case TARGET_HOUSEOWNER:		HouseOwnerTarget( s );				break;
				case TARGET_HOUSEEJECT:		HouseEjectTarget( s );				break;
				case TARGET_HOUSEBAN:		HouseBanTarget( s );				break;
				case TARGET_HOUSEFRIEND:	HouseFriendTarget( s );				break;
				case TARGET_HOUSEUNLIST:	HouseUnlistTarget( s );				break;
				case TARGET_HOUSELOCKDOWN:	HouseLockdown( s );					break;
				case TARGET_HOUSERELEASE:	HouseRelease( s );					break;
				// Pets
				case TARGET_FOLLOW:			FollowTarget( s );					break;
				case TARGET_ATTACK:			AttackTarget( s );					break;
				case TARGET_TRANSFER:		TransferTarget( s );				break;
				case TARGET_GUARD:			GuardTarget( s );					break;
				case TARGET_FRIEND:			FriendTarget( s );					break;
				// Magic
				case TARGET_CASTSPELL:		Magic->CastSpell( s, mChar );		break;
				case TARGET_RECALL:			Magic->Recall( s );					break;					
				case TARGET_MARK:			Magic->Mark( s );					break;
				case TARGET_GATE:			Magic->Gate( s );					break;					
				case TARGET_HEAL:			Magic->Heal( s );					break;
				// Skills Functions
				case TARGET_ARMSLORE:		Skills->ArmsLoreTarget( s );		break;
				case TARGET_ANATOMY:		Skills->AnatomyTarget( s );			break;
				case TARGET_ITEMID:			Skills->ItemIDTarget( s );			break;
				case TARGET_EVALINT:		Skills->EvaluateIntTarget( s );		break;
				case TARGET_TAME:			Skills->TameTarget( s );			break;
				case TARGET_FISH:			Skills->FishTarget( s );			break;
				case TARGET_SMITH:			Skills->Smith( s );					break;
				case TARGET_MINE:			Skills->Mine( s );					break;
				case TARGET_SMELTORE:		Skills->SmeltOre( s );				break;
				case TARGET_REPAIRLEATHER:	Skills->RepairLeather( s );			break;
				case TARGET_REPAIRBOW:		Skills->RepairBow( s );				break;
				case TARGET_REPAIRMETAL:	Skills->RepairMetal( s );			break;
				case TARGET_SMELT:			SmeltTarget( s );					break;
				case TARGET_DETECTHIDDEN:	Skills->DetectHidden( s );			break;
				case TARGET_PROVOCATION:	Skills->ProvocationTarget1( s );	break;
				case TARGET_PROVOCATION2:	Skills->ProvocationTarget2( s );	break;
				case TARGET_ENTICEMENT:		Skills->EnticementTarget1( s );		break;
				case TARGET_ENTICEMENT2:	Skills->EnticementTarget2( s );		break;
				case TARGET_ALCHEMY:		Skills->AlchemyTarget( s );			break;
				case TARGET_CREATEBANDAGE:	Skills->CreateBandageTarget( s );	break;
				case TARGET_HEALING:		Skills->HealingSkillTarget( s );	break;
				case TARGET_CARPENTRY:		Skills->Carpentry( s );				break;
				case TARGET_BEGGING:		Skills->BeggingTarget( s );			break;
				case TARGET_ANIMALLORE:		Skills->AnimalLoreTarget( s );		break;
				case TARGET_FORENSICS:		Skills->ForensicsTarget( s );		break;
				case TARGET_APPLYPOISON: 
											mChar->SetSkillItemSerial( s->GetDWord( 7 ) );
											s->target( 0, TARGET_POISONITEM, 1613 );
											break;
				case TARGET_POISONITEM:		Skills->PoisoningTarget( s );		break;
				case TARGET_INSCRIBE:		Skills->Inscribe( s );				break;
				case TARGET_LOCKPICK:		Skills->LockPick( s );				break;
				case TARGET_WHEEL:			Skills->Wheel( s );					break;
				case TARGET_LOOM:			Skills->Loom( s );					break;
				case TARGET_TAILORING:		Skills->Tailoring( s );				break;
				case TARGET_COOKING:		Skills->handleCooking( s );			break;
				case TARGET_FLETCHING:		Skills->Fletching( s );				break;
				case TARGET_TINKERING:		Skills->Tinkering( s );				break; 
				case TARGET_TINKERAXEL:		Skills->TinkerAxel( s );			break;
				case TARGET_TINKERAWG:		Skills->TinkerAwg( s );				break;
				case TARGET_TINKERCLOCK:	Skills->TinkerClock( s );			break;
				case TARGET_STEALING:		Skills->StealingTarget( s );		break;
				case TARGET_SPAWNOBJ:		SpawnObjTarget( s, false );			break;
				case TARGET_SPAWNOBJLIST:	SpawnObjTarget( s, true );			break;
				default:					Console.Error( 2, " MultiTarget() was passed a bad targetID (%u)", static_cast<UI08>(targetID) );
			}
		}
	}
}

#if defined( _MSC_VER )
#pragma note( "Function Warning: cTargets::BanTarg() does nothing" )
#endif
void cTargets::BanTarg( cSocket *s )
{

}

//o---------------------------------------------------------------------------o
//|   Function    -  void AddTarget( cSocket *s)
//|   Date        -  UnKnown
//|   Programmer  -  UnKnown  (Touched tabstops by Tauriel Dec 29, 1998)
//o---------------------------------------------------------------------------o
//|   Purpose     -  Adds an item when using /add # # .
//o---------------------------------------------------------------------------o
void cTargets::AddTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;
	bool pileable = false;
	CTile tile;
	
	if( s->AddID1() == 0x40 )
	{
		switch( s->AddID2() )
		{
			case 100:
			case 102:
			case 104:
			case 106:
			case 108:
			case 110:
			case 112:
			case 114:
			case 116:
			case 118:
			case 120:
			case 122:
			case 124:
			case 126:
			case 140:
				buildHouse( s, s->AddID3() );//If its a valid house, send it to buildhouse!
				return; // Morrolan, here we WANT fall-thru, don't mess with this switch
		}
	}
	UI16 modelNum = (UI16)(( ( s->AddID1() )<<8 ) + s->AddID2());
	Map->SeekTile( modelNum, &tile );
	if( tile.Stackable() ) 
		pileable = true;
	
	CItem *c = Items->SpawnItem( NULL, s->CurrcharObj(), 1, "#", pileable, modelNum, 0, false, false );
	if( c == NULL ) 
		return;
	c->SetPriv( 0 );	//Make them not decay
	const SI16 tX = s->GetWord( 11 );
	const SI16 tY = s->GetWord( 13 );
	const SI08 tZ = (SI08)(s->GetByte( 16 ) + Map->TileHeight( s->GetWord( 17 ) ));
	
	if( c->GetCont() == NULL )
		c->SetLocation( tX, tY, tZ );
	else
	{
		c->SetX( tX );
		c->SetY( tY );
		c->SetZ( tZ );
	}
	s->AddID1( 0 );
	s->AddID2( 0 );
}

void cTargets::RenameTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial = s->GetDWord( 7 );
	if( serial >= BASEITEMSERIAL )
	{
		CItem *i = calcItemObjFromSer( serial );
		if( i != NULL )
		{
			if( s->AddX() == 1 ) //rename2
				i->SetName2( s->XText() );
			else
				i->SetName( s->XText() );
		}
	}
	else
	{
		CChar *c = calcCharObjFromSer( serial );
		if( c != NULL )
		{
			c->SetName( s->XText() );
			s->statwindow( c );
		}
	}
}

void cTargets::TeleTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;

	SERIAL serial = s->GetDWord( 7 );

	CItem *i = calcItemObjFromSer( serial );
	if( i != NULL )
	{
		s->sysmessage( 717 );
		return;
	}

	SI16 targX, targY;
	SI08 targZ;
	CChar *c = calcCharObjFromSer( serial );
	if( c != NULL )
	{
		targX = c->GetX();
		targY = c->GetY();
		targZ = c->GetZ();
	}
	else
	{
		targX = s->GetWord( 11 );
		targY = s->GetWord( 13 );
		targZ = (SI08)(s->GetByte( 16 ) + Map->TileHeight( s->GetWord( 17 ) ));
	}
	CChar *mChar = s->CurrcharObj();

	if( mChar->IsGM() || LineOfSight( s, mChar, targX, targY, targZ, WALLS_CHIMNEYS + DOORS + ROOFING_SLANTED ) )	
	{
		if( s->CurrentSpellType() != 2 )  // not a wand cast
		{
			Magic->SubtractMana( mChar, 3 );  // subtract mana on scroll or spell
			if( s->CurrentSpellType() == 0 )             // del regs on normal spell
			{
				reag_st toDel;
				toDel.drake = 1;
				toDel.moss = 1;
				Magic->DelReagents( mChar, toDel );
			}
		}
		
		Effects->PlaySound( s, 0x01FE, true );
		
		mChar->SetLocation( targX, targY, targZ );
		mChar->Teleport();
		Effects->staticeffect( mChar, 0x372A, 0x09, 0x06 );
	} 
}

void cTargets::RemoveTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i != NULL )
	{
		s->sysmessage( 1013 );
		Items->DeleItem( i );
    } 
	else 
	{
		CChar *c = calcCharObjFromSer( serial );
		if( c == NULL )
			return;
		if( c->GetAccount().wAccountIndex != AB_INVALID_ID && !c->IsNpc() )
		{ 
			s->sysmessage( 1014 );
			return;
		}
		s->sysmessage( 1015 );
		Npcs->DeleteChar( c );
	}
}

void cTargets::DyeTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	int b;
	UI16 colour, k;
	CItem *i		= NULL;
	CChar *c		= NULL;
	SERIAL serial	= s->GetDWord( 7 );
	if( s->AddID1() == 0xFF && s->AddID2() == 0xFF )
	{
		CPDyeVat toSend;
		if( serial >= BASEITEMSERIAL )
		{
			i = calcItemObjFromSer( serial );
			if( i != NULL )
			{
				toSend = (*i);
				s->Send( &toSend );
			}
		}
		else
		{
			c = calcCharObjFromSer( serial );
			if( c != NULL )
			{
				toSend = (*c);
				toSend.Model( 0x2106 );
				s->Send( &toSend );
			}
		}
	}
	else
	{
		if( serial >= BASEITEMSERIAL )
		{
			i = calcItemObjFromSer( serial );
			if( i == NULL )
				return;
			colour = (UI16)(( (s->AddID1())<<8 ) + s->AddID2());
			if( !s->DyeAll() )
			{
				if( colour < 0x0002 || colour > 0x03E9 )
					colour = 0x03E9;
			}
			
			b = ((colour&0x4000)>>14) + ((colour&0x8000)>>15);   
			if( !b )
				i->SetColour( colour );
		}
		else
		{
			c = calcCharObjFromSer( serial );
			if( c == NULL )
				return;
			UI16 body = c->GetID();
			k = (UI16)(( ( s->AddID1() )<<8 ) + s->AddID2());
			
			b = k&0x4000; 
			if( b == 16384 && ( body >= 0x0190 && body <= 0x03E1 ) ) 
				k = 0xF000; // but assigning the only "transparent" value that works, namly semi-trasnparency.
			
			if( k != 0x8000 ) // 0x8000 also crashes client ...
			{	
				c->SetSkin( k );
				c->SetOrgSkin( k );
				c->Teleport();
			}
		}
	}
}

void cTargets::NewzTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i != NULL )
	{
		i->SetZ( static_cast<SI08>(s->AddX()) );
		return;
	}

	CChar *c = calcCharObjFromSer( serial );
	if( c != NULL )
	{
		c->SetDispZ( static_cast<SI08>(s->AddX()) );
		c->SetZ( static_cast<SI08>(s->AddX()) );
		c->Teleport();
	}
}

void cTargets::TypeTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetType( s->AddID1() );
}


void cTargets::IDtarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	UI16 targID		= ( (s->AddID1())<<8 ) + s->AddID2();
	if( serial >= BASEITEMSERIAL )
	{
		CItem *i = calcItemObjFromSer( serial );
		if( i == NULL )
			return;
		i->SetID( targID );
		return;
	}
	else
	{
		CChar *c = calcCharObjFromSer( serial );
		if( c == NULL )
			return;
		if( targID <= 0x7CF )
		{
			c->SetID( targID );
			c->SetOrgID( targID );
			c->Teleport();
		}
	}
}

void cTargets::XTeleport( cSocket *s, UI08 x )
{
	VALIDATESOCKET( s );
	SERIAL serial	= INVALIDSERIAL;
	CChar *c		= NULL;
	// Abaddon 17th February 2000 Converted if to switch (easier to read)
	// Also made it so that it calls teleport, not updatechar, else you don't get the items in range
	CChar *mChar = s->CurrcharObj();
	switch( x )
	{
		case 0:
			serial	= s->GetDWord( 7 );
			c		= calcCharObjFromSer( serial );
			break;
		case 5:
			serial	= calcserial( (UI08)Commands->GetArgument( 1 ), (UI08)Commands->GetArgument( 2 ), (UI08)Commands->GetArgument( 3 ), (UI08)Commands->GetArgument( 4 ) );
			c		= calcCharObjFromSer( serial );
			break;
		case 2:
			cSocket *mSock;
			mSock = calcSocketObjFromSock( Commands->GetArgument( 1 ) );
			if( mSock != NULL )
				c = mSock->CurrcharObj();
			else 
				return;
			break;
		default:
			s->sysmessage( 1654, x );
			return;
	}
	
	if( c != NULL )
	{
		c->SetLocation( mChar );
		c->Teleport();
		return;
	}

	CItem *i = calcItemObjFromSer( serial );
	if( i != NULL )
		i->SetLocation( mChar );
}

void cTargets::XgoTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;

	i->SetLocation( static_cast<SI16>(s->AddX()), static_cast<SI16>(s->AddY()), s->AddZ() );
	i->Teleport();
}

void cTargets::MoreXYZTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;

	i->SetMoreX( s->AddX() );
	i->SetMoreY( s->AddY() );
	i->SetMoreZ( s->AddZ() );
}

void cTargets::MoreXTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetMoreX( s->AddX() );
}

void cTargets::MoreYTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetMoreY( s->AddX() );
}

void cTargets::MoreZTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetMoreZ( s->AddX() );
}

void cTargets::PrivTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;

	CChar *mChar = s->CurrcharObj();
	char temp[1024], temp2[1024];
	sprintf( temp, "%s.log", mChar->GetName().c_str() );
	sprintf( temp2, "%s has given %s Priv [%x][%x]", mChar->GetName().c_str(), i->GetName().c_str(), s->AddID1(), s->AddID2() );
	i->SetPriv( ((s->AddID1()<<8) + s->AddID2()) );
	Console.Log( temp2, temp );
}

void cTargets::MoreTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetMore( s->AddID() );
}

void cTargets::KeyTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;

	CChar *mChar		= s->CurrcharObj();
	SERIAL moreSerial	= s->AddID();
	if( i->GetMore() == 0 )
	{       
		if( i->GetType() == 7 && objInRange( mChar, i, DIST_NEARBY ) )
		{
			if( !Skills->CheckSkill( mChar, TINKERING, 400, 1000 ) ) 
			{
				s->sysmessage( 1016 );
				Items->DeleItem( i );
				return;
			}
			i->SetMore( moreSerial );
			s->sysmessage( 1017 );
			return;
		}
	}
	else if( i->GetMore() == moreSerial || s->AddID1() == 0xFF )
	{
		if( ( i->GetType() == 1 || i->GetType() == 63 ) && objInRange( mChar, i, DIST_NEARBY ) )
		{
			if( i->GetType() == 1 ) 
				i->SetType( 8 );
			else if( i->GetType() == 63 ) 
				i->SetType( 64 );
			s->sysmessage( 1018 );
			return;
		}
		if( i->GetType() == 7 && objInRange( mChar, i, DIST_NEARBY ) )
		{
			mChar->SetSpeechItem( i->GetSerial() );
			mChar->SetSpeechMode( 5 );
			s->sysmessage( 1019 );
			return;
		}
		if( ( i->GetType() == 8 || i->GetType() == 64 ) && objInRange( mChar, i, DIST_NEARBY ) )
		{
			if( i->GetType() == 8 ) 
				i->SetType( 1 );
			if( i->GetType() == 64 ) 
				i->SetType( 63 );
			s->sysmessage( 1020 );
			return;
		}
		if( i->GetType() == 12 && objInRange( mChar, i, DIST_NEARBY ) )
		{
			i->SetType( 13 );
			s->sysmessage( 1021 );
			Effects->PlaySound( s, 0x0049, true );
			return;
		}
		if( i->GetType() == 13 && objInRange( mChar, i, DIST_NEARBY ) )
		{
			i->SetType( 12 );
			s->sysmessage( 1022 );
			Effects->PlaySound( s, 0x0049, true );
			return;
		}
		if( i->GetID() == 0x0BD2 )
		{
			s->sysmessage( 1023 );
			mChar->SetSpeechMode( 8 );
			mChar->SetSpeechItem( i->GetSerial() );
			return;
		}
		if( i->GetType() == 117 )
			Boats->OpenPlank( i );
	}
	else
	{
		if( i->GetType() == 7 ) 
			s->sysmessage( 1024 );
		else if( i->GetMore( 1 ) == 0 ) 
			s->sysmessage( 1025 );
		else 
			s->sysmessage( 1026 );
	}
}


void cTargets::IstatsTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CTile tile;
	if( s->GetDWord( 7 ) == 0 )
	{
		UI08 worldNumber	= 0;
		CChar *mChar		= s->CurrcharObj();
		if( mChar != NULL )
			worldNumber = mChar->WorldNumber();
		UI16 targetID = s->GetWord( 0x11 );
		SI16 targetX = s->GetWord( 0x0B );		// store our target x y and z locations
		SI16 targetY = s->GetWord( 0x0D );
		SI08 targetZ = s->GetByte( 0x10 );
		if( targetID != 0 )	// we might have a static rock or mountain
		{
			MapStaticIterator msi( targetX, targetY, worldNumber );
			staticrecord *stat = NULL;
			while( ( ( stat = msi.Next() ) != NULL ) )
			{
				msi.GetTile(&tile);
				if( targetZ == stat->zoff )
				{
					GumpDisplay staticStat( s, 300, 300 );
					staticStat.SetTitle( "Item [Static]" );
					staticStat.AddData( "ID", targetID, 5 );
					staticStat.AddData( "Height", tile.Height() );
					staticStat.AddData( "Name", tile.Name() );
					staticStat.Send( 4, false, INVALIDSERIAL );
				}
			}
		}
		else		// or it could be a map only
		{
			// manually calculating the ID's if a maptype
			map_st map1;
			CLand land;
			map1 = Map->SeekMap0( targetX, targetY, worldNumber );
			Map->SeekLand( map1.id, &land );
			GumpDisplay mapStat( s, 300, 300 );
			mapStat.SetTitle( "Item [Map]" );
			mapStat.AddData( "ID", targetID, 5 );
			mapStat.AddData( "Name", land.Name() );
			mapStat.Send( 4, false, INVALIDSERIAL );
		}
	}
	else
	{
		CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
		if( i == NULL )
			return;
		GumpDisplay dynamicStat( s, 300, 300 );
		dynamicStat.SetTitle( "Item [Dynamic]" );

		SERIAL itemSerial	= i->GetSerial();
		SERIAL contSerial	= i->GetContSerial();
		SERIAL moreVal		= i->GetMore();
		SERIAL ownerSerial	= i->GetOwner();
		UI16 itemID			= i->GetID();
		UI16 itemColour		= i->GetColour();
		int hpStuff			= ((i->GetHP())<<8) + i->GetMaxHP();
		int itemDamage		= ((i->GetLoDamage())<<8) + i->GetHiDamage();
		long int decayTime	= int(R64(int(i->GetDecayTime()-cwmWorldState->GetUICurrentTime())/1000));

		dynamicStat.AddData( "Serial", itemSerial, 3 );
		dynamicStat.AddData( "ID", itemID, 5 );
		dynamicStat.AddData( "Name", i->GetName().c_str() );
		dynamicStat.AddData( "Name2", i->GetName2() );
		dynamicStat.AddData( "Colour", itemColour, 5 );
		dynamicStat.AddData( "Cont Serial", contSerial, 3 );
		dynamicStat.AddData( "Layer", i->GetLayer() );
		dynamicStat.AddData( "Type", i->GetType() );
		dynamicStat.AddData( "Moveable", i->GetMovable() );
		dynamicStat.AddData( "More", moreVal, 3 );
		dynamicStat.AddData( "X coord", i->GetX() );
		dynamicStat.AddData( "Y coord", i->GetY() );
		dynamicStat.AddData( "Z coord", i->GetZ() );
		dynamicStat.AddData( "Amount", i->GetAmount() );
		dynamicStat.AddData( "Owner", ownerSerial, 3 );
		dynamicStat.AddData( "Privs", i->GetPriv() );
		dynamicStat.AddData( "Strength", i->GetStrength() );
		dynamicStat.AddData( "HP/Max", hpStuff, 6 );
		dynamicStat.AddData( "Damage", itemDamage, 6 );
		dynamicStat.AddData( "Defense", i->GetDef() );
		dynamicStat.AddData( "Rank", i->GetRank() );
		dynamicStat.AddData( "More X", i->GetMoreX() );
		dynamicStat.AddData( "More Y", i->GetMoreY() );
		dynamicStat.AddData( "More Z", i->GetMoreZ() );
		dynamicStat.AddData( "Poisoned", i->GetPoisoned() );
		dynamicStat.AddData( "Weight", i->GetWeight() );
		dynamicStat.AddData( "Creator", i->GetCreator() );
		dynamicStat.AddData( "Madewith", i->GetMadeWith() );
		dynamicStat.AddData( "DecayTime", decayTime );
		dynamicStat.AddData( "Decay", i->isDecayable()?1:0 );
		dynamicStat.AddData( "Good", i->GetGood() );
		dynamicStat.AddData( "RandomValueRate", i->GetRndValueRate() );
		dynamicStat.AddData( "Buy Value", i->GetBuyValue() );
		dynamicStat.AddData( "Sell Value", i->GetSellValue() );
		dynamicStat.AddData( "Carve", i->GetCarve() );
		if( i->isSpawnerList() )
			dynamicStat.AddData( "Spawnobjlist", i->GetDesc() );
		else
			dynamicStat.AddData( "Spawnobj", i->GetDesc() );
		dynamicStat.Send( 4, false, INVALIDSERIAL );
	}
}

void cTargets::CstatsTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;
	GumpDisplay charStat( s, 300, 300 );
	charStat.SetTitle( "Character" );
	UI16 charID			= i->GetID();
	UI16 charSkin		= i->GetSkin();
	SERIAL charSerial	= i->GetSerial();

	ACCOUNTSBLOCK actbTemp;
	actbTemp=i->GetAccount();

	charStat.AddData( "Serial", charSerial, 3 );
	charStat.AddData( "Body Type", charID, 5 );
	charStat.AddData( "Name", i->GetName().c_str() );
	charStat.AddData( "Skin", charSkin, 5 );
	charStat.AddData( "Account", actbTemp.wAccountIndex );
	charStat.AddData( "Privs", i->GetPriv(), 5 );
	charStat.AddData( "Strength", i->GetStrength() );
	charStat.AddData( "Dexterity", i->GetDexterity() );
	charStat.AddData( "Intelligence", i->GetIntelligence() );
	charStat.AddData( "Mana", i->GetMana() );
	charStat.AddData( "Hitpoints", i->GetHP() );
	charStat.AddData( "Stamina", i->GetStamina() );
	charStat.AddData( "X coord", i->GetX() );
	charStat.AddData( "Y coord", i->GetY() );
	charStat.AddData( "Z coord", i->GetZ() );
	charStat.AddData( "Fame", i->GetFame() );
	charStat.AddData( "Karma", i->GetKarma() );
	charStat.AddData( "Deaths", i->GetDeaths() );
	charStat.AddData( "Kills", i->GetKills() );
	charStat.AddData( "AI Type", i->GetNPCAiType()  );
	charStat.AddData( "NPC Wander", i->GetNpcWander() );
	charStat.AddData( "Weight", i->GetWeight() );
	charStat.AddData( "Poisoned", i->GetPoisoned() );
	charStat.AddData( "Poison", i->GetPoisonStrength() );
	charStat.AddData( "Hunger", i->GetHunger() );
	charStat.AddData( "Attacker", ( i->GetAttacker() != NULL ) ? i->GetAttacker()->GetSerial() : INVALIDSERIAL );
	charStat.AddData( "Target", i->GetTarg() );
	charStat.AddData( "Carve", i->GetCarve() );
	charStat.AddData( "Race", i->GetRace() );
	charStat.AddData( "RaceGate", i->GetRaceGate() );
	charStat.AddData( "CommandLevel", i->GetCommandLevel() );
	// 
	if( actbTemp.wAccountIndex != AB_INVALID_ID )
		charStat.AddData( "Last On", i->GetLastOn().c_str() );
	//
	charStat.Send( 4, false, INVALIDSERIAL );
	
#if defined( _MSC_VER )
#pragma note( "This Gumps->Open() locks up later clients, commenting out for now... what is it FOR?" )
#endif
//	Gumps->Open( s, i, 8 );
	s->statwindow( i );
}

void cTargets::WstatsTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;
	GumpDisplay wStat( s, 300, 300 );
	wStat.SetTitle( "Walking Stats" );
	SERIAL charSerial = i->GetSerial();
	UI16 charID = i->GetID();
	wStat.AddData( "Serial", charSerial, 3 );
	wStat.AddData( "Body ID", charID, 5 );
	wStat.AddData( "Name", i->GetName().c_str() );
	wStat.AddData( "X", i->GetX() );
	wStat.AddData( "Y", i->GetY() );
	char temp[15];
	sprintf( temp, "%d/%d", i->GetZ(), i->GetDispZ() );
	wStat.AddData( "Z/DispZ", temp );
	wStat.AddData( "Wander", i->GetNpcWander() );
	wStat.AddData( "FX1", i->GetFx( 1 ) );
	wStat.AddData( "FY1", i->GetFy( 1 ) );
	wStat.AddData( "FZ1", i->GetFz() );
	wStat.AddData( "FX2", i->GetFx( 2 ) );
	wStat.AddData( "FY2", i->GetFy( 2 ) );
	wStat.Send( 4, false, INVALIDSERIAL );
}

void cTargets::KillTarget( cSocket *s, UI08 layer )
{
	VALIDATESOCKET( s );
	CChar *k = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( k != NULL )
	{
		CItem *i = k->GetItemAtLayer( layer );
		if( i != NULL )
			Items->DeleItem( i );
	}
}

void cTargets::FontTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetFontType( s->AddID1() );
}


void cTargets::GhostTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;
	if( !i->IsDead() )
	{
		Effects->bolteffect( i );
		Effects->PlaySound( i, 0x0029 );
		doDeathStuff( i );
	} 
	else 
		s->sysmessage( 1028 );
}


void cTargets::BoltTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		Effects->bolteffect( i );
		Effects->PlaySound( i, 0x0029 );
	}
}

void cTargets::AmountTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetAmount( s->AddX() );
}

void cTargets::CloseTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )	
		return;
	cSocket *j = calcSocketObjFromChar( i );
	if( j != NULL )
	{
		s->sysmessage( 1029 );
		j->sysmessage( 1030 );
		Network->Disconnect( j );
	}
}

// sockets
CItem * cTargets::AddMenuTarget( cSocket *s, bool x, char *addmitem ) //Tauriel 11-22-98 updated for new items
{
	if( s == NULL )
		return NULL;

	if( s->GetDWord( 11 ) == INVALIDSERIAL ) 
		return NULL;

	CChar *mChar = s->CurrcharObj();
	CItem *c = Items->CreateScriptItem( s, addmitem, false, mChar->WorldNumber() );
	if( c == NULL ) 
		return NULL;

	return c;
}

CChar *cTargets::NpcMenuTarget( cSocket *s )
// Abaddon 17th February, 2000
// Need to return the character we've made, else summon creature at least will fail
// We make the char, but never pass it back up the chain
{
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return NULL;
	
	char *x			= s->XText();
	CChar *mChar	= s->CurrcharObj();
	return Npcs->AddNPC( s, NULL, x, mChar->WorldNumber() );
}

void cTargets::MovableTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetMovable( static_cast<SI08>(s->AddX()) );
}

void cTargets::VisibleTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial = s->GetDWord( 7 );
	if( s->GetByte( 7 ) >= 0x40 )
	{
		CItem *i = calcItemObjFromSer( serial );
		if( i != NULL )
			i->SetVisible( static_cast<SI08>(s->AddX()) );
	} 
	else 
	{
		CChar *c = calcCharObjFromSer( serial );
		if( c != NULL )
		{
			c->SetHidden( static_cast<SI08>(s->AddX()) );
			c->Update();
		}
	}
}

void cTargets::OwnerTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial = s->GetDWord( 7 );
	if( serial < BASEITEMSERIAL )
	{
		CChar *i = calcCharObjFromSer( serial );
		if( i != NULL )
			i->SetOwner( (cBaseObject *)calcCharObjFromSer( s->AddID() ) );
	}
	else
	{
		CItem *c = calcItemObjFromSer( serial );
		if( c != NULL )
			c->SetOwner( (cBaseObject *)calcCharObjFromSer( s->AddID() ) );
	}
}

void cTargets::ColorsTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;
	if( i->GetID() == 0x0FAB || i->GetID() == 0x0EFF || i->GetID() == 0x0E27 )	// dye vat, hair dye
	{
		CPDyeVat toSend = (*i);
		s->Send( &toSend );
	}
	else
		s->sysmessage( 1031 );
}

void cTargets::DvatTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i == NULL )
		return;

	CChar *mChar = s->CurrcharObj();
	if( i->isDyeable() )
	{
		if( i->GetCont() != NULL )
		{
			CChar *c = getPackOwner( i );
			if( c != NULL && c != mChar )
			{
				s->sysmessage( 1032 );
				return;
			}
		}
		i->SetColour( ( ( s->AddID1() )<<8) + s->AddID2() );
		Effects->PlaySound( s, 0x023E, true );
	}
	else
		s->sysmessage( 1033 );
}

void cTargets::AddNpcTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;

	CHARACTER npcOff;
	CChar *npc = Npcs->MemCharFree( npcOff );
	if( npc == NULL )
		return;

	npc->SetName( "Dummy" );
	npc->SetID( ( ( s->AddID1() )<<8 ) + s->AddID2() );
	npc->SetOrgID( npc->GetID() );
	npc->SetSkin( 0 );
	npc->SetSkillTitles( true );
	npc->SetLocation( s->GetWord( 11 ), s->GetWord( 13 ), s->GetByte( 16 ) + Map->TileHeight( s->GetWord( 17 ) ) );
	npc->SetNpc( true );
	npc->Update();
}

void cTargets::FreezeTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetFrozen( true );
}

void cTargets::UnfreezeTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetFrozen( false );
}

// DarkStorm: Updated AllSetTarget to process the string information sent by the client.
void cTargets::AllSetTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL Serial			= s->GetDWord( 7 );
	cBaseObject *myObject	= NULL;
	if( Serial >= BASEITEMSERIAL )
		myObject = calcItemObjFromSer( Serial );
	else
		myObject = calcCharObjFromSer( Serial );

	if( myObject == NULL )
		return;
	
	UString xtext( s->XText() );
	xtext = xtext.simplifyWhiteSpace();

	UString key		= xtext.section( " ", 0, 0 ).stripWhiteSpace();
	UString value	= xtext.section( " ", 1, 1 ).stripWhiteSpace();
	key				= key.upper();
	// Log EVERYTHING
	// Check whatever we targetted
	if( myObject->GetObjType() == OT_CHAR )
		Commands->Log( "/set", s->CurrcharObj(), (CChar*)myObject, xtext.c_str() );
	else 
	{
		// It gets a bit more complicated for items
		char Message[1024];
		sprintf( Message, "Setting '%s'on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		Commands->Log( "/set", s->CurrcharObj(), NULL, Message );
	}

	// Do Character specific stuff here...
	if( myObject->GetObjType() == OT_CHAR )
	{	
		CChar *myChar		= static_cast< CChar* >( myObject );
		cSocket *charSock	= calcSocketObjFromChar( myChar );

		// Allskills
		if( key == "ALLSKILLS" )
		{
			for( UI08 i = 0; i < ALLSKILLS; ++i )
			{
				myChar->SetBaseSkill( value.toUShort(), i );
				Skills->updateSkillLevel( myChar, i );

				if( charSock != NULL ) 
					charSock->updateskill( i );
			}
			s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myChar->GetName().c_str() );				
			return;
		}
		// OrgSkin
		else if( key == "ORGSKIN" )
		{
			myChar->SetOrgSkin( value.toUShort() );
			s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myChar->GetName().c_str() );
			return;
		}
		else
		{
			// Look for matching skills
			for( UI08 i = 0; i < ALLSKILLS; ++i )
			{
				if( key == skillname[ i ] )
				{
					myChar->SetBaseSkill( value.toUShort(), i );
					Skills->updateSkillLevel( myChar, i );

					if( charSock != NULL ) 
						charSock->updateskill( i );

					s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myChar->GetName().c_str() );				
					return;
				}
			}
		}
	}

	// Settings for normal Base Objects
	// Set the name
	if( key == "NAME" )
	{
		myObject->SetName( value.c_str() );
		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
	// Set the Strength
	else if( key == "STR" || key == "STRENGTH" )
	{
		myObject->SetStrength( value.toShort() );
	
		if( myObject->GetObjType() == OT_CHAR )
			updateStats( (CChar*)myObject, 0 );

		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
	// Set the Dexterity
	else if( key == "DEX" || key == "DEXTERITY" )
	{
		myObject->SetDexterity( value.toShort() );

		if( myObject->GetObjType() == OT_CHAR )
			updateStats( (CChar*)myObject, 0 );

		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
	// Set the intelligence
	else if( key == "INT" || key == "INTELLIGENCE" )
	{
		myObject->SetIntelligence( value.toShort() );

		if( myObject->GetObjType() == OT_CHAR )
			updateStats( (CChar*)myObject, 0 );

		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
	// Set the fame
	else if( key == "FAME" )
	{
		myObject->SetFame( value.toShort() );
		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
	// Set the karma
	else if( key == "KARMA" )
	{
		myObject->SetKarma( value.toShort() );
		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
	// Set the kills
	else if( key == "KILLS" )
	{
		myObject->SetKills( value.toShort() );
		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
	// Set the Color/Hue/Colour
	else if( key == "COLOR" || key == "COLOUR" || key == "HUE" )
	{
		myObject->SetColour( value.toUShort() );
		s->sysmessage( "Succesfully set '%s' on '%s'", xtext.c_str(), myObject->GetName().c_str() );
		return;
	}
}

void cTargets::InfoTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;

	map_st map1;
	CLand land;
	
	SI16 x = s->GetWord( 0x0B );
	SI16 y = s->GetWord( 0x0D );
//	SI08 z = s->GetByte( 0x10 );

	UI08 worldNumber = 0;
	CChar *mChar = s->CurrcharObj();
	if( mChar != NULL )
		worldNumber = mChar->WorldNumber();
	if( s->GetWord( 0x11 ) == 0 )
	{
		// manually calculating the ID's if it's a maptype
		map1 = Map->SeekMap0( x, y, worldNumber );
		Map->SeekLand( map1.id, &land );
		GumpDisplay mapStat( s, 300, 300 );
		mapStat.SetTitle( "Map Tile" );

		mapStat.AddData( "Tilenum", map1.id );
		mapStat.AddData( "Flag1", land.Flag1(), 1 );
		mapStat.AddData( "Flag2", land.Flag2(), 1 );
		mapStat.AddData( "Flag3", land.Flag3(), 1 );
		mapStat.AddData( "Flag4", land.Flag4(), 1 );
		mapStat.AddData( "Unknown1", land.Unknown1(), 1 );
		mapStat.AddData( "Unknown2", land.Unknown2(), 1 );
		mapStat.AddData( "Name", land.Name() );
		mapStat.Send( 4, false, INVALIDSERIAL );
	} 
	else
	{
		CTile tile;
		UI16 tilenum = s->GetWord( 0x11 );
		Map->SeekTile( tilenum, &tile );

		GumpDisplay statTile( s, 300, 300 );
		statTile.SetTitle( "Map Tile" );

		statTile.AddData( "Tilenum", tilenum );
		statTile.AddData( "Flag1", tile.Flag1(), 1 );
		statTile.AddData( "Flag2", tile.Flag2(), 1 );
		statTile.AddData( "Flag3", tile.Flag3(), 1 );
		statTile.AddData( "Flag4", tile.Flag4(), 1 );
		statTile.AddData( "Weight", tile.Weight() );
		statTile.AddData( "Layer", tile.Layer(), 1 );
		statTile.AddData( "Anim", tile.Animation(), 1 );
		statTile.AddData( "Unknown1", tile.Unknown1(), 1 );
		statTile.AddData( "Unknown2", tile.Unknown2(), 1 );
		statTile.AddData( "Unknown3", tile.Unknown3(), 1 );
		statTile.AddData( "Height", tile.Height(), 1 );
		statTile.AddData( "Name", tile.Name() );
		statTile.Send( 4, false, INVALIDSERIAL );
	}
	s->sysmessage( 1034 );
}

void cTargets::TweakTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CChar *c		= calcCharObjFromSer( serial );
	if( c != NULL )
		tweakCharMenu( s, c );
	else 
	{
		CItem *i = calcItemObjFromSer( serial );
		if( i != NULL )
			tweakItemMenu( s, i );
	}
}

void cTargets::LoadCannon( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;
	SERIAL moreSerial = s->AddID();
	if( i->GetMore() == moreSerial || s->AddID1() == 0xFF )
	{
		if( i->GetMoreZ() == 0 && objInRange( s, i, DIST_NEARBY ) )
		{
			i->SetMoreZ( 1 );
			s->sysmessage( 1035 );
		}
		else
		{
			if( i->GetMore( 1 ) == 0x00 ) 
				s->sysmessage( 1036 );
			else 
				s->sysmessage( 1037 );
		}
	}
}

void cTargets::SetInvulFlag( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetInvulnerable( s->AddX() == 1 );
}

void cTargets::Tiling( cSocket *s )  // Clicking the corners of tiling calls this function - Crwth 01/11/1999
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;

	if( s->ClickX() == -1 && s->ClickY() == -1 )
	{
		s->ClickX( s->GetWord( 11 ) );
		s->ClickY( s->GetWord( 13 ) );
		s->target( 0, TARGET_TILING, 1038 );
		return;
	}

	SI16 x1 = s->ClickX(), x2 = s->GetWord( 11 );
	SI16 y1 = s->ClickY(), y2 = s->GetWord( 13 );
	SI16 j;

	s->ClickX( -1 );
	s->ClickY( -1 );
	
	if( x1 > x2 ) 
	{ 
		j = x1;
		x1 = x2;
		x2 = j;
	}
	if( y1 > y2 ) 
	{ 
		j = y1; 
		y1 = y2; 
		y2 = j;
	}
	
	if( s->AddID1() == 0x40 )
	{
		switch( s->AddID2() )
		{
			case 100:
			case 102:
			case 104:
			case 106:
			case 108:
			case 110:
			case 112:
			case 114:
			case 116:
			case 118:
			case 120:
			case 122:
			case 124:
			case 126:
			case 140:
				AddTarget( s );
				return;
		}
	}

	bool pileable = false;
	UI16 addid = (UI16)(( ( s->AddID1() ) << 8 ) + s->AddID2());
	CTile tile;
	Map->SeekTile( addid, &tile );
	if( tile.Stackable() ) 
		pileable = true;

	CItem *c = NULL;
	for( SI16 x = x1; x <= x2; ++x )
	{
		for( SI16 y = y1; y <= y2; ++y ) 
		{
			c = Items->SpawnItem( NULL, s->CurrcharObj(), 1, "#", pileable, addid, 0, false, false );
			if( c == NULL ) 
				return;
			c->SetPriv( 0 );	//Make them not decay
			c->SetLocation( x, y, s->GetByte( 16 ) + Map->TileHeight( s->GetWord( 17 ) ) );
		}
	}
	s->AddID1( 0 );
	s->AddID2( 0 );
}

void cTargets::AreaCommand( cSocket *s )
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;
	
	if( s->ClickX() == -1 && s->ClickY() == -1 )
	{
		s->ClickX( s->GetWord( 11 ) );
		s->ClickY( s->GetWord( 13 ) );
		s->target( 0, TARGET_AREACOMMAND, 1040 );
		return;
	}
	
	SI16 x1 = s->ClickX(), x2 = s->GetWord( 11 );
	SI16 y1 = s->ClickY(), y2 = s->GetWord( 13 );
	
	s->ClickX( -1 );
	s->ClickY( -1 );
	
	SI16 c;
	
	if( x1 > x2 ) 
	{ 
		c = x1;
		x1 = x2;
		x2 = c;
	}
	if( y1 > y2 ) 
	{ 
		c = y1; 
		y1 = y2; 
		y2 = c;
	}

	enum AreaCommandTypes
	{
		ACT_DYE = 0,
		ACT_WIPE,
		ACT_INCX,
		ACT_INCY,
		ACT_INCZ,
		ACT_SETX,
		ACT_SETY,
		ACT_SETZ,
		ACT_SETTYPE,
		ACT_NEWBIE,
		ACT_SETSCPTRIG,
		ACT_MOVABLE,
		INVALID_CMD,
		ACT_CMDCOUNT
	};

	const std::string areaCommandStrings[ACT_CMDCOUNT] =
	{
		"dye",
		"wipe",
		"incx",
		"incy",
		"incz",
		"setx",
		"sety",
		"setz",
		"settype",
		"newbie",
		"setscptrig",
		"movable",
		""
	};

	AreaCommandTypes cmdType = INVALID_CMD;
	UString orgString( s->XText() );
	UString key, value;

	if( orgString.sectionCount( " " ) != 0 )
	{
		key		= orgString.section( " ", 0, 0 ).stripWhiteSpace();
		value	= orgString.section( " ", 1, 1 ).stripWhiteSpace();
	}
	else
		key = orgString;

	for( AreaCommandTypes k = ACT_DYE; k < INVALID_CMD; k = (AreaCommandTypes)((int)k + 1) )
	{
		if( key == areaCommandStrings[k] )
		{
			cmdType = k;
			break;
		}
	}

	if( cmdType == INVALID_CMD )
	{
		s->sysmessage( "Unknown command type" );
		return;
	}

	for( ITEM i = 0; i < cwmWorldState->GetItemCount(); ++i )
	{
		if( items[i].isFree() )
			continue;
		if( items[i].GetCont() != NULL )
			continue;
		if( items[i].GetX() >= x1 && items[i].GetX() <= x2 && items[i].GetY() >= y1 && items[i].GetY() <= y2 )
		{
			switch( cmdType )
			{
				case ACT_DYE:	// dye
					items[i].SetColour( value.toUShort() );
					break;
				case ACT_WIPE:	// wipe
					Items->DeleItem( &items[i] );
					break;
				case ACT_INCX: // incx
					MapRegion->RemoveItem( &items[i] );
					items[i].IncX( value.toShort() );
					MapRegion->AddItem( &items[i] );
					break;
				case ACT_INCY:	// incy
					MapRegion->RemoveItem( &items[i] );
					items[i].IncY( value.toShort() );
					MapRegion->AddItem( &items[i] );
					break;
				case ACT_INCZ:	// incz
					items[i].IncZ( value.toByte() );
					break;
				case ACT_SETX: // setx
					MapRegion->RemoveItem( &items[i] );
					items[i].SetX( value.toShort() );
					MapRegion->AddItem( &items[i] );
					break;
				case ACT_SETY:	// sety
					MapRegion->RemoveItem( &items[i] );
					items[i].SetY( value.toShort() );
					MapRegion->AddItem( &items[i] );
					break;
				case ACT_SETZ:	// setz
					items[i].SetZ( value.toByte() );
					break;
				case ACT_SETTYPE: // settype
					items[i].SetType( value.toUByte() );
					break;
				case ACT_NEWBIE:	// newbie
					items[i].SetNewbie( value.toUShort() != 0 );
					break;
				case ACT_SETSCPTRIG:	// set script #
					items[i].SetScriptTrigger( value.toUShort() );
					break;
				case ACT_MOVABLE:	// movable property
					items[i].SetMovable( value.toByte() );
					break;
				default:
					break;
			}
			// process command here!
		}
	}
}


void cTargets::Wiping( cSocket *s )  // Clicking the corners of wiping calls this function - Crwth 01/11/1999
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;
	
	if( s->ClickX() == -1 && s->ClickY() == -1 )
	{
		s->ClickX( s->GetWord( 11 ) );
		s->ClickY( s->GetWord( 13 ) );
		if( s->AddID1() ) 
			s->target( 0, TARGET_WIPING, 1039 );
		else 
			s->target( 0, TARGET_WIPING, 1040 );
		return;
	}
	
	SI16 x1 = s->ClickX(), x2 = s->GetWord( 11 );
	SI16 y1 = s->ClickY(), y2 = s->GetWord( 13 );
	
	s->ClickX( -1 );
	s->ClickY( -1 );
	
	SI16 c;
	
	if( x1 > x2 ) 
	{ 
		c = x1;
		x1 = x2;
		x2 = c;
	}
	if( y1 > y2 ) 
	{ 
		c = y1; 
		y1 = y2; 
		y2 = c;
	}

	ITEM i;
	if( s->AddID1() == 1  )
	{  // addid1[s]==1 means to inverse wipe
		for( i = 0; i < cwmWorldState->GetItemCount(); ++i )
		{
			if( !(items[i].GetX() >= x1 && items[i].GetX() <= x2 && items[i].GetY() >= y1 && items[i].GetY() <= y2 )
				&& items[i].GetCont() == NULL && !items[i].isWipeable() )
				Items->DeleItem( &items[i] );
		}
	}
	else 
	{
		for( i = 0; i < cwmWorldState->GetItemCount(); ++i )
		{
			if( items[i].GetX() >= x1 && items[i].GetX() <= x2 && items[i].GetY() >= y1 && items[i].GetY() <= y2 
				&& items[i].GetCont() == NULL && !items[i].isWipeable() )
				Items->DeleItem( &items[i] );
		}
	}
}

void cTargets::SquelchTarg( cSocket *s )
{
	CChar *p = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( p != NULL )
	{
		cSocket *pSock = calcSocketObjFromChar( p );
		if( pSock == NULL )
		{
			s->sysmessage( "That character is not online" );
			return;
		}
		if( p->IsGM() )
		{
			s->sysmessage( 1042 );
			return;
		}
		if( p->GetSquelched() )
		{
			p->SetSquelched( 0 );
			s->sysmessage( 1655 );
			if( pSock != NULL )
				pSock->sysmessage( 1043 );
			p->SetTimer( tPC_MUTETIME, 0 );
		} 
		else 
		{
			p->SetTimer( tPC_MUTETIME, 0 );
			p->SetSquelched( 1 );
			s->sysmessage( 1044 );
			if( pSock != NULL )
				pSock->sysmessage( 761 );
			if( s->AddID1() != 0xFF && s->AddID1() != 0 )			// 255 used to be -1, not good for UI08s
			{
				p->SetTimer( tPC_MUTETIME, BuildTimeValue( static_cast<R32>(s->AddID1()) ) );
				s->AddID1( 255 );
				p->SetSquelched( 2 );
			}
		}
	}
}


void cTargets::TeleStuff( cSocket *s )
{
	VALIDATESOCKET( s );
    static UI32 targ = INVALIDSERIAL;//What/who to tele
    if( targ == INVALIDSERIAL )
	{
		SERIAL serial = s->GetDWord( 7 );
		if( s->GetByte( 7 ) == 0xFF ) 
			return;
		targ = calcCharFromSer( serial );
		
		if( targ != INVALIDSERIAL )
		{
			targ += 1000000;
			s->target( 0, TARGET_TELESTUFF, 1045 );
		} 
		else 
		{
			targ = calcItemFromSer( serial );
			if( targ != INVALIDSERIAL )
				s->target( 0, TARGET_TELESTUFF, 1046 );
		}
		return;
	} 
	else 
	{
		if( s->GetByte( 11 ) == 0xFF )
			return;
		CHARACTER i;
		SI16 x = s->GetWord( 11 );
		SI16 y = s->GetWord( 13 );
		SI08 z = s->GetByte( 16 ) + Map->TileHeight( s->GetWord( 17 ) );
		
		if( targ > 999999 )//character
		{
			s->sysmessage( 1047 );
			i = targ - 1000000;
			chars[i].SetLocation( x, y, z );
			chars[i].Teleport();
		} 
		else 
		{
			i = targ;
			items[i].SetLocation( x, y, z );
			s->sysmessage( 1048 );
		}
		targ = INVALIDSERIAL;
	}
}

void cTargets::SwordTarget( cSocket *s )
{
	VALIDATESOCKET( s );

	CChar *p		= calcCharObjFromSer( s->GetDWord( 7 ) );
	CChar *mChar	= s->CurrcharObj();

	if( mChar == NULL )
		return;

	if( p != NULL )
	{
		if( p->GetID() == 0xCF )
		{
			// Unshorn sheep
			// -> Add Wool and change id of the Sheep
			CItem *c = Items->SpawnItemToPack( s, mChar, "0x0DF8", false );

			p->SetID( 0xDF );			
			p->Teleport();

			// Add an effect so the sheep can regain it's wool
			cDice myDice( 2, 3, 0 );
			UI32 Delay = myDice.roll();

			Effects->tempeffect( p, p, 43, static_cast<UI16>(Delay*300), 0, 0 );
		}
		else
		{
			// Already sheered
			//s->sysmessage( "" );
		}

		return;
	}
	
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;

	UI16 targetID = s->GetWord( 0x11 );

	switch( targetID )
	{
		case 0x0CD0:
		case 0x0CD3:
		case 0x0CD6:
		case 0x0CD8:
		case 0x0CDA:
		case 0x0CDD:
		case 0x0CE0:
		case 0x0CE3:
		case 0x0CE6:
		case 0x0CCA:
		case 0x0CCB:
		case 0x0CCC:
		case 0x0CCD:
		case 0x0C12:
		case 0x0CB8:
		case 0x0CB9:
		case 0x0CBA:
		case 0x0CBB:
		{
			SI16 targetX = s->GetWord( 0x0B );		// store our target x y and z locations
			SI16 targetY = s->GetWord( 0x0D );
			SI08 targetZ = s->GetByte( 0x10 );

			SI08 distZ = abs( targetZ - mChar->GetZ() );
			SI16 distY = abs( targetY - mChar->GetY() );
			SI16 distX = abs( targetX - mChar->GetX() );

			if( distY > 5 || distX > 5 || distZ > 9 )
			{
				s->sysmessage( 393 );
				return;
			}
			if( !mChar->IsOnHorse() )
				Effects->action( s, 0x0D );
			else 
				Effects->action( s, 0x1D );
			Effects->PlaySound( s, 0x013E, true );
			CItem *c = Items->SpawnItem( NULL, mChar, 1, "#", true, 0x0DE1, 0, false, false ); //Kindling
			if( c == NULL ) 
				return;

			c->SetLocation( mChar );
			s->sysmessage( 1049 );
			return;
		}
		case 0x09CC: 
		case 0x09CD: 
		case 0x09CE: 
		case 0x09CF: 
		{	
			CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) ); 
			if( i == NULL ) 
				return;		
			if( getPackOwner( i ) != mChar ) 
			{ 
				s->sysmessage( 775 ); 
				return; 
			} 
			else 
			{ 
				UI32 getAmt = GetAmount( mChar, i->GetID() ); 
				if( getAmt < 1 ) 
				{ 
					s->sysmessage( 776 ); 
					return; 
				} 
				Effects->PlaySound( s, 0x013E, true); // I'm not sure 
				CItem *c = Items->SpawnItem( s, mChar, 4, "raw fish steak", true, 0x097A, 0, true, true); 
				if( c == NULL ) 
				{ 
					return; 
				} 
				DecreaseItemAmount( i ); 
				return; 
			} 
		}
		case 0x1BDD:
		case 0x1BE0:
		{
			Skills->BowCraft( s );
			return;
		}
		case 0x2006:
		{
			CorpseTarget( s );
			return;
		}
	}
	s->sysmessage( 1050 );
}

void cTargets::CorpseTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
		return;
	bool n = false;
	if( objInRange( s, i, DIST_NEARBY ) ) 
	{
		s->AddMItem( i );
		Effects->action( s, 0x20 );
		n = true;
		if( i->GetMore( 1 ) == 0 )
		{
			i->SetMore( 1, 1 );
			if( i->GetMoreY() || i->GetCarve() != -1 )
				newCarveTarget( s, i );
		} 
		else 
			s->sysmessage( 1051 );
	}
	if( !n ) 
		s->sysmessage( 393 );
}


//o--------------------------------------------------------------------------o
//|	Function/Class	-	
//|	Date			-	09/22/2002
//|	Developer(s)	-	Unknown
//|	Company/Team	-	UOX3 DevTeam
//|	Status				-	
//o--------------------------------------------------------------------------o
//|	Description		-	Target carving system.
//|									
//|	Modification	-	unknown   	-	Human-corpse carving code added
//|									
//|	Modification	-	unknown   	-	Scriptable carving product added
//|									
//|	Modification	-	09/22/2002	-	Xuri - Fixed erroneous names for body parts 
//|									& made all body parts that are carved from human corpse	
//|									lie in same direction.
//o--------------------------------------------------------------------------o
//|	Returns				-	N/A 
//o--------------------------------------------------------------------------o	
void cTargets::newCarveTarget( cSocket *s, CItem *i )
{
	VALIDATESOCKET( s );
	bool deletecorpse = false;
	CChar *mChar = s->CurrcharObj();
	CItem *c = Items->SpawnItem( NULL, mChar, 1, "#", false, 0x122A, 0, false, false ); // add the blood puddle
	if( c == NULL ) 
		return;
	MapRegion->RemoveItem( c );

	c->SetLocation( s->AddMItem() );
	c->SetMovable( 2 );
	c->SetDecayTime( BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->SystemTimer( DECAY ) )) );

	char temp[1024];
	// if it's a human corpse
	// Sept 22, 2002 - Xuri - Corrected the alignment of body parts that are carved.
	if( i->GetMoreY() )
	{
		// create the Head
		sprintf( temp, Dictionary->GetEntry( 1058 ).c_str(), i->GetName2() );
		c = Items->SpawnItem( NULL, mChar, 1, temp, true, 0x1DAE, 0, false, false );
		if( c == NULL ) 
			return;
		c->SetLayer( 0x01 );
		c->SetCont( i );
		c->SetOwner( i->GetOwnerObj() );

		// create the Body
		sprintf( temp, Dictionary->GetEntry( 1059 ).c_str(), i->GetName2() );
		c = Items->SpawnItem( NULL, mChar, 1, temp, true, 0x1CED, 0, false, false );
		if( c == NULL ) 
			return;
		c->SetLayer( 0x01 );
		c->SetCont( i );
		c->SetOwner( i->GetOwnerObj() );

		sprintf( temp, Dictionary->GetEntry( 1057 ).c_str(), i->GetName2() );
		c = Items->SpawnItem( NULL, mChar, 1, temp, true, 0x1DAD, 0, false, false );
		if( c == NULL ) 
			return;
		c->SetLayer( 0x01 );
		c->SetCont( i );
		c->SetOwner( i->GetOwnerObj() );

		// create the Left Arm
		sprintf( temp, Dictionary->GetEntry( 1060 ).c_str(), i->GetName2() );
		c = Items->SpawnItem( NULL, mChar, 1, temp, true, 0x1D80, 0, false, false );
		if( c == NULL ) 
			return;
		c->SetLayer( 0x01 );
		c->SetCont( i );
		c->SetOwner( i->GetOwnerObj() );

		// create the Right Arm
		sprintf( temp, Dictionary->GetEntry( 1061 ).c_str(), i->GetName2() );
		c = Items->SpawnItem( NULL, mChar, 1, temp, true, 0x1DAF, 0, false, false );
		if( c == NULL ) 
			return;
		c->SetLayer( 0x01 );
		c->SetCont( i );
		c->SetOwner( i->GetOwnerObj() );

		// create the Left Leg
		sprintf( temp, Dictionary->GetEntry( 1062 ).c_str(), i->GetName2() );
		c = Items->SpawnItem( NULL, mChar, 1, temp, true, 0x1DB2, 0, false, false );
		if( c == NULL ) 
			return;
		c->SetLayer( 0x01 );
		c->SetCont( i );
		c->SetOwner( i->GetOwnerObj() );

		// create the Right Leg
		sprintf( temp, Dictionary->GetEntry( 1063 ).c_str(), i->GetName2() );
		c = Items->SpawnItem( NULL, mChar, 1, temp, true, 0x1D81, 0, false, false );
		if( c == NULL ) 
			return;
		c->SetLayer( 0x01 );
		c->SetCont( i );
		c->SetOwner( i->GetOwnerObj() );

		//human: always delete corpse!
		deletecorpse = true;
		criminal( mChar );
	} 
	else
	{
		char sect[512];
		sprintf( sect, "CARVE %i", i->GetCarve() );
		ScriptSection *toFind = FileLookup->FindEntry( sect, carve_def );
		if( toFind == NULL )
			return;
		UString tag;
		UString data;
		UI08 worldNumber = mChar->WorldNumber();
		for( tag = toFind->First(); !toFind->AtEnd(); tag = toFind->Next() )
		{
			data = toFind->GrabData();
			if( tag.upper() == "ADDITEM" )
			{
				if( data.sectionCount( "," ) != 0 )
					Items->menuAddItem( s, data.section( ",", 0, 0 ).stripWhiteSpace(), data.section( ",", 1, 1 ).stripWhiteSpace().toUShort() );
				else
					Items->menuAddItem( s, data );
			}
		}
	}

	if( deletecorpse )
	{
		for( c = i->FirstItem(); !i->FinishedItems(); c = i->NextItem() )
		{
			if( c != NULL )
			{
				c->SetCont( NULL );
				c->SetLocation( i );
				c->SetDecayTime( BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->SystemTimer( DECAY ) )) );
			}
		}
		Items->DeleItem( i );
	}
}

void cTargets::TitleTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetTitle( s->XText() );
}

void cTargets::NpcTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		s->AddID( i->GetSerial() );
		s->target( 0, TARGET_NPCFOLLOW2, "Select NPC to follow this player." );
	}
}

void cTargets::NpcTarget2( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		if( i->IsNpc() )
		{
			i->SetFTarg( calcCharFromSer( s->AddID() ) );
			i->SetNpcWander( 1 );
		}
	}
}

void cTargets::NpcRectTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		if( i->IsNpc() )
		{
			i->SetFx( static_cast<SI16>(s->AddX()), 1 );
			i->SetFy( static_cast<SI16>(s->AddY()), 1 );
			i->SetFz( -1 );
			i->SetFx( static_cast<SI16>(s->AddX2()), 2 );
			i->SetFy( static_cast<SI16>(s->AddY2()), 2 );
			i->SetNpcWander( 3 );
		}
	}
}

void cTargets::NpcCircleTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		if( i->IsNpc() )
		{
			i->SetFx( static_cast<SI16>(s->AddX()), 1 );
			i->SetFy( static_cast<SI16>(s->AddY()), 1 );
			i->SetFz( -1 );
			i->SetFx( static_cast<SI16>(s->AddX2()), 2 );
			i->SetNpcWander( 4 );
		}
	}
}

void cTargets::NpcWanderTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		if( i->IsNpc() )
			i->SetNpcWander( static_cast<SI08>(s->TempInt()) );
	}
}

void cTargets::NpcAITarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetNPCAiType( static_cast<SI16>(s->AddX()) );
}

void cTargets::xBankTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		openBank( s, i );
}

void cTargets::DupeTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	if( s->AddID1() >= 1 )
	{
		UI08 dupetimes = s->AddID1();
		CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
		if( i != NULL )
		{
			for( UI08 dupeit = 0; dupeit < dupetimes; ++dupeit )
				Commands->DupeItem( s, i, i->GetAmount() );
		}
	}
}

void cTargets::MoveToBagTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i == NULL )
		return;
	CChar *mChar = s->CurrcharObj();
	CItem *p = getPack( mChar );
	if( i->GetSerial() == serial )
	{
		if( i->GetCont() == NULL )
			MapRegion->RemoveItem( i );
		i->SetX( RandomNum( 0, 79 ) + 50 );
		i->SetY( RandomNum( 0, 79 ) + 50 );
		i->SetZ( 9 );
		if( p != NULL ) 
			i->SetCont( p );
		i->SetDecayTime( 0 );//reset decaytimer
//		i->RemoveFromSight();
	}
}

void cTargets::SellStuffTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		sendSellStuff( s, i );
}

void cTargets::ReleaseTarget( cSocket *s, SERIAL c )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( c );
	if( i != NULL )
	{
		if( !i->IsJailed() )
			s->sysmessage( 1064 );
		else
		{
			JailSys->ReleasePlayer( i );
			s->sysmessage( 1065, i->GetName().c_str() );
		}
	}
}

void cTargets::GmOpenTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *toCheck = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( toCheck == NULL )
	{
		s->sysmessage( 1066 );
		return;
	}

	CItem *i = toCheck->GetItemAtLayer( static_cast<UI08>(s->TempInt()) );
	if( i != NULL )
	{
		openPack( s, i );
		return;
	}
	s->sysmessage( 1067 );
}

void cTargets::StaminaTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		Effects->PlaySound( i, 0x01F2 );
		Effects->staticeffect( i, 0x376A, 0x09, 0x06 );
		i->SetStamina( i->GetMaxStam() );
		updateStats( i, 2 );
		return;
	}
	s->sysmessage( 1066 );
}

void cTargets::ManaTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		Effects->PlaySound( i, 0x01F2 );
		Effects->staticeffect( i, 0x376A, 0x09, 0x06 );
		i->SetMana( i->GetMaxMana() );
		updateStats( i, 1 );
		return;
	}
	s->sysmessage( 1066 );
}

void cTargets::MakeShopTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		Commands->MakeShop( i );
		s->sysmessage( 1068 );
		return;
	}
	s->sysmessage( 1069 );
}

void cTargets::JailTarget( cSocket *s, SERIAL c )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( c );
    if( i == NULL ) 
		return;

	if( i->IsJailed() )
	{
		s->sysmessage( 1070 );
		return;
	}
    if( !JailSys->JailPlayer( i, 0xFFFFFFFF ) )
		s->sysmessage( 1072 );
}

void cTargets::AttackTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *target	= calcCharObjFromSer( s->AddID() );
	CChar *target2	= calcCharObjFromSer( s->GetDWord( 7 ) );
	
	if( target2 == NULL || target == NULL ) 
		return;
	if( target2 == target )
	{
		s->sysmessage( 1073 );
		return;
	}
	Combat->AttackTarget( target, target2 );
	if( target2->IsInnocent() && target2 != target->GetOwnerObj() )
	{
		CChar *pOwner = (CChar *)target->GetOwnerObj();
		if( pOwner != NULL )
			criminal( pOwner );
	}
}

void cTargets::FollowTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *char1	= calcCharObjFromSer( s->AddID() );
	CHARACTER char2 = calcCharFromSer( s->GetDWord( 7 ) );
	if( char1 == NULL || char2 == INVALIDSERIAL )
		return;
	
	char1->SetFTarg( char2 );
	char1->SetNpcWander( 1 );
}

void cTargets::TransferTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *char1 = calcCharObjFromSer( s->AddID() );
	CChar *char2 = calcCharObjFromSer( s->GetDWord( 7 ) );
	
	if( char1 == NULL )
		return;

	if( char2 == NULL )
	{
		s->sysmessage( "Please select a valid character to transfer the house to" );
		return;
	}
	if( char1 == char2 )
	{
		s->sysmessage( "You cannot transfer it to yourself" );
		return;
	}
	
	char1->talkAll( 1074, false, char1->GetName().c_str(), char2->GetName().c_str() );
	
	char1->SetOwner( (cBaseObject *)char2 );
	char1->SetNpcWander( 1 );
	
	char1->SetFTarg( INVALIDSERIAL );
	char1->SetNpcWander( 0 );
}

void cTargets::BuyShopTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CChar *i		= calcCharObjFromSer( serial );
	if( i != NULL )
	{
		BuyShop( s, i );
		return;
	}
	s->sysmessage( 1075 );
}

bool cTargets::BuyShop( cSocket *s, CChar *c )
{
	if( s == NULL )
		return false;
	if( c == NULL )
		return false;
	CItem *buyPack		= c->GetItemAtLayer( 0x1A );
	CItem *boughtPack	= c->GetItemAtLayer( 0x1B );
	
	if( buyPack == NULL || boughtPack == NULL )
		return false;
	
	buyPack->Sort();
	boughtPack->Sort();
	c->SendToSocket( s, false, c );

	CPItemsInContainer iic( buyPack,c );	s->Send( &iic );
	CPOpenBuyWindow obw( buyPack, c );		s->Send( &obw );

	CPItemsInContainer iic2( boughtPack, c );	s->Send( &iic2 );
	CPOpenBuyWindow obw2( boughtPack, c );		s->Send( &obw2 );

	CPDrawContainer toSend;
	toSend.Model( 0x0030 );
	toSend.Serial( c->GetSerial() );
	s->Send( &toSend );
	s->statwindow( s->CurrcharObj() ); // Make sure the gold total has been sent.
	return true;
}

void cTargets::SetBuyValueTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		i->SetBuyValue( s->AddX() );
		return;
	}
	s->sysmessage( 1076 );
}

void cTargets::SetSellValueTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		i->SetSellValue( s->AddX() );
		return;
	}
	s->sysmessage( 1076 );
}

void cTargets::SetRestockTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		i->SetRestock( static_cast<UI16>(s->AddX()) );
		return;
	}
	s->sysmessage( 1076 );
}


void cTargets::permHideTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		if( i->GetHidden() == 1 )
		{
			s->sysmessage( 833 );
			return;
		}
		i->SetPermHidden( true );
		i->SetHidden( 1 );
		i->Update();
	}
}

void cTargets::unHideTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *c = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( c != NULL )
	{
		if( !c->IsGM() && !c->IsCounselor() )
			c->SetPermHidden( false );
	}
	s->AddX( 0 );
	VisibleTarget( s );	// better code reuse!
}

void cTargets::SetWipeTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetWipeable( s->AddID1() != 0 );
}
void cTargets::SetSpAttackTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetSpAttack( static_cast<SI16>(s->TempInt()) );
}

void cTargets::SetSpaDelayTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetSpDelay( static_cast<SI08>(s->TempInt()) );
}

void cTargets::SetPoisonTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetPoisonStrength( static_cast<UI08>(s->TempInt() ));
}

void cTargets::SetPoisonedTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		i->SetPoisoned( static_cast<SI08>(s->TempInt()) );
		i->SetTimer( tCHAR_POISONWEAROFF, BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->SystemTimer( POISON ) )) );
		i->SendToSocket( calcSocketObjFromChar( i ), true, i );
	}
}

void cTargets::FullStatsTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		Effects->PlaySound( i, 0x01F2 );
		Effects->staticeffect( i, 0x376A, 0x09, 0x06 );
		i->SetMana( i->GetMaxMana() );
		i->SetHP( i->GetMaxHP() );
		i->SetStamina( i->GetMaxStam() );
		updateStats( i, 0 );
		updateStats( i, 1 );
		updateStats( i, 2 );
		return;
	}
	s->sysmessage( 1077 );
}


void cTargets::SetAdvObjTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetAdvObj( static_cast<UI16>(s->TempInt()) );
}

//---------------------------------------------------------------------------o
//|	Class		:	CanTrainTarget( cSocket *s )
//|	Date		:	1-3-99
//|	Programmer	:	Antrhacks
//o---------------------------------------------------------------------------o
//| Purpose		:Used for training by NPC's
//o---------------------------------------------------------------------------o
void cTargets::CanTrainTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		if( !i->IsNpc() )
		{
			s->sysmessage( 1078 );
			return;
		}
		i->SetCanTrain( !i->CanTrain() );
	}
}

void cTargets::SetSplitTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetSplit( static_cast<UI08>(s->TempInt()) );
}

void cTargets::SetSplitChanceTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		i->SetSplitChance( static_cast<UI08>(s->TempInt()) );
}

void cTargets::AxeTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	if( s->GetDWord( 11 ) == INVALIDSERIAL )
		return;
	
	UI16 realID = s->GetWord( 0x11 );
    // [krazyglue] it may take more lines, but at least its readable and easier to debug =)
	if( realID == 0x0CD0 || realID == 0x0CD3 || realID == 0x0CD6 || realID == 0x0CD8 || realID == 0x0CDA || 
		realID == 0x0CDD || realID == 0x0CE0 || realID == 0x0CE3 || realID == 0x0CE6 || realID == 0x0D58 || 
		realID >= 0x0CCA && realID <= 0x0CCE || realID >= 0x12B8 && realID <= 0x12BB || realID == 0x0D42 ||
		realID == 0x0D43 || realID == 0x0D58 || realID == 0x0D59 || realID == 0x0D70 || realID == 0x0D85 || 
		realID == 0x0D94 || realID == 0x0D95 || realID == 0x0D98 || realID == 0x0DA4 || realID == 0x0DA8 )
	{
		Skills->TreeTarget( s );
	}
	else if( realID == 0x2006 )
		CorpseTarget( s );
    else 
		Skills->BowCraft( s );
}

void cTargets::ObjPrivTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		switch( s->AddID1() )
		{
			case 0:	i->SetDecayable( false );	break;
			case 1:	i->SetDecayable( true );	break;
			case 3:	i->SetPriv( s->AddID2() );	break;
		}
	}
}

void cTargets::SetDirTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial = s->GetDWord( 7 );
	if( s->GetByte( 7 ) >= 0x40 )
	{
		CItem *i = calcItemObjFromSer( serial );
		if( i != NULL )
		{
			i->SetDir( static_cast<UI08>(s->AddX()) );
			return;
		}
	}
	else
	{
		CChar *c = calcCharObjFromSer( serial );
		if( c != NULL )
		{
			c->SetDir( static_cast<UI08>((s->AddX()) & 0x0F) );	// make sure high-bits are cleared
			c->Update();
			return;
		}
	}
}

//o---------------------------------------------------------------------------o
//|   Function		-  void npcresurrecttarget( CChar *i )
//|   Date			-  UnKnown
//|   Programmer	-  UnKnown  (Touched tabstops by Tauriel Dec 28, 1998)
//|									
//|	Modification	-	09/22/2002	-	Xuri - Made players not appear with full 
//|									health/stamina after being resurrected by NPC Healer
//o---------------------------------------------------------------------------o
//|   Purpose     -  Resurrects a character
//o---------------------------------------------------------------------------o
void cTargets::NpcResurrectTarget( CChar *i )
{
	if( i == NULL )
		return;

	if( i->IsNpc() )
	{
		Console.Error( 2, Dictionary->GetEntry( 1079 ).c_str(), i );
		return;
	}
	cSocket *mSock = calcSocketObjFromChar( i );
	// the char is a PC, but not logged in.....
	if( mSock != NULL )
	{
		if( i->IsDead() )
		{
			Fame( i, 0 );
			Effects->PlaySound( i, 0x0214 );
			i->SetID( i->GetOrgID() );
			i->SetSkin( i->GetOrgSkin() );
			i->SetDead( false );
			// Sept 22, 2002 - Xuri
			i->SetHP( i->GetMaxHP() / 10 );
			i->SetStamina( i->GetMaxStam() / 10 );
			//
			i->SetMana( i->GetMaxMana() / 10 );
			i->SetAttacker( NULL );
			i->SetAttackFirst( false );
			i->SetWar( false );
			i->SetHunger( 6 );
			CItem *c = NULL;
			for( CItem *j = i->FirstItem(); !i->FinishedItems(); j = i->NextItem() )
			{
				if( j != NULL && !j->isFree() )
				{
					if( j->GetLayer() == 0x1A )
					{
						j->SetLayer( 0x15 );	
						i->SetPackItem( j );
					}
					if( j->GetSerial() == i->GetRobe() )
					{
						Items->DeleItem( j );
						
						c = Items->SpawnItem( NULL, i, 1, "a resurrect robe", false, 0x1F03, 0, false, false );
						if( c != NULL )
						{
							c->SetLayer( 0x16 );
							if( c->SetCont( i ) )
								c->SetDye( true );
						}
					}
				}
			}
			i->Teleport();
			UI16 targTrig = i->GetScriptTrigger();
			cScript *toExecute = Trigger->GetScript( targTrig );
			if( toExecute != NULL )
				toExecute->OnResurrect( i );
		}
		else
			mSock->sysmessage( 1080 );
	}
	else
		Console.Warning( 2, "Attempt made to resurrect a PC (serial: %x) that's not logged in", i->GetSerial() );
}


void cTargets::NewXTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i != NULL )
	{
		MapRegion->RemoveItem( i );
		i->SetX( static_cast<SI16>(s->AddX()) );
		MapRegion->AddItem( i );
	}
	
	CChar *c = calcCharObjFromSer( serial );
	if( c != NULL )
	{
		c->SetLocation( static_cast<SI16>(s->AddX()), static_cast<SI16>(c->GetY()), static_cast<SI08>(c->GetZ()) );
		c->Teleport();
	}
	
}

void cTargets::NewYTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i != NULL )
	{
		MapRegion->RemoveItem( i );
		i->SetY( static_cast<SI16>(s->AddX()) );
		MapRegion->AddItem( i );
	}
	CChar *c = calcCharObjFromSer( serial );
	if( c != NULL )
	{
		c->SetLocation( static_cast<SI16>(c->GetX()), static_cast<SI16>(s->AddX()), c->GetZ() );
		c->Teleport();
	}
	
}

void cTargets::IncXTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i != NULL )
	{
		MapRegion->RemoveItem( i );
		i->IncX( static_cast<SI16>(s->AddX()) );
		MapRegion->AddItem( i );
	}
	CChar *c = calcCharObjFromSer( serial );
	if( c != NULL )
	{
		c->SetLocation( static_cast<SI16>(c->GetX() + s->AddX()), static_cast<SI16>(c->GetY()), c->GetZ() );
		c->Teleport();
	}
}

void cTargets::IncYTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	SERIAL serial	= s->GetDWord( 7 );
	CItem *i		= calcItemObjFromSer( serial );
	if( i != NULL )
	{
		MapRegion->RemoveItem( i );
		i->IncY( static_cast<SI16>(s->AddX()) );
		MapRegion->AddItem( i );
	}
	CChar *c = calcCharObjFromSer( serial );
	if( c != NULL )
	{
		c->SetLocation( static_cast<SI16>(c->GetX()), static_cast<SI16>(c->GetY() + s->AddX()), c->GetZ() );
		c->Teleport();
	}
}

void cTargets::HouseOwnerTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *mChar = s->CurrcharObj();
	if( mChar == NULL )
		return;

	SERIAL o_serial = s->GetDWord( 7 );
	if( o_serial == INVALIDSERIAL ) 
		return;

	CChar *own		= calcCharObjFromSer( o_serial );
	cSocket *oSock	= calcSocketObjFromChar( own );
	CItem *sign		= calcItemObjFromSer( s->AddID() );
	CItem *house	= calcItemObjFromSer( sign->GetMore() );
	if( sign == NULL || house == NULL )
		return;
	sign->SetOwner( (cBaseObject *)own );
	house->SetOwner( (cBaseObject *)own );

	killKeys( house->GetSerial() );

	CItem *key = NULL;
	if( oSock != NULL ) 
	{
		key = Items->SpawnItem( oSock, own, 1, "a house key", false, 0x100F, 0, true, true );//gold key for everything else
		if( key == NULL ) 
			return;
	} 
	else 
	{
		key = Items->SpawnItem( NULL, own, 1, "a house key", false, 0x100F, 0, false, false );//gold key for everything else
		if( key == NULL ) 
			return;
		key->SetLocation( own );
	}

	key->SetMore( house->GetSerial() );
	key->SetType( 7 );

	s->sysmessage( 1081, own->GetName().c_str() );
	oSock->sysmessage( 1082, mChar->GetName().c_str() );
}

void cTargets::HouseEjectTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *c = calcCharObjFromSer( s->GetDWord( 7 ) );
	CItem *h = calcItemObjFromSer( s->AddID() );
	if( c != NULL && h != NULL ) 
	{
		SI16 sx, sy, ex, ey;
		Map->MultiArea( (CMultiObj *)h, sx, sy, ex, ey );
		if( c->GetX() >= sx && c->GetY() >= sy && c->GetX() <= ex && c->GetY() <= ey )
		{
			c->SetLocation( ex, ey, c->GetZ() );
			c->Teleport();
			s->sysmessage( 1083 );
		} 
		else 
			s->sysmessage( 1084 );
	}
}

void cTargets::HouseBanTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	// first, eject the player
	HouseEjectTarget( s );
	CChar *c = calcCharObjFromSer( s->GetDWord( 7 ) );
	CItem *h = calcItemObjFromSer( s->AddID() );
	if( c != NULL && h != NULL ) 
	{
		CMultiObj *house = static_cast<CMultiObj *>(h);
		UI08 r = AddToHouse( house, c, 1 );
		if( r == 1 ) 
			s->sysmessage( 1085, c->GetName().c_str() );
		else if( r == 2 ) 
			s->sysmessage( 1086 );
		else 
			s->sysmessage( 1087 );
	}
}

void cTargets::HouseFriendTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *c = calcCharObjFromSer( s->GetDWord( 7 ) );
	CItem *h = calcItemObjFromSer( s->AddID() );
	if( c != NULL && h != NULL ) 
	{
		CMultiObj *house = static_cast<CMultiObj *>(h);
		UI08 r = AddToHouse( house, c );
		if( r == 1 ) 
		{
			cSocket *cSock = calcSocketObjFromChar( c );
			if( cSock != NULL )
				cSock->sysmessage( 1089 );
			s->sysmessage( 1088, c->GetName().c_str() );
		} 
		else if( r == 2 ) 
			s->sysmessage( 1090 );
		else 
			s->sysmessage( 1091 );
	}
}

void cTargets::HouseUnlistTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *c = calcCharObjFromSer( s->GetDWord( 7 ) );
	CItem *h = calcItemObjFromSer( s->AddID() );
	if( c != NULL && h != NULL ) 
	{
		CMultiObj *house = static_cast<CMultiObj *>(h);
		bool r = DeleteFromHouseList( house, c );
		if( r )
			s->sysmessage( 1092, c->GetName().c_str() );
		else 
			s->sysmessage( 1093 );
	}
}

void cTargets::GlowTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL ) 
	{ 
		s->sysmessage( 1095 ); 
		return; 
	}
	CChar *mChar = s->CurrcharObj(), *k = NULL;
	if( mChar == NULL )
		return;

	if( i->GetGlow() != INVALIDSERIAL ) 
	{
		s->sysmessage( 1097 );
		return;
	}

	cBaseObject *getCont = i->GetCont();
	if( getCont != NULL )
	{
		if( getCont->GetObjType() == OT_ITEM )
			k = getPackOwner( (CItem *)getCont );
		else
			k = (CChar *)getCont;

		if( k != mChar )
		{
			s->sysmessage( 1096 );
			return;
		}
	}

	i->SetGlowColour( i->GetColour() );

	CItem *c = Items->SpawnItem( s, mChar, 1, "glower", false, 0x1647, 0, false, true ); // spawn light emitting object
	if( c == NULL )
		return;

    c->SetDir( 29 );
	c->SetVisible( 0 );
	c->SetMovable( 2 );
	MapRegion->RemoveItem( c );
	//c->SetLayer( items[i].GetLayer() );
	if( getCont == NULL || getCont->GetObjType() == OT_ITEM ) // if not equipped -> coords of the light-object = coords of the 
	{
		c->SetX( i->GetX() );
		c->SetY( i->GetY() );
		c->SetZ( i->GetZ() );
	} 
	else // if equipped -> place lightsource at player ( height= approx hand level )
	{ 
		c->SetX( mChar->GetX() );
		c->SetY( mChar->GetY() );
		c->SetZ( mChar->GetZ() + 4 );
	}

    c->SetPriv( 0 ); // doesnt decay

	i->SetGlow( c->GetSerial() ); // set glow-identifier

	mChar->SendToSocket( s, false, mChar );
	s->sysmessage( 1098 );
}

void cTargets::UnglowTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
	{ 
		s->sysmessage( 1099 ); 
		return; 
	}
	CChar *mChar = s->CurrcharObj(), *k = NULL;
	if( mChar == NULL )
		return;

	cBaseObject *getCont = i->GetCont();
	if( getCont != NULL ) 
	{      
		if( getCont->GetObjType() == OT_ITEM )
			k = getPackOwner( (CItem *)getCont );
		else
			k = (CChar *)getCont;

		if( k != mChar )
		{
			s->sysmessage( 1100 );
			return;
		}
	}

	CItem *j = calcItemObjFromSer( i->GetGlow() );
	if( i->GetGlow() == INVALIDSERIAL || j == NULL ) 
	{
		s->sysmessage( 1101 );
		return;
	}

	i->SetColour( i->GetGlowColour() );

	Items->DeleItem( j );   
	i->SetGlow( INVALIDSERIAL );
	mChar->SendToSocket( s, false, mChar );

	s->sysmessage( 1102 );
}

void cTargets::ShowSkillTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *p = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( p == NULL )
	{
		s->sysmessage( 1103 );
		return;
	}
	char spc[2]="\x20";
	char temp[1024];
	int j, k, zz, ges = 0;
	char skill_info[(ALLSKILLS+1)*40];
	UString sk;

	SI32 z = s->AddX();
	if( z < 0 || z > 3 ) 
		z = 0;
	if( z == 2 || z == 3 ) 
		sprintf( skill_info, "%s's skills:", p->GetName().c_str() ); 
	else 
		sprintf( skill_info, "%s's baseskills:", p->GetName().c_str() );
	size_t b = p->GetName().size() + 11;
	if( b > 23 ) 
		b = 23;

	UI08 c;
	for( c = b; c <= 26; ++c )
	{
		strcpy( temp, spc );
		strcpy( &skill_info[strlen( skill_info )], temp );
	}

	sk = UString::number( ges );
	sprintf( temp, "sum: %s", sk.c_str() );
	strcpy( &skill_info[strlen( skill_info )], temp );
	for( UI08 a = 0; a < ALLSKILLS; ++a )
	{
		if( z == 0 || z == 1 ) 
			k = p->GetBaseSkill( a ); 
		else 
			k = p->GetSkill( a );
		if( z == 0 || z == 2 ) 
			zz = 9; 
		else 
			zz = -1;

		if( k > zz ) // show only if skills >= 1
		{
			if( z == 2 || z == 3 ) 
				j = p->GetSkill( a )/10; 
			else 
				j = p->GetBaseSkill( a )/10; // get skill value
			sk = UString::number( j );	// skill value string in sk
			ges += j;
			sprintf( temp, "%s%s%s", skillname[a], spc, sk.c_str() );
			strcpy( &skill_info[strlen( skill_info )], temp );

			b = strlen( skillname[a] ) + sk.length() + 1; // it doesn't like \n's, so insert spaces till end of line
			if( b > 23 )
				b = 23;
			for( c = b; c <= 26; ++c )
			{
				strcpy( temp, spc );
				strcpy( &skill_info[strlen( skill_info )], temp );
			}
		}
	}

	sk = UString::number( ges );
	sprintf( temp, "sum: %s  ", sk.c_str() );
	strcpy( &skill_info[ strlen( skill_info )], temp );

	CPUpdScroll toSend( 2 );
	toSend.AddString( skill_info );
	toSend.Finalize();
	s->Send( &toSend );
}

void cTargets::FriendTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *mChar = s->CurrcharObj();
	if( mChar == NULL )
		return;

	CChar *targChar = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( targChar == NULL )
	{
		s->sysmessage( 1103 );
		return;
	}
	if( targChar->IsNpc() || !isOnline( targChar ) || targChar == mChar )
	{
		s->sysmessage( 1622 );
		return;
	}

	CChar *pet = calcCharObjFromSer( s->AddID() );
	if( Npcs->checkPetFriend( targChar, pet ) )
	{
		s->sysmessage( 1621 );
		return;
	}

	CHARLIST *petFriends = pet->GetFriendList();
	// Make sure to cover the STL response
	if( petFriends != NULL )
	{
		if( petFriends->size() >= 10 )
		{
			s->sysmessage( 1623 );
			return;
		}
	}

	pet->AddFriend( targChar );
	s->sysmessage( 1624, pet->GetName().c_str(), targChar->GetName().c_str() );

	cSocket *targSock = calcSocketObjFromChar( targChar );
	if( targSock != NULL )
		targSock->sysmessage( 1625, mChar->GetName().c_str(), pet->GetName().c_str() );
}

void cTargets::GuardTarget( cSocket *s )
//PRE:	Pet has been commanded to guard
//POST: Pet guards person, if owner currently
//DEV:	Abaddon
//DATE: 3rd October
{
	VALIDATESOCKET( s );
	CChar *mChar = s->CurrcharObj();
	if( mChar == NULL )
		return;

	CChar *petGuarding = calcCharObjFromSer( s->AddID() );
	if( petGuarding == NULL )
		return;

	CChar *charToGuard = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( charToGuard != NULL )
	{
		if( charToGuard != petGuarding->GetOwnerObj() && !Npcs->checkPetFriend( charToGuard, petGuarding ) )
		{
			s->sysmessage( 1628 );
			return;
		}
		petGuarding->SetNPCAiType( aiPET_GUARD ); // 32 is guard mode
		petGuarding->SetGuarding( charToGuard->GetSerial() );
		mChar->SetGuarded( true );
		return;
	}
	CItem *itemToGuard = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( itemToGuard != NULL && !itemToGuard->isPileable() )
	{
		CItem *p = getPack( mChar );
		if( p != NULL )
		{
			if( itemToGuard->GetCont() == p || itemToGuard->GetCont() == mChar )
			{
				itemToGuard->SetGuarded( true );
				petGuarding->SetGuarding( calcItemFromSer( itemToGuard->GetSerial() ) );
			}
		}
		else
		{
			CMultiObj *multi = findMulti( itemToGuard );
			if( multi != NULL )
			{
				if( multi->GetOwnerObj() == mChar )
				{
					itemToGuard->SetGuarded( true );
					petGuarding->SetGuarding( itemToGuard->GetSerial() );
				}
			}
			else
				s->sysmessage( 1628 );
		}
	}
}

void cTargets::ResurrectionTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
		NpcResurrectTarget( i );	// npcresurrect checks to see it's valid
}

void cTargets::HouseLockdown( cSocket *s ) // Abaddon
// PRE:		S is the socket of a valid owner/coowner and is in a valid house
// POST:	either locks down the item, or puts a message to the owner saying he's a moron
// CODER:	Abaddon
// DATE:	17th December, 1999
{
	VALIDATESOCKET( s );
	CItem *itemToLock = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( itemToLock != NULL )
	{
		if( !itemToLock->CanBeLockedDown() )
		{
			s->sysmessage( 1106 );
			return;
		}
		CItem *house = calcItemObjFromSer( s->AddID() );
		// time to lock it down!
		CMultiObj *multi = findMulti( itemToLock );
		if( multi != NULL )
		{
			if( multi != house )
			{
				s->sysmessage( 1105 );
				return;
			}
			multi->LockDownItem( itemToLock );
			return;
		}
		// not in a multi!
		s->sysmessage( 1107 );
	}
	else
		s->sysmessage( 1108 );
}

void cTargets::HouseRelease( cSocket *s ) // Abaddon
// PRE:		S is the socket of a valid owner/coowner and is in a valid house, the item is locked down
// POST:	either releases the item from lockdown, or puts a message to the owner saying he's a moron
// CODER:	Abaddon
// DATE:	17th December, 1999
{
	VALIDATESOCKET( s );
	CItem *itemToLock = calcItemObjFromSer( s->GetDWord( 7 ) );

	if( itemToLock != NULL || !itemToLock->IsLockedDown() )
	{
		CItem *house = calcItemObjFromSer( s->AddID() );	// let's find our house
		// time to lock it down!
		CMultiObj *multi = findMulti( itemToLock );
		if( multi != NULL )
		{
			if( multi != house )
			{
				s->sysmessage( 1109 );
				return;
			}
			multi->RemoveLockDown( itemToLock );	// Default as stored by the client, perhaps we should keep a backup?
			return;
		}
		// not in a multi!
		s->sysmessage( 1107 );
	}
	else
		s->sysmessage( 1108 );
}

void cTargets::ShowDetail( cSocket *s ) // Abaddon
// PRE:		S is the socket of the person getting the item information
// POST:	prints out detailed information about an item
// CODER:	Abaddon
// DATE:	11th January, 2000
{
	VALIDATESOCKET( s );
	char message[512];
	CItem *itemToCheck = calcItemObjFromSer( s->GetDWord( 7 ) );

	if( itemToCheck != NULL )
	{
		switch( itemToCheck->GetType() )
		{
			case 0:
				strcpy( message, "Default type" );
				break;
			case 1:	// container/backpack
				strcpy( message, "Container/backpack:" );
				if( itemToCheck->GetMoreB( 1 ) > 0 )
					strcat( message, "Magically trapped" );
				break;
			case 2:	// opener for castle gate 1
				strcpy( message, "Opener for order gate" );
				break;
			case 3:	// castle gate 1
				strcpy( message, "Order gate" );
				break;
			case 4:	// opener for castle gate 2
				strcpy( message, "Opener for Chaos gate" );
				break;
			case 5:	// castle gate 2
				strcpy( message, "Chaos gate" );
				break;
			case 6:	// teleporter rune
				strcpy( message, "Teleporter rune, acts like teleport was cast" );
				break;
			case 7:	// key
				strcpy( message, "Key" );
				break;
			case 8:	// locked container
				strcpy( message, "Locked container:" );
				if( itemToCheck->GetMoreB( 1 ) > 0 )
					strcat( message, "Magically trapped" );
				break;
			case 9:	// Spellbook (item 14FA)
				strcpy( message, "Spellbook" );
				break;
			case 10:	// map( item 0E EB )
				strcpy( message, "This opens a map based on the serial num of the item" );
				break;
			case 11:	// book
				sprintf( message, "A book:Entry in misc.scp: %li", itemToCheck->GetMore() );
				break;
			case 12:	// doors
				strcpy( message, "Unlocked door" );
				break;
			case 13:	// locked door
				strcpy( message, "Locked door" );
				break;
			case 14:	// food
				strcpy( message, "Food item, reduces hunger by one point" );
				break;
			case 15:	// magic wand
				sprintf( message, "Magic wand\nCircle: %u:Spell within circle: %u:Charges left: %u", itemToCheck->GetMoreX(), itemToCheck->GetMoreY(), itemToCheck->GetMoreZ() );
				break;
			case 16:	// resurrection object
				strcpy( message, "Resurrection object" );
				break;
			case 17:	// full mortar (alchemy)
				strcpy( message, "Full mortar/alchemy, more info at a later date" );
				break;
			case 18:	// enchanted item (Crystal ball)
				strcpy( message, "Enchanted item that displays a random message" );
				break;
			case 19:	// potion
				strcpy( message, "Potion: " );
				switch( itemToCheck->GetMoreY() )
				{
					case 1: // Agility Potion
						switch( itemToCheck->GetMoreZ() )
						{
							case 1:		strcat( message, "Agility potion" );	break;
							case 2:		strcat( message, "Greater Agility potion" );	break;
							default:	strcat( message, "Unknown Agility potion" );	break;
						}
						break;
					case 2: // Cure Potion
						switch( itemToCheck->GetMoreZ() )
						{
							case 1:		strcat( message, "Lesser cure potion" ); break;
							case 2:		strcat( message, "Cure potion" ); break;
							case 3:		strcat( message, "Greater Cure potion" ); break;
							default:	strcat( message, "Unknown cure potion" ); break;
						}
						break;
					case 3: // Explosion Potion
						strcat( message, "Explosion potion" );
						break;
					case 4: // Heal Potion
						switch( itemToCheck->GetMoreZ() )
						{
							case 1:		strcat( message, "Lesser Heal potion" );	break;
							case 2:		strcat( message, "Heal potion" ); break;
							case 3:		strcat( message, "Greater Heal potion" ); break;
							default:	strcat( message, "Unknown Heal potion" );
						}
						break;
					case 5: // Night Sight Potion
						strcat( message, "Night sight potion" );
						break;
					case 6: // Poison Potion
						switch( itemToCheck->GetMoreZ() )
						{
							case 0:		strcat( message, "Poison potion with no poison" ); break;
							case 1:		strcat( message, "Lesser Poison potion" ); break;
							case 2:		strcat( message, "Poison potion" ); break;
							case 3:		strcat( message, "Greater Poison potion" ); break;
							case 4:		strcat( message, "Deadly Poison potion" ); break;
							default:	strcat( message, "Unknown Poison potion" );  break;
						}
						break;
					case 7: // Refresh Potion
						switch( itemToCheck->GetMoreZ() )
						{
							case 1:		strcat( message, "Lesser Refresh potion" ); break;
							case 2:		strcat( message, "Refresh potion" );	break;
							default:	strcat( message, "Unknown Refresh potion" ); break;
						}
						break;
					case 8: // Strength Potion
						switch( itemToCheck->GetMoreZ() )
						{
							case 1:		strcat( message, "Lesser Strength potion" );	break;
							case 2:		strcat( message, "Strength potion" ); break;
							default:	strcat( message, "Unknown Strength potion" ); break;
						}
						break;
					case 9: // Mana Potion
						switch( itemToCheck->GetMoreZ() )
						{
							case 1:		strcat( message, "Lesser Mana potion" );	break;
							case 2:		strcat( message, "Mana potion" ); break;
							default:	strcat( message, "Unknown Mana potion" ); break;
						}
						break;
					default:
						strcat( message, "Unknown potion" );
						break;
				}
				break;
			case 35:	// townstone deed/stone
				if( itemToCheck->GetID() == 0x14F0 )
					strcpy( message, "Townstone deed, will make townstone" );
				else
					strcpy( message, "Townstone, use to find out information on the town" );
				break;
			case 50:	// recall rune
				if( itemToCheck->GetMoreX() == 0 && itemToCheck->GetMoreY() == 0 && itemToCheck->GetMoreZ() == 0 )	// changed, to fix, Lord Vader
					strcpy( message,"Unmarked recall rune");
				else
					strcpy( message, "This will rename a recall rune" );
				break;
			case 51:	// start gate
				CItem *otherGate1;
				otherGate1 = calcItemObjFromSer( itemToCheck->GetMoreX() );
				sprintf( message, "Start gate going to X %i Y %i Z %i", otherGate1->GetX(), otherGate1->GetY(), otherGate1->GetZ() );
				break;
			case 52:	// end gate
				CItem *otherGate2;
				otherGate2 = calcItemObjFromSer( itemToCheck->GetMoreX() );
				sprintf( message, "End gate going to X %i Y %i Z %i", otherGate2->GetX(), otherGate2->GetY(), otherGate2->GetZ() );
				break;
			case 60:	// object teleporter
				sprintf( message, "A teleporter going to X %i Y %i Z %i", itemToCheck->GetMoreX(), itemToCheck->GetMoreY(), itemToCheck->GetMoreZ() );
				break;
			case 61:	// item spawner
				sprintf( message, "Item spawner:NpcNum: %i:Respawn max time: %i:Respawn min time: %i", itemToCheck->GetMoreX(), itemToCheck->GetMoreZ(), itemToCheck->GetMoreY() );
				break;
			case 62:	// monster/npc spanwer
				sprintf( message, "Monster/NPC spawner:Amount: %i:NpcNum: %i:Respawn max time: %i:Respawn min time: %i", itemToCheck->GetAmount(), itemToCheck->GetMoreX(), itemToCheck->GetMoreZ(), itemToCheck->GetMoreY() );
				break;
			case 63:	// spawn container
				strcpy( message, "Item Spawn container:" );
				if( itemToCheck->GetMoreB( 1 ) > 0 )
					strcat( message, "Magically trapped:" );
				sprintf( message, "%sRespawn max time: %i:Respawn min time: %i", message, itemToCheck->GetMoreZ(), itemToCheck->GetMoreY() );
				break;
			case 64:	// locked spawn container
				strcpy( message, "Locked item spawn container:" );
				if( itemToCheck->GetMoreB( 1 ) > 0 )
					strcat( message, "Magically trapped:" );
				sprintf( message, "%sRespawn max time: %i:Respawn min time: %i", message, itemToCheck->GetMoreZ(), itemToCheck->GetMoreY() );
				break;
			case 65:	// unlockable item spawner container
				strcpy( message, "Unlockable item spawner container" );
				break;
			case 69:	// area spawner
				sprintf( message, "Area spawner:X +/- value: %i:Y +/- value: %i:Amount: %i:NpcNum: %i:Respawn max time: %i:Respawn min time: %i", itemToCheck->GetMore( 3 ), itemToCheck->GetMore( 4 ), itemToCheck->GetAmount(), itemToCheck->GetMoreX(), itemToCheck->GetMoreZ(), itemToCheck->GetMoreY() );
				break;
			case 80:	// single use advancement gate
				sprintf( message, "Single use advancement gate: advance.scp entry %i", itemToCheck->GetMoreX() );
				break;
			case 81:	// multi-use advancement gate
				sprintf( message, "Multi use advancement gate: advance.scp entry %i", itemToCheck->GetMoreX() );
				break;
			case 82:	// monster gate
				sprintf( message, "Monster gate: monster number %i", itemToCheck->GetMoreX() );
				break;
			case 83:	// race gates
				sprintf( message, "Race Gate:Turns into %s:", Races->Name( static_cast<RACEID>(itemToCheck->GetMoreX()) ) );
				if( itemToCheck->GetMoreY() == 1 )
					strcat( message, "Constantly reuseable:" );
				else
					strcat( message, "One time use only:" );
				if( Races->IsPlayerRace( static_cast<RACEID>(itemToCheck->GetMoreX() )) )
					strcat( message, "Player Race:" );
				else
					strcat( message, "NPC Race only:" );
				break;
			case 85:	// damage object
				sprintf( message, "Damage object:Minimum damage %i:Maximum Damage %i", itemToCheck->GetMoreX() + itemToCheck->GetMoreY(), itemToCheck->GetMoreX() + itemToCheck->GetMoreZ() );
				break;
			case 86:	// sound object
				sprintf( message, "Sound object:Percent chance of sound being played: %i:Sound effect to play: %i %i", itemToCheck->GetMoreZ(), itemToCheck->GetMoreX(), itemToCheck->GetMoreY() );
				break;
			case 87:	// trash container
				strcpy( message, "A trash container" );
				break;
			case 88:	// sound object
				sprintf( message, "Sound object that plays whenever someone is near:Soundeffect: %i\nRadius: %i\nProbability: %i", itemToCheck->GetMoreX(), itemToCheck->GetMoreY(), itemToCheck->GetMoreZ() );
				break;
			case 89:	// map change
				sprintf( message, "Map Change object that changes world:World: %u", itemToCheck->GetMore() );
				break;
			case 100:	// not documented
				strcpy( message, "Looks like hide/unhide object:More detail to come later\n" );
				break;
			case 101:	// morph object morex = npc# in npc.scp
				sprintf( message, "Morph object:Morphs char into body %x %x", itemToCheck->GetMoreX()>>8, itemToCheck->GetMoreX()%256 );
				break;
			case 102:	// unmorph
				strcpy( message, "Unmorph object, unmorphs back to body before using it as type 101, switches to 101 again" );
				break;
			case 103:	// army enlistment object
				sprintf( message, "Army enlistment object: Army #%u", itemToCheck->GetMoreX() );
				break;
			case 104:	// teleport object (use triggers now)
				sprintf( message, "Teleport object: X %u Y %u Z %u", itemToCheck->GetMoreX(), itemToCheck->GetMoreY(), itemToCheck->GetMoreZ() );
				break;
			case 105:	// drink object
				strcpy( message, "You can drink this" );
				break;
			case 117:	// backpack?
				strcpy( message, "I believe that this is a backpack, though it could be ship related" );
				break;
			case 125:	// escort npc spawner
				strcpy( message, "Escort NPC spawner, behaves like type 62/69" );
				break;
			case 181:	// fireworks wand
				sprintf( message, "A fireworks wand with %u charges left on it", itemToCheck->GetMoreX() );
				break;
			case 185:	// smoking type
				sprintf( message, "Smoking object: Duration %u secs", itemToCheck->GetMoreX() );
				break;
			case 202:	// guildstone deed
				strcpy( message, "Guildstone deed" );
				break;
			case 203:	// opens gump menu
				strcpy( message, "Opens housing gump: " );
				break;
			case 217:	// player vendor deed
				strcpy( message, "Player vendor deed");
				break;
			case 234:	// world change gate
				sprintf( message, "World change gate: %u", itemToCheck->GetMoreX() );
				break;
		}
		s->sysmessage( message );
	}
	else
		s->sysmessage( 1656 );
}

void cTargets::CommandLevel( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *targToEdit = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( targToEdit != NULL )
		targToEdit->SetCommandLevel( static_cast<UI08>(s->AddX()) );
}

void cTargets::MakeTownAlly( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *mChar = s->CurrcharObj();
	if( mChar == NULL )
		return;

	CChar *targetChar = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( targetChar == NULL )
	{
		s->sysmessage( 1110 );
		return;
	}
	UI08 srcTown = mChar->GetTown();
	UI08 trgTown = targetChar->GetTown();

//	if( srcTown == trgTown )
//		s->sysmessage( "That person already belongs to your town" );
//	else
//	{
		if( !region[srcTown]->MakeAlliedTown( trgTown ) )	
			s->sysmessage( 1111 );
//	}
}

void cTargets::MakeStatusTarget( cSocket *sock )
{
	VALIDATESOCKET( sock );
	CChar *targetChar = calcCharObjFromSer( sock->GetDWord( 7 ) );
	if( targetChar == NULL )
	{
		sock->sysmessage( 1110 );
		return;
	}
	UI08 origCommand			= targetChar->GetCommandLevel();
	commandLevel_st *targLevel	= Commands->GetClearance( sock->XText() );
	commandLevel_st *origLevel	= Commands->GetClearance( origCommand );
	
	if( targLevel == NULL )
	{
		sock->sysmessage( 1112 );
		return;
	}
	CChar *mChar = sock->CurrcharObj();
	char temp[1024], temp2[1024];

	UI08 targetCommand = targLevel->commandLevel;
	sprintf( temp, "account%i.log", mChar->GetAccount().wAccountIndex );
	sprintf( temp2, "%s has made %s a %s.\n", mChar->GetName().c_str(), targetChar->GetName().c_str(), targLevel->name );
	Console.Log( temp2, temp );

	DismountCreature( targetChar );

	if( targLevel->targBody != 0 )
	{
		targetChar->SetID( targLevel->targBody );
		targetChar->SetSkin( targLevel->bodyColour );
		targetChar->SetOrgID( targLevel->targBody );
		targetChar->SetOrgSkin( targLevel->bodyColour );
	}

	targetChar->SetPriv( targLevel->defaultPriv );
	targetChar->SetCommandLevel( targetCommand );
	
	if( targLevel->allSkillVals != 0 )
	{
		for( UI08 j = 0; j < ALLSKILLS; ++j )
		{
			targetChar->SetBaseSkill( targLevel->allSkillVals, j );
			targetChar->SetSkill( targLevel->allSkillVals, j );
		}
		targetChar->SetStrength( 100 );
		targetChar->SetDexterity( 100 );
		targetChar->SetIntelligence( 100 );
		targetChar->SetStamina(  100 );
		targetChar->SetHP( 100 );
		targetChar->SetMana( 100 );
	}
	char *playerName = new char[targetChar->GetName().size() + 1];
	strcpy( playerName, targetChar->GetName().c_str() );

	if( targetCommand != 0 && targetCommand != origCommand )
	{
		if( origLevel != NULL )
		{	// Strip name off it
			if( !strnicmp( origLevel->name, playerName, strlen( origLevel->name ) ) )
				playerName += ( strlen( origLevel->name ) + 1 );
		}
		sprintf( temp, "%s %s", targLevel->name, playerName );
		targetChar->SetName( temp );
	}
	else if( origCommand != 0 )
	{
		if( origLevel != NULL )
		{	// Strip name off it
			if( !strnicmp( origLevel->name, playerName, strlen( origLevel->name ) ) )
				playerName += ( strlen( origLevel->name ) + 1 );
		}
		strcpy( temp, (const char*)playerName );
		targetChar->SetName( temp );
	}
	CItem *retitem = NULL;
	CItem *mypack = getPack( targetChar );

	if( targLevel->stripOff )
	{
		for( CItem *z = targetChar->FirstItem(); !targetChar->FinishedItems(); z = targetChar->NextItem() )
		{
			if( z != NULL )
			{
				switch( z->GetLayer() )
				{
					case 0x0B:
					case 0x10:
						Items->DeleItem( z );
						break;
					case 0x15:
					case 0x1D:
						break;
					default:
						if( mypack == NULL )
							mypack = getPack( targetChar );
						if( mypack == NULL )
						{
							CItem *iMade = Items->SpawnItem( NULL, targetChar, 1, "#", false, 0x0E75, 0, false, false );
							targetChar->SetPackItem( iMade );
							if( iMade == NULL ) 
								return;
							iMade->SetLayer( 0x15 );
							if( iMade->SetCont( targetChar ) )
							{
								iMade->SetType( 1 );
								iMade->SetDye( true );
								mypack = iMade;
								retitem = iMade;
							}
						}
						z->SetX( ( RandomNum( 0, 79 ) + 50 ) );
						z->SetY( ( RandomNum( 0, 79 ) + 50 ) );
						z->SetZ( 9 );
						z->SetCont( mypack );

//						z->RemoveFromSight();
						break;
				}
			}
		}
	}
	if( targLevel->targBody != 0 )
		targetChar->Teleport();
	else
		targetChar->Update();
	delete [] playerName;
}

void cTargets::SmeltTarget( cSocket *s )
{
	VALIDATESOCKET( s );
#if defined( _MSC_VER )
#pragma note( "Smelting needs to be heavily updated" )
#endif
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL || i->GetCont() == NULL )
		return;
	if( i->GetCreator() == INVALIDSERIAL )
	{
		s->sysmessage( 1113 );
		return;
	}
	UI16 iMadeFrom = i->EntryMadeFrom();

	createEntry *ourCreateEntry = Skills->FindItem( iMadeFrom );
	if( iMadeFrom == 0 || ourCreateEntry == NULL )
	{
		s->sysmessage( 1114 );
		return;
	}

	CChar *mChar = s->CurrcharObj();

	R32 avgMin = ourCreateEntry->AverageMinSkill();
	if( mChar->GetSkill( MINING ) < avgMin )
	{
		s->sysmessage( 1115 );
		return;
	}
	R32 avgMax = ourCreateEntry->AverageMaxSkill();

	Skills->CheckSkill( mChar, MINING, (SI16)avgMin, (SI16)avgMax );

	R32 sumAmountRestored = 0.0f;

	for( UI32 skCtr = 0; skCtr < ourCreateEntry->resourceNeeded.size(); ++skCtr )
	{
		UI16 amtToRestore = ourCreateEntry->resourceNeeded[skCtr].amountNeeded / 2;
		UI16 itemID = ourCreateEntry->resourceNeeded[skCtr].itemID;
		UI16 itemColour = ourCreateEntry->resourceNeeded[skCtr].colour;
		sumAmountRestored += amtToRestore;
		Items->SpawnItem( s, mChar, amtToRestore, "#", true, itemID, itemColour, true, true );
	}

	s->sysmessage( 1116, sumAmountRestored );
	Items->DeleItem( i );
}

void cTargets::IncZTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CItem *i = calcItemObjFromSer( s->GetDWord( 7 ) );
	if( i != NULL )
	{
		i->IncZ( static_cast<SI16>(s->AddX()) );
		return;
	}
	CChar *c = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( c != NULL )
	{
		c->SetLocation( c->GetX(), c->GetY(), static_cast<SI08>(c->GetZ() + s->AddX()) );
		c->Teleport();
	}
}

void cTargets::DeleteCharTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *c = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( s->CurrcharObj() == c )
	{
		s->sysmessage( "You cannot delete yourself" );
		return;
	}
	if( c != NULL )
	{
		if( c->IsNpc() )
		{
			s->sysmessage( 1658 );
			return;
		}
		if( isOnline( c ) )
		{
			cSocket *tSock = calcSocketObjFromChar( c );
			if( tSock != NULL )
				tSock->sysmessage( 1659 );
			Network->Disconnect( tSock );
		}
		Npcs->DeleteChar( c );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cTargets::PossessTarget( cSocket *s ) 
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Allow a GM to take over the body of another Character
//o---------------------------------------------------------------------------o
void cTargets::PossessTarget( cSocket *s ) 
{
	VALIDATESOCKET( s );
	s->sysmessage( 1635 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void cTargets::RemoveShop( cSocket *s )
//|	Programmer	-	Abaddon
//o---------------------------------------------------------------------------o
//|	Purpose		-	Removes shopkeeper layers from a character
//o---------------------------------------------------------------------------o
void cTargets::RemoveShopTarget( cSocket *s )
{
	VALIDATESOCKET( s );
	CChar *i = calcCharObjFromSer( s->GetDWord( 7 ) );
	if( i == NULL )
	{
		s->sysmessage( 366 );
		return;
	}

	i->SetShop( false );
	CItem *buyLayer   = i->GetItemAtLayer( 0x1A );
	CItem *sellLayer  = i->GetItemAtLayer( 0x1B );
	CItem *otherLayer = i->GetItemAtLayer( 0x1C );
	if( buyLayer != NULL )
		Items->DeleItem( buyLayer );
	if( sellLayer != NULL )
		Items->DeleItem( sellLayer );
	if( otherLayer != NULL )
		Items->DeleItem( otherLayer );
	s->sysmessage( 367 );
}

void cTargets::SpawnObjTarget( cSocket *s, bool isList )
{
	VALIDATESOCKET( s );
	SERIAL ser = s->GetDWord( 7 );
	if( ser != INVALIDSERIAL && ser >= BASEITEMSERIAL )
	{	// valid item
		CItem *i = calcItemObjFromSer( ser );
		if( i != NULL )
		{	// valid item
			i->SetDesc( s->XText() );
			i->SetSpawnerList( isList );
		}
	}
}

