#include "uox3.h"
#include "townregion.h"
#include "cRaces.h"
#include "cServerDefinitions.h"
#include "cSpawnRegion.h"
#include "skills.h"
#include "speech.h"
#include "ssection.h"
#include "gump.h"
#include "mapstuff.h"
#include "scriptc.h"
#include "fileio.h"
#include "packets.h"

#ifndef va_start
	#include <cstdarg>
#endif

// Implementation of town regions

const SI08 MAYOR = 0x01;
const SI08 ENEMY = 0x02;
const SI08 JOINER = 0x03;

cTownRegion::cTownRegion( UI08 region ) : race( 0 ), weather( 255 ), priv( 0 ), regionNum( region ), midilist( 0 ),
mayorSerial( INVALIDSERIAL ), taxedResource( 0x0EED ), taxedAmount( 0 ), goldReserved( 0 ), guardsPurchased( 0 ),
resourceCollected( 0 ), visualAppearance( WRLD_SPRING ), health( 30000 ), timeToElectionClose( 0 ), timeToNextPoll( 0 ), 
timeSinceGuardsPaid( 0 ), timeSinceTaxedMembers( 0 ), worldNumber( 0 ), jsScript( 0xFFFF )
{
	guards.resize( 0 );
	townMember.resize( 0 );
	alliedTowns.resize( 0 );
	memset( goodsell, 0, sizeof( goodsell[0] ) * 256 );
	memset( goodbuy,  0, sizeof(  goodbuy[0] ) * 256 );
	memset( goodrnd1, 0, sizeof( goodrnd1[0] ) * 256 );
	memset( goodrnd2, 0, sizeof( goodrnd2[0] ) * 256 );
	name = Dictionary->GetEntry( 1117 );
	strcpy( guardowner, Dictionary->GetEntry( 1118 ).c_str() );
}

cTownRegion::~cTownRegion()
{
	guards.resize( 0 );
	townMember.resize( 0 );
	alliedTowns.resize( 0 );
}

bool cTownRegion::Load( Script *ss )
// ss is a script section containing all the data!
{
	size_t location = 0xFFFFFFFF;
	size_t guardUpper;
	char temp[512];
	UString tag;
	UString data;
	sprintf( temp, "TOWNREGION %i", regionNum );
	if( !ss->isin( temp ) )	// doesn't exist
		return false;

	ScriptSection *target = ss->FindEntry( temp );
	for( tag = target->First(); !target->AtEnd(); tag = target->Next() )
	{
		data = target->GrabData();
		switch( (tag.data()[0]) )
		{
			case 'A':
				if( tag.upper() == "ALLYTOWN" )
				{
					guardUpper = alliedTowns.size();
					alliedTowns.resize( guardUpper + 1 );
					alliedTowns[guardUpper] = data.toUByte();
				}
				break;
			case 'E':
				if( tag.upper() == "ELECTIONTIME" )
					timeToElectionClose = data.toLong();
				break;
			case 'G':
				if( tag.upper() == "GUARDOWNER" )
					strncpy( guardowner, data.c_str(), 49 );
				else if( tag.upper() == "GUARD" )	// load our purchased guard
				{
					guardUpper = guards.size();
					guards.resize( guardUpper + 1 );
					guards[guardUpper] = data.toLong();
				}
				else if( tag.upper() == "GUARDSBOUGHT" ) // num guards bought
					guardsPurchased = data.toShort();
				break;
			case 'H':
				if( tag.upper() == "HEALTH" )
					health = data.toShort();
				break;
			case 'M':
				if( tag.upper() == "MEMBER" )
				{
					location = townMember.size();
					townMember.resize( location + 1 );
					townMember[location].targVote = INVALIDSERIAL;
					townMember[location].townMember = data.toULong();
				}
				else if( tag.upper() == "MAYOR" )
					mayorSerial = data.toULong();
				break;
			case 'N':
				if( tag.upper() == "NAME" )
					name = data.substr( 0, 49 );
				break;
			case 'P':
				if( tag.upper() == "PRIV" )
					priv = data.toUByte();
				else if( tag.upper() == "POLLTIME" )
					timeToNextPoll = data.toLong();
				break;
			case 'R':
				if( tag.upper() == "RACE" )
					race = data.toUShort();
				else if( tag.upper() == "RESOURCEAMOUNT" )
					goldReserved = data.toLong();
				else if( tag.upper() == "RESOURCECOLLECTED" )
					resourceCollected = data.toLong();
				break;
			case 'T':
				if( tag.upper() == "TAXEDID" )
					taxedResource = data.toUShort();
				else if( tag.upper() == "TAXEDAMOUNT" )
					taxedAmount = data.toUShort();
				else if( tag.upper() == "TIMET" )
					timeSinceTaxedMembers = data.toLong();
				else if( tag.upper() == "TIMEG" )
					timeSinceGuardsPaid = data.toLong();
				break;
			case 'V':
				if( tag.upper() == "VOTE" && location != 0xFFFFFFFF )
					townMember[location].targVote = data.toULong();
				break;
			case 'W':
				if( tag.upper() == "WEATHER" )
					weather = data.toUByte();
				else if( tag.upper() == "WORLD" )
					worldNumber = data.toUByte();
				break;
		}
	}
	return true;
}
bool cTownRegion::Save( FILE *fp )
// entry is the region #, fp is the file to save in
{
	fprintf( fp, "[TOWNREGION %i]\n{\n", regionNum );
	fprintf( fp, "WEATHER=%i\n", weather );
	fprintf( fp, "RACE=%i\n", race );
	fprintf( fp, "NAME=%s\n", name.c_str() );
	fprintf( fp, "GUARDOWNER=%s\n", guardowner );
	fprintf( fp, "MAYOR=%i\n", mayorSerial );
	fprintf( fp, "PRIV=%i\n", priv );
	fprintf( fp, "RESOURCEAMOUNT=%i\n", goldReserved );
	fprintf( fp, "TAXEDID=%x\n", taxedResource );
	fprintf( fp, "TAXEDAMOUNT=%i\n", taxedAmount );
	fprintf( fp, "GUARDSPURCHASED=%i\n", guardsPurchased );
	fprintf( fp, "TIMEG=%li\n", timeSinceGuardsPaid );		// time since guards paid
	fprintf( fp, "TIMET=%li\n", timeSinceTaxedMembers );	// time since taxed
	fprintf( fp, "RESOURCECOLLECTED=%li\n", resourceCollected );	// amount of taxes we've collected
	fprintf( fp, "HEALTH=%i\n", health );
	fprintf( fp, "ELECTIONTIME=%li\n", timeToElectionClose );	// time to election close
	fprintf( fp, "POLLTIME=%li\n", timeToNextPoll );	// time to next poll
	fprintf( fp, "WORLD=%i\n", worldNumber );	// time to next poll
	for( UI32 counter = 0; counter < townMember.size(); ++counter )
	{
		fprintf( fp, "MEMBER=%i\n", townMember[counter].townMember );
		fprintf( fp, "VOTE=%i\n", townMember[counter].targVote );
	}
	for( UI32 guardCount = 0; guardCount < guards.size(); ++guardCount )
	{
		fprintf( fp, "GUARD=%i\n", guards[guardCount] );
	}
	for( UI32 allyCount = 0; allyCount < alliedTowns.size(); ++allyCount )
	{
		fprintf( fp, "ALLYTOWN=%i\n", alliedTowns[allyCount] );
	}

	fprintf( fp, "}\n\n" );
	return true;
}
void cTownRegion::CalcNewMayor( void )
// There has got to be a better way than this hideous O(n^2) algy
{
	if( townMember.size() == 0 )
		return;
	// if there are no members, there are no new mayors
	std::vector< int > votes;
	votes.resize( townMember.size() );
	for( UI32 counter = 0; counter < votes.size(); ++counter )
	{
		votes[counter] = 0;	// init the count before adding
		for( UI32 counter2 = 0; counter2 < votes.size(); ++counter2 )
		{
			if( townMember[counter].townMember == townMember[counter2].targVote )
				++votes[counter];
		}
	}
	int maxIndex = 0;
	for( UI32 indexCounter = 1; indexCounter < votes.size(); ++indexCounter )
	{
		if( votes[indexCounter] > votes[maxIndex] )
			maxIndex = indexCounter;
	}
	// maxIndex is now our new mayor!
	CChar *oldMayor = GetMayor();
	if( mayorSerial == townMember[maxIndex].townMember )
	{
		cSocket *targSock = calcSocketObjFromChar( calcCharObjFromSer( mayorSerial ) );
		if( targSock != NULL )
			targSock->sysmessage( 1119 );
		// welcome the mayor back for another term
	}
	if( votes[maxIndex] > 0 )
		mayorSerial = townMember[maxIndex].townMember;
	else
		mayorSerial = 0xFFFFFFF;

	CChar *newMayor = GetMayor();
	if( oldMayor != NULL )
		oldMayor->SetTownpriv( 1 );
	if( newMayor != NULL )
		newMayor->SetTownpriv( 2 );	// set mayor priv last
	if( newMayor != oldMayor )
	{
		if( oldMayor == NULL && newMayor != NULL )
			TellMembers( 1120, newMayor->GetName().c_str() );
		else if( oldMayor != NULL && newMayor == NULL )
			TellMembers( 1121 );
		else if( oldMayor != NULL && newMayor != NULL )
			TellMembers( 1122, oldMayor->GetName().c_str(), newMayor->GetName().c_str() );
		else
			TellMembers( 1123 );
	}
	else if( newMayor != NULL )
		TellMembers( 1124, oldMayor->GetName().c_str() );
	else
		TellMembers( 1123 );
}
CChar * cTownRegion::GetMayor( void )
// returns the index into chars[] of the mayor
{
	return calcCharObjFromSer( mayorSerial );
}

SERIAL cTownRegion::GetMayorSerial( void ) const
// returns the mayor's serial
{
	return mayorSerial;
}

bool cTownRegion::AddAsTownMember( CChar& toAdd )
// toAdd is the character to add
{
	if( Races->CompareByRace( toAdd.GetRace(), race ) == 1 )	// if we're racial enemies
		return false;	// we can't have a racial enemy in the town!

	for( UI32 counter = 0; counter < townMember.size(); ++counter )		// exhaustive error checking
	{
		if( townMember[counter].townMember == toAdd.GetSerial() )
			return false;	// already exists in our town!
	}
	townMember.resize( townMember.size() + 1 );
	townMember[townMember.size()-1].townMember = toAdd.GetSerial();
	townMember[townMember.size()-1].targVote = INVALIDSERIAL;
	townMember[townMember.size()-1].PlayerBank = NULL;
	toAdd.SetTown( regionNum );
	toAdd.SetTownpriv( 1 );	// set as resident
	return true;
}
bool cTownRegion::RemoveTownMember( CChar& toAdd )
{
	if( toAdd.GetTown() != regionNum )
		return false;	// not in our town

	for( UI32 counter = 0; counter < townMember.size(); ++counter )
	{
		if( toAdd.GetSerial() == townMember[counter].townMember )
		{
			RemoveCharacter( counter );
			toAdd.SetTown( 255 );	// don't know what default town is, now it's 255 :>
			toAdd.SetTownpriv( 0 );
			return true;
		}
	}
	return false;	// we're not in our town
}

bool cTownRegion::InitFromScript( int& l )
{
	ScriptSection *Regions	= NULL;
	UString tag;
	UString data;
	int actgood				= -1;
	bool orePrefLoaded		= false;
	guards.resize( 10 );
	char sect[512];
	UI32 a;
	for( a = 0; a < 10; ++a )
	{
		guards[a] = RandomNum( 1000, 1001 );
	}
	a					= 0;
	minColourSkill		= 600;
	chanceFindBigOre	= 80;
	chanceColourOre		= 10;
	sprintf( sect, "REGION %i", regionNum );
	Regions = FileLookup->FindEntry( sect, regions_def );
	if( Regions == NULL ) 
		return false;
	else
	{
		for( tag = Regions->First(); !Regions->AtEnd(); tag = Regions->Next() )
		{
			data = Regions->GrabData();
			switch( (tag.data()[0]) )
			{
				case 'a':
				case 'A':
					if( tag.upper() == "ABWEATH" )
					{
						weather = data.toUByte();
					}
					else if( tag.upper() == "APPEARANCE" )
					{
						visualAppearance = WRLD_COUNT;
						for( WorldType wt = WRLD_SPRING; wt < WRLD_COUNT; wt = (WorldType)(wt + (WorldType)1) )
						{
							if( data == WorldTypeNames[wt] )
							{
								visualAppearance = wt;
								break;
							}
						}
						if( visualAppearance == WRLD_COUNT )
							visualAppearance = WRLD_UNKNOWN;
					}
					break;
				case 'b':
				case 'B':
					if( tag.upper() == "BUYABLE" )
					{
						if( actgood >-1 )
							goodbuy[actgood] = data.toLong();
						else
							Console.Error( 2, "regions dfn -> You must write BUYABLE after GOOD <num>!" );
					}
					break;

				case 'c':
				case 'C':
					if( tag.upper() == "COLOURMINSKILL" )
						minColourSkill = data.toShort();
					else if( tag.upper() == "CHANCEFORBIGORE" )
						chanceFindBigOre = data.toUByte();
					else if( tag.upper() == "CHANCEFORCOLOUR" )
						chanceColourOre = data.toUByte();
					break;

				case 'd':
				case 'D':
					if( tag.upper() == "DUNGEON" )
					{
						if( data.toUByte() != 0 )
							priv |= 0x80;
					}
					break;
				case 'e':
				case 'E':
					if( tag.upper() == "ESCORTS" ) 
					{
						// Load the region number in the global array of valid escortable regions
						if( data.toUByte() == 1 )
						{
							// Store the region index into the valid escort region array
							cwmWorldState->SetValidEscortRegion( cwmWorldState->GetEscortRegions(), regionNum );
							cwmWorldState->IncEscortRegions();
						}
					} // End - Dupois
					break;

				case 'g':
				case 'G':
					if( tag.upper() == "GUARDNUM" )
					{
						if( a < 10 )
						{
							if( a >= guards.size() )
								guards.resize( a + 1 );

							guards[a++] = data.toLong();
						}
						else
							Console.Error( 2, "Region %i has more than 10 'GUARDNUM'.  The ones after 10 will be discarded", regionNum );
					}
					else if( tag.upper() == "GUARDOWNER" )
						strcpy( guardowner, data.c_str() );
					else if( tag.upper() == "GUARDED" )
					{
						if( data.toUByte() ) 
							priv |= 0x01;   
					}
					else if( tag.upper() == "GATE" )
					{
						if( data.toUByte() ) 
							priv |= 0x04;
					}
					else if( tag.upper() == "GOOD" )
						actgood = data.toLong();
					break;
				case 'm':
				case 'M':
					if( tag.upper() == "MIDILIST" )
						midilist = data.toLong();
					else if( tag.upper() == "MAGICDAMAGE" )
					{
						if( !data.toUByte() )
							priv |= 0x40;
					}
					else if( tag.upper() == "MARK" )
					{
						if( data.toUByte() )
							priv |= 0x02;
					}
					break;
				case 'n':
				case 'N':
					if( tag.upper() == "NAME" )
					{
						name = data;
						actgood = -1; // Magius(CHE)
					}
					break;
				case 'o':
				case 'O':
					if( tag.upper() == "OREPREF" )
					{
						UString numPart;
						char oreName[512];
						orePref toPush;
						data = data.simplifyWhiteSpace();
						strcpy( oreName, data.section( " ", 0, 0 ).c_str() );
						toPush.oreIndex			= Skills->GetOreIndex( oreName );
						if( toPush.oreIndex != 0xFFFFFFFF )
						{
							if( data.sectionCount( " " ) == 0 )
								toPush.percentChance = 100 / Skills->GetNumberOfOres();
							else
								toPush.percentChance = data.section( " ", 1, 1 ).toByte();
							orePreferences.push_back( toPush );
							orePrefLoaded = true;
						}
						else
							Console.Error( 2, "Invalid ore preference in region %i as %s", regionNum, oreName );
					}
					break;
				case 'r':
				case 'R':
					if( tag.upper() == "RECALL" )
					{
						if( data.toUByte() ) 
							priv |= 0x08;
					}
					else if( tag.upper() == "RANDOMVALUE" )
					{
						if( actgood > -1 )
						{
							if( data.sectionCount( " " ) != 0 )
							{
								goodrnd1[actgood] = data.section( " ", 0, 0 ).toLong();
								goodrnd2[actgood] = data.section( " ", 1, 1 ).toLong();
							}
							else
							{
								goodrnd1[actgood] = data.toLong();
								goodrnd2[actgood] = goodrnd1[actgood];
							}
							if( goodrnd2[actgood] < goodrnd1[actgood] )
							{
								Console.Error( 2, " regions dfn -> You must write RANDOMVALUE NUM2[%i] greater than NUM1[%i].", goodrnd2[actgood], goodrnd1[actgood] );
								goodrnd2[actgood] = goodrnd1[actgood] = 0;
							}
						}
						else
							Console.Error( 2, " regions dfn -> You must write RANDOMVALUE after GOOD <num>!" );
					}
					else if( tag.upper() == "RACE" )
						race = data.toUShort();
					break;

				case 's':
				case 'S':
					if( tag.upper() == "SELLABLE" )
					{
						if( actgood > -1 )
							goodsell[actgood] = data.toLong();
						else
							Console.Error( 2, " regions dfn -> You must write SELLABLE after GOOD <num>!" );
					}
					else if( tag.upper() == "SPAWN" )
					{
						cSpawnRegion *spawnReg = spawnregion[cwmWorldState->GetTotalSpawnRegions()];
						if( spawnReg != NULL )
						{
							spawnReg->SetX1( cwmWorldState->location[l-1].x1 );
							spawnReg->SetY1( cwmWorldState->location[l-1].y1 );
							spawnReg->SetX2( cwmWorldState->location[l-1].x2 );
							spawnReg->SetY2( cwmWorldState->location[l-1].y2 );
							FileIO->LoadPreDefSpawnRegion( cwmWorldState->GetTotalSpawnRegions(), data.c_str() );
						}
						cwmWorldState->IncTotalSpawnRegions();
					}
					else if( tag.upper() == "SCRIPT" )
						jsScript = data.toUShort();
					break;
				case 'w':
				case 'W':
					if( tag.upper() == "WORLD" )
						worldNumber = data.toUByte();
					break;
				case 'x':
				case 'X':
					if( tag.upper() == "X1" )
						cwmWorldState->location[l].x1 = data.toShort();
					else if( tag.upper() == "X2" )
						cwmWorldState->location[l].x2 = data.toShort();
					break;
				case 'y':
				case 'Y':
					if( tag.upper() == "Y1" )
						cwmWorldState->location[l].y1 = data.toShort();
					else if( tag.upper() == "Y2" )
					{
						cwmWorldState->location[l].y2 = data.toShort();
						cwmWorldState->location[l].region = regionNum;
						++l;
					}
					break;
			}
		}
	}
	if( !orePrefLoaded )
	{
		orePref toLoad;
		size_t numOres = Skills->GetNumberOfOres();
		for( size_t myCounter = 0; myCounter < Skills->GetNumberOfOres(); ++myCounter )
		{
			toLoad.oreIndex			= myCounter;
			toLoad.percentChance	= (UI08)(100 / numOres);
			orePreferences.push_back( toLoad );
		}
	}
	return true;
}

bool cTownRegion::IsGuarded( void ) const
{
	return ( ( priv&0x01 ) == 0x01 );
}

bool cTownRegion::CanMark( void ) const
{
	return ( ( priv&0x02 ) == 0x02 );
}

bool cTownRegion::CanGate( void ) const
{
	return ( ( priv&0x04 ) == 0x04 );
}

bool cTownRegion::CanRecall( void ) const
{
	return ( ( priv&0x08 ) == 0x08 );
}

bool cTownRegion::CanCastAggressive( void ) const
{
	return ( ( priv&0x40 ) == 0x40 );
}

std::string cTownRegion::GetName( void ) const
{
	return name;
}
const char *cTownRegion::GetOwner( void ) const
{
	return guardowner;
}

weathID cTownRegion::GetWeather( void ) const
{
	return weather;
}

SI32 cTownRegion::GetGoodSell( UI08 index ) const
{
	return goodsell[index];
}
SI32 cTownRegion::GetGoodBuy( UI08 index ) const
{
	return goodbuy[index];
}
SI32 cTownRegion::GetGoodRnd1( UI08 index ) const
{
	return goodrnd1[index];
}
SI32 cTownRegion::GetGoodRnd2( UI08 index ) const
{
	return goodrnd2[index];
}

SI32 cTownRegion::GetMidiList( void ) const
{
	return midilist;
}

long cTownRegion::GetRandomGuard( void )
{
	size_t guardCount = guards.size();
	if( guardCount == 0 )
		return 1000;
	else
		return ( guards[rand()%guardCount] );
}

bool cTownRegion::DisplayTownMenu( CItem *used, cSocket *sock, SI08 flag )
{
	if( flag == MAYOR )
	{
		SendMayorGump( sock );
		return true;
	}
	CChar *tChar = sock->CurrcharObj();
	if( !IsMemberOfTown( tChar ) )
	{
		if( Races->CompareByRace( tChar->GetRace(), race ) > 0 )	// if we're racial enemies
		{
			if( tChar->GetTown() != 255 )
				SendEnemyGump( sock );
			else
				sock->sysmessage( 1125 );
			return true;
		}
		else if( tChar->GetTown() != 255 ) // another town person
		{
			if( tChar->GetTown() == regionNum )	// they think we're in this region!!!
			{
				if( !AddAsTownMember( (*tChar) ) )
				{
					sock->sysmessage( 1126 );
					return false;
				}
				DisplayTownMenu( used, sock, flag );
				return true;
			}
			SendBasicInfo( sock );
			return true;
		}
		else
		{
			SendPotentialMember( sock );
			return true;
		}
	}	

	SendDefaultGump( sock );
	return true;
}

bool cTownRegion::IsMemberOfTown( CChar *player ) const
{
	if( player == NULL )
		return false;
	for( UI32 counter = 0; counter < townMember.size(); ++counter )
	{
		if( townMember[counter].townMember == player->GetSerial() )
			return true;
	}
	return false;
}

void cTownRegion::SendEnemyGump( cSocket *sock )
{
	CPSendGumpMenu toSend;
	toSend.UserID( INVALIDSERIAL );
	toSend.GumpID( 3 );

	toSend.AddCommand( "noclose" );
	toSend.AddCommand( "page 0" );
	toSend.AddCommand( "resizepic 0 0 %u 320 340", cwmWorldState->ServerData()->BackgroundPic() );
	toSend.AddCommand( "button 280 10 %u %i 1 0 1", cwmWorldState->ServerData()->ButtonCancel(), cwmWorldState->ServerData()->ButtonCancel() + 1); //OKAY
	toSend.AddCommand( "text 70 10 %u 0", cwmWorldState->ServerData()->TitleColour() );           //text <Spaces from Left> <Space from top> <Length, Color?> <# in order>
	toSend.AddCommand( "page 1" );

	toSend.AddText( "Enemy" );
	toSend.AddCommand( "gumppic 25 50 1141" );	// town name
	toSend.AddCommand( "text 35 51 %u 1", cwmWorldState->ServerData()->RightTextColour() );	// town name
	toSend.AddCommand( "text 25 71 %u 2", cwmWorldState->ServerData()->RightTextColour() );	// population
	toSend.AddCommand( "text 55 111 %u 3", cwmWorldState->ServerData()->RightTextColour() );	// Seize townstone
	toSend.AddCommand( "text 55 131 %u 4", cwmWorldState->ServerData()->RightTextColour() );	// Destroy townstone

	toSend.AddCommand( "button 25 111 %u %i 1 0 61", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// seize townstone
	toSend.AddCommand( "button 25 131 %u %i 1 0 62", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// destroy townstone
	toSend.AddText( "%s (%s)", name.c_str(), Races->Name( race ) );
	toSend.AddText( "Population %i", GetPopulation() );
	toSend.AddText( "Seize Townstone" );
	toSend.AddText( "Attack Townstone" );

	toSend.Finalize();
	sock->Send( &toSend );
}
void cTownRegion::SendBasicInfo( cSocket *sock )
{
	GumpDisplay BasicGump( sock );
	BasicGump.SetTitle( "Basic Townstone gump" );
	BasicGump.Send( 4, false, INVALIDSERIAL );
}
void cTownRegion::SendPotentialMember( cSocket *sock )
{
	UnicodeTypes sLang	= sock->Language();
	CPSendGumpMenu toSend;
	toSend.UserID( INVALIDSERIAL );
	toSend.GumpID( 3 );

	toSend.AddCommand( "noclose" );
	toSend.AddCommand( "page 0" );
	toSend.AddCommand( "resizepic 0 0 %u 320 340", cwmWorldState->ServerData()->BackgroundPic() );
	toSend.AddCommand( "button 280 10 %u %i 1 0 1", cwmWorldState->ServerData()->ButtonCancel(), cwmWorldState->ServerData()->ButtonCancel() + 1); //OKAY
	toSend.AddCommand( "text 70 10 %u 0", cwmWorldState->ServerData()->TitleColour() );           //text <Spaces from Left> <Space from top> <Length, Color?> <# in order>
	toSend.AddCommand( "page 1" );

	toSend.AddText( "Outsider" );	// our title
	toSend.AddCommand( "gumppic 25 50 1141" );	// town name
	toSend.AddCommand( "text 35 51 %u 1", cwmWorldState->ServerData()->RightTextColour() );	// town name
	toSend.AddCommand( "text 25 71 %u 2", cwmWorldState->ServerData()->RightTextColour() );	// population
	toSend.AddCommand( "text 55 91 %u 3", cwmWorldState->ServerData()->RightTextColour() );	// join town
	toSend.AddCommand( "text 55 111 %u 4", cwmWorldState->ServerData()->RightTextColour() );	// view taxes (to help make decisions about joining?)

	toSend.AddCommand( "button 25 91 %u %i 1 0 41", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// leave town
	toSend.AddCommand( "button 25 111 %u %i 1 0 3", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// view taxes

	toSend.AddText( "%s (%s)", name.c_str(), Races->Name( race ) );
	toSend.AddText( Dictionary->GetEntry( 1127, sLang ).c_str(), GetPopulation() );
	toSend.AddText( Dictionary->GetEntry( 1128, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1129, sLang ).c_str() );

	toSend.Finalize();
	sock->Send( &toSend );
}

void cTownRegion::SendMayorGump( cSocket *sock )
{
	UnicodeTypes sLang	= sock->Language();
	CPSendGumpMenu toSend;
	toSend.UserID( INVALIDSERIAL );
	toSend.GumpID( 3 );

	toSend.AddCommand( "noclose" );
	toSend.AddCommand( "page 0" );
	toSend.AddCommand( "resizepic 0 0 %u 320 340", cwmWorldState->ServerData()->BackgroundPic() );
	toSend.AddCommand( "button 280 10 %u %i 1 0 1", cwmWorldState->ServerData()->ButtonCancel(), cwmWorldState->ServerData()->ButtonCancel() + 1); //OKAY
	toSend.AddCommand( "text 70 10 %u 0", cwmWorldState->ServerData()->TitleColour() );           //text <Spaces from Left> <Space from top> <Length, Color?> <# in order>
	toSend.AddCommand( "page 1" );

	toSend.AddText( "Mayor Controls" );	// our title

	toSend.AddCommand( "gumppic 25 50 1141" );	// town name
	toSend.AddCommand( "gumppic 25 260 1141" );
	toSend.AddCommand( "text 35 51 %u 1", cwmWorldState->ServerData()->RightTextColour() );	// town name
	toSend.AddCommand( "text 25 71 %u 2", cwmWorldState->ServerData()->RightTextColour() );	// population
	toSend.AddCommand( "text 55 91 %u 3", cwmWorldState->ServerData()->RightTextColour() ); // set taxes
	toSend.AddCommand( "text 55 280 %u 4", cwmWorldState->ServerData()->RightTextColour() ); // return to main menu
	toSend.AddCommand( "text 55 111 %u 5", cwmWorldState->ServerData()->RightTextColour() ); // list town members
	toSend.AddCommand( "text 55 131 %u 6", cwmWorldState->ServerData()->RightTextColour() ); // force early election
	toSend.AddCommand( "text 55 151 %u 7", cwmWorldState->ServerData()->RightTextColour() ); // purchase more guards
	toSend.AddCommand( "text 55 171 %u 8", cwmWorldState->ServerData()->RightTextColour() ); // fire a guard
	toSend.AddCommand( "text 55 261 %u 9", cwmWorldState->ServerData()->RightTextColour() ); // treasury amount
	toSend.AddCommand( "text 55 191 %u 10", cwmWorldState->ServerData()->RightTextColour() );	// make ally

	toSend.AddCommand( "button 25 91 %u %i 1 0 21", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 ); // set taxes 
	toSend.AddCommand( "button 25 111 %u %i 1 0 22", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 ); // list town members 
	toSend.AddCommand( "button 25 131 %u %i 1 0 23", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 ); // force early election
	toSend.AddCommand( "button 25 151 %u %i 1 0 24", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 ); // purchase more guards
	toSend.AddCommand( "button 25 171 %u %i 1 0 25", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 ); // fire a guard
	toSend.AddCommand( "button 25 280 %u %i 1 0 40", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 ); // return to main menu
	toSend.AddCommand( "button 25 191 %u %i 1 0 26", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 ); // make ally of other town

	toSend.AddText( "%s (%s)", name.c_str(), Races->Name( race ) );
	toSend.AddText( Dictionary->GetEntry( 1130, sLang ).c_str(), GetPopulation() );
	toSend.AddText( Dictionary->GetEntry( 1131, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1132, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1133, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1134, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1135, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1136, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1137, sLang ).c_str(), goldReserved );
	toSend.AddText( Dictionary->GetEntry( 1138, sLang ).c_str() );

	toSend.Finalize();
	sock->Send( &toSend );
}

void cTownRegion::SendDefaultGump( cSocket *sock )
{
	CPSendGumpMenu toSend;
	toSend.UserID( INVALIDSERIAL );
	toSend.GumpID( 3 );

	toSend.AddCommand( "noclose" );
	toSend.AddCommand( "page 0" );
	toSend.AddCommand( "resizepic 0 0 %u 320 340", cwmWorldState->ServerData()->BackgroundPic() );
	toSend.AddCommand( "button 280 10 %u %i 1 0 1", cwmWorldState->ServerData()->ButtonCancel(), cwmWorldState->ServerData()->ButtonCancel() + 1); //OKAY
	toSend.AddCommand( "text 70 10 %u 0", cwmWorldState->ServerData()->TitleColour() );           //text <Spaces from Left> <Space from top> <Length, Color?> <# in order>
	toSend.AddCommand( "page 1" );

	toSend.AddText( "Generic View" );	// our title
	toSend.AddCommand( "gumppic 25 50 1141" );	// town name
	toSend.AddCommand( "text 35 51 %u 1", cwmWorldState->ServerData()->RightTextColour() );	// town name
	toSend.AddCommand( "text 25 71 %u 2", cwmWorldState->ServerData()->RightTextColour() );	// population
	toSend.AddCommand( "text 55 91 %u 3", cwmWorldState->ServerData()->RightTextColour() );	// leave town
	toSend.AddCommand( "text 55 111 %u 4", cwmWorldState->ServerData()->RightTextColour() );	// view taxes
	toSend.AddCommand( "text 55 131 %u 5", cwmWorldState->ServerData()->RightTextColour() );	// toggle town title on/off
	toSend.AddCommand( "text 55 151 %u 6", cwmWorldState->ServerData()->RightTextColour() );	// vote for mayor
	toSend.AddCommand( "text 55 171 %u 7", cwmWorldState->ServerData()->RightTextColour() );	// donate resource
	toSend.AddCommand( "tilepic 205 171 %u", GetResourceID() );				// picture of the resource
	toSend.AddCommand( "text 55 191 %u 8", cwmWorldState->ServerData()->RightTextColour() );	// view budget
	toSend.AddCommand( "text 55 211 %u 9", cwmWorldState->ServerData()->RightTextColour() );	// view allied towns
	toSend.AddCommand( "text 55 231 %u 10", cwmWorldState->ServerData()->RightTextColour() );	// view enemy towns

	toSend.AddCommand( "button 25 91 %u %i 1 0 2", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// leave town
	toSend.AddCommand( "button 25 111 %u %i 1 0 3", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// view taxes
	toSend.AddCommand( "button 25 131 %u %i 1 0 4", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// toggle title
	toSend.AddCommand( "button 25 151 %u %i 1 0 5", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// vote for mayor
	toSend.AddCommand( "button 25 171 %u %i 1 0 6", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// donate gold
	toSend.AddCommand( "button 25 191 %u %i 1 0 7", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// view budget
	toSend.AddCommand( "button 25 211 %u %i 1 0 8", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// view allied towns
	toSend.AddCommand( "button 25 231 %u %i 1 0 9", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );	// view enemy towns

	CChar *mChar		= sock->CurrcharObj();
	UnicodeTypes sLang	= sock->Language();
	toSend.AddText( "%s (%s)", name.c_str(), Races->Name( race ) );
	toSend.AddText( Dictionary->GetEntry( 1139, sLang ).c_str(), GetPopulation() );
	toSend.AddText( Dictionary->GetEntry( 1140, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1141, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1142, sLang ).c_str(), mChar->GetTownTitle()?"Off":"On" );
	toSend.AddText( Dictionary->GetEntry( 1143, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1144, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1145, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1146, sLang ).c_str() );
	toSend.AddText( Dictionary->GetEntry( 1147, sLang ).c_str() );

	if( mChar->GetTownPriv() == 2 || mChar->IsGM() ) // if we've got a mayor (remove isGM check!)
	{
		toSend.AddCommand( "button 25 281 %u %i 1 0 20", cwmWorldState->ServerData()->ButtonRight(), cwmWorldState->ServerData()->ButtonRight() + 1 );
		toSend.AddCommand( "text 55 281 %u 11", cwmWorldState->ServerData()->LeftTextColour() );
		toSend.AddText( Dictionary->GetEntry( 1148, sLang ).c_str() );
	}
	toSend.Finalize();
	sock->Send( &toSend );
}
size_t cTownRegion::GetPopulation( void ) const
{
	return townMember.size();
}

void cTownRegion::DisplayTownMembers( cSocket *sock )
{
	GumpDisplay townListing( sock, 300, 300 );
	townListing.SetTitle( Dictionary->GetEntry( 1149, sock->Language() ).c_str() );
	CChar *sChar = sock->CurrcharObj();
	for( size_t counter = 0; counter < townMember.size(); ++counter )
	{
		CChar *townChar = calcCharObjFromSer( townMember[counter].townMember );
		if( townChar != NULL )
		{
			if( sChar->IsGM() )
				townListing.AddData( townChar->GetName().c_str(), townChar->GetSerial(), 3 );
			else
				townListing.AddData( townChar->GetName().c_str(), " " );	// don't want them to know the serial
		}
		else
		{
			RemoveCharacter( counter );
			--counter;
		}
	}
	townListing.Send( 4, false, INVALIDSERIAL );
}

bool cTownRegion::VoteForMayor( cSocket *sock )
{
	SERIAL serial	= sock->GetDWord( 7 );
	CChar *srcChar	= sock->CurrcharObj();
	CChar *target	= calcCharObjFromSer( serial );
	if( target != NULL )
	{
		if( !IsMemberOfTown( target ) )	// he's not in our town!
		{
			sock->sysmessage( 1150 );
			return false;
		}
		SERIAL serialPos = FindPositionOf( (*srcChar) );
		if( serialPos == INVALIDSERIAL )
		{
			sock->sysmessage( 1151 );
			return false;
		}
		if( townMember[serialPos].targVote != INVALIDSERIAL )
		{
			sock->sysmessage( 1152 );
			return false;
		}
		townMember[serialPos].targVote = target->GetSerial();
		sock->sysmessage( 1153 );
		return true;
	}
	else
	{
		sock->sysmessage( 1154 );
		return false;
	}
}

SERIAL cTownRegion::FindPositionOf( CChar& toAdd )
{
	for( SERIAL counter = 0; counter < townMember.size(); ++counter )
	{
		if( townMember[counter].townMember == toAdd.GetSerial() )
			return counter;
	}
	return INVALIDSERIAL;
}

UI16 cTownRegion::GetResourceID( void ) const
{
	return taxedResource;
}

bool cTownRegion::DonateResource( cSocket *s, SI32 amount )
{
	goldReserved += amount;
	CChar *tChar = s->CurrcharObj();
	if( amount > 1000 )
		s->sysmessage( 1155, tChar->GetName().c_str() );
	else if( amount > 750 )
		s->sysmessage( 1156, tChar->GetName().c_str() );
	else if( amount > 500 )
		s->sysmessage( 1157, tChar->GetName().c_str() );
	else if( amount > 250 )
		s->sysmessage( 1158, tChar->GetName().c_str() );
	else
		s->sysmessage( 1159, tChar->GetName().c_str() );
	return true;
}

bool cTownRegion::PurchaseGuard( cSocket *sock, UI08 number )
{
	if( number * 10000 < goldReserved )	// if we don't have the cash
	{
		sock->sysmessage( 1160 );
		return false;
	}

	goldReserved -= (number * 10000);
	size_t maxGuardIndex = guards.size();
	guards.resize( maxGuardIndex + number );

	for( size_t counter = maxGuardIndex; counter < guards.size(); ++counter )
		guards[counter] = RandomNum( 1000, 1001 );
	return true;
}

bool cTownRegion::ViewBudget( cSocket *sock )
{
	UnicodeTypes sLang = sock->Language();
	GumpDisplay Budget( sock, 300, 300 );
	Budget.SetTitle( Dictionary->GetEntry( 1161, sLang ).c_str() );
	Budget.AddData( Dictionary->GetEntry( 1162, sLang ).c_str(), guardsPurchased );
	Budget.AddData( Dictionary->GetEntry( 1163, sLang ).c_str(), guards.size() );
	Budget.AddData( Dictionary->GetEntry( 1164, sLang ).c_str(), guards.size() * 20 );
	Budget.Send( 4, false, INVALIDSERIAL );

	return true;
}

bool cTownRegion::PeriodicCheck( void )
{
	time_t now;
	time( &now );
	if( difftime( now, timeSinceTaxedMembers ) >= cwmWorldState->ServerData()->TownTaxPeriod() )
	{
		for( UI32 memberCounter = 0; memberCounter < townMember.size(); ++memberCounter )
		{
			CChar *townMem = calcCharObjFromSer( townMember[memberCounter].townMember );
			if( townMem != NULL )
			{
				UI16 resourceID = GetResourceID();
				UI32 numResources = GetAmount( townMem, resourceID );

				if( taxedAmount > numResources )
				{
					UI32 bankAmount = GetBankCount( townMem, resourceID, 0 );
					if( taxedAmount > ( numResources + bankAmount ) )
						JailSys->JailPlayer( townMem, 900 );
					else
					{
						DeleteQuantity( townMem, resourceID, numResources );
						DeleteBankItem( townMem, resourceID, 0, bankAmount - numResources );
					}
				}
				else
				{
					DeleteQuantity( townMem, resourceID, taxedAmount );
					resourceCollected += taxedAmount;
				}
			}
		}
		timeSinceTaxedMembers = now;
	}
	if( difftime( now, timeSinceGuardsPaid ) >= cwmWorldState->ServerData()->TownGuardPayment() )	// init to 1000 seconds
	{
		if( goldReserved < static_cast<SI32>(( 20 * guards.size() )) )
		{
			// insufficient resources
			bool enoughGuards = false;
			while( !enoughGuards )
			{
				guards.resize( guards.size() - 1 );
				enoughGuards = ( goldReserved >= static_cast<SI32>(( 20 * guards.size() ) ));
			}
		}
		goldReserved -= 20 * guards.size();
		timeSinceGuardsPaid = now;
	}

	if( now > timeToNextPoll && townMember.size() != 0 )
	{
		TellMembers( 1165 );
		for( size_t counter = 0; counter < townMember.size(); ++counter )
			townMember[counter].targVote = INVALIDSERIAL;
		timeToElectionClose = now + cwmWorldState->ServerData()->TownNumSecsPollOpen();	// 2 days polls are open
		timeToNextPoll		= timeToElectionClose + cwmWorldState->ServerData()->TownNumSecsAsMayor();	// secs as mayor over the top of the end of the poll
		CChar *mayor		= GetMayor();
		if( mayor != NULL )
			mayor->SetTownpriv( 1 );	// mayor becomes a basic member again, so he can't do anything while the poll is occuring :>
	}

	if( now > timeToElectionClose && townMember.size() != 0 )	// election finished
	{
		TellMembers( 1166 );
		CalcNewMayor();
		timeToElectionClose = timeToNextPoll + 1000;	// just so long as it's after the next election
	}
	return true;
}	

WorldType cTownRegion::GetAppearance( void ) const
{
	return visualAppearance;
}

void cTownRegion::ViewTaxes( cSocket *sock )
{
	CPSendGumpMenu toSend;
	toSend.UserID( INVALIDSERIAL );
	toSend.GumpID( 3 );

	toSend.AddCommand( "noclose" );
	toSend.AddCommand( "page 0" );
	toSend.AddCommand( "resizepic 0 0 %u 320 340", cwmWorldState->ServerData()->BackgroundPic() );
	toSend.AddCommand( "button 280 10 %u %i 1 0 1", cwmWorldState->ServerData()->ButtonCancel(), cwmWorldState->ServerData()->ButtonCancel() + 1); //OKAY
	toSend.AddCommand( "text 70 10 %u 0", cwmWorldState->ServerData()->TitleColour() );           //text <Spaces from Left> <Space from top> <Length, Color?> <# in order>
	toSend.AddCommand( "page 1" );

	toSend.AddText( "Taxes" );	// our title
	toSend.AddCommand( "gumppic 25 50 1141" );	// town name
	toSend.AddCommand( "text 35 51 %u 1", cwmWorldState->ServerData()->RightTextColour() );	// town name
	toSend.AddCommand( "text 35 71 %u 2", cwmWorldState->ServerData()->RightTextColour() );	// population
	toSend.AddCommand( "text 35 111 %u 3", cwmWorldState->ServerData()->RightTextColour() ); // # of resources
	toSend.AddCommand( "tilepic 5 111 %u", GetResourceID() );				// picture of the resource

	toSend.AddText( "%s (%s)", name.c_str(), Races->Name( race ) );
	toSend.AddText( "Population %i", GetPopulation() );
	CTile tile;
	Map->SeekTile( GetResourceID(), &tile );
	toSend.AddText( "%i %ss", taxedAmount, tile.Name() );
	toSend.Finalize();
	sock->Send( &toSend );
}

SI16 cTownRegion::GetHealth( void ) const
{
	return health;
}

void cTownRegion::DoDamage( SI16 reduction )
{
	health -= reduction;
	if( health < 0 )
		health = 0;
	if( health == 0 )
		TellMembers( 1167 );
	else if( health <= 1000 )
		TellMembers( 1168 );
	else if( health <= 5000 )
		TellMembers( 1169 );
	else
		TellMembers( 1170 );
	
}

bool cTownRegion::IsAlliedTown( UI08 townToCheck ) const
{
	for( UI32 counter = 0; counter < alliedTowns.size(); ++counter )
	{
		if( alliedTowns[counter] == townToCheck )
			return true;
	}
	return false;
}

bool cTownRegion::MakeAlliedTown( UI08 townToMake )
{
	if( regionNum == townToMake )
		return false;

	if( IsAlliedTown( townToMake ) )	// already allied
	{
		TellMembers( 1171 );
		return false;
	}

	if( Races->CompareByRace( region[townToMake]->GetRace(), race ) == 1 )	// if we're racial enemies
		return false;

	size_t location = alliedTowns.size();
	alliedTowns.resize( location + 1 );
	alliedTowns[location] = townToMake;	// let's ally ourselves
	TellMembers( 1172, name.c_str(), region[townToMake]->GetName().c_str() );
	return true;

}

void cTownRegion::TellMembers( SI32 dictEntry, ...) // System message (In lower left corner)
{
	char msg[512];
	char tmpMsg[512];

	for( UI32 memberCounter = 0; memberCounter < townMember.size(); ++memberCounter )
	{
		CChar *targetChar = calcCharObjFromSer( townMember[memberCounter].townMember );
		cSocket *targetSock = calcSocketObjFromChar( targetChar );
		if( targetSock != NULL )
		{
			std::string txt = Dictionary->GetEntry( dictEntry, targetSock->Language() );

			strcpy( msg, "TOWN: " );
			va_list argptr;
			va_start( argptr, dictEntry );
			vsprintf( tmpMsg, txt.c_str(), argptr );
			va_end( argptr );
			strcat( msg, tmpMsg );

			CSpeechEntry *toAdd = SpeechSys->Add();
			toAdd->Speech( msg );
			toAdd->Font( FNT_NORMAL );
			toAdd->Speaker( INVALIDSERIAL );
			toAdd->SpokenTo( townMember[memberCounter].townMember );
			toAdd->Colour( 0x000B );
			toAdd->Type( SYSTEM );
			toAdd->At( cwmWorldState->GetUICurrentTime() );
			toAdd->TargType( SPTRG_INDIVIDUAL );
		}
	}
}

RACEID cTownRegion::GetRace( void ) const
{
	return race;
}

void cTownRegion::SendAlliedTowns( cSocket *sock )
{
	GumpDisplay Ally( sock, 300, 300 );
	char temp[100];
	sprintf( temp, Dictionary->GetEntry( 1173, sock->Language() ).c_str(), alliedTowns.size() );
	Ally.SetTitle( temp );
	for( UI32 counter = 0; counter < alliedTowns.size(); ++counter )
		Ally.AddData( region[alliedTowns[counter]]->GetName().c_str(), " " );

	Ally.Send( 4, false, INVALIDSERIAL );
}

void cTownRegion::ForceEarlyElection( void )
{
	time_t now;
	time(&now);
	CChar *mayor	= GetMayor();
	timeToNextPoll	= now;	// time to open poll
	TellMembers( 1174 );
	for( UI32 counter = 0; counter < townMember.size(); ++counter )
		townMember[counter].targVote = INVALIDSERIAL;
	if( mayor != NULL )
		mayor->SetTownpriv( 1 );
}

void cTownRegion::SendEnemyTowns( cSocket *sock )
{
	GumpDisplay Enemy( sock, 300, 300 );
	char temp[100];
	UI08 enemyCount = 0;
	for( UI16 counter = 0; counter < 256; ++counter )
	{
		if( counter != regionNum && Races->CompareByRace( race, region[counter]->GetRace() ) == 1 )	// if we're racial enemies, and not the same as ourselves
		{
			++enemyCount;
			Enemy.AddData( region[counter]->GetName().c_str(), Races->Name( region[counter]->GetRace() ) );
		}
	}
	sprintf( temp, "Enemy Towns (%u)", enemyCount );
	Enemy.SetTitle( temp );
	Enemy.Send( 4, false, INVALIDSERIAL );
}

void cTownRegion::Possess( UI08 possessorTown )
{
	region[possessorTown]->SetRace( race );
	region[possessorTown]->TellMembers( 1175 );
	region[possessorTown]->SetReserves( region[possessorTown]->GetReserves() + GetReserves() / 3 * 2 );
	region[possessorTown]->SetTaxesLeft( region[possessorTown]->GetTaxes() + GetTaxes() / 3 * 2 );
	TellMembers( 1176 );
	MakeAlliedTown( possessorTown );
	region[possessorTown]->MakeAlliedTown( regionNum );

	// remove the old members, preparing the way for the new ones
	CChar *targChar;
	for( size_t counter = townMember.size() - 1; counter >= 0 && counter < townMember.size(); ++counter )
	{
		targChar = calcCharObjFromSer( townMember[counter].townMember );
		RemoveCharacter( counter );
		if( targChar != NULL )
		{
			targChar->SetTown( 255 );
			targChar->SetTownpriv( 0 );
		}
	}
	townMember.resize( 0 );
}

long cTownRegion::GetReserves( void ) const
{
	return resourceCollected;
}
long cTownRegion::GetTaxes( void ) const
{
	return goldReserved;
}
void cTownRegion::SetTaxesLeft( long newValue )
{
	goldReserved = newValue;
}
void cTownRegion::SetReserves( long newValue )
{
	resourceCollected = newValue;
}
void cTownRegion::SetRace( RACEID newRace )
{
	race = newRace;
}
SI16 cTownRegion::GetMinColourSkill( void ) const
{
	return minColourSkill;
}
UI08 cTownRegion::GetChanceBigOre( void ) const
{
	return chanceFindBigOre;
}
UI08 cTownRegion::GetChanceColourOre( void ) const
{
	return chanceColourOre;
}
bool cTownRegion::RemoveCharacter( size_t position )
{
	for( size_t counter2 = position; counter2 < ( townMember.size() - 1 ); ++counter2 )
	{
		townMember[counter2].townMember = townMember[counter2+1].townMember;
		townMember[counter2].targVote   = townMember[counter2+1].targVote;
		townMember[counter2].PlayerBank = townMember[counter2+1].PlayerBank;
	}
	townMember.resize( townMember.size() - 1 );
	return true;
}
size_t cTownRegion::GetNumOrePreferences( void ) const
{
	return orePreferences.size();
}
const orePref *cTownRegion::GetOrePreference( size_t targValue ) const
{
	if( targValue >= orePreferences.size() )
		return NULL;
	return &orePreferences[targValue];
}

SI32 cTownRegion::GetOreChance( void ) const
{
	int sumReturn = 0;
	for( size_t i = 0; i < orePreferences.size(); ++i )
		sumReturn += orePreferences[i].percentChance;
	return sumReturn;
}
bool cTownRegion::IsDungeon( void ) const
{
	return ( (priv&0x80) == 0x80 );
}

size_t cTownRegion::NumGuards( void ) const
{
	return guards.size();
}

UI08 cTownRegion::WorldNumber( void ) const
{
	return worldNumber;
}

void cTownRegion::IsGuarded( bool value )
{
	if( value )
		priv |= 0x01;
	else
		priv &= 0xFE;
}
void cTownRegion::CanMark( bool value )
{
	if( value )
		priv |= 0x02;
	else
		priv &= 0xFD;
}
void cTownRegion::CanGate( bool value )
{
	if( value )
		priv |= 0x04;
	else
		priv &= 0xFB;
}
void cTownRegion::CanRecall( bool value )
{
	if( value )
		priv |= 0x08;
	else
		priv &= 0xF7;
}
void cTownRegion::CanCastAggressive( bool value )
{
	if( value )
		priv |= 0x40;
	else
		priv &= 0xBF;
}
void cTownRegion::IsDungeon( bool value )
{
	if( value )
		priv |= 0x80;
	else
		priv &= 0x7F;
}
void cTownRegion::SetName( std::string toSet )
{
	name = toSet.substr( 0, 49 );
}

void cTownRegion::TaxedAmount( UI16 amount )
{
	taxedAmount = amount;
}

UI16 cTownRegion::TaxedAmount( void ) const
{
	return taxedAmount;
}

void cTownRegion::SetResourceID( UI16 resID )
{
	taxedResource = resID;
}

void cTownRegion::SetHealth( SI16 newValue )
{
	health = newValue;
}

void cTownRegion::SetChanceBigOre( UI08 newValue )
{
	chanceFindBigOre = newValue;
}
void cTownRegion::SetChanceColourOre( UI08 newValue )
{
	chanceColourOre = newValue;
}

UI16 cTownRegion::GetScriptTrigger( void ) const
{
	return jsScript;
}
void cTownRegion::SetScriptTrigger( UI16 newValue )
{
	jsScript = newValue;
}
