#include "uox3.h"
#include "classes.h"
#include "packets.h"

#undef DBGFILE
#define DBGFILE "trade.cpp"

CItem *startTrade( cSocket *mSock, CChar *i )
{
	if( mSock == NULL || i == NULL )
		return NULL;

	CChar *mChar	= mSock->CurrcharObj();
	CItem *bps		= getPack( mChar );
	CItem *bpi		= getPack( i );
	cSocket *nSock	= calcSocketObjFromChar( i );

	if( nSock == NULL )
		return NULL;

	if( bps == NULL )
	{
		mSock->sysmessage( 773 );
		nSock->sysmessage( 1357, mChar->GetName().c_str() );
		return NULL;
	}
	
	if( bpi == NULL )
	{
		nSock->sysmessage( 773 );
		mSock->sysmessage( 1357, i->GetName().c_str() );
		return NULL;
	}

	CItem *ps = Items->SpawnItem( NULL, i, 1, "#", false, 0x1E5E, 0, false, false );
	if( ps == NULL ) 
		return NULL;
	ps->SetX( 26 );
	ps->SetY( 0 );
	ps->SetZ( 0 );
	ps->SetLayer( 0 );
	ps->SetType( 1 );
	ps->SetDye( false );
	if( !ps->SetCont( mChar ) )
		return NULL;
	ps->SendPackItemToSocket( mSock );
	ps->SendPackItemToSocket( nSock );
	CItem *pi = Items->SpawnItem( NULL, i, 1, "#", false, 0x1E5E, 0, false, false );
	if( pi == NULL ) 
		return NULL;
	pi->SetX( 26 );
	pi->SetY( 0 );
	pi->SetZ( 0 );
	pi->SetLayer( 0 );
	if( !pi->SetCont( i ) )
		return NULL;
	pi->SetType( 1 );
	pi->SetDye( false );
	pi->SendPackItemToSocket( mSock );
	pi->SendPackItemToSocket( nSock );
	
	pi->SetMoreB( ps->GetSerial() );
	ps->SetMoreB( pi->GetSerial() );
	pi->SetMoreZ( 0 );
	ps->SetMoreZ( 0 );
	
	CPSecureTrading cpstOne( (*i), (*ps), (*pi) );
	cpstOne.Name( i->GetName().c_str() );
	mSock->Send( &cpstOne );
	
	CPSecureTrading cpstTwo( (*mChar), (*pi), (*ps) );
	cpstTwo.Name( mChar->GetName().c_str() );
	nSock->Send( &cpstTwo );

	return ps;
}

void clearTrades( void )
{
	for( ITEM i = 0; i < cwmWorldState->GetItemCount(); ++i )
	{
		if( items[i].GetType() == 1 && items[i].GetX() == 26 && items[i].GetY() == 0 && items[i].GetZ() == 0 && items[i].GetID() == 0x1E5E )
		{
			CChar *k = (CChar *)items[i].GetCont();
			CItem *p = getPack( k );
			if( p != NULL )	// can we move this check to outside the for loop?? I should think so!
			{
				for( CItem *j = items[i].FirstItem(); !items[i].FinishedItems(); j = items[i].NextItem() )
				{
					if( j != NULL )
						j->SetCont( p );
				}
			}
			Items->DeleItem( &items[i] );
		}
	}
}

void doTradeMsg( cSocket *mSock )
{
	CItem *cont1 = NULL, *cont2 = NULL;
	switch( mSock->GetByte( 3 ) )
	{
		case 0://Start trade - Never happens, sent out by the server only.
			break;
		case 2://Change check marks.  Possibly conclude trade
			cont1 = calcItemObjFromSer( mSock->GetDWord( 4 ) );
			if( cont1 != NULL )
				cont2 = calcItemObjFromSer( cont1->GetMoreB() );
			if( cont2 != NULL )
			{
				cont1->SetMoreZ( mSock->GetByte( 11 ) );
				sendTradeStatus( cont1, cont2 );
				if( cont1->GetMoreZ() && cont2->GetMoreZ() )
				{
					doTrade( cont1, cont2 );
					endTrade( mSock->GetDWord( 4 ) );
				}
			}
			break;
		case 1://Cancel trade.  Send each person cancel messages, move items.
			endTrade( mSock->GetDWord( 4 ) );
			break;
		default:
			Console.Error( 2, " Fallout of switch statement without default. trade.cpp, trademsg()" );
			break;
	}
}

void sendTradeStatus( CItem *cont1, CItem *cont2 )
{
	CChar *p1 = (CChar *)cont1->GetCont();
	CChar *p2 = (CChar *)cont2->GetCont();
	cSocket *s1 = calcSocketObjFromChar( p1 );
	cSocket *s2 = calcSocketObjFromChar( p2 );

	if( s1 != NULL )
	{
		CPSecureTrading cpstOne( (*cont1), (cont1->GetMoreZ() % 256), (cont2->GetMoreZ() % 256) );
		cpstOne.Action( 2 );
		s1->Send( &cpstOne );
	}

	if( s2 != NULL ) 
	{
		CPSecureTrading cpstTwo( (*cont2), (cont2->GetMoreZ() % 256), (cont1->GetMoreZ() % 256) );
		cpstTwo.Action( 2 );
		s2->Send( &cpstTwo );
	}
}

void endTrade( SERIAL targSerial )
{
	CItem *cont1 = calcItemObjFromSer( targSerial );
	if( cont1 == NULL )
		return;

	CItem *cont2 = calcItemObjFromSer( cont1->GetMoreB() );
	if( cont2 == NULL )
		return;

	CChar *p1 = (CChar *)cont1->GetCont();
	CChar *p2 = (CChar *)cont2->GetCont();

	CItem *bp1 = getPack( p1 );
	if( bp1 == NULL ) 
		return;

	CItem *bp2 = getPack( p2 );
	if( bp2 == NULL ) 
		return;

	cSocket *mSock = calcSocketObjFromChar( p1 );
	cSocket *nSock = calcSocketObjFromChar( p2 );
	
	if( mSock != NULL ) 
	{
		CPSecureTrading cpstOne( (*cont1), 0, 0 );
		cpstOne.Action( 1 );
		mSock->Send( &cpstOne );
	}
	
	if( nSock != NULL ) 
	{
		CPSecureTrading cpstTwo( (*cont2), 0, 0 );
		cpstTwo.Action( 1 );
		nSock->Send( &cpstTwo );
	}
	CItem *i = NULL, *j = NULL;
	for( i = cont1->FirstItem(); !cont1->FinishedItems(); i = cont1->NextItem() )
	{
		if( i != NULL )
		{
			i->SetCont( bp1 );
			i->SetX( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetY( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetZ( 9 );
			if( i->GetGlow() != INVALIDSERIAL )
			{
				j = calcItemObjFromSer( i->GetGlow() );
				if( j != NULL )
				{
					j->SetCont( bp1 );
					j->SetX( i->GetX() );
					j->SetY( i->GetY() );
					j->SetZ( i->GetZ() );
				}
			}
		}
	}
	for( i = cont2->FirstItem(); !cont2->FinishedItems(); i = cont2->NextItem() )
	{
		if( i != NULL )
		{
			i->SetCont( bp2 );  
			i->SetX( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetY( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetZ( 9 );
			if( i->GetGlow() != INVALIDSERIAL )
			{
				j = calcItemObjFromSer( i->GetGlow() );
				if( j != NULL )
				{
					j->SetCont( bp2 );
					j->SetX( i->GetX() );
					j->SetY( i->GetY() );
					j->SetZ( i->GetZ() );
				}
			}
		}
	}
	Items->DeleItem( cont1 );
	Items->DeleItem( cont2 );
}

void doTrade( CItem *cont1, CItem *cont2 )
{
	CChar *p1 = (CChar *)cont1->GetCont();
	if( p1 == NULL ) 
		return;

	CChar *p2 = (CChar *)cont2->GetCont();
	if( p2 == NULL ) 
		return;

	CItem *bp1 = getPack( p1 );
	if( bp1 == NULL ) 
		return;

	CItem *bp2 = getPack( p2 );
	if( bp2 == NULL ) 
		return;

	cSocket *mSock = calcSocketObjFromChar( p1 );
	cSocket *nSock = calcSocketObjFromChar( p2 );
	
	CItem *i = NULL, *j = NULL;
	for( i = cont1->FirstItem(); !cont1->FinishedItems(); i = cont1->NextItem() )
	{
		if( i != NULL )
		{
			i->SetCont( bp2 );
			i->SetX( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetY( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetZ( 9 );
			if( i->GetGlow() != INVALIDSERIAL )
			{
				j = calcItemObjFromSer( i->GetGlow() );
				if( j != NULL )
				{
					j->SetCont( bp2 );
					j->SetX( i->GetX() );
					j->SetY( i->GetY() );
					j->SetZ( i->GetZ() );
				}
			}

		}
	}
	for( i = cont2->FirstItem(); !cont2->FinishedItems(); i = cont2->NextItem() )
	{
		if( i != NULL )
		{
			i->SetCont( bp1 );
			i->SetX( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetY( 50 + ( RandomNum( 0, 79 ) ) );
			i->SetZ( 9 );
			if( i->GetGlow() != INVALIDSERIAL )
			{
				j = calcItemObjFromSer( i->GetGlow() );
				if( j != NULL )
				{
					j->SetCont( bp1 );
					j->SetX( i->GetX() );
					j->SetY( i->GetY() );
					j->SetZ( i->GetZ() );
				}
			}
		}
	}
}

void killTrades( CChar *i )
{
	for( CItem *j = i->FirstItem(); !i->FinishedItems(); j = i->NextItem() )
	{
		if( j != NULL )
		{
			if( j->GetType() == 1 && j->GetX() == 26 && j->GetY() == 0 && j->GetZ() == 0 && j->GetID() == 0x1E5E )
				endTrade( j->GetSerial() );
		}
	}
}
