/*

  Ultima Offline eXperiment III (UOX3)
  UO Server Emulation Program
  
	Copyright 1997, 98 by Marcus Rating (Cironian)
	
	  This program is free software; you can redistribute it and/or modify
	  it under the terms of the GNU General Public License as published by
	  the Free Software Foundation; either version 2 of the License, or
	  (at your option) any later version.
	  
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
		
		  You should have received a copy of the GNU General Public License
		  along with this program; if not, write to the Free Software
		  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
		  
			* In addition to that license, if you are running this program or modified  *
			* versions of it on a public system you HAVE TO make the complete source of *
			* the version used by you available or provide people with a location to    *
			* download it.                                                              *
			
			  You can contact the author by sending email to <cironian@stratics.com>.
			  
*/

#include <algorithm>
#include "uox3.h"

#include "weight.h"
#include "boats.h"
#include "books.h"
#include "cGuild.h"
#include "combat.h"
#include "msgboard.h"
#include "townregion.h"
#include "cWeather.hpp"
#include "movement.h"
#include "cRaces.h"
#include "cServerDefinitions.h"
#include "skills.h"
#include "commands.h"
#include "cSpawnRegion.h"
#include "wholist.h"
#include "cMagic.h"
#include "PageVector.h"
#include "speech.h"
#include "cSpeech.h"
#include "cVersionClass.h"
#include "ssection.h"
#include "cHTMLSystem.h"
#include "gump.h"
#include "trigger.h"
#include "cScript.h"
#include "cEffects.h"
#include "teffect.h"
#include "packets.h"
#include "fileio.h"
#include "classes.h"
#include "cThreadQueue.h"
#include "regions.h"
#include "magic.h"

#if defined(__unix__)
	#include <errno.h>
	#include <signal.h>
#endif

#include <set>

extern timeval current;

#undef DBGFILE
#define DBGFILE "uox3.cpp"

void DumpCreatures( void );
void checkkey( void );

#if defined(__unix__)
	typedef void *HANDLE;
	pthread_t cons, netw;
#endif

bool cluox_io          = false;   // is cluox-IO enabled?
bool cluox_nopipe_fill = false;   // the stdin-pipe is known to be none-empty, no need to fill it.
HANDLE cluox_stdin_writeback = 0; // the write-end of the stdin-pipe

//o---------------------------------------------------------------------------o
//|	Function	-	cl_getch
//|	Programmer	-	knoxos
//o---------------------------------------------------------------------------o
//|	Purpose		-	Read a character from stdin, in a cluox compatble way.
//|                 This routine is non-blocking!
//|	Returns		-	>0 -> character read
//|                 -1 -> no char available.
//o---------------------------------------------------------------------------o
//
// now cluox is GUI wrapper over uox using stdin and stdout redirection to capture
// the console, if it is active uox can't use kbhit() to determine if there is a 
// character aviable, it can only get one directly by getch().
// However the problem arises that uox will get blocked if none is aviable.
// The solution to this problem is that cluox also hands over the second pipe-end
// of stdin so uox can write itself into this stream. To determine if a character is 
// now done that way. UOX write's itself a ZERO on the other end of the pipe, and reads
// a charecter, if it is again the same ZERO just putted in nothing was entered. However
// it is not a zero the user has entered a char.
// 
int cl_getch( void )
{
#if defined(__unix__)
	// first the linux style, don't change it's behavoir
	UI08 c = 0;
	fd_set KEYBOARD;
	FD_ZERO( &KEYBOARD );
	FD_SET( 0, &KEYBOARD );
	int s = select( 1, &KEYBOARD, NULL, NULL, &cwmWorldState->uoxtimeout );
	if( s < 0 )
	{
		Console.Error( 1, "Error scanning key press" );
		Shutdown( 10 );
	}
	if( s > 0 )
	{
		read( 0, &c, 1 );
		if( c == 0x0A )
			return -1;
	}
#else
	// now the windows one
	if( !cluox_io )
	{
		// uox is not wrapped simply use the kbhit routine
		if( kbhit() )
			return getch();
		else 
			return -1;
	}
	// the wiered cluox getter.
	UI08 c = 0;
	UI32 bytes_written = 0;
	int asw = 0;
	if( !cluox_nopipe_fill )
		asw = WriteFile( cluox_stdin_writeback, &c, 1, &bytes_written, NULL );
	if( bytes_written != 1 || asw == 0 )
	{
		Console.Warning( 1, "Using cluox-io" );
		Shutdown( 10 );
	}
	c = (UI08)fgetc( stdin );
	if( c == 0 )
	{
		cluox_nopipe_fill = false;
		return -1;
	}
#endif
	// here an actual charater is read in
	return c;
}

//o---------------------------------------------------------------------------o
//|   Function    :  void DoMessageLoop( void )
//|   Date        :  Unknown
//|   Programmer  :  Unknown
//o---------------------------------------------------------------------------o
//|   Purpose     :  Watch for messages thrown by UOX
//o---------------------------------------------------------------------------o
void DoMessageLoop( void )
{
	while( !messageLoop.Empty() )
	{
		MessagePassed tVal = messageLoop.GrabMessage();
		switch( tVal.actualMessage )
		{
			case MSG_SHUTDOWN:		cwmWorldState->SetKeepRun( false );				break;
			case MSG_COUNT:															break; 
			case MSG_WORLDSAVE:		cwmWorldState->SetOldTime( 0 );					break;
			case MSG_PRINT:			Console << tVal.data << myendl;					break;
			case MSG_RELOADJS:		Trigger->ReloadJS();	Console.PrintDone();	break;
			case MSG_CONSOLEBCAST:	Speech->consolebroadcast( tVal.data );			break;
			case MSG_PRINTDONE:		Console.PrintDone();							break;
			case MSG_PRINTFAILED:	Console.PrintFailed();							break;
			case MSG_SECTIONBEGIN:	Console.PrintSectionBegin();					break;
			case MSG_UNKNOWN:
			default:				Console.Error( 2, "Unknown message type" );		break;
		}
	}
}
//------------------------------------------------------------
//---------------------------------------------------------------------------

//	EviLDeD	-	June 21, 1999
//	Ok here is thread number one its a simple thread for the checkkey() function
#if !defined(__unix__)
CRITICAL_SECTION sc;	//
#endif
bool conthreadcloseok = false;
bool netpollthreadclose = false;
void NetworkPollConnectionThread( void *params );

#if defined(__unix__)
void *CheckConsoleKeyThread( void *params )
#else
void CheckConsoleKeyThread( void *params )
#endif
{
	messageLoop << "Thread: CheckConsoleThread has started";
	conthreadcloseok = false;
	while( !conthreadcloseok )
	{
		checkkey();
		UOXSleep( 500 );
	}
#if !defined(__unix__)
	_endthread();		// linux will kill the thread when it returns
#endif
	messageLoop << "Thread: CheckConsoleKeyThread Closed";
#if defined(__unix__)
	return NULL;
#endif
}
//	EviLDeD	-	End

#if !defined(__unix__)
///////////////////

//HANDLE hco;
//CONSOLE_SCREEN_BUFFER_INFO csbi;

///////////////////
#endif

#if defined(__unix__)

void closesocket( UOXSOCKET s )
{
	shutdown( s, 2 );
	close( s );
}
#endif

//o--------------------------------------------------------------------------o
//|	Function		-	bool isOnline( CChar *c )
//|	Date			-	
//|	Developers		-	EviLDeD
//|	Organization	-	UOX3 DevTeam
//|	Status			-	Currently under development
//o--------------------------------------------------------------------------o
//|	Description		-	Check if the socket owning character c is still connected
//o--------------------------------------------------------------------------o
//| Modifications	-	
//o--------------------------------------------------------------------------o
bool isOnline( CChar *c )
{
	if( c == NULL )
		return false;
	if( c->IsNpc() )
		return false;
	ACCOUNTSBLOCK actbTemp = c->GetAccount();
	if( actbTemp.wAccountIndex != AB_INVALID_ID )
	{
		if( actbTemp.dwInGame == c->GetSerial() )
			return true;
	}
	Network->PushConn();
	for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
	{
		if( tSock->CurrcharObj() == c )
		{
			Network->PopConn();
			return true;
		}
	}
	Network->PopConn();
	return false;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void doGCollect( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Remove items in invalid locations (cleanup deleted packs and
//|					their items)
//o---------------------------------------------------------------------------o
void doGCollect( void )
{
	CChar *c;
	CItem *j;
	cBaseObject *iCont;
	UI32 rtotal = 0, removed = 0;
	bool idelete = false;
//	cwmWorldState->SetUICurrentTime( 0 );	// Why do we want to do this?
	
	Console << "Performing Garbage Collection...";
	do
	{
		removed = 0;
		for( ITEM i = 0; i < cwmWorldState->GetItemCount(); ++i )
		{
			if( !items[i].isFree() )
			{
				idelete = false; 
				iCont = items[i].GetCont();
				if( iCont != NULL )
				{
					idelete = true;
					if( iCont->GetObjType() == OT_CHAR ) // container is a character...verify the character??
					{
						c = (CChar *)iCont;
						if( c != NULL )
						{
							if( !c->isFree() )
								idelete = false;
						}
					} 
					else 
					{// find the container if there is one.
						j = (CItem *)iCont;
						if( j != NULL )
						{
							if( !j->isFree() )
								idelete = false;
						}
					}
				}
				if( idelete )
				{
					Items->DeleItem( &items[i] );
					++removed;
				}
			}
		}
		rtotal += removed;
	} while( removed > 0 );
	
	
//	cwmWorldState->SetUICurrentTime( getclock() );
    Console << " Removed " << rtotal << " items." << myendl;
}

//o---------------------------------------------------------------------------o
//|	Function	-	char *RealTime( char *time_str )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Returns the real time
//o---------------------------------------------------------------------------o
char *RealTime(char *time_str)
{
	struct tm *curtime;
	time_t bintime;
	time(&bintime);
	curtime = localtime(&bintime);
	strftime(time_str, 256, "%B %d %I:%M:%S %p", curtime);
	return time_str;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *getPack( CChar *p )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get characters main pack
//o---------------------------------------------------------------------------o
CItem *getPack( CChar *p )
{
	if( p == NULL )
		return NULL;

	CItem *i = p->GetPackItem();
	if( i != NULL )
	{
		if( i->GetCont() == p && i->GetLayer() == 0x15 )
			return i;
	}

	CItem *packItem = p->GetItemAtLayer( 0x15 );
	if( packItem != NULL )
	{
		p->SetPackItem( packItem );
		return packItem;
	}
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void openPack( cSocket *s, CItem *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Open backpack and display contents
//o---------------------------------------------------------------------------o
void openPack( cSocket *s, CItem *i )
{
	if( i == NULL )
	{
		Console.Warning( 2, "openPack() was passed an invalid item" );
		return;
	}
	CPDrawContainer contSend = (*i);
	contSend.Model( 0x3C );

	if( i->GetID( 1 ) == 0x3E )            // boats
		contSend.Model( 0x4C );
	else
	{
		switch( i->GetID() )
		{
			case 0x2006:      // coffin
				contSend.Model( 0x09 );
				break;
			case 0x0E75:      // backpack
			case 0x0E79:      // pouch
			case 0x09B0:	  // pouch
				contSend.Model( 0x3C );
				break;
			case 0x0E76:      // leather bag
				contSend.Model( 0x3D );
				break;
			case 0x0E77:      // barrel
			case 0x0E7F:      // keg
			case 0x0E83:
			case 0x0FAE:	// barrel with lids
			case 0x1AD7:	// potion kegs
			case 0x1940:	// barrel with lids
				contSend.Model( 0x3E );
				break;
			case 0x0E7A:      // square basket
				contSend.Model( 0x3F );
				break;
			case 0x0990:      // round basket
			case 0x09AC:
			case 0x09B1:
				contSend.Model( 0x41 );
				break;
			case 0x0E40:      // metal & gold chest
			case 0x0E41:      // metal & gold chest
				contSend.Model( 0x42 );
				break;
			case 0x0E7D:      // wooden box
			case 0x09AA:      // wooden box
				contSend.Model( 0x43 );
				break;
			case 0x0E3C:      // large wooden crate
			case 0x0E3D:      // large wooden crate
			case 0x0E3E:      // small wooden create
			case 0x0E3F:      // small wooden crate
			case 0x0E7E:      // wooden crate
			case 0x09A9:      // small wooden crate
				contSend.Model( 0x44 );
				break;
			case 0x2AF8:		// Shopkeeper buy, sell and sold layers
				contSend.Model( 0x47 );
				break;
			case 0x0A30:   // chest of drawers (fancy)
			case 0x0A38:   // chest of drawers (fancy)
				contSend.Model( 0x48 );
				break;
			case 0x0E42:      // wooden & gold chest
			case 0x0E43:      // wooden & gold chest
				contSend.Model( 0x49 );
				break;
			case 0x0E80:      // brass box
			case 0x09A8:      // metal box
				contSend.Model( 0x4B );
				break;
			case 0x0E7C:      // silver chest
			case 0x09AB:      // metal & silver chest
				contSend.Model( 0x4A );
				break;
				
			case 0x0A97:   // bookcase
			case 0x0A98:   // bookcase
			case 0x0A99:   // bookcase
			case 0x0A9A:   // bookcase
			case 0x0A9B:   // bookcase
			case 0x0A9C:   // bookcase
			case 0x0A9D:	// bookcase (empty)
			case 0x0A9E:	// bookcase (empty)
				contSend.Model( 0x4D );
				break;

			case 0x0A4C:   // fancy armoire (open)
			case 0x0A4D:   // fancy armoire
			case 0x0A50:   // fancy armoire (open)
			case 0x0A51:   // fancy armoire
				contSend.Model( 0x4E );
				break;
				
			case 0x0A4E:   // wooden armoire (open)
			case 0x0A4F:   // wooden armoire
			case 0x0A52:   // wooden armoire (open)
			case 0x0A53:   // wooden armoire
				contSend.Model( 0x4F );
				break;
			case 0x0A2C:   // chest of drawers (wood)
			case 0x0A34:   // chest of drawers (wood)
			case 0x0A35:   // dresser
			case 0x0A3C:   // dresser
			case 0x0A3D:   // dresser
			case 0x0A44:   // dresser
				contSend.Model( 0x51 );
				break;
			case 0x09B2:      // bank box ..OR.. backpack 2
				if( i->GetMoreX() == 1 )
					contSend.Model( 0x4A );
				else 
					contSend.Model( 0x3C );
				break;
		}
	}   
	s->Send( &contSend );
	CPItemsInContainer itemsIn( i );
	s->Send( &itemsIn );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void explodeItem( cSocket *mSock, CItem *nItem )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Explode an item
//o---------------------------------------------------------------------------o
void explodeItem( cSocket *mSock, CItem *nItem )
{
	CChar *c = mSock->CurrcharObj();
	UI32 dmg = 0;
	UI32 dx, dy, dz;
	// - send the effect (visual and sound)
	if( nItem->GetCont() != NULL )
	{
		Effects->staticeffect( c, 0x36B0, 0x00, 0x09 );
		nItem->SetCont( NULL );
		nItem->SetLocation(  c );
		Effects->PlaySound( c, 0x0207 );
	}
	else
	{
		Effects->staticeffect( nItem, 0x36B0, 0x00, 0x09, 0x00);
		Effects->PlaySound( nItem, 0x0207 );
	}
	UI32 len	= nItem->GetMoreX() / 250; //4 square max damage at 100 alchemy
	dmg			= RandomNum( nItem->GetMoreZ() * 5, nItem->GetMoreZ() * 10 );
	
	if( dmg < 5 )
		dmg = RandomNum( 5, 10 );  // 5 points minimum damage
	if( len < 2 )
		len = 2;  // 2 square min damage range
	SI16 xOffset = MapRegion->GetGridX( nItem->GetX() );
	SI16 yOffset = MapRegion->GetGridY( nItem->GetY() );
	
	UI08 worldNumber = nItem->WorldNumber();

	for( dx = 0xffffffff; dx <= 1; ++dx )
	{
		for( dy = 0xffffffff; dy <= 1; ++dy )
		{
			SubRegion *Cell = MapRegion->GetGrid( xOffset+dx, yOffset+dy, worldNumber );
			bool chain = false;
	
			Cell->PushChar();
			Cell->PushItem();
			for( CChar *tempChar = Cell->FirstChar(); !Cell->FinishedChars(); tempChar = Cell->GetNextChar() )
			{
				dx = abs( tempChar->GetX() - nItem->GetX() );
				dy = abs( tempChar->GetY() - nItem->GetY() );
				dz = abs( tempChar->GetZ() - nItem->GetZ() );
				if( dx <= len && dy <= len && dz <= len )
				{
					if( !tempChar->IsGM() && !tempChar->IsInvulnerable() && ( tempChar->IsNpc() || isOnline( tempChar ) ) )
					{
						if( tempChar->IsInnocent() && tempChar->GetSerial() != c->GetSerial() )
							criminal( c );
						tempChar->SetHP( (UI16)( tempChar->GetHP() - ( dmg + ( 2 - UOX_MIN( dx, dy ) ) ) ) );
						updateStats( tempChar, 0 );
						if( tempChar->GetHP() <= 0 )
							doDeathStuff( tempChar );
						else
							Combat->AttackTarget( c, tempChar );
					}
				}
			}
			Cell->PopChar();
			// Handle STL properly
			ITEMLIST_ITERATOR I;
			//for( CItem *tempItem = Cell->FirstItem(); !Cell->FinishedItems(); tempItem = Cell->GetNextItem() )
			for( I=Cell->itemData.begin(); I!=Cell->itemData.end(); ++I )
			{
				CItem * tempItem;
				tempItem=(*I);
				if( tempItem->GetID() == 0x0F0D && tempItem->GetType() == 19 )
				{
					dx = abs( nItem->GetX() - tempItem->GetX() );
					dy = abs( nItem->GetY() - tempItem->GetY() );
					dz = abs( nItem->GetZ() - tempItem->GetZ() );
			
					if( dx <= 2 && dy <= 2 && dz <= 2 && !chain ) // only trigger if in a 2*2*2 cube
					{
						if( !( dx == 0 && dy == 0 && dz == 0 ) )
						{
							if( RandomNum( 0, 1 ) == 1 )
								chain = true;
							Effects->tempeffect( c, tempItem, 17, 0, 1, 0 );
						}
					}
				}
			}
			Cell->PopItem();
		}
	}
	Items->DeleItem( nItem );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void updateStats( CChar *c, char x )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Updates characters stats
//o---------------------------------------------------------------------------o
void updateStats( CChar *c, char x )
{
	CPUpdateStat toSend( (*c), x );
	Network->PushConn();
	for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
	{
		if( tSock->CurrcharObj() == c )
		{
			tSock->Send( &toSend );
			tSock->statwindow( c );
		}
		if( charInRange( tSock->CurrcharObj(), c ) )
			tSock->Send( &toSend );
	}
	Network->PopConn();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void MountCreature( CChar *s, CChar *x )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Mount a ridable creature
//|									
//|	Modification	-	09/22/2002	-	Xuri - Unhide players when mounting horses etc.
//o---------------------------------------------------------------------------o
//|	Returns			- N/A
//o--------------------------------------------------------------------------o	
void MountCreature( CChar *s, CChar *x )
{
	cSocket *sockPtr = calcSocketObjFromChar( s );
	if( !objInRange( s, x, DIST_NEXTTILE ) )
		return;
	char temp[1024];
	if( x->GetOwnerObj() == s || s->IsGM() )
	{
		if( s->IsOnHorse() )
		{
			if( !cwmWorldState->ServerData()->CharHideWhileMounted() )
				s->ExposeToView();
		}
		strcpy( temp, x->GetName().c_str() );
		s->SetOnHorse( true );
		CItem *c = Items->SpawnItem( NULL, s, 1, temp, false, 0x0915, x->GetSkin(), false, false );

		// Weazel 12 July, 2001 - Not all 3rd dawn creature mount id's are correct still missing a faction horse/dragon horse and
		// the ethereal llama and ostards.
		switch( x->GetID() )
		{
			case 0x72:	c->SetID( 0x3EA9 );	break;	// Dark Steed
			case 0x73:	c->SetID( 0x3EAA );	break;	// Etheral Horse
			case 0x74:	c->SetID( 0x3EB5 );	break;	// Nightmare
			case 0x75:	c->SetID( 0x3EA8 );	break;	// Silver Steed
			case 0x76:	c->SetID( 0x3EB2 );	break;	// Faction Horse
			case 0x77:	c->SetID( 0x3EB1 );	break;	// Faction Horse
			case 0x78:	c->SetID( 0x3EAF );	break;	// Faction Horse
			case 0x79:	c->SetID( 0x3EB0 );	break;	// Faction Horse
			case 0x7A:	c->SetID( 0x3EB4 );	break;	// Unicorn
			case 0x84:	c->SetID( 0x3EAD );	break;	// Kirin
			case 0x90:	c->SetID( 0x3EB3 );	break;	// Sea Horse
			case 0xAA:	c->SetID( 0x3EAB );	break;	// Etheral Llama
			case 0xAB:	c->SetID( 0x3EAC );	break;	// Etheral Ostard
			case 0xB1:	c->SetID( 0x3EA7 );	break;	// Nightmare
			case 0xB2:	c->SetID( 0x3EA9 );	break;	// Nightmare
			case 0xB3:	c->SetID( 0x3EB7 );	break;	// TD Nightmare
			case 0xBB:	c->SetID( 0x3EB8 );	break;	// Ridgeback
			case 0xBE:	c->SetID( 0x3E9E );	break;	// Fire Steed
			case 0xBF:	c->SetID( 0x3E9C );	break;	// Etheral Kirin
			case 0xCC:	c->SetID( 0x3EA2 );	break;	// horse
			case 0xC0:	c->SetID( 0x3EB4 );	break;	// Etheral Unicorn
			case 0xC1:	c->SetID( 0x3E9A );	break;	// Etheral Ridgeback
			case 0xC2:	c->SetID( 0x3E98 );	break;	// Etheral Swampdragon
			case 0xC3:	c->SetID( 0x3E97 );	break;	// Etheral Beetle
			case 0xC8:	c->SetID( 0x3E9F );	break;	// horse
			case 0xD2:	c->SetID( 0x3EA3 );	break;	// Desert Ostard
			case 0xDA:	c->SetID( 0x3EA4 );	break;	// Frenzied Ostard
			case 0xDB:	c->SetID( 0x3EA5 );	break;	// Forest Ostard
			case 0xDC:	c->SetID( 0x3EA6 );	break;	// llama
			case 0xE2:	c->SetID( 0x3EA0 );	break;	// horse
			case 0xE4:	c->SetID( 0x3EA1 );	break;	// horse
			case 0x319:	c->SetID( 0x3EBB );	break;  // Skeletal Mount 
			case 0x317:	c->SetID( 0x3EBC );	break;  // Giant Beetle  
			case 0x31A:	c->SetID( 0x3EBD );	break;  // Swamp Dragon  
			case 0x31F:	c->SetID( 0x3EBE );	break;  // Armored Swamp Dragon  
			case 0x3E6:	c->SetID( 0x3EAD );	break;	// Kirin
			default:	c->SetID( 0x3E00 );	break;	// Bad
		}
		
		c->SetLayer( 0x19 );

		if( !c->SetCont( s ) )
		{
			s->SetOnHorse( false );	// let's get off our horse again
			return;
		}
		//s->WearItem( c );
		s->SendWornItems( sockPtr ); // send update to current socket
		Network->PushConn();
		for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() ) // and to all inrange sockets (without re-sending to current socket )
		{
			if( sockPtr != tSock && charInRange( s, tSock->CurrcharObj() ) )
				s->SendWornItems( tSock );
		}
		Network->PopConn();

		if( x->GetTarg() != INVALIDSERIAL )	// zero out target, under all circumstances
		{
			x->SetTarg( INVALIDSERIAL );
			x->SetWar( false );
		}
		if( x->GetAttacker() != NULL )
			x->GetAttacker()->SetTarg( INVALIDSERIAL );
		x->SetLocation( 7000, 7000, 0 );
		x->SetFrozen( true );
		c->SetMoreX( x->GetSerial() );
		if( x->GetTimer( tNPC_SUMMONTIME ) != 0 )
			c->SetDecayTime( x->GetTimer( tNPC_SUMMONTIME ) );

		x->Update();
	}
	else
		sockPtr->sysmessage( 1214 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void DismountCreature( CChar *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Dismount a ridable creature
//o---------------------------------------------------------------------------o
void DismountCreature( CChar *s )
{
	if( s == NULL )
		return;
	CItem *ci = s->GetItemAtLayer( 0x19 );
	if( ci == NULL || ci->isFree() )	// if no horse item, or it's our default one
		return;

	s->SetOnHorse( false );
	CChar *tMount = calcCharObjFromSer( ci->GetMoreX() );
	if( tMount != NULL )
	{
		tMount->SetLocation( s );
		tMount->SetFrozen( false );
		if( ci->GetDecayTime() != 0 )
			tMount->SetTimer( tNPC_SUMMONTIME, ci->GetDecayTime() );
		tMount->SetDir( s->GetDir() );
		tMount->SetVisible( 0 );
		tMount->Update();
	}
	Items->DeleItem( ci );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void endmessage( int x )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Global message players with shutdown message
//o---------------------------------------------------------------------------o
void endmessage( int x )
{
	x = 0;
	UI32 igetclock = cwmWorldState->GetUICurrentTime();
	if( cwmWorldState->GetEndTime() < igetclock )
		cwmWorldState->SetEndTime( igetclock );
	char temp[1024];
	sprintf( temp, Dictionary->GetEntry( 1209 ).c_str(), ((cwmWorldState->GetEndTime()-igetclock)/ 1000 ) / 60 );
	Speech->sysbroadcast( temp );
}

#if defined(__unix__)
void illinst( int x = 0 ) //Thunderstorm linux fix
{
	Speech->sysbroadcast( "Fatal Server Error! Bailing out - Have a nice day!" );
	Console.Error( 0, "Illegal Instruction Signal caught - attempting shutdown" );
	endmessage( x );
}

void aus( int signal )
{
	Console.Error( 2, "Server crash averted! Floating point exception caught." );
} 

#endif

//o---------------------------------------------------------------------------o
//|	Function	-	void weblaunch( cSocket *s, char *txt )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Launch a webpage from within the client
//o---------------------------------------------------------------------------o
void weblaunch( cSocket *s, const char *txt )
{
	s->sysmessage( 1210 );
	CPWebLaunch toSend( txt );
	s->Send( &toSend );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void scriptcommand( cSocket *s, char *cmd, char *data )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Execute a command from scripts
//o---------------------------------------------------------------------------o
void scriptcommand( cSocket *s, const char *cmd2, const char *data2 )
{
	char tstring[1024];
	int total, ho, mi, se;
	CChar *mChar = s->CurrcharObj();
	char idname[256];
	if( !s || !cmd2 || !data2 )
		return;

	UString cmd( cmd2 );
	UString data( data2 );
	cmd		= cmd.upper();
	data	= data.upper();
	
	switch( cmd.data()[0] )
	{
		case 'A':
			if( cmd == "ADDITEM" )
			{
				if( data.sectionCount( "," ) != 0 )
				{
					UString tmp	= data.section( ",", 0, 0 ).stripWhiteSpace();
					UI16 num	= data.section( ",", 1, 1 ).stripWhiteSpace().toUShort();
					Items->menuAddItem( s, tmp, num );
				}
				else
					Items->menuAddItem( s, data );
			}
			break;
		case 'B':
			if( cmd == "BATCH" )
				cwmWorldState->SetExecuteBatch( data.toLong() );
			break;
		case 'C':
			if( cmd == "CPAGE" )
				Commands->CPage( s, data.c_str() );
			else if( cmd == "CREATETRACKINGMENU" )
				Skills->CreateTrackingMenu( s, data.toLong() );
			break;
		case 'G':
			if( cmd == "GMMENU" )
				gmmenu( s, data.toLong() );
			else if( cmd == "GMPAGE" )
				Commands->GMPage( s, data.c_str() );
			else if( cmd == "GCOLLECT" )
				doGCollect();
			else if( cmd == "GOPLACE" )
			{
				Commands->MakePlace( s, data.toLong() );
				if( s->AddX() != 0 )
				{
					mChar->SetLocation( (UI16)s->AddX(), (UI16)s->AddY(), s->AddZ() );
					mChar->Teleport();
				}
			}
			else if( cmd == "GUIINFORMATION" )
			{
				GumpDisplay guiInfo( s, 400, 300 );
				guiInfo.SetTitle( "Server status" );

				total	= (cwmWorldState->GetUICurrentTime() - cwmWorldState->GetStartTime() ) / 1000;
				ho		= total / 3600;
				total	-= ho * 3600;
				mi		= total / 60;
				total	-= mi * 60;
				se		= total;
				total	= 0;
				UString builtString = "";
				if( ho < 10 )
					builtString += "0";
				builtString += UString::number( ho ) + ":";
				if( mi < 10 )
					builtString += "0";
				builtString += UString::number( mi ) + ":";
				if( se < 10 )
					builtString += "0";
				builtString += UString::number( se );
				guiInfo.AddData( "Uptime", builtString.c_str() );
				guiInfo.AddData( "Accounts", Accounts->size() );
				guiInfo.AddData( "Items", items.Count() );
				guiInfo.AddData( "Chars", chars.Count() );
				guiInfo.AddData( "Players in world", cwmWorldState->GetPlayersOnline() );
				sprintf( idname, "%s v%s(%s) [%s] Compiled by %s ", cVersionClass::GetProductName().c_str(), cVersionClass::GetVersion().c_str(), cVersionClass::GetBuild().c_str(), OS_STR, cVersionClass::GetName().c_str() );
				guiInfo.AddData( idname, idname, 7 );
				guiInfo.Send( 0, false, INVALIDSERIAL );
			}
			break;
		case 'I':
			if( cmd == "ITEMMENU" )
				NewAddMenu( s, data.toLong() );
			else if( cmd == "INFORMATION" )
			{
				total	= (cwmWorldState->GetUICurrentTime() - cwmWorldState->GetStartTime() ) / 1000;
				ho		= total / 3600;
				total	-= ho * 3600;
				mi		= total / 60;
				total	-= mi * 60;
				se		= total;
				total	= 0;
				s->sysmessage( 1211, ho, mi, se, cwmWorldState->GetPlayersOnline(), Accounts->size(), items.Count(), chars.Count() );
			}
			break;
		case 'M':
			if( cmd == "MAKEMENU" )
				Skills->NewMakeMenu( s, data.toLong(), (UI08)s->AddID() );
			break;
		case 'N':
			if( cmd == "NPC" )
			{
				s->XText( data.c_str() );
				sprintf( tstring, Dictionary->GetEntry( 1212, s->Language() ).c_str(), s->XText() );
				s->target( 0, TARGET_NPCMENU, tstring );
			}
			break;
		case 'P':
			if( cmd == "POLY" )
			{
				UI16 newBody = data.toUShort();
				mChar->SetID( newBody );
				mChar->SetOrgID( newBody );
				mChar->Teleport();	// why this is needed I'm not sure
			}
			break;
		case 'S':
			if( cmd == "SYSMESSAGE" )
				s->sysmessage( data.c_str() );
			else if( cmd == "SKIN" )
			{
				COLOUR newSkin = data.toUShort();
				mChar->SetSkin( newSkin );
				mChar->SetOrgSkin( newSkin );
				mChar->Teleport();
			}
			break;
		case 'T':
			if( cmd == "TRACKINGMENU" )
				Skills->TrackingMenu( s, data.toLong() );
			break;
		case 'V':
			if( cmd == "VERSION" )
			{
				sprintf( idname, "%s v%s(%s) [%s] Compiled by %s ", cVersionClass::GetProductName().c_str(), cVersionClass::GetVersion().c_str(), cVersionClass::GetBuild().c_str(), OS_STR, cVersionClass::GetName().c_str() );
				s->sysmessage( idname );
			}
			break;
		case 'W':
			if( cmd == "WEBLINK" )
				weblaunch( s, data.c_str() );
			break;
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void batchcheck( cSocket *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check if we run a batch file
//o---------------------------------------------------------------------------o
void batchcheck( cSocket *s )
{
	char temp[1024];
	sprintf( temp, "BATCH %i", cwmWorldState->GetExecuteBatch() );
	ScriptSection *Batch = FileLookup->FindEntry( temp, menus_def );
	if( Batch == NULL )
		return;
	UString data;
	UString tag;
	for( tag = Batch->First(); !Batch->AtEnd(); tag = Batch->Next() )
	{
		data = Batch->GrabData();
		scriptcommand( s, tag.c_str(), data.c_str() );
	}
	cwmWorldState->SetExecuteBatch( 0 );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void callguards( CChar *mChar )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Used when a character calls "Guards" Will look for a criminal
//|					first checking for anyone attacking him. If no one is attacking
//|					him it will look for any people nearby who are criminal or
//|					murderers
//o---------------------------------------------------------------------------o
void callGuards( CChar *mChar )
{
	if( mChar == NULL )
		return;

	if( !region[mChar->GetRegion()]->IsGuarded() || !cwmWorldState->ServerData()->GuardsStatus() )
		return;

	SI16 xOffset = MapRegion->GetGridX( mChar->GetX() );
	SI16 yOffset = MapRegion->GetGridY( mChar->GetY() );

	SubRegion *toCheck = MapRegion->GetGrid( xOffset, yOffset, mChar->WorldNumber() );
	if( toCheck == NULL )
		return;

	CChar *attacker = mChar->GetAttacker();
	if( attacker != NULL )
	{
		if( !attacker->IsDead() && ( attacker->IsCriminal() || attacker->IsMurderer() ) )
		{
			if( charInRange( mChar, attacker ) )
			{
				callGuards( mChar, attacker );
				return;
			}
		}
	}

	for( CChar *tempChar = toCheck->FirstChar(); !toCheck->FinishedChars(); tempChar = toCheck->GetNextChar() )
	{
		if( tempChar == NULL )
			break;

		if( !tempChar->IsDead() && ( tempChar->IsCriminal() || tempChar->IsMurderer() ) )
		{
			if( charInRange( tempChar, mChar ) )
			{
				callGuards( mChar, tempChar );
				return;
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void callguards( CChar *mChar, CChar *targChar )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Used when a character calls guards on another character, will
//|					ensure that character is not dead and is either a criminal or
//|					murderer, and that he is in visual range of the victim, will
//|					then spawn a guard to take care of the criminal.
//o---------------------------------------------------------------------------o
void callGuards( CChar *mChar, CChar *targChar )
{
	if( mChar == NULL || targChar == NULL )
		return;

	if( !region[mChar->GetRegion()]->IsGuarded() || !cwmWorldState->ServerData()->GuardsStatus() )
		return;

	if( !targChar->IsDead() && ( targChar->IsCriminal() || targChar->IsMurderer() ) )
	{
		if( charInRange( mChar, targChar ) )
			Combat->SpawnGuard( mChar, targChar, targChar->GetX(), targChar->GetY(), targChar->GetZ() );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void DisplaySettings( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	UOX startup stuff
//| Moved that here because we need it in processkey now
//|									
//|	Modification	-	10/21/2002	-	EviLDeD - Xuri found the bug in one spot, just
//|									happened upon this quick fix. for BackUp operation.
//o---------------------------------------------------------------------------o
void DisplaySettings( void )
{
	std::map< bool, std::string > activeMap;
	activeMap[true] = "Activated!";
	activeMap[false] = "Disabled!";

	// Server.scp status --- By Magius(CHE)
	Console << "Server Settings:" << myendl;
	
	Console << "   -Archiving[";
	if( cwmWorldState->ServerData()->ServerBackupStatus() )
		Console << "Enabled]. (" << cwmWorldState->ServerData()->Directory( CSDDP_BACKUP ) << ")" << myendl;
	else 
		Console << "Disabled]" << myendl;
	
	Console << "   -Weapons & Armour Rank System: ";
	Console << activeMap[cwmWorldState->ServerData()->RankSystemStatus()].c_str() << myendl;
	
	Console << "   -Vendors buy by item name: ";
	Console << activeMap[cwmWorldState->ServerData()->SellByNameStatus()].c_str() << myendl;
	
	Console << "   -Adv. Trade System: ";
	Console << activeMap[cwmWorldState->ServerData()->TradeSystemStatus()].c_str() << myendl;
	
	Console << "   -Crash Protection: ";
	if( cwmWorldState->ServerData()->ServerCrashProtectionStatus() < 1 )
		Console << "Disabled!" << myendl;
#ifndef _CRASH_PROTECT_
	else 
		Console << "Unavailable in this version" << myendl;
#else
	else if( cwmWorldState->ServerData()->ServerCrashProtectionStatus() == 1 )
		Console << "Save on crash" << myendl;
	else 
		Console << "Save & Restart Server" << myendl;
#endif

	Console << "   -xGM Remote: ";
	Console << activeMap[cwmWorldState->GetXGMEnabled()].c_str() << myendl;

	Console << "   -Races: " << static_cast< UI32 >(Races->Count()) << myendl;
	Console << "   -Guilds: " << static_cast< UI32 >(GuildSys->NumGuilds()) << myendl;
	Console << "   -Char count: " << cwmWorldState->GetCMem() << myendl;
	Console << "   -Item count: " << cwmWorldState->GetIMem() << myendl;
	Console << "   -Num Accounts: " << Accounts->size() << myendl;
	Console << "   Directories: " << myendl;
	Console << "   -Shared:          " << cwmWorldState->ServerData()->Directory( CSDDP_SHARED ) << myendl;
	Console << "   -Archive:         " << cwmWorldState->ServerData()->Directory( CSDDP_BACKUP ) << myendl;
	Console << "   -Data:            " << cwmWorldState->ServerData()->Directory( CSDDP_DATA ) << myendl;
	Console << "   -Defs:            " << cwmWorldState->ServerData()->Directory( CSDDP_DEFS ) << myendl;
	Console << "   -Scripts:         " << cwmWorldState->ServerData()->Directory( CSDDP_SCRIPTS ) << myendl;
	Console << "   -HTML:            " << cwmWorldState->ServerData()->Directory( CSDDP_HTML ) << myendl;
	Console << "   -Books:           " << cwmWorldState->ServerData()->Directory( CSDDP_BOOKS ) << myendl;
	Console << "   -MessageBoards:   " << cwmWorldState->ServerData()->Directory( CSDDP_MSGBOARD ) << myendl;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void processkey( int c )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle keypresses in console
//o---------------------------------------------------------------------------o
void processkey( int c )
{
	char outputline[128], temp[1024];
	bool kill = false;
	int indexcount = 0;
	int j;
	int keyresp;

	if( c == '*' )
	{
		if( cwmWorldState->GetSecure() )
			messageLoop << "Secure mode disabled. Press ? for a commands list";
		else
			messageLoop << "Secure mode re-enabled";
		cwmWorldState->SetSecure( !cwmWorldState->GetSecure() );
		return;
	} 
	else 
	{
		if( cwmWorldState->GetSecure() )
		{
			messageLoop << "Secure mode prevents keyboard commands! Press '*' to disable";
			return;
		}
		
		switch( c )
		{
			case '!':
				// Force server to save accounts file
				messageLoop << "CMD: Saving Accounts... ";
				Accounts->Save();
				messageLoop << MSG_PRINTDONE;
				break;
			case '@':
				// Force server to save all files.(Manual save)
				messageLoop << MSG_WORLDSAVE;
				break;
		case 'Y':
#if defined( _MSC_VER )
#pragma note("Console Broadcast needs to not require cout anymore.  Is there a better way?")
#endif
			//messageLoop << "Console> ";
			std::cout << "System: ";
			while( !kill )
			{
				keyresp = cl_getch();
				switch( keyresp )
				{
					case -1:	// no key pressed
					case 0:
						break;
					case 0x1B:
						memset( outputline, 0x00, sizeof( outputline ) );
						indexcount = 0;
						kill = true;
						std::cout << std::endl;
						messageLoop << "| CMD: System broadcast canceled.";
						break;
					case 0x08:
						--indexcount;
						if( indexcount < 0 )	
							indexcount = 0;
						else
							std::cout << "\b \b";
						break;
					case 0x0A:
					case 0x0D:
						outputline[indexcount] = 0;
						messageLoop.NewMessage( MSG_CONSOLEBCAST, outputline );
						indexcount = 0;
						kill = true;
						std::cout << std::endl;
						sprintf( temp, "| CMD: System broadcast sent message \"%s\"", outputline );
						memset( outputline, 0x00, sizeof( outputline ) );
						messageLoop << temp;
						break;
					default:
						if( indexcount < sizeof( outputline ) )
						{
							outputline[indexcount++] = (UI08)(keyresp);
							std::cout << (char)keyresp;
						}
						break;
				}
				keyresp = 0x00;
			}
			break;
			case 0x1B:
			case 'Q':
				messageLoop << MSG_SECTIONBEGIN;
				messageLoop << "CMD: Immediate Shutdown initialized!";
				messageLoop << MSG_SHUTDOWN;
				break;
			case '1':
				// Reload server ini file
				messageLoop << "CMD: Loading Server INI... ";
				cwmWorldState->ServerData()->load();
				messageLoop << MSG_PRINTDONE;
				break;
			case '2':
				// Reload accounts, and update Access.adm if new accounts available.
				messageLoop << "CMD: Loading Accounts... ";
				Accounts->Load();
				messageLoop << MSG_PRINTDONE;
				break;
			case '3':
				// Reload Region Files
				messageLoop << "CMD: Loading Regions... ";
				FileIO->LoadRegions();
				messageLoop << MSG_PRINTDONE;
				break;
			case '4':
				// Reload the serve spawn regions
				messageLoop << "CMD: Loading Spawn Regions... ";
				FileIO->LoadSpawnRegions();
				messageLoop << MSG_PRINTDONE;
				break;
			case '5':
				// Reload the current Spells 
				messageLoop << "CMD: Loading spells... ";
				Magic->LoadScript();
				messageLoop << MSG_PRINTDONE;
				break;
			case '6':
				// Reload the server command list
				messageLoop << "CMD: Loading commands... ";
				Commands->Load();
				messageLoop << MSG_PRINTDONE;
				break;
			case '7':
				// Reload the server defantion files
				messageLoop << "CMD: Loading Server DFN... ";
				FileLookup->Reload();
				messageLoop << MSG_PRINTDONE;
				break;
			case '8':
				// messageLoop access is REQUIRED, as this function is executing in a different thread, so we need thread safety
				messageLoop << "CMD: Loading JSE Scripts... ";
				messageLoop << MSG_RELOADJS;
				break;
			case '9':
				// Reload the HTML output templates
				messageLoop << "CMD: Loading HTML Templates... ";
				HTMLTemplates->Load(ETT_ALLTEMPLATES);
				messageLoop << MSG_PRINTDONE;
				break;
			case '0':
				// Reload all the files. If there are issues with these files change the order reloaded from here first.
				cwmWorldState->ServerData()->load();
				messageLoop << "CMD: Loading All";
				messageLoop << "     Server INI... ";
				// Reload accounts, and update Access.adm if new accounts available.
				messageLoop << "     Loading Accounts... ";
				Accounts->Load();
				messageLoop << MSG_PRINTDONE;
				// Reload Region Files
				messageLoop << "     Loading Regions... ";
				FileIO->LoadRegions();
				messageLoop << MSG_PRINTDONE;
				// Reload the serve spawn regions
				messageLoop << "     Loading Spawn Regions... ";
				FileIO->LoadSpawnRegions();
				messageLoop << MSG_PRINTDONE;
				// Reload the current Spells 
				messageLoop << "     Loading spells... ";
				Magic->LoadScript();
				messageLoop << MSG_PRINTDONE;
				// Reload the server command list
				messageLoop << "     Loading commands... ";
				Commands->Load();
				messageLoop << MSG_PRINTDONE;
				// Reload DFN's
				messageLoop << "     Loading Server DFN... ";
				FileLookup->Reload();
				messageLoop << MSG_PRINTDONE;
				// messageLoop access is REQUIRED, as this function is executing in a different thread, so we need thread safety
				messageLoop << "     Loading JSE Scripts... ";
				messageLoop << MSG_RELOADJS;
				// Reload the HTML output templates
				messageLoop << "     Loading HTML Templates... ";
				HTMLTemplates->Load(ETT_ALLTEMPLATES);
				messageLoop << MSG_PRINTDONE;
				break;
		case 'T':
			// Timed shut down(10 minutes)
			messageLoop << "CMD: 10 Minute Server Shutdown Announced(Timed)";
			cwmWorldState->SetEndTime( BuildTimeValue( 600 ) );
			endmessage(0);
			break;
		case 'L':
			// Show Layer info
			if( cwmWorldState->GetDisplayLayers() )
				messageLoop << "CMD: Show Layer Disabled";
			else
				messageLoop << "CMD: Show Layer Enabled";
			cwmWorldState->SetDisplayLayers( !cwmWorldState->GetDisplayLayers() );
			break;
		case  'D':    
			// Disconnect account 0 (useful when client crashes)
			cSocket *tSock;
			for( tSock = Network->LastSocket(); tSock != NULL; tSock = Network->PrevSocket() )
			{
				if( tSock->AcctNo() == 0 )
					Network->Disconnect( tSock );
			}
			messageLoop << "CMD: Socket Disconnected(Account 0).";
			break;
		case 'K':
		{
			// mass disconnect
			size_t massCount = cwmWorldState->GetPlayersOnline();
			// size_t is unsigned, so it WILL wrap around
			for( size_t m = massCount - 1; m >= 0 && m < massCount; --m )
				Network->Disconnect( m );
			messageLoop << "CMD: All Connections Closed.";
		}
			break;
		case 'P':                
			// 1/13/2003 - Dreoth - Log Performance Information enhancements
			Console.LogEcho( true );
			Console.Log( "--- Starting Performance Dump ---", "performance.log");
			Console.Log( "Performace Dump:", "performance.log");
			Console.Log( "Network code: %.2fmsec [%i samples]", "performance.log", (R32)((R32)networkTime/(R32)networkTimeCount), networkTimeCount);
			Console.Log( "Timer code: %.2fmsec [%i samples]", "performance.log", (R32)((R32)timerTime/(R32)timerTimeCount), timerTimeCount);
			Console.Log( "Auto code: %.2fmsec [%i samples]", "performance.log", (R32)((R32)autoTime/(R32)autoTimeCount), autoTimeCount);
			Console.Log( "Loop Time: %.2fmsec [%i samples]", "performance.log", (R32)((R32)loopTime/(R32)loopTimeCount), loopTimeCount);
			Console.Log( "Characters: %i/%i - Items: %i/%i (Dynamic)", "performance.log", chars.Count(), cwmWorldState->GetCMem(), items.Count(), cwmWorldState->GetIMem());
			Console.Log( "Simulation Cycles: %f per sec", "performance.log", (1000.0*(1.0/(R32)((R32)loopTime/(R32)loopTimeCount))));
			Console.Log( "Bytes sent: %i", "performance.log", globalSent);
			Console.Log( "Bytes Received: %i", "performance.log", globalRecv);
			Console.Log( "--- Performance Dump Complete ---", "performance.log");
			Console.LogEcho( false );
			break;
		case 'W':                
			// Display logged in chars
			messageLoop << "CMD: Current Users in the World:";
			j = 0;
			cSocket *iSock;
			Network->PushConn();
			for( iSock = Network->FirstSocket(); !Network->FinishedSockets(); iSock = Network->NextSocket() )
			{
				++j;
				CChar *mChar = iSock->CurrcharObj();
				sprintf( temp, "     %i) %s [%x %x %x %x]", j - 1, mChar->GetName().c_str(), mChar->GetSerial( 1 ), mChar->GetSerial( 2 ), mChar->GetSerial( 3 ), mChar->GetSerial( 4 ) );
				messageLoop << temp;
			}
			Network->PopConn();
			sprintf( temp, "     Total users online: %i", j );
			messageLoop << temp;
			break;
		case 'M':
			size_t tmp, total;
			total = 0;
			tmp = 0;
			messageLoop << "CMD: UOX Memory Information:";
			messageLoop << "     Cache:";
			sprintf( temp, "        Tiles: %i bytes", Map->TileMem );
			messageLoop << temp;
			sprintf( temp, "        Statics: %i bytes", Map->StaMem );
			messageLoop << temp;
			sprintf( temp, "        Version: %i bytes", Map->versionMemory );
			messageLoop << temp;
			sprintf( temp, "        Map0: %i bytes [%i Hits - %i Misses]", 9*MAP0CACHE, Map->Map0CacheHit, Map->Map0CacheMiss );
			messageLoop << temp;
			total += tmp = chars.Size() + cwmWorldState->GetCMem()*sizeof( teffect_st ) + cwmWorldState->GetCMem()*sizeof(char) + cwmWorldState->GetCMem()*sizeof(int)*5;
			sprintf( temp, "     Characters: %u bytes [%u chars ( %u allocated )]", tmp, chars.Count(), cwmWorldState->GetCMem() );
			messageLoop << temp;
			total += tmp = items.Size() + cwmWorldState->GetIMem()*sizeof(int)*4;
			sprintf( temp, "     Items: %u bytes [%u items ( %u allocated )]", tmp, items.Count(), cwmWorldState->GetIMem() );
			messageLoop << temp;
			sprintf( temp, "        You save I: %i & C: %i bytes!", cwmWorldState->GetIMem() * sizeof(CItem) - items.Size(), cwmWorldState->GetCMem() * sizeof( CChar ) - chars.Size() );
			total += tmp = 69 * sizeof( SpellInfo );
			sprintf( temp, "     Spells: %i bytes", tmp );
			messageLoop << "     Sizes:";
			sprintf( temp, "        CItem  : %i bytes", sizeof( CItem ) );
			messageLoop << temp;
			sprintf( temp, "        CChar  : %i bytes", sizeof( CChar ) );
			messageLoop << temp;
			sprintf( temp, "        TEffect: %i bytes (%i total)", sizeof( teffect_st ), sizeof( teffect_st ) * TEffects->Count() );
			messageLoop << temp;
			total += tmp = Map->TileMem + Map->StaMem + Map->versionMemory;
			sprintf( temp, "        Approximate Total: %i bytes", total );
			messageLoop << temp;
			break;
		case 'e':
		case 'E':
			// Toggle Layer errors
			j = 0;
			UI08 iLayer;
			for( iLayer = 0; iLayer < MAXLAYERS; ++iLayer )
			{
				if( cwmWorldState->GetErroredLayer( iLayer ) != 0 )
				{
					++j;
					if( iLayer < 10 )
						sprintf( temp, "| ERROR: Layer 0%i -> %i errors", iLayer, cwmWorldState->GetErroredLayer( iLayer ) );
					else
						sprintf( temp, "| ERROR: Layer %i -> %i errors", iLayer, cwmWorldState->GetErroredLayer( iLayer ) );
					messageLoop << temp;
				}
			}
			sprintf( temp, "| ERROR: Found errors on %u layers.", j );
			messageLoop << temp;
			break;
		case '?':
			messageLoop << MSG_SECTIONBEGIN;
			messageLoop << "Console commands:";
			messageLoop << MSG_SECTIONBEGIN;
			messageLoop << " ShardOP:";
			messageLoop << "    * - Lock/Unlock Console ? - Commands list(this)";
			messageLoop << "    C - Configuration       H - Heart Beat";
			messageLoop << "    Y - Console Broadcast   Q - Quit/Exit           ";
			messageLoop << " Load Commands:";
			messageLoop << "    1 - Ini                 2 - Accounts";
			messageLoop << "    3 - Regions             4 - Spawn Regions";
			messageLoop << "    5 - Spells              6 - Commands";
			messageLoop << "    7 - Dfn's               8 - JavaScript";
			messageLoop << "    9 - HTML Templates      0 - ALL(1-9)";
			messageLoop << " Save Commands:";
			messageLoop << "    ! - Accounts            @ - World(w/AccountImport)";
			messageLoop << "    # - Unused              $ - Unused";
			messageLoop << "    % - Unused              ^ - Unused";
			messageLoop << "    & - Unused              ( - Unused";
			messageLoop << "    ) - Unused";
			messageLoop << " Server Maintenence:";
			messageLoop << "    P - Performance         W - Characters Online";
			messageLoop << "    M - Memory Information  T - 10 Minute Shutdown";
			messageLoop << "    V - Dump Lookups(Devs)  N - Dump Npc.Dat";
			messageLoop << "    L - Layer Information"; 
			messageLoop << " Network Maintenence:";
			messageLoop << "    D - Disconnect Acct0    K - Disconnect All";
			messageLoop << "    Z - Socket Logging      ";
			messageLoop << MSG_SECTIONBEGIN;
			break;
		case 'v':
		case 'V':
			// Dump look up data to files so developers working with extending the ini will have a table to use
			messageLoop << "| CMD: Creating Server.scp and Uox3.ini Tag Lookup files(For Developers)....";
			cwmWorldState->ServerData()->dumpLookup( 0 );
			cwmWorldState->ServerData()->save( "./uox.tst.ini" );
			messageLoop << MSG_PRINTDONE;
			break;
		case 'z':
		case 'Z':
		{
			// Log socket activity
			Network->PushConn();
			bool loggingEnabled = false;
			cSocket *snSock = Network->FirstSocket();
			if( snSock != NULL )
				loggingEnabled = !snSock->Logging();
			for( ; !Network->FinishedSockets(); snSock = Network->NextSocket() )
			{
				if( snSock != NULL )
					snSock->Logging( !snSock->Logging() );
			}
			Network->PopConn();
			if( loggingEnabled )
				messageLoop << "CMD: Network Logging Enabled.";
			else
				messageLoop << "CMD: Network Logging Disabled.";
			break;
		}
		case 'N':
			//. Dump a file that contains the id and sound some toher misc data about monster NPC.
			DumpCreatures();
			break;
		case 'c':
		case 'C':
			// Shows a configuration header
			DisplaySettings();
			break;
		default:
			sprintf( temp, "Key \'%c\' [%i] does not perform a function", (char)c, c );
			messageLoop << temp;
			break;
		}
	}
}

//o----------------------------------------------------------------------------o
//|   Function -	 void checkkey( void )
//|   Date     -	 Unknown
//|   Programmer  -  Unknown  (Touched up by EviLDeD)
//o----------------------------------------------------------------------------o
//|   Purpose     -  Facilitate console control. SysOp keys, and localhost 
//|					 controls.
//o----------------------------------------------------------------------------o
void checkkey( void )
{
	int c = cl_getch();
	if( c > 0 )
	{
		if( (cluox_io) && ( c == 250 ) )
		{  // knox force unsecure mode, need this since cluox can't know
			//      how the toggle status is.
			if( cwmWorldState->GetSecure() )
			{
				Console << "Secure mode disabled. Press ? for a commands list." << myendl;
				cwmWorldState->SetSecure( false );
				return;
			}
		}
		c = toupper(c);
		processkey( c );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool genericCheck( CChar *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check characters status.  Returns true if character was killed
//o---------------------------------------------------------------------------o
bool genericCheck( CChar *i )
{
	if( i->IsDead() )
		return false;

	UI16 c;
	bool strUpdate = false;
	bool intUpdate = false;
	bool dexUpdate = false;

	bool isOn = isOnline( i ), isNPC = i->IsNpc();

	cSocket *s = NULL;
	if( !isNPC )
		s = calcSocketObjFromChar( i );

	if( i->GetHP() > i->GetMaxHP() )
	{
		i->SetHP( i->GetMaxHP() );
		strUpdate = true;
	}
	if( i->GetStamina() > i->GetMaxStam() )
	{
		i->SetStamina( i->GetMaxStam() );
		dexUpdate = true;
	}
	if( i->GetMana() > i->GetMaxMana() )
	{
		i->SetMana( i->GetMaxMana() );
		intUpdate = true;
	}

	if( i->GetRegen( 0 ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
	{
		if( i->GetHP() < i->GetMaxHP() && ( i->GetHunger() > (SI16)cwmWorldState->ServerData()->HungerThreshold() || cwmWorldState->ServerData()->HungerRate() == 0 ) )
		{
			for( c = 0; c < i->GetMaxHP() + 1; ++c )
			{
				if( i->GetHP() <= i->GetMaxHP() && ( i->GetRegen( 0 ) + ( c * cwmWorldState->ServerData()->SystemTimer( HITPOINT_REGEN ) * 1000) ) <= cwmWorldState->GetUICurrentTime() )
				{
					if( i->GetSkill( HEALING ) < 500 )
						i->IncHP( 1 );
					else if( i->GetSkill( HEALING ) < 800 )
						i->IncHP( 2 );
					else 
						i->IncHP( 3 );
					if( i->GetHP() > i->GetMaxHP() )
					{
						i->SetHP( i->GetMaxHP() ); 
						break;
					}
					strUpdate = true;
				}
				else			// either we're all healed up, or all time periods have passed
					break;
			}
		}
		i->SetRegen( BuildTimeValue( (R32)cwmWorldState->ServerData()->SystemTimer( HITPOINT_REGEN ) ), 0 );
	}
	if( i->GetRegen( 1 ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
	{
		if( i->GetStamina() < i->GetMaxStam() )
		{
			for( c = 0; c < i->GetMaxStam() + 1; ++c )
			{
				if( ( i->GetRegen( 1 ) + ( c * cwmWorldState->ServerData()->SystemTimer( STAMINA_REGEN ) * 1000) ) <= cwmWorldState->GetUICurrentTime() && i->GetStamina() <= i->GetMaxStam() )
				{
					i->SetStamina( (SI16)( i->GetStamina() + 1 ) );
					if( i->GetStamina() > i->GetMaxStam() )
					{
						i->SetStamina( i->GetMaxStam() );
						break;
					}
					dexUpdate = true;
				}
				else
					break;
			}
		}
		i->SetRegen( BuildTimeValue( (R32)cwmWorldState->ServerData()->SystemTimer( STAMINA_REGEN ) ), 1 );
	}

	// CUSTOM START - SPUD:MANA REGENERATION:Rewrite of passive and active meditation code
	if( i->GetRegen( 2 ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
	{
		if( i->GetMana() < i->GetMaxMana() )
		{
			for( c = 0; c < i->GetMaxMana() + 1; ++c )
			{
				if( i->GetRegen( 2 ) + ( c * cwmWorldState->ServerData()->SystemTimer( MANA_REGEN ) * 1000) <= cwmWorldState->GetUICurrentTime() && i->GetMana() <= i->GetMaxMana() )
				{
					Skills->CheckSkill( i, MEDITATION, 0, 1000 );	// Check Meditation for skill gain ala OSI
					i->SetMana( (SI16)( i->GetMana() + 1 ) );	// Gain a mana point
					if( i->GetMana() > i->GetMaxMana() )
					{
						if( i->GetMed() ) // Morrolan = Meditation
						{
							if( s != NULL )
								s->sysmessage( 969 );
							i->SetMed( false );
						}
						i->SetMana( i->GetMaxMana() ); 
						break;
					}
					intUpdate = true;
				}
			}
		}
		R32 MeditationBonus = ( .00075f * i->GetSkill( MEDITATION ) );	// Bonus for Meditation
		int NextManaRegen = static_cast<int>(cwmWorldState->ServerData()->SystemTimer( MANA_REGEN ) * ( 1 - MeditationBonus ) * 1000);
		if( cwmWorldState->ServerData()->SystemTimer( ARMORAFFECTMANA_REGEN ) )	// If armor effects mana regeneration...
		{
			R32 ArmorPenalty = Combat->calcDef( i, 0, false );	// Penalty taken due to high def
			if( ArmorPenalty > 100 )	// For def higher then 100, penalty is the same...just in case
				ArmorPenalty = 100;
			ArmorPenalty = 1 + (ArmorPenalty / 25);
			NextManaRegen = static_cast<int>(NextManaRegen * ArmorPenalty);
		}
		if( i->GetMed() )	// If player is meditation...
			i->SetRegen( ( cwmWorldState->GetUICurrentTime() + ( NextManaRegen / 2 ) ), 2 );
		else
			i->SetRegen( ( cwmWorldState->GetUICurrentTime() + NextManaRegen ), 2 );
	}
	// CUSTOM END
	if( i->GetHidden() == 2 && ( i->GetTimer( tCHAR_INVIS ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) && !i->IsPermHidden() )
		i->ExposeToView();
	if( isOn || isNPC )
	{
		if( i->GetHungerStatus() && cwmWorldState->ServerData()->HungerRate() > 1 && ( i->GetTimer( tCHAR_HUNGER ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) )
		{
			if( i->GetHunger() > 0 && ( !i->IsCounselor() && !i->IsGM() ) )
				i->DecHunger(); //Morrolan GMs and Counselors don't get hungry
			
			UI16 HungerTrig = i->GetScriptTrigger();
			cScript *toExecute = Trigger->GetScript( HungerTrig );
			bool doHunger = true;
			if( toExecute != NULL )
				doHunger = !toExecute->OnHungerChange( i, i->GetHunger() );
			if( doHunger && isOn )
			{
				switch( i->GetHunger() )
				{
					case 6: break;
					case 5:	s->sysmessage( 1222 );	break;
					case 4:	s->sysmessage( 1223 );	break;
					case 3:	s->sysmessage( 1224 );	break;
					case 2:	s->sysmessage( 1225 );	break;
					case 1:	s->sysmessage( 1226 );	break;
					case 0:
						if( !i->IsCounselor() && !i->IsGM() )
							s->sysmessage( 1227 );
						break;	
				}
			}
			i->SetTimer( tCHAR_HUNGER, BuildTimeValue( (R32)cwmWorldState->ServerData()->HungerRate() ) );
		}
		if( cwmWorldState->ServerData()->HungerDamage() > 0 && ( cwmWorldState->GetHungerDamageTimer() <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) ) // Damage them if they are very hungry
		{
			cwmWorldState->SetHungerDamageTimer( BuildTimeValue( (R32)cwmWorldState->ServerData()->HungerDamageRateTimer() ) ); // set new hungertime
			if( i->GetHungerStatus() && i->GetHP() > 0 && i->GetHunger() < 2 && ( !i->IsCounselor() && !i->IsGM() ) && !i->IsDead() )
			{   
				if( isOn )
					s->sysmessage( 1228 );
				i->IncHP( (SI16)( -cwmWorldState->ServerData()->HungerDamage() ) );
				updateStats( i, 0 );
				if( i->GetHP() <= 0 )
				{ 
					if( isOn )
						s->sysmessage( 1229 );
					doDeathStuff( i );
				}
			}
		}
	}
	if( !i->IsInvulnerable() && i->GetPoisoned() && ( isNPC || isOn ) )
	{
		if( i->GetTimer( tCHAR_POISONTIME ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
		{
			if( i->GetTimer( tCHAR_POISONWEAROFF ) > cwmWorldState->GetUICurrentTime() )
			{
				SI16 pcalc = 0;
				switch( i->GetPoisoned() )
				{
					case 1:
						i->SetTimer( tCHAR_POISONTIME, BuildTimeValue( 5 ) );
						if( i->GetTimer( tCHAR_POISONTEXT ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
						{
							i->SetTimer( tCHAR_POISONTEXT, BuildTimeValue( 10 ) );
							i->emoteAll( 1240, true, i->GetName().c_str() );
						}
						i->IncHP( (SI16)( -RandomNum( 1, 2 ) ) );
						strUpdate = true;
						break;
					case 2:
						i->SetTimer( tCHAR_POISONTIME, BuildTimeValue( 4 ) );
						if( i->GetTimer( tCHAR_POISONTEXT ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
						{
							i->SetTimer( tCHAR_POISONTEXT, BuildTimeValue( 10 ) );
							i->emoteAll( 1241, true, i->GetName().c_str() );
						}
						pcalc = (SI16)( ( i->GetHP() * RandomNum( 2, 5 ) / 100 ) + RandomNum( 0, 2 ) ); // damage: 1..2..5% of hp's+ 1..2 constant
						i->IncHP( (SI16)( -pcalc ) );
						strUpdate = true;
						break;
					case 3:
						i->SetTimer( tCHAR_POISONTIME, BuildTimeValue( 3 ) );
						if( i->GetTimer( tCHAR_POISONTEXT ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
						{
							i->SetTimer( tCHAR_POISONTEXT, BuildTimeValue( 10 ) );
							i->emoteAll( 1242, true, i->GetName().c_str() );
						}
						pcalc = (SI16)( ( i->GetHP() * RandomNum( 5, 10 ) / 100 ) + RandomNum( 1, 3 ) ); // damage: 5..10% of hp's+ 1..2 constant
						i->IncHP( (SI16)( -pcalc ) );
						strUpdate = true;
						break;
					case 4:
						i->SetTimer( tCHAR_POISONTIME, BuildTimeValue( 3 ) );
						if( i->GetTimer( tCHAR_POISONTEXT ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
						{
							i->SetTimer( tCHAR_POISONTEXT, BuildTimeValue( 10 ) );
							i->emoteAll( 1243, true, i->GetName().c_str() );
						}
						pcalc = (SI16)( i->GetHP() / 5 + RandomNum( 3, 6 ) ); // damage: 20% of hp's+ 3..6 constant, quite deadly <g>
						i->IncHP( (SI16)( -pcalc ) );
						strUpdate = true;
						break;
					default:
						Console.Error( 2, " Fallout of switch statement without default. uox3.cpp, genericCheck()" );
						i->SetPoisoned( 0 );
						break;
				}
				if( i->GetHP() < 1 )
				{
					doDeathStuff( i );
					if( isOn )
						s->sysmessage( 1244 );
				} 
			}
		}
	}

	if( i->GetTimer( tCHAR_POISONWEAROFF ) <= cwmWorldState->GetUICurrentTime() )
	{
		if( i->GetPoisoned() )
		{
            i->SetPoisoned( 0 );
			i->SendToSocket( s, true, i );
			if( isOn )
				s->sysmessage( 1245 );
		}
	}

	bool part1 = doLightEffect( i );
	bool part2 = doRainEffect( i );
	bool part3 = doSnowEffect( i );
	bool part4 = doHeatEffect( i );
	bool part5 = doColdEffect( i );

	if( strUpdate || part1 || part2 || part3 || strUpdate || part4 || part5 )
		updateStats( i, 0 );

	if( dexUpdate || part4 )
		updateStats( i, 2 );

	if( intUpdate )
		updateStats( i, 1 );
	
	if( i->GetHP() <= 0 && !i->IsDead() )
	{
		doDeathStuff( i );
		return true;
	}
	return false;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void checkPC( CChar *i, bool doWeather )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check a PC's status
//o---------------------------------------------------------------------------o
void checkPC( CChar *i, bool doWeather )
{
	LIGHTLEVEL toShow;
	cSocket *mSock = calcSocketObjFromChar( i );
	
	Magic->CheckFieldEffects2( i, 1 );
	
	bool isOn = isOnline( i );

	if( !i->IsDead() )
		Combat->CombatLoop( i );	
	
	if( doWeather )
	{
		UI08 curLevel = cwmWorldState->ServerData()->WorldLightCurrentLevel();
		if( Races->VisLevel( i->GetRace() ) > curLevel )
			toShow = 0;
		else
			toShow = static_cast<UI08>( curLevel - Races->VisLevel( i->GetRace() ) );

		doLight( mSock, toShow );
		Weather->DoPlayerStuff( i );
	}
	
	if( isOn && i->GetSquelched() == 2 )
	{
		if( i->GetTimer( tPC_MUTETIME ) != 0 && ( i->GetTimer( tPC_MUTETIME ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) )
		{
			i->SetSquelched( 0 );
			i->SetTimer( tPC_MUTETIME, 0 );
			if( mSock != NULL )
				mSock->sysmessage( 1237 );
		}
	}
	
	if( isOn )
	{
		if( i->IsCriminal() && i->GetTimer( tCHAR_CRIMFLAG ) != 0 && ( i->GetTimer( tCHAR_CRIMFLAG ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) )
		{
			if( mSock != NULL )
				mSock->sysmessage( 1238 );
			i->SetTimer( tCHAR_CRIMFLAG , 0 );
			setcharflag( i );
		}
		if( i->GetTimer( tCHAR_MURDERRATE ) > 0 )
			i->SetTimer( tCHAR_MURDERRATE, i->GetTimer( tCHAR_MURDERRATE ) - 1 );
		
		if( i->GetTimer( tCHAR_MURDERRATE ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
		{
			if( i->GetKills() > 0 )
				i->SetKills( (SI16)( i->GetKills() - 1 ) );
			if( i->GetKills() == cwmWorldState->ServerData()->RepMaxKills() && mSock != NULL )
				mSock->sysmessage( 1239 );
			i->SetTimer( tCHAR_MURDERRATE, BuildTimeValue( (R32)cwmWorldState->ServerData()->RepMurderDecay() ) );//Murder rate (in mins) to seconds. (checkauto is done about 25 times per second)
		}
		setcharflag( i );
	}
	
	if( i->GetCasting() )	// Casting a spell
	{
		i->SetNextAct( i->GetNextAct() - 1 );
		if( i->GetTimer( tCHAR_SPELLTIME ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )//Spell is complete target it.
		{
			if( spells[i->GetSpellCast()].RequireTarget() )
			{
				i->SetCasting( false );
				i->SetFrozen( false );
				mSock->target( 0, TARGET_CASTSPELL, spells[i->GetSpellCast()].StringToSay() );
			}
			else
			{
				i->SetCasting( false );
				Magic->CastSpell( mSock, i );
				i->SetTimer( tCHAR_SPELLTIME, 0 );
				i->SetFrozen( false );
			}
		} 
		else if( i->GetNextAct() <= 0 )//redo the spell action
		{
			i->SetNextAct( 75 );
			if( !i->IsOnHorse() )
				Effects->impaction( mSock, i->GetSpellAction() );
		}
	}
	
	if( cwmWorldState->ServerData()->WorldAmbientSounds() >= 1 )
	{
		if( cwmWorldState->ServerData()->WorldAmbientSounds() > 10 )
			cwmWorldState->ServerData()->WorldAmbientSounds( 10 );
		SI16 soundTimer = static_cast<SI16>(cwmWorldState->ServerData()->WorldAmbientSounds() * 100);
		if( isOn && !i->IsDead() && ( rand()%( soundTimer ) ) == ( soundTimer / 2 ) )
			Effects->bgsound( i ); // bgsound uses array positions not sockets!
	}
	
	if( i->GetTimer( tPC_SPIRITSPEAK ) > 0 && i->GetTimer( tPC_SPIRITSPEAK) < cwmWorldState->GetUICurrentTime() )
		i->SetTimer( tPC_SPIRITSPEAK, 0 );
	
	if( i->GetTimer( tPC_TRACKING ) > cwmWorldState->GetUICurrentTime() && isOn )
	{
		if( i->GetTimer( tPC_TRACKINGDISPLAY ) <= cwmWorldState->GetUICurrentTime() )
		{
			i->SetTimer( tPC_TRACKINGDISPLAY, BuildTimeValue( (R32)cwmWorldState->ServerData()->TrackingRedisplayTime() ) );
			Skills->Track( i );
		}
	}
	else
	{
		if( i->GetTimer( tPC_TRACKING ) > ( cwmWorldState->GetUICurrentTime() / 10 ) ) // dont send arrow-away packet all the time
		{
			i->SetTimer( tPC_TRACKING, 0 );
			CPTrackingArrow tSend = chars[i->GetTrackingTarget()];
			tSend.Active( 0 );
			mSock->Send( &tSend );
		}
	}
	
	if( i->GetTimer( tPC_FISHING ) )
	{
		if( i->GetTimer( tPC_FISHING ) <= cwmWorldState->GetUICurrentTime() )
		{
			Skills->Fish( i );
			i->SetTimer( tPC_FISHING, 0 );
		}
	}
	
	if( i->IsOnHorse() )
	{
		CItem *horseItem = i->GetItemAtLayer( 0x19 );
		if( horseItem == NULL )
			i->SetOnHorse( false );	// turn it off, we aren't on one because there's no item!
		else if( horseItem->GetDecayTime() != 0 && ( horseItem->GetDecayTime() <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) )
		{
			i->SetOnHorse( false );
			Items->DeleItem( horseItem );
		}
	}
	
}

//o---------------------------------------------------------------------------o
//|	Function	-	void checkNPC( CChar *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check NPC's status
//o---------------------------------------------------------------------------o
void checkNPC( CChar *i )
{
	if( i == NULL || i->isFree() )
		return;
	// okay, this will only ever trigger after we check an npc...  Question is:
	// should we remove the time delay on the AI check as well?  Just stick with AI/movement
	// AI can never be faster than how often we check npcs
	// This periodically generates access violations.  No idea why either
	UI16 AITrig			= i->GetScriptTrigger();
	cScript *toExecute	= Trigger->GetScript( AITrig );
	bool doAICheck		= true;
	if( toExecute != NULL )
	{
		if( toExecute->OnAISliver( i ) )
			doAICheck = false;
	}
	if( doAICheck )
		Npcs->CheckAI( i );
	Movement->NpcMovement( i );
	setcharflag( i );		// possibly not... How many times do we want to set this? We've set it twice in the calling function

	if( cwmWorldState->GetShopRestockTime() <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
		restockNPC( i, false );

	if( i->GetOwnerObj() != NULL && i->GetHunger() == 0 && i->GetNPCAiType() != aiPLAYERVENDOR ) // tamed animals but not player vendors ;)=
	{
		Effects->tempeffect( i, i, 44, 0, 0, 0 ); // (go wild in some minutes ...)-effect
		i->SetHunger( -1 );
	}
	
	if( i->GetTimer( tNPC_SUMMONTIME ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
	{
		if( i->GetTimer( tNPC_SUMMONTIME ) > 0 )
		{
			// Dupois - Added Dec 20, 1999
			// QUEST expire check - after an Escort quest is created a timer is set
			// so that the NPC will be deleted and removed from the game if it hangs around
			// too long without every having its quest accepted by a player so we have to remove 
			// its posting from the messageboard before icing the NPC
			// Only need to remove the post if the NPC does not have a follow target set
			if( i->GetQuestType() == ESCORTQUEST && i->GetFTarg() == INVALIDSERIAL )
			{
				MsgBoardQuestEscortRemovePost( i );
				MsgBoardQuestEscortDelete( i );
				return;
			}
			// Dupois - End
			if( i->GetNPCAiType() == aiGUARD && i->IsAtWar() )
			{
				i->SetTimer( tNPC_SUMMONTIME, BuildTimeValue( 25 ) );
				return;
			}
			Effects->PlaySound( i, 0x01FE );
			i->SetDead( true );
			Npcs->DeleteChar( i );
			return;
		}
	}
	
	if( i->GetFleeAt() == 0 )
		i->SetFleeAt( cwmWorldState->ServerData()->CombatNPCBaseFleeAt() );
	if( i->GetReattackAt() == 0 )
		i->SetReattackAt( cwmWorldState->ServerData()->CombatNPCBaseReattackAt() );
	
	if( i->GetNpcWander() != 5 && ( i->GetHP() < i->GetStrength() * i->GetFleeAt() / 100 ) )
	{
		i->SetOldNpcWander( i->GetNpcWander() );
		i->SetNpcWander( 5 );
		i->SetTimer( tNPC_MOVETIME, BuildTimeValue( (R32)( cwmWorldState->ServerData()->NPCSpeed() / 2 ) ) );	// fleeing enemies are 2x faster
	}
	
	if( i->GetNpcWander() == 5 && (i->GetHP() > i->GetStrength() * i->GetReattackAt() / 100))
	{
		i->SetNpcWander( i->GetOldNpcWander() );
		i->SetTimer( tNPC_MOVETIME, BuildTimeValue( (R32)cwmWorldState->ServerData()->NPCSpeed() ) );
		i->SetOldNpcWander( 0 ); // so it won't save this at the wsc file
	}

	Magic->CheckFieldEffects2( i, 0 );

	try
	{
		if( !i->IsDead() )
			Combat->CombatLoop( i );
	}
	catch( ... )
	{
		Console << "EXCEPTION: checkNPC() - NPC or object no longer exists. Skipping combat loop this cycle." << myendl;
	}
}

void checkItem( SubRegion *toCheck, UI32 checkitemstime )
{
	// Exception Spinner. This is cheap, but should allow for some sureness that the loop is finished
	UI16 nSpinner=0;
	// Handle the STL correctly
	ITEMLIST_ITERATOR I;
	//for( CItem *itemCheck = toCheck->FirstItem(); !toCheck->FinishedItems(); itemCheck = toCheck->GetNextItem() )
	for( I=toCheck->itemData.begin(); I!=toCheck->itemData.end(); ++I )
	{
		CItem * itemCheck;
		itemCheck=(*I);
		if( itemCheck == NULL || I == toCheck->itemData.end())
			continue;
		try
		{
			if( checkitemstime <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
			{
				Items->RespawnItem( itemCheck );
				if( itemCheck->GetType() == 51 || itemCheck->GetType() == 52 )
				{
					if( itemCheck->GetGateTime() <= cwmWorldState->GetUICurrentTime() )
					{
						Items->DeleItem( itemCheck );
						continue;
					}
				}
				if( Items->DecayItem( itemCheck ) )
					continue;

				if( itemCheck->GetType() == 88 && itemCheck->GetMoreY() < 25 )
				{
					if( (UI32)RandomNum( 1, 100 ) <= itemCheck->GetMoreZ() )
					{
						Network->PushConn();
						for( cSocket *eSock = Network->FirstSocket(); !Network->FinishedSockets(); eSock = Network->NextSocket() )
						{
							if( objInRange( eSock, itemCheck, (UI16)itemCheck->GetMoreY() ) )
									Effects->PlaySound( eSock, (UI16)itemCheck->GetMoreX(), false );
						}
						Network->PopConn();
					}
				}
			} 
			if( itemCheck->GetType() == 117 && ( itemCheck->GetType2() == 1 || itemCheck->GetType2() == 2 ) && 
				( itemCheck->GetGateTime() <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) )
			{
				Network->PushConn();
				for( cSocket *bSock = Network->FirstSocket(); !Network->FinishedSockets(); bSock = Network->NextSocket() )
				{
					if( objInRange( bSock, itemCheck, DIST_SAMESCREEN ) )
					{
						if( itemCheck->GetType2() == 1 )
							Boats->MoveBoat( bSock, itemCheck->GetDir(), itemCheck );
						else 
						{
							UI08 dir = (UI08)( itemCheck->GetDir() + 4 );
							if( dir > 7 )
								dir -= 8;
							Boats->MoveBoat( bSock, dir, itemCheck );
						}
					}
				}
				Network->PopConn();
				itemCheck->SetGateTime( BuildTimeValue( (R32)cwmWorldState->ServerData()->CheckBoatSpeed() ) );
			}
		}
		catch( ... )
		{
			Console << "EXCEPTION: checkItem() - Invalid object is ignored.[Spinner:" << nSpinner << "]" << myendl;
			++nSpinner;
			if( nSpinner >= 50 )
				return;
			continue;
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void checkauto( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check automatic and timer controlled functions
//o---------------------------------------------------------------------------o
void checkauto( void )
{
	static UI32 checkspawnregions=0; 
	static UI32 checkitemstime=0;
	static UI32 uiSetFlagTime = 0;
	static UI32 accountFlush = 0;
	static UI32 regionCheck = 0;
	static UI32 counter = 0;
	bool doWeather = false;
	ACCOUNTSBLOCK actbTemp;
	MAPUSERNAMEID_ITERATOR I;
	//
	Trigger->PeriodicTriggerCheck();
	// modify this stuff to take into account more variables
	if( accountFlush <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
	{
		bool reallyOn = false;
		// time to flush our account status!
		//for( ourAccount = Accounts->FirstAccount(); !Accounts->FinishedAccounts(); ourAccount = Accounts->NextAccount() )
		for(I=Accounts->begin();I!=Accounts->end();++I)
		{
			ACCOUNTSBLOCK actbBlock;
			actbBlock=I->second;
			if( actbBlock.wAccountIndex==AB_INVALID_ID)
				continue;

			if( actbTemp.wFlags&AB_FLAGS_ONLINE )
			{
				reallyOn = false;	// to start with, there's no one really on
				Network->PushConn();
				for( cSocket *tSock = Network->FirstSocket(); !Network->FinishedSockets(); tSock = Network->NextSocket() )
				{
					CChar *tChar = tSock->CurrcharObj();
					if( tChar == NULL )
						continue;
					if( tChar->GetAccount().wAccountIndex == actbTemp.wAccountIndex)
						reallyOn = true;
				}
				Network->PopConn();
				if( !reallyOn )	// no one's really on, let's set that
					actbTemp.wFlags&=0xFFF7;
			}
		}
		accountFlush = BuildTimeValue( (R32)cwmWorldState->ServerData()->AccountFlushTimer() * 60 );
	}
	Network->On();

	if( cwmWorldState->GetWorldSaveProgress() == 0 )
	{
		size_t totalPlayers = cwmWorldState->GetPlayersOnline();
		for( size_t ij = totalPlayers - 1; ij >= 0 && ij < totalPlayers; --ij )
		{
			cSocket *tSock = calcSocketObjFromSock( ij );
			if( tSock->IdleTimeout() != -1 && (UI32)tSock->IdleTimeout() <= cwmWorldState->GetUICurrentTime() )
			{
				CChar *tChar = tSock->CurrcharObj();
				if( tChar == NULL )
					continue;
				if( !tChar->IsGM() )
				{
					tSock->IdleTimeout( -1 );
					tSock->sysmessage( 1246 );
					Network->Disconnect( ij );
				}
			}
			else if( ( ( (UI32)( tSock->IdleTimeout() + 300 * 1000 ) <= cwmWorldState->GetUICurrentTime() && (UI32)( tSock->IdleTimeout()+200*1000) >= cwmWorldState->GetUICurrentTime() ) || cwmWorldState->GetOverflow() ) && !tSock->WasIdleWarned()  )
			{//is their idle time between 3 and 5 minutes, and they haven't been warned already?
				CPIdleWarning warn( 0x07 );
				tSock->Send( &warn );
				tSock->WasIdleWarned( true );
			}
		}
	}
	else if( cwmWorldState->GetWorldSaveProgress() == 2 )	// if we've JUST saved, do NOT kick anyone off (due to a possibly really long save), but reset any offending players to 60 seconds to go before being kicked off
	{
		Network->PushConn();
		for( cSocket *wsSocket = Network->FirstSocket(); !Network->FinishedSockets(); wsSocket = Network->NextSocket() )
		{
			if( wsSocket != NULL )
			{
				if( (UI32)wsSocket->IdleTimeout() < cwmWorldState->GetUICurrentTime() )
				{
					wsSocket->IdleTimeout( BuildTimeValue( 60.0f ) );
					wsSocket->WasIdleWarned( true );//don't give them the message if they only have 60s
				}
			}
		}
		Network->PopConn();
		cwmWorldState->SetWorldSaveProgress( 0 );
	}
	Network->Off();
	if( regionCheck <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
	{
		for( UI16 regionCounter = 0; regionCounter < 256; ++regionCounter )
			region[regionCounter]->PeriodicCheck();
		regionCheck = BuildTimeValue( 10 );	// do checks every 10 seconds or so, rather than every single time
		JailSys->PeriodicCheck();
	}

	if( checkspawnregions <= cwmWorldState->GetUICurrentTime() && cwmWorldState->ServerData()->CheckSpawnRegionSpeed() != -1 )//Regionspawns
	{
		for( UI16 i = 1; i < cwmWorldState->GetTotalSpawnRegions(); ++i )
		{
			cSpawnRegion *spawnReg = spawnregion[i];
			if( spawnReg != NULL && (UI32)spawnReg->GetNextTime() <= cwmWorldState->GetUICurrentTime() )
			{
				spawnReg->doRegionSpawn();
				spawnReg->SetNextTime( BuildTimeValue( (R32)( 60 * RandomNum( spawnReg->GetMinTime(), spawnReg->GetMaxTime() ) ) ) );
			}
		}
		checkspawnregions = BuildTimeValue( (R32)cwmWorldState->ServerData()->CheckSpawnRegionSpeed() );//Don't check them TOO often (Keep down the lag)
	}
	
	HTMLTemplates->Poll( ETT_ALLTEMPLATES );

	UI32 saveinterval = cwmWorldState->ServerData()->ServerSavesTimerStatus();
	if( saveinterval != 0 )
	{
		UI32 oldTime = cwmWorldState->GetOldTime();
		if( !cwmWorldState->GetAutoSaved() )
		{
			cwmWorldState->SetAutoSaved( true );
			time((time_t *)&oldTime);
			cwmWorldState->SetOldTime( oldTime );
		}
		UI32 newTime = cwmWorldState->GetNewTime();
		time((time_t *)&newTime);
		cwmWorldState->SetNewTime( newTime );

		if( difftime( cwmWorldState->GetNewTime(), cwmWorldState->GetOldTime() ) >= saveinterval || cwmWorldState->Saving() )
		{
			// Dupois - Added Dec 20, 1999
			// After an automatic world save occurs, lets check to see if
			// anyone is online (clients connected).  If nobody is connected
			// Lets do some maintenance on the bulletin boards.
			if( !cwmWorldState->GetPlayersOnline() && !cwmWorldState->Saving() )
			{
				Console << "No players currently online. Starting bulletin board maintenance" << myendl;
				Console.Log( "Bulletin Board Maintenance routine running (AUTO)", "server.log" );
				MsgBoardMaintenance();
			}

			cwmWorldState->SetAutoSaved( false );
			cwmWorldState->savenewworld( false );
		}
	}
	
	//Time functions
	if( cwmWorldState->GetUOTickCount() <= cwmWorldState->GetUICurrentTime() || ( cwmWorldState->GetOverflow() ) )
	{
		if( cwmWorldState->ServerData()->incMinute() )
			Weather->NewDay();
		cwmWorldState->SetUOTickCount( BuildTimeValue( cwmWorldState->GetSecondsPerUOMinute() ) );
	}
	
	if( cwmWorldState->GetLightTime() <= cwmWorldState->GetUICurrentTime() || ( cwmWorldState->GetOverflow() ) )
	{
		counter = 0;		
		cwmWorldState->doWorldLight();  //Changes lighting, if it is currently time to.
		Weather->DoStuff();	// updates the weather types
		cwmWorldState->SetLightTime( BuildTimeValue( (R32)cwmWorldState->ServerData()->SystemTimer( WEATHER ) ) );	// for testing purposes
		doWeather = true;
	}
	else
		doWeather = false;

	std::set< SubRegion * > regionList;	// we'll get around our npc problem this way, hopefully
	Network->PushConn();
	for( cSocket *iSock = Network->FirstSocket(); !Network->FinishedSockets(); iSock = Network->NextSocket() )
	{
		if( iSock == NULL )
			continue;
		CChar *mChar = iSock->CurrcharObj();
		UI08 worldNumber = mChar->WorldNumber();
		if( isOnline( mChar ) && mChar->GetAccount().wAccountIndex == iSock->AcctNo() )
		{
			if( uiSetFlagTime <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
				setcharflag( mChar ); // only set flag on npcs every 60 seconds (save a little extra lag)
			genericCheck( mChar );
			checkPC( mChar, doWeather );

			SI16 xOffset = MapRegion->GetGridX( mChar->GetX() );
			SI16 yOffset = MapRegion->GetGridY( mChar->GetY() );
			for( SI08 counter = -1; counter <= 1; ++counter ) // Check 3 x colums
			{
				for( SI08 ctr2 = -1; ctr2 <= 1; ++ctr2 ) // Check 3 y colums
				{
					SubRegion *tC = MapRegion->GetGrid( xOffset + counter, yOffset + ctr2, worldNumber );
					if( tC == NULL )
						continue;
					regionList.insert( tC );
				}
			}
		}
	}
	Network->PopConn();
	std::set< SubRegion * >::iterator tcCheck = regionList.begin();
	while( tcCheck != regionList.end() )
	{
		SubRegion *toCheck = (*tcCheck);
		toCheck->PushChar();
		toCheck->PushItem();
		for( CChar *charCheck = toCheck->FirstChar(); !toCheck->FinishedChars(); charCheck = toCheck->GetNextChar() )
		{
			if( charCheck == NULL )//|| charCheck (int)charCheck == 0xCDCDCDCD )	// for some reason, 0xcdcdcdcd is always the bad thing here
				continue;
			if( charCheck->isFree() )
				continue;
			if( uiSetFlagTime <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() )
				setcharflag( charCheck ); // only set flag on npcs every 60 seconds (save a little extra lag)
			if( charCheck->IsNpc() )
			{
				bool delChar = genericCheck( charCheck );
				if( !delChar )
					checkNPC( charCheck );
			}
			else if( charCheck->GetTimer( tPC_LOGOUT ) != 0 )
			{
				actbTemp = charCheck->GetAccount();
				if( actbTemp.wAccountIndex != AB_INVALID_ID )
				{
					SERIAL oaiw = actbTemp.dwInGame;
					if( oaiw == INVALIDSERIAL )
					{
						charCheck->SetTimer( tPC_LOGOUT, 0 );
						charCheck->Update();
					}
					else if( oaiw == charCheck->GetSerial() && ( (UI32)charCheck->GetTimer( tPC_LOGOUT ) <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) )
					{
						actbTemp.dwInGame = INVALIDSERIAL;
						charCheck->SetTimer( tPC_LOGOUT, 0 );
						charCheck->Update();
					}
				}
			}
		}
		toCheck->PopChar();

		checkItem( toCheck, checkitemstime );

		toCheck->PopItem();
		++tcCheck;
	}

	Effects->checktempeffects();
	SpeechSys->Poll();
	if( uiSetFlagTime <= cwmWorldState->GetUICurrentTime() )
		uiSetFlagTime = BuildTimeValue( 30 ); // Slow down lag "needed" for setting flags, they are set often enough ;-)
	if( checkitemstime <= cwmWorldState->GetUICurrentTime() )
		checkitemstime = BuildTimeValue( (R32)cwmWorldState->ServerData()->CheckItemsSpeed() );
	if( cwmWorldState->GetShopRestockTime() <= cwmWorldState->GetUICurrentTime() )
		cwmWorldState->SetShopRestockTime( BuildTimeValue( (R32)( cwmWorldState->ServerData()->SystemTimer( SHOP_SPAWN ) ) ) );
	if( cwmWorldState->GetNextNPCAITime() <= cwmWorldState->GetUICurrentTime() )
		cwmWorldState->SetNextNPCAITime( BuildTimeValue( (R32)cwmWorldState->ServerData()->CheckNpcAISpeed() ) );
	if( cwmWorldState->GetNextFieldEffectTime() <= cwmWorldState->GetUICurrentTime() )
		cwmWorldState->SetNextFieldEffectTime( BuildTimeValue( 0.5f ) );

	// Implement RefreshItem() / statwindow() queue here

	QUEUEMAP_CITERATOR iqIter = itemQueue.begin();
	QUEUEMAP_CITERATOR iqIterEnd = itemQueue.end();
	while( iqIter != iqIterEnd )
	{
		CItem *mItem = (CItem *)iqIter->first;
		if( mItem != NULL )
			mItem->Refresh();
		++iqIter;
	}
	itemQueue.clear();


	npcQueue.clear();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void LoadJSEngine( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Loads JavaScript engine
//o---------------------------------------------------------------------------o
void LoadJSEngine( void )
{
	const SI32 DefEngineSize = 0x1000000;

	std::ifstream engineData( "engine.dat" );
	SI32 engineSize = DefEngineSize;
	if( engineData.is_open() )
	{
		char line[1024];
		engineData.getline( line, 1024 );
		UString sLine( line );
		sLine = sLine.removeComment().stripWhiteSpace();
		if( !sLine.empty() )
		{
			engineSize = UOX_MAX( sLine.toLong(), DefEngineSize );
		}
		engineData.close();
	}
	jsRuntime = JS_NewRuntime( engineSize );
	
	Console.PrintSectionBegin();
	Console << "Starting JavaScript Engine...." << myendl;
	
	if( jsRuntime == NULL )
		Shutdown( FATAL_UOX3_JAVASCRIPT );
	jsContext = JS_NewContext( jsRuntime, 0x2000 );
	if( jsContext == NULL )
		Shutdown( FATAL_UOX3_JAVASCRIPT );
	jsGlobal = JS_NewObject( jsContext, &global_class, NULL, NULL ); 
	if( jsGlobal == NULL )
		Shutdown( FATAL_UOX3_JAVASCRIPT );
	JS_InitStandardClasses( jsContext, jsGlobal ); 
	Console << "JavaScript engine startup complete." << myendl;
	Console.PrintSectionBegin();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void InitClasses( void )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Initialize UOX classes
//o---------------------------------------------------------------------------o
void InitClasses( void )
{
	Boats			= NULL;	Gumps		= NULL;			Speech = NULL;
	Combat			= NULL;	Commands	= NULL;		
	Items			= NULL;	Map			= NULL;
	Npcs			= NULL;	Skills		= NULL;	
	Weight			= NULL;	Targ		= NULL;
	Network			= NULL;	Magic		= NULL;		
	Races			= NULL;	Weather		= NULL;
	Movement		= NULL;	TEffects	= NULL;	
	WhoList			= NULL;	OffList		= NULL;
	Books			= NULL;	GMQueue		= NULL;	
	Dictionary		= NULL;	Accounts	= NULL;
	MapRegion		= NULL;	SpeechSys	= NULL;
	CounselorQueue	= NULL;	GuildSys	= NULL;
	HTMLTemplates	= NULL;	Effects		= NULL;

	// MAKE SURE IF YOU ADD A NEW ALLOCATION HERE THAT YOU FREE IT UP IN Shutdown(...)
	if(( Dictionary		= new CDictionaryContainer )			== NULL ) Shutdown( FATAL_UOX3_ALLOC_DICTIONARY );
	if(( Boats			= new cBoat )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_BOATS );
	if(( Combat			= new cCombat )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_COMBAT );
	if(( Commands		= new cCommands )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_COMMANDS );
	if(( Gumps			= new cGump )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_GUMPS );
	if(( Items			= new cItem )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_ITEMS );
	if(( Map			= new cMapStuff )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_MAP );
	if(( Npcs			= new cCharStuff )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_NPCS );
	if(( Skills			= new cSkills )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_SKILLS );
	if(( Weight			= new cWeight )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_WEIGHT );
	if(( Targ			= new cTargets )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_TARG );
	if(( FileIO			= new cFileIO )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_FILEIO );
	if(( Network		= new cNetworkStuff )					== NULL ) Shutdown( FATAL_UOX3_ALLOC_NETWORK );
	if(( Magic			= new cMagic )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_MAGIC );
	if(( Races			= new cRaces )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_RACES );
	if(( Weather		= new cWeatherAb )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_WEATHER );
	if(( Movement		= new cMovement )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_MOVE );
	if(( TEffects		= new cTEffect )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_TEMPEFFECTS );	// addition of TEffect class, memory reduction (Abaddon, 17th Feb 2000)
	if(( WhoList		= new cWhoList )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_WHOLIST );	// wholist
	if(( OffList		= new cWhoList( false ) )				== NULL ) Shutdown( FATAL_UOX3_ALLOC_WHOLIST );	// offlist
	if(( Books			= new cBooks )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_BOOKS );
	if(( GMQueue		= new PageVector( "GM Queue" ) )		== NULL ) Shutdown( FATAL_UOX3_ALLOC_PAGEVECTOR );
	if(( CounselorQueue	= new PageVector( "Counselor Queue" ) )	== NULL ) Shutdown( FATAL_UOX3_ALLOC_PAGEVECTOR );
	if(( Trigger		= new Triggers )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_TRIGGERS );
	if(( MapRegion		= new cMapRegion )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_MAPREGION );
	if(( Effects		= new cEffects )						== NULL ) Shutdown( FATAL_UOX3_ALLOC_EFFECTS );
	if(( Speech			= new cSpeech )							== NULL ) Shutdown( FATAL_UOX3_ALLOC_SPEECH );
	if(( HTMLTemplates	= new cHTMLTemplates )					== NULL ) Shutdown( FATAL_UOX3_ALLOC_HTMLTEMPLATES );
	if(( Accounts		= new cAccountClass( cwmWorldState->ServerData()->Directory( CSDDP_ACCOUNTS ) ) ) == NULL ) Shutdown( FATAL_UOX3_ALLOC_ACCOUNTS );
	if(( SpeechSys		= new CSpeechQueue()	)				== NULL ) Shutdown( FATAL_UOX3_ALLOC_SPEECHSYS );
	if(( GuildSys		= new CGuildCollection() )				== NULL ) Shutdown( FATAL_UOX3_ALLOC_GUILDS );
	if(( FileLookup		= new cServerDefinitions() )			== NULL ) Shutdown( FATAL_UOX3_ALLOC_SCRIPTS );
	if(( JailSys		= new JailSystem() )					== NULL ) Shutdown( FATAL_UOX3_ALLOC_WHOLIST );
	Map->Cache			= cwmWorldState->ServerData()->ServerMulCachingStatus();	
}

//o---------------------------------------------------------------------------o
//|	Function	-	void ParseArgs( int argc, char *argv[] )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Unknown
//o---------------------------------------------------------------------------o
void ParseArgs( int argc, char *argv[] )
{
	for( int i = 1; i < argc; ++i )
	{
		if( !strcmp( argv[i], "-xgm" ) )
		{
			Console.PrintSectionBegin();
			Console << "XGM Enabled! Initializing... ";
			cwmWorldState->SetXGMEnabled( true );
			Console << "Done!" << myendl;
		}
		else if( !strcmp( argv[i], "-ERROR" ) )
		{
			cwmWorldState->SetErrorCount( UString( argv[i+1] ).toULong() );
			++i;
		}
		else if( !strcmp( argv[i], "-dumptags" ) )
		{
			cwmWorldState->ServerData()->dumpLookup( 0 );
			cwmWorldState->ServerData()->save( "./uox.tst.ini" );
			Shutdown(FATAL_UOX3_SUCCESS);
		}
		else if( !strcmp( argv[i], "-cluox100" ) )
		{
			Console << "Using CLUOX Streaming-IO" << myendl;
			setvbuf( stdout, NULL, _IONBF, 0 );
			setvbuf( stderr, NULL, _IONBF, 0 );
			cluox_io = true;
			++i;
			if( i > argc )
			{
				Console.Error( 0, "Fatal error in CLUOX arguments" );
				Shutdown( 10 );
			}
			char *dummy;
			cluox_stdin_writeback = (void *)strtol( argv[i], &dummy, 16 );
		}
		else if( !strncmp( argv[i], "+add:", sizeof(char)*5 ) )
		{
			//	EviLDeD:	030902:	Added this so people could add an account at the command line when they started the server
			Console << "|  Importing Accounts command line \n"; 
			std::string username,password,email;
			char *left=strtok(argv[1],":");
			username=strtok(NULL,",");
			password=strtok(NULL,",");
			email=strtok(NULL,"\n");
			if( left==NULL || username.empty() || password.empty() || email.empty() )
			{
				// there is an error with the command line so we don't want to do anything
				break;
			}
			ACCOUNTSBLOCK actbTemp;
			if( !Accounts->GetAccountByName(username.c_str(),actbTemp) )
			{
				Accounts->AddAccount( username, password, email );
				Console << "| AccountImport: Added: " << username << " @ " << email << "\n";
			}
			else
			{
				Console << "| AccountImport: Failure\n";
			}
		}
		else if( !strncmp( argv[i], "+import:", sizeof(char)*8 ) )
		{
			//	EviLDeD:	030902:	Added this so people could add accounts froma file that contains username/password/email format per line
			std::string  filename,username,password,email;
			char *left;
			left			= strtok(argv[1],":");
			filename	= strtok(NULL,",");
			std::ifstream inFile;
			inFile.open( filename.c_str() );
			char szBuffer[127];
			if( inFile.is_open() )
			{
				// Ok the files is open lets read it. Otherwise just skip it
				Console << "|  Importing Accounts from \"" << filename << "\"\n"; 
				inFile.getline( szBuffer,127 );
				while(  !inFile.eof() && !inFile.fail() )
				{
					if( (left = strtok(szBuffer,":")) == NULL)
					{
						inFile.getline(szBuffer,127);
						continue;
					}
					left		= strtok( szBuffer, ":" );
					username	= strtok( NULL, "," );
					password	= strtok( NULL, "," );
					email		= strtok( NULL, "\n" );
					if( left==NULL || username.empty() || password.empty() || email.empty() )
					{
						inFile.getline(szBuffer,127);
						continue;
					}
					ACCOUNTSBLOCK actbTemp;
					if( !Accounts->GetAccountByName(username.c_str(),actbTemp) )
					{
						Accounts->AddAccount(username, password, email );
						Console << "| AccountImport: Added: " << username << " @ " << email << "\n";
					}
					else
					{
						Console << "| AccountImport: Failure\n";
					}
					inFile.getline( szBuffer,127);
				};
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void ResetVars( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Reset global variables	(Many are now set in CWorldMain())
//o---------------------------------------------------------------------------o
void ResetVars( void )
{
	Console.PrintSectionBegin();
	Console << "Initializing global variables  ";
	globalSent = 0;
	globalRecv = 0;
	
	strcpy( cwmWorldState->skill[ALCHEMY].madeword,			"mixed" );
	strcpy( cwmWorldState->skill[ANATOMY].madeword,			"made" );
	strcpy( cwmWorldState->skill[ANIMALLORE].madeword,		"made" );
	strcpy( cwmWorldState->skill[ITEMID].madeword,			"made" );
	strcpy( cwmWorldState->skill[ARMSLORE].madeword,		"made" );
	strcpy( cwmWorldState->skill[PARRYING].madeword,		"made" );
	strcpy( cwmWorldState->skill[BEGGING].madeword,			"made" );
	strcpy( cwmWorldState->skill[BLACKSMITHING].madeword,	"forged" );
	strcpy( cwmWorldState->skill[BOWCRAFT].madeword,		"bowcrafted" );
	strcpy( cwmWorldState->skill[PEACEMAKING].madeword,		"made" );
	strcpy( cwmWorldState->skill[CAMPING].madeword,			"made" );
	strcpy( cwmWorldState->skill[CARPENTRY].madeword,		"made" );
	strcpy( cwmWorldState->skill[CARTOGRAPHY].madeword,		"wrote" );
	strcpy( cwmWorldState->skill[COOKING].madeword,			"cooked" );
	strcpy( cwmWorldState->skill[DETECTINGHIDDEN].madeword, "made" );
	strcpy( cwmWorldState->skill[ENTICEMENT].madeword,		"made" );
	strcpy( cwmWorldState->skill[EVALUATINGINTEL].madeword, "made" );
	strcpy( cwmWorldState->skill[HEALING].madeword,			"made" );
	strcpy( cwmWorldState->skill[FISHING].madeword,			"made" );
	strcpy( cwmWorldState->skill[FORENSICS].madeword,		"made" );
	strcpy( cwmWorldState->skill[HERDING].madeword,			"made" );
	strcpy( cwmWorldState->skill[HIDING].madeword,			"made" );
	strcpy( cwmWorldState->skill[PROVOCATION].madeword,		"made" );
	strcpy( cwmWorldState->skill[INSCRIPTION].madeword,		"wrote" );
	strcpy( cwmWorldState->skill[LOCKPICKING].madeword,		"made" );
	strcpy( cwmWorldState->skill[MAGERY].madeword,			"envoked" );
	strcpy( cwmWorldState->skill[MAGICRESISTANCE].madeword, "made" );
	strcpy( cwmWorldState->skill[TACTICS].madeword,			"made" );
	strcpy( cwmWorldState->skill[SNOOPING].madeword,		"made" );
	strcpy( cwmWorldState->skill[MUSICIANSHIP].madeword,	"made" );
	strcpy( cwmWorldState->skill[POISONING].madeword,		"made" );
	strcpy( cwmWorldState->skill[ARCHERY].madeword,			"made" );
	strcpy( cwmWorldState->skill[SPIRITSPEAK].madeword,		"made" );
	strcpy( cwmWorldState->skill[STEALING].madeword,		"made" );
	strcpy( cwmWorldState->skill[TAILORING].madeword,		"sewn" );
	strcpy( cwmWorldState->skill[TAMING].madeword,			"made" );
	strcpy( cwmWorldState->skill[TASTEID].madeword,			"made" );
	strcpy( cwmWorldState->skill[TINKERING].madeword,		"made" );
	strcpy( cwmWorldState->skill[TRACKING].madeword,		"made" );
	strcpy( cwmWorldState->skill[VETERINARY].madeword,		"made" );
	strcpy( cwmWorldState->skill[SWORDSMANSHIP].madeword,	"made" );
	strcpy( cwmWorldState->skill[MACEFIGHTING].madeword,	"made" );
	strcpy( cwmWorldState->skill[FENCING].madeword,			"made" );
	strcpy( cwmWorldState->skill[WRESTLING].madeword,		"made" );
	strcpy( cwmWorldState->skill[LUMBERJACKING].madeword,	"made" );
	strcpy( cwmWorldState->skill[MINING].madeword,			"smelted" );
	strcpy( cwmWorldState->skill[MEDITATION].madeword,		"envoked" );
	strcpy( cwmWorldState->skill[STEALTH].madeword,			"made" );
	strcpy( cwmWorldState->skill[REMOVETRAPS].madeword,		"made" );
#ifdef __NEW_SKILLS__
	strcpy( cwmWorldState->skill[NECROMANCY].madeword,		"made" );
	strcpy( cwmWorldState->skill[FOCUS].madeword,			"made" );
	strcpy( cwmWorldState->skill[CHIVALRY].madeword,		"made" );
#endif

	Console.PrintDone();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void InitMultis( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Initialize Multis
//o---------------------------------------------------------------------------o
void InitMultis( void )
{
	Console << "Initializing multis            ";

	CMultiObj *multi = NULL;
	for( CHARACTER c = 0; c < cwmWorldState->GetCharCount(); ++c )
	{
		if( !chars[c].isFree() )
		{
			multi = findMulti( &chars[c] );
			if( multi != NULL )
				chars[c].SetMulti( multi );
			else
				chars[c].SetMulti( INVALIDSERIAL );
		}
	}
	for( ITEM i = 0; i < cwmWorldState->GetItemCount(); ++i )
	{
		if( !items[i].isFree() && items[i].GetCont() == NULL )
		{
			multi = findMulti( &items[i] );
			if( multi != NULL )
			{
				if( multi != &items[i] )
					items[i].SetMulti( multi );
				else 
					items[i].SetMulti( INVALIDSERIAL );
			}
			else
				items[i].SetMulti( INVALIDSERIAL );
		}
	}
	Console.PrintDone();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void DisplayBanner( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	UOX startup stuff
//o---------------------------------------------------------------------------o
void DisplayBanner( void )
{
	Console.PrintSectionBegin();

	char idname[256];
	sprintf( idname, "%s v%s(%s) [%s]\n| Compiled by %s\n| Programmed by: %s", cVersionClass::GetProductName().c_str(), cVersionClass::GetVersion().c_str(), cVersionClass::GetBuild().c_str(), OS_STR, cVersionClass::GetName().c_str(), cVersionClass::GetProgrammers().c_str() );
 
	/*Console << myendl;
	Console << "Configured for connections with Ignition support and 2.x compatability" << myendl;
	Console << "Copyright (C) 1997, 98 Marcus Rating (Cironian)" << myendl;//<< "|" << myendl;
	Console << "This program is free software; you can redistribute it and/or modify" << myendl;
	Console << "it under the terms of the GNU General Public License as published by" << myendl;
	Console << "the Free Software Foundation; either version 2 of the License, or" << myendl;
	Console << "(at your option) any later version." << myendl << "|" << myendl;
	Console << myendl;*/
	
	Console.TurnYellow();
	Console << "Compiled on ";
	Console.TurnNormal();
	Console << __DATE__ << " (" << __TIME__ << ")" << myendl;

	Console.TurnYellow();
	Console << "Compiled by ";
	Console.TurnNormal();
	Console << cVersionClass::GetName() << myendl;

	Console.TurnYellow();
	Console << "Contact: ";
	Console.TurnNormal();
	Console << cVersionClass::GetEmail() << myendl;
	
	Console.PrintSectionBegin();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void SleepNiceness( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Based on server.scp entry, choose how much CPU UOX will hog
//o---------------------------------------------------------------------------o
void SleepNiceness( void )
{
	switch( cwmWorldState->ServerData()->Niceness() )
	{
		case 0:						break;  // very unnice - hog all cpu time
		case 1:
			if( cwmWorldState->GetPlayersOnline() != 0 )
				UOXSleep( 10 );
			else 
				UOXSleep( 90 );
			break;
		case 2: UOXSleep( 10 );		break;
		case 3: UOXSleep( 40 );		break;// very nice
		default: UOXSleep( 10 );	break;
	}

}

//o---------------------------------------------------------------------------o
//|	Function	-	int main( int argc, char *argv[] )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	UOX startup stuff
//o---------------------------------------------------------------------------o
int main( int argc, char *argv[] )
{
	UI32 uiNextCheckConn = 0;
	UI32 tempSecs, tempMilli, tempTime;
	UI32 loopSecs, loopMilli;

	// EviLDeD: 042102: I moved this here where it basically should be for any windows application or dll that uses WindowsSockets.
#if !defined(__unix__)
	WSADATA wsaData;
	WORD wVersionRequested = MAKEWORD( 2, 0 );
	SI32 err = WSAStartup( wVersionRequested, &wsaData );
	if( err )
	{
		Console.Error( 0, "Winsock 2.0 not found on your system..." );
		return 1;
	}
#endif

	char temp[1024];
#ifdef _CRASH_PROTECT_
	try 
	{//Crappy error trapping....
#endif
#if defined( __unix__ )
		gettimeofday( &current, NULL );
#else
		timeb local;
		::ftime( &local );
		current.tv_sec = local.time;
		current.tv_usec = local.millitm * 1000;
#endif
		UI32 currentTime = 0;
		
#if !defined(__unix__)
		sprintf( temp, "%s v%s(%s)", cVersionClass::GetProductName().c_str(), cVersionClass::GetVersion().c_str(), cVersionClass::GetBuild().c_str() );
		Console.Start( temp );
#else
		signal( SIGPIPE, SIG_IGN ); // This appears when we try to write to a broken network connection
		signal( SIGTERM, &endmessage );
		signal( SIGQUIT, &endmessage );
		signal( SIGINT, &endmessage ); 
		signal( SIGILL, &illinst );
		signal( SIGFPE, &aus );
		
#endif
		
		Console.PrintSectionBegin();
		Console << "UOX Server start up!" << myendl << "Welcome to " << cVersionClass::GetProductName() << " v" << cVersionClass::GetVersion() << "(" << cVersionClass::GetBuild() << ")" << myendl;
		Console.PrintSectionBegin();

		LoadJSEngine();

		if(( cwmWorldState = new CWorldMain ) == NULL ) 
			Shutdown( FATAL_UOX3_ALLOC_WORLDSTATE );
		FileIO->LoadINIFile();

		Console << "Initializing and creating class pointers... " << myendl;
		InitClasses();
		cwmWorldState->SetUICurrentTime( currentTime );

		ParseArgs( argc, argv );
		ResetVars();
		
		Console << "Loading skill advancement      ";
		FileIO->LoadSkills();
		Console.PrintDone();
		
		cwmWorldState->SetKeepRun( Network->kr ); // for some technical reasons global varaibles CANT be changed in constructors in c++.
		cwmWorldState->SetError( Network->faul );  // i hope i can find a cleaner solution for that, but this works !!!
		// has to here and not at the cal cause it would get overriten later

		// Moved BulkStartup here, dunno why that function was there...
		Console << "Loading dictionaries...        " << myendl ;
		Console.PrintBasedOnVal( Dictionary->LoadDictionary() >= 0 );

		Console << "Loading teleport               ";
		FileIO->LoadTeleportLocations();
		Console.PrintDone();
		
		srand( current.tv_sec ); // initial randomization call
		
		// moved all the map loading into cMapStuff - fur
		Map->Load();
		
		Skills->Load();
		
		Console << "Loading Spawn Regions          ";
		FileIO->LoadSpawnRegions();
		Console.PrintDone();

		Console << "Loading Regions                ";
		FileIO->LoadRegions();
		Console.PrintDone();

		Magic->LoadScript();
		
		Console << "Loading Races                  ";
		Races->load();
		Console.PrintDone();

		Console << "Loading Weather                ";
		Weather->Load();
		Weather->NewDay();
		Console.PrintDone();

		Console << "Loading Commands               ";
		Commands->Load();
		Console.PrintDone();
		
		// Rework that...
		Console << "Loading World now              ";
		FileIO->LoadNewWorld();

		Console.PrintSectionBegin();
		Console << "Clearing all trades            ";
		clearTrades();
		Console.PrintDone();
		InitMultis();
		
		cwmWorldState->SetStartTime( cwmWorldState->GetUICurrentTime() );
		doGCollect();

		cwmWorldState->SetEndTime( 0 );
		cwmWorldState->SetLClock( 0 );

		// no longer Que, because that's taken care of by PageVector
		Console << "Initializing Jail system       ";	
		JailSys->ReadSetup();
		JailSys->ReadData();
		Console.PrintDone();
		
		Console << "Initializing Status system     ";	
		HTMLTemplates->Load( ETT_ALLTEMPLATES );
		Console.PrintDone();

		Console << "Loading custom titles          ";
		FileIO->LoadCustomTitle();
		Console.PrintDone();

		Console << "Loading temporary Effects      ";
		Effects->LoadEffects();
		Console.PrintDone();
			
		if( cwmWorldState->ServerData()->ServerAnnounceSavesStatus() )
			cwmWorldState->announce( 1 );
		else
			cwmWorldState->announce( 0 );

		FileIO->LoadCreatures();
		DisplayBanner();
		//DisplaySettings(); << Moved that to the configuration

		Console << "Loading Accounts               ";
		Accounts->Load();
		//Console.PrintDone(); 

		Console.Log( "-=Server Startup=-\n=======================================================================", "server.log" );
//		cwmWorldState->SetUICurrentTime( getclock() );

		Console << myendl << "Initialize Console Thread      ";
#if defined(__unix__)
		int conthreadok = pthread_create(&cons,NULL,CheckConsoleKeyThread , NULL );
#else
		int conthreadok = _beginthread( CheckConsoleKeyThread , 0 , NULL );
#endif
#ifdef __LOGIN_THREAD__
 #if defined(__unix__)
		pthread_create(&netw,NULL, NetworkPollConnectionThread,  NULL );
 #else
		_beginthread( NetworkPollConnectionThread, 0, NULL );
 #endif //linux
#endif
		Console.PrintDone();

		Console.PrintSectionBegin();
		// we've really finished loading here
		cwmWorldState->SetLoaded( true );
		Console << "UOX: Startup Complete" << myendl;
		Console.PrintSectionBegin();
		
		// MAIN SYSTEM LOOP
		while( cwmWorldState->GetKeepRun() )
		{
			//	EviLDeD	-	February 27, 2000
			//	Just in case the thread doesn't start then use the main threaded copy
			if( conthreadok == -1 )
				checkkey();
			SleepNiceness();
			//	EviLDeD	-	End
			if( loopTimeCount >= 1000 )
			{
				loopTimeCount = 0;
				loopTime = 0;
			}
			++loopTimeCount;
			
			StartMilliTimer( loopSecs, loopMilli );
			
			if( networkTimeCount >= 1000 )
			{
				networkTimeCount = 0;
				networkTime = 0;
			}
			
			StartMilliTimer( tempSecs, tempMilli );
#ifndef __LOGIN_THREAD__
			if( uiNextCheckConn <= cwmWorldState->GetUICurrentTime() || cwmWorldState->GetOverflow() ) // Cut lag on CheckConn by not doing it EVERY loop.
			{
				Network->CheckConnections();
				uiNextCheckConn = BuildTimeValue( 1.0f );
			}
			Network->CheckMessages();
#else
			Network->CheckMessage();
//			Network->CheckXGM();
#endif
			
			tempTime = CheckMilliTimer( tempSecs, tempMilli );
			networkTime += tempTime;
			++networkTimeCount;
			
			if( timerTimeCount >= 1000 )
			{
				timerTimeCount = 0;
				timerTime = 0;
			}
			
			StartMilliTimer( tempSecs, tempMilli );
			
			cwmWorldState->CheckTimers();
			cwmWorldState->SetUICurrentTime( getclock() );
			tempTime = CheckMilliTimer( tempSecs, tempMilli );
			timerTime += tempTime;
			++timerTimeCount;
			
			if( autoTimeCount >= 1000 )
			{
				autoTimeCount = 0;
				autoTime = 0;
			}
			StartMilliTimer( tempSecs, tempMilli );
			
			checkauto();
			
			tempTime = CheckMilliTimer( tempSecs, tempMilli );
			autoTime  += tempTime;
			++autoTimeCount;
			StartMilliTimer( tempSecs, tempMilli );
			Network->ClearBuffers();
			tempTime = CheckMilliTimer( tempSecs, tempMilli );
			networkTime += tempTime;
			tempTime = CheckMilliTimer( loopSecs, loopMilli );
			loopTime += tempTime;
			DoMessageLoop();
		}

		
		Console.PrintSectionBegin();
		Speech->sysbroadcast( "The server is shutting down." );
		Console.PrintDone();
		Console << "Closing sockets...";
		netpollthreadclose = true;
		UOXSleep( 1000 );
		Network->SockClose();
		Console.PrintDone();

		if( !cwmWorldState->Saving() )
		{
			do 
			{
				cwmWorldState->savenewworld( true );
			} while( cwmWorldState->Saving() );
		}
		
		cwmWorldState->ServerData()->save();

		Console.Log( "Server Shutdown!\n=======================================================================\n" , "server.log" );

		conthreadcloseok = true;	//	This will signal the console thread to close
		Shutdown( 0 );

#ifdef _CRASH_PROTECT_
	}
	catch ( ... ) 
	{//Crappy error handling...
		Console << "Unknown exception caught, hard crash avioded!" << myendl;
		Shutdown( UNKNOWN_ERROR );
	}
#endif
	
	return ( 0 );	
}

//o---------------------------------------------------------------------------o
//|            Function     - Restart()
//|            Date         - 1/7/00
//|            Programmer   - Zippy
//o---------------------------------------------------------------------------o
//|            Purpose      - Restarts the server, passes the server number of 
//|								Number of crashes so far, if < 10 then the
//|								Server will restart itself.
//o---------------------------------------------------------------------------o
void Restart( UI16 ErrorCode = UNKNOWN_ERROR )
{
	if( !ErrorCode )
		return;
	char temp[1024];
	if( cwmWorldState->ServerData()->ServerCrashProtectionStatus() > 1 )
	{		
		if( cwmWorldState->GetErrorCount() < 10 )
		{
			cwmWorldState->IncErrorCount();
			
			sprintf( temp, "Server crash #%u from unknown error, restarting.", cwmWorldState->GetErrorCount() );
			Console.Log( temp, "server.log" );
			Console << temp << myendl;
			
			sprintf(temp, "uox.exe -ERROR %u", cwmWorldState->GetErrorCount() );
			
			if( cwmWorldState->GetXGMEnabled() )
				strcat( temp, " -xgm" );
			
			delete cwmWorldState;
			system( temp );
			exit(ErrorCode); // Restart successful Don't give them key presses or anything, just go out.
		} 
		else 
		{
			Console.Log( "10th Server crash, server shutting down.", "server.log" );
			Console << "10th Server crash, server shutting down" << myendl;
		}
	} 
	else 
		Console.Log( "Server crash!", "server.log" );
}

//o---------------------------------------------------------------------------o
//|            Function     - void Shutdown( int retCode )
//|            Date         - Oct. 09, 1999
//|            Programmer   - Krazyglue
//o---------------------------------------------------------------------------o
//|            Purpose      - Handled deleting / free() ing of pointers as neccessary
//|                                   as well as closing open file handles to avoid file
//|                                   file corruption.
//|                                   Exits with proper error code.
//o---------------------------------------------------------------------------o
void Shutdown( SI32 retCode )
{
	Console.PrintSectionBegin();
	Console << "Beginning UOX final shut down sequence..." << myendl;

	if( HTMLTemplates )
	{
		Console << "HTMLTemplates object detected. Writing Offline HTML Now..." << myendl;
		HTMLTemplates->Poll( ETT_OFFLINE );
	}
	else
		Console << "HTMLTemplates object not found." << myendl;


	if( cwmWorldState->ServerData()->ServerCrashProtectionStatus() >= 1 && retCode && cwmWorldState->GetLoaded() && cwmWorldState && !cwmWorldState->Saving() )
	{//they want us to save, there has been an error, we have loaded the world, and WorldState is a valid pointer.
		do
		{
			cwmWorldState->savenewworld( true );
		} while( cwmWorldState->Saving() );
	}
	
	Console << "Cleaning up item and character memory... ";
	items.Cleanup();
	chars.Cleanup();
	Console.PrintDone();

	Console << "Destroying class objects and pointers... ";
	// delete any objects that were created (delete takes care of NULL check =)

	delete Boats;
	delete Combat;
	delete Commands;
	delete Gumps;
	delete Items;
	try
	{
		delete Map;
	}
	catch( ... )
	{
		Console << "ERROR: Either Map was already deleted, or UOX3 was unable to delete object." << myendl;
	}
	delete Npcs;
	delete Skills;
	delete Weight;
	delete Targ;
	delete Magic;
	delete Races;
	delete Weather;
	delete Movement;
	delete Network;
	delete TEffects;
	delete WhoList;
	delete OffList;
	delete Books;
	delete GMQueue;
	delete HTMLTemplates;
	delete CounselorQueue;
	delete Dictionary;
	delete Accounts;
	if( Trigger )
		Trigger->Cleanup();//must be called to delete some things the still reference Trigger.
	delete Trigger;
	delete MapRegion;
	delete SpeechSys;
	delete GuildSys;
	delete FileLookup;
	delete JailSys;
	delete Effects;
	delete FileIO;
	delete Speech;
	Console.PrintDone();

	//Lets wait for console thread to quit here
#if defined(__unix__)
	pthread_join( cons, NULL );
#ifdef __LOGIN_THREAD__
	pthread_join( newt, NULL );
#endif
#endif

	// don't leave file pointers open, could lead to file corruption
	if( loscache )			
		delete [] loscache;
	if( itemids )			
		delete [] itemids;
	Console << "Destroying JS instances... ";
	JS_DestroyContext( jsContext );
	JS_DestroyRuntime( jsRuntime );
	Console.PrintDone();

	Console.PrintSectionBegin();
	if( retCode && cwmWorldState->GetLoaded() )//do restart unless we have crashed with some error.
		Restart( (UI16)retCode );
	else
		delete cwmWorldState;

	Console.TurnGreen();
	Console << "Server shutdown complete!" << myendl;
	Console << "Thank you for supporting " << cVersionClass::GetName() << myendl;
	Console.TurnNormal();
	Console.PrintSectionBegin();
	
	// dispay what error code we had
	// don't report errorlevel for no errors, this is confusing ppl - fur
	if( retCode )
	{
		Console.TurnRed();
		Console << "Exiting UOX with errorlevel " << retCode << myendl;
		Console.TurnNormal();
#if !defined(__unix__)
		Console << "Press Return to exit " << myendl;
		std::string throwAway;
		std::getline(std::cin, throwAway);
#endif
	}
	else
	{
		Console.TurnGreen();
		Console << "Exiting UOX with no errors..." << myendl;
		Console.TurnNormal();
	}
	
	Console.PrintSectionBegin();
	exit(retCode);
}

//o---------------------------------------------------------------------------o
//|	Function	-	SI32 getStringValue( std::string string )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Pulls tokens from a string, if one value returns it, if two
//|					values it finds a random number between the values
//o---------------------------------------------------------------------------o
SI32 getStringValue( std::string input ) 
{
	UString sInput( input );
	sInput			= sInput.simplifyWhiteSpace();
	SI32 lovalue	= sInput.section( " ", 0, 0 ).toLong();
	SI32 hivalue	= 0;
	if( sInput.sectionCount( " " ) > 0 )
	{
		lovalue = RandomNum( lovalue, sInput.section( " ", 1, 1 ).toLong() );
	}
	return lovalue;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void advanceObj( CChar *s, UI16 x, bool multiUse )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Handle advancement objects (stat / skill gates)
//o---------------------------------------------------------------------------o
void advanceObj( CChar *s, UI16 x, bool multiUse )
{
	if( s->GetAdvObj() == 0 || multiUse )
	{
		Effects->staticeffect( s, 0x373A, 0, 15);
		Effects->PlaySound( s, 0x01E9 );
		s->SetAdvObj( x );
		char sect[512];
		sprintf( sect, "ADVANCEMENT %i", x );
		ScriptSection *Advancement = FileLookup->FindEntry( sect, advance_def );
		if( Advancement == NULL )
		{
			Console << "ADVANCEMENT OBJECT: Script section not found, Aborting" << myendl;
			s->SetAdvObj( 0 );
			return;
		}
		else
		{
			UI08 val = 0;
			CItem *retitem = NULL;
			UString tag;
			UString data;
			for( tag = Advancement->First(); !Advancement->AtEnd(); tag = Advancement->Next() )
			{
				data = Advancement->GrabData();
				switch( (tag.data()[0]) )
				{
					case 'a':
					case 'A':
						if( tag.upper() == "ALCHEMY" )
							val = ALCHEMY;
						else if( tag.upper() == "ANATOMY" )
							val = ANATOMY;
						else if( tag.upper() == "ANIMALLORE" )
							val = ANIMALLORE;
						else if( tag.upper() == "ARMSLORE" )
							val = ARMSLORE;
						else if( tag.upper() == "ARCHERY" )
							val = ARCHERY;
						else if( tag.upper() == "ADVOBJ" )
							s->SetAdvObj( data.toUShort() );
						break;
					case 'b':
					case 'B':
						if( tag.upper() == "BEGGING" )
							val = BEGGING;
						else if( tag.upper() == "BLACKSMITHING" )
							val = BLACKSMITHING;
						else if( tag.upper() == "BOWCRAFT" )
							val = BOWCRAFT;
						break;
					case 'c':
					case 'C':
						if( tag.upper() == "CAMPING" )
							val = CAMPING;
						else if( tag.upper() == "CARPENTRY" )
							val = CARPENTRY;
						else if( tag.upper() == "CARTOGRAPHY" )
							val = CARTOGRAPHY;
#ifdef __NEW_SKILLS__
						else if( tag.upper() == "CHIVALRY" )
							val = CHIVALRY;
#endif
						else if( tag.upper() == "COOKING" )
							val = COOKING;
						break;
					case 'd':
					case 'D':
						if( tag.upper() == "DEX" || tag.upper() == "DEXTERITY" )
							s->SetDexterity( static_cast<SI16>(getStringValue( data.c_str() )) );
						else if( tag.upper() == "DETECTINGHIDDEN" )
							val = DETECTINGHIDDEN;
						else if( tag.upper() == "DYEHAIR" )
						{
							CItem *hairobject = s->GetItemAtLayer( 0x0B );
							if( hairobject != NULL )
								hairobject->SetColour( data.toUShort() );
						}
						else if( tag.upper() == "DYEBEARD" )
						{
							CItem *beardobject = s->GetItemAtLayer( 0x10 );
							if( beardobject != NULL )
								beardobject->SetColour( data.toUShort() );
						}
						break;
					case 'e':
					case 'E':
						if( tag.upper() == "ENTICEMENT" )
							val = ENTICEMENT;
						else if( tag.upper() == "EVALUATINGINTEL" )
							val = EVALUATINGINTEL;
						else if( tag.upper() == "EQUIPITEM" )
						{
							retitem = Items->CreateScriptItem( NULL, data.c_str(), false, s->WorldNumber() );
							if( retitem != NULL )
							{
								retitem->SetX( (SI16)( 50 + (RandomNum( 0, 79 )) ) );
								retitem->SetY( (SI16)( 50 + (RandomNum( 0, 79 )) ) ); 
								retitem->SetZ( 9 );
								if( !retitem->SetCont( s ) )
									retitem->SetCont( getPack( s ) );

							}
						}
						break;
					case 'f':
					case 'F':
						if( tag.upper() == "FAME" )
							s->SetFame( data.toShort() );
						else if( tag.upper() == "FENCING" )
							val = FENCING;
						else if( tag.upper() == "FISHING" )
							val = FISHING;
#ifdef __NEW_SKILLS__
						else if( tag.upper() == "FOCUS" )
							val = FOCUS;
#endif
						else if( tag.upper() == "FORENSICS" )
							val = FORENSICS;
						break;
					case 'h':
					case 'H':
						if( tag.upper() == "HEALING" )
							val = HEALING;
						else if( tag.upper() == "HERDING" )
							val = HERDING;
						else if( tag.upper() == "HIDING" )
							val = HIDING;
						break;
					case 'i':
					case 'I':
						if( tag.upper() == "INT" || tag.upper() == "INTELLIGENCE" )
							s->SetIntelligence( static_cast<SI16>(getStringValue( data.c_str() )) );
						else if( tag.upper() == "ITEMID" )
							val = ITEMID;
						else if( tag.upper() == "INSCRIPTION" )
							val = INSCRIPTION;
						break;
					case 'k':
					case 'K':
						if( tag.upper() == "KARMA" )
							s->SetKarma( data.toShort() );
						else if( tag.upper() == "KILLHAIR" )
						{
							retitem = s->GetItemAtLayer( 0x0B );
							if( retitem != NULL )
								Items->DeleItem( retitem );
						}
						else if( tag.upper() == "KILLBEARD" )
						{
							retitem = s->GetItemAtLayer( 0x10 );
							if( retitem != NULL )
								Items->DeleItem( retitem );
						}
						else if( tag.upper() == "KILLPACK" )
						{
							retitem = s->GetItemAtLayer( 0x15 );
							if( retitem != NULL )
								Items->DeleItem( retitem );
						}
						break;
					case 'l':
					case 'L':
						if( tag.upper() == "LOCKPICKING" )
							val = LOCKPICKING;
						else if( tag.upper() == "LUMBERJACKING" )
							val = LUMBERJACKING;
						break;
					case 'm':
					case 'M':
						if( tag.upper() == "MAGERY" )
							val = MAGERY;
						else if( tag.upper() == "MAGICRESISTANCE" )
							val = MAGICRESISTANCE;
						else if( tag.upper() == "MACEFIGHTING" )
							val = MACEFIGHTING;
						else if( tag.upper() == "MEDITATION" )
							val = MEDITATION;
						else if( tag.upper() == "MINING" )
							val = MINING;
						else if( tag.upper() == "MUSICIANSHIP" )
							val = MUSICIANSHIP;
						break;
#ifdef __NEW_SKILLS__
					case 'n':
					case 'N':
						if( tag.upper() == "NECROMANCY" )
							val = NECROMANCY;
						break;
#endif
					case 'p':
					case 'P':
						if( tag.upper() == "PARRYING" )
							val = PARRYING;
						else if( tag.upper() == "PEACEMAKING" )
							val = PEACEMAKING;
						else if( tag.upper() == "POISONING" )
							val = POISONING;
						else if( tag.upper() == "PROVOCATION" )
							val = PROVOCATION;
						else if( tag.upper() == "POLY" )
							s->SetID( data.toUShort() );
						else if( tag.upper() == "PACKITEM" )
						{
#if defined( _MSC_VER )
#pragma note( "Verify that the ustring stuff works well here" )
#endif
							if( !data.empty() )
							{
								if( data.sectionCount( "," ) != 0 )
								{
									retitem = Items->SpawnItemToPack( NULL, s, data.section( ",", 0, 0 ).stripWhiteSpace().upper(), false );
									if( retitem != NULL )
									{
										if( retitem->isPileable() )
											retitem->SetAmount( data.section( ",", 1, 1 ).stripWhiteSpace().toUShort() );
										else
										{
											std::string mitem	= data.section( ",", 0, 0 ).stripWhiteSpace();
											int mtotal			= data.section( ",", 1, 1 ).stripWhiteSpace().toUShort() - 1;	// we all ready created one
											// We have to create a series of individual items, for it doesn't stack
											for( int mindex = 0; mindex < mtotal; ++mindex )
											{
												retitem = Items->SpawnItemToPack( NULL, s, mitem, false );
											}
										}
									}
								}
								else
									retitem = Items->SpawnItemToPack( NULL, s, data.c_str(), false );
							}
						}
						break;
					case 'r':
					case 'R':
						if( tag.upper() == "REMOVETRAPS" )
							val = REMOVETRAPS;
						break;
					case 's':
					case 'S':
						if( tag.upper() == "STR" || tag.upper() == "STRENGTH" )
							s->SetStrength( static_cast<SI16>(getStringValue( data.c_str() )) );
						else if( tag.upper().substr( 0, 5 ) == "SKILL" )	// get number code
							val = UString( tag.substr(5)).toUByte();
						else if( tag.upper() == "SKIN" )
							s->SetSkin( data.toUShort() );
						else if( tag.upper() == "SNOOPING" )
							val = SNOOPING;
						else if( tag.upper() == "SPIRITSPEAK" )
							val = SPIRITSPEAK;
						else if( tag.upper() == "STEALING" )
							val = STEALING;
						else if( tag.upper() == "STEALTH" )
							val = STEALTH;
						else if( tag.upper() == "SWORDSMANSHIP" )
							val = SWORDSMANSHIP;
						break;
					case 't':
					case 'T':
						if( tag.upper() == "TACTICS" )
							val = TACTICS;
						else if( tag.upper() == "TAILORING" )
							val = TAILORING;
						else if( tag.upper() == "TAMING" )
							val = TAMING;
						else if( tag.upper() == "TASTEID" )
							val = TASTEID;
						else if( tag.upper() == "TINKERING" )
							val = TINKERING;
						else if( tag.upper() == "TRACKING" )
							val = TRACKING;
						break;
					case 'v':
					case 'V':
						if( tag.upper() == "VETERINARY" )
							val = VETERINARY;
						break;
					case 'w':
					case 'W':
						if( tag.upper() == "WRESTLING" )
							val = WRESTLING;
						break;
				}
				
				if( val > 0 )
				{
					s->SetBaseSkill( static_cast<UI16>(getStringValue( data.c_str() )), val );
					val = 0;	// reset for next time through
				}
			}
			s->Teleport();
		}
	}
	else
	{
		cSocket *sock = calcSocketObjFromChar( s );
		if( sock != NULL )
			sock->sysmessage( 1366 );
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void objTeleporters( CChar *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Called when a character walks into/on an object teleporter
//o---------------------------------------------------------------------------o
void objTeleporters( CChar *s )
{
	SI16 x = s->GetX(), y = s->GetY();
	
	SI16 xOffset = MapRegion->GetGridX( s->GetX() );
	SI16 yOffset = MapRegion->GetGridY( s->GetY() );
	UI08 worldNumber = s->WorldNumber();
	for( SI08 counter1 = -1; counter1 <= 1; ++counter1 )
	{
		for( SI08 counter2 = -1; counter2 <= 1; ++counter2 )
		{
			SubRegion *toCheck = MapRegion->GetGrid( xOffset + counter1, yOffset + counter2, worldNumber );
			if( toCheck == NULL )
				continue;
			toCheck->PushItem();
			// Handle the STL properly
			ITEMLIST_ITERATOR I;
			//for( CItem *itemCheck = toCheck->FirstItem(); !toCheck->FinishedItems(); itemCheck = toCheck->GetNextItem() )
			for( I=toCheck->itemData.begin(); I!=toCheck->itemData.end(); ++I )
			{
				CItem * itemCheck;
				itemCheck=(*I);
				if( itemCheck == NULL || I == toCheck->itemData.end())
					continue;
				if( itemCheck->GetX() == x && itemCheck->GetY() == y && 
					( abs( itemCheck->GetZ() ) + 10 >= abs( s->GetZ() ) ) && 
					( abs( itemCheck->GetZ() ) - 10 <= abs( s->GetZ() ) ) )
				{
					switch( itemCheck->GetType() )
					{
						case 60:														// teleporters
							if( itemCheck->GetMoreX() + itemCheck->GetMoreY() + itemCheck->GetMoreZ() > 0 )
							{
								s->SetLocation( (UI16)itemCheck->GetMoreX(), (UI16)itemCheck->GetMoreY(), (SI08)itemCheck->GetMoreZ() );
								s->Teleport();
							}
							break;
						case 80:														// advancement gates
						case 81:
							if( !s->IsNpc() )
							{
								if( itemCheck->GetMore() != 0 )
								{
									if( s->GetSerial() == itemCheck->GetMore() )
										advanceObj( s, (UI16)itemCheck->GetMoreX(), ( itemCheck->GetType() == 81 ) );
								}
								else
									advanceObj( s, (UI16)itemCheck->GetMoreX(), ( itemCheck->GetType() == 81 ) );
							}
							break;
						case 82:	MonsterGate( s, itemCheck->GetMoreX() );	break;	// monster gates
						case 83:														// race gates
							Races->gate( s, static_cast<RACEID>(itemCheck->GetMoreX()), itemCheck->GetMoreY() != 0 );
							break;
						case 85:														// damage objects
							if( !s->IsInvulnerable() )
							{
								s->SetHP( (SI16)( s->GetHP() - ( itemCheck->GetMoreX() + RandomNum( itemCheck->GetMoreY(), itemCheck->GetMoreZ() ) ) ) );
								if( s->GetHP() < 1 )
									s->SetHP( 0 );
								updateStats( s, 0 );
								if( s->GetHP() <= 0 )
									doDeathStuff( s );
							}
							break;
						case 86:														// sound objects
							if( (UI32)RandomNum(1,100) <= itemCheck->GetMoreZ() )
								Effects->PlaySound( itemCheck, (UI16)( (itemCheck->GetMoreX()<<8) + itemCheck->GetMoreY() ) );
							break;

						case 89:
							SocketMapChange( calcSocketObjFromChar( s ), s, itemCheck );
							break;
						case 111:	s->SetKills( 0 );		break;			// zero kill gate
						case 234:	
							if( itemCheck->GetType() == 234 && !s->IsNpc() )				// world change gate
								SendWorldChange( (WorldType)itemCheck->GetMoreX(), calcSocketObjFromChar( s ) );
							break;
					}
				}
			}
			toCheck->PopItem();
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	bool checkBoundingBox( SI16 xPos, SI16 yPos, SI16 fx1, SI16 fy1, SI08 fz1, SI16 fx2, SI16 fy2 )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check bounding box
//o---------------------------------------------------------------------------o
bool checkBoundingBox( SI16 xPos, SI16 yPos, SI16 fx1, SI16 fy1, SI08 fz1, SI16 fx2, SI16 fy2, UI08 worldNumber )
{
	if( xPos >= ( ( fx1 < fx2 ) ? fx1 : fx2 ) && xPos <= ( ( fx1 < fx2 ) ? fx2 : fx1 ) )
	{
		if( yPos >= ( ( fy1 < fy2 ) ? fy1 : fy2 ) && yPos <= ( ( fy1 < fy2 ) ? fy2 : fy1 ) )
		{
			if( fz1 == -1 || abs( fz1 - Map->Height( xPos, yPos, fz1, worldNumber ) ) <= 5 )
				return true;
		}
	}
	return false;
}

//o---------------------------------------------------------------------------o
//|	Function	-	checkBoundingCircle( SI16 xPos, SI16 yPos, SI16 fx1, SI16 fy1, SI08 fz1, SI16 radius, UI08 worldNumber )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check bounding circle
//o---------------------------------------------------------------------------o
bool checkBoundingCircle( SI16 xPos, SI16 yPos, SI16 fx1, SI16 fy1, SI08 fz1, SI16 radius, UI08 worldNumber )
{
	if( ( xPos - fx1 ) * ( xPos - fx1 ) + ( yPos - fy1 ) * ( yPos - fy1 ) <= radius * radius )
	{
		if( fz1 == -1 || abs( fz1 - Map->Height( xPos, yPos, fz1, worldNumber ) ) <= 5 )
			return true;
	}
	return false;
}

//o---------------------------------------------------------------------------o
//|	Function	-	UI32 getclock( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Return CPU time used, Emulates clock()
//o---------------------------------------------------------------------------o
UI32 getclock( void )
{
	struct timeval tv;
#if defined( __unix__ )
	gettimeofday( &tv, NULL );
#else
	timeb local;
	ftime( &local );
	tv.tv_sec	= local.time;
	tv.tv_usec	= local.millitm * 1000;
#endif
	// We want to normalise our timer to that we established at the start.
	// System start up time becomes 0, even though it's not. So we have to
	// subtract the original timer value from our current reading, to get 
	// a process start normalised time.
	tv.tv_usec	-= current.tv_usec;
	if( tv.tv_usec < 0 )	// Start microseconds were higher than our current value
	{
		tv.tv_sec -= 1;		// remove a second and add a second's worth of microseconds
		tv.tv_usec += 1000000;
	}
	// Now we have a microsecond correct seconds value, we normalise our seconds value too
	tv.tv_sec	-= current.tv_sec;
	// We're wanting to return a millisecond value
	// Milliseconds = seconds * 1000 + microseconds / 1000
	return ( tv.tv_sec * 1000 ) + ( tv.tv_usec / 1000 );
//	return tv.tv_sec + tv.tv_usec / 1000;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void doLight( cSocket *s, UI08 level )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Send light level to players client
//o---------------------------------------------------------------------------o
void doLight( cSocket *s, UI08 level )
{
	if( s == NULL )
		return;

	CChar *mChar = s->CurrcharObj();
	CPLightLevel toSend( level );

	if( mChar->GetFixedLight() != 255 )
	{
		toSend.Level( 0 );
		s->Send( &toSend );
		return;
	}

	UI08 curRegion = mChar->GetRegion();
	CWeather *wSys =  Weather->Weather( region[curRegion]->GetWeather() );
	LIGHTLEVEL toShow;

	LIGHTLEVEL dunLevel = cwmWorldState->ServerData()->DungeonLightLevel();
	// we have a valid weather system
	if( wSys != NULL )
	{
		R32 lightMin = wSys->LightMin();
		R32 lightMax = wSys->LightMax();
		if( lightMin < 300 && lightMax < 300 )
		{
			R32 i = wSys->CurrentLight();
			if( Races->VisLevel( mChar->GetRace() ) > i )
				toShow = 0;
			else
				toShow = static_cast<LIGHTLEVEL>(i - Races->VisLevel( mChar->GetRace() ));
			toSend.Level( level );
		}
		else
			toSend.Level( level );
	}
	else
	{
		if( mChar->inDungeon() )
		{
			if( Races->VisLevel( mChar->GetRace() ) > dunLevel )
				toShow = 0;
			else
				toShow = static_cast<LIGHTLEVEL>(dunLevel - Races->VisLevel( mChar->GetRace() ));
			toSend.Level( toShow );
		}
	}
	s->Send( &toSend );
}

//o---------------------------------------------------------------------------o
//|	Function	-	void telltime( cSocket *s )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get in-game time
//o---------------------------------------------------------------------------o
void telltime( cSocket *s )
{
	UI08 hour			= cwmWorldState->ServerData()->ServerTimeHours();
	UI08 minute			= cwmWorldState->ServerData()->ServerTimeMinutes();
	bool ampm			= cwmWorldState->ServerData()->ServerTimeAMPM();
	UnicodeTypes sLang	= s->Language();

	std::string tstring, tstring2;
	if( minute <= 14 )
		tstring = Dictionary->GetEntry( 1248, sLang );
	else if( minute >= 15 && minute <= 30 )
		tstring = Dictionary->GetEntry( 1249, sLang );
	else if( minute >= 30 && minute <= 45 )
		tstring = Dictionary->GetEntry( 1250, sLang );
	else
	{
		tstring = Dictionary->GetEntry( 1251, sLang );
		++hour;
		if( hour == 12 )
			hour = 0;
	}
	if( hour >= 1 && hour <= 11 )
		tstring2 = tstring + Dictionary->GetEntry( 1251 + hour, sLang );
	else if( hour == 1 && ampm )
		tstring2 = tstring + Dictionary->GetEntry( 1263, sLang );
	else
		tstring2 = tstring + Dictionary->GetEntry( 1264, sLang );
	
	if( hour == 0 )
		tstring = tstring2;
	else if( ampm )
	{
		if( hour >= 1 && hour < 6 )
			tstring2 += Dictionary->GetEntry( 1265 );
		else if( hour >= 6 && hour < 9 )
			tstring2 += Dictionary->GetEntry( 1266 );
		else 
			tstring2 += Dictionary->GetEntry( 1267 );
		s->sysmessage( tstring2.c_str() );
		return;
	}
	else
	{
		if( hour >= 1  && hour < 5 )
			tstring2 += Dictionary->GetEntry( 1268 );
		else 
			tstring2 += Dictionary->GetEntry( 1269 );
		s->sysmessage( tstring2.c_str() );
		return;
	}
	s->sysmessage( tstring .c_str());
}

//o---------------------------------------------------------------------------o
//|	Function	-	UI08 getCharDir( CChar *a, SI16 x, SI16 y )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Get characters direction
//o---------------------------------------------------------------------------o
UI08 getCharDir( CChar *a, SI16 x, SI16 y )
{
	SI16 xdif = (SI16)( x - a->GetX() );
	SI16 ydif = (SI16)( y - a->GetY() );
	
	if( xdif == 0 && ydif < 0 )
		return NORTH;
	else if( xdif > 0 && ydif < 0 )
		return NORTHEAST;
	else if( xdif > 0 && ydif ==0 )
		return EAST;
	else if( xdif > 0 && ydif > 0 )
		return SOUTHEAST;
	else if( xdif ==0 && ydif > 0 )
		return SOUTH;
	else if( xdif < 0 && ydif > 0 )
		return SOUTHWEST;
	else if( xdif < 0 && ydif ==0 )
		return WEST;
	else if( xdif < 0 && ydif < 0 )
		return NORTHWEST;
	else
		return UNKNOWNDIR;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void openBank( cSocket *s, CChar *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Opens players bank box
//o---------------------------------------------------------------------------o
void openBank( cSocket *s, CChar *i )
{
	CItem *bankBox = NULL;
	char temp[1024];
	ITEMLIST *ownedItems = i->GetOwnedItems();

	bankBox = i->GetItemAtLayer( 0x1D );
	if( bankBox != NULL )
	{
		if( bankBox->GetType() == 1 && bankBox->GetMoreX() == 1 )
		{
			CPWornItem toWearO = (*bankBox);
			s->Send( &toWearO );
			openPack(s, bankBox );
			return;
		}
	}
	
	sprintf( temp, Dictionary->GetEntry( 1283 ).c_str(), i->GetName().c_str() );
	bankBox = Items->SpawnItem( NULL, i, 1, temp, false, 0x09AB, 0, false, false );
	
	bankBox->SetLayer( 0x1D );
	bankBox->SetOwner( (cBaseObject *)i );
	if( !bankBox->SetCont( i ) )
		return;
	bankBox->SetMoreX( 1 );
	bankBox->SetType( 1 );
	CPWornItem toWear = (*bankBox);
	s->Send( &toWear );
	openPack( s, bankBox );
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *getRootPack( CItem *item )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Gets the root container an item is in (if any)
//o---------------------------------------------------------------------------o
CItem *getRootPack( CItem *item )
{
	if( item == NULL || item->GetCont() == NULL || item->GetContSerial() < BASEITEMSERIAL )	// Item has no containing item
		return NULL;

	while( item != NULL )
	{
		if( item->GetCont() == NULL || item->GetContSerial() < BASEITEMSERIAL )		// Item is on the ground or on a character
			break;
		item = (CItem *)item->GetCont();
	}
	return item;
}

//o---------------------------------------------------------------------------o
//|	Function	-	CChar *getPackOwner( CItem *p )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Returns a containers owner
//o---------------------------------------------------------------------------o
CChar *getPackOwner( CItem *p )
{
	if( p == NULL || p->GetCont() == NULL )
		return NULL;

	if( p->GetContSerial() < BASEITEMSERIAL )								// Items container is a character, return it
		return (CChar *)p->GetCont();

	CItem *rootPack = getRootPack( p );										// Find the root container for the item
	if( rootPack != NULL && rootPack->GetContSerial() < BASEITEMSERIAL )	// Ensure the root packs container is a character
		return (CChar *)rootPack->GetCont();								// Return the root packs owner
	return NULL;
}

//o---------------------------------------------------------------------------o
//|	Function	-	int getTileName( CItem *i, char* itemname )
//|	Programmer	-	UOX3 DevTeam
//o---------------------------------------------------------------------------o
//|	Purpose		-	Returns the length of an items name from tiledata.mul and
//|					sets itemname to the name
//o---------------------------------------------------------------------------o
// Can't return std::size_t as VC6 says it's not in that namespace
// The format it accepts is same as UO style - %plural/single% or %plural%
// eg arrow%s%
//    loa%ves/f% of bread
size_t getTileName( CItem *i, std::string& itemname )
{
	CTile tile;
	Map->SeekTile( i->GetID(), &tile );
	UString temp	= i->GetName() ;
	temp			= temp.simplifyWhiteSpace();
	if( temp.substr( 0, 1 ) == "#" )
	{
		temp =  static_cast< UString >( tile.Name() );
	}
	
	UI16 getAmount = i->GetAmount();
	if( getAmount == 1 )
	{
		if( tile.DisplayAsAn() )
			temp = "an " + temp;
		else if( tile.DisplayAsA() )
			temp = "a " + temp;
	}
	else
		temp = UString::number( getAmount ) + " " + temp;
	// Find out if the name has a % in it
	if( temp.sectionCount("%") > 0 )
	{
		UString single;
		UString first	= temp.section( "%", 0, 0 );
		UString rest	= temp.section( "%", 2 );
		UString plural	= rest.section( "%", 1, 1 );
		if( plural.sectionCount("/") > 0 )
		{
			single = plural.section( "/", 1 );
			plural = plural.section( "/", 0, 0 );
		}
		if( getAmount < 2 )
			temp = first + single + rest;
		else
			temp = first + plural + rest;
	}
	itemname = static_cast< std::string >( temp.simplifyWhiteSpace() );
	return itemname.size() + 1;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void checkRegion( CChar *i )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Check what region a character is in
//o---------------------------------------------------------------------------o
void checkRegion( CChar *i )
{
	if( i == NULL )
		return;
	int j;
	UI08 calcReg = calcRegionFromXY( i->GetX(), i->GetY(), i->WorldNumber() );
	if( calcReg != i->GetRegion() )
	{
		UI08 iRegion	= i->GetRegion();
		cSocket *s		= calcSocketObjFromChar( i );
		if( s != NULL )
		{
			if( region[iRegion] != NULL && region[calcReg] != NULL )
			{
				// Drake 08-30-99 If region name are the same, do not display message
				//                for playing music when approaching Tavern
				if( region[iRegion]->GetName() != region[calcReg]->GetName() )
				{
					//Changed this and four others to compare the first character of the string to '\0' rather than
					//to NULL.
					//-Shadowlord Nov-3-2001
					if( !region[iRegion]->GetName().empty() )
						s->sysmessage( 1358, region[iRegion]->GetName().c_str() );

					if( !region[calcReg]->GetName().empty() )
						s->sysmessage( 1359, region[calcReg]->GetName().c_str() );
				}
				j = strcmp( region[calcReg]->GetOwner(), region[iRegion]->GetOwner() );
				if( ( region[calcReg]->IsGuarded() && j ) || ( !( region[calcReg]->IsGuarded() && region[iRegion]->IsGuarded() ) ) )
				{
					if( region[calcReg]->IsGuarded() )
					{
						const char *calcRegOwner = region[calcReg]->GetOwner();
						if( calcRegOwner != NULL )
						{
							if( calcRegOwner[0] == '\0' )
								s->sysmessage( 1360 );
							else
								s->sysmessage( 1361, calcRegOwner );
						}
					} 
					else
					{
						const char *iRegionOwner = region[iRegion]->GetOwner();
						if( iRegionOwner != NULL )
						{
							if( iRegionOwner[0] == '\0' )
								s->sysmessage( 1362 );
							else
								s->sysmessage( 1363, iRegionOwner );
						}
					}
				}
				if( region[calcReg]->GetAppearance() != region[iRegion]->GetAppearance() )	 // if the regions look different
					SendWorldChange( (WorldType)region[calcReg]->GetAppearance(), s );
				if( calcReg == i->GetTown() )	// enter our home town
				{
					s->sysmessage( 1364 );
					CItem *packItem = getPack( i );
					if( packItem != NULL )
					{
						for( CItem *toScan = packItem->FirstItem(); !packItem->FinishedItems(); toScan = packItem->NextItem() )
						{
							if( toScan != NULL )
							{
								if( toScan->GetType() == 35 )
								{
									UI08 targRegion = (UI08)toScan->GetMoreX();
									s->sysmessage( 1365, region[targRegion]->GetName().c_str() );
									region[targRegion]->DoDamage( region[targRegion]->GetHealth() );	// finish it off
									region[targRegion]->Possess( calcReg );
									i->SetFame( (SI16)( i->GetFame() + i->GetFame() / 5 ) );	// 20% fame boost
									break;
								}
							}
						}
					}
				}
			}
		}
		UI16 leaveScript = i->GetScriptTrigger();
		cScript *tScript = Trigger->GetScript( leaveScript );
		if( tScript != NULL )
		{
			tScript->OnLeaveRegion( i, i->GetRegion() );
			tScript->OnEnterRegion( i, calcReg );
		}

		UI16 regLeaveScript	= region[i->GetRegion()]->GetScriptTrigger();
		cScript *trScript	= Trigger->GetScript( regLeaveScript );
		if( trScript != NULL )
			trScript->OnLeaveRegion( i, i->GetRegion() );

		UI16 regEnterScript	= region[calcReg]->GetScriptTrigger();
		cScript *teScript	= Trigger->GetScript( regEnterScript );
		if( teScript != NULL )
			teScript->OnEnterRegion( i, calcReg );

		i->SetRegion( calcReg );
		if( s != NULL )
		{
			Effects->dosocketmidi( s );
			doLight( s, cwmWorldState->ServerData()->WorldLightCurrentLevel() );	// force it to update the light level straight away
			Weather->DoPlayerStuff( i );	// force a weather update too
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void DumpCreatures( void )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Dumps creatures to npc.dat
//o---------------------------------------------------------------------------o
void DumpCreatures( void )
{
	std::ofstream toWrite( "npc.dat" );
	if( !toWrite.is_open() )
		return;
	for( UI16 i = 0; i < 2048; ++i )
	{
		toWrite << "[CREATURE " << i << "]" << std::endl << "{" << std::endl;
		toWrite << "BASESOUND=" << creatures[i].BaseSound() << std::endl;
		toWrite << "ICON=" << (int)creatures[i].Icon() << std::endl;
		toWrite << "SOUNDFLAG=" << (SI32)creatures[i].SoundFlag() << std::endl;
		if( creatures[i].CanFly() )
			toWrite << "FLIES" << std::endl;
		if( creatures[i].AntiBlink() )
			toWrite << "ANTIBLINK" << std::endl;
		if( creatures[i].IsAnimal() )
			toWrite << "ANIMAL" << std::endl;
		if( creatures[i].IsWater() )
			toWrite << "WATERCREATURE" << std::endl;
		toWrite << "}" << std::endl << std::endl;
	}
	toWrite.close();
}

//o---------------------------------------------------------------------------o
//|	Function	-	void criminal( CChar *c )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Make character a criminal
//o---------------------------------------------------------------------------o
void criminal( CChar *c )
{
	if( !c->IsCriminal() )
	{
		cSocket *cSock = calcSocketObjFromChar( c );
		c->SetTimer( tCHAR_CRIMFLAG, BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->RepCrimTime()) ) );
		if( cSock != NULL )
			cSock->sysmessage( 1379 );
		setcharflag( c );
		if( region[c->GetRegion()]->IsGuarded() && cwmWorldState->ServerData()->GuardsStatus() )
			Combat->SpawnGuard( c, c, c->GetX(), c->GetY(), c->GetZ() );
	}
	else
	{		// let's update their flag, as another criminal act will reset the timer
		c->SetTimer( tCHAR_CRIMFLAG, BuildTimeValue( static_cast<R32>(cwmWorldState->ServerData()->RepCrimTime()) ) );
		// chcek to see if there is a guard nearby, otherwise spawn us a new one
		if( !region[c->GetRegion()]->IsGuarded() || !cwmWorldState->ServerData()->GuardsStatus() )
			return;
		SI16 xOffset = MapRegion->GetGridX( c->GetX() );
		SI16 yOffset = MapRegion->GetGridY( c->GetY() );
		UI08 worldNumber = c->WorldNumber();
		for( SI08 counter1 = -1; counter1 <= 1; ++counter1 )
		{
			for( SI08 counter2 = -1; counter2 <= 1; ++counter2 )
			{
				SubRegion *toCheck = MapRegion->GetGrid( xOffset + counter1, yOffset + counter2, worldNumber );
				if( toCheck == NULL )
					continue;
				toCheck->PushChar();
				for( CChar *charCheck = toCheck->FirstChar(); !toCheck->FinishedChars(); charCheck = toCheck->GetNextChar() )
				{
					if( charCheck == NULL )
						continue;
					if( charCheck->GetNPCAiType() == aiGUARD )
					{
						if( objInRange( c, charCheck, cwmWorldState->ServerData()->CombatMaxRange() ) )
						{
							Combat->AttackTarget( charCheck, c );
							toCheck->PopChar();
							return;
						}
					}
				}
				toCheck->PopChar();
			}
		}
	}
}

//o---------------------------------------------------------------------------o
//|	Function	-	void setcharflag( CChar *c )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Set characters flag
//o---------------------------------------------------------------------------o
void setcharflag( CChar *c )
{
	if( c == NULL )
		return;
	UI08 oldFlag = c->GetFlag();
	if( !c->IsNpc() )
	{
		if( c->GetKills() > cwmWorldState->ServerData()->RepMaxKills() )
			c->SetFlagRed();
		else if( c->GetTimer( tCHAR_CRIMFLAG ) != 0 )
			c->SetFlagGray();
		else 
			c->SetFlagBlue();
	} 
	else 
	{
		switch( c->GetNPCAiType() )
		{
			case aiEVIL:		// Evil
			case aiHEALER_E:	// Evil healer
			case aiEV_BS:		// BS/EV
				c->SetFlagRed();
				break;
			case aiHEALER_G:	// Good Healer
			case aiPLAYERVENDOR:// Player Vendor
			case aiGUARD:		// Guard
			case aiBANKER:		// Banker
				c->SetFlagBlue();
				break;
			default:
				if( c->GetID() == 0x0190 || c->GetID() == 0x0191 )
				{
					c->SetFlagBlue();
					break;
				}
				else if( cwmWorldState->ServerData()->CombatAnimalsGuarded() && creatures[c->GetID()].IsAnimal() )
				{
					if( region[c->GetRegion()]->IsGuarded() )	// in a guarded region, with guarded animals, animals == blue
						c->SetFlagBlue();
					else
						c->SetFlagGray();
				}
				else	// if it's not a human form, and animal's aren't guarded, then they're gray
					c->SetFlagGray();
				if( c->GetOwnerObj() != NULL && c->IsTamed() )
				{
					CChar *i = (CChar *)c->GetOwnerObj();
					if( i != NULL )
						c->SetFlag( i->GetFlag() );
					else
						c->SetFlagBlue();
					if( c->IsInnocent() && !cwmWorldState->ServerData()->CombatAnimalsGuarded() )
						c->SetFlagBlue();
				}
				break;
		}
	}
	UI08 newFlag = c->GetFlag();
	if( oldFlag != newFlag )
	{
		UI16 targTrig = c->GetScriptTrigger();
		cScript *toExecute = Trigger->GetScript( targTrig );
		if( toExecute != NULL )
			toExecute->OnFlagChange( c, newFlag, oldFlag );
	}
}

//o---------------------------------------------------------------------------o
//|   Function    -  void doDeathStuff( CChar *i )
//|   Date        -  UnKnown
//|   Programmer  -  UnKnown  (Touched tabstops by Tauriel Dec 28, 1998)
//o---------------------------------------------------------------------------o
//|   Purpose     -  Performs death stuff. I.E.- creates a corpse, moves items
//|                  to it, take out of war mode, does animation and sound, etc.
//o---------------------------------------------------------------------------o
void doDeathStuff( CChar *i )
{
	if( i == NULL || i->IsDead() || i->IsInvulnerable() )	// don't kill them if they are dead or invulnerable!
		return;

	cSocket *pSock	= calcSocketObjFromChar( i );
	UI08 nType		= 0;

	if( i->GetID() != i->GetOrgID() )
	{
		i->SetID( i->GetOrgID() );
		i->Teleport();
	}

	if( ( i->GetID( 1 ) == 0x00 ) && ( i->GetID( 2 ) == 0x0C || ( i->GetID( 2 ) >= 0x3B && i->GetID( 2 ) <= 0x3D ) ) ) // If it's a dragon, 50/50 chance you can carve it
		nType = (UI08)RandomNum( 0, 1 );

	CChar *murderer = NULL;
	if( i->GetAttacker() != NULL )
		murderer = i->GetAttacker();

	if( pSock != NULL )
		DismountCreature( i );
	killTrades( i );

	if( !i->IsNpc() )
	{
		if( i->GetOrgID() == 0x0190 )
			i->SetID( 0x0192 );  // Male or Female
		else
			i->SetID( 0x0193 );
	}
	Effects->playDeathSound( i );
	i->SetSkin( 0x83EA );
	i->SetDead( true );
	i->StopSpell();
	i->SetHP( 0 );
	i->SetPoisoned( 0 );
	i->SetPoisonStrength( 0 );

	CItem *corpsenum = GenerateCorpse( i, nType, murderer );

	if( !i->IsNpc() )
	{ 
		CItem *c = Items->SpawnItem( NULL, i, 1, Dictionary->GetEntry( 1610 ), 0, 0x204E, 0, false, false );

		if( c == NULL )
			return;
		i->SetRobe( c->GetSerial() );
		c->SetLayer( 0x16 );
		if( c->SetCont( i ) )
			c->SetDef( 1 );
	}
	if( cwmWorldState->ServerData()->DeathAnimationStatus() )
		Effects->deathAction( i, corpsenum );
	if( i->GetAccount().wAccountIndex != AB_INVALID_ID )
	{
		if( pSock != NULL )
		{
			CPResurrectMenu toSend( 0 );
			pSock->Send( &toSend );
		}
		// The fade to gray is done by the CPResurrectMenu packet
		i->Teleport();
	}		
	
	UI16 targTrig		= i->GetScriptTrigger();
	cScript *toExecute	= Trigger->GetScript( targTrig );
	if( toExecute != NULL )
		toExecute->OnDeath( i );

	if( i->IsNpc() )
		Npcs->DeleteChar( i );
}

//o---------------------------------------------------------------------------o
//|	Function	-	CItem *GenerateCorpse( CChar *i, UI08 nType, char *murderername )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Generates a corpse based on skin of the character killed
//o---------------------------------------------------------------------------o
CItem *GenerateCorpse( CChar *i, UI08 nType, CChar *murderer )
{
	char temp[512];
	CItem *k, *c, *p = getPack( i );
	sprintf( temp, Dictionary->GetEntry( 1612 ).c_str(), i->GetName().c_str() );
	bool corpse = !( ( i->GetID() >= 13 && i->GetID() <= 16 && i->GetID() != 14 ) || i->GetID() == 574 );

	if( corpse )
	{
		c = Items->SpawnItem( NULL, i, 1, temp, false, 0x2006, i->GetSkin(), false, false );
		if( c == NULL )
			return NULL;
		c->SetCorpse( true );
		c->SetCarve( i->GetCarve() );
		c->SetMovable( 2 );//non-movable
		c->SetDir( i->GetDir() );
		c->SetAmount( i->GetID() );
	} 
	else
	{
		c = Items->SpawnItem( NULL, i, 1, Dictionary->GetEntry( 1611 ), false, 0x09B2, 0x0000, false, false );
		if( c== NULL )
			return NULL;
		c->SetCorpse( false );
	}
	c->SetMoreY( i->isHuman() );
	c->SetName2( i->GetName().c_str() );

	c->SetType( 1 );
	c->SetLocation( i );

	c->SetMore( (UI08)nType, 1 );
	c->SetDecayTime( BuildTimeValue( (R32)cwmWorldState->ServerData()->SystemTimer( DECAY ) ) );
	
	CMultiObj *iMulti = findMulti( c );
	if( iMulti != NULL )
		c->SetMulti( iMulti );

	if( !i->IsNpc() )
	{
		c->SetOwner( (cBaseObject *)i );
		c->SetMore( char( cwmWorldState->ServerData()->SystemTimer( PLAYER_CORPSE )&0xff ), 4 ); // how many times longer for the player's corpse to decay
	}
	c->SetMoreZ( i->GetFlag() );
	if( murderer == NULL )
		c->SetMoreX( INVALIDSERIAL );
	else
		c->SetMoreX( murderer->GetSerial() );
	c->SetMurderTime( cwmWorldState->GetUICurrentTime() );
	
	for( CItem *j = i->FirstItem(); !i->FinishedItems(); j = i->NextItem() )
	{
		if( j == NULL )
			continue;
		
		if( j->GetLayer() != 0x0B && j->GetLayer() != 0x10 )
		{
			if( j->GetType() == 1 && j->GetLayer() != 0x1A && j->GetLayer() != 0x1B && j->GetLayer() != 0x1C && j->GetLayer() != 0x1D )
			{
				for( k = j->FirstItem(); !j->FinishedItems(); k = j->NextItem() )
				{
					if( k == NULL )
						continue;
				
					if( !k->isNewbie() && k->GetType() != 9 )
					{
						k->SetCont( c );
						k->SetX( (SI16)( 20 + ( RandomNum( 0, 49 ) ) ) );
						k->SetY( (SI16)( 85 + ( RandomNum( 0, 75 ) ) ) );
						k->SetZ( 9 );
					}
				}
			}

			if( j != p && j->GetLayer() != 0x1D )
			{
				if( j->isNewbie() && p != NULL )
					j->SetCont( p );
				else
					j->SetCont( c );
			}

			if( j->GetLayer() == 0x15 && !i->IsShop() && corpse )
				j->SetLayer( 0x1A );
			j->SetX( (SI16)( 20 + ( RandomNum( 0, 49 ) ) ) );
			j->SetY( (SI16)( 85 + ( RandomNum( 0, 74 ) ) ) );
			j->SetZ( 9 );

//			j->RemoveFromSight();
		}

		if( j->GetLayer() == 0x0B || j->GetLayer() == 0x10 )
		{
			j->SetName( "Hair/Beard" );
			j->SetX( 0x47 );
			j->SetY( 0x93 );
			j->SetZ( 0 );
		}
	}
	return c;
}

//o---------------------------------------------------------------------------o
//|	Function	-	void SendWorldChange( WorldType season, cSocket *sock )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Send world changes to socket
//o---------------------------------------------------------------------------o
void SendWorldChange( WorldType season, cSocket *sock )
{
	CPWorldChange wrldChange( season, 1 );
	sock->Send( &wrldChange );
}

void SendMapChange( UI08 worldNumber, cSocket *sock, bool initialLogin )
{
	if( sock == NULL )
		return;
	CPMapChange mapChange( worldNumber );
	if( !initialLogin && worldNumber > 1 )
	{
		switch( sock->ClientType() )
		{
			case CV_UO3D:
			case CV_KRRIOS:
				break;
			default:
				//mapChange.SetMap( 0 );
				break;
		}
	}
	sock->Send( &mapChange );
	CChar *mChar = sock->CurrcharObj();
	if( !initialLogin )
		mChar->Teleport();
}

void SocketMapChange( cSocket *sock, CChar *charMoving, CItem *gate )
{
	if( gate == NULL || ( sock == NULL && charMoving == NULL ) )
		return;
	UI08 tWorldNum = (UI08)gate->GetMore();
	if( !Map->MapExists( tWorldNum ) )
		return;
	CChar *toMove = charMoving;
	if( sock != NULL && charMoving == NULL )
		toMove = sock->CurrcharObj();
	if( toMove == NULL )
		return;
	switch( sock->ClientType() )
	{
		case CV_UO3D:
		case CV_KRRIOS:
			toMove->SetLocation( (SI16)gate->GetMoreX(), (SI16)gate->GetMoreY(), (SI08)gate->GetMoreZ(), tWorldNum );
			break;
		default:
			//if( tWorldNum <= 1 )
				toMove->SetLocation( (SI16)gate->GetMoreX(), (SI16)gate->GetMoreY(), (SI08)gate->GetMoreZ(), tWorldNum );
	//		else
	//			toMove->SetLocation( (SI16)gate->GetMoreX(), (SI16)gate->GetMoreY(), (SI08)gate->GetMoreZ(), 0 );
			break;
	}
	toMove->RemoveFromSight();
	SendMapChange( tWorldNum, sock );
	toMove->Teleport();

}

//o---------------------------------------------------------------------------o
//|	Function	-	void NetworkPollConnectionThread( void *params )
//|	Programmer	-	Unknown
//o---------------------------------------------------------------------------o
//|	Purpose		-	Watch for new connections
//o---------------------------------------------------------------------------o
#if defined( _MSC_VER )
#pragma note( "Param Warning: in NetworkPollConnectionThread(), params is unrefrenced" )
#endif
void NetworkPollConnectionThread( void *params )
{
	messageLoop << "Thread: NetworkPollConnection has started";
	netpollthreadclose = false;
	while( !netpollthreadclose )
	{
		Network->CheckConnections();
		Network->CheckLoginMessage();
		UOXSleep( 20 );
	}
#if defined(__unix__)
	pthread_exit( NULL );
#else
	_endthread();
#endif
	messageLoop << "Thread: NetworkPollConnection has Closed";
}





