//o---------------------------------------------------------------------------o
//| UOX3.h
//o---------------------------------------------------------------------------o
//| This is the main UOX3 header file
//o---------------------------------------------------------------------------o
//| Version History
//| 1.5		Added comments and changed int variables into typefefs
//|         (Mr. Fixit 5. November 2001)
//| 1.6		Removed the warning promotions from level 3 to 4 off.
//|					(EviLDeD 022602)
//|	1.7		Most all global includes are now located here (rather than included in other global includes)
//|			Early class declarations to allow removal of most files from global scope
//|			Removed a majority of header files from global scope
//|			Removed some unneeded / duplicate external header includes
//|			Moved or Removed many global variables (many to cWorldMain class)
//|			( Zane(giwo) 4/01/2003 )
//o---------------------------------------------------------------------------o


//o---------------------------------------------------------------------------o
// Only run once
//o---------------------------------------------------------------------------o
#define __REENTRANT
#ifndef __UOX3_H
#define __UOX3_H

//o---------------------------------------------------------------------------o
// FIXMEs / TODOs / NOTE macros
//o---------------------------------------------------------------------------o
#define _QUOTE(x) # x
#define QUOTE(x) _QUOTE(x)
#define __FILE__LINE__ __FILE__ "(" QUOTE(__LINE__) ") : "
#define NOTE( x )  message( x )
#define FILE_LINE  message( __FILE__LINE__ )
#define TODO( x )  message( __FILE__LINE__"\n"           \
        "+------------------------------------------------\n" \
        "|  TODO :   " #x "\n" \
        "+-------------------------------------------------\n" )
#define FIXME( x )  message(  __FILE__LINE__"\n"           \
        "+------------------------------------------------\n" \
        "|  FIXME :  " #x "\n" \
        "+-------------------------------------------------\n" )
#define todo( x )  message( __FILE__LINE__" TODO :   " #x "\n" )
#define fixme( x )  message( __FILE__LINE__" FIXME:   " #x "\n" )
#define note( x )  message( __FILE__LINE__" NOTE :   " #x "\n" )

//o---------------------------------------------------------------------------o
// Set what OS UOX is compiled under
//o---------------------------------------------------------------------------o
#if defined(__unix__)
	#define XP_UNIX			// JS API Requires we define OS we compile with
	#define OS_STR "Linux"
	#define BOOL bool
	#define VOID void
	#define closesocket( s ) close( s )
	#define ioctlsocket( s, b, c ) ioctl( s, b, c )
#elif defined _WIN32
	#define XP_WIN			// JS API Requires we define OS we compile with
	#define XP_PC
	#define OS_STR "Win32"
#else
	#error "Failed to define either __unix__ or _WIN32"
#endif

//o---------------------------------------------------------------------------o
// Skip some errors
//o---------------------------------------------------------------------------o
#if defined( _MSC_VER )
	#pragma warning( disable : 4511 )	// copy constructor could not be generated
	#pragma warning( disable : 4512 )	// assignment operator could not be generated
	#pragma warning( disable : 4663 )	// C++ language change: to explicitly specialize class template '' use the following syntax...
	#pragma warning( disable : 4786 )	// Gets rid of BAD stl warnings
#endif

//o---------------------------------------------------------------------------o
// Include files
//o---------------------------------------------------------------------------o
#include <vector>
#include <map>
#include <fstream>
#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <string>
#include <ctime>
#include <cmath>
#include <sys/types.h>
#include <assert.h>
#include <sstream>
#include <queue>

//o---------------------------------------------------------------------------o
// OS specific includes
//o---------------------------------------------------------------------------o
#include "socket_interface.h"
#if defined(__unix__)		// Unix, Linux, etc
	#include <ctype.h>
	#include <sys/time.h>
	#include <sys/signal.h>
	#include <unistd.h>
#else						// Windows (Currently this compiles fine with MSVC 6.0, if it is missing includes
							//			in other compilers then we need special cases for those compilers, not
							//			simply anyone running a Windows-based OS.)
	#define WIN32_LEAN_AND_MEAN
	#include <process.h>
	#include <conio.h>
	#include <sys/timeb.h>
	typedef long int32;
#endif

//o---------------------------------------------------------------------------o
// Javascript engine includes
//o---------------------------------------------------------------------------o
#include "jsapi.h"

//o---------------------------------------------------------------------------o
// Class Declarations
//o---------------------------------------------------------------------------o
class cWeight;
class cBoat;
class cBooks;
class CGuildCollection;
class cCombat;
class cTownRegion;
class JailSystem;
class cWeatherAb;
class cMovement;
class cRaces;
class cServerDefinitions;
class cCommands;
class cWhoList;
class cSkills;
class cMagic;
class cTargets;
class PageVector;
class CSpeechQueue;
class cHTMLTemplates;
class cGump;
class Triggers;
class cMapStuff;
class Script;
class cTEffect;
class cEffects;
class cNetworkStuff;
class cFileIO;
class cPBaseBuffer;
class cPUOXBuffer;
class cPXGMBuffer;
class cPInputBuffer;
class cBaseObject;
class CChar;
class CNpc;
class CPlayer;
class CItem;
class cSocket;
class CMultiObj;
class cSpawnRegion;
class ScriptSection;
class cScript;
class cItem;
class cCharStuff;
class CThreadQueue;
class cSpeech;
class cMapRegion;
class cStringTools;
class SpellInfo;

#if !defined(_MSC_VER)
	// VC6 doesn't define std::min and std::max correctly, we're told to use _MIN and _MAX instead
	#define UOX_MIN (std::min)
	#define UOX_MAX (std::max)
#elif defined( _MSC_VER ) && _MSC_VER >= 1300	 
	#define UOX_MIN (std::min)		// without the brackets, VC7.1 kicks up a stink
	#define UOX_MAX (std::max)
#else
	#define UOX_MIN (std::_MIN)
	#define UOX_MAX (std::_MAX)
#endif

//o---------------------------------------------------------------------------o
// The UOX project includes
//o---------------------------------------------------------------------------o
#include "enums.h"
#include "typedefs.h"
#include "uoxstruct.h"
#include "hash.h"
#include "cAccountClass.h"
#include "cBaseObject.h"
#include "cChar.h"
#include "cItem.h"
#include "cMultiObj.h"
#include "handlers.h"
#include "cConsole.h"
#include "cSocket.h"
#include "cServerData.h"
#include "worldmain.h"
#include "Dictionary.h"
#include "funcdecl.h"

//o---------------------------------------------------------------------------o
// LINUX Definitions
//o---------------------------------------------------------------------------o
#if defined(__unix__)
	extern "C" {
		char *strlwr(char *);
		char *strupr(char *);
	}
#endif

//o---------------------------------------------------------------------------o
// JS Stuff
//o---------------------------------------------------------------------------o
extern JSRuntime *jsRuntime;
extern JSContext *jsContext;
extern JSObject *jsGlobal;
extern JSClass global_class;
extern JSClass uox_class;

//o---------------------------------------------------------------------------o
// Item / Character Variables
//o---------------------------------------------------------------------------o
extern creat_st creatures[2048];
extern char skillname[SKILLS+1][20];
extern UI16 doorbase[DOORTYPES];

//o---------------------------------------------------------------------------o
// Misc Variables
//o---------------------------------------------------------------------------o
extern HashTable< ITEM >			nitemsp;
extern HashTableMulti< ITEM >		nspawnsp;
extern HashTable< CHARACTER >		ncharsp;
extern HashTableMulti< CHARACTER >	ncspawnsp;

extern cTownRegion *region[256];
extern cSpawnRegion *spawnregion[4098];	// Zippy
extern ITEM *loscache;
extern UI16 *itemids;
extern CEndL myendl;
extern std::vector< SpellInfo > spells; //:Terrin: adding variable for spell system "cache" had to make global for skills.cpp as a quick fix

//o---------------------------------------------------------------------------o
// Profiling
//o---------------------------------------------------------------------------o
extern UI32 networkTime;
extern UI32 timerTime;
extern UI32 autoTime;
extern UI32 loopTime;
extern UI32 networkTimeCount;
extern UI32 timerTimeCount;
extern UI32 autoTimeCount;
extern UI32 loopTimeCount;
extern SI32 globalRecv;
extern SI32 globalSent;


//o---------------------------------------------------------------------------o
// Time inline functions
//o---------------------------------------------------------------------------o
UI32 getclock( void );
#if defined(__unix__)
	inline void StartMilliTimer( UI32 &Seconds, UI32 &Milliseconds )
	{
		struct timeval t;
		gettimeofday( &t, NULL );
		Seconds			= t.tv_sec;
		Milliseconds	= t.tv_usec / 1000;
	}
	inline UI32 CheckMilliTimer( UI32 &Seconds, UI32 &Milliseconds ) 
	{ 
		struct timeval t; 
		gettimeofday( &t, NULL ); 
		return ( 1000 * ( t.tv_sec - Seconds ) + ( (t.tv_usec/1000) - Milliseconds ) ); 
	}
	inline void UOXSleep( int toSleep )
	{
		usleep( toSleep * 1000 );
	}
#else
	inline void StartMilliTimer( UI32 &Seconds, UI32 &Milliseconds ) 
	{ 
		struct timeb t; 
		ftime( &t ); 
		Seconds = t.time; 
		Milliseconds = t.millitm; 
	};
	inline UI32 CheckMilliTimer( UI32 &Seconds, UI32 &Milliseconds ) 
	{ 
		struct timeb t; 
		ftime( &t ); 
		return ( 1000 * ( t.time - Seconds ) + ( t.millitm - Milliseconds ) ); 
	};
	inline void UOXSleep( int toSleep )
	{
		Sleep( toSleep );
	}
#endif

//o---------------------------------------------------------------------------o
// Classes Definitions
//o---------------------------------------------------------------------------o
extern cBoat					*Boats;
extern CWorldMain				*cwmWorldState;
extern cCombat					*Combat;
extern cCommands				*Commands;
extern cGump					*Gumps;
extern cHTMLTemplates			*HTMLTemplates;
extern cItem					*Items;
extern cMapStuff				*Map;
extern cCharStuff				*Npcs;
extern cSkills					*Skills;
extern cWeight					*Weight;
extern cTargets					*Targ;
extern cNetworkStuff			*Network;
extern cMagic					*Magic;
extern cRaces					*Races;
extern cWeatherAb				*Weather;
extern cMovement				*Movement;
extern cWhoList					*WhoList;
extern cWhoList					*OffList;
extern cTEffect					*TEffects;
extern cBooks					*Books;
extern PageVector				*GMQueue;
extern PageVector				*CounselorQueue;
extern CDictionaryContainer		*Dictionary;
extern cAccountClass			*Accounts;
extern Triggers					*Trigger;
extern CConsole					Console;
extern cMapRegion				*MapRegion;
extern CSpeechQueue				*SpeechSys;
extern CGuildCollection			*GuildSys;
extern cServerDefinitions		*FileLookup;
extern JailSystem				*JailSys;
extern cEffects					*Effects;
extern cFileIO					*FileIO;
extern cSpeech					*Speech;
extern CThreadQueue				messageLoop;
extern cItemHandle				items;
extern cCharacterHandle			chars;

extern QUEUEMAP itemQueue, npcQueue;

inline UI32 BuildTimeValue( R32 timeFromNow ) 
{ 
	return (UI32)( cwmWorldState->GetUICurrentTime() + ( (R32)(1000) * timeFromNow ) );	
}


#endif // __UOX3_H
